/* ***** Programme de traitement de l'EXAFS
     ------- Gestion d'un ensemble d'atomes pour les modles de calcul
             Tout ce qui concerne la gestion GEM de ces atomes

   (c) Emmanuel CURIS, 1998

 27 mars 1998
   * Nettoyage des inclusions

 16 janvier 1998
   * Cration du fichier
   * Fonction ATM_init() : prpare les arbres
*/

#define _ATOMES_GEM_ 

#include <stdio.h>
#include <stdlib.h>

#ifdef _UNIX_

# include "../SYSTEME.H"
# include "../EXAFS.H"

#else /* Par dfaut : mode DOS / Atari */

# include "..\SYSTEME.H"
# include "..\EXAFS.H"

#endif /* _UNIX_ */

#include "ATOMES.H"
#include "ATM_ERR.H"

/* ~~~~~~~~~~~~~ Dfinitions des fonctions internes ~~~~~~~~~~~~~~~~~~ */

boolean GATM_import(int num_obj, int x, int y);

/* ~~~~~~~~~~~~~ Initialisation du module ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ------------- Procdure d'initialisation ---------------------------\
 |   Ne renvoie rien                                                   |
 |   Ne prend aucun paramtre                                          |
 \-------------------------------------------------------------------- */

void ATM_init(void)
{
	arbres[IMPORTE_ATOMES].fonction.arbre = GATM_import; /* Fonction qui gre cet arbre */
}

/* ~~~~~~~~~~~~~ Gestion des arbres GEM de ce module ~~~~~~~~~~~~~~~~ */

/* ------------- Arbre de paramtrage du type d'import  raliser ----\
 |   Mmes dfinitions que les autres arbres de gestion...
 \------------------------------------------------------------------- */

boolean GATM_import(int num_obj, int x, int y)
{
	const int num_arbre = /* DECAL_ATOMES + */ IMPORTE_ATOMES;
	boolean etat;

	switch (num_obj)
	{
	case ATM_IMP_ASCII: /* On demande d'importer un fichier ASCII bte */
		OBJ_choisit(num_arbre, ATM_IMP_ASCII, FORCE_OUI);
		OBJ_choisit(num_arbre, IMP_ATM_JNL, FORCE_NON);
		OBJ_choisit(num_arbre, IMP_ATM_COR, FORCE_NON);

		OBJ_active(num_arbre, IMP_ATM_NUM_LIT, FORCE_NON); /* Options pour les autres formats */
		OBJ_active(num_arbre, IMP_ATM_REF, FORCE_NON);
		OBJ_active(num_arbre, IMP_COR_NUMERO, FORCE_NON);
		OBJ_active(num_arbre, IMP_COR_REF, FORCE_NON);

		OBJ_active(num_arbre, IMP_ATM_LGN_SAUT, FORCE_OUI);
		OBJ_active(num_arbre, IMP_COR_ORTHO, FORCE_OUI);
		OBJ_active(num_arbre, IMP_COR_FRAC,  FORCE_OUI);

		etat = OBJ_choisit(num_arbre, IMP_COR_FRAC, QUESTION); /* Utilise-t-on la maille ? */
		OBJ_active(num_arbre, IMP_ATM_A, etat);
		OBJ_active(num_arbre, IMP_ATM_B, etat);
		OBJ_active(num_arbre, IMP_ATM_C, etat);
		OBJ_active(num_arbre, IMP_ATM_ALPHA, etat);
		OBJ_active(num_arbre, IMP_ATM_BETA,  etat);
		OBJ_active(num_arbre, IMP_ATM_GAMMA, etat);
		break;

  case IMP_ATM_JNL: /* On demande d'importer un fichier *.JNL */
		OBJ_choisit(num_arbre, ATM_IMP_ASCII, FORCE_NON);
		OBJ_choisit(num_arbre, IMP_ATM_JNL, FORCE_OUI);
		OBJ_choisit(num_arbre, IMP_ATM_COR, FORCE_NON);

		OBJ_active(num_arbre, IMP_ATM_NUM_LIT, FORCE_NON); /* Options pour les autres formats */
		OBJ_active(num_arbre, IMP_ATM_REF,     FORCE_NON);
		OBJ_active(num_arbre, IMP_COR_NUMERO,  FORCE_NON);
		OBJ_active(num_arbre, IMP_COR_REF,     FORCE_NON);

		OBJ_active(num_arbre, IMP_ATM_LGN_SAUT, FORCE_NON);

		OBJ_active(num_arbre, IMP_COR_ORTHO, FORCE_NON); /* Contenu dans le fichier */
		OBJ_active(num_arbre, IMP_COR_FRAC,  FORCE_NON);

		OBJ_active(num_arbre, IMP_ATM_A, FORCE_NON);
		OBJ_active(num_arbre, IMP_ATM_B, FORCE_NON);
		OBJ_active(num_arbre, IMP_ATM_C, FORCE_NON);
		OBJ_active(num_arbre, IMP_ATM_ALPHA, FORCE_NON);
		OBJ_active(num_arbre, IMP_ATM_BETA,  FORCE_NON);
		OBJ_active(num_arbre, IMP_ATM_GAMMA, FORCE_NON);
 	  break;

  case IMP_ATM_COR: /* On demande d'importer un fichier *.COR */
		OBJ_choisit(num_arbre, ATM_IMP_ASCII, FORCE_NON);
		OBJ_choisit(num_arbre, IMP_ATM_JNL, FORCE_NON);
		OBJ_choisit(num_arbre, IMP_ATM_COR, FORCE_OUI);

		OBJ_active(num_arbre, IMP_COR_NUMERO,  FORCE_OUI);
		OBJ_active(num_arbre, IMP_COR_REF,     FORCE_OUI);
		etat = OBJ_choisit(num_arbre, IMP_COR_REF, QUESTION);
		OBJ_active(num_arbre, IMP_ATM_REF, etat);
		OBJ_active(num_arbre, IMP_ATM_NUM_LIT, !etat);

		OBJ_active(num_arbre, IMP_ATM_LGN_SAUT, FORCE_NON);

		OBJ_active(num_arbre, IMP_COR_ORTHO, FORCE_OUI);
		OBJ_active(num_arbre, IMP_COR_FRAC,  FORCE_OUI);

		etat = OBJ_choisit(num_arbre, IMP_COR_FRAC, QUESTION); /* Utilise-t-on la maille ? */
		OBJ_active(num_arbre, IMP_ATM_A, etat);
		OBJ_active(num_arbre, IMP_ATM_B, etat);
		OBJ_active(num_arbre, IMP_ATM_C, etat);
		OBJ_active(num_arbre, IMP_ATM_ALPHA, etat);
		OBJ_active(num_arbre, IMP_ATM_BETA,  etat);
		OBJ_active(num_arbre, IMP_ATM_GAMMA, etat);
  	break;

  case IMP_COR_NUMERO: /* On veut charger les coordonnes du i-me compos dans le fichier *.COR */
  	OBJ_choisit(num_arbre, IMP_COR_NUMERO, FORCE_OUI);
  	OBJ_choisit(num_arbre, IMP_COR_REF, FORCE_NON);

		OBJ_active(num_arbre, IMP_ATM_NUM_LIT, FORCE_OUI);
		OBJ_active(num_arbre, IMP_ATM_REF, FORCE_NON);
		break;

  case IMP_COR_REF: /* On veut charger les coordonnes du compos dont le code est ... */
  	OBJ_choisit(num_arbre, IMP_COR_NUMERO, FORCE_NON);
  	OBJ_choisit(num_arbre, IMP_COR_REF, FORCE_OUI);

		OBJ_active(num_arbre, IMP_ATM_NUM_LIT, FORCE_NON);
		OBJ_active(num_arbre, IMP_ATM_REF, FORCE_OUI);
		break;  
	}

	return VRAI; /* Rien de spcial  faire sinon */
}
