/* Pas mal de dfinitions pour l'utilisation de modles en chimie

 27 mars 1998
   * Corrig les fins de ligne... Maintenant au format PC partout

  5 fvrier 1998
   * Dclaration des fonctions pour l'import des atomes

 16 janvier 1998
   * Cration de l'historique
   * Dclar ATM_init() et ATM_termine()
   * Cration du type Liaison, pour les infos de liaison sur un atome
*/

#ifndef _ATOMES_H_

#define _ATOMES_H_ 

#define HYDROGENE       1
#define HELIUM          2
#define LITHIUM         3
#define BERYLLIUM       4
#define BORE            5
#define CARBONE         6
#define AZOTE           7
#define OXYGENE         8
#define FLUOR           9
#define NEON           10
#define SODIUM         11
#define MAGNESIUM      12
#define ALUMINIUM      13
#define SILICIUM       14
#define PHOSPHORE      15
#define SOUFRE         16
#define CHLORE         17
#define ARGON          18
#define POTASSIUM      19
#define CALCIUM        20
#define SCANDIUM       21
#define TITANE         22
#define VANADIUM       23
#define CHROME         24
#define MANGANESE      25
#define FER            26
#define COBALT         27
#define NICKEL         28
#define CUIVRE         29
#define ZINC           30
#define GALLIUM        31
#define GERMANIUM      32
#define ARSENIC        33
#define SELENIUM       34
#define BROME          35
#define KRYPTON        36
#define RUBIDIUM       37
#define STRONTIUM      38
#define YTTRIUM        39
#define ZIRCONIUM      40
#define NIOBIUM        41
#define MOLYBDENE      42
#define TECHNETIUM     43
#define RUTHENIUM      44
#define RHODIUM        45 /* Rh */
#define PALLADIUM      46
#define ARGENT         47
#define CADMIUM        48
#define INDIUM         49
#define ETAIN          50
#define ANTIMOINE      51
#define TELLURE        52
#define IODE           53
#define XENON          54
#define CESIUM         55
#define BARYUM         56
#define LANTHANE       57
#define CERIUM         58
#define PRASEODYME     59
#define NEODYME        60
#define PROMETHIUM     61 /* Pm */
#define SAMARIUM       62
#define EUROPIUM       63
#define GADOLINIUM     64
#define TERBIUM        65
#define DYSPROSIUM     66
#define HOLMIUM        67
#define ERBIUM         68
#define THULIUM        69 /* Tm */
#define YTTERBIUM      70
#define LUTETIUM       71
#define HAFNIUM        72
#define TANTALE        73
#define TUNGSTENE      74
#define RHENIUM        75
#define OSMIUM         76
#define IRIDIUM        77
#define PLATINE        78
#define OR             79
#define MERCURE        80
#define THALLIUM       81
#define PLOMB          82
#define BISMUTH        83
#define POLONIUM       84
#define ASTATE         85
#define RADON          86
#define FRANCIUM       87
#define RADIUM         88
#define ACTINIUM       89
#define THORIUM        90 /*  vrifier - Th */
#define PROTACTINIUM   91
#define URANIUM        92
#define NEPTUNIUM      93
#define PLUTONIUM      94
#define AMERICIUM      95
#define CURIUM         96 /* Cm */
#define BERKELIUM      97 /* Bk */
#define CALIFORNIUM    98
#define EINSTEINIUM    99
#define FERMIUM       100 /*  vrifier - Fm */
#define MENDELEVIUM   101
#define NOBELIUM      102
#define LAWRENCIUM    103
#define RUTHERFORDIUM 104 /*  vrifier - Rf */
#define DUBNIUM       105 /* Db */
#define SEABORGIUM    106 /* Sg */
#define BOHRIUM       107 /* Bh */
#define HASSIUM       108 /* Hs */
#define MEITNERIUM    109 /* Mt */
#define UNUNNILIUM    110 /* Uun */
#define UNUNUNIUM     111 /* Uuu */
#define UNUNBIUM      112
#define UNUNTRIUM     113
#define UNUNQUADIUM   114
#define UNUNPENTIUM   115
#define UNUNHEXIUM    116
#define UNUNSEPTIUM   117
#define UNUNOCTIUM    118

typedef struct liaison
{
  struct liaison *suivante;   /* La liaison suivante forme par l'atome */
  struct liaison *precedente; /* La liaison prcdente                  */
  float multiplicite;
  double longueur;
  double energie;
  double k_force; /* Constante de force, modle de l'oscillateur harmonique */
} Liaison;

typedef struct atome
{
  struct atome  *suivant;      /* Atome prcdent dans le modle */
  struct atome  *precedent;    /* Atome suivant dans le modle   */
  int            num_atomique; /* Numro atomique de l'lment   */
  double         x, y, z;      /* Coordonnes atomiques          */
  double         charge;       /* Charge (partielle) de l'atome  */
  double         rayon;        /* Rayon de l'atome considr     */
  int            nombre_lies;  /* Nombre d'atomes lis           */
  struct atome  *lies[15];     /* Pointeurs sur les atomes lis  */
} Atome;

/* ************ Fonctions de gestion des atomes (cration,...) ****************** */

void ATM_init(void); /* Initialisation du module */

Atome *ATM_creer   (int numero_atomique, double x, double y, double z,
		    int nbr_lies, Atome **lies, float charge, double rayon);
int    ATM_efface  (Atome *atome);
Atome *ATM_moyenne (int nbr_atomes, Atome **atomes, int effacer);
int    ATM_ttefface(int numero_atomique); /* Efface tous les atomes du mme type */

void ATM_termine(void); /* On quitte le module */

/* ************ Informations sur les atomes ************************************* */

typedef struct
{
  char  *symbole;          /* Symbole de l'lment                  */
  char  *nom;              /* Nom de l'lment - ici en franais... */
  float m_molaire;         /* Masse molaire de l'lment */
  float electronegativite; /* lectrongativit de l'lment */
  float rayon_covalent;
  float rayon_vdw;           /* Rayon de Van der Waals              */
  unsigned long nbr_autres;  /* Nombre d'autres paramtres associs */
  double *autres;            /* Autres paramtres associs          */
} ATM_infos;

#ifndef _ATOMES_

extern ATM_infos ATM_donnees[];
extern Atome *ATM_premier, *ATM_dernier;

#endif /* _ATOMES_ */

int ATM_num_atomique(char *symbole);

/* ************ Fonctions de chargement des atomes ****************************** */

void ATM_import(void); /* Import des atomes : on choisit le type */

void JNL_charge(void); /* Lecture d'un fichier *.JNL de la CCDC */
void COR_charge(void); /* Lecture d'un fichier *.COR de la CCDC */

#endif /* _ATOMES_H_ : protection contre l'inclusion multiple */
