/* Bibliothque GEM / X-Window - (C) 1998, Emmanuel CURIS
   Gestion des arbres d'objet
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "Bib_X.h"

extern int EVNT_relache (XKeyEvent *clavier);
extern int EVNT_redessin(Fenetre *fenetre, XExposeEvent *redessin);

/* --------------- Initialise un arbre GEM ----------------------------\
 |    Ne renvoie rien                                                  |
 |                                                                     |
 | num_arbre : le numro de l'arbre que l'on initialise                |
 | interne   : VRAI = arbre dfini dans le programme -> coordonnes... |
 \-------------------------------------------------------------------- */

void ARB_init(int num_arbre, boolean interne)
{
  OBJECT *adresse;
  int objet, dern_objet = ROOT;

  adresse = arbres[num_arbre].adresse;
  arbres[num_arbre].premier_edit = ROOT;
  arbres[num_arbre].dernier_edit = ROOT;

  /* Quel est le dernier objet de l'arbre ? */
  
  while ( OB_TAIL(adresse, dern_objet) != -1 ) /* Premire tape : on va le plus loin possible
						  en sautant les objets fils */
    dern_objet = OB_TAIL(adresse, dern_objet);
  
  while ( ( OB_FLAGS(adresse, dern_objet) & LASTOB ) == FAUX ) /* Ce n'est pas le dernier objet ->
								  on fait les derniers un par un */
    dern_objet++; /* On passe donc au suivant */

  /* dern_objet contient ainsi le numro du dernier objet de l'arbre */

  arbres[num_arbre].nbr_objets = dern_objet + 1;
  arbres[num_arbre].objets = (Window *) malloc( sizeof(Window) * (dern_objet + 1) );
  if (arbres[num_arbre].objets == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("dans X_arbres.c\n");
    return;
  }

  /* On recherche le premier et le dernier objet ditable, le dernier aussi */

  for (objet = ROOT; objet <= dern_objet ; objet++) /* On parcours tous les objets */
  {
    if ( (OB_FLAGS(adresse, objet) & EDITABLE)   /* Objet ditable */
	 && ( (OB_STATE(adresse, objet) & DISABLED) == FAUX ) ) /* Non dsactiv  */
    {
      if ( arbres[num_arbre].premier_edit == ROOT )
      {
	arbres[num_arbre].premier_edit = objet; /* Premier objet ditable actif */
	arbres[num_arbre].edit_courant = objet; /* Correspond aussi au courant  */
      }
      
      arbres[num_arbre].dernier_edit = objet; /* Dernier ditable */
    }
    
    if (interne)
    {
      rsrc_obfix(adresse, objet);
      if ( (OB_TYPE(adresse, objet) == G_FBOXTEXT) || (OB_TYPE(adresse, objet) == G_FTEXT) )
      {
	char *chaine = adresse[objet].ob_spec.tedinfo->te_ptext;

	adresse[objet].ob_spec.tedinfo->te_ptext = strdup(chaine);
	if ( adresse[objet].ob_spec.tedinfo->te_ptext == NULL )
	{
	  printf("ARB_init : impossible d'initialiser correctement les objets TEDINFO\n");
	  printf("Plantage du programme risqu dans la suite\n");
	  ERREUR(MANQUE_MEMOIRE);
	  adresse[objet].ob_spec.tedinfo->te_ptext = chaine;
	}
	else
	  chaine = adresse[objet].ob_spec.tedinfo->te_ptext;

	if ( chaine[0] == '@' )
	{
	  unsigned long i;
	  for (i = 0 ; i < adresse[objet].ob_spec.tedinfo->te_txtlen ; i++)
	    chaine[i] = 0;
	}
      }
    }
  }
}

/* ------------- Appelle un arbre GEM -------------------------------------- */

Fenetre *appelle_arbre(int arbre, int attributs, char *titre, char mode)
{
  if (mode == 0) /* Type d'arbre dpend du paramtrage */
  {
  }

  if (mode == 2 ) /* Mode 2 ==> force en non-fentre */
  {
    ext_formdo(arbre); /* On appelle le form_do tendu */
    return NULL; /* et on ne cre pas de fentre */
  }

  /* Arbre en fentre      */

  return creer_gem_fenetre(attributs, arbre, titre);
}

/* --------------- Renvoie l'arbre associ  un objet ---------------- */

Arbre *quel_arbre(OBJECT *objet)
{
  Arbre *arbre = arbres;
  unsigned int i;

  /* On vrifie que ce n'est pas un objet racine */

  for (i = 0, arbre = arbres ; i < _nbr_arbres ; i++, arbre++)
  {
    if (arbre->adresse == objet)
      return arbre;
  }

  /* Il faut maintenant vrifier que ce n'est pas un objet interne

     PAS ENCORE RALIS : PAS L'USAGE POUR L'INSTANT... */

  /* Aucun arbre trouv associ  cet objet */

  return NULL;
}

/* --------------- Renvoie la fentre associe  un arbre -------------\
 |                                                                     |
 | arbre : adresse de la structure associe  l'arbre nous intressant |
 \-------------------------------------------------------------------- */

Fenetre *ARB_fenetre(Arbre *arbre)
{
  Fenetre *fenetre;

  if (arbre == NULL) return NULL; /* Pas d'arbre associ */
  if (_premiere_fenetre == NULL) return NULL; /* Aucune fentre ouverte */

  for (fenetre = _premiere_fenetre; fenetre ; fenetre = fenetre->suivante)
  {
    if (fenetre->arbre == arbre)
    {
      if (fenetre->drapeaux.ouverte == FAUX) /* Fentre ferme */
	return NULL;
      else
	return fenetre;
    }
  }
  return NULL; /* Aucune fentre associe */
}

/* ------------- Affichage d'un message dans un arbre GEM pendant un temps donn ----\
 |   Renvoie <0 en cas de problme, VRAI sinon
 |   Bloque l'utilisation du programme (ct interface) tant qu'il est affich
 |
 | num_arbre : le numro de l'arbre  afficher comme message
 | temps : -> si bit 0 mis (impair), on attends ce temps-la (en s) pour effacer
 |         -> sinon (pair) : on attends le temps ou clic / clavier
 |         -> 0 : on ouvre juste, utilisateur ferme quand il n'en a plus besoin (en fentre alors)
 |         -> 1 : on ouvre jusqu' ce que l'utilisateur clique ou appuie sur une touche
 |         -> 2 : on ferme un message prcdemment ouvert
 \---------------------------------------------------------------------------------- */

int ARB_message(int num_arbre, unsigned int temps)
{
  Fenetre *fenetre; /* La fentre associe au message */
  unsigned long compteur;
  int masque = 0, tampon[8], i;
  unsigned int j;

  if (temps == 2) /* On ferme ce qui a t ouvert */
  {
    for (fenetre = _premiere_fenetre ; fenetre ; fenetre = fenetre->suivante)
    {
      if ( (fenetre->numero_arbre == num_arbre) && (fenetre->type == GEM_FENETRE) )
      {
	ferme_fenetre(fenetre);
	return VRAI;
      }
    }
    return -1; /* chec */
  }

  fenetre = creer_gem_fenetre(CLOSER|NAME|MOVER, num_arbre, "Message");
  if (fenetre == NULL)
    return -1; /* Ouverture a choue */

  if (temps == 0) /* On ne fait qu'ouvrir une fentre avec le message */
    return VRAI;

  if (temps > 2) /* C'est que l'on limite  un certain temps */
  {
    compteur = temps * 1000; /* Temps en millisecondes */
    masque |= MU_TIMER; /* On attend le timer */
  }

  if (temps & 1) /* Nombre impair -> on peut aussi quitter par clavier/souris */
  {
    evnt_button(1, 1, 0, &i, &i, &i, &i); /* On attend que le bouton soit relch */
    masque |= MU_KEYBD | MU_BUTTON;
  }

  Evnt_multi(masque, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, tampon,
	     (int) (compteur & 0xFFFF), (int) (compteur >> 16),
	     &i, &i, &i, &i, &j, &i);

  ferme_fenetre(fenetre); /* Fini avec ce message */

  return VRAI;
}

/* ~~~~~~~~~~~~~~~~~~~~ Gestion d'un dfileur ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
 *                                                                                               *
 * Les textes dans le dfileur sont dans un arbre et tous au format STRING                       *
 *************************************************************************************************/

/* -------------------- Prparation du dfileur -------------------------------------------------\
 |    Ne renvoie rien                                                                            |
 |                                                                                               |
 | arbre              : numro de l'arbre o se trouve le dfileur                               |
 | prem_texte         : numro de l'objet contenant le premier texte visible                     |
 | dern_texte         : numro de l'objet contenant le dernier texte visible                     |
 | avant, apres       : numro des flches permettant de monter/descendre                        |
 | fd_asc, bouton_asc : objets composant l'ascenseur                                             |
 | arbre_textes       : numro de l'arbre contenant les textes possibles                         |
 | texte_1            : numro de l'objet contenant le premier texte dans l'arbre ci-dessus      |
 | texte_2            : numro de l'objet contenant le dernier texte dans l'arbre ci-dessus      |
 | num_depart         : numro du premier objet apparaissant dans le dfileur                    |
 | num_select         : numro de l'objet slectionn dans le dfileur                           |
 \---------------------------------------------------------------------------------------------- */

void PRP_defileur(int arbre, int prem_texte, int dern_texte,
		  int avant, int apres, int fd_asc, int bouton_asc,
                  int arbre_textes, int texte_1, int texte_2, int num_depart, int num_select)
{
  float inter; /* Pour l'ascenseur */
  int nbr_textes, nbr_visibles, i;
  OBJECT *adresse;

  adresse = arbres[arbre].adresse;

  nbr_visibles = 1 + dern_texte - prem_texte; /* Nombre de textes visibles */
  nbr_textes   = 1 + texte_2    - texte_1;    /* Nombre de textes au total */

  if (num_depart < texte_1) num_depart = texte_1; /* vite de mauvaises surprises... */
  if (num_depart + nbr_visibles > texte_2) num_depart = texte_2 - nbr_visibles + 1;
  
  if (nbr_visibles >= nbr_textes) /* Moins de textes que de cases */
  {
    OB_STATE(adresse, avant)      |= DISABLED; /* Ascenseur n'a pas de raison d'tre */
    OB_STATE(adresse, apres)      |= DISABLED;
    OB_STATE(adresse, fd_asc)     |= DISABLED;
    OB_STATE(adresse, bouton_asc) |= DISABLED;

    OBJ_position(arbre, bouton_asc, -1, 0, NULL, NULL);
    OBJ_taille(arbre, bouton_asc, -1, OB_HEIGHT(adresse, fd_asc), NULL, NULL);
    
    for (i = 0 ; i < nbr_visibles ; i++)
    {
      if (i < nbr_textes)
      {
	char *chaine = (char *) OB_SPEC(arbres[arbre_textes].adresse, texte_1 + i);
	met_chaine(arbre, prem_texte + i, 0, chaine, TXT_REMPLACE);
	if (num_depart + i == num_select)
	{
	  OB_STATE(adresse, prem_texte + i) |= SELECTED;
	}
	else
	{
	  OB_STATE(adresse, prem_texte + i) &= ~SELECTED;
	}
      }
      else
      {
	met_chaine(arbre, prem_texte + i, 0, "", TXT_REMPLACE);
	OB_STATE(adresse, prem_texte + i) |=  DISABLED;
	OB_STATE(adresse, prem_texte + i) &= ~SELECTED;
      }
    }
    
    return; /* On a fini */
  }
  
  /* On est ici ==> il y a plus de textes que de place disponible */
  
  OB_STATE(adresse, fd_asc)     &= ~DISABLED; /* Ascenseur actif */
  OB_STATE(adresse, bouton_asc) &= ~DISABLED;
  
  if (num_depart == texte_1) /* On est en haut de la liste */
    OB_STATE(adresse, avant) |= DISABLED; /* Rien avant */
  else
    OB_STATE(adresse, avant) &= ~DISABLED; /* Quelque chose avant */
  
  if (num_depart + nbr_visibles - 1 == texte_2)  /* On est en bas de la liste */
    OB_STATE(adresse, apres) |= DISABLED; /* Rien avant */
  else
    OB_STATE(adresse, apres) &= ~DISABLED; /* Quelque chose avant */
  
  for (i = 0 ; i < nbr_visibles ; i++)
  {
    char *chaine = (char *) OB_SPEC(arbres[arbre_textes].adresse, num_depart + i);
    met_chaine(arbre, prem_texte + i, 0, chaine, TXT_REMPLACE);
    if (num_depart + i == num_select)
    {
      OB_STATE(adresse, prem_texte + i) |= SELECTED;
    }
    else
    {
      OB_STATE(adresse, prem_texte + i) &= ~SELECTED;
    }
  }

  /* Reste juste  mettre l'ascenseur o il faut */
  
  inter = (float) (nbr_visibles) / (float) (nbr_textes);
  inter *= OB_HEIGHT(adresse, fd_asc);
  if (inter < 5) inter = 5;                     /* Au moins 5 pixels de haut       */
  OBJ_taille(arbre, bouton_asc, -1, (int) inter, NULL, NULL);
  
  inter  = OB_HEIGHT(adresse, fd_asc) - OB_HEIGHT(adresse, bouton_asc);
  inter *= (num_depart - texte_1);
  inter /= (nbr_textes - nbr_visibles);
  OBJ_position(arbre, bouton_asc, -1, (int) inter, NULL, NULL);
}

/* -------------------- Gestion du dfileur -----------------------------------------------------\
 |    Renvoie le numro de l'option choisie                                                      |
 |                                                                                               |
 | arbre              : numro de l'arbre o se trouve le dfileur                               |
 | prem_texte         : numro de l'objet contenant le premier texte visible                     |
 | dern_texte         : numro de l'objet contenant le dernier texte visible                     |
 | avant, apres       : numro des flches permettant de monter/descendre                        |
 | fd_asc, bouton_asc : objets composant l'ascenseur                                             |
 | arbre_textes       : numro de l'arbre contenant les textes possibles                         |
 | texte_1            : numro de l'objet contenant le premier texte dans l'arbre ci-dessus      |
 | texte_2            : numro de l'objet contenant le dernier texte dans l'arbre ci-dessus      |
 | num_depart         : numro du 1er objet dans le dfileur [retour : nouvelle valeur]          |
 | num_select         : numro de l'objet slectionn [retour : nouvelle valeur]                 |
 | objet              : numro de l'objet que l'on a cliqu                                      |
 | x_souris, y_souris : coordonnes de la souris (pour l'ascenseur)                              |
 \---------------------------------------------------------------------------------------------- */

int GER_defileur(int arbre, int prem_texte, int dern_texte,
		 int avant, int apres, int fd_asc, int bouton_asc,
                 int arbre_textes, int texte_1, int texte_2, int *num_depart, int *num_select,
                 int objet, int x_souris, int y_souris)
{
  float inter; /* Pour l'ascenseur */
  int nbr_textes, nbr_visibles, i;
  OBJECT *adresse;

  adresse = arbres[arbre].adresse;

  nbr_visibles = 1 + dern_texte - prem_texte; /* Nombre de textes visibles */
  nbr_textes   = 1 + texte_2    - texte_1;    /* Nombre de textes au total */

  if (*num_depart < texte_1) *num_depart = texte_1; /* vite de mauvaises surprises... */
  if (*num_depart + nbr_visibles > texte_2) *num_depart = texte_2 - nbr_visibles + 1;

  if (nbr_visibles >= nbr_textes) /* Moins de textes que de cases */
  {
    if ( (objet >= prem_texte) && (objet <= dern_texte) ) /* Seul choix valable... */
    {
      OB_STATE(adresse, *num_select - texte_1 + prem_texte) &= ~SELECTED;
      dessine_objet(arbre, *num_select - texte_1 + prem_texte);
      *num_select = objet - texte_1 + prem_texte;
      OB_STATE(adresse, *num_select - texte_1 + prem_texte) &= ~SELECTED;
      dessine_objet(arbre, *num_select - texte_1 + prem_texte);
    }

    return *num_select; /* On a fini */
  }

  /* On est ici ==> il y a plus de textes que de place disponible */

  if (objet == avant) /* Clic sur l'objet pour dfiler vers le haut */
  {
    char *chaine;
	
    /* On dcale tout d'un cran */
    for (i = nbr_visibles - 1 ; i > 0 ; i--)
    {
      OB_SPEC(adresse, prem_texte + i)  = OB_SPEC(adresse, prem_texte + i - 1);
      OB_STATE(adresse, prem_texte + i) = OB_STATE(adresse, prem_texte + i - 1);
      dessine_objet(arbre, prem_texte + i);
    }
    (*num_depart)--;

    chaine = (char *) OB_SPEC(arbres[arbre_textes].adresse, *num_depart);
    met_chaine(arbre, prem_texte, 0, chaine, TXT_REMPLACE);
    OBJ_choisit(arbre, prem_texte, (*num_depart == *num_select) );

    if (*num_depart == texte_1)
    {
      OBJ_active(arbre, avant, FORCE_NON);
    }
    OB_STATE(adresse, apres) &= ~DISABLED;
    dessine_objet(arbre, apres);
  }
  else if (objet == apres)
  {
    char *chaine;
	
    /* On dcale tout d'un cran */
    for (i = 0 ; i < nbr_visibles - 1 ; i++)
    {
      OB_SPEC(adresse, prem_texte + i)  = OB_SPEC(adresse, prem_texte + i + 1);
      OB_STATE(adresse, prem_texte + i) = OB_STATE(adresse, prem_texte + i + 1);
      dessine_objet(arbre, prem_texte + i);
    }
    (*num_depart)++;

    chaine = (char *) OB_SPEC(arbres[arbre_textes].adresse, *num_depart + nbr_visibles - 1);
    met_chaine(arbre, dern_texte, 0, chaine, TXT_REMPLACE);
    if (*num_depart + nbr_visibles - 1 == *num_select)
    {
      OB_STATE(adresse, dern_texte) |= SELECTED;
    }
    else
    {
      OB_STATE(adresse, dern_texte) &= ~SELECTED;
    }
    dessine_objet(arbre, dern_texte);
    
    if (*num_depart == texte_2 + 1 - nbr_visibles)
    {
      OB_STATE(adresse, apres) |= DISABLED;
      dessine_objet(arbre, apres);
    }
    OB_STATE(adresse, avant) &= ~DISABLED;
    dessine_objet(arbre, avant);
  }
  else if ( (objet >= prem_texte) && (objet <= dern_texte) ) /* Clic dans l'arbre */
  {
    *num_select = objet + *num_depart - prem_texte;
    for (i = 0 ; i < nbr_visibles ; i++)
    {
      if (i + *num_depart == *num_select)
	OB_STATE(adresse, i + prem_texte) |=  SELECTED;
      else
	OB_STATE(adresse, i + prem_texte) &= ~SELECTED;
      dessine_objet(arbre, i + prem_texte);
    }
  }
  else if ( objet == fd_asc ) /* Clic sur le fond de l'ascenseur */
  {
    int y_objet;
    
    objc_offset(adresse, fd_asc, &x_souris, &y_objet);
    if ( y_souris < OB_Y(adresse, bouton_asc) ) /* Clic avant le curseur, monter de nbr_noms - 1 */
    {
      if (*num_depart > nbr_visibles - 1)
	*num_depart -= nbr_visibles - 1;
      else
	*num_depart = 0;
    }
    else
    {
      *num_depart += nbr_visibles - 1;
      if ( *num_depart > texte_2 + 1 - nbr_visibles)
	*num_depart = texte_2 + 1 - nbr_visibles;
    }
    PRP_defileur(arbre, prem_texte, dern_texte, avant, apres, fd_asc, bouton_asc,
		 arbre_textes, texte_1, texte_2, *num_depart, *num_select);
    dessine_objet(arbre, prem_texte - 1);
    dessine_objet(arbre, avant);
    dessine_objet(arbre, apres);
    dessine_objet(arbre, fd_asc);
  }
  else if ( objet == bouton_asc )
  {
    inter  = graf_slidebox(adresse, fd_asc, bouton_asc, 1); /* Position en pour-mille  - vertical */
    inter *= nbr_textes - nbr_visibles;
    inter /= 1000;
    *num_depart = texte_1 + (int) inter;
    if ( *num_depart < texte_1 )
      *num_depart = texte_1;
    if ( *num_depart > texte_2 + 1 - nbr_visibles )
      *num_depart = texte_2 + 1 - nbr_visibles;
    
    PRP_defileur(arbre, prem_texte, dern_texte, avant, apres, fd_asc, bouton_asc,
		 arbre_textes, texte_1, texte_2, *num_depart, *num_select);
    dessine_objet(arbre, prem_texte - 1);
    dessine_objet(arbre, avant);
    dessine_objet(arbre, apres);
    dessine_objet(arbre, fd_asc);
  }
  
  /* On remet l'ascenseur */

  inter  = OB_HEIGHT(adresse, fd_asc) - OB_HEIGHT(adresse, bouton_asc);
  inter *= (*num_depart - texte_1);
  inter /= (nbr_textes - nbr_visibles);
  OBJ_position(arbre, bouton_asc, -1, (int) inter, NULL, NULL);
  dessine_objet(arbre, fd_asc);
  
  return *num_select;
}

/* ~~~~~~~~~~~~~~~~~~~~ Gestion d'un dfileur ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
 *                                                                                               *
 * Les textes dans le dfileur sont dans un tableau de chanes de caractre                      *
 * Possibilit de slectionner plusieurs objets                                                  *
 *************************************************************************************************/

/* -------------------- Prparation du dfileur -------------------------------------------------\
 |    Ne renvoie rien                                                                            |
 |                                                                                               |
 | arbre              : numro de l'arbre o se trouve le dfileur                               |
 | prem_texte         : numro de l'objet contenant le premier texte visible                     |
 | dern_texte         : numro de l'objet contenant le dernier texte visible                     |
 | avant, apres       : numro des flches permettant de monter/descendre                        |
 | fd_asc, bouton_asc : objets composant l'ascenseur                                             |
 | nbr_textes         : nombre de textes dans le tableau des textes
 | textes             : liste des pointeurs sur les chanes des textes
 | choisi             : tableau indiquant la slection pour chaque texte [VRAI : choisi]         |
 |                      NULL : pas de sens -> on s'en fiche, on utilise num_choisi : un seul...
 | num_choisi         : numro de l'objet choisi si UN SEUL objet doit tre choisi
 |                      _par rapport au premier texte_
 | prem_affiche       : numro du premier texte affich dans le dfileur [0...]
 |                      nglig si moins de textes que d'affichables
 | max_long           : longueur maximale autorise pour les chanes (tronque au-del)
 \---------------------------------------------------------------------------------------------- */

void PDFL_textes(int arbre, int prem_texte, int dern_texte, int avant, int apres,
		 int fd_asc, int bouton_asc,
                 unsigned long nbr_textes, char **textes, boolean *choisi,
		 unsigned long num_choisi,
                 int max_long, unsigned long prem_affiche)
{
  float inter; /* Pour l'ascenseur */
  int nbr_visibles, haut_fond, i;

  if (textes == NULL)
    nbr_textes = 0; /* Pas de texte */

  nbr_visibles = 1 + dern_texte - prem_texte; /* Nombre de textes visibles */

  OBJ_taille(arbre, fd_asc, -1, -1, &i, &haut_fond);

  if (nbr_visibles >= nbr_textes) /* Moins de textes que de cases */
  {
    int j;

    OBJ_active(arbre, avant , FORCE_NON); /* Ascenseur n'a pas de raison d'tre */
    OBJ_active(arbre, fd_asc, FORCE_NON);
    OBJ_active(arbre, bouton_asc, FORCE_NON);
    OBJ_active(arbre, apres, FORCE_NON);

    OBJ_position(arbre, bouton_asc, -1, 0, &j, &j);
    OBJ_taille(arbre, bouton_asc, -1, haut_fond, &j, &j);

    for (i = 0 ; i < nbr_textes ; i++)
    {
      OBJ_active(arbre, prem_texte + i, FORCE_OUI);
      met_chaine(arbre, prem_texte + i, max_long, textes[i], TXT_REMPLACE);
      if (choisi)
	OBJ_choisit(arbre, prem_texte + i, choisi[i]);
      else
	OBJ_choisit(arbre, prem_texte + i, i == num_choisi);
    }
    for ( ; i < nbr_visibles ; i++)
    {
      met_chaine(arbre, prem_texte + i, max_long, "", TXT_REMPLACE);
      OBJ_choisit(arbre, prem_texte + i, FORCE_NON);
      OBJ_active(arbre, prem_texte + i, FORCE_NON);
    }

    return; /* On a fini */
  }

  /* On est ici ==> il y a plus de textes que de place disponible */

  OBJ_active(arbre, avant, (prem_affiche > 0) );
  OBJ_active(arbre, fd_asc,     FORCE_OUI); /* Ascenseur actif */
  OBJ_active(arbre, bouton_asc, FORCE_OUI);
  OBJ_active(arbre, apres, (prem_affiche + nbr_visibles) < nbr_textes );

  for (i = 0 ; i < nbr_visibles ; i++)
  {
    OBJ_active(arbre, prem_texte + i, FORCE_OUI);
    met_chaine(arbre, prem_texte + i, max_long, textes[prem_affiche + i], TXT_REMPLACE);
    if (choisi)
      OBJ_choisit(arbre, prem_texte + i, choisi[prem_affiche + i]);
    else
      OBJ_choisit(arbre, prem_texte + i, num_choisi == (prem_affiche + i) );
  }

  /* Reste juste  mettre l'ascenseur o il faut */

  inter = (float) (nbr_visibles) / (float) (nbr_textes);
  inter *= haut_fond;
  if (inter < 5) inter = 5;                       /* Au moins 5 pixels de haut       */
  OBJ_taille(arbre, bouton_asc, -1, (int) inter, &i, &i); /* Position de l'ascenseur */
  
  inter  = haut_fond - (int) inter;
  inter *= prem_affiche;
  inter /= (nbr_textes - nbr_visibles);
  OBJ_position(arbre, bouton_asc, -1, (int) inter, &i, &i); /* Position de l'ascenseur */
}

/* -------------------- Gestion du dfileur -----------------------------------------------------\
 |    Renvoie le numro du texte cliqu _par rapport au premier du slecteur_                    |
 |                                                                                               |
 | arbre              : numro de l'arbre o se trouve le dfileur                               |
 | prem_texte         : numro de l'objet contenant le premier texte visible                     |
 | dern_texte         : numro de l'objet contenant le dernier texte visible                     |
 | avant, apres       : numro des flches permettant de monter/descendre                        |
 | fd_asc, bouton_asc : objets composant l'ascenseur                                             |
 | nbr_textes         : nombre de textes dans le tableau des textes
 | textes             : liste des pointeurs sur les chanes des textes
 | choisi             : tableau indiquant la slection pour chaque texte [VRAI : choisi]         |
 |                      NULL : pas de sens -> on s'en fiche
 | prem_affiche       : numro du 1er objet dans le dfileur [retour : nouvelle valeur]          |
 | num_choisi         : numro de l'objet slectionn [retour : nouvelle valeur]                 |
 |                      _par rapport au premier texte_
 | objet              : numro de l'objet que l'on a cliqu                                      |
 | y_souris           : position de la souris (pour l'ascenseur)                                 |
 \---------------------------------------------------------------------------------------------- */

int GDFL_textes(int arbre, int prem_texte, int dern_texte, int avant, int apres,
		int fd_asc, int bouton_asc,
                unsigned long nbr_textes, char **textes, boolean *choisi,
		unsigned long *num_choisi,
                int max_long, unsigned long *prem_affiche, int objet, int y_souris)
{
  float inter; /* Pour l'ascenseur */
  int nbr_visibles, i;
  unsigned long prem_visible;
  OBJECT *adresse;
  adresse = arbres[arbre].adresse;

  if ( (nbr_textes == 0) || (textes == NULL) ) return -1; /* Aucun objet slectionn... */
  prem_visible = *prem_affiche;
  nbr_visibles = 1 + dern_texte - prem_texte; /* Nombre de textes visibles */

  if (nbr_visibles >= nbr_textes) /* Moins de textes que de cases */
  {
    if ( (objet >= prem_texte) && (objet <= dern_texte) ) /* Seul choix valable... */
    {
      unsigned long num_texte = objet - prem_texte;
      
      if (choisi) /* Slecteur avec slection multiple */
      {
	choisi[num_texte] = !(choisi[num_texte]);
	OBJ_choisit(arbre, (int) num_texte, choisi[num_texte]);
      }
      else
      {
	long ancien_choisi = *num_choisi - prem_visible;
	ancien_choisi += prem_texte;
	
	if ( (ancien_choisi >= prem_texte) && (ancien_choisi <= dern_texte) )
	  OBJ_choisit(arbre, (int) ancien_choisi, FORCE_NON);
	
	*num_choisi = (objet - prem_texte) + prem_visible; /* Nouvel objet choisi */
	OBJ_choisit(arbre, objet, FORCE_OUI);
      }
    }
    
    return objet - prem_texte; /* On a fini */
  }

  /* On est ici ==> il y a plus de textes que de place disponible */

  if ( (objet >= prem_texte) && (objet <= dern_texte) ) /* Clic dans l'arbre */
  {
    unsigned long num_texte = objet - prem_texte;
    
    if (choisi) /* Slecteur avec slection multiple */
    {
      choisi[num_texte] = !(choisi[num_texte]);
      OBJ_choisit(arbre, (int) num_texte, choisi[num_texte]);
    }
    else
    {
      long ancien_choisi = *num_choisi - prem_visible;
      ancien_choisi += prem_texte;
      
      if ( (ancien_choisi >= prem_texte) && (ancien_choisi <= dern_texte) )
	OBJ_choisit(arbre, (int) ancien_choisi, FORCE_NON);
      
      *num_choisi = (objet - prem_texte) + prem_visible; /* Nouvel objet choisi */
      OBJ_choisit(arbre, objet, FORCE_OUI);
    }
    
    return objet - prem_texte;
  }
  
  if ( objet == fd_asc ) /* Clic sur le fond de l'ascenseur */
  {
    int x_objet, y_objet;

    objc_offset(adresse, fd_asc, &x_objet, &y_objet);
    if ( y_souris < OB_Y(adresse, bouton_asc) ) /* Clic avant le curseur: monter de nbr_noms - 1 */
    {
      if (prem_visible > nbr_visibles - 1)
	prem_visible -= nbr_visibles - 1;
      else
	prem_visible = 0;
    }
    else
    {
      prem_visible += nbr_visibles - 1;
      if ( prem_visible > nbr_textes - nbr_visibles)
	prem_visible = nbr_textes - nbr_visibles;
    }

    PDFL_textes(arbre, prem_texte, dern_texte, avant, apres, fd_asc, bouton_asc,
                nbr_textes, textes, choisi, *num_choisi, max_long, prem_visible);
    *prem_affiche = prem_visible;
    dessine_objet(arbre, fd_asc);

    return -1;
  }

  if ( objet == bouton_asc )
  {
    inter  = graf_slidebox(adresse, fd_asc, bouton_asc, 1); /* Position en pour-mille - vertical */
    inter *= nbr_textes - nbr_visibles;
    inter /= 1000;
    prem_visible = (unsigned long) inter;
    if ( prem_visible + nbr_visibles > nbr_textes )
      prem_visible = nbr_textes - nbr_visibles;
    
    PDFL_textes(arbre, prem_texte, dern_texte, avant, apres, fd_asc, bouton_asc,
                nbr_textes, textes, choisi, *num_choisi, max_long, prem_visible);
    *prem_affiche = prem_visible;
    dessine_objet(arbre, fd_asc);
    
    return -1;
  }
  
  if (objet == avant) /* Clic sur l'objet pour dfiler vers le haut */
  {
    for (i = nbr_visibles - 1 ; i > 0 ; i--) /* On dcale tout d'un cran vers le bas */
    {
      OB_SPEC(adresse, prem_texte + i)  = OB_SPEC(adresse, prem_texte + i - 1);
      OB_STATE(adresse, prem_texte + i) = OB_STATE(adresse, prem_texte + i - 1);
      dessine_objet(arbre, prem_texte + i);
    }
    prem_visible--;
    
    met_chaine(arbre, prem_texte, max_long, textes[prem_visible], TXT_REMPLACE);
    if (choisi)
      OBJ_choisit(arbre, prem_texte, choisi[prem_visible]);
    else
      OBJ_choisit(arbre, prem_texte, prem_visible == *num_choisi);
    
    if (prem_visible == 0)
      OBJ_active(arbre, avant, FORCE_NON);
    OBJ_active(arbre, apres, FORCE_OUI);
    *prem_affiche = prem_visible;
  }
  else if (objet == apres)
  {
    for (i = 0 ; i < nbr_visibles - 1 ; i++) /* On dcale tout d'un cran vers le haut */
    {
      OB_SPEC(adresse, prem_texte + i)  = OB_SPEC(adresse, prem_texte + i + 1);
      OB_STATE(adresse, prem_texte + i) = OB_STATE(adresse, prem_texte + i + 1);
      dessine_objet(arbre, prem_texte + i);
    }
    prem_visible++;
    
    met_chaine(arbre, dern_texte, max_long, textes[prem_visible + nbr_visibles - 1], TXT_REMPLACE);
    if (choisi)
      OBJ_choisit(arbre, dern_texte, choisi[prem_visible + nbr_visibles - 1]);
    else
      OBJ_choisit(arbre, dern_texte, *num_choisi == prem_visible + nbr_visibles - 1);
    
    if (prem_visible + nbr_visibles == nbr_textes)
      OBJ_active(arbre, apres, FORCE_NON);
    OBJ_active(arbre, avant, FORCE_OUI);
    *prem_affiche = prem_visible;
  }

  /* On remet l'ascenseur */
  
  inter  = OB_HEIGHT(adresse, fd_asc) - OB_HEIGHT(adresse, bouton_asc);
  inter *= prem_visible;
  inter /= (nbr_textes - nbr_visibles);
  OBJ_position(arbre, bouton_asc, -1, (int) inter, NULL, NULL);
  dessine_objet(arbre, fd_asc);

  return -1;
}

/* ------------------------ Gestion d'un ascenseur entier, dans un arbre GEM ------------------ */

int gere_ascenseur(int num_arbre, int premier, int min, int max, int pas, boolean mode,
		   int obj_clic, int x, int y)
{
  OBJECT *adresse;
  int x_fond,y_fond,larg_asc;
  int valeur;

  adresse = arbres[num_arbre].adresse;

  valeur = prend_valeur(num_arbre,premier + 2); /* Valeur actuelle */
  if (obj_clic == premier)   /* Clic sur flche gauche / basse */
  {
    valeur -= pas; /* Valeur prcdente */
    if (valeur < min) valeur = min;
  }
  else if (obj_clic == premier+1)  /* Clic sur le fond de l'ascenseur */
  {
    objc_offset(adresse, premier + 1, &x_fond, &y_fond); /* Position du fond */
    
    if (mode == VRAI) /* Ascenseur vertical */
    {
      if ( y < OB_Y(adresse, premier + 2) ) /* Au-dessus du poussoir */
      {
	valeur += 10*pas;
	if (valeur > max) valeur = max;
      }
      else
      {
	valeur -= 10*pas;
	if (valeur < min) valeur = min;
      }
    }
    else              /* Ascenseur horizontal */
    {
      if ( x < OB_X(adresse, premier + 2) ) /* Avant le poussoir */
      {
	valeur -= 10*pas;
	if (valeur < min) valeur = min;
      }
      else
      {
	valeur += 10*pas;
	if (valeur > max) valeur = max;
      }
    }
  }
  else if (obj_clic == premier + 2) /* Clic sur le poussoir */
  {
    valeur = graf_slidebox(adresse,premier+1,premier+2,mode);
    if (mode == VRAI) /* Ascenseur vertical */
    {
      valeur = max - valeur * (max - min) / 1000;
    }
    else
    {
      valeur = min + valeur * (max - min) / 1000;
    }
  }
  else if (obj_clic == premier + 3) /* Clic sur la flche droite / haute */
  {
    valeur += pas;
    if (valeur > max) valeur = max;
  }

  /* Positionnement du poussoir */

  met_valeur(num_arbre, premier + 2,valeur); /* Nouvelle valeur      */

  if (mode == VRAI) /* Ascenseur vertical */
  {
    int y_asc;
    larg_asc = OB_HEIGHT(adresse,premier+1) - OB_HEIGHT(adresse,premier+2);
    y_asc = ((float) (max - valeur)) / ((float) (max - min)) * (float) larg_asc;
    OBJ_position(num_arbre, premier + 2, -1, y_asc, &y_asc, &y_asc);
  }
  else
  {
    int x_asc;
    larg_asc = OB_WIDTH(adresse,premier+1) - OB_WIDTH(adresse,premier+2);
    x_asc = ((float) (valeur - min)) / ((float) (max - min)) * (float) larg_asc;
    OBJ_position(num_arbre, premier + 2, x_asc, -1, &x_asc, &x_asc);
  }

  /* tat des flches */

  OBJ_active(num_arbre, premier, valeur > min);
  OBJ_active(num_arbre, premier + 3, valeur < max);

  dessine_objet(num_arbre, premier + 2);     /* On redessine le bouton */
  
  return valeur;
}

/* -------------------- Gestion d'un ascenseur rel, dans un arbre GEM ----------------------- */

float gere_fascenseur(int num_arbre, int premier, float min, float max, float pas, boolean mode,
		      int obj_clic, int x, int y)
{
  OBJECT *adresse;
  int x_fond,y_fond,larg_asc;
  float valeur;

  adresse = arbres[num_arbre].adresse;

  valeur = (float) prend_fvaleur(num_arbre,premier + 2); /* Valeur actuelle */
  if (obj_clic == premier)   /* Clic sur flche gauche / basse */
  {
    valeur -= pas; /* Valeur prcdente */
    if (valeur < min) valeur = min;
  }
  else if (obj_clic == premier+1)  /* Clic sur le fond de l'ascenseur */
  {
    objc_offset(adresse, premier + 1, &x_fond, &y_fond); /* Position du fond */

    if (mode == VRAI) /* Ascenseur vertical */
    {
      if ( y < OB_Y(adresse, premier + 2) ) /* Au-dessus du poussoir */
      {
	valeur += 10*pas;
	if (valeur > max) valeur = max;
      }
      else
      {
	valeur -= 10*pas;
	if (valeur < min) valeur = min;
      }
    }
    else              /* Ascenseur horizontal */
    {
      if ( x < OB_X(adresse, premier + 2) ) /* Avant le poussoir */
      {
	valeur -= 10*pas;
	if (valeur < min) valeur = min;
      }
      else
      {
	valeur += 10*pas;
	if (valeur > max) valeur = max;
      }
    }
  }
  else if (obj_clic == premier + 2) /* Clic sur le poussoir */
  {
    valeur = graf_slidebox(adresse,premier+1,premier+2,mode);
    if (mode == VRAI) /* Ascenseur vertical */
    {
      valeur = max - valeur * (max - min) / 1000;
    }
    else
    {
      valeur = min + valeur * (max - min) / 1000;
    }
  }
  else if (obj_clic == premier + 3) /* Clic sur la flche droite / haute */
  {
    valeur += pas;
    if (valeur > max) valeur = max;
  }

  /* Positionnement du poussoir */

  if (mode == VRAI) /* Ascenseur vertical */
  {
    float pos_asc;

    larg_asc = OB_HEIGHT(adresse,premier+1) - OB_HEIGHT(adresse,premier+2);
    pos_asc  = (float) (max - valeur);
    pos_asc /= (float) (max - min);
    pos_asc *= (float) larg_asc;

    OBJ_position(num_arbre, premier + 2, -1, (int) pos_asc, NULL, NULL);
  }
  else
  {
    float pos_asc;

    larg_asc = OB_WIDTH(adresse,premier+1) - OB_WIDTH(adresse,premier+2);
    pos_asc  = (float) (valeur - min);
    pos_asc /= (float) (max - min);
    pos_asc *= (float) larg_asc;

    OBJ_position(num_arbre, premier + 2 , (int) pos_asc, -1, NULL, NULL);
  }

  /* tat des flches */

  OBJ_active(num_arbre, premier, valeur > min);
  OBJ_active(num_arbre, premier + 3, valeur < max);
  met_fvaleur(num_arbre,premier + 2,valeur); /* Nouvelle valeur      */
  dessine_objet(num_arbre,premier + 1);     /* On redessine le fond */
  dessine_objet(num_arbre,premier + 2);     /* On redessine le fond */

  return valeur;
}

/* --------------- Gestion d'un sous-menu ---------------------------------\
 |  Renvoie l'objet du sous-menu sur lequel on a cliqu (et quitte donc)   |
 |
 | arbre   : arbre servant de sous-menu
 | largeur : largeur de ce sous-menu (<= 0 ==> celle dfinie dans l'arbre) |
 | x, y    : positions d'affichage du sous-menu                            |
 \------------------------------------------------------------------------ */

int GER_ssmenu(int arbre, int largeur, int x, int y)
{
  Fenetre *fenetre;
  OBJECT *adresse;
  boolean sortie = FAUX;
  int objet;
  XEvent evenement;
  Window vx_fenetre;

  adresse = arbres[arbre].adresse;

  if (largeur > 0) /* Largeur impose par l'appelant */
  {
    OB_WIDTH(adresse, ROOT) = largeur;
  }

  fenetre = creer_gem_fenetre(NAME|MOVER, arbre, "Sous-menu - clic pour sortir");
  if (fenetre == NULL) return 0;

  while (sortie == FAUX)
  {
    Fenetre *fen_evenement;

    XNextEvent(_Display, &evenement);

    _Fen_redessin = evenement.xany.window;
    if (_Fen_redessin != vx_fenetre)
    {
      vx_fenetre = _Fen_redessin;
      fen_evenement = quelle_fenetre(_Fen_redessin);
    }

    switch ( evenement.type )
    {
    case KeyRelease: /* On ignore - sauf pour les touches d'tat ? */
      EVNT_relache( &(evenement.xkey) );
      break;

    case Expose:
      EVNT_redessin(fen_evenement, &(evenement.xexpose));
      break;

    case ButtonPress:
      if (fen_evenement == fenetre)
	sortie = VRAI;
      break;
    }
  }

  for (objet = ROOT ; arbres[arbre].objets[objet] != evenement.xbutton.window ; objet++)
    if (objet > arbres[arbre].nbr_objets)
    {
      ferme_fenetre(fenetre);
      return 0;
    }

  ferme_fenetre(fenetre);
  return objet; /* On renvoie l'objet sur lequel on a cliqu pour sortir */
}
