/**** Bibliothque mixte GEM / X_windows
  (C) 1998 Emmanuel CURIS

  Slecteur de fichiers - compatible gem_fen.c [rfrence]
*/

#define _LIBRAIRIE_X_ /* Pour accder aux fonctions internes aussi */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#ifdef _UNIX_

# include <dirent.h> /* Fichier  inclure pour Unix, System V - ex : SunOS, Solaris */
# include <sys/param.h>

# ifdef _SUN_OS_

char *getwd(char *pathname); /* Pas dans les fichiers include ! */

# endif /* _SUN_OS_ */

# ifdef _SOLARIS_
#  include <unistd.h>

char *getwd(char *pathname); /* Pas dans les fichiers include, apparemment ! */

# endif /* _SOLARIS_ */

# ifdef _IRIX_
#  include <unistd.h>
# endif /* _IRIX_ */

# ifdef _LINUX_
#  define __USE_BSD
#  include <unistd.h>
# endif /* _LINUX_ */

#endif

#ifdef _BSD_
# include <sys/dir.h> /* Fichier  inclure pour Unix, BSD 4.3 */
#endif

#include "Bib_X.h"

#include "FSEL.H"
#include "FSEL.RSH"

/* ******** Structures et variables internes ************************************************** */

typedef struct
{
  char *nom_fichier;
  boolean dossier; /* VRAI si c'est un dossier, FAUX sinon */
  boolean choisit; /* VRAI si le fichier est slectionn   */
} Entree_selecteur;

static unsigned long prem_affiche = 0;
static int num_choisi = -1; /* -1 : aucun fichier choisi */

static unsigned long nbr_fichiers = 0;
static unsigned long nbr_dossiers = 0;
static unsigned long nbr_entrees  = 0;
static char **dossiers = NULL;
static char **fichiers = NULL;
static Entree_selecteur *entrees;
static boolean mode_selection = FAUX; /* VRAI : slection multiple autorise */
static char *chemin = NULL; /* Le chemin avec lequel on travaille */

OBJECT *adresse_selecteur = rs_object; /* Adresse du slecteur */

/* ******** Fonctions  usage interne seulement *********************************************** */

static char *X_strdup(const char *nom);
static char *X_strcat(const char *chemin, const char *nom);

static int   X_fliste(const char *chemin);
static int   X_liste (const char *masque);
static int   X_finit (void); /* Initialisation   */
static int   X_ffin  (void); /* Fin du slecteur */
static int   X_fgere (int objet, int x_souris, int y_souris);

static boolean X_gere_selecteur(int num_objet, int x_souris, int y_souris);

/* ******** Ralisation effective des fonctions *********************************************** */

/* ---------- Renvoie la liste des noms des dossiers et des fichiers dans le rep. chemin ---- */

static int X_fliste(const char *chemin)
{
  DIR *repertoire;
  struct dirent *fichier;
  unsigned long /* i_fichier = 0, i_dossier = 0, */ total = 0;
  long debut;

  repertoire = opendir(chemin); /* On ouvre le rpertoire courant */
  if (repertoire == NULL)
  {
    ERREUR(ERREUR_LECTURE);
    return -1;
  }

  debut = telldir(repertoire);

  for ( fichier = readdir(repertoire) ; fichier ; fichier = readdir(repertoire) )
  {
    if ( ( strcmp(fichier->d_name, ".") )
	&& ( strcmp(fichier->d_name, "..") ) )
    {
      total++;
    }
  }

  nbr_dossiers = 0;
  nbr_fichiers = 0;

  if ( total == 0 )
  {
    dossiers = NULL;
    fichiers = NULL;
    return 0;
  }

  dossiers = ( char ** ) calloc( total, sizeof(char *) );
  if (dossiers == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return -MANQUE_MEMOIRE;
  }
  fichiers = dossiers + total; /* Pointe juste aprs le dernier */

  seekdir(repertoire, debut);

  for ( fichier = readdir(repertoire) ; fichier ; fichier = readdir(repertoire) )
  {
    char *complet;
    DIR *tst_dossier;

    complet = X_strcat(chemin, fichier->d_name);
    tst_dossier = opendir(complet);
    free(complet);

    if (tst_dossier)
    {
      if ( ( strcmp(fichier->d_name, "." ) )
	&& ( strcmp(fichier->d_name, "..") ) )
      {
	dossiers[nbr_dossiers] = X_strdup(fichier->d_name);
	dossiers[nbr_dossiers++][0] = 'D';
      }
      closedir(tst_dossier);
    }
    else
    {
      nbr_fichiers++;
      fichiers--; /* On passe au prcdent */
      *fichiers = X_strdup(fichier->d_name);
    }
  }
  /* fichiers pointe maintenant sur le premier fichier... */

/*  printf("Nombre de fichiers : %ld\n", nbr_fichiers);
  printf("Nombre de dossiers : %ld\n", nbr_dossiers); */

  closedir(repertoire);
  return 0; /* Pas d'erreur */
}

/* ------------ Prpare la liste des entres pour le slecteur -------------------------------- */

static int X_liste(const char *masque)
{
  unsigned long i, n_entree = nbr_dossiers; /* Les dossiers apparaissent toujours */

  /* On compte combien de fichier il faut afficher au total */

  n_entree += nbr_fichiers; /* Pour l'instant, pas fin : on affiche tout */

  if (n_entree == 0) /* Aucune entre */
  {
    nbr_entrees = 0;
    prem_affiche = 0;
    return 0; /* Pas de problme */
  }

  entrees = (Entree_selecteur *) calloc(n_entree, sizeof(Entree_selecteur));
  if (entrees == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    nbr_entrees = 0;
    prem_affiche = 0;
    return -MANQUE_MEMOIRE;
  }
  nbr_entrees = n_entree;

  for (i = 0 ; i < nbr_dossiers ; i++)
  {
    entrees[i].nom_fichier = dossiers[i];
    entrees[i].dossier = VRAI;
    entrees[i].choisit = FAUX;
  }

  for ( ; i < n_entree ; i++)
  {
    /* Vrifier que l'on veut bien ce fichier... */

    entrees[i].nom_fichier = fichiers[i - nbr_dossiers];
    entrees[i].dossier = FAUX;
    entrees[i].choisit = FAUX;
  }

  prem_affiche = 0; /* On recommence l'affichage _en haut_ de la liste */

  return 0; /* Pas de problme */
}

/* ----------------------- Effacement de toutes les structures (sauf le chemin) --------------- */

static int X_ffin(void)
{
  unsigned long j;

  free(entrees); /* On efface les anciennes entres */
  for (j = 0 ; j < nbr_dossiers ; j++)
  {
    free(dossiers[j]); /* On efface les anciens noms de dossier */
  }
  for (j = 0 ; j < nbr_fichiers ; j++)
  {
    free(fichiers[j]); /* On efface les anciens noms de dossier */
  }
  free(dossiers); /* Ainsi que la liste */  

  return 0; /* Pas de problme */
}

/* ----------------------- Affichage de la liste des fichiers dans le dfileur ---------------- */

#define MAX_VISIBLES (FSEL_DER_NOM - FSEL_PREM_NOM + 1) /* Nombre max. de fichiers affichable */

static int X_finit(void)
{
  float inter;
  int i;
  int haut_asc;

  OBJ_taille(_nbr_arbres + 1, FSEL_ASC, -1, -1, NULL, &haut_asc);
  OBJ_active(_nbr_arbres + 1, FSEL_AVANT  , FORCE_NON);

  if (nbr_entrees <= MAX_VISIBLES)
  {
    OBJ_active(_nbr_arbres + 1, FSEL_APRES  , FORCE_NON);
    OBJ_active(_nbr_arbres + 1, FSEL_ASC    , FORCE_NON);
    OBJ_active(_nbr_arbres + 1, FSEL_BOUTON, FORCE_NON);

    OBJ_position(_nbr_arbres + 1, FSEL_BOUTON,  0,        0, NULL, NULL);
    OBJ_taille  (_nbr_arbres + 1, FSEL_BOUTON, -1, haut_asc, NULL, NULL);

    for (i = 0 ; i < nbr_entrees ; i++)
    {
      met_chaine (_nbr_arbres + 1, FSEL_PREM_NOM + i, 50, entrees[i].nom_fichier, TXT_REMPLACE);
      OBJ_active (_nbr_arbres + 1, FSEL_PREM_NOM + i, FORCE_OUI);
      OBJ_choisit(_nbr_arbres + 1, FSEL_PREM_NOM + i, FORCE_NON);
    }
    for ( ; i < MAX_VISIBLES ; i++)
    {
      met_chaine (_nbr_arbres + 1, FSEL_PREM_NOM + i, 50, "", TXT_REMPLACE);
      OBJ_active (_nbr_arbres + 1, FSEL_PREM_NOM + i, FORCE_NON);
      OBJ_choisit(_nbr_arbres + 1, FSEL_PREM_NOM + i, FORCE_NON);
    }

    return 0; /* Pas de problme */
  }

  /* On est l: il y a plus d'entres que de place disponible
     On affiche en commenant au premier */

  OBJ_active(_nbr_arbres + 1, FSEL_APRES  , FORCE_OUI);
  OBJ_active(_nbr_arbres + 1, FSEL_ASC    , FORCE_OUI);
  OBJ_active(_nbr_arbres + 1, FSEL_BOUTON, FORCE_OUI);

  OBJ_position(_nbr_arbres + 1, FSEL_BOUTON,  0,        0, NULL, NULL);
  inter = (float) (MAX_VISIBLES) / (float) (nbr_entrees);
  inter *= haut_asc;
  if (inter < 5) inter = 5;                     /* Au moins 5 pixels de haut       */
  OBJ_taille(_nbr_arbres + 1, FSEL_BOUTON, -1, (int) inter, NULL, NULL);

  for (i = 0 ; i < MAX_VISIBLES ; i++)
  {
    met_chaine (_nbr_arbres + 1, FSEL_PREM_NOM + i, 50, entrees[i].nom_fichier, TXT_REMPLACE);
    OBJ_active (_nbr_arbres + 1, FSEL_PREM_NOM + i, FORCE_OUI);
    OBJ_choisit(_nbr_arbres + 1, FSEL_PREM_NOM + i, FORCE_NON);
  }

  prem_affiche = 0;
  return 0;
}

static int X_fgere(int objet, int x_souris, int y_souris)
{
  int haut_asc, haut_bouton;
  unsigned long i;
  float pos_asc;

  if (objet == FSEL_AVANT) /* Clic sur la flche vers le haut */
  {
    prem_affiche--;

    if (prem_affiche == 0)
      OBJ_active(_nbr_arbres + 1, FSEL_AVANT, FORCE_NON);
    OBJ_active(_nbr_arbres + 1, FSEL_APRES, FORCE_OUI);
  }

  if (objet == FSEL_APRES)
  {
    prem_affiche++;

    if (prem_affiche + MAX_VISIBLES == nbr_entrees)
      OBJ_active(_nbr_arbres + 1, FSEL_APRES, FORCE_NON);
    OBJ_active(_nbr_arbres + 1, FSEL_AVANT, FORCE_OUI);    
  }

  if (objet == FSEL_ASC)
  {
    int y_bouton;

    OBJ_position(_nbr_arbres + 1, FSEL_BOUTON, -1, -1, NULL, &y_bouton);
    if (y_souris < y_bouton) /* On est _avant_ le poussoir */
    {
      if (prem_affiche > MAX_VISIBLES)
	prem_affiche -= MAX_VISIBLES;
      else
      {
	prem_affiche = 0;
	OBJ_active(_nbr_arbres + 1, FSEL_AVANT, FORCE_NON);
      }
      OBJ_active(_nbr_arbres + 1, FSEL_APRES, FORCE_OUI);
    }
    else
    {
      prem_affiche += MAX_VISIBLES;
      if (prem_affiche + MAX_VISIBLES >= nbr_entrees)
      {
	prem_affiche = nbr_entrees - MAX_VISIBLES;
	OBJ_active(_nbr_arbres + 1, FSEL_APRES, FORCE_NON);
      }
      OBJ_active(_nbr_arbres + 1, FSEL_AVANT, FORCE_OUI);    
    }
  }

  if (objet == FSEL_BOUTON)
  {
    int nv_pos;

    nv_pos = X_ascenseur( _ARB_fselect->objets[FSEL_ASC],
			  _ARB_fselect->objets[FSEL_BOUTON], VERTICAL);

    prem_affiche = ( nv_pos * (nbr_entrees - MAX_VISIBLES) ) / 1000;
  }

  for (i = 0 ; i < MAX_VISIBLES ; i++)
  {
    met_chaine (_nbr_arbres + 1, FSEL_PREM_NOM + i,
		50, entrees[i + prem_affiche].nom_fichier, TXT_REMPLACE);
    OBJ_active (_nbr_arbres + 1, FSEL_PREM_NOM + i, FORCE_OUI);
    OBJ_choisit(_nbr_arbres + 1, FSEL_PREM_NOM + i, entrees[i + prem_affiche].choisit);
  }

  OBJ_taille(_nbr_arbres + 1, FSEL_ASC, -1, -1, NULL, &haut_asc);
  OBJ_taille(_nbr_arbres + 1, FSEL_BOUTON, -1, -1, NULL, &haut_bouton);

  pos_asc  = haut_asc - haut_bouton;
  pos_asc *= prem_affiche;
  pos_asc /= (nbr_entrees - MAX_VISIBLES);
  OBJ_position(_nbr_arbres + 1, FSEL_BOUTON, -1, (int) pos_asc, NULL, NULL);

  return 0; /* Pas de problme */
}

/* ----------------------- Cration d'un nom de fichier complet  partir de chemin + nom ------ */

static char *X_strcat(const char *chemin, const char *nom)
{
  char *resultat;
  unsigned long j, debut;

  if ( (nom == NULL) || (chemin == NULL) ) return NULL;

  resultat = (char *) malloc(strlen(nom) + strlen(chemin) + 2); /* 0 final et / de sparation */
  if (resultat == NULL)
    return NULL;

  strcpy(resultat, chemin); /* On met le chemin */
  if ( resultat[strlen(resultat) - 1] != '/' )
  {
    j = strlen(resultat);
    resultat[j] = '/'; /* On ajoute un '/' pour marquer le changement */
    resultat[j + 1] = 0;
  }

  debut = strlen(resultat);
  for (j = 0 ; j <= strlen(nom) ; j++) /* On recopie aussi le 0 final */
  {
    resultat[debut + j] = nom[j];
  }

  return resultat;
}

/* ----------------------- Conversion nom de fichier -> entre du slecteur ------------------- */

static char *X_strdup(const char *nom)
{
  char *copie;

  if (nom == NULL) return NULL;

  copie = (char *) malloc( strlen(nom) + 3 ); /* Deux espaces avant + 0 final */
  if (copie)
  {
    memcpy(copie + 2, nom, strlen(nom));
    copie[0] = ' ';
    copie[1] = ' ';
    copie[strlen(nom) + 2] = 0;
  }

  return copie;
}

/* ----------------------- Gestion du slecteur de fichier ------------------------------------ */

static boolean X_gere_selecteur(int objet, int x_souris, int y_souris)
{
  if ( (objet >= FSEL_PREM_NOM) && (objet <= FSEL_DER_NOM) )
  {
    if ( entrees[prem_affiche + objet - FSEL_PREM_NOM].dossier == VRAI ) /* C'est un dossier */
    {
      char *nv_chemin;

      nv_chemin = X_strcat(chemin, entrees[prem_affiche + objet - FSEL_PREM_NOM].nom_fichier + 2);
      if (nv_chemin)
      {
	OBJ_choisit(_nbr_arbres + 1, objet, FORCE_OUI);
	free(chemin);
	chemin = nv_chemin;

	X_ffin(); /* On efface les anciennes infos */
	X_fliste(chemin); /* Nouvelle liste des fichiers & dossiers */
	X_liste("*");
	X_finit(); /* Et on l'affiche */

	met_chaine(_nbr_arbres + 1, FSEL_CHEMIN, 50, chemin, TXT_REMPLACE);
      }

      return VRAI;
    }

    if (mode_selection == VRAI) /* Slection multiple */
    {
      if ( OBJ_choisit(_nbr_arbres + 1, objet, ECHANGE) )
	entrees[prem_affiche + objet - FSEL_PREM_NOM].choisit = VRAI;
      else
	entrees[prem_affiche + objet - FSEL_PREM_NOM].choisit = FAUX;
    }
    else /* Slection simple */
    {
      if (num_choisi) /* Il y avait dj un fichier choisi */
	OBJ_choisit(_nbr_arbres + 1, num_choisi, FORCE_NON);

      num_choisi = objet;
      OBJ_choisit(_nbr_arbres + 1, objet, FORCE_OUI);
    }

    met_chaine(_nbr_arbres + 1, FSEL_FICHIER, 50,
	       entrees[prem_affiche + objet - FSEL_PREM_NOM].nom_fichier + 2, TXT_COPIER);
  }

  switch (objet)
  {
  case FSEL_REMONTE: /* On change le chemin du slecteur */
  {
    if ( strlen(chemin) > 1 ) /* Plus long que '/' -> on n'est pas  la racine */
    {
      unsigned long j;

      if ( chemin[strlen(chemin) - 1] == '/' )
	chemin[strlen(chemin) - 1] = 0;

      j = strlen(chemin) - 1;

      while ( chemin[j] != '/' ) /* On rogne jusqu'au dossier prcdent */
	chemin[j--] = 0;

      X_ffin(); /* On efface les anciennes infos */
      X_fliste(chemin); /* Nouvelle liste des fichiers & dossiers */
      X_liste("*");
      X_finit(); /* Et on l'affiche */

      met_chaine(_nbr_arbres + 1, FSEL_CHEMIN, 50, chemin, TXT_REMPLACE);
    }
    return VRAI;
  }

  case FSEL_AVANT:
  case FSEL_APRES:
  case FSEL_ASC:
  case FSEL_BOUTON:
    X_fgere(objet, x_souris, y_souris);
    return VRAI;

  case FSEL_ANNULER: /* On annule l'ouverture du fichier */
    return -FSEL_ANNULER;

  case FSEL_OK: /* On demande l'ouverture des fichiers */
    return -FSEL_OK;

  }

  return VRAI; /* Par dfaut... */
}

/* ----------------------- Le slecteur proprement dit -----------------------------------------\
 |  Renvoie le nombre de fichiers choisis (< 0 : erreur; 0 : aucun ou annul)                  |
 |                                                                                              |
 | masque    : quels fichiers doit-on afficher ?                                                |
 | titre     : le titre qu'il faut donner au slecteur de fichiers                              |
 | nbr_ext   : nombre d'extensions  installer dans le slecteur d'extensions                   |
 | extension : la liste des extensions  placer                                                 |
 | fichiers  : la liste des fichiers choisis                                                    |
 | multiple  : VRAI si on veut pouvoir faire de la slection multiple                           |
 \--------------------------------------------------------------------------------------------- */

int X_selecteur(char *masque, char *titre, int nbr_ext, char *(*extension)[],
		char ***fichiers, boolean multiple)
{
  int reponse = 0;

  if (chemin == NULL) /* Aucun chemin dfini */
  {
    chemin = (char *) malloc(MAXPATHLEN); /* Pour pouvoir faire des changements de chemin... */
    if (chemin == NULL)
    {
      ERREUR(MANQUE_MEMOIRE);
      printf("dans X_fichiers.c, X_selecteur, chemin courant\n");
      return -MANQUE_MEMOIRE;
    }
    if ( getwd(chemin) == NULL )
    {
      printf("Erreur dans le slecteur : \n %s\n", chemin);
      return -1;
    }
  }

  met_chaine(_nbr_arbres + 1, FSEL_CHEMIN, 50, chemin, TXT_REMPLACE);

  if (titre)
    met_chaine(_nbr_arbres + 1, FSEL_TITRE, 50, titre, TXT_REMPLACE);
  else
  {
#ifdef _FRANCAIS_
    met_chaine(_nbr_arbres + 1, FSEL_TITRE, 50, "S"e_AIGU"lecteur de fichiers", TXT_REMPLACE);
#endif
#ifdef _ENGLISH_
    met_chaine(_nbr_arbres + 1, FSEL_TITRE, 50, "File selector", TXT_REMPLACE);
#endif
  }

  _ARB_fselect->fonction.arbre = X_gere_selecteur;
  mode_selection = multiple;

  /* Liste des fichiers dans le chemin choisi */

  X_fliste((const char *) chemin);

  /* On en dduit la liste des fichiers  afficher */

  X_liste(masque);
  X_finit(); /* Et on l'affiche */

  reponse = ext_formdo(_nbr_arbres + 1);
  if (reponse == FSEL_ANNULER) /* On annule la demande */
  {
    *fichiers = NULL;
    X_ffin();
    return 0; /* Aucun fichier slectionn */
  }

  /* Ici : on a choisi des fichiers */

  if (mode_selection == VRAI) /* Slection multiple autorise */
  {
    int i;
    for (i = 0 ; i < nbr_entrees ; i++)
    {
      if ( (entrees[i].dossier == FAUX) && (entrees[i].choisit == VRAI) )
      {
	reponse++; /* Un fichier choisit de plus */
      }
    }

    if (reponse == 0) /* Aucun fichier slectionn */
    {
      *fichiers = (char **) malloc( sizeof(char *) );
      if (*fichiers == NULL)
      {
	ERREUR(MANQUE_MEMOIRE);
	printf("Dans X_fichiers.c, X_selecteur : nom du fichier dans le champ ditable\n");
      }
      else
      {
	reponse = 1;
	**fichiers = X_strcat(chemin, (_ARB_fselect->adresse)[FSEL_FICHIER].ob_spec.tedinfo->te_ptext);
      }
    }
    else
    {
      *fichiers = (char **) calloc(reponse, sizeof(char *));
      if (*fichiers == NULL)
      {
	ERREUR(MANQUE_MEMOIRE);
	printf("Dans X_fichiers.c, X_selecteur : nom des fichiers slectionns\n");
	reponse = 0;
      }
      else
      {
	reponse = 0;
	for (i = 0 ; i < nbr_entrees ; i++)
	{
	  if ( (entrees[i].dossier == FAUX) && (entrees[i].choisit == VRAI) )
	  {
	    (*fichiers)[reponse] = X_strcat(chemin, entrees[i].nom_fichier + 2);
	    reponse++; /* Un fichier choisit de plus */
	  }
	}
      }
    }
  }
  else /* Slection unique == le nom qui est dans le champ nom... */
  {
    *fichiers = (char **) malloc(sizeof(char *));
    if (*fichiers == NULL)
    {
      ERREUR(MANQUE_MEMOIRE);
      printf("Dans X_fichiers.c, X_selecteur : nom du fichier dans le champ ditable, 1 slection\n");
      reponse = 0;
    }
    else
    {
      char *entre = (_ARB_fselect->adresse)[FSEL_FICHIER].ob_spec.tedinfo->te_ptext;

      if (num_choisi >= 0) /* Il y a effectivement un fichier slectionn */
      {
	char *choix = entrees[prem_affiche + num_choisi - FSEL_PREM_NOM].nom_fichier + 2;
	if ( strcmp(choix, entre) ) /* Utilisateur a tap un nom diffrent */
	  (*fichiers)[0] = X_strcat(chemin, entre);
	else
	  (*fichiers)[0] = X_strcat(chemin, choix);
      }
      else
	(*fichiers)[0] = X_strcat(chemin, entre);

      reponse = 1;
    }
  }

  /* Effacer toutes les structures devenues inutiles */
  X_ffin();

  return reponse;
}

/* ******** Interface avec l'extrieur ******************************************************** */

/* ----------------------- Slecteur spcialis dans la sauvegarde -----------------------------\
 |   Renvoie le nom du fichier choisi (NULL en cas de problme)                                 |
 |         Chane  librer avec free() quand il n'y en a plus besoin...                        |
 |                                                                                              |
 | masque : masque de sauvegarde                                                                |
 | titre  : titre  mettre dans le slecteur                                                    |
 \--------------------------------------------------------------------------------------------- */
 
char *SV_selecteur(char *masque, char *titre)
{
  char **nom_fichier = NULL;
  char *fichier = NULL;
  unsigned long nbr_fichiers;

  nbr_fichiers = X_selecteur(masque, titre, 0, NULL, &nom_fichier, FAUX); /* Slection simple */
  if (nbr_fichiers == 0)
  {
    if (nom_fichier) /* Normalement, devrait jamais arriv */
    {
      printf("SV_selecteur : nom_fichier non NULL alors qu'il n'y a pas de fichier slectionn\n");
      free(nom_fichier);
      nom_fichier = NULL;
    }
    return NULL; /* Aucun fichier choisi */
  }

  fichier = nom_fichier[0];
  free(nom_fichier);
  return fichier;
}

/* ----------------------- Slecteur spcialis dans le chargement ----------------------- *
               Autorise la slection multiple; renvoi un pointeur sur la liste des noms.
               Le nombre de fichiers dans la liste est dans le dernier paramtre                */

char **CH_selecteur(char *masque, char *titre, int nbr_ext, char *(*extension)[],
		    int *nbr_fichiers)
{
  char **nom_fichier = NULL;

  *nbr_fichiers = X_selecteur(masque, titre, nbr_ext, extension, &nom_fichier, VRAI);
  if (*nbr_fichiers == 0)
  {
    if (nom_fichier) /* Normalement, devrait jamais arriv */
    {
      printf("CH_selecteur : nom_fichier non NULL alors qu'il n'y a pas de fichier slectionn\n");
      free(nom_fichier);
      nom_fichier = NULL;
    }
  }

  return nom_fichier;
}
