/* Bibliothque X-Window pour les interfaces
   Routines pour les menus (barre de menu et sous-menus "pop-up")

   (C) 1998, Emmanuel CURIS - Cration le 11 septembre 1998
*/

#define _LIBRAIRIE_X_

#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Bib_X.h"

/* ********** Variables globales limites  ce module ************** */

static XSizeHints *taille_menu = NULL;

/**************** Fonctions internes  ce module *****************/


/* ********** Barre de menu principale *********** */

/* ---------------- Installation d'une barre de menu -----------------\
 |    Remplace la barre existante s'il y en a dj une                |
 |                                                                    |
 | num_arbre : arbre GEM  installer comme menu                       |
 | gestion   : routine de gestion du menu                             |
 \------------------------------------------------------------------- */

int installe_menu(int num_arbre, boolean (*gestion)(int option) )
{
  OBJECT *adresse;
  int x_menu = 1, y_menu = 1, i;
  unsigned int larg_menu, haut_menu;
  int prem_titre, pere_menus;
  int num_titre;
  XSetWindowAttributes parametres;

  adresse = arbres[num_arbre].adresse; /* Adresse du menu  installer */

  prem_titre = OB_HEAD(adresse, ROOT); /* Pointe sur une bote contenant tout ? */

  larg_menu = OB_WIDTH(adresse, prem_titre); /* Largeur : contient tout... */
  haut_menu  = _Fonte->max_bounds.ascent + _Fonte->max_bounds.descent;
  haut_menu += _Fonte->ascent + _Fonte->descent;

  if (_Menu) /* Il y a dj un menu */
  {
    if ( _Menu->adresse == adresse )
    {
      return 0; /* Pas de problme */
    }

    free(_Menu->titres);
    free(_Menu->prem_option);

    XDestroySubwindows(_Display, _Menu->racine);     /* On dtruit les sous-fentres */
    XResizeWindow(_Display, _Menu->racine, larg_menu, haut_menu); /* Nouvelle taille */
  }
  else
  {
    _Menu = (Menu *) malloc( sizeof(Menu) );
    if (_Menu == NULL)
    {
      ERREUR(MANQUE_MEMOIRE);
      printf("dans X_gene.c\n");
      return -MANQUE_MEMOIRE;
    }

    parametres.background_pixel = _Couleurs[BLANC].pixel; /* Fond en blanc */
    parametres.event_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask
      | ButtonReleaseMask | ExposureMask | PointerMotionMask
      | EnterWindowMask | LeaveWindowMask; /* Tous vnements transmis */

    _Menu->racine = XCreateWindow(_Display, _Fen_racine, x_menu, y_menu, larg_menu, haut_menu + 2,
				  0, CopyFromParent, InputOutput, _Visual,
				  CWBackPixel | CWEventMask, &parametres);

#ifdef _FRANCAIS_
    XStoreName(_Display, _Menu->racine, "Barre de menus");
#endif
#ifdef _ENGLISH_
    XStoreName(_Display, _Menu->racine, "Menu bar");
#endif

    taille_menu = XAllocSizeHints();
  }

  if ( taille_menu ) /* A pu tre cr */
  {
    taille_menu->flags = PMinSize | PMaxSize; /* On ne peut pas en changer la taille */
    taille_menu->min_width  = larg_menu;
    taille_menu->min_height = haut_menu + 2;
    taille_menu->max_width  = larg_menu;
    taille_menu->max_height = haut_menu + 2;
    XSetWMNormalHints(_Display, _Menu->racine, taille_menu);
  }

  _Menu->adresse = adresse; /* Adresse de l'arbre pour le menu */
  _PRG_menu = num_arbre;    /* Menu correspondant conserv !   */

  /* On cr les sous-fentres du menu */

  prem_titre = OB_HEAD(adresse, prem_titre);
  _Menu->nbr_titres = OB_TAIL(adresse, prem_titre) - OB_HEAD(adresse, prem_titre) + 1;
 
  _Menu->titres = (Window *) calloc( _Menu->nbr_titres, sizeof(Window) );
  if (_Menu->titres == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("dans X_menu.c\n");
    XDestroyWindow(_Display, _Menu->racine);
    free(_Menu);
    _Menu = NULL;
    return -MANQUE_MEMOIRE; /* Erreur : pas assez de fentres */
  }
  _Menu->nbr_options = (int *) calloc( _Menu->nbr_titres, sizeof(int) );
  if (_Menu->nbr_options == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("dans X_menu.c\n");
    XDestroyWindow(_Display, _Menu->racine);
    free(_Menu->titres);
    free(_Menu);
    _Menu = NULL;
    return -MANQUE_MEMOIRE; /* Erreur : pas assez de fentres */    
  }
  _Menu->prem_option = (int *) calloc( _Menu->nbr_titres, sizeof(int) );
  if (_Menu->prem_option == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("dans X_gene.c\n");
    XDestroyWindow(_Display, _Menu->racine);
    free(_Menu->nbr_options);
    free(_Menu->titres);
    free(_Menu);
    _Menu = NULL;
    return -MANQUE_MEMOIRE; /* Erreur : pas assez de fentres */    
  }

  prem_titre = OB_HEAD(adresse, prem_titre); /* On se place sur le premier titre */
  _Menu->prem_titre = prem_titre;

  parametres.background_pixel = _Couleurs[GRIS].pixel;
  parametres.event_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask
    | ButtonReleaseMask | ExposureMask | PointerMotionMask
    | EnterWindowMask | LeaveWindowMask; /* Tous vnements transmis */

  x_menu = 0; y_menu = 0;
  for (i = 0 ; i < _Menu->nbr_titres ; i++)
  {
    larg_menu  = OB_WIDTH(adresse, prem_titre + i);
    OB_HEIGHT(adresse, prem_titre + i) = haut_menu;

    _Menu->titres[i] = XCreateWindow(_Display, _Menu->racine, x_menu, y_menu, larg_menu, haut_menu,
				     1, CopyFromParent, InputOutput, _Visual,
				     CWBackPixel | CWEventMask, &parametres);

    x_menu += larg_menu + 2;
  }

  /* On repre, pour chaque menu, la premire option et le nombre */

  prem_titre += i; /* G_IBOX aprs le dernier titre */
  pere_menus = prem_titre;

  for (i = OB_HEAD(adresse, prem_titre), num_titre = 0 ;
       i != pere_menus ;
       i = OB_NEXT(adresse, i), num_titre++)
  {
    int prem_option, dern_option;

    prem_option = OB_HEAD(adresse, i); /* Premire option pointe */
    dern_option = OB_TAIL(adresse, i); /* Dernire option pointe */
    (_Menu->prem_option)[num_titre] = prem_option;
    (_Menu->nbr_options)[num_titre] = dern_option - prem_option + 1;
  }

  _Menu->gestion = gestion; /* Fonction de gestion du menu */

  /* On affiche le menu */

  XMapWindow(_Display, _Menu->racine);
  XMapSubwindows(_Display, _Menu->racine);

  return 0; /* Pas d'erreur */
}

/* ---------------- Suppression de la barre de menu ---------------- */

int supprime_menu(void)
{
  XDestroySubwindows(_Display, _Menu->racine);
  XDestroyWindow(_Display, _Menu->racine);

  if (taille_menu) /* On avait pu bloquer la taille avec les XSizeHints */
    XFree(taille_menu);

  free(_Menu->prem_option);
  free(_Menu->nbr_options);
  free(_Menu->titres);
  free(_Menu);
  _Menu = NULL;

  return 0; /* Tout s'est bien pass */
}

/* ~~~~~~~~ Gestion d'un menu : droulement et tout et tout ~~~~~~~~~~
   Renvoie <0 si erreur
            0 si menu quitt sans option appele
	   >0 numro d'objet de l'option appele                       */

int CLIC_menu(int num_titre)
{
  int sortie = FAUX; /* VRAI si on sort */
  int i, x_menu, y_menu;
  unsigned int larg_menu, haut_menu, _rien_; /* du menu qui se droule */
  int prem_option, opt_choisie = -1;
  Window  racine;
  Window *options;
  XSetWindowAttributes parametres;
  XSizeHints *taille_fenetre;

  OB_STATE(_Menu->adresse, _Menu->prem_titre + num_titre) |= SELECTED;
  OBJ_dessine(_Menu->titres[num_titre], _Menu->adresse + _Menu->prem_titre + num_titre);

  prem_option = _Menu->prem_option[num_titre];

  options = (Window *) calloc(_Menu->nbr_options[num_titre], sizeof(Window));
  if (options == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("dans X_menu.c, CLIC_menu\n");
    OB_STATE(_Menu->adresse, _Menu->prem_titre + num_titre) &= ~SELECTED;
    OBJ_dessine(_Menu->titres[num_titre], _Menu->adresse + _Menu->prem_titre + num_titre);
    return -MANQUE_MEMOIRE;
  }

  /* Infos sur la fentre des titres, pour placer la nouvelle fentre */

  racine = _Menu->titres[num_titre]; /* Fentre du titre */
  x_menu = 0; y_menu = 0;
  while (racine != _Fen_racine)
  {
    int x, y;
    Window w, *fils;

    XGetGeometry(_Display, racine, &w, &x, &y, &larg_menu, &haut_menu,
		 &_rien_, &_rien_);

    x_menu += x;
    y_menu += y;
    if ( racine == _Menu->titres[num_titre] )
      y_menu += haut_menu;

    XQueryTree(_Display, racine, &w, &racine, &fils, &_rien_);
    XFree(fils);
  }

  larg_menu = OB_WIDTH(_Menu->adresse, prem_option - 1); /* Dimensions du menu */
  haut_menu = OB_HEIGHT(_Menu->adresse, prem_option - 1);

  parametres.background_pixel  = _Couleurs[GRIS].pixel;
  parametres.border_pixel = _Couleurs[NOIR].pixel;
  parametres.override_redirect = False /* True */;
  parametres.event_mask = ButtonPressMask | ButtonReleaseMask | ExposureMask | KeyPressMask;

  racine = XCreateWindow(_Display, _Fen_racine, x_menu, y_menu, larg_menu, haut_menu + 2,
			 2, CopyFromParent, InputOutput, _Visual,
			 CWBackPixel | CWEventMask | CWOverrideRedirect | CWBorderPixel,
			 &parametres);

  taille_fenetre = XAllocSizeHints(); /*  librer  la fin avec Xfree() */
  if ( taille_fenetre ) /* A pu tre cr */
  {
    taille_fenetre->flags = USPosition | PPosition | PMinSize | PMaxSize;
    taille_fenetre->x = x_menu;
    taille_fenetre->y = y_menu;
    taille_fenetre->min_width  = larg_menu;
    taille_fenetre->min_height = haut_menu;
    taille_fenetre->max_width  = larg_menu;
    taille_fenetre->max_height = haut_menu;
    XSetWMNormalHints(_Display, racine, taille_fenetre);
  }

  XStoreName(_Display, racine, (_Menu->adresse)[_Menu->prem_titre + num_titre].ob_spec.texte );
  XSetTransientForHint(_Display, racine, _Menu->titres[num_titre]);

  parametres.event_mask |= EnterWindowMask | LeaveWindowMask;

  for (i = 0 ;
       i < _Menu->nbr_options[num_titre] ; i++)
  {
    options[i] = XCreateWindow(_Display, racine, 
			       OB_X(_Menu->adresse, i + _Menu->prem_option[num_titre]),
			       OB_Y(_Menu->adresse, i + _Menu->prem_option[num_titre]),
			       OB_WIDTH(_Menu->adresse, i + _Menu->prem_option[num_titre]),
			       OB_HEIGHT(_Menu->adresse, i + _Menu->prem_option[num_titre]),
			       0, CopyFromParent, InputOutput, _Visual,
			       CWBackPixel | CWEventMask, &parametres); 
  }

  XMapSubwindows(_Display, racine);
  XMapWindow(_Display, racine);

  while (sortie == FAUX)
  {
    XEvent evenement;

    XNextEvent(_Display, &evenement);

    switch (evenement.type)
    {
    case KeyPress:
      sortie = VRAI;
      break;

    case KeyRelease:
      EVNT_relache( &(evenement.xkey) );
      break;

    case Expose:
      for (i = 0 ; i < _Menu->nbr_options[num_titre] ; i++)
      {
	if (evenement.xexpose.window == options[i])
	{
	  OBJ_dessine(evenement.xexpose.window, 
		      _Menu->adresse + _Menu->prem_option[num_titre] + i);
	  /* Et on limine tous les messages de redessin de mme type */
	  while ( XCheckTypedWindowEvent(_Display, evenement.xexpose.window,
					 Expose, &evenement) );
	  break;
	}
      }
      if ( (i >= _Menu->nbr_options[num_titre]) /* Non trait */
	   && (evenement.xexpose.window != racine) )
      {
	_Fen_redessin = evenement.xexpose.window;
	EVNT_redessin(quelle_fenetre(_Fen_redessin), (XExposeEvent *) &evenement );
      }

      break;

    case ButtonPress:
      for (i = 0 ; i < _Menu->nbr_options[num_titre] ; i++)
      {
	if (evenement.xbutton.window == options[i])
	{
	  ( (_Menu->adresse)[_Menu->prem_option[num_titre] + i]).ob_type = G_STRING;
	  opt_choisie = i;
	  sortie = VRAI; /* On a fini */
	  break;
	}
      }
      /* Autre fentre : on ferme le menu le clic */
      if ( i >= _Menu->nbr_options[num_titre] )
      {
	int j; /* Pour vrifier que le clic n'est pas sur un autre titre */

	for (j = 0 ; j < _Menu->nbr_titres ; j++)
	{
	  if (evenement.xbutton.window == _Menu->titres[j])
	  {
	    if ( ! ( OB_STATE(_Menu->adresse, _Menu->prem_titre + j) & DISABLED ) )
	      opt_choisie = - (100 + j); /* Seulement si titre non dsactiv */
	    break;
	  }
	}
	if ( j >= _Menu->nbr_titres )
	  opt_choisie = -1;
	if ( (opt_choisie > -99) || ( -opt_choisie - 100 != num_titre ) )
	  sortie = VRAI; /* On a cliqu sur autre chose que le titre du menu en cours */
      }
      
      break;

    case EnterNotify:
      for (i = 0 ; i < _Menu->nbr_options[num_titre] ; i++)
      {
	if ( (evenement.xexpose.window == options[i])
	  && (((_Menu->adresse)[_Menu->prem_option[num_titre] + i]).ob_state != DISABLED) )
	{
	  ((_Menu->adresse)[_Menu->prem_option[num_titre] + i]).ob_type = G_BUTTON;
	  OBJ_dessine(evenement.xexpose.window, 
		      _Menu->adresse + _Menu->prem_option[num_titre] + i);
	  break;
	}
      }
      break;

    case LeaveNotify:
      for (i = 0 ; i < _Menu->nbr_options[num_titre] ; i++)
      {
	if (evenement.xexpose.window == options[i])
	{
	  ((_Menu->adresse)[_Menu->prem_option[num_titre] + i]).ob_type = G_STRING;
	  OBJ_dessine(evenement.xexpose.window, 
		      _Menu->adresse + _Menu->prem_option[num_titre] + i);
	  break;
	}
      }
      break;
    }
  }

  XFree(taille_fenetre);
  FEN_ote_evenement(racine); /* On te tous les vnements associs */
  XDestroyWindow(_Display, racine); /* Plus besoin de a */

  if (opt_choisie == -1) /* On annule en fait l'appel... */
  {
    OB_STATE(_Menu->adresse, _Menu->prem_titre + num_titre) &= ~SELECTED;
    OBJ_dessine(_Menu->titres[num_titre], _Menu->adresse + _Menu->prem_titre + num_titre);
    return VRAI; /* Trait */
  }

  if (opt_choisie < -99) /* On change de menu... */
  {
    OB_STATE(_Menu->adresse, _Menu->prem_titre + num_titre) &= ~SELECTED;
    OBJ_dessine(_Menu->titres[num_titre], _Menu->adresse + _Menu->prem_titre + num_titre);
    return opt_choisie; /* Trait */
  }

  opt_choisie += _Menu->prem_option[num_titre];
  if ( OB_STATE(_Menu->adresse, opt_choisie) & DISABLED ) /* Non actif -> rien */
  {
    OB_STATE(_Menu->adresse, _Menu->prem_titre + num_titre) &= ~SELECTED;
    OBJ_dessine(_Menu->titres[num_titre], _Menu->adresse + _Menu->prem_titre + num_titre);
    return VRAI; /* Trait */
  }

  if (_Menu->gestion) /* Il y a une routine de gestion du menu */
  {
    int retour = (_Menu->gestion)(opt_choisie);

    OB_STATE(_Menu->adresse, _Menu->prem_titre + num_titre) &= ~SELECTED;
    OBJ_dessine(_Menu->titres[num_titre], _Menu->adresse + _Menu->prem_titre + num_titre);
    return retour;
  }
  else /* On s'envoie un message en interne */
  {
    buffer[0] = MN_SELECTED; /* Message : menu appel */
    buffer[1] = 0;
    buffer[2] = 0;
    buffer[3] = _Menu->prem_titre + num_titre;
    buffer[4] = opt_choisie;
  }

  OB_STATE(_Menu->adresse, _Menu->prem_titre + num_titre) &= ~SELECTED;
  OBJ_dessine(_Menu->titres[num_titre], _Menu->adresse + _Menu->prem_titre + num_titre);

  return VRAI; /* Trait */
}
