/* Bibliothque X-Window pour les interfaces
            Routines de gestion des objets GEM [~rcriture de l'AES...]
	    Fichier  inclure
	    Compatible avec gem_gene.c (version GEM : rfrence)
   (C) 1998, Emmanuel CURIS

 31 avril 1998
   * Modification du ob_spec : union de types -> plus propre...

 26 mars 1998
   * Cration du fichier
*/

#ifndef _X_OBJETS_H_ /* Protection contre l'inclusion multiple */

#define _X_OBJETS_H_ 

/* -------- Infos supplmentaires : objets de type texte */

#define IBM              3 /* font types */
#define SMALL            5

#define TE_LEFT          0 /* editable text justification */
#define TE_RIGHT         1
#define TE_CNTR          2

typedef struct text_edinfo
{
  char *te_ptext;     /* ptr to text (must be 1st) -
			  [] pour avoir droit d'y crire mme si dclar dans le programme */
  char *te_ptmplt;    /* ptr to template               */
  char *te_pvalid;    /* ptr to validation chrs.       */
  int  te_font;           /* font                          */
  int  te_junk1;          /* junk word                     */
  int  te_just;           /* justification- left, right... */
  unsigned int te_color;  /* color information word        */
  int  te_junk2;          /* junk word                     */
  int  te_thickness;      /* border thickness              */
  int  te_txtlen;         /* length of text string         */
  int  te_tmplen;         /* length of template string     */
} TEDINFO;

typedef struct icon_block
{
  char *ib_pmask;       /* ptr to mask of icon                */
  char *ib_pdata;       /* ptr to data of icon                */
  char *ib_ptext;       /* ptr to text of icon                */
  unsigned int ib_char; /* character in icon                  */
  int   ib_xchar;       /* x-coordinate of ib_char            */
  int   ib_ychar;       /* y-coordinate of ib_char            */
  int   ib_xicon;       /* x-coordinate of icon               */
  int   ib_yicon;       /* y-coordinate of icon               */
  int   ib_wicon;       /* width of icon in pixels            */
  int   ib_hicon;       /* height of icon in pixels           */
  int   ib_xtext;       /* x-coordinate of the icon's text    */
  int   ib_ytext;       /* y-coordinate of the icon's text    */
  int   ib_wtext;       /* width of rectangle for icon's text */
  int   ib_htext;       /* height of icon's text in pixels    */
} ICONBLK;

typedef struct cicon_data
{
  int num_planes; /* number of planes in the following data          */
  int *col_data;
  int *col_mask;  /* pointer to single plane mask of col_data        */
  int *sel_data;  /* pointer to color bitmap of selected icon        */
  int *sel_mask;  /* pointer to single plane mask of selected icon   */
  struct cicon_data *next_res;	/* pointer to next icon for a different resolution */
} CICON;

typedef struct cicon_blk
{
  ICONBLK monoblk; /* default monochrome icon                         */
  CICON *mainlist; /* list of color icons for different resolutions */
} CICONBLK;

typedef struct bit_block
{
  int *bi_pdata;   /* ptr to bit forms data  */
  int  bi_wb;      /* width of form in bytes */
  int  bi_hl;      /* height in scan lines   */
  int  bi_x;       /* source x in bit form   */
  int  bi_y;       /* source y in bit form   */
  int  bi_color;   /* fg color of blt        */
} BITBLK;

struct parm_blk; /* La dfinition suit */

typedef struct user_blk
{
  int (*ub_code) (struct parm_blk *pb); /* pointer to drawing function */
  long ub_parm;              /* parm for drawing function   */
} USERBLK;

typedef struct
{
  int ob_next;  /* Numro de l'objet suivant */
  int ob_head;  /* Premier objet fils        */
  int ob_tail;  /* Dernier objet fils        */
  int ob_type;  /* Nature de l'objet         */
  int ob_flags; /* Indicateurs divers */
  int ob_state; /* tat de l'objet */
  union
  {
    char     *texte;      /* Pour Button, String, Title            */
    TEDINFO  *tedinfo;    /* Pour Text, Boxtext, Ftext et FBoxtext */
    USERBLK  *util;       /* Pour Userdef                          */
    ICONBLK  *icone;      /* Pour Icon                             */
    CICONBLK *icone_coul; /* Pour les icnes couleur, Cicon        */
    long      autre;
  } ob_spec; /* Caractristiques diverses */
  int ob_x;     /* Position de l'objet RELATIVE AU PRE */
  int ob_y;
  int ob_width; /* Dimensions de l'objet */
  int ob_height;
} OBJECT;

#define OB_NEXT(arbre, id)     arbre [id].ob_next
#define OB_HEAD(arbre, id)     arbre [id].ob_head
#define OB_TAIL(arbre, id)     arbre [id].ob_tail
#define OB_TYPE(arbre, id)    (arbre [id].ob_type & 0xff)
#define OB_EXTYPE(arbre, id)  (arbre [id].ob_type >> 8)
#define OB_FLAGS(arbre, id)    arbre [id].ob_flags
#define OB_STATE(arbre, id)    arbre [id].ob_state
#define OB_SPEC(arbre, id)     arbre [id].ob_spec.autre /* Par compatibilit... */
#define OB_X(arbre, id)        arbre [id].ob_x
#define OB_Y(arbre, id)        arbre [id].ob_y
#define OB_WIDTH(arbre, id)    arbre [id].ob_width
#define OB_HEIGHT(arbre, id)   arbre [id].ob_height

typedef struct parm_blk
{
  OBJECT *pb_tree;                    /* ptr to obj tree for user defined obj */
  int     pb_obj;                     /* index of user defined object         */
  int     pb_prevstate;               /* old state to be changed              */
  int     pb_currstate;               /* changed (new) state of object        */
  int     pb_x, pb_y, pb_w, pb_h;     /* location of object on screen         */
  int     pb_xc, pb_yc, pb_wc, pb_hc; /* current clipping rectangle on screen */
  long    pb_parm;                    /* same as ub_parm in USERBLK struct    */
} PARMBLK;

/* -------- Les diffrents types d'objets possibles -------- */

#define ROOT 0
#define NIL             -1 /* nil object index */
#define MAX_LEN         81 /* max string length */
#define MAX_DEPTH        8 /* max depth of search or draw for objects */

#define G_CLOSER        -1 /* Objets de type interne */
#define G_INFOS         -2
#define G_FULLER        -3
#define G_FLECHE_HAUT   -4
#define G_ASC_VERT      -5
#define G_FLECHE_BAS    -6
#define G_FLECHE_DROITE -7
#define G_ASC_HOR       -8
#define G_FLECHE_GAUCHE -9
#define G_ICONIFIER     -10

#define G_BOX           20 /* graphic types of obs */
#define G_TEXT          21
#define G_BOXTEXT       22
#define G_IMAGE         23
#define G_USERDEF       24
#define G_PROGDEF       24 /* for compatibility */
#define G_IBOX          25
#define G_BUTTON        26
#define G_BOXCHAR       27
#define G_STRING        28
#define G_FTEXT         29
#define G_FBOXTEXT      30
#define G_ICON          31
#define G_TITLE         32
#define G_CICON         33

/* -------- Attributs possibles des objets -------- */

#define NONE        0x0000 /* Object flags */
#define SELECTABLE  0x0001
#define DEFAULT     0x0002
#define EXIT        0x0004
#define EDITABLE    0x0008
#define RBUTTON     0x0010
#define LASTOB      0x0020
#define TOUCHEXIT   0x0040
#define HIDETREE    0x0080
#define INDIRECT    0x0100

/* -------- tats possibles des objets ---------- */

#define NORMAL      0x0000 /* Object states */
#define SELECTED    0x0001
#define CROSSED     0x0002
#define CHECKED     0x0004
#define DISABLED    0x0008
#define OUTLINED    0x0010
#define SHADOWED    0x0020
#define WHITEBAK    0x0040
#define DRAW3D      0x0080

/* -------- Infos supplmentaires sur les icnes -------- */

/* --- Paramtres de objc_edit() --- */

#define ED_INIT        1
#define ED_CHAR        2
#define ED_END         3

int ext_formdo(int num_arbre);

#define FORCE_NON 0 /* On force pour que l'objet ne soit pas dans cet tat */
#define FORCE_OUI 1 /* On force l'objet  tre dans cet tat               */
#define ECHANGE   2 /* On change l'objet d'tat                            */
#define QUESTION  3 /* On ne fait que demande quel est l'tat actuel       */

boolean OBJ_active  (int arbre, int objet, boolean mode);
boolean OBJ_cache   (int num_arbre, int num_objet, boolean mettre);
boolean OBJ_choisit (int arbre, int objet, boolean mode);
void    OBJ_position(int arbre, int objet, int x, int y, int *nv_x, int *nv_y);
void    OBJ_taille  (int arbre, int objet, int larg, int haut, int *nv_larg, int *nv_haut);

int OBJ_dessine(Window fenetre, OBJECT *objet);

/****************** Redessin d'un objet GEM *******************
 *                                                            *
 * arbre  : arbre  redessiner                                *
 * objet  : objet  redessiner                                *
 **************************************************************/

void dessine_objet(int arbre, int objet);

/****************** criture d'une valeur numrique ***********
 *                                                            *
 * arbre  : arbre o crire la valeur                         *
 * objet  : objet de cet arbre o crire la valeur            *
 * valeur : valeur  crire                                   *
 **************************************************************/

#define TXT_COPIER   0
#define TXT_REMPLACE 1

void met_chaine  (int arbre, int objet, unsigned long l_max, char *texte, char mode);
void met_valeur  (int arbre, int objet, int  valeur);
void met_lvaleur (int arbre, int objet, long valeur);
void met_ulvaleur(int arbre, int objet, unsigned long valeur);
void met_fvaleur (int arbre, int objet, double valeur);
void met_svaleur (int arbre, int objet, double valeur);

/******************* Lecture d'une valeur numrique ***********
 *  Renvoie la valeur lue                                     *
 *                                                            *
 * arbre : arbre o lire la valeur                            *
 * objet : objet de cet arbre o lire la valeur               *
 **************************************************************/

char         *lit_chaine    (int arbre, int objet, unsigned long l_max, char *texte, char mode);
int           prend_valeur  (int arbre, int objet);
long          prend_lvaleur (int arbre, int objet);
unsigned long prend_ulvaleur(int arbre, int objet);
float         prend_fvaleur (int arbre, int objet);
double        prend_svaleur (int arbre, int objet);

#ifdef _LIBRAIRIE_X_ /* Fonctions internes  la librairie X */

int USR_separateur(struct parm_blk *parametres); /* Dessin d'un sparateur (ligne hor. ou ver.) */

/* -------- Fonctions utilises dans le dessin des objets -------- */

char *faire_chaine     (TEDINFO *tedinfo);
void  DOBJ_boite       (Window fenetre, OBJECT *objet);
void  DOBJ_bouton      (Window fenetre, OBJECT *objet);
void  DOBJ_boxchar     (Window fenetre, OBJECT *objet);
void  DOBJ_chaine      (Window fenetre, OBJECT *objet);
void  DOBJ_interne     (Window fenetre, OBJECT *objet);
int   OBJ_der_caractere(TEDINFO *tedinfo);

#endif /* _LIBRAIRIE_X_ */

#endif /* _X_OBJETS_H_ */
