/* Bibliothque GEM / X-Windows
   (C) 1998, Emmanuel CURIS

   Chargement des ressources : coeur; prvu pour plusieurs ressources ds l'origine...

   Inspir, en partie, de xrsrc.c par Oliver Groeger & Olaf Meisiek

 10 avril 1998
   * Fonctions RSC_creer() et RSC_effacer()

 1er avril 1998
   * Dbut du travail
*/

#define _LIBRAIRIE_X_

#include <errno.h>
#include <memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "Bib_X.h"

#if ISO_8859_1 /* Codage latin-1 et non atari */

const char conv_8859_1[128] = 
{
                                          '', '', /* 128 - 129 */
  '', '', '', '', '', '', '', '', '', '', /* 130 - 139 */
  '', '', '', '', '', '', '', '', '', '', /* 140 - 149 */
  '', '', '', '', '', '', '', '', '', 'f', /* 150 - 159 */
  '', '', '', '', '', '', '', '', '', '?', /* 160 - 169 */
  '', '', '', '', '', '', '', '', '', '', /* 170 - 179 */
  '?', '?', '', '', '', '', '', '', '', '', /* 180 - 189 */
  '', '?', '?', '?', '?', '?', '?', '?', '?', '?', /* 190 - 199 */
  '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', /* 200 - 209 */
  '?', '?', '?', '?', '?', '?', '?', '?', '?', '?', /* 210 - 219 */
  '?', '', '?', '?', 'a', '', 'G', 'p', 'S', '?', /* 220 - 229 */
  '', 't', 'F', 't', 'O', 'd', 'f', 'f', 'E', '?', /* 230 - 239 */
  '=', '', '>', '<', '?', '?', '', '~', '', '?', /* 240 - 249 */
  '', '?', 'n', '', '', ''                      /* 250 - 255 */
};

#endif /* ISO_8859_1 */

#ifdef _XRSRC_ORIGINAL_

/****** Compilerswitches ******************************************************/

#define COLOR_ICONS FALSE	/* icnes couleur ?                       */
#define SAVE_MEMORY FALSE	/* rserver de la place pour les icnes ? */

/****** TYPES ****************************************************************/

#ifndef __MYDIAL__
typedef struct
{
  USERBLK	ublk;
  UWORD		old_type;
} OBBLK;
#endif

/****** VARIABLES ************************************************************/

static int  xgl_wbox, xgl_hbox;
static GRECT xdesk;

#if COLOR_ICONS == TRUE
WORD xvdi_handle;
#endif

static int    *rs_global;
static RSXHDR *rs_hdr;
static RSXHDR *hdr_buf;

#if COLOR_ICONS == TRUE
WORD    farbtbl[256][32];
ULONG   farbtbl2[32];
WORD    is_palette;
WORD    rgb_palette[256][4];
WORD    xpixelbytes;
WORD    xscrn_planes;
#endif

#endif /* _XRSRC_ORIGINAL_ */

static Ressource *premier_RSC = NULL;

/****** FUNCTIONS ************************************************************/

boolean RSC_fix_objets (Ressource *ressource, RSC_infos *infos);
boolean RSC_fix_tedinfo(Ressource *ressource, long nbr, long debut_texte);

boolean RSC_lit_chaines (FILE *fichier, unsigned char *chaines, size_t nbr_cars);
boolean RSC_lit_objets  (FILE *fichier, long index_dep, long nbr, OBJECT  *adresse);
boolean RSC_lit_tedinfos(FILE *fichier, long index_dep, long nbr, TEDINFO *adresse);

static int RSC_lit_mot     (FILE *fichier);
static long RSC_lit_longmot(FILE *fichier);

#ifdef LITTLE_ENDIAN

/* Conversion d'un ULONG atari vers un unsigned long intel */

static unsigned long RSC_UL_v_ul(ULONG valeur)
{
  ULONG convertit;

  convertit = ((valeur >> 24) & 0xFF) | ((valeur >> 8) & 0xFF00)
    | ((valeur << 8) & 0xFF0000) | ((valeur << 24) & 0xFF000000);
  return (unsigned long) convertit;  
}

#endif

/* --------------- Cration des infos ncesaires  un nouveau ressource ----------------- */

Ressource *RSC_creer(void)
{
  Ressource *ressource;

  ressource = (Ressource *) malloc( sizeof(Ressource) );
  if (ressource == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return NULL;
  }

  ressource->Bitblk   = NULL; /* Tous les champs internes pouvant servir */
  ressource->Cicon    = NULL;
  ressource->Ciconblk = NULL;
  ressource->Iconblk  = NULL;
  ressource->Object   = NULL;
  ressource->Tedinfo  = NULL;
  ressource->Chaines  = NULL;
  ressource->Chaines_libres = NULL;
  ressource->Images_libres  = NULL;

  /* On met  jour la chane */

  ressource->suivant = premier_RSC;
  premier_RSC = ressource;

  return ressource;
}

/* --------------- Effacement des infos d'un ressource ---------------------------------- */

int RSC_effacer(Ressource *ressource)
{
  if (ressource == NULL)
    return 0; /* Rien de spcial */

  if (ressource == premier_RSC) /* On est sur le premier */
  {
    premier_RSC = ressource->suivant;
  }
  else
  {
    Ressource *avant = premier_RSC;

    while (avant->suivant != ressource)
    {
      avant = avant->suivant;
      if (avant == NULL)
      {
	printf("Le ressource demand n'est pas dans la liste !\n");
	return -1; /* Erreur survenue */
      }
    }
    avant->suivant = ressource->suivant;
  }

  free(ressource->Bitblk);
  free(ressource->Cicon);
  free(ressource->Ciconblk);
  free(ressource->Iconblk);
  free(ressource->Object);
  free(ressource->Tedinfo);
  free(ressource->Chaines);
  free(ressource->Chaines_libres);
  free(ressource->Images_libres);
  free(ressource);

  return 0; /* Tout s'est bien pass */
}

/* --------------- Effacer tous les ressources en mmoire ------------------------------- */

void RSC_tt_effacer(void)
{
  while (premier_RSC)
    RSC_effacer(premier_RSC);
}

/* --------------- Lecture de l'adresse d'un objet du RSC ------------------------------- */

int RSC_lit_adresse(Ressource *ressource, int type, int index, void *adresse)
{
  OBJECT *adr_objet;
  TEDINFO *adr_tedinfo;
  int i;

  switch (type) /* Type d'objet que l'on cherche */
  {
  case R_TREE: /* On cherche un arbre d'objet */
    if ( index >= ressource->nombre_arbres ) /* Pas assez d'arbres */
    {
      printf("Le ressource indiqu ne contient pas d'arbre %d (seulement %ld)\n", index,
	     ressource->nombre_arbres);
      return -2;
    }

    i = 0;
    adr_objet = ressource->Object; /* Adresse de l'arbre */
    while (i < index) /* Tant que ce n'est pas le bon */
    {
      int dern_objet = 0;

      /* On commence par aller le plus loin possible dans cet arbre */
      while ( OB_TAIL(adr_objet, dern_objet) != -1 ) /* On saute le plus d'objets fils possible */
	dern_objet = OB_TAIL(adr_objet, dern_objet);

      /* On termine en parcourant  la main les derniers */
      while ( ( OB_FLAGS(adr_objet, dern_objet) & LASTOB ) == FAUX )
	dern_objet ++; /* On passe donc au suivant */

      adr_objet += dern_objet + 1; /* On se place sur le dbut de l'arbre suivant */
      i++;
    }

    *( (OBJECT **) adresse) = adr_objet; /* Pointeur sur le premier objet de l'arbre */
    return 0; /* Tout va bien */
  }

  return -3; /* Arriv  la fin sans avoir rien fait */
}

/*****************************************************************************/

int RSC_charge(const char *fname, Ressource *ressource)
{
  RSC_infos infos;
  FILE *fichier;
  UWORD version; /* Version du RSC */

#ifdef LITTLE_ENDIAN
  UWORD lu;
#endif

  fichier = fopen(fname, "rb"); /* On ouvre le fichier - premier essai : rpertoire de travail */
  if (fichier == NULL) /* a n'a pas march */
  {
    char *nom_rsc;

    nom_rsc = (char *) malloc( strlen(_LIB_RSC_) + strlen(fname) + 2 );
    if (nom_rsc == NULL)
    {
      printf("Mmoire insuffisante - ouverture du fichier RSC annule\n");
      return FAUX;
    }

    sprintf(nom_rsc, "%s/%s", _LIB_RSC_, fname);
    printf("Nouveau lieu de recherche : %s\n", nom_rsc);

    fichier = fopen(nom_rsc, "rb");
    free(nom_rsc); /* Plus besoin d'elle */

    if (fichier == NULL) /* a n'a pas fonctionn non plus */
    {
      printf("Ouverture du fichier ressource impossible - erreur : %d\n", errno);
      return FAUX;
    }
  }

#ifdef LITTLE_ENDIAN

  fread(&lu, sizeof(UWORD), 1, fichier);
  version =  (UWORD) ( (lu & 0xFF) << 8) | (UWORD) ( (lu & 0xFF00) >> 8);

#else

  fread(&version, sizeof(UWORD), 1, fichier);

#endif

  printf("Version du fichier : %d\n", version);

  if (version == 3) /* Format tendu */
  {
    RSXHDR en_tete;

    en_tete.rsh_vrsn = version;

    fseek(fichier, 0, 0); /* On se place au dbut */
    fread(&en_tete, sizeof( RSXHDR ), 1, fichier); /* On lit l'en-tte */

#ifdef LITTLE_ENDIAN

    infos.deb_objets     = RSC_UL_v_ul(en_tete.rsh_object);
    infos.deb_tedinfo    = RSC_UL_v_ul(en_tete.rsh_tedinfo);
    infos.deb_iconblk    = RSC_UL_v_ul(en_tete.rsh_iconblk);
    infos.deb_bitblk     = RSC_UL_v_ul(en_tete.rsh_bitblk);
    infos.deb_ch_libres  = RSC_UL_v_ul(en_tete.rsh_frstr);
    infos.deb_chaines    = RSC_UL_v_ul(en_tete.rsh_string);
    infos.deb_images     = RSC_UL_v_ul(en_tete.rsh_imdata);
    infos.deb_img_libres = RSC_UL_v_ul(en_tete.rsh_frimg);
    infos.table_arbres   = RSC_UL_v_ul(en_tete.rsh_trindex);
    infos.nbr_objets     = RSC_UL_v_ul(en_tete.rsh_nobs);
    infos.nbr_arbres     = RSC_UL_v_ul(en_tete.rsh_ntree);
    infos.nbr_tedinfos   = RSC_UL_v_ul(en_tete.rsh_nted);
    infos.nbr_iconblk    = RSC_UL_v_ul(en_tete.rsh_nib);
    infos.nbr_bitblk     = RSC_UL_v_ul(en_tete.rsh_nbb);
    infos.nbr_chaines    = RSC_UL_v_ul(en_tete.rsh_nstring);
    infos.nbr_images     = RSC_UL_v_ul(en_tete.rsh_nimages);
    infos.taille_ressource = RSC_UL_v_ul(en_tete.rsh_rssize);

#else /* Codage comme sur atari -> conversion directe */

    infos.deb_objets     = (unsigned long) en_tete.rsh_object;
    infos.deb_tedinfo    = (unsigned long) en_tete.rsh_tedinfo;
    infos.deb_iconblk    = (unsigned long) en_tete.rsh_iconblk;
    infos.deb_bitblk     = (unsigned long) en_tete.rsh_bitblk;
    infos.deb_ch_libres  = (unsigned long) en_tete.rsh_frstr;
    infos.deb_chaines    = (unsigned long) en_tete.rsh_string;
    infos.deb_images     = (unsigned long) en_tete.rsh_imdata;
    infos.deb_img_libres = (unsigned long) en_tete.rsh_frimg;
    infos.table_arbres   = (unsigned long) en_tete.rsh_trindex;
    infos.nbr_objets     = (unsigned long) en_tete.rsh_nobs;
    infos.nbr_arbres     = (unsigned long) en_tete.rsh_ntree;
    infos.nbr_tedinfos   = (unsigned long) en_tete.rsh_nted;
    infos.nbr_iconblk    = (unsigned long) en_tete.rsh_nib;
    infos.nbr_bitblk     = (unsigned long) en_tete.rsh_nbb;
    infos.nbr_chaines    = (unsigned long) en_tete.rsh_nstring;
    infos.nbr_images     = (unsigned long) en_tete.rsh_nimages;
    infos.taille_ressource = (unsigned long) en_tete.rsh_rssize;

#endif
  }
  else if (version > 3)
  {
    printf("Version du fichier ressource inconnue.\n");
    printf("Chargement annul\n");
    return FAUX;
  }
  else
  {
    RSHDR en_tete; /* En-tte du fichier RSC */

    en_tete.rsh_vrsn = version;

#ifdef LITTLE_ENDIAN /* On doit inverser le format */

    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_object =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_tedinfo =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_iconblk =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_bitblk =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_frstr =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_string =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_imdata =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_frimg =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_trindex =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_nobs =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);  /* Nbr. objets */
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_ntree =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8); /* Nbr. arbres */
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_nted =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);  /* Nbr. tedinfos */
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_nib =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);   /* Nbr. icnes */
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_nstring =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8); /* Nbr. chanes */
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_nimages =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8); /* Nbr. images */
    fread(&lu, sizeof(UWORD), 1, fichier);
    en_tete.rsh_rssize =  (UWORD) ( (lu & 0xFF) << 8) | ( (lu & 0xFF00) >> 8);  /* Taille RSC */

#else /* On travaille en BIG_ENDIAN, comme sur l'atari */

    fread(&(en_tete.rsh_object ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_tedinfo), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_iconblk), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_bitblk ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_frstr  ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_string ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_imdata ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_frimg  ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_trindex), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_nobs   ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_ntree  ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_nted   ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_nib    ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_nbb    ), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_nstring), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_nimages), sizeof(UWORD), 1, fichier);
    fread(&(en_tete.rsh_rssize ), sizeof(UWORD), 1, fichier);

#endif

    infos.deb_objets     = (unsigned long) en_tete.rsh_object;
    infos.deb_tedinfo    = (unsigned long) en_tete.rsh_tedinfo;
    infos.deb_iconblk    = (unsigned long) en_tete.rsh_iconblk;
    infos.deb_bitblk     = (unsigned long) en_tete.rsh_bitblk;
    infos.deb_ch_libres  = (unsigned long) en_tete.rsh_frstr;
    infos.deb_chaines    = (unsigned long) en_tete.rsh_string;
    infos.deb_images     = (unsigned long) en_tete.rsh_imdata;
    infos.deb_img_libres = (unsigned long) en_tete.rsh_frimg;
    infos.table_arbres   = (unsigned long) en_tete.rsh_trindex;
    infos.nbr_objets     = (unsigned long) en_tete.rsh_nobs;
    infos.nbr_arbres     = (unsigned long) en_tete.rsh_ntree;
    infos.nbr_tedinfos   = (unsigned long) en_tete.rsh_nted;
    infos.nbr_iconblk    = (unsigned long) en_tete.rsh_nib;
    infos.nbr_bitblk     = (unsigned long) en_tete.rsh_nbb;
    infos.nbr_chaines    = (unsigned long) en_tete.rsh_nstring;
    infos.nbr_images     = (unsigned long) en_tete.rsh_nimages;
    infos.taille_ressource = (unsigned long) en_tete.rsh_rssize;
  }

  ressource->nombre_arbres  = infos.nbr_arbres;
  ressource->nombre_chaines = infos.nbr_chaines;
  ressource->nombre_objets  = infos.nbr_objets;

  printf("Dbut des objets  : %ld\n", infos.deb_objets);
  printf("Dbut des tedinfo : %ld\n", infos.deb_tedinfo);
  printf("Dbut des icnes  : %ld\n", infos.deb_iconblk);
  printf("Dbut des bitblk  : %ld\n", infos.deb_bitblk);
  printf("Dbut des chanes libres : %ld\n", infos.deb_ch_libres);
  printf("Dbut des chanes : %ld\n", infos.deb_chaines);
  printf("Dbut des images  : %ld\n", infos.deb_images);
  printf("Nombre d'objets   : %ld\n", infos.nbr_objets);
  printf("Nombre d'arbres   : %ld\n", infos.nbr_arbres);
  printf("Nombre de TEDINFO : %ld\n", infos.nbr_tedinfos);
  printf("Nombre d'ICONBLK  : %ld\n", infos.nbr_iconblk);
  printf("Nombre d'BITBLK   : %ld\n", infos.nbr_bitblk);
  printf("Nombre de chanes : %ld\n", infos.nbr_chaines);
  printf("Nombre d'images   : %ld\n", infos.nbr_images);
  
  /* On rserve la mmoire pour tout a */

  ressource->Chaines = (char *) malloc( (size_t) (infos.deb_images - infos.deb_chaines) );
  /*  printf("Longueur des chanes : %d\n", en_tete.rsh_imdata - en_tete.rsh_string); */
  if (ressource->Chaines == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    fclose(fichier);
    return FAUX;
  }
  fseek(fichier, infos.deb_chaines, 0);                /* On se place au dbut des chanes */
  RSC_lit_chaines(fichier, (unsigned char *) (ressource->Chaines),
		  (size_t) (infos.deb_images - infos.deb_chaines));

  ressource->Object = (OBJECT *) calloc(infos.nbr_objets, sizeof(OBJECT));
  if (ressource->Object == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    fclose(fichier);
    return FAUX;
  }

  ressource->Tedinfo = (TEDINFO *) calloc(infos.nbr_tedinfos, sizeof(TEDINFO));
  if (ressource->Tedinfo == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    fclose(fichier);
    return FAUX;
  }

  /* On lit les objets */

  if ( RSC_lit_objets(fichier, infos.deb_objets, infos.nbr_objets,
		      ressource->Object) == FAUX)
  {
    fclose(fichier);
    return FAUX;
  }

  if ( RSC_lit_tedinfos(fichier, infos.deb_tedinfo, infos.nbr_tedinfos,
			ressource->Tedinfo) == FAUX)
  {
    fclose(fichier);
    return FAUX;
  }   

  /* On installe les chanes */

  RSC_fix_objets (ressource, &infos);
  RSC_fix_tedinfo(ressource, infos.nbr_tedinfos, infos.deb_chaines);

  printf("Chargement termin, tout va bien\n");

  return VRAI; /* On est l : le chargement s'est bien pass */
}

/* ~~~~~~~~~~ Fonctions de lecture des units de base - gestion endianness au passage ~~~~~~~ */

static long RSC_lit_longmot(FILE *fichier)
{
  LONG lg_mot;
  int nbr_lus;
  long lg_mot_lu;
#ifdef LITTLE_ENDIAN
  LONG convertit;
#endif

  nbr_lus = fread(&lg_mot, sizeof(LONG), 1, fichier);
  if (nbr_lus != 1)
  {
    printf("Chargement d'un ressource, RSC_lig_longmot : erreur  la lecture du fichier !\n");
    return -2;
  }

#ifdef LITTLE_ENDIAN

  convertit = ((lg_mot >> 24) & 0xFF) | ((lg_mot >> 8) & 0xFF00)
    | ((lg_mot << 8) & 0xFF0000) | ((lg_mot << 24) & 0xFF000000);
  lg_mot_lu = (long) convertit;

#else

  lg_mot_lu = (long) lg_mot;

#endif

  return lg_mot_lu;
}

static int RSC_lit_mot(FILE *fichier)
{
  WORD mot;
  int nbr_lus;
  int mot_lu;
#ifdef LITTLE_ENDIAN
  WORD convertit;
#endif

  nbr_lus= fread(&mot, sizeof(WORD), 1, fichier);
  if (nbr_lus != 1)
  {
    printf("Chargement d'un ressource : erreur  la lecture du fichier !\n");
    return -2;
  }

#ifdef LITTLE_ENDIAN

  convertit =  ( (mot & 0xFF) << 8) | ( (mot & 0xFF00) >> 8);
  mot_lu = (int) convertit;

#else

  mot_lu = (int) mot;

#endif

  return mot_lu;
}

/* ~~~~~~~~~~ Chargement des chanes du ressource ~~~~~~~~~~~ */

boolean RSC_lit_chaines(FILE *fichier, unsigned char *chaines, size_t nbr_cars)
{
  size_t i;

  if (chaines == NULL) return -1;
  if (fichier == NULL) return -2;

  /* On lit tout d'un coup */

  i = fread(chaines, 1, nbr_cars, fichier); /* On lit */
  if ( i != nbr_cars )
  {
    printf("RSC_lit_chaines : erreur au chargement du ressource\n");
    return -3;
  }

#if ATARI == 0 /* Il faut donc faire une conversion */

  for (i = 0 ; i < nbr_cars ; i++)
  {
    if ( ((unsigned char) chaines[i]) > 127) /* codage Atari  prendre en compte */
    {

#if ISO_8859_1 == 1 /* Codage latin-1 */
      chaines[i] = conv_8859_1[ (int) ( ( (unsigned char) chaines[i]) - 128) ];
#endif

    }
  }

#endif /* ATARI == 0 */

  return 0;
}

/* ~~~~~~~~~~ chargements des objets du ressource ~~~~~~~~~~~~ *
 |  Renvoie VRAI si tout s'est bien pass
 |
 | fichier : le fichier en cours d'analyse
 | debut   : index de dbut des objets dans le fichier
 | nbr_objets : nombre d'objets  lire
 | adresse : adresse o enregistrer les objets
 \------------------------------------------------------------ */

boolean RSC_lit_objets(FILE *fichier, long debut, long nbr_objets, OBJECT *adresse)
{
  int num_objet;

  if ( (adresse == NULL) || (fichier == NULL) )
    return FAUX;

  fseek(fichier, debut, 0); /* On se place l : dbut des objets si j'ai tout bien suivi... */

  for (num_objet = 0 ; num_objet < nbr_objets ; num_objet++, adresse++)
  {
    adresse->ob_next  = RSC_lit_mot(fichier);
    adresse->ob_head  = RSC_lit_mot(fichier);
    adresse->ob_tail  = RSC_lit_mot(fichier);
    adresse->ob_type  = RSC_lit_mot(fichier);
    adresse->ob_flags = RSC_lit_mot(fichier);
    adresse->ob_state = RSC_lit_mot(fichier);
    adresse->ob_spec.autre = RSC_lit_longmot(fichier);
    adresse->ob_x      = RSC_lit_mot(fichier);
    adresse->ob_y      = RSC_lit_mot(fichier);
    adresse->ob_width  = RSC_lit_mot(fichier);
    adresse->ob_height = RSC_lit_mot(fichier);
  }

  return VRAI; /* Tout s'est bien pass */
}

/* ~~~~~~~~~~ chargements des structures TEDINFO du ressource ~~~~~~~~~~~~ *
 |  Renvoie VRAI si tout s'est bien pass                                  |
 |                                                                         |
 | fichier    : le fichier en cours d'analyse                              |
 | debut      : index de dbut des objets dans le fichier                  |
 | nbr_objets : nombre de structures TEDINFO  lire                        |
 | adresse    : adresse o enregistrer les structures TEDINFO              |
 \------------------------------------------------------------------------ */

boolean RSC_lit_tedinfos(FILE *fichier, long debut, long nbr_objets, TEDINFO *adresse)
{
  int num_objet;

  if ( (adresse == NULL) || (fichier == NULL) )
    return FAUX;

  fseek(fichier, debut, 0); /* On se place l : dbut des tedinfos si j'ai tout bien suivi... */

  for (num_objet = 0 ; num_objet < nbr_objets ; num_objet++, adresse++)
  {
    UWORD val_lue;
    int nbr_lu;
#ifdef LITTLE_ENDIAN
    UWORD convertit;
#endif

    adresse->te_ptext  = (char *) RSC_lit_longmot(fichier);
    adresse->te_ptmplt = (char *) RSC_lit_longmot(fichier);
    adresse->te_pvalid = (char *) RSC_lit_longmot(fichier);

    adresse->te_font  = RSC_lit_mot(fichier);
    adresse->te_junk1 = RSC_lit_mot(fichier);
    adresse->te_just  = RSC_lit_mot(fichier);

    nbr_lu = fread(&val_lue, 1, 2, fichier); /* Suppose le mot au mme format que l'atari */
#ifdef LITTLE_ENDIAN
    convertit = ( (val_lue >> 8) & 0xFF ) | ( (val_lue << 8) & 0xFF00 );
    adresse->te_color = (unsigned int) convertit;
#else
    adresse->te_color = (unsigned int) val_lue;
#endif

    adresse->te_junk2     = RSC_lit_mot(fichier);
    adresse->te_thickness = RSC_lit_mot(fichier);
    adresse->te_txtlen    = RSC_lit_mot(fichier);
    adresse->te_tmplen    = RSC_lit_mot(fichier);
  }
  return VRAI; /* Tout s'est bien pass */
}

/* ~~~~~~~~~~~ Reconstruction des liens, pointeurs et tout et tout ~~~~~~~~~~~~~~~~~ */

boolean RSC_fix_objets(Ressource *ressource, RSC_infos *infos)
{
  int larg_car, haut_car;
  unsigned long dep_tedinfo, dep_chaines;
  unsigned long num_objet /*, nbr_objets */;
  OBJECT *objet;

  if ( (ressource == NULL) || (infos == NULL) )
    return FAUX;

  dep_tedinfo = infos->deb_tedinfo;
  dep_chaines = infos->deb_chaines;

  larg_car  = _Fonte->max_bounds.width + 1;
  haut_car  = _Fonte->max_bounds.ascent;
  haut_car += _Fonte->ascent; /* Semble donner des boutons trop hauts... mais bon, a passe */

  objet = ressource->Object;

  for (num_objet = 0 ; num_objet < infos->nbr_objets ; num_objet++, objet++)
  {
    long ob_spec = objet->ob_spec.autre;

    switch ( objet->ob_type )
    {
    case G_STRING:
    case G_TITLE:
    case G_BUTTON: /* Ob_spec = pointeur sur la chane de caractre */
      ob_spec -= dep_chaines;
      objet->ob_spec.texte = ressource->Chaines + ob_spec;
      if (objet->ob_type == G_TITLE)
	objet->ob_width++; /* Ajout artificiel d'un espace, pour arranger... */
      break;

    case G_TEXT:
    case G_FTEXT:
    case G_BOXTEXT:
    case G_FBOXTEXT: /* Ob_spec = pointeur sur une structure TEDINFO */
      ob_spec -= dep_tedinfo;
      ob_spec /= 28; /* Taille d'une TEDINFO _dans le RSC_, en octets */
      objet->ob_spec.tedinfo = ressource->Tedinfo + ob_spec;
      break;

    case G_USERDEF:
      objet->ob_spec.util = (USERBLK *) malloc(sizeof(USERBLK));
      if (objet->ob_spec.util == NULL)
      {
	ERREUR(MANQUE_MEMOIRE);
	return FAUX;
      }
      objet->ob_spec.util->ub_code = USR_separateur; /* Dfinition par dfaut des USERDEF : sparateur */
      objet->ob_spec.util->ub_parm = 0L;
      break;

      /* Autres types : rien de spcial dans ob_spec */
    }

    /* Conversion caractres -> pixels */

    objet->ob_x      *= larg_car;
    objet->ob_y      *= haut_car;
    objet->ob_width  *= larg_car;
    objet->ob_height *= haut_car;
  }

  return VRAI; /* Pas de problme */
}

boolean RSC_fix_tedinfo(Ressource *ressource, long nbr_tedinfos, long debut_chaines)
{
  char *adr_texte;
  long num_tedinfo;
  TEDINFO *adresse = ressource->Tedinfo;

  for (num_tedinfo = 0 ; num_tedinfo < nbr_tedinfos ; num_tedinfo++, adresse++)
  {
    adr_texte = ressource->Chaines + ( ((long) adresse->te_ptext) - debut_chaines );
    adresse->te_ptext = adr_texte;
    if ( adr_texte[0] == '@' )
    {
      int i;

      for (i = 0 ; i < adresse->te_txtlen ; i++)
	adr_texte[i] = 0;
    }

    adr_texte = ressource->Chaines + ( ((long) adresse->te_ptmplt) - debut_chaines );
    adresse->te_ptmplt = adr_texte;

    adr_texte = ressource->Chaines + ( ((long) adresse->te_pvalid) - debut_chaines );
    adresse->te_pvalid = adr_texte;
  }

  return VRAI; /* Tout s'est bien pass */
}


#ifdef _XRSRC_ORIGINAL_
/*****************************************************************************/

void rs_sglobal (int *base)
{
  rs_global = base;
  hdr_buf = (RSXHDR *)*(LONG *)&rs_global[7];
  (LONG)rs_hdr = (LONG)hdr_buf + sizeof (RSXHDR);

  return;
}

/*****************************************************************************/

int rs_free (int *base)
{
#if COLOR_ICONS == TRUE
  WORD     i;
  CICON    *color_icn;
#endif

  rs_global = base;

#if COLOR_ICONS == TRUE

  if ((color_icn = (CICON *)*(LONG *)&rs_global[2]) != NULL)
  {
    for (i = 0; i < rs_global[4]; i++)
    {
      if (color_icn[i].num_planes > 1)
      {
	if (color_icn[i].col_data != NULL)
	  free (color_icn[i].col_data);
	if (color_icn[i].sel_data != NULL)
	  free (color_icn[i].sel_data);
	if (color_icn[i].sel_data == NULL && color_icn[i].sel_mask != NULL)
	  free (color_icn[i].sel_mask);
      }
    }
    free (color_icn);
  }
  if (*(LONG *)rs_global)
    free ((void *)*(LONG *)rs_global);
#endif

  free ((RSXHDR *)*(LONG *)&rs_global[7]);

  return (TRUE);
}

/*****************************************************************************/

int rs_sadd (int *base, WORD rs_stype, WORD rs_sindex, OBJECT *re_saddr)
{
  OBJECT *old_addr;

  rs_sglobal (base);

  old_addr = get_address (rs_stype, rs_sindex);

  if (old_addr == (OBJECT *)NULL)
    return (FALSE);

  *old_addr = *re_saddr;

  return (TRUE);
}

/*****************************************************************************/

void *get_address (WORD type, WORD index)
{
  void *the_addr = (void *)NULL;
  union
  {
    void	*dummy;
    char	*string;
    OBJECT     **dpobject;
    OBJECT	*object;
    TEDINFO	*tedinfo;
    ICONBLK	*iconblk;
    BITBLK	*bitblk;
  } all_ptr;

  switch (type)
  {
  case R_TREE:
    all_ptr.dpobject = (OBJECT **)(*(long **)&rs_global[5]);
    the_addr = all_ptr.dpobject[index];
    break;

  case R_OBJECT:
    the_addr = get_sub (index, hdr_buf->rsh_object, sizeof(OBJECT));
    break;

  case R_TEDINFO:
  case R_TEPTEXT:
    the_addr = get_sub (index, hdr_buf->rsh_tedinfo, sizeof(TEDINFO));
    break;

  case R_ICONBLK:
  case R_IBPMASK:
    the_addr = get_sub (index, hdr_buf->rsh_iconblk, sizeof(ICONBLK));
    break;

  case R_BITBLK:
  case R_BIPDATA:
    the_addr = get_sub (index, hdr_buf->rsh_bitblk, sizeof(BITBLK));
    break;

  case R_OBSPEC:
    all_ptr.object = get_address(R_OBJECT, index);
    the_addr = &all_ptr.object->ob_spec;
    break;

  case R_TEPVALID:
  case R_TEPTMPLT:
    all_ptr.tedinfo = get_address(R_TEDINFO, index);
    if (type == R_TEPVALID)
      the_addr = &all_ptr.tedinfo->te_pvalid;
    else
      the_addr = &all_ptr.tedinfo->te_ptmplt;
    break;

  case R_IBPDATA:
  case R_IBPTEXT:
    all_ptr.iconblk = get_address(R_ICONBLK, index);
    if (type == R_IBPDATA)
      the_addr = &all_ptr.iconblk->ib_pdata;
    else
      the_addr = &all_ptr.iconblk->ib_ptext;
    break;
    
  case R_STRING:
    the_addr = get_sub (index, hdr_buf->rsh_frstr, sizeof (char *));
    the_addr = (void *) * (char *)the_addr;
    break;

  case R_IMAGEDATA:
    the_addr = get_sub (index, hdr_buf->rsh_imdata, sizeof (char *));
    the_addr = (void *)*(char *)the_addr;
    break;

  case R_FRIMG:
    the_addr = get_sub (index, hdr_buf->rsh_frimg, sizeof (char *));
    the_addr = (void *)*(char *)the_addr;
    break;

  case R_FRSTR:
    the_addr = get_sub (index, hdr_buf->rsh_frstr, sizeof (char *));
    break;
  }

  return (the_addr);
}

/*****************************************************************************/

void *get_sub (WORD index, LONG offset, WORD size)
{
  unsigned char *ptr = (unsigned char *)rs_hdr;

  ptr += offset;
  ptr += (index * size);

  return ((void *)ptr);
}

/*****************************************************************************/

void fix_chp (int *pcoord, WORD flag)
{
  WORD ncoord;

  ncoord = *pcoord & 0xff;

  if (!flag && ncoord == 0x50)
    ncoord = xdesk.g_w;				/* xdesk.w = largeur de l'cran en points */
  else
    ncoord *= (flag ? xgl_hbox : xgl_wbox);	/* xgl_wbox, xgl_hbox = largeur & hauteur du dessin en points */

  if (((*pcoord >> 8) & 0xff) > 0x80)
    ncoord += (((*pcoord >> 8) & 0xff) | 0xff00);
  else
    ncoord += ((*pcoord >> 8) & 0xff);
  
  *pcoord = ncoord;
}

/*****************************************************************************/
/* initialiser les icnes couleur pour la rsolution actuelle                */
/*****************************************************************************/

void do_ciconfix (ULONG header, RSXHDR *rsxhdr, LONG rs_len)
{
  LONG   *cicon_liste;
  WORD   i;
  OBJECT *obj;

  cicon_liste = (LONG *)(*(LONG *)(rsxhdr->rsh_rssize + (rsxhdr->rsh_rssize & 1L) + header + sizeof (LONG)) + header);
  if ((LONG)cicon_liste - header > rsxhdr->rsh_rssize && (LONG)cicon_liste - header < rs_len)
  {
    if (fill_cicon_liste (cicon_liste, header, rsxhdr) != NIL)
    {
#if COLOR_ICONS == TRUE
      WORD nub = 0, work_out [57], *palette;
			
      if (*(LONG *)rs_global && *(LONG *)&rs_global[2])
      {
	vq_extnd (xvdi_handle, TRUE, work_out);	/* Nombre de plans */
	xscrn_planes = work_out[4];

	xpixelbytes = test_rez ();
	palette = (WORD *)*(LONG *)(rsxhdr->rsh_rssize + (rsxhdr->rsh_rssize & 1L) + header + 2 * sizeof (LONG));
	if (palette != NULL)
	{
	  (LONG)palette += header;
	  memcpy (rgb_palette, palette, sizeof (rgb_palette));
	  is_palette = TRUE;
	}
	else
	  is_palette = FALSE;
	
	xfill_farbtbl ();
			
	for (i = 0; i < rsxhdr->rsh_nobs; i++)
        {
	  obj = &((OBJECT *)(rsxhdr->rsh_object + header))[i];
	  if ((obj->ob_type & 0xff) == G_CICON)
	  {
	    if (xadd_cicon ((CICONBLK *)obj->ob_spec, obj, nub++) == FALSE)
	    {
	      memset (&((CICON *)*(LONG *)&rs_global[2])[nub-1], 0, sizeof (CICON));
	      obj->ob_type = (obj->ob_type & 0xff00) | G_ICON;
	    }
	    obj->ob_spec -= header;
	  }
	}
      }
      else
#endif
      {
	for (i = 0; i < rsxhdr->rsh_nobs; i++)
	{
	  obj = &((OBJECT *)(rsxhdr->rsh_object + header))[i];
	  if ((obj->ob_type & 0xff) == G_CICON)
	  {
	    obj->ob_type = (obj->ob_type & 0xff00) | G_ICON;
	    obj->ob_spec -= header;
	  }
	}
      }
    }
  }
}

/*****************************************************************************/
/* initialisation du pointeur pour les icnes couleurs dans le RSC           */
/*****************************************************************************/

int fill_cicon_liste (LONG *cicon_liste, ULONG header, RSXHDR *rsxhdr)
{
  WORD     i, i2, num = 0;
  char     *p;
  LONG     iclen, num_cicon, ob, p2;
  CICONBLK *cblk;
  CICON    *cicon, *cold;
  OBJECT   *pobject;

  while (!cicon_liste[num])
    num++;
	
  if (cicon_liste[num] != -1L)
    return (NIL);
  
  cblk = (CICONBLK *)&cicon_liste[num+1];
	
  for (i = 0; i < num; i++)
  {
    cicon_liste[i] = (LONG)cblk;
    p = (char *)&cblk[1];
    cblk->monoblk.ib_pdata = (int *)p;
    iclen = cblk->monoblk.ib_wicon / 8 * cblk->monoblk.ib_hicon;
    p += iclen;
    cblk->monoblk.ib_pmask = (int *)p;
    p += iclen;
    p2 = (LONG)cblk->monoblk.ib_ptext;
    if (!p2 || header + p2 == (LONG)p || p2 < rsxhdr->rsh_string || p2 > rsxhdr->rsh_rssize)
      cblk->monoblk.ib_ptext = (char *) p;
    else
      (LONG)cblk->monoblk.ib_ptext = header + (LONG)cblk->monoblk.ib_ptext;
    
    cicon = (CICON *)&p[12];
    p += 12L;
    cold = cicon;
    if ((num_cicon = (LONG)cblk->mainlist) > 0)
    {
      cblk->mainlist = cicon;
	
      for (i2 = 0; i2 < num_cicon; i2++)
      {
	p = (char *)&cicon[1];
	cicon->col_data = (WORD *)p;
	p += iclen * cicon->num_planes;
	cicon->col_mask = (WORD *)p;
	p += iclen;
	if (cicon->sel_data != NULL)
	{
	  cicon->sel_data = (WORD *)p;
	  p += iclen * cicon->num_planes;
	  cicon->sel_mask = (WORD *)p;
	  p += iclen;
	}
	cicon->next_res = (CICON *)p;
	cold = cicon;
	cicon = (CICON *)p;
      }
      cold->next_res = NULL;
    }
    cblk = (CICONBLK *)p;
  }
  
  if (num != NIL)
  {
    pobject = (OBJECT *)(header + rsxhdr->rsh_object);
    
    for (ob = 0; ob < rsxhdr->rsh_nobs; ob++)
      if ((pobject[ob].ob_type & 0xff) == G_CICON)
	pobject[ob].ob_spec = cicon_liste[pobject[ob].ob_spec];
    
    if ((*(LONG *)rs_global = (LONG)malloc (num * sizeof (OBBLK))) != 0L)
      memset ((void *)*(LONG *)rs_global, 0, num * sizeof (OBBLK));
    if ((*(LONG *)&rs_global[2] = (LONG)malloc (num * sizeof (CICON))) != 0L)
      memset ((void *)*(LONG *)&rs_global[2], 0, num * sizeof (CICON));
    rs_global[4] = num;
  }
  
  return (num);
}

#if COLOR_ICONS == TRUE /* Toute la suite ne concerne que les icnes couleurs */

/*****************************************************************************/
/* changer le format de l'icne et l'adapter aux autres rsolutions.         */
/*****************************************************************************/

WORD xadd_cicon (CICONBLK *cicnblk, OBJECT *obj, WORD nub)
{
  WORD     x, y, line, xmax, best_planes, find_planes;
  CICON    *cicn, *color_icn, *best_icn = NULL;
  LONG     len, *next;
  MFDB     d;
  OBBLK    *ub;
#if SAVE_MEMORY == TRUE
  CICON    *max_icn = NULL;
#endif

  len = cicnblk->monoblk.ib_wicon / 8 * cicnblk->monoblk.ib_hicon;
  
  color_icn = &((CICON *)*(LONG *)&rs_global[2])[nub];

  best_planes = 1;
  if (xscrn_planes > 8)
    find_planes = 4;
  else
    find_planes = xscrn_planes;

  cicn = cicnblk->mainlist;
  next = (LONG *)&cicnblk->mainlist;
  
  while (cicn != NULL)
  {
    *next = (LONG)cicn;
    next = (LONG *)&cicn->next_res;

#if SAVE_MEMORY == TRUE
    if (cicn->num_planes > xscrn_planes)
      max_icn = cicn;
#endif
    if (cicn->num_planes >= best_planes && cicn->num_planes <= find_planes)
    {
      best_planes = cicn->num_planes;
      best_icn = cicn;
    }
    cicn = cicn->next_res;
  }
	
  if (best_icn == NULL)		/* aucune icne couleur trouve */
    return (FALSE);
  else
    *color_icn = *best_icn;
  
  if (best_planes > 1)
    color_icn->num_planes = xscrn_planes;
  else
    color_icn->num_planes = 1;
	
  /* Allouer la place pour le format universel */
  if ((color_icn->col_data = malloc (len * color_icn->num_planes)) == NULL)
    return (FALSE);
  if (color_icn->sel_data)
  {
    if ((color_icn->sel_data = malloc (len * color_icn->num_planes)) == NULL)
    {
      free (color_icn->col_data);
      return (FALSE);
    }
  }

  if (best_planes > 1)
  {
    if (best_icn->sel_data == NULL)
    {
      /* Prparer le masque slectionn */
      if ((color_icn->sel_mask = malloc (len)) == NULL)
	{
	  free (color_icn->col_data);
	  if (color_icn->sel_data)
	    free (color_icn->sel_data);
	  return (FALSE);
	}
      
      xmax = cicnblk->monoblk.ib_wicon / 16;
      
      for (y = 0; y < cicnblk->monoblk.ib_hicon; y++)
      {
	line = y * xmax;
	
	for (x = 0; x < xmax; x++)
	{
	  if (y & 1)
	    color_icn->sel_mask[line + x] = best_icn->col_mask[line + x] & 0xaaaa;
	  else
	    color_icn->sel_mask[line + x] = best_icn->col_mask[line + x] & 0x5555;
	}
      }
    }
		
    d.mp  = color_icn->col_data;
    d.fwp = cicnblk->monoblk.ib_wicon;
    d.fh	= cicnblk->monoblk.ib_hicon;
    d.fww = d.fwp >> 4;
    d.ff	= TRUE;
    d.np	= xscrn_planes;
	
    xfix_cicon ((UWORD *)best_icn->col_data, len, best_planes, xscrn_planes, &d);
    if (best_icn->sel_data)
    {
      d.mp = color_icn->sel_data;
      xfix_cicon ((UWORD *)best_icn->sel_data, len, best_planes, xscrn_planes, &d);
    }
  }
  else
  {
    memcpy (color_icn->col_data, best_icn->col_data, len);
    memcpy (color_icn->sel_data, best_icn->sel_data, len);
  }

#if SAVE_MEMORY == TRUE
  if (best_icn->num_planes < color_icn->num_planes && max_icn != NULL)
  {
    if (best_icn->sel_data == NULL || max_icn->sel_data != NULL)
    {
      best_icn->col_data = max_icn->col_data;
      best_icn->sel_data = max_icn->sel_data;
      best_icn->num_planes = xscrn_planes;
    }
  }
	
  if (best_icn->num_planes == color_icn->num_planes)
  {
    memcpy (best_icn->col_data, color_icn->col_data, len * color_icn->num_planes);
    free (color_icn->col_data);
    color_icn->col_data = NULL;
    if (best_icn->sel_data != NULL)
    {
      memcpy (best_icn->sel_data, color_icn->sel_data, len * color_icn->num_planes);
      free (color_icn->sel_data);
      color_icn->sel_data = NULL;
    }
    else
    {
      memcpy (cicnblk->monoblk.ib_pmask, color_icn->sel_mask, len);
      best_icn->sel_mask = cicnblk->monoblk.ib_pmask;
      free (color_icn->sel_mask);
      color_icn->sel_mask = NULL;
    }
  }
  else
  {
    if (best_icn->sel_data == NULL)
    {
      memcpy (cicnblk->monoblk.ib_pmask, color_icn->sel_mask, len);
      free (color_icn->sel_mask);
      color_icn->sel_mask = NULL;
      best_icn->col_data = color_icn->col_data;
      best_icn->sel_mask = cicnblk->monoblk.ib_pmask;
      best_icn->num_planes = color_icn->num_planes;
    }
    else
      *best_icn = *color_icn;
  }
  cicnblk->mainlist = best_icn;
#else
  color_icn->next_res = cicnblk->mainlist;
  cicnblk->mainlist = color_icn;
#endif
  ub = (OBBLK *)*(LONG *)rs_global;
  ub[nub].old_type = G_CICON;
  ub[nub].ublk.ub_parm = obj->ob_spec;
  ub[nub].ublk.ub_code = xdraw_cicon;
  obj->ob_spec = (LONG)&ub[nub].ublk;
  obj->ob_type = (obj->ob_type & 0xff00) | G_USERDEF;
  
  return (TRUE);
}

/*****************************************************************************/
/* combiens de bytes par pixels seront changs au format                    */
/*****************************************************************************/

LOCAL WORD test_rez ()

{	WORD     i, np, color, pxy[8], rgb[3], bpp = 0;
	UWORD    backup[32], test[32];
	WORD     black[3] = {0, 0, 0};
	WORD     white[3] = {1000, 1000, 1000};
	MFDB     screen;
	MFDB     pixel = {NULL, 16, 1, 1, 0, 1, 0, 0, 0};
	MFDB     stdfm = {NULL, 16, 1, 1, 1, 1, 0, 0, 0};

	if (xscrn_planes >= 8)
	{
		stdfm.np = pixel.np = xscrn_planes;

		if (xscrn_planes == 8)
		{
			color = 0xff;
			memset (test, 0, xscrn_planes * sizeof (WORD));
			for (np = 0; np < xscrn_planes; np++)
				test[np] = (color & (1 << np)) << (15 - np);
	
			pixel.mp = stdfm.mp = test;
			vr_trnfm (xvdi_handle, &stdfm, &pixel);
			
			for (i = 1; i < xscrn_planes; i++)
				if (test[i])	break;
			
			if (i >= xscrn_planes && !(test[0] & 0x00ff))
				bpp = 1;
		}
		else
		{
			xrect2array (&xdesk, pxy);
			vs_clip (xvdi_handle, FALSE, pxy);
			screen.mp = NULL;
			
			memset (backup, 0, sizeof (backup));
		
			vswr_mode (xvdi_handle, MD_REPLACE);
			vsl_ends (xvdi_handle, 0, 0);
			vsl_type (xvdi_handle, 1);
			vsl_width (xvdi_handle, 1);
			memset (pxy, 0, sizeof (pxy));
			
			graf_mouse (M_OFF, NULL);
		
			pixel.mp = backup;	
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
		
			/* sauver l'ancienne couleur */
			vq_color (xvdi_handle, 15, 1, rgb);
	
			/* tester le format universel */
			pixel.mp = test;
			vsl_color (xvdi_handle, 15);
			vs_color (xvdi_handle, 15, white);
			v_pline (xvdi_handle, 2, pxy);
			
			memset (test, 0, xscrn_planes * sizeof (WORD));
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
			
			for (i = (xscrn_planes + 15) / 16 * 2; i < xscrn_planes; i++)
				if (test[i])	break;
			
			if (i >= xscrn_planes)
			{
				vs_color (xvdi_handle, 15, black);
				v_pline (xvdi_handle, 2, pxy);
				
				memset (test, 0, xscrn_planes * sizeof (WORD));
				vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
				
				for (i = (xscrn_planes + 15) / 16 * 2; i < xscrn_planes; i++)
					if (test[i])	break;
				
				if (i >= xscrn_planes)
					bpp = (xscrn_planes + 7) / 8;
			}

			/* restaurer l'ancienne couleur */
			vs_color (xvdi_handle, 15, rgb);
	
			pixel.mp = backup;	/* restaurer le point */
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &pixel, &screen);
		
			graf_mouse (M_ON, NULL);
	
			xrect2array (&xdesk, pxy);
			vs_clip (xvdi_handle, TRUE, pxy);
		}
	}

	return (bpp);
}

/*****************************************************************************/
/* En mode True-Color, passer la valeur de point en palette RVB              */
/*****************************************************************************/

LOCAL VOID xfill_farbtbl ()

{	WORD np, color, pxy[8], backup[32], rgb[3];
	MFDB screen;
	MFDB pixel = {NULL, 16, 1, 1, 0, 1, 0, 0, 0};
	MFDB stdfm = {NULL, 16, 1, 1, 1, 1, 0, 0, 0};
	WORD pixtbl[16] = {0, 2, 3, 6, 4, 7, 5, 8, 9, 10, 11, 14, 12, 15, 13, 16};
	
	if (xscrn_planes >= 8)
	{
		if (xscrn_planes > 8)
		{
			if (is_palette == FALSE)	/* pas de palette dans le Ressource */
			{
				for (color = 0; color < 255; color++)
				{	if (color < 16)
					{	vq_color (xvdi_handle, pixtbl[color], 1, rgb_palette[color]);
						rgb_palette[color][3] = pixtbl[color];
					}
					else
					{	vq_color (xvdi_handle, color - 1, 1, rgb_palette[color]);
						rgb_palette[color][3] = color - 1;
					}
				}
				vq_color (xvdi_handle, 1, 1, rgb_palette[255]);
				rgb_palette[255][3] = 1;
				is_palette = TRUE;
			}
			
			vs_clip (xvdi_handle, FALSE, pxy);
			graf_mouse (M_OFF, NULL);

			memset (backup, 0, sizeof (backup));
	 		memset (farbtbl, 0, 32 * 256 * sizeof (WORD));
			screen.mp = NULL;
			stdfm.np = pixel.np = xscrn_planes;
		
			vswr_mode (xvdi_handle, MD_REPLACE);
			vsl_ends (xvdi_handle, 0, 0);
			vsl_type (xvdi_handle, 1);
			vsl_width (xvdi_handle, 1);
			memset (pxy, 0, sizeof (pxy));
			
			pixel.mp = backup;	/* calculer le point */
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
		
			/* calculer l'ancienne couleur */
			vq_color (xvdi_handle, 15, 1, rgb);

			for (color = 0; color < 256; color++)
			{
				vs_color (xvdi_handle, 15, rgb_palette[color]);
				vsl_color (xvdi_handle, 15);
				v_pline (xvdi_handle, 2, pxy);
				
				stdfm.mp = pixel.mp = farbtbl[color];
		
				/* vro_cpyfm,  car v_get_pixel ne fonctionne pas en TrueColor (>=24 Plans) */
				vro_cpyfm (xvdi_handle, S_ONLY, pxy, &screen, &pixel);
	
				if (farbtbl2 != NULL && xpixelbytes)
				{	farbtbl2[color] = 0L;
					memcpy (&farbtbl2[color], pixel.mp, xpixelbytes);
				}
					
				vr_trnfm (xvdi_handle, &pixel, &stdfm);
				for (np = 0; np < xscrn_planes; np++)
					if (farbtbl[color][np])
						farbtbl[color][np] = 0xffff;
			}
		
			/* restaurer l'ancienne couleur */
			vs_color (xvdi_handle, 15, rgb);

			pixel.mp = backup;	/* restaurer le point */
			vro_cpyfm (xvdi_handle, S_ONLY, pxy, &pixel, &screen);
		
			graf_mouse (M_ON, NULL);
		}
	}
}

/*****************************************************************************/
/* adapter l'icne  la rsolution graphique actuelle                        */
/* (par exemple : une icne de 4 Plans vers 24 Plans TrueColor)              */
/*****************************************************************************/

LOCAL VOID xfix_cicon (UWORD *col_data, LONG len, WORD old_planes, WORD new_planes, MFDB *s)

{	LONG  x, i, old_len, rest_len, mul[32], pos;
	UWORD np, *new_data, mask, pixel, bit, color, back[32], old_col[32], maxcol;
	WORD  got_mem = FALSE;
	MFDB  d;
	
	len >>= 1;

	if (old_planes == new_planes)
	{	if (s != NULL)
		{	if (new_planes == xscrn_planes)
			{
				d = *s;
				d.ff = FALSE;
				s->mp = col_data;
				if (d.mp == s->mp)
				{	if ((d.mp = malloc (len * 2 * new_planes)) == NULL)
						d.mp = s->mp;
					else
						got_mem = TRUE;
				}
				
				vr_trnfm (xvdi_handle, s, &d);
				if (d.mp != s->mp && got_mem == TRUE)
				{
					memcpy (s->mp, d.mp, len * 2 * new_planes);
					free (d.mp);
				}
			}
			else
				memcpy (s->mp, col_data, len * 2 * new_planes);
		}
		return;
	}
	
	if (new_planes <= 8)
	{
		old_len  = old_planes * len;
		rest_len = new_planes * len - old_len;

		if (s != NULL)
		{
			new_data = &((UWORD *)s->mp)[old_len];
			memset (new_data, 0, rest_len * 2);
			memcpy (s->mp, col_data, old_len * 2);
			col_data = s->mp;
		}
		else
			new_data = (UWORD *)&col_data[old_len];
		
		for (x = 0; x < len; x++)
		{
			mask = 0xffff;
	
			for (i = 0; i < old_len; i += len)
				mask &= (UWORD)col_data[x+i];
			
			if (mask)
				for (i = 0; i < rest_len; i += len)
					new_data[x+i] |= mask;
		}

		if (s != NULL)	/* convertir au format universel */
		{
			d = *s;
			d.ff = 0;
			if ((d.mp = malloc (len * 2 * new_planes)) == NULL)
				d.mp = s->mp;
			
			vr_trnfm (xvdi_handle, s, &d);
			if (d.mp != s->mp)
			{
				memcpy (s->mp, d.mp, len * 2 * new_planes);
				free (d.mp);
			}
		}
	}
	else	/* TrueColor, valeur de points en RVB */
	{
		if (!xpixelbytes || s == NULL)
		{
			for (i = 0; i < new_planes; i++)
				mul[i] = i * len;
			
			if (old_planes < 8)
			{
				maxcol = (1 << old_planes) - 1;
				memcpy (old_col, farbtbl[maxcol], new_planes * sizeof (WORD));
				memset (farbtbl[maxcol], 0, new_planes * sizeof (WORD));
			}
	
			if (s != NULL)
			{
				new_data = &((UWORD *)s->mp)[old_len];
				memset (new_data, 0, rest_len * 2);
				memcpy (s->mp, col_data, old_len * 2);
				col_data = s->mp;
			}
			
			for (x = 0; x < len; x++)
			{
				bit = 1;
				for (np = 0; np < old_planes; np++)
					back[np] = col_data[mul[np] + x];
				
				for (pixel = 0; pixel < 16; pixel++)
				{
					color = 0;
					for (np = 0; np < old_planes; np++)
					{
						color += ((back[np] & 1) << np);
						back[np] >>= 1;
					}
					
					for (np = 0; np < new_planes; np++)
					{	pos = mul[np] + x;
						col_data[pos] = (col_data[pos] & ~bit) | (farbtbl[color][np] & bit);
					}
					
					bit <<= 1;
				}
			}
			if (old_planes < 8)
				memcpy (farbtbl[maxcol], old_col, new_planes * sizeof (WORD));

			if (s != NULL)	/* convertir au format universel */
			{
				d = *s;
				d.ff = 0;
				if ((d.mp = malloc (len * 2 * new_planes)) == NULL)
					d.mp = s->mp;
				
				vr_trnfm (xvdi_handle, s, &d);
				if (d.mp != s->mp)
				{
					memcpy (s->mp, d.mp, len * 2 * new_planes);
					free (d.mp);
				}
			}
		}
		else
			std_to_byte (col_data, len, old_planes, farbtbl2, s);
	}
}

/*****************************************************************************/
/* std_to_byte transforme un dessin du format standard au format             */
/* universel (pour les rsolutions >= 16 Plans)                              */
/*****************************************************************************/

LOCAL VOID std_to_byte (col_data, len, old_planes, farbtbl2, s)
UWORD *col_data;
LONG  len;
WORD  old_planes;
ULONG *farbtbl2;
MFDB  *s;

{	LONG  x, i, mul[32], pos;
	UWORD np, *new_data, pixel, color, back[32];
	WORD  memflag = FALSE;
	UBYTE *p1, *p2;
	ULONG  colback;

	if (s->mp == col_data)
	{
		if ((col_data = malloc (len * 2 * s->np)) == NULL)
			return;
		memcpy (col_data, s->mp, len * 2 * s->np);
		memflag = TRUE;
	}
	new_data = (UWORD *)s->mp;
	p1 = (UBYTE *)new_data;

	if (old_planes < 8)
	{
		colback = farbtbl2[(1 << old_planes) - 1];
		farbtbl2[(1 << old_planes) - 1] = farbtbl2[255];
	}
		
	for (i = 0; i < old_planes; i++)
		mul[i] = i * len;
	
	pos = 0;
	
	for (x = 0; x < len; x++)
	{
		for (np = 0; np < old_planes; np++)
			back[np] = col_data[mul[np] + x];
		
		for (pixel = 0; pixel < 16; pixel++)
		{
			color = 0;
			for (np = 0; np < old_planes; np++)
			{
				color |= ((back[np] & 0x8000) >> (15 - np));
				back[np] <<= 1;
			}
			
			switch (xpixelbytes)
			{
				case 2:
					new_data[pos++] = *(UWORD *)&farbtbl2[color];
					break;

				case 3:
					p2 = (UBYTE *)&farbtbl2[color];
					*(p1++) = *(p2++);
					*(p1++) = *(p2++);
					*(p1++) = *(p2++);
					break;

				case 4:
					((ULONG *)new_data)[pos++] = farbtbl2[color];
					break;
			}
		}
	}

	if (old_planes < 8)
		farbtbl2[(1 << old_planes) - 1] = colback;

	if (memflag)
		free (col_data);
}

/*****************************************************************************/
/* dessine l'icne couleur                                                   */
/*****************************************************************************/

int CDECL xdraw_cicon (PARMBLK *pb)
{
  int	   ob_x, ob_y, x, y, dummy, pxy[4], m_mode, i_mode, mskcol, icncol;
  long	   ob_spec;
  ICONBLK *iconblk;
  CICON   *cicn;
  WORD    *mask, *data, *dark = NULL;
  char     letter[2];
  WORD     selected, mindex[2], iindex[2], buf, xy[4];
  boolean  invert = FALSE;
	
  selected = pb->pb_currstate & SELECTED;
	
  xrect2array ((GRECT *)&pb->pb_xc, xy);
  vs_clip (xvdi_handle, TRUE, xy);   /* dfinit la zone rectangulaire */

  ob_spec	= pb->pb_parm;
  ob_x		= pb->pb_x;
  ob_y		= pb->pb_y;

  iconblk = (ICONBLK *)ob_spec;
  cicn    = ((CICONBLK *)ob_spec)->mainlist;
  m_mode  = MD_TRANS;

  if (selected) /* it was an objc_change */
  {
    if (cicn->sel_data != NULL)
    {
      mask = cicn->sel_mask;
      data = cicn->sel_data;
      if (cicn->num_planes > 1)
      {
	if (cicn->num_planes > 8)	/* TrueColor, carte couleur en mode RVB? */
	  i_mode = S_AND_D;
	else
	  i_mode = S_OR_D;
      }
      else
	i_mode = MD_TRANS;
    }
    else
    {
      mask = cicn->col_mask;
      data = cicn->col_data;
      
      if (cicn->num_planes > 1)
      {
	if (cicn->num_planes > 8)
	  i_mode = S_AND_D;
	else
	  i_mode = S_OR_D;
	dark = cicn->sel_mask;
      }
      else
	invert = TRUE;
    }
  }
  else
  {
    mask = cicn->col_mask;
    data = cicn->col_data;
	
    if (cicn->num_planes > 1)
    {
      if (cicn->num_planes > 8)
	i_mode = S_AND_D;
      else
	i_mode = S_OR_D;
    }
    else
      i_mode = MD_TRANS;
  }
	
  mindex [0] = ((iconblk->ib_char & 0x0f00) != 0x0100) ? (iconblk->ib_char & 0x0f00) >> 8 : WHITE;
  mindex [1] = WHITE;
	
  icncol = iindex[0] = (WORD)(((UWORD)iconblk->ib_char & 0xf000U) >> 12U);
  iindex[1] = WHITE;
  
  mskcol = (iconblk->ib_char & 0x0f00) >> 8;
  
  x = ob_x + iconblk->ib_xicon;
  y = ob_y + iconblk->ib_yicon;
  
  if (invert)
  {
    buf       = iindex[0];
    iindex[0] = mindex[0];
    mindex[0] = buf;
    i_mode    = MD_TRANS;
  }
  if (selected)
  {
    buf    = icncol;
    icncol = mskcol;
    mskcol = buf;
  }
	
  draw_bitblk (mask, x, y, iconblk->ib_wicon, iconblk->ib_hicon, 1, m_mode, mindex);
  draw_bitblk (data, x, y, iconblk->ib_wicon, iconblk->ib_hicon, cicn->num_planes, i_mode, iindex);
	
  if (dark)
  {
    mindex [0] = BLACK;
    mindex [1] = WHITE;
    draw_bitblk (dark, x, y, iconblk->ib_wicon, iconblk->ib_hicon, 1, MD_TRANS, mindex);
  }

  if (iconblk->ib_ptext[0])
  {
    x = ob_x + iconblk->ib_xtext;
    y = ob_y + iconblk->ib_ytext;
    
    pxy[0] = x;
    pxy[1] = y;
    pxy[2] = x + iconblk->ib_wtext - 1;
    pxy[3] = y + iconblk->ib_htext - 1;
		
    vswr_mode     (xvdi_handle, MD_REPLACE);		/* dessiner la boite texte (TextBox) */
    vsf_color     (xvdi_handle, mskcol);
    vsf_interior  (xvdi_handle, FIS_SOLID);
    vsf_perimeter (xvdi_handle, FALSE);
    v_bar         (xvdi_handle, pxy);
  }

  vswr_mode     (xvdi_handle, MD_TRANS);
  vst_font      (xvdi_handle, 1);	/* Systemfont */
  vst_height    (xvdi_handle, 4, &dummy, &dummy, &dummy, &dummy);
  vst_color     (xvdi_handle, icncol);
  vst_effects   (xvdi_handle, TXT_NORMAL);
  vst_alignment (xvdi_handle, ALI_LEFT, ALI_TOP, &dummy, &dummy);
  vst_rotation  (xvdi_handle, 0);
	
  if (iconblk->ib_ptext[0])
  {
    x += (iconblk->ib_wtext - strlen (iconblk->ib_ptext) * 6) / 2;
    y += (iconblk->ib_htext - 6) / 2;
  
    v_gtext (xvdi_handle, x, y, iconblk->ib_ptext);
  }

  letter[0] = iconblk->ib_char & 0xff;
  if (letter[0])
  {
    letter[1] = '\0';
    x = ob_x + iconblk->ib_xicon + iconblk->ib_xchar;
    y = ob_y + iconblk->ib_yicon + iconblk->ib_ychar;
    
    v_gtext (xvdi_handle, x, y, letter);
  }
	
  return (pb->pb_currstate & ~ SELECTED);
} /* draw_userdef */

/*****************************************************************************/

void draw_bitblk (WORD *p, WORD x, WORD y, WORD w, WORD h, WORD num_planes, WORD mode, WORD *index)
{
  WORD	 	pxy[8];
  MFDB	 	s, d;

  d.mp	= NULL; /* screen */
  s.mp	= (VOID *)p;
  s.fwp = w;
  s.fh	= h;
  s.fww = w >> 4;
  s.ff	= FALSE;
  s.np	= num_planes;

  pxy[0] = 0;
  pxy[1] = 0;
  pxy[2] = s.fwp - 1;
  pxy[3] = s.fh - 1;
  
  pxy[4] = x;
  pxy[5] = y;
  pxy[6] = pxy[4] + pxy [2];
  pxy[7] = pxy[5] + pxy [3];

  if (num_planes > 1)
    vro_cpyfm (xvdi_handle, mode, pxy, &s, &d);
  else
    vrt_cpyfm (xvdi_handle, mode, pxy, &s, &d, index);	 /* copy it */
}

/*****************************************************************************/

void xrect2array (const GRECT *rect, int *array)
{
  *array++ = rect->g_x;
  *array++ = rect->g_y;
  *array++ = rect->g_x + rect->g_w - 1;
  *array   = rect->g_y + rect->g_h - 1;
} /* xrect2array */

/* #endif */
/*****************************************************************************/

boolean init_xrsrc (int vdi_handle, GRECT *desk, int gl_wbox, int gl_hbox)
{
  xvdi_handle = vdi_handle;
  xdesk = *desk;
  xgl_wbox = gl_wbox;
  xgl_hbox = gl_hbox;
	
  return (TRUE);
}

/*****************************************************************************/

void term_xrsrc (void)
{
}
#endif

#endif /* def _XRSRC_ORIGINAL_ */
