/* Bibliothque GEM pour X-Windows
   (C) 1998, Emmanuel CURIS

   Bibliothque de ressources (RSRC lib)
   Attention : structure RSHDR dfinie dans X_ressources.h

   10 avril 1998
     * Fonction rsrc_load() fonctionne (aux limitations de X_ressources.c prs)
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <stdlib.h>

#include <Bib_X.h>

static Ressource *rsrc_ressource = NULL; /* Le ressource, unique, charg par rsrc_load */

/* --------------- rsrc_load() : charge un fichier *.RSC ---------------- *
 *   Renvoie FAUX si le chargement a chou                               *
 *           VRAI sinon                                                   *
 * ---------------------------------------------------------------------- */

int rsrc_load(const char *nom_fichier)
{

  if (nom_fichier == NULL)
  {
    printf("Nom de fichier incorrect\n");
    return FAUX;
  }

  if (rsrc_ressource) /* Dj un fichier charg par rsrc_load() */
  {
    printf("On ne peut charger qu'une seule ressource  la fois\n");
    printf("avec les fonctions rsrc_...().\n");
    printf("Pour utiliser davantage de ressources, utiliser\n");
    printf("les fonctions internes - cf. X_ressources.h\n");
    return FAUX;
  }

  /* On cr une structure pour charger le ressource */

  rsrc_ressource = RSC_creer();
  if (rsrc_ressource == NULL) /* Cration a rat */
    return FAUX;

  if ( RSC_charge(nom_fichier, rsrc_ressource) == FAUX ) /* Erreur  la lecture */
  {
    printf("rsrc_load : erreur au chargement\n");
    return FAUX;
  }

  printf("rsrc_load : fini, tout va bien\n");
  return VRAI; /* Fonctionne bien */
}

/* --------------- rsrc_free() : libre le ressource charg par rsrc_load -------- */

int rsrc_free(void)
{
  if (rsrc_ressource == NULL)
  {
    printf("Aucun fichier RSC charg par rsrc_load()");
    return -1; /* Pas de ressource charg */
  }

  RSC_effacer(rsrc_ressource); /* On efface le ressource */

  rsrc_ressource = NULL; /* Maintenant, possible d'en charger un autre */
  return 0; /* Pas d'erreur */
}

/* --------------- rsrc_gaddr() : cherche une adresse dans le ressource charg par rsrc_load */

int rsrc_gaddr(int type, int index, void *adresse)
{
  OBJECT *obj;
  unsigned long i;

  if (rsrc_ressource == NULL)
  {
    printf("Aucun fichier RSC charg par rsrc_load()");
    return -1; /* Pas de ressource charg */
  }

  return RSC_lit_adresse(rsrc_ressource, type, index, adresse);
}

int  rsrc_saddr      (int rstype, int rsid, void *lngval);

/* --------------- rsrc_obfix() : on convertit les coords. car. en coords. cran ----
   ATTENTION : c'est pour un objet -> valable quel que soit le ressource...           */

int rsrc_obfix(OBJECT *adresse, int objet)
{
  int haut_car, larg_car;
  int coord, frac;

  if (_Fonte == NULL)
    return FAUX; /* Problme : pas de fonte... */

  larg_car  = _Fonte->max_bounds.width;
  haut_car  = _Fonte->max_bounds.ascent + _Fonte->max_bounds.descent;
  /*  haut_car += _Fonte->ascent; */ /* Semble donner des boutons trop hauts */
  haut_car += _Fonte->descent;

  coord =  OB_X(adresse, objet) & 0xFF ; /* Premier octet = caractres */
  frac = ( OB_X(adresse, objet) & 0xFF00 ) >> 8; /* Deuxime octet = fractions de caractre */
  OB_X(adresse, objet)      = coord * larg_car + frac * larg_car / 256;

  coord =  OB_Y(adresse, objet) & 0xFF ; /* Premier octet = caractres */
  frac = ( OB_Y(adresse, objet) & 0xFF00 ) >> 8; /* Deuxime octet = fractions de caractre */
  OB_Y(adresse, objet)      = coord * haut_car + frac * haut_car / 256;

  OB_WIDTH(adresse, objet)  *= larg_car;
  OB_HEIGHT(adresse, objet) *= haut_car;

  return VRAI;
}
