/* Bibliothque GEM pour X-Windows
   (C) 1998, Emmanuel CURIS

   Bibliothque de base VDI (fonctions v_xxx)
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <stdlib.h>

#include "Bib_X.h"

/****** Control library ******************************************************/

void v_get_driver_info (int device_id, int info_select, unsigned char *info_string);
void v_opnwk           (int *work_in, int *handle, int *work_out);
void v_clswk           (int handle);
void v_clrwk           (int handle);

void v_updwk(int handle)
{
  Station_VDI *station;

  station = quelle_station(handle);
  if (station == NULL)
    return;

  if ( (station->type >= 0) && (station->type < 10) ) /* Station cran */
    return; /* Rien  faire */

  printf("Seules les stations cran sont gres pour l'instant\n");
}

void v_opnvwk(int *work_in, int *handle, int *work_out);

void v_clsvwk          (int handle);
void v_set_app_buff    (void *address, int nparagraphs);
int  v_bez_on          (int handle);
int  v_bez_off         (int handle);
int  v_bez_qual        (int handle, int prcnt);
void v_pat_rotate      (int handle, int angle);

/****** Output library *******************************************************/

/* ~~~~ Trac d'une ligne brise ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

void v_pline(int handle, int nbr_pts, int *coords)
{
  int i;
  Station_VDI *station;
  XPoint *points;

  if (nbr_pts <= 0)
  {
    printf("Nombre de points ngatif ou nul : opration annule\n");
    return;
  }

  station = quelle_station(handle);
  if (station == NULL)
    return;

  XSetForeground(_Display, station->gr_contexte, _Couleurs[station->lgn_couleur].pixel);

  /* On met le style de ligne */

  if (station->lgn_style <= SOLID)
  {
    XSetLineAttributes(_Display, station->gr_contexte, station->lgn_epaisseur, LineSolid,
		       CapNotLast, JoinMiter);
  }
  else
  {
    char tirets[6] = {5, 5, 6, 6, 8, 8};
    int nbr_tirets = 1;

    switch (station->lgn_style)
    {
    case LDASHED:
      tirets[0] =  4;
      tirets[1] =  4;
      nbr_tirets = 2;
      break;

    case DOTTED:
      tirets[0] = 2;
      tirets[1] = 6;
      nbr_tirets = 2;
      break;

    case DASHDOT:
      tirets[0] = 8;
      tirets[1] = 3;
      tirets[2] = 2;
      tirets[3] = 3;
      nbr_tirets = 4;
      break;

    case DASH:
      tirets[0] = 8;
      tirets[1] = 8;
      nbr_tirets = 2;
      break;

    case DASHDOTDOT:
      tirets[0] = 5;
      tirets[1] = 3;
      tirets[2] = 2;
      tirets[3] = 2;
      tirets[4] = 1;
      tirets[5] = 3;
      nbr_tirets = 6;
      break;
    }

    XSetDashes(_Display, station->gr_contexte, 0, tirets, nbr_tirets);
    XSetLineAttributes(_Display, station->gr_contexte, station->lgn_epaisseur, LineOnOffDash,
		       CapNotLast, JoinMiter);
  }

  if (nbr_pts == 1) /* Un seul point */
  {
    XDrawPoint(_Display, _Fen_redessin, station->gr_contexte, coords[0], coords[1]);
    return;
  }

  if (nbr_pts == 2) /* Deux points = une droite */
  {
    XDrawLine(_Display, _Fen_redessin, station->gr_contexte,
	      coords[0], coords[1], coords[2], coords[3]);
    return;
  }

  /* Plus de points : il faut faire une conversion pour XDrawLines... */

  points = (XPoint *) calloc(nbr_pts, sizeof(XPoint));
  if (points == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("Dans X_v_lib.c\n");
    return;
  }

  for (i = 0 ; i < nbr_pts ; i++)
  {
    points[i].x = (short) coords[2*i];
    points[i].y = (short) coords[2*i + 1];
  }

  XDrawLines(_Display, _Fen_redessin, station->gr_contexte, points, nbr_pts, CoordModeOrigin);

  free(points);
}

/* ----------- Polymark : pour l'instant, n'affiche que des points -------- */

void v_pmarker(int handle, int nbr_pts, int *xy)
{
  int i;
  Station_VDI *station;
  XPoint *points;

  station = quelle_station(handle);
  if (station == NULL)
    return;

  XSetForeground(_Display, station->gr_contexte, _Couleurs[station->mk_couleur].pixel);

  if (nbr_pts == 1)
  {
    XDrawPoint(_Display, _Fen_redessin, station->gr_contexte, xy[0], xy[1]);
    return;
  }

  if ( station->mk_type == PM_PLUS) /* Croix de type classique */
  {
    XSegment segments[2];

    for (i = 0 ; i < nbr_pts ; i++)
    {
      segments[0].x1 = (short) (xy[2 * i] - station->mk_taille);
      segments[0].y1 = (short) (xy[2 * i + 1]);
      segments[0].x2 = (short) (xy[2 * i] + station->mk_taille);
      segments[0].y2 = (short) (xy[2 * i + 1]);

      segments[1].x1 = (short) (xy[2 * i]);
      segments[1].y1 = (short) (xy[2 * i + 1]  - station->mk_taille);
      segments[1].x2 = (short) (xy[2 * i]);
      segments[1].y2 = (short) (xy[2 * i + 1] + station->mk_taille);

      XDrawSegments(_Display, _Fen_redessin, station->gr_contexte, segments, 2);
    }

    return;
  }

  points = (XPoint *) calloc(nbr_pts, sizeof(XPoint));
  if (points ==  NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("Dans X_v_lib.c\n");
    return;
  }

  for (i = 0 ; i < nbr_pts ; i++)
  {
    points[i].x = (short) xy[2 * i];
    points[i].y = (short) xy[2 * i + 1];
  }

  XDrawPoints(_Display, _Fen_redessin, station->gr_contexte, points, nbr_pts, CoordModeOrigin);
}

void v_gtext(int handle, int x, int y, char *texte)
{
  int x_texte, y_texte, larg_texte, haut_texte;
  unsigned long lg_texte;
  Station_VDI *station;

  if (texte == NULL) return;

  station = quelle_station(handle);
  if (station == NULL)
    return;

  lg_texte = strlen(texte);

  larg_texte = XTextWidth(_Fonte, texte, lg_texte);
  if (station->txt_just == ALI_CENTER) /* On centre sur x*/
    x_texte  = x - larg_texte / 2;
  else if (station->txt_just == ALI_RIGHT) /* On justifie  droite = fin du texte en x*/
    x_texte = x - larg_texte;
  else
    x_texte = x; /* Justif.  gauche : dbut du texte en x */

  if ( station->txt_vert == ALI_TOP )
    y_texte = y + _Fonte->max_bounds.ascent;
  else if ( station->txt_vert == ALI_HALF )
    y_texte = y + (_Fonte->max_bounds.ascent - _Fonte->max_bounds.descent) / 2;
  else if (station->txt_vert == ALI_BOTTOM )
    y_texte = y - _Fonte->max_bounds.descent;
  else
    y_texte = y; /* Pour voir */

  XSetBackground(_Display, station->gr_contexte, _Couleurs[BLANC].pixel); /* fond blanc sur atari */
  XSetForeground(_Display, station->gr_contexte, _Couleurs[station->txt_couleur].pixel);
  XDrawImageString(_Display, _Fen_redessin, station->gr_contexte,
		   x_texte, y_texte, texte, lg_texte);
}

void v_etext           (int handle, int x, int y, unsigned char *string, int *offsets);
void v_fillarea        (int handle, int count, int *xy);
void v_cellarray       (int handle, int *pxyarray, int row_length,
			int el_used, int num_rows, int wrt_mode, int *colarray);

void v_bar (int handle, int *coords)
{
  Station_VDI *station;

  station = quelle_station(handle);
  if (station == NULL)
    return;

  XSetForeground(_Display, station->gr_contexte, _Couleurs[station->fil_couleur].pixel);

  XFillRectangle(_Display, _Fen_redessin, station->gr_contexte, coords[0], coords[1],
		 coords[2] - coords[0] + 1, coords[3] - coords[1] + 1);
}

void v_arc             (int handle, int x, int y, int radius, int begang, int endang);
void v_pieslice        (int handle, int x, int y, int radius, int begang, int endang);
void v_circle          (int handle, int x, int y, int radius);
void v_ellipse         (int handle, int x, int y, int xradius, int yradius);
void v_ellarc          (int handle, int x, int y, int xradius, int yradius,
			int begang, int endang);
void v_ellpie          (int handle, int x, int y, int xradius,
			int yradius, int begang, int endang);
void v_rbox            (int handle, int *xyarray);
void v_rfbox           (int handle, int *xyarray);
void v_justified       (int handle, int x, int y, char *string,
			int length, int word_space, int char_space);
void v_contourfill     (int handle, int x, int y, int index);
void v_bez             (int handle, int count, int *xyarr,
			unsigned char *bezarr, int *minmax, int *npts, int *nmove);
void v_bez_fill        (int handle, int count, int *xyarr,
                          unsigned char *bezarr, int *minmax, int *npts, int *nmove);
void v_get_pixel       (int handle, int x, int y, int *pel, int *index);
void v_show_c          (int handle, int reset);
void v_hide_c          (int handle);
void v_exit_cur        (int handle);
void v_enter_cur       (int handle);
void v_curup           (int handle);
void v_curdown         (int handle);
void v_curright        (int handle);
void v_curleft         (int handle);
void v_curhome         (int handle);
void v_eeos            (int handle);
void v_eeol            (int handle);
void v_curtext         (int handle, char *string);
void v_rvon            (int handle);
void v_rvoff           (int handle);
void v_hardcopy        (int handle);
void v_dspcur          (int handle, int x, int y);
void v_rmcur           (int handle);
void v_form_adv        (int handle);
void v_output_window   (int handle, int *xyarray);
void v_clear_disp_list (int handle);
void v_bit_image       (int handle, char *filename,
                          int aspect, int x_scale, int y_scale,
                          int h_align, int v_align, int *xyarray);
void v_alpha_text      (int handle, char *string);
void v_sound           (int handle, int frequency, int duration);

void v_meta_extents(int handle, int min_x, int min_y, int max_x, int max_y)
{
  printf("Stations Metafile  pas supportes dans cette version\n");
}

void v_write_meta      (int handle, int num_intin, int *intin, int num_ptsin, int *ptsin);
void v_copies          (int handle, int count);
void v_orient          (int handle, int orientation);
void v_tray            (int handle, int tray);
int  v_xbit_image      (int handle, char *filename, int aspect,
                          int x_scale, int y_scale,
                          int h_align, int v_align, int rotate,
                          int background, int foreground, int *xy);
void v_setrgbi         (int handle, int primtype, int r, int g, int b, int i);
void v_topbot          (int handle, int height,
                          int *char_width, int *char_height,
                          int *cell_width, int *cell_height);
void v_offset          (int handle, int offset);
void v_fontinit        (int handle, int fh_high, int fh_low);
void v_escape2000      (int times);
