/* Bibliothque X-Window pour les interfaces
	    Compatible avec gem_gene.c (version GEM : rfrence)
	    Routines VDI avances
   (C) 1998, Emmanuel CURIS

 2 avril 1998
   * Cration du fichier
*/

#define _LIBRAIRIE_X_

#include <stdio.h>
#include <stdlib.h>

#include "Bib_X.h"

static Station_VDI *premiere_station = NULL;
static Station_VDI *derniere_station = NULL;
static unsigned int nbr_stations = 0;

int VDI_prem_handle(void);

/* ---------- Ouverture d'une station VDI ----------\
 | Renvoie la strucure prte  l'emploi             |
 |                                                  |
 | type : le type de station  ouvrir               |
 \------------------------------------------------- */

Station_VDI *VDI_ouvre(int type)
{
  XGCValues valeurs;
  unsigned long masque;
  Station_VDI *station;
  Window w;
  int x, y, handle;
  unsigned int larg, haut, bord, plans;

  if (type > 9) /* Autre que l'cran */
  {
    printf(" Pour avoir un affichage sur autre chose que l'cran,\n");
    printf("il faut utiliser la version TOS (atari) du programme.\n");
    return NULL;
  }

  station = (Station_VDI *) malloc( sizeof(Station_VDI) );
  if (station == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("Dans X_vdi.c\n");
    return NULL;
  }

  if (derniere_station == NULL) /* Aucune station */
  {
    premiere_station = station;
    derniere_station = station;
    handle = 1; /* Premire station possible */
  }
  else
  {
    handle = VDI_prem_handle();
    if ( handle < 0 )
    {
      printf("Plus de station disponible\n");
      free(station);
      return NULL;
    }

    derniere_station->suivante = station;
    derniere_station = station;
  }

  station->handle = handle;
  station->suivante = NULL;
  nbr_stations++;
  
  masque = GCFunction | GCForeground | GCBackground | GCLineWidth |
    GCLineStyle | GCFillStyle | GCFillRule | GCFont;

  valeurs.function   = GXcopy; /* quivalent du mode remplacement */
  valeurs.foreground = _Couleurs[NOIR].pixel;
  valeurs.background = _Couleurs[BLANC].pixel;
  valeurs.line_width = 1;
  valeurs.line_style = LineSolid; /* Ligne continue */
  valeurs.fill_style = FillSolid;
  valeurs.fill_rule  = WindingRule;
  valeurs.font       = _Fonte->fid;

  station->type        = type;
  station->gr_contexte = XCreateGC(_Display, _Fen_racine, masque, &valeurs);

  XGetGeometry(_Display, _Fen_racine, &w, &x, &y, &larg, &haut, &bord, &plans);
  station->larg = larg;
  station->haut = haut;
  station->n_plans = plans;

  station->x_pixel = ( (float) WidthMMOfScreen(_Ecran))  / ( (float) WidthOfScreen(_Ecran)) ; /* [mm / pixel] */
  station->y_pixel = (float) HeightMMOfScreen(_Ecran) / (float) HeightOfScreen(_Ecran); /* [mm / pixel] */

  if (gdos != -2)
  {
  }
  else
    station->nbr_fontes = 0;

  station->fonte       = _Fonte; /* Fonte systme par dfaut */

  station->lgn_couleur   = NOIR;
  station->lgn_style     = SOLID;
  station->lgn_epaisseur = 1;

  station->txt_couleur = NOIR;
  station->txt_just    = ALI_LEFT;
  station->txt_vert    = ALI_BOTTOM;

  station->mk_couleur  = NOIR;
  station->mk_taille   = 5; /* 5 pixels de haut  */
  station->mk_type     = 0; /* Points par dfaut */

  return station;
}

/* --- Fonction interne : donne le premier handle libre... ------- */

int VDI_prem_handle(void)
{
  Station_VDI *station = premiere_station;
  int handle = nbr_stations;
  boolean existe;

  if (station == NULL)
    return 1;

  do
  {
    handle++;
    existe = FAUX; /* On suppose qu'elle n'existe pas dj */
    for (station = premiere_station ; station ; station = station->suivante)
    {
      if ( station->handle == handle ) /* La station existe dj */
	existe = VRAI;
    }
  } while (existe == VRAI);

  return handle; /* Handle libre [pas forcment le premier...] */
}

Station_VDI *quelle_station(int handle)
{
  Station_VDI *station;

  if (premiere_station == NULL)
    return NULL;

  for (station = premiere_station ; station ; station = station->suivante)
  {
    if (station->handle == handle)
      return station;
  }

  printf("Aucune station n'a le handle indiqu\n");
  return NULL;
}

/* -------------- Fermeture d'une station VDI -------------------- */

int VDI_ferme(Station_VDI *station)
{
  Station_VDI *avant;

  if (station == NULL)
    return -1;
  
  XFreeGC(_Display, station->gr_contexte); /* Plus besoin du GC */

  if (premiere_station == station)
  {
    premiere_station = station->suivante;
    if (premiere_station == NULL) /* C'tait la dernire station libre */
      derniere_station = NULL;
  }
  else
  {
    for (avant = premiere_station ; avant->suivante != station ; avant = avant->suivante);
    avant->suivante = station->suivante; /* On la sort du chanage */

    if (derniere_station == station)
      derniere_station = avant;
  }

  nbr_stations--;
  free(station); /* Ni de la station */

  return 0; /* Pas d'erreur */
}
