/* Traitement de l'E.X.A.F.S - novembre 1997 - (c) Emmanuel CURIS
			   Tout ce qui concerne le traitement des spectres complexes

  27 mars 1998
    * Utilisation de systeme.h
    * Noms de fichier en majuscules

  26 mars 1998
    * Renomm err_pts_diff en nbrpts_incorrect

  16 mars 1998
    * Inclusion de erreurs.h [pour l'EXAFS]

  24 novembre 1997
    * Cration de l'historique
    * Inclusion de gem_var.h
*/

#define _COMPLEXE_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "ERREURS.H"
#include "SCIENCE.H"

/* ----------------------- Conversion graphe complexe ---> graphes rels ---------------------- *
 *    On peut rcuprer partie relle, partie imaginaire, module ou argument                    *
 ************************************************************************************************/

/* ~~~~~~~~~~~~~~~~~~~~~~~ Interface GEM pour ces oprations ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

void CMP_decompose(Graphe *complexe)
{
  OBJECT *adresse = arbres[DECOMP_FOURIER].adresse;

  if (complexe == NULL) return; /* Pas de graphe... */
  if (complexe->drapeaux.complexe == GR_NORMAL)
  {
    EXF_ERREUR(NON_COMPLEXE);
    return;
  }

  if (ext_formdo(DECOMP_FOURIER) == DTF_ANNULE)
    return; /* On annule le calcul */

  if ( OB_STATE(adresse, DTF_REELLE) & SELECTED ) /* On veut la partie relle      */
    CMP_reelle(complexe, NULL);
  
  if ( OB_STATE(adresse,DTF_IMAG) & SELECTED )    /* On veut la partie imaginaire  */
    CMP_imaginaire(complexe, NULL);

  if ( OB_STATE(adresse,DTF_MODULE) & SELECTED )  /* On veut le module (amplitude) */
    CMP_module(complexe, NULL);

  if ( OB_STATE(adresse,DTF_ARG) & SELECTED )     /* On veut l'argument (phase)    */
    CMP_argument(complexe, NULL);
}

/* ~~~~~~~~~~~~~~~~~~~~~~~ Oprations proprement dites ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ----------------------- Extraction de la partie imaginaire ----------------------------------\
 |    Ne renvoie rien (pour l'instant...)                                                       |
 |                                                                                              |
 | complexe : le graphe complexe (y = contient les valeurs complexes)                           |
 | imag     : le graphe contenant la partie imaginaire (NULL --> on le cre sur place)          |
 \--------------------------------------------------------------------------------------------- */

void CMP_imaginaire(Graphe *complexe, Graphe *imag)
{
  register Complexe *y_comp;
  register double   *x_comp, *x_reel, *y_reel;
  unsigned long index;

  if (complexe == NULL) return;
  if (complexe->drapeaux.complexe == 0)
  {
    EXF_ERREUR(NON_COMPLEXE);
    return;
  }

  if (imag == NULL) /* On cre un graphe pour recevoir la partie relle */
  {
    char titre[100];

#ifdef _FRANCAIS_
    sprintf(titre, "Partie imaginaire de %s", complexe->nom);
#endif
#ifdef _ENGLISH_
    sprintf(titre, "Imaginary part of %s", complexe->nom);
#endif

    imag = GRF_creer(complexe->nombre_point, GR_COMPIMA, GR_NORMAL, titre,
		     complexe->axe_x, complexe->axe_y, complexe->unit_x, complexe->unit_y);
    if (imag == NULL) return;
  }
  else if ( complexe->nombre_point != imag->nombre_point )
  {
    EXF_ERREUR(NBRPTS_INCORRECT);
    return;
  }

  y_comp = (Complexe *) complexe->y;

  *(imag->x) = *(complexe->x);
  *(imag->y) = y_comp->im; y_comp++;
  imag->y_min = *(imag->y);
  imag->y_max = *(imag->y);

  x_reel = imag->x + 1; y_reel = imag->y + 1;
  x_comp = complexe->x + 1;

  for (index = 1 ; index < complexe->nombre_point ; index++)
  {
    *x_reel = *x_comp;
    *y_reel = y_comp->im;

    if ( *y_reel < imag->y_min) imag->y_min = *y_reel;
    if ( *y_reel > imag->y_max) imag->y_max = *y_reel;

    x_reel++; x_comp++; y_reel++; y_comp++;
  }

  imag->x_min = complexe->x_min;
  imag->x_max = complexe->x_max;
}

/* ----------------------- Extraction du module ------------------------------------------------\
 |    Ne renvoie rien (pour l'instant...)                                                       |
 |                                                                                              |
 | complexe : le graphe complexe (y = contient les valeurs complexes)                           |
 | module   : le graphe contenant le module (NULL --> on le cre sur place)                     |
 \--------------------------------------------------------------------------------------------- */

void CMP_module(Graphe *complexe, Graphe *module)
{
  register Complexe *y_comp;
  register double   *x_comp, *x_reel, *y_reel;
  unsigned long index;

  if (complexe == NULL) return;
  if (complexe->drapeaux.complexe == GR_NORMAL)
  {
    EXF_ERREUR(NON_COMPLEXE);
    return;
  }

  if (module == NULL) /* On cre un graphe pour recevoir la partie relle */
  {
    char titre[100];

#ifdef _FRANCAIS_
    sprintf(titre, "Module de %s", complexe->nom);
#endif
#ifdef _ENGLISH_
    sprintf(titre, "Module of %s", complexe->nom);
#endif

    module = GRF_creer(complexe->nombre_point, GR_COMPMOD, GR_NORMAL, titre,
		       complexe->axe_x, complexe->axe_y, complexe->unit_x, complexe->unit_y);
    if (module == NULL) return;
  }
  else if ( complexe->nombre_point != module->nombre_point )
  {
    EXF_ERREUR(NBRPTS_INCORRECT);
    return;
  }

  x_reel = module->x; y_reel = module->y;
  x_comp = complexe->x;
  y_comp = (Complexe *) complexe->y;

  for (index = 0 ; index < complexe->nombre_point ; index++)
  {
    *x_reel = *x_comp;
    *y_reel = sqrt(y_comp->re * y_comp->re + y_comp->im * y_comp->im);
    
    x_reel++; x_comp++; y_reel++; y_comp++;
  }

  module->x_min = complexe->x_min; /* Pas modifi par l'opration */
  module->x_max = complexe->x_max;
  GRF_bornes(module, NE_PAS_FAIRE, DESORDONNE);
}

/* ----------------------- Extraction de l'argument (phase) ------------------------------------\
 |    Ne renvoie rien (pour l'instant...)                                                       |
 |                                                                                              |
 | complexe : le graphe complexe (y = contient les valeurs complexes)                           |
 | phase    : le graphe contenant l'argument (NULL --> on le cre sur place)                    |
 \--------------------------------------------------------------------------------------------- */

void CMP_argument(Graphe *complexe, Graphe *phase)
{
  register Complexe *y_comp;
  register double   *x_comp, *x_reel, *y_reel;
  unsigned long i, k = 1;
  double inter = 0;

  if (complexe == NULL) return;
  if (complexe->drapeaux.complexe == GR_NORMAL)
  {
    EXF_ERREUR(NON_COMPLEXE);
    return;
  }

  if (phase == NULL) /* On cre un graphe pour recevoir l'argument */
  {
    char titre[100];

#ifdef _FRANCAIS_
    sprintf(titre, "Argument de %s", complexe->nom);
#endif
#ifdef _ENGLISH_
    sprintf(titre, "Argument of %s", complexe->nom);
#endif

    phase = GRF_creer(complexe->nombre_point, GR_COMPARG, GR_NORMAL, titre,
		      complexe->axe_x, complexe->axe_y, complexe->unit_x, complexe->unit_y);
    if (phase == NULL) return;
  }
  else if ( complexe->nombre_point != phase->nombre_point )
  {
    EXF_ERREUR(NBRPTS_INCORRECT);
    return;
  }

  x_reel = phase->x; y_reel = phase->y;
  x_comp = complexe->x;
  y_comp = (Complexe *) complexe->y;

  for (i = 0 ; i < phase->nombre_point ; i++)
  {
    double x, y;
    *x_reel = *x_comp;

    if ( y_comp->re == 0) /* Imaginaire pur */
    {
      if ( y_comp->im < 0 )
      {
	x = HUGE_VAL;
	y = -M_PI_2;
      }
      else
      {
	x = -HUGE_VAL;
	y = M_PI_2;
      }
    }
    else
    {
      x = y_comp->im / y_comp->re;
      y = atan(x);
    }
    
    if (x < inter) k++;
    *y_reel = y + k * M_PI;
    inter = x;
    if ( (i > 0) && (*y_reel - *(y_reel -1) > M_PI_2 ) )
    {
      k--;
      *y_reel -= M_PI;
    }

    x_reel++; x_comp++; y_reel++; y_comp++;
  }

  phase->x_min = complexe->x_min; /* Pas modifis par l'opration */
  phase->x_max = complexe->x_max;
  GRF_bornes(phase, NE_PAS_FAIRE, DESORDONNE);
}

/* ----------------------- Extraction de la partie relle --------------------------------------\
 |    Ne renvoie rien (pour l'instant...)                                                       |
 |                                                                                              |
 | complexe : le graphe complexe (y = contient les valeurs complexes)                           |
 | reelle   : le graphe contenant la partie relle (NULL --> on le cre sur place)              |
 \--------------------------------------------------------------------------------------------- */

void CMP_reelle(Graphe *complexe, Graphe *reelle)
{
  register Complexe *y_comp;
  register double   *x_comp, *x_reel, *y_reel;
  unsigned long index;

  if (complexe == NULL) return;
  if (complexe->drapeaux.complexe == 0)
  {
    EXF_ERREUR(NON_COMPLEXE);
    return;
  }

  if (reelle == NULL) /* On cre un graphe pour recevoir la partie relle */
  {
    char titre[100];

    sprintf(titre, "Partie r"e_AIGU"elle de %s", complexe->nom);
    reelle = GRF_creer(complexe->nombre_point, GR_COMPREEL, GR_NORMAL, titre,
		       complexe->axe_x, complexe->axe_y, complexe->unit_x, complexe->unit_y);
    if (reelle == NULL) return;
  }
  else if ( complexe->nombre_point != reelle->nombre_point )
  {
    EXF_ERREUR(NBRPTS_INCORRECT);
    return;
  }

  y_comp = (Complexe *) complexe->y;
  
  *(reelle->x) = *(complexe->x);
  *(reelle->y) = y_comp->re; y_comp++;
  reelle->y_min = *(reelle->y);
  reelle->y_max = *(reelle->y);
  
  x_reel = reelle->x + 1; y_reel = reelle->y + 1;
  x_comp = complexe->x + 1;

  for (index = 1 ; index < complexe->nombre_point ; index++)
  {
    *x_reel = *x_comp;
    *y_reel = y_comp->re;

    if ( *y_reel < reelle->y_min) reelle->y_min = *y_reel;
    if ( *y_reel > reelle->y_max) reelle->y_max = *y_reel;

    x_reel++; x_comp++; y_reel++; y_comp++;
  }

  reelle->x_min = complexe->x_min;
  reelle->x_max = complexe->x_max;
}
