/* Traitement de l'E.X.A.F.S. sur Atari
				Tout ce qui concerne l'EXAFS sens strict (extraction des oscillations)
   (c) 1996 - Emmanuel CURIS

    2 avril 1998
     * Augment paramtre de lissage possible : -> 9999 maintenant (!)

   16 & 17, 19 & 20, 24 - 26 mars 1998
     * Continu la cration du fichier d'historique
     * Utilisation de _PRG_station partout
     * Termin remplacement de effacer_graphe() par GRF_efface()
     * Continu la cration du fichier d'historique
     * Extraction "automatique" de l'exafs : deuxime et troisime tape devraient fonctionner
     * Remplac les inclusions d'interface par le fichier systeme.h
     * Commenc  adapter les accents pour la portabilit...
     * Extraction de mu_0, premire tape en cours
     * Remplac quelques effacer_graphes par GRF_efface()
     * Extraction de la matrice par reproduction : sauvegarde de l'historique
     * Inclus "extrait.h" : extraction du signal
     * Inclus "erreur.h" (pour l'exafs)
     * Commenc la routine d'extraction en une seule tape

   12 janvier 1998
     * Rorganis l'historique
     * Inclus les messages d'erreur GEM

   15 dcembre 1997
     * Essais sur MU3_calc() : bizarreries pour les points extrmes
         --> rendu un peu plus propre et ajout des commentaires. Semble fonctionner pour le moment...
     * Remplac les OB_STATE() par OBJ_choisit() : plus portable...
     * Corrig EXF_k_p_n() : crait un Nan si division par zro...

	 21, 22 novembre 1997
	   * dplac le traitement des points aberrants dans grf_stat.c
     * corrig une bte erreur dans mu2_ouvre -> (graphes exafs et mu_0 confondus -> plantage)
     * remarqu une chose : si on prend un mu_0 cr  l'tape 2 pour normaliser, graphe inutilisable (aberrances...)
     * essais de vitesse  : comparaison **tableaux + tableaux[][] et *tableaux[] (Bruno...)
                 Rsultat : mthode **tableaux un poil + rapide avec Turbo C sur Hades 40
*/

#define _EXAFS_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "ERREURS.H"
#include "EXTRAIT.H"
#include "FOURIER.H" /* Pour TF_ouvre() dans traitement auto. */
#include "SCIENCE.H"

#include "LISSAGE.H" /* Pour les lissages par spline */
#include "XANES.H"

/* ******* Modlisation et limination de tout ce qui n'est pas le spectre proprement dit *******
 ********************************************************************************************** */

/* ~~~~~~~~~~~~~~~~~ Extraction en une seule tape du signal E.X.A.F.S. ~~~~~~~~~~~~~~~~~~~\
 |      Ne renvoie rien                                                                    |
 |                                                                                         |
 | brut : le graphe brut qu'il faudra traiter                                              |
 \---------------------------------------------------------------------------------------- */

void EXF_reproduit(Graphe *brut)
{
  int retour;
  Graphe *resultat = NULL;

  do
  {
    retour = ext_formdo(EXAFS_GENERAL);
    if ( retour == EXF_ANNULER ) /* On annule le calcul */
      return;

    if ( retour == EXF_SAUVER ) /* On sauve les paramtres de calcul */
    {
      EXF_svparams();
    }

    if ( retour == EXF_CHARGER ) /* On recharge les paramtres de calcul */
      {
      }
  } while (retour != EXF_CALCULER);

  if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU_MATRICE, QUESTION) ) /* On limine la matrice */
  {
    MAT_infos infos;
    char titre[300];

    sprintf(titre, "%s - "MU"(matrice)", brut->nom);

    resultat = GRF_creer(brut->nombre_point, GR_MOINSMAT, GR_NORMAL, titre,
			 E_AIGU"nergie", "Absorbance", UNIT_EV, UNIT_ARB);
    if (resultat == NULL)
      return;

    resultat->x_min = brut->x_min;
    resultat->x_max = brut->x_max;
    memcpy(resultat->x, brut->x, sizeof(double) * brut->nombre_point);

    infos.e_1 = prend_fvaleur(EXAFS_GENERAL, EXF_VALEUR_E1);
    if ( OBJ_choisit(EXAFS_GENERAL, EXF_MUM_DROITE, QUESTION) ) /* On utilise une droite */
      infos.type = MUM_DROITE;
    else if ( OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT1, QUESTION) ) /* On utilise une victorine */
      infos.type = MUM_VICTORINE_1;
    else if ( OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT2, QUESTION) ) /* On utilise une victorine */
      infos.type = MUM_VICTORINE_2;

    MAT_calc(brut, &infos); /* On calcule le polynme modlisant la matrice */
    MAT_soustrait(brut, resultat, &infos);

    if ( Histoire ) /* On sauve les informations d'extraction */
    {
      fprintf(Histoire, "\n--- Soustraction de la contribution de la matrice ---\n");
      fprintf(Histoire, "Spectre initial  : %s\n", brut->nom);
      fprintf(Histoire, "Spectre obtenu   : %s\n", resultat->nom);
      fprintf(Histoire, "M"e_AIGU"thode utilis"e_AIGU"e : ");
      if (infos.type == MUM_DROITE)
      {
	fprintf(Histoire, " soustraction d'une droite, A + B * X\n");
	fprintf(Histoire, "                   A = %f\n", infos.coefs[0]);
	fprintf(Histoire, "                   B = %f\n", infos.coefs[1]);
	fprintf(Histoire, "                   Valeur de E1 : %f\n", infos.e_1);
      }
      if (infos.type == MUM_VICTORINE_1)
      {
	fprintf(Histoire, " soustraction d'une victorine, A + B / X^3\n");
	fprintf(Histoire, "                   A = %f\n", infos.coefs[0]);
	fprintf(Histoire, "                   B = %f\n", infos.coefs[1]);
	fprintf(Histoire, "                   Valeur de E1 : %f\n", infos.e_1);
      }
      if (infos.type == MUM_VICTORINE_2)
      {
	fprintf(Histoire, " soustraction d'une victorine, A + B / X^3 + C / X^4\n");
	fprintf(Histoire, "                   A = %f\n", infos.coefs[0]);
	fprintf(Histoire, "                   B = %f\n", infos.coefs[1]);
	fprintf(Histoire, "                   C = %f\n", infos.coefs[2]);
	fprintf(Histoire, "                   Valeur de E1 : %f\n", infos.e_1);
      }
    }

    brut = resultat; /* Pour la suite, on utilise celui-ci */
  }

  if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_REFAIT, QUESTION) ) /* ter l'absorption atomique, 1re tape */
  {
    Graphe *original = brut;
    Graphe *mu_0 = NULL; /* Le polynme soustrait */
    MU1_infos infos;
    MU2_infos infos_2;
    MU3_infos infos_3;
    char titre[300];
    unsigned long index;

    sprintf(titre, "%s - "MU"0, "e_AIGU"tape 1", brut->nom);
    resultat = GRF_creer(brut->nombre_point, GR_EXAFS1, GR_NORMAL, titre,
			 "Nombre d'onde", "Signal E.X.A.F.S.", UNIT_1_ANG, UNIT_ARB);
    if (resultat == NULL) /* On ne peut rien faire */
      return;

    sprintf(titre, MU"0 "o_CIRC"t"e_AIGU" "a_GRAVE" %s, "e_AIGU"tape 1", brut->nom);
    mu_0 = GRF_creer(brut->nombre_point, GR_MU0, GR_NORMAL, titre,
		     "Nombre d'onde", MU"0 "o_CIRC"t"e_AIGU, UNIT_1_ANG, SS_UNITE);
    if (mu_0 == NULL)
      return; /* On ne peut rien faire */

    infos.e_0   = prend_fvaleur(EXAFS_GENERAL, EXF_VALEUR_E0);
    infos.e_2   = prend_fvaleur(EXAFS_GENERAL, EXF_VALEUR_E2);
    infos.degre = prend_valeur (EXAFS_GENERAL, EXV_DEGRE_MU0_I);
    infos.deg_affiche = prend_valeur(EXAFS_GENERAL, EXF_VALEUR_K);
    infos.exafs = resultat;
    infos.mu_0  = mu_0;

    if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_RAPIDE, QUESTION) )
      infos.methode = ELIM_MU0_RAPIDE;
    if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_LENT  , QUESTION) )
      infos.methode = ELIM_MU0_LENT;

    graf_mouse(S_ABEILLE, NULL); /* Souris en abeille  */
    MU1_calc   (brut, &infos);     /* Calcul du polynme */
    MU1_extrait(brut, &infos);     /* Extraction         */
    graf_mouse(S_FLECHE, NULL);  /* Souris normale     */

    if (Histoire) /* On ralise donc l'historique */
    {
      int i;

      fprintf(Histoire, "\n--- "E_AIGU"limination de l'absorption atomique ---\n");
      fprintf(Histoire, "Spectre trait"e_AIGU" : %s\n", brut->nom);
      fprintf(Histoire, "Signal extrait : %s\n", resultat->nom);
      fprintf(Histoire, "                 (correspond  k^%d * "KHI"(k)\n", infos.deg_affiche);
      fprintf(Histoire, "M"e_AIGU"thode        : soustraction d'un polyn"o_CIRC"me\n");
      fprintf(Histoire, "                 obtenu par lissage d'un spectre complet au-del"a_GRAVE" de E2\n");
      fprintf(Histoire, "                 extrapol"e_AIGU" entre E0 et E2\n");
      fprintf(Histoire, "                 (premi"e_GRAVE"re "e_AIGU"tape du logiciel EDA)\n");
      fprintf(Histoire, "Polyn"o_CIRC"me       : ");
      if ( infos.methode == ELIM_MU0_RAPIDE )
	fprintf(Histoire, "meilleur polyn"o_CIRC"me au sens des moindres carr"e_AIGU"s\n");
      else if ( infos.methode == ELIM_MU0_LENT )
	fprintf(Histoire, "obtenu par lissage [algorithme d'EDA]\n");
      fprintf(Histoire, "           degr"e_AIGU" : %d (A + B * X + ...)\n", infos.degre);
      for (i = 0 ; i <= infos.degre ; i++)
      {
	fprintf(Histoire, "           %c = ", 'A' + i);
	if ( infos.coefs[i] < .0001 )
	  fprintf(Histoire, "%e\n", infos.coefs[i]);
	else
	  fprintf(Histoire, "%f\n", infos.coefs[i]);
      }
      fprintf(Histoire, "Seuil       E0 = %f eV\n", infos.e_0);
      fprintf(Histoire, "E2             = %f eV\n", infos.e_2);
      fprintf(Histoire, "Polyn"o_CIRC"me utilis"e_AIGU" : %s\n", mu_0->nom);
    }

    brut = resultat; /* Pour la suite */

		/* ~~~~~~~~~~~ On passe  la seconde tape de lissage ~~~~~~~~~~~~~ */

		sprintf(titre, "Signal extrait de %s, "e_AIGU"tape 2", original->nom);
		resultat = GRF_creer(brut->nombre_point, GR_EXAFS2, GR_NORMAL, titre, "Nombre d'onde", "Signal E.X.A.F.S.", UNIT_1_ANG, UNIT_ARB);
		if (resultat == NULL) /* On ne peut rien faire */
			return;
		infos_2.exafs = resultat; /* Le signal exafs corrig aprs cette tape */

		infos_2.mu_0_ref = mu_0; /* Polynme soustrait  l'tape d'avant */
		sprintf(titre, MU"0 "o_CIRC"t"e_AIGU" "a_GRAVE" %s, "e_AIGU"tape 2", original->nom);
		mu_0 = GRF_creer(brut->nombre_point, GR_MU0, GR_NORMAL, titre, "Nombre d'onde", MU"0 "o_CIRC"t"e_AIGU, UNIT_1_ANG, SS_UNITE);
		if (mu_0 == NULL)
			return; /* On ne peut rien faire */
		infos_2.mu_0 = mu_0;

		infos_2.degre = prend_valeur(EXAFS_GENERAL, EXF_DEGRE_MU0_II); /* Degr du polynme */
		if ( OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_RAPID, QUESTION) )  /* Objet slectionn */
			infos_2.methode = ELIM2_RAPIDE;
		else
			infos_2.methode = ELIM2_PRECISE;

		graf_mouse(S_ABEILLE, NULL); /* Souris en abeille  */

		memcpy(resultat->x, brut->x, sizeof(double) * brut->nombre_point); /* Prparation des X */
		memcpy(mu_0->x,     brut->x, sizeof(double) * brut->nombre_point);

		mu_0->x_min     = brut->x_min; mu_0->x_max     = brut->x_max;
		resultat->x_min = brut->x_min; resultat->x_max = brut->x_max;

		MU2_calc   (brut, &infos_2); /* Calcul du polynme */
		MU2_extrait(brut, &infos_2); /* Extraction         */
		graf_mouse(S_FLECHE, NULL);  /* Souris normale     */

		if (Histoire)
		{
			int i;

			fprintf(Histoire, "\n--- "E_AIGU"limination de l'absorption atomique ---\n");
			fprintf(Histoire, "    (Correction de l'extraction : 2e "e_AIGU" de l'extraction\n");
			fprintf(Histoire, "Spectre trait"e_AIGU"   : %s\n", brut->nom);
			fprintf(Histoire, "Spectre originel : %s\n", original->nom);
			fprintf(Histoire, "Signal extrait   : %s\n", resultat->nom);
			fprintf(Histoire, "                 (correspond  k^%d * "KHI"(k)\n", infos.deg_affiche);
			fprintf(Histoire, "M"e_AIGU"thode          : soustraction d'un polyn"o_CIRC"me\n");
			fprintf(Histoire, "                   obtenu par lissage du spectre complet\n");
			fprintf(Histoire, "                   (seconde "e_AIGU"tape du logiciel EDA)\n");
			fprintf(Histoire, "Polyn"o_CIRC"me         : ");
			if ( infos_2.methode == ELIM2_RAPIDE )
				fprintf(Histoire, "meilleur polyn"o_CIRC"me au sens des moindres carr"e_AIGU"s\n");
			else if ( infos_2.methode == ELIM2_PRECISE )
				fprintf(Histoire, "obtenu par lissage [algorithme d'EDA]\n");
			fprintf(Histoire, "             degr"e_AIGU" : %d (A + B * X + ...)\n", infos_2.degre);
			for (i = 0 ; i <= infos_2.degre ; i++)
			{
				fprintf(Histoire, "           %c = ", 'A' + i);
				if ( infos_2.coefs[i] < .0001 )
					fprintf(Histoire, "%e\n", infos_2.coefs[i]);
				else
					fprintf(Histoire, "%f\n", infos_2.coefs[i]);
			}
			fprintf(Histoire, "Absorption atomique compl"e_GRAVE"te : %s\n", mu_0->nom);
		}

		brut = resultat; /* Pour la suite */

		/* ~~~~~~~~~~~ On passe  la dernire tape de lissage ~~~~~~~~~~~~~ */

		sprintf(titre, "Signal extrait de %s, "e_AIGU"tape 3", original->nom);
		resultat = GRF_creer(brut->nombre_point, GR_EXAFS, GR_NORMAL, titre, "Nombre d'onde", "Signal E.X.A.F.S.", UNIT_1_ANG, UNIT_ARB);
		if (resultat == NULL) /* On ne peut rien faire */
			return;
		infos_3.exafs = resultat; /* Le signal exafs corrig aprs cette tape */

		infos_3.mu_0_ref = mu_0; /* Polynme soustrait  l'tape d'avant */
		sprintf(titre, MU"0 "o_CIRC"t"e_AIGU" "a_GRAVE" %s, "e_AIGU"tape 3", original->nom);
		mu_0 = GRF_creer(brut->nombre_point, GR_MU0, GR_NORMAL, titre, "Nombre d'onde", MU"0 "o_CIRC"t"e_AIGU, UNIT_1_ANG, SS_UNITE);
		if (mu_0 == NULL)
			return; /* On ne peut rien faire */
		infos_3.mu_0 = mu_0;

		infos_3.parametre = prend_fvaleur(EXAFS_GENERAL, EXF_PARAM_LISSAG);
		
		graf_mouse(S_ABEILLE, NULL); /* Souris en abeille  */

		memcpy(resultat->x, brut->x, sizeof(double) * brut->nombre_point); /* Prparation des X */
		memcpy(mu_0->x,     brut->x, sizeof(double) * brut->nombre_point);

		mu_0->x_min     = brut->x_min; mu_0->x_max     = brut->x_max;
		resultat->x_min = brut->x_min; resultat->x_max = brut->x_max;

		MU3_calc   (brut, &infos_3); /* Calcul du polynme */
		MU3_extrait(brut, &infos_3); /* Extraction         */

		/* On normalise le spectre extrait */

		for (index = 0 ; index < mu_0->nombre_point ; index++)
		{
			(mu_0->y)[index] += (infos_3.mu_0_ref->y)[index];

			if ( (mu_0->y)[index] )
				(resultat->y)[index] /= (mu_0->y)[index]; /* On normalise */
			else
				EXF_ERREUR(DIV_ZERO);
		}

		EXF_k_p_n(resultat, infos.deg_affiche); /* Pour conserver le spectre en k^n * khi(k) */

		GRF_bornes(resultat, NE_PAS_FAIRE, DESORDONNE); /* Bornes en Y ont pu changer... */
		GRF_bornes(mu_0,     NE_PAS_FAIRE, DESORDONNE);

		graf_mouse(S_FLECHE, NULL);  /* Souris normale     */

		if (Histoire)
		{
			fprintf(Histoire, "\n--- "E_AIGU"limination de l'absorption atomique ---\n");
			fprintf(Histoire, "    (Correction de l'extraction : 3e "e_AIGU" de l'extraction\n");
			fprintf(Histoire, "Spectre trait"e_AIGU"   : %s\n", brut->nom);
			fprintf(Histoire, "Spectre originel : %s\n", original->nom);
			fprintf(Histoire, "Signal extrait   : %s\n", resultat->nom);
			fprintf(Histoire, "                 (correspond  k^%d * "KHI"(k)\n", infos.deg_affiche);
			fprintf(Histoire, "                 (le signal est normalis"e_AIGU"\n");
			fprintf(Histoire, "M"e_AIGU"thode          : soustraction d'une courbe\n");
			fprintf(Histoire, "                   obtenue par lissage du spectre complet\n");
			fprintf(Histoire, "                   (troisi"e_GRAVE"me "e_AIGU"tape du logiciel EDA)\n");
			fprintf(Histoire, "Param"e_GRAVE"tre de lissage : ");
			if ( fabs(infos_3.parametre) < 0.001 )
				fprintf(Histoire, "%e\n", infos_3.parametre);
			else
				fprintf(Histoire, "%f\n", infos_3.parametre);
			fprintf(Histoire, "Absorption atomique compl"e_GRAVE"te : %s\n", mu_0->nom);
		}

		brut = resultat; /* Pour la suite */
	}
}

/* ------------ Sauvegarde des paramtres d'extraction ----------------------------------- */

void EXF_svparams(void)
{
  FILE *fichier;
  char *nom_fichier;

#ifdef _FRANCAIS_
  nom_fichier = SV_selecteur("*.CNF", "Sauver les param"e_GRAVE"tres d'extraction");
#endif
#ifdef _ENGLISH_
  nom_fichier = SV_selecteur("*.CNF", "Save extraction parameters");
#endif
  if ( nom_fichier == NULL ) /* Problme */
  {
    ERREUR(ERREUR_ECRITURE);
    return;
  }

  fichier = fopen(nom_fichier, "w");
  free(nom_fichier);
  if (fichier == NULL)
  {
    ERREUR(ERREUR_ECRITURE);
    return;
  }

  fclose(fichier);
}

/* ----------------- Soustraction d'un 0 dj calcul au pralable ---------------------- *
 *******************************************************************************************/

/* ~~~~~~~~~~~~~~~~~ Ouverture de la fentre associe  cette tape ~~~~~~~~~~~~~~~~~~~~~~ */

void MU0_ouvre(Graphe *brut, Graphe *mu_0)
{
  Fenetre *fenetre;
  MU0_infos *infos = NULL;
  char titre[100];

  if (brut == NULL) return; /* Vraiment rien  faire */
  if (mu_0 == NULL) return;

  if ( arbres[OTE_MU0_CALC].ouvert == VRAI ) /* Arbre dj ouvert */
  {
    Graphe *vieux; /* Ancien spectre travaill dans ce module */
    int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

    fenetre = ARB_fenetre( &(arbres[OTE_MU0_CALC]) );
    vieux = (Graphe *) fenetre->pointeur;
    if ( (reponse == 1) || (vieux == brut) ) /* On active seulement */
    {
      FEN_premierplan(fenetre); /* Fentre au premier plan */
      return; /* Rien d'autre  faire */
    }
    else if (reponse == 2) /* On rappelle, pour le nouveau graphe */
    {
      if ( brut->drapeaux.protege )
      {
	EXF_ERREUR(GRF_PROTEGE);
	return;
      }

      infos = (MU0_infos *) fenetre->adresse;
      vieux->drapeaux.protege = FAUX;       /* On a fini de travailler avec */
      infos->mu_0->drapeaux.protege = FAUX;
      
      GRF_efface(infos->exafs, VRAI, VRAI);
      
      FEN_premierplan(fenetre); /* Fentre au premier plan */
    }
    else /* On annule l'appel */
      return;
  }
  else /* Arbre non ouvert : on l'ouvre */
  {
    if ( brut->drapeaux.protege )
    {
      EXF_ERREUR(GRF_PROTEGE);
      return;
		}

		fenetre = creer_mixte_fenetre(NAME|CLOSER|FULLER|INFO|SIZER|MOVER, OTE_MU0_CALC, HAUT,
		                        RDS_mu0, ( boolean (*)() ) NULL, CLC_mu0,
	  	                      MSG_mu0, RCT_pointeur, ( boolean (*)() ) NULL);
		if (fenetre == NULL) return; /* Rien  faire... */
	}

	if (infos == NULL) /* Aucune info n'existe dj */
	{
		infos = (MU0_infos *) malloc( (size_t) sizeof(MU0_infos) );
		if (infos == NULL)
		{
			ferme_fenetre(fenetre);
			ERREUR(MANQUE_MEMOIRE);
			return;
		}
		infos->b = NULL; infos->c = NULL; infos->d = NULL;
	}
	fenetre->pointeur = brut;  /* Graphe trait              */
	fenetre->adresse  = infos; /* Infos d'extraction de mu_0 */

	sprintf(titre, "Spectre E.X.A.F.S. de %s", brut->nom);

	infos->exafs = GRF_creer(brut->nombre_point, GR_EXAFS1, GR_NORMAL, titre, "", "khi(k)", UNIT_ARB, UNIT_NORM);
	if (infos->exafs == NULL)
	{
		free(infos);
		ferme_fenetre(fenetre);
		return;
	}
	infos->mu_0 = mu_0;

	infos->exafs->drapeaux.protege = VRAI; /* On protge le graphe de toute btise */
	brut->drapeaux.protege = VRAI; /* Calcul en cours sur ces spectres -> rien de faisable */
	mu_0->drapeaux.protege = VRAI;

	if ( OBJ_choisit(OTE_MU0_CALC, MU0CALC_ECH_K, QUESTION) == VRAI ) /* chelle des nombres d'onde */
	{	
		infos->methode = MU0CALC_ECH_K;
		GRF_legende(infos->exafs, AXE_X, "Nombre d'onde, k", UNIT_1_ANG);
	}
	if ( OBJ_choisit(OTE_MU0_CALC, MU0CALC_ECH_EV, QUESTION) == VRAI ) /* chelle des nergies       */
	{
		infos->methode = MU0CALC_ECH_EV;
		GRF_legende(infos->exafs, AXE_X, E_AIGU"nergie", UNIT_EV);
	}

	/* On cherche la valeur de E0 la plus plausible */
	infos->e_0   = cherche_e0  (brut, FAUX);                  /* E0 : maximum de la drive */
	infos->degre = prend_valeur(OTE_MU0_CALC, MU0CALC_DEGRE);

	graf_mouse(S_ABEILLE, NULL); /* Souris en abeille  */
	MU0_prepare(infos);          /* Prparation        */
	MU0_extrait(brut, infos);    /* Extraction         */
	graf_mouse(S_FLECHE, NULL);  /* Souris normale     */

	fenetre->drapeaux.redessin = VRAI;
	fenetre->drapeaux.pointeur = VRAI;
	fenetre->msg_masque.fermeture    = VRAI; /* Messages traits par MSG_mu01 */
	fenetre->msg_masque.taille       = VRAI;
	fenetre->msg_masque.premier_plan = VRAI;

	met_fvaleur(OTE_MU0_CALC, MU0CALC_E0, infos->e_0);
}

/* ~~~~~~~~~~~~~ Gestion de la fentre d'limination de 0 dja calcul ~~~~~~~~~~~~~~~ */

/* ------------- Redessin de la fentre d'limination de 0 dja calcul ---------------\
 |   Le clipping est dj install correctement lors de l'appel                            |
 |
 | fenetre    : structure associe  la fentre considre
 | x, y       : coordonnes de la zone de redessin
 | larg, haut : dimensions de la zone de redessin
 \---------------------------------------------------------------------------------------- */

void RDS_mu0(Fenetre *fenetre, int x, int y, int larg, int haut)
{
  Graphe *graphe;
  MU0_infos *infos;
  DES_graphe param_dessin;
  double x_echelle, y_echelle;
  int y_trace, haut_trace;

  vsl_type (_PRG_station->handle, 1);
  vswr_mode(_PRG_station->handle, MD_REMPLACE);
  vsf_color(_PRG_station->handle, BLANC); /* couleur de remplissage: blanc */

  points[0] = x;            points[1] = y;
  points[2] = x + larg - 1; points[3] = y + haut + 1;
  v_bar(_PRG_station->handle, points); /* On met le fond au blanc */

  y_trace    = fenetre->w_y    + OB_HEIGHT(arbres[OTE_MU0_CALC].adresse, ROOT);
  haut_trace = fenetre->w_haut - OB_HEIGHT(arbres[OTE_MU0_CALC].adresse, ROOT);

  graphe = (Graphe *)    fenetre->pointeur;
  infos  = (MU0_infos *) fenetre->adresse;
  x_echelle = ( (double) fenetre->w_larg ) / (graphe->x_max - graphe->x_min);
  y_echelle = ( (double) haut_trace )      / (graphe->y_max - graphe->y_min);

  /* On trace la courbe exprimentale - en noir */

  param_dessin.graphe  = graphe;
  param_dessin.couleur = NOIR;
  param_dessin.style   = 1;
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	     graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);

  /* On trace la courbe  soustraire - en bleu */	

  if (infos->methode == MU0CALC_ECH_EV) /* chelle d'nergie -> pas de problme */
  {
    param_dessin.graphe  = infos->mu_0;
    param_dessin.couleur = BLEU;
    param_dessin.style   = 1;
    TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	       graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);
  }
  else if (infos->methode == MU0CALC_ECH_K) /* chelle des k ==> convertir avant de dessiner */
  {
    double coefficient = 2 * MASSE_ELECTRON * EV_V_JOULE / (H_BARRE * H_BARRE);
    double x;
    register double *zone_x = infos->mu_0->x, *zone_y = infos->mu_0->y;
    unsigned long index, compteur = 0;

    vsl_color(_PRG_station->handle, BLEU);

    for (index = 0 ; index < infos->mu_0->nombre_point ; index++)
    {
      x = *zone_x * M_V_ANGSTROM; /* Conversion en m^-1 */
      x *= x / coefficient;
      if ( x < 0 )
      {
	x = infos->e_0 - x;
      }
      else
      {
	x += infos->e_0;
      }

      points[compteur]     = fenetre->w_x + x_echelle * (x - graphe->x_min);
      points[compteur + 1] = y_trace      + y_echelle * (graphe->y_max - *zone_y);
      compteur += 2;
      if (compteur >= 256) /* Tableau de points complet */
      {
	v_pline(_PRG_station->handle, 128, points);
	points[0] = points[254];
	points[1] = points[255];
	compteur = 2;
      }

      zone_x++; zone_y++;
    }

    v_pline(_PRG_station->handle, (int) (compteur+1)/2, points);
  }

  /* On met les barres */

  vsl_color(_PRG_station->handle, ROUGE); /* Barre rouge : E0 */
  points[0] = fenetre->w_x + x_echelle * (infos->e_0 - graphe->x_min);
  points[1] = y_trace;
  points[2] = points[0];
  points[3] = y_trace + haut_trace;
  v_pline(_PRG_station->handle,2,points);

  /* S'il le faut, on met le pointeur */

#if GEM
  if ( (_x_pointeur > fenetre->w_x) && (_x_pointeur < fenetre->w_x + fenetre->w_larg)
    && (_y_pointeur > y_trace     ) && (_y_pointeur < fenetre->w_y + fenetre->w_haut) )
  {
    int handle; /* On vrifie que l'on est bien la fentre active */
    wind_get(fenetre->handle, WF_TOP, &handle, &_rien_, &_rien_, &_rien_);

    if (handle == fenetre->handle)
    {
      vswr_mode(_PRG_station->handle, MD_INVERSE);
      vsl_type (_PRG_station->handle, 1);
      vsl_color(_PRG_station->handle, NOIR);
      points[0] = _x_pointeur; 	points[1] = y_trace;
      points[2] = _x_pointeur;	points[3] = fenetre->w_y + fenetre->w_haut;
      v_pline(_PRG_station->handle, 2, points);
      vswr_mode(_PRG_station->handle, MD_REMPLACE);
    }
  }
#endif

  /* On trace le graphe EXAFS extrait, en gris - en fin, car chelle diffrente */

  x_echelle = ( (double) fenetre->w_larg ) / (infos->exafs->x_max - infos->exafs->x_min);
  y_echelle = ( (double) haut_trace      ) / (infos->exafs->y_max - infos->exafs->y_min);

  param_dessin.graphe  = infos->exafs;
  param_dessin.couleur = GRIS_FONCE;
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace,
	     infos->exafs->x_min, infos->exafs->x_max, infos->exafs->y_min, infos->exafs->y_max,
	     x_echelle, y_echelle);

  /* On trace l'axe des X associ, en violet */

  vsl_color (_PRG_station->handle, VIOLET);
  points[0] = fenetre->w_x;
  points[1] = y_trace + y_echelle * infos->exafs->y_max;
  points[2] = fenetre->w_x + fenetre->w_larg;
  points[3] = points[1];
  v_pline(_PRG_station->handle, 2, points);
}

/* -------------- Messages reus par la fentre d'limination de 0 dja calcul ---------- */

boolean MSG_mu0(Fenetre *fenetre)
{
	int retour;
	Graphe *brut, *exafs, *mu_0;
	MU0_infos *infos;

	brut  = (Graphe *) fenetre->pointeur;
	infos = (MU0_infos *) fenetre->adresse;

	switch ( buffer[0] ) /* Nature du message */
	{
	case WM_TOPPED:
		FEN_premierplan(fenetre); /* Fentre au premier plan */

		points[1] = fenetre->w_y + OB_HEIGHT(arbres[OTE_MU0_CALC].adresse, ROOT);
		if ( (_x_pointeur > fenetre->w_x) && (_x_pointeur < fenetre->w_x + fenetre->w_larg)
		  && (_y_pointeur > points[1] )   && (_y_pointeur < fenetre->w_y + fenetre->w_haut) )
		{
			wind_update(BEG_UPDATE);
			graf_mouse(M_OFF, NULL);

			vswr_mode(_PRG_station->handle, MD_INVERSE);
			vsl_type (_PRG_station->handle, 1);
			vsl_color(_PRG_station->handle, NOIR);
			points[0] = _x_pointeur;
			points[2] = _x_pointeur;
			points[3] = fenetre->w_y + fenetre->w_haut;
			v_pline(_PRG_station->handle, 2, points);

			graf_mouse(M_ON,NULL);
			wind_update(END_UPDATE);
		}
		return VRAI;

	case WM_CLOSED: /* On veut fermer la fentre */
		retour = EXF_ERREUR(FIN_MODULE);
		if (retour == 3) /* On annule la fermeture */
		{
			return VRAI;
		}

		/* De toute faon, on ferme */

		exafs = infos->exafs;
		mu_0  = infos->mu_0;
		free(infos);
		ferme_fenetre(fenetre);

		if (retour == 2) /* On ferme sans soustraire */
		{
			GRF_efface(exafs, VRAI, VRAI);

			mu_0->drapeaux.protege = FAUX;  /* On n'utilise plus le mu0   */
			brut->drapeaux.protege = FAUX;

			GRF_aller(brut);                /* On va au spectre de dpart */

			return VRAI;
		}

		/* On veut extraire le spectre */

		sprintf(_intermediaire, "%s - %c0, "e_AIGU"tape 1%c", brut->nom, 230, 0);
		strncpy(exafs->nom, _intermediaire, LG_TITRE);

		exafs->drapeaux.sauve = FAUX;   /* Graphe non sauv            */
		exafs->drapeaux.protege = FAUX; /* Graphe non protg a priori */
		GRF_aller(exafs); /* On va  ce graphe */
		mu_0->drapeaux.protege = FAUX;  /* Graphe non protg a priori */

		brut->drapeaux.protege = FAUX;

		return VRAI;
	}

	return VRAI;
}

/* -------------- Clic sur la fentre d'limination de 0 dja calcul ---------------------- */

boolean CLC_mu0(Fenetre *fenetre, int bouton, int n_clic, int etat, int x_clic, int y_clic)
{
	MU0_infos *infos;
	Graphe *graphe;
	double x_echelle;

#if GEM
	int handle;

	wind_get(0, WF_TOP, &handle, &n_clic, &etat, &_rien_);
	if (handle != fenetre->handle)
	{
		wind_set(fenetre->handle, WF_TOP);
		return VRAI;
	}
#endif

	if ( (x_clic < fenetre->w_x) || (x_clic > fenetre->w_x + fenetre->w_larg)
	  || (y_clic < fenetre->w_y) || (y_clic > fenetre->w_y + fenetre->w_haut) )
	{
		return VRAI; /* Clic en-dehors de la zone de travail */
	}

	infos  = (MU0_infos *) fenetre->adresse;
	graphe = (Graphe *)    fenetre->pointeur;

	x_echelle = (graphe->x_max - graphe->x_min) / ( (double) (fenetre->w_larg) );
	if (x_echelle)
	{
		graf_mouse(S_ABEILLE, NULL); /* Souris en abeille  */
		if (bouton == 1) /* Clic du bouton gauche -> E0 */
		{
			infos->e_0 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
			met_fvaleur(OTE_MU0_CALC, MU0CALC_E0, infos->e_0);
		}

		MU0_extrait(graphe, infos); /* On rextrait l'EXAFS */
		graf_mouse(S_FLECHE, NULL);

		FEN_redessin (fenetre);     /* On redessine         */
	}

	return VRAI;
}

/* ~~~~~~~~~~~~~~~~~ On ralise l'limination proprement dite ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ----------------- Prparation de 0 avant extraction ------------------------------- */

void MU0_prepare(MU0_infos *infos)
{
	free(infos->b); free(infos->c); free(infos->d); /* On libre les anciens tableaux */
	SPL_tt_prep(infos->mu_0, &(infos->b), &(infos->c), &(infos->d));
}

/* ----------------- Extraction proprement dite ------------------------------------------ */

void MU0_extrait(Graphe *brut, MU0_infos *infos)
{
	register double *zone_x;
	unsigned long nbr_points = 0, index;

	zone_x = brut->x;
	for (index = 0 ; index < brut->nombre_point ; index++)
	{
		if ( *(zone_x) >= infos->e_0 ) /* Le point convient */
			nbr_points++;
		zone_x++;
	}

	infos->exafs->nombre_point = nbr_points; /* Nombre de points rel du spectre */

	if (infos->methode == MU0CALC_ECH_K) /* 0 donn en chelle des k (1/) */
	{
		register double *zone_x = brut->x, *zone_y = brut->y;
		unsigned long idx_2 = 0;

		/* On rcupre la partie intressante */

		for (index = 0 ; index < brut->nombre_point ; index++)
		{
			if ( *(zone_x) >= infos->e_0 ) /* Le point convient */
			{
				*(infos->exafs->x + idx_2) = *(zone_x); /* Valeur en X */
				*(infos->exafs->y + idx_2) = *(zone_y);
				idx_2++;
			}
			zone_x++; zone_y++;
		}

		/* On convertit en chelle des k */
		EXF_e_v_k(infos->exafs, infos->e_0);

		/* Et on soustrait _0 */

		zone_x = infos->exafs->x; zone_y = infos->exafs->y;
		for (index = 0 ; index < infos->exafs->nombre_point ; index++)
		{
			double y_mu0;  /* Valeur de mu_0 pour cette nergie */

			y_mu0 = SPL_tt_calc(infos->mu_0, *zone_x, infos->b, infos->c, infos->d);
			*zone_y -= y_mu0; /* On soustrait */

			if (y_mu0)
				*zone_y /= y_mu0;     /* On normalise */
			else
				EXF_ERREUR(DIV_ZERO);

			zone_x++; zone_y++;
		}	
	}
	else if (infos->methode == MU0CALC_ECH_EV) /* 0 donn en chelle des E (eV) */
	{
		register double *zone_x = brut->x, *zone_y = brut->y;
		unsigned long idx_2 = 0;

		for (index = 0 ; index < brut->nombre_point ; index++)
		{
			if ( *(zone_x) >= infos->e_0 ) /* Le point convient */
			{
				double y_mu0; /* Valeur de mu_0 pour cette nergie */

				*(infos->exafs->x + idx_2) = *(zone_x); /* Valeur en X */
				y_mu0 = SPL_tt_calc(infos->mu_0, *zone_x, infos->b, infos->c, infos->d);
				*(infos->exafs->y + idx_2) = *(zone_y) - y_mu0;

				if (y_mu0)
					*(infos->exafs->y + idx_2) /= y_mu0;
				else
					EXF_ERREUR(DIV_ZERO);

				idx_2++;
			}
			zone_x++; zone_y++;
		}

		/* Reste  convertir en chelle des k le spectre EXAFS */

		EXF_e_v_k(infos->exafs, infos->e_0);
	}

	/* On multiplie par k^n le rsultat */

	EXF_k_p_n(infos->exafs, infos->degre);

	GRF_bornes(infos->exafs, NE_PAS_FAIRE, DESORDONNE); /* Calcul des bornes */
}

/* ----------------- Soustration de 0, deuxime tape -------------------------------- *
 *******************************************************************************************/

/* ~~~~~~~~~~~~~~~~~ Gestion de l'interface ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ----------------- Ouverture de la fentre lie  cette opration -----------------------\
 |   Ne renvoie rien
 |
 | brut : le graphe qui va subir la soustraction
 | mu_0 : le polynme soustrait utilis  l'tape prcdente (NULL : on en cre un)        |
 \---------------------------------------------------------------------------------------- */

void MU2_ouvre(Graphe *brut, Graphe *mu_0)
{
	Fenetre *fenetre;
	MU2_infos *infos = NULL;
	unsigned long index;
	register double *exafs_x, *brut_x, *mu0_x;
	char titre[100];

	if (brut == NULL) return; /* Vraiment rien  faire */

	if ( arbres[ELIM_MUO_DEUX].ouvert == VRAI ) /* Arbre dj ouvert */
	{
		Graphe *vieux; /* Ancien spectre travaill dans ce module */
		int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

		fenetre = ARB_fenetre( &(arbres[ELIM_MUO_DEUX]) );
		vieux = (Graphe *) fenetre->pointeur;
		if ( (reponse == 1) || (vieux == brut) ) /* On active seulement */
		{
			FEN_premierplan(fenetre); /* Fentre au premier plan */
			return; /* Rien d'autre  faire */
		}
		else if (reponse == 2) /* On rappelle, pour le nouveau graphe */
		{
			if ( brut->drapeaux.protege )
			{
				EXF_ERREUR(GRF_PROTEGE);
				return;
			}

			infos = (MU2_infos *) fenetre->adresse;
			vieux->drapeaux.protege = FAUX; /* On a fini de travailler avec */
			GRF_efface(infos->exafs, VRAI, VRAI);   /* On efface le graphe associ  */
			GRF_efface(infos->mu_0 , VRAI, VRAI);

			FEN_premierplan(fenetre); /* Fentre au premier plan */
		}
		else /* On annule l'appel */
			return;
	}
	else /* Arbre non ouvert : on l'ouvre */
	{
		if ( brut->drapeaux.protege )
		{
			EXF_ERREUR(GRF_PROTEGE);
			return;
		}

		fenetre = creer_mixte_fenetre(NAME|CLOSER|FULLER|INFO|SIZER|MOVER, ELIM_MUO_DEUX, HAUT,
		                        RDS_mu02, ( boolean (*)() ) NULL, ( boolean (*)() ) NULL, 
	  	                      MSG_mu02, ( boolean (*)() ) NULL, ( boolean (*)() ) NULL);
		if (fenetre == NULL) return; /* Rien  faire... */
	}

	if (infos == NULL) /* Aucune info n'existe dj */
	{
		infos = (MU2_infos *) malloc( (size_t) sizeof(MU2_infos) );
		if (infos == NULL)
		{
			ferme_fenetre(fenetre);
			ERREUR(MANQUE_MEMOIRE);
			return;
		}
	}
	fenetre->pointeur = brut;  /* Graphe trait              */
	fenetre->adresse  = infos; /* Infos d'extraction de mu_0 */

	sprintf(titre, "%s - "MU"0, 2e "e_AIGU"tape", brut->nom);
	infos->exafs = GRF_creer(brut->nombre_point, GR_EXAFS2, GR_NORMAL, titre, "Nombre d'onde", "khi(k)", UNIT_1_ANG, UNIT_ARB);
	if (infos->exafs == NULL)
	{
		free(infos);
		ferme_fenetre(fenetre);
		return;
	}

	if (mu_0) /* On a envoy un polynme */
	{
		if ( mu_0->nombre_point != brut->nombre_point )
		{
			EXF_ERREUR(NBRPTS_INCORRECT);
			mu_0 = NULL;
		}
	}

	infos->mu_0_ref = mu_0; /* Le graphe mu_0 de rfrence */

	sprintf(titre, "0 t  %s, 2e tape", brut->nom);
	infos->mu_0 = GRF_creer(brut->nombre_point, GR_MU0, GR_NORMAL, titre, "Nombre d'onde", MU"0", UNIT_1_ANG, UNIT_ARB);
	if (infos->mu_0 == NULL)
	{
		GRF_efface(infos->exafs, VRAI, VRAI);
		free(infos);
		ferme_fenetre(fenetre);
		return;
	}

	brut->drapeaux.protege = VRAI; /* Calcul en cours sur ce spectre -> rien de faisable */
	if ( OBJ_choisit(ELIM_MUO_DEUX, ELIM2_RAPIDE , QUESTION) == VRAI )
		infos->methode = ELIM2_RAPIDE;
	if ( OBJ_choisit(ELIM_MUO_DEUX, ELIM2_PRECISE, QUESTION) == VRAI )
		infos->methode = ELIM2_PRECISE;

	/* On prpare les spectres pour gagner du temps */

	mu0_x = infos->mu_0->x; exafs_x = infos->exafs->x; brut_x = brut->x;
	for (index = 0 ; index < brut->nombre_point ; index++)
	{
		*mu0_x   = *brut_x;
		*exafs_x = *brut_x;
		mu0_x++; exafs_x++; brut_x++;
	}
	infos->mu_0->x_min  = brut->x_min; infos->mu_0->x_max  = brut->x_max;
	infos->exafs->x_min = brut->x_min; infos->exafs->x_max = brut->x_max;

	/* On calcule la fonction  soustraire */

	infos->coefs[0] = 0; /* On met  zro les coefficients */
	infos->coefs[1] = 0; infos->coefs[2] = 0; infos->coefs[3] = 0; infos->coefs[4] = 0;
	infos->coefs[5] = 0; infos->coefs[6] = 0; infos->coefs[7] = 0; infos->coefs[8] = 0;
	infos->coefs[9] = 0; infos->coefs[10] = 0;
	infos->degre       = prend_valeur(ELIM_MUO_DEUX, ELMU2_VALEUR);

	infos->exafs->drapeaux.protege = VRAI; /* On protge le graphe de toute btise */
	infos->mu_0->drapeaux.protege  = VRAI; /* On protge le graphe de toute btise */

	graf_mouse(S_ABEILLE, NULL); /* Souris en abeille  */
	MU2_calc(brut, infos);       /* Calcul du polynme */
	MU2_extrait(brut, infos);    /* Extraction         */
	graf_mouse(S_FLECHE, NULL);  /* Souris normale     */

	fenetre->drapeaux.redessin = VRAI;
	fenetre->msg_masque.fermeture    = VRAI; /* Messages traits par MSG_mu01 */
}

/* ------------- Redessin de la fentre d'limination de mu_0, deuxime tape -------------\
 |   Le clipping est dj install correctement lors de l'appel                            |
 |
 | fenetre    : structure associe  la fentre considre
 | x, y       : coordonnes de la zone de redessin
 | larg, haut : dimensions de la zone de redessin
 \---------------------------------------------------------------------------------------- */

void RDS_mu02(Fenetre *fenetre, int x, int y, int larg, int haut)
{
	Graphe *graphe;
	MU2_infos *infos;
	DES_graphe param_dessin;
	double x_echelle, y_echelle;
	int y_trace, haut_trace;

	vsl_type (_PRG_station->handle, 1);
	vswr_mode(_PRG_station->handle, MD_REMPLACE);
	vsf_color(_PRG_station->handle, BLANC); /* couleur de remplissage: blanc */

	points[0] = x;            points[1] = y;
	points[2] = x + larg - 1; points[3] = y + haut + 1;
	v_bar(_PRG_station->handle, points); /* On met le fond au blanc */

	y_trace    = fenetre->w_y    + OB_HEIGHT(arbres[ELIM_MUO_DEUX].adresse, ROOT);
	haut_trace = fenetre->w_haut - OB_HEIGHT(arbres[ELIM_MUO_DEUX].adresse, ROOT);

	graphe = (Graphe *)    fenetre->pointeur;
	infos  = (MU2_infos *) fenetre->adresse;
	x_echelle = ( (double) fenetre->w_larg ) / (graphe->x_max - graphe->x_min);
	y_echelle = ( (double) haut_trace )      / (graphe->y_max - graphe->y_min);

	/* On trace la courbe exprimentale - en noir */

	param_dessin.graphe  = graphe;
	param_dessin.couleur = NOIR;
	param_dessin.style   = 1;
	TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	           graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);

	/* On trace la courbe  soustraire - en bleu */	

	vsl_color(_PRG_station->handle, BLEU);
	vsl_type (_PRG_station->handle, 1);
	TRC_poly(graphe->nombre_point, infos->degre, infos->coefs, graphe->x,
	         x_echelle, y_echelle, fenetre->w_x, y_trace,
	         graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max);

	/* On trace le graphe EXAFS extrait, en bordeaux - en fin, car chelle diffrente */

	y_echelle = ( (double) haut_trace      ) / (infos->exafs->y_max - infos->exafs->y_min);

	param_dessin.graphe  = infos->exafs;
	param_dessin.couleur = GRIS_FONCE;
	TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace,
	           infos->exafs->x_min, infos->exafs->x_max, infos->exafs->y_min, infos->exafs->y_max,
	           x_echelle, y_echelle);

	/* On trace l'axe des X associ, en violet */

	vsl_color (_PRG_station->handle, VIOLET);
	points[0] = fenetre->w_x;
	points[1] = y_trace + y_echelle * infos->exafs->y_max;
	points[2] = fenetre->w_x + fenetre->w_larg;
	points[3] = points[1];
	v_pline(_PRG_station->handle, 2, points);
}

/* -------------- Messages reus par la fentre d'limination de mu_0, 2e tape ---------- */

boolean MSG_mu02(Fenetre *fenetre)
{
	int retour;
	Graphe *brut, *exafs, *mu_0, *mu_ref;
	MU2_infos *infos;

	brut  = (Graphe *)    fenetre->pointeur;
	infos = (MU2_infos *) fenetre->adresse;

	switch ( buffer[0] ) /* Nature du message */
	{
	case WM_CLOSED: /* On veut fermer la fentre */
		retour = EXF_ERREUR(FIN_MODULE);
		if (retour == 3) /* On annule la fermeture */
		{
			return VRAI;
		}

		/* De toute faon, on ferme */

		exafs  = infos->exafs;
		mu_0   = infos->mu_0;
		mu_ref = infos->mu_0_ref;
		ferme_fenetre(fenetre);
		if (mu_ref) /* Il y a un graphe mu_0 de rfrence */
			mu_ref->drapeaux.protege = FAUX;
		brut->drapeaux.protege   = FAUX;

		if (retour == 2) /* On ferme sans soustraire */
		{
			GRF_efface(exafs, VRAI, VRAI);
			GRF_efface(mu_0 , VRAI, VRAI);
			GRF_aller(brut);                /* On va au spectre de dpart */
			free(infos);
			return VRAI;
		}

		/* On veut extraire le spectre */

		/* On installe les valeurs dans l'arbre de paramtrage de l'exafs */

		met_valeur(EXAFS_GENERAL, EXF_DEGRE_MU0_II, infos->degre);
		OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_RAPID, (infos->methode == ELIM2_RAPIDE) );
		OBJ_choisit(EXAFS_GENERAL, EXF_MU0_II_LENT,  (infos->methode == ELIM2_PRECISE) );

		exafs->drapeaux.protege  = FAUX;
		mu_0->drapeaux.protege   = FAUX;
		exafs->drapeaux.sauve = FAUX;   /* Graphe non sauv            */
		mu_0->drapeaux.sauve  = FAUX;   /* Graphe non sauv            */
		GRF_aller(exafs); /* On va  ce graphe */

		if (Histoire)
		{
			int i;

			fprintf(Histoire, "\n--- "E_AIGU"limination de l'absorption atomique ---\n");
			fprintf(Histoire, "    (Correction de l'extraction : 2e "e_AIGU" de l'extraction\n");
			fprintf(Histoire, "Spectre trait"e_AIGU"   : %s\n", brut->nom);
			fprintf(Histoire, "Signal extrait   : %s\n", exafs->nom);
			fprintf(Histoire, "M"e_AIGU"thode          : soustraction d'un polyn"o_CIRC"me\n");
			fprintf(Histoire, "                   obtenu par lissage du spectre complet\n");
			fprintf(Histoire, "                   (seconde "e_AIGU"tape du logiciel EDA)\n");
			fprintf(Histoire, "Polyn"o_CIRC"me         : ");
			if ( infos->methode == ELIM2_RAPIDE )
				fprintf(Histoire, "meilleur polyn"o_CIRC"me au sens des moindres carr"e_AIGU"s\n");
			else if ( infos->methode == ELIM2_PRECISE )
				fprintf(Histoire, "obtenu par lissage [algorithme d'EDA]\n");
			fprintf(Histoire, "             degr"e_AIGU" : %d (A + B * X + ...)\n", infos->degre);
			for (i = 0 ; i <= infos->degre ; i++)
			{
				fprintf(Histoire, "           %c = ", 'A' + i);
				if ( infos->coefs[i] < .0001 )
					fprintf(Histoire, "%e\n", infos->coefs[i]);
				else
					fprintf(Histoire, "%f\n", infos->coefs[i]);
			}
			fprintf(Histoire, "Absorption atomique compl"e_GRAVE"te : %s\n", mu_0->nom);
		}

		free(infos);

		if (TT_exafs) /* On veut faire le traitement de tout l'EXAFS d'un coup */
		{
			int reponse;

			mu0_soustrait = mu_ref; /* Pour compatibilit */

			reponse = EXF_ERREUR(OTE_ABERRANCE); /* Y a-t-il des aberrances ? */
			if (reponse == 1) /* Pas d'aberrance */
				MU3_ouvre(exafs, mu_0);
			else
				aberrance(exafs); /* On limine des points aberrants */
		}

		return VRAI;
	}

	return VRAI;
}

/* ~~~~~~~~~~~~~~~~~ On ralise l'limination proprement dite ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ----------------- Calcul du polynome servant  modliser au mieux mu_0 -----------------\
 |  ne renvoie rien
 |
 | brut  : le spectre brut dont on veut extraire l'EXAFS
 | infos : les informations ncessaires  la modlisation de mu_0
 \---------------------------------------------------------------------------------------- */

void MU2_calc(Graphe *brut, MU2_infos *infos)
{
	if ( infos->methode == ELIM2_RAPIDE )    /* Mthode rapide     */
	{
		moindre2(brut, infos->degre, 0, NULL, infos->coefs);
	}
	else if ( infos->methode == ELIM2_PRECISE ) /* Mthode plus lente */
	{
		double bornes[2];
		bornes[0] = brut->x_min; bornes[1] = brut->x_max;
	
		moindrej(brut, infos->degre, 1, bornes, infos->coefs);
	}
}

/* ----------------- Extraction proprement dite du spectre E.X.A.F.S. -------------------- */

void MU2_extrait(Graphe *brut, MU2_infos *infos)
{
	register double *brut_x, *brut_y, *exafs_y, *mu0_y, *mu_ref_y;
	unsigned long index;

	brut_x  = brut->x; brut_y = brut->y;
	exafs_y = infos->exafs->y; mu0_y = infos->mu_0->y;
	if (infos->mu_0_ref)
		mu_ref_y = infos->mu_0_ref->y;
	
	for (index = 0 ; index < brut->nombre_point ; index++)
	{
		double x, y, mu_0 = 0;
		int i;

		x = *(brut_x);
		y = infos->coefs[0];
		for (i = 1 ; i <= infos->degre ; i++)
		{
			y += x * infos->coefs[i];
			x *= *(brut_x);
		}
		if (infos->mu_0_ref) /* Polynme de rfrence */
		{
			mu_0 = *mu_ref_y;
			mu_ref_y++;
		}
		*mu0_y   = y + mu_0;
		*exafs_y = *brut_y - y;
		brut_x++; brut_y++; exafs_y++; mu0_y++;
	}

	/* Recherche des bornes */

	GRF_bornes(infos->exafs, ORDONNE, DESORDONNE); /* Le graphe est normalement tri */ 
	GRF_bornes(infos->mu_0 , ORDONNE, DESORDONNE); /* Le graphe est normalement tri */
}

/* ----------------- Quelques petites routines au passage -------------------------------- *
 *******************************************************************************************/

/* ~~~~~~~~~~~~~~~~~ Multiplier le spectre par k^n, quel que soit n de -9  10 ~~~~~~~~~~~ */

/* ----------------- Interface GEM pour cette opration ---------------------------------- */

Graphe *magnifier(Graphe *source)
{
	int exposant;    /* Valeur de n  utiliser */
	unsigned long index;
	int retour;
	char chaine[60];
	register double *zone_x,*zone_y;
	Graphe *destination;

	if (ext_formdo(DEMANDE_N) == MAG_ANNULER)
		return source; /* On ne veut pas, finalement */

	if ( OBJ_choisit(DEMANDE_N, MAG_NV_GRAPHE, QUESTION) == VRAI ) /* On veut un nouveau graphe */
	{
		destination = creer_graphe(source->nombre_point, source->type);
		if (destination == NULL) return source; /* Pas pu crer le graphe */
	}
	else
	{
		destination = source;
	}

	exposant = prend_valeur(DEMANDE_N, MAG_VALEUR);
	if (exposant == 0) return destination; /* Multiplication par 1 */

	zone_x = destination->x;
	zone_y = destination->y;
	for (index = 0 ; index < source->nombre_point ; index++)
	{
		*(zone_x) = *(source->x + index);
		if ( (*zone_x == 0) && (exposant < 0) )
		{
			EXF_ERREUR(DIV_ZERO);
			*zone_y = *(source->y + index); /* On ne peut rien faire de mieux... */
		}
		else
			*(zone_y) = *(source->y + index) * pow( *(source->x + index) , exposant );

		zone_x++; zone_y++;
	}
	
	destination->x_min = source->x_min;        /* Les nouvelles bornes */
	destination->x_max = source->x_max;
	destination->y_min = *(destination->y);
	destination->y_max = *(destination->y);
	zone_y = destination->y + 1;
	for (index = 1 ; index < destination->nombre_point ; index++)
	{
		if (*(zone_y) > destination->y_max) destination->y_max = *(zone_y);
		if (*(zone_y) < destination->y_min) destination->y_min = *(zone_y);
		zone_y++;
	}

	retour = 0; /* Premier caractre du titre  utiliser */
	if ( (source->nom)[0] == 'k' ) /* On suppose qu'il y a dj eu une multiplication */
	{
		if ( ((source->nom)[1] == ' ') && ((source->nom)[2] == '*') )
		{
			exposant++;
			retour = 4; /* Premier caractre aprs */
		}
		else if ((source->nom)[1] == '^')
		{
			retour = 2;
			while ( (source->nom)[retour] != ' ' ) /* On lit la valeur de l'exposant */
			{
				chaine[retour-2] = (source->nom)[retour];
				retour++;
			}
			chaine[retour-1] = 0;
			exposant += atoi(chaine);
			retour += 3;
		}
	}

	if (exposant == 1)
		sprintf(chaine,"k * %s%c",source->nom + retour,0);
	else if (exposant == 0) /* Seule chance : il y avait un k^... avant */
		strcpy(chaine,source->nom + retour);
	else
		sprintf(chaine,"k^%d * %s%c",exposant,source->nom + retour,0);

	if (strlen(chaine) > LG_TITRE)
		chaine[49] = 0;
	strcpy(destination->nom,chaine);

	GRF_aller(destination);

	return destination; /* Le graphe aprs magnification */
}

/* -------------- Ralisation de l'opration elle-mme --------------------------------\
 |   Renvoie 0 si tout va bien, un numro d'erreur sinon                               |
 |                                                                                     |
 | spectre : le spectre sur lequel raliser l'opration                                |
 | degre   : le degre  utiliser (on aura Y ---> Y * X^degre)                          |
 \------------------------------------------------------------------------------------ */

int EXF_k_p_n(Graphe *spectre, int degre)
{
	unsigned long index;
	int sortie = 0;

	if (spectre == NULL) return E_V_K_PAS_GRAPHE;
	if (degre   == 0)    return 0; /* Rien  faire si on multiplie par X^0 = 1... */

	if (degre > 0) /* Aucune vrification  faire */
	{
		register double *zone_x = spectre->x, *zone_y = spectre->y;

		for (index = 0 ; index < spectre->nombre_point ; index++, zone_x++, zone_y++)
		{
			double x = pow( *zone_x , degre );
			*zone_y *= x;
		}
	}
	else
	{
	  register double *zone_x = spectre->x, *zone_y = spectre->y;

	  for (index = 0 ; index < spectre->nombre_point ; index++, zone_x++, zone_y++)
	  {
	    if (*zone_x == 0) /* On diviserait par zro */
	    {
	      sortie = DIV_ZERO;
	      *zone_y = 0; /* Pourquoi pas, tant que l'on y est... */
	    }
	    else
	    {
	      double x = pow( *zone_x, -degre );
	      *zone_y /= x;
	    }
	  }
	}

	GRF_bornes(spectre, NE_PAS_FAIRE, DESORDONNE); /* Bornes en Y ont chang... pas en X */
	return sortie;
}

/* ~~~~~~~~~~~~~~~~ Conversions k <-> E ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ---------------- Interface GEM pour l'appel de ces fonctions -------------------------------- */

Graphe *convert(Graphe *graphe)
{
	int retour;
	Graphe *dest;
	double e_0;   /* nergie de seuil             */

	if (graphe == NULL) return NULL; /* Rien  faire */

	dest = graphe; /* Par dfaut: graphe converti = graphe de dpart */

	retour = ext_formdo(CONVERT_K_E);
	if (retour == CONV_ANNULE) return NULL; /* On annule : anomalie... */

	e_0  = prend_fvaleur(CONVERT_K_E, VAL_CONVERT);
	if ( OBJ_choisit(CONVERT_K_E, CONV_NV_GRAPHE, QUESTION) & SELECTED )
	{
		dest = copier_graphe(graphe);
		if (dest == NULL) return graphe; /* Conversion n'a pas pu avoir lieu => retour au graphe de dpart */
	}
	else
		dest = graphe;

	if ( OBJ_choisit(CONVERT_K_E, CONV_EVK, QUESTION) & SELECTED ) /* Conversion de E vers k */
	{
		int erreur = EXF_e_v_k(dest, e_0);
		if (erreur) /* Erreur lors de la conversion */
		{
			form_alert(1,"[2][ Des rsultats inattendus ont t obtenus | lors de ce calcul : prendre le | rsultat avec mfiance ][ Ok ]");
		}
	}
	else
	{
		int erreur = EXF_k_v_e(dest, e_0);
		if (erreur)
		{
			form_alert(1,"[2][ Des rsultats inattendus ont t obtenus | lors de ce calcul : prendre le | rsultat avec mfiance ][ Ok ]");
		}
	}

	return dest;
}

/* ----------------- Conversion de l'nergie en nombres d'onde (eV -> 1/) ----------------------\
 |   Renvoie 0 si pas d'erreur, -xxx en cas d'erreur
 |
 | spectre : le spectre dont on veut faire la conversion
 | e_0     : la valeur de e_0, toujours pour cette conversion
 \---------------------------------------------------------------------------------------------- */

int EXF_e_v_k(Graphe *spectre, double e_0)
{
	unsigned long index;
	register double *zone_x;
	double coefficient, x;
	int retour = 0; /* Pas d'erreur */

	if (spectre == NULL) return E_V_K_PAS_GRAPHE; /* Erreur : pas de graphe en entre */
	if (e_0 <= 0)        return E_V_K_E0_FAUX;    /* Erreur : E0 Absurde              */

	coefficient  = 2 * MASSE_ELECTRON / (H_BARRE * H_BARRE); /* Coefficient rel */
	coefficient *= EV_V_JOULE;              /* Conversion des nergies en Joules */ 

	zone_x = spectre->x;
	
	for (index = 0 ; index < spectre->nombre_point ; index++)
	{
		x = coefficient * ( *(zone_x) - e_0 );
		if (x < 0) /* Normalement, ne devrait pas arriver */
		{
			*(zone_x) = - sqrt(-x) / M_V_ANGSTROM;
			retour = E_V_K_VAL_NEGATIVE; /* On a du convertir des valeurs de E bizarres */
		}
		else
		{
			*(zone_x) =   sqrt(x) / M_V_ANGSTROM; /* Donne des m^-1, on convertit en ^-1 */
		}
		zone_x++;
	}

	/* On met les nouvelles bornes en X */
	
	x  = (spectre->x_min - e_0); /* nergie en Joules */
	x *= coefficient;
	if (x < 0)
		spectre->x_min = -sqrt(-x) / M_V_ANGSTROM;
	else
		spectre->x_min =  sqrt( x) / M_V_ANGSTROM;

	x  = (spectre->x_max - e_0); /* nergie en Joules */
	x *= coefficient;
	if (x < 0)
		spectre->x_max = -sqrt(-x) / M_V_ANGSTROM;
	else
		spectre->x_max =  sqrt( x) / M_V_ANGSTROM;

	GRF_legende(spectre, AXE_X, "Nombre d'onde", UNIT_1_ANG);

	return retour;
}

/* ----------------- Conversion des nombres d'onde en  l'nergie (1/ -> eV) --------------------\
 |   Renvoie 0 si pas d'erreur, -xxx en cas d'erreur
 |
 | spectre : le spectre dont on veut faire la conversion
 | e_0     : la valeur de e_0, toujours pour cette conversion
 \---------------------------------------------------------------------------------------------- */

int EXF_k_v_e(Graphe *spectre, double e_0)
{
  unsigned long index;
  register double *zone_x;
  double coefficient, x;
  int retour = 0; /* Pas d'erreur */

  if (spectre == NULL) return E_V_K_PAS_GRAPHE; /* Erreur : pas de graphe en entre */
  if (e_0 <= 0)        return E_V_K_E0_FAUX;    /* Erreur : E0 Absurde              */

  coefficient  = 2 * MASSE_ELECTRON / (H_BARRE * H_BARRE); /* Coefficient rel */
  coefficient *= EV_V_JOULE;              /* Conversion des nergies en Joules */ 

  zone_x = spectre->x;
	
  for (index = 0 ; index < spectre->nombre_point ; index++)
  {
    x = *zone_x * M_V_ANGSTROM; /* Conversion en m^-1 */
    x *= x / coefficient;
    if ( *zone_x < 0 )
    {
      *zone_x = e_0 - x;
      retour = E_V_K_VAL_NEGATIVE;
    }
    else
    {
      *zone_x = e_0 + x;
    }
    zone_x++;
  }
  
  /* On met les nouvelles bornes en X */
	
  x  = spectre->x_min * M_V_ANGSTROM;
  x *= x / coefficient;
  if (spectre->x_min < 0)
    spectre->x_min = e_0 - x;
  else
    spectre->x_min = e_0 + x;

  x  = spectre->x_max * M_V_ANGSTROM;
  x *= x / coefficient;
  if (spectre->x_max < 0)
    spectre->x_max = e_0 - x;
  else
    spectre->x_max = e_0 + x;

  GRF_legende(spectre, AXE_X, E_AIGU"nergie", UNIT_EV);

  return retour;
}

/* ----------------- Correction des nergies (en eV) en absisse ------------------------ */

void corr_energie(Graphe *spectre)
{
	double e0_th, e0_exp;
	register double *x;
	unsigned long i;

	if (spectre == NULL) return;

	e0_exp = cherche_e0(spectre, FAUX); /* On cherche le E0 exprimental */
	met_fvaleur(CORR_ENERGIE, SEUIL_MESURE , e0_exp);
	met_fvaleur(CORR_ENERGIE, SEUIL_ATTENDU, e0_exp); /* On suppose le mme par dfaut */

	if (ext_formdo(CORR_ENERGIE) == COR_NRJ_ANNULE) return; /* On annule le calcul */

	e0_exp = prend_fvaleur(CORR_ENERGIE, SEUIL_MESURE ); /* E0 exprimental */
	e0_th  = prend_fvaleur(CORR_ENERGIE, SEUIL_ATTENDU); /* E0 thorique    */

	if ( OBJ_choisit(CORR_ENERGIE, COR_NRJ_DECALE, QUESTION) == VRAI ) /* Simple dcalage */
	{
		double delta_x = e0_th - e0_exp;

		x = spectre->x;
		for (i = 0 ; i < spectre->nombre_point ; i++)
		{
			*x += delta_x;
			x++;
		}
		spectre->x_min += delta_x;
		spectre->x_max += delta_x;
	}
	else                                                /* On utilise la loi de Bragg */
	{
	  const double h = 6.62617811e-34; /* Constante de Planck,   en J.s */
	  const double c = 3.0e18;         /* Vitesse de la lumire, en /s */
	  double e_v_angle, dist_plans;
	  double d_theta;
	  double theta_exp;

	  dist_plans = prend_fvaleur(CORR_ENERGIE, COR_NRJ_DIST); /* Distance interrticulaire, en  */

	  e_v_angle  = h * c / (2 * dist_plans); /* Conversion theta <-> nergie */
	  e_v_angle /= 1.602189e-19; /* Conversion eV <-> J */

	  d_theta = asin( e_v_angle / e0_th) - asin(e_v_angle / e0_exp); /* Correction */
	  
	  x = spectre->x;
	  for (i = 0 ; i < spectre->nombre_point ; i++)
	  {
	    theta_exp = asin(e_v_angle / *x);
	    *x = e_v_angle / sin(d_theta + theta_exp);
	    
	    x++;
	  }

	  GRF_bornes(spectre, DESORDONNE, DESORDONNE); /* On recalcule les bornes */
	}
}
