/* LASE - Traitement de l'E.X.A.F.S. sur Atari & X-Window
   Tout ce qui concerne l'EXAFS sens strict (extraction des oscillations)

   (c) 1996, 1998 - Emmanuel CURIS ; cration du fichier : 27 mai 1998

	 Premire tape de la soustraction de mu_0 : modlisation par un polynme
	 Rcupr de ../EXAFS.C et ../EXF_GEM.C (pour GER_...)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "ERREURS.H"
#include "EXTRAIT.H"
#include "SCIENCE.H"
#include "LISSAGE.H"

/* ***************** Fonctions locales de ce module ************************************** */

static void    RDS_mu01(Fenetre *fen, int x, int y, int larg, int haut);
static boolean MSG_mu01(Fenetre *fen);
static boolean CLC_mu01(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

/* *-*-*-*-*-*-*-*-* Calcul proprement dit *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* */

/* ----------------- Calcul du polynome servant  modliser au mieux mu_0 -----------------\
 |  ne renvoie rien
 |
 | brut  : le spectre brut dont on veut extraire l'EXAFS
 | infos : les informations ncessaires  la modlisation de mu_0
 \---------------------------------------------------------------------------------------- */

void MU1_calc(Graphe *brut, MU1_infos *infos)
{
  double erreur;    /* Erreur entre le polynme et la courbe */
  double bornes[2]; /* Intervalle d'approximation du polynme  ter */

  bornes[0] = infos->e_2; 
  bornes[1] = brut->x_max;

  if ( infos->methode == ELIM_MU0_RAPIDE )    /* Mthode rapide     */
  {
    erreur = moindre2(brut, infos->degre, 1, bornes, infos->coefs );
    met_fvaleur(ELIM_MU0_1, MU01_ERREUR, erreur); /* Affichage de l'erreur */
  }
  else if ( infos->methode == ELIM_MU0_LENT ) /* Mthode plus lente */
  {
    erreur = moindrej(brut, infos->degre, 1, bornes, infos->coefs);
    met_fvaleur(ELIM_MU0_1, MU01_ERREUR, erreur); /* Affichage de l'erreur */
  }
}

/* ----------------- Extraction proprement dite du spectre E.X.A.F.S. -------------------- */

void MU1_extrait(Graphe *brut, MU1_infos *infos)
{
  register double *zone_x;
  unsigned long nbr_points = 0, index, index_2;

  zone_x = brut->x;
  for (index = 0 ; index < brut->nombre_point ; index++)
  {
    if ( *(zone_x) >= infos->e_0 ) /* Le point convient */
      nbr_points++;
    zone_x++;
  }

  infos->exafs->nombre_point = nbr_points; /* Nombre de points rel du spectre */
  infos->mu_0->nombre_point  = nbr_points;
	
  zone_x  = brut->x;
  index_2 = 0;
  for (index = 0 ; index < brut->nombre_point ; index++)
  {
    if ( *(zone_x) >= infos->e_0 ) /* Le point convient bien => on le traite */
    {
      double x, y;
      int i;

      x = *(zone_x);
      *(infos->exafs->x + index_2) = x;         /* Valeur en X */
      *(infos->mu_0->x + index_2) = x;
      y = infos->coefs[0];
      for (i = 1 ; i <= infos->degre ; i++)
      {
	y += x * infos->coefs[i];
	x *= *(zone_x);
      }
      (infos->mu_0->y) [index_2] = y;
      (infos->exafs->y)[index_2] = *(brut->y + index) - y;
      index_2++;
    }
    zone_x++;
  }

  /* On convertit en chelle des k (calcule les bornes en X en mme temps) */
	
  EXF_e_v_k(infos->exafs, infos->e_0);
  EXF_e_v_k(infos->mu_0 , infos->e_0);

  /* Multiplication par k^n */

  EXF_k_p_n(infos->exafs, infos->deg_affiche); /* Multiplication par k^n du signal extrait */
  EXF_k_p_n(infos->mu_0,  infos->deg_affiche); /* et du polynome soustrait                 */

  /* Recherche des bornes */

  GRF_bornes(infos->exafs, ORDONNE, DESORDONNE); /* Le graphe est normalement tri en X */ 
  GRF_bornes(infos->mu_0 , ORDONNE, DESORDONNE); /* Le graphe est normalement tri en X */ 
}

/* *-*-*-*-*-*-*-*-* Tout ce qui concerne l'interface *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- */

/* ~~~~~~~~~~~~~~~~~ Ouverture de la fentre associe  cette tape ~~~~~~~~~~~~~~~~~~~~~~ */

void MU1_ouvre(Graphe *brut)
{
  Fenetre *fenetre;
  MU1_infos *infos = NULL;
  char titre[100];

  if (brut == NULL) return; /* Vraiment rien  faire */
  
  if ( arbres[ELIM_MU0_1].ouvert == VRAI ) /* Arbre dj ouvert */
  {
    Graphe *vieux; /* Ancien spectre travaill dans ce module */
    int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

    fenetre = ARB_fenetre( &(arbres[ELIM_MU0_1]) );
    vieux = (Graphe *) fenetre->pointeur;
    if ( (reponse == 1) || (vieux == brut) ) /* On active seulement */
    {
      FEN_premierplan(fenetre); /* Fentre au premier plan */
      return; /* Rien d'autre  faire */
    }
    else if (reponse == 2) /* On rappelle, pour le nouveau graphe */
    {
      if ( brut->drapeaux.protege )
      {
	EXF_ERREUR(GRF_PROTEGE);
	return;
      }

      infos = (MU1_infos *) fenetre->adresse;
      vieux->drapeaux.protege = FAUX; /* On a fini de travailler avec */

      GRF_efface(infos->exafs, VRAI, VRAI); /* On efface le graphe associ */
      GRF_efface(infos->mu_0 , VRAI, VRAI); /* On efface le graphe associ */
      
      FEN_premierplan(fenetre); /* Fentre au premier plan */
    }
    else /* On annule l'appel */
      return;
  }
  else /* Arbre non ouvert : on l'ouvre */
  {
    if ( brut->drapeaux.protege )
    {
      EXF_ERREUR(GRF_PROTEGE);
      return;
    }

    fenetre = creer_mixte_fenetre(NAME|CLOSER|FULLER|INFO|SIZER|MOVER, ELIM_MU0_1, HAUT,
				  RDS_mu01, ( boolean (*)(Fenetre *, unsigned int, int) ) NULL, CLC_mu01, 
				  MSG_mu01, RCT_pointeur, ( boolean (*)(Fenetre *) ) NULL);
    if (fenetre == NULL) return; /* Rien  faire... */

#ifdef _FRANCAIS_
    FEN_titre(fenetre, "Soustraction de l'absorption atomique, premi"e_GRAVE"re "e_AIGU"tape");
#endif
#ifdef _ENGLISH_
    FEN_titre(fenetre, "Substraction of atomic absorption, first step");
#endif
  }

  if (infos == NULL) /* Aucune info n'existe dj */
  {
    infos = (MU1_infos *) malloc( (size_t) sizeof(MU1_infos) );
    if (infos == NULL)
    {
      ferme_fenetre(fenetre);
      ERREUR(MANQUE_MEMOIRE); /* Aprs sinon risque de choses "mauvaises" sous X */
      return;
    }
  }
  
  fenetre->pointeur = brut;  /* Graphe trait              */
  fenetre->adresse  = infos; /* Infos d'extraction de mu_0 */

#ifdef _FRANCAIS_
  sprintf(titre, "%s - "MU"0, 1re "e_AIGU"tape", brut->nom);
  infos->exafs = GRF_creer(brut->nombre_point, GR_EXAFS1, GR_NORMAL,
			   titre, "Nombre d'onde", "khi(k)", UNIT_1_ANG, UNIT_ARB);
#endif
#ifdef _ENGLISH_
  sprintf(titre, "%s - "MU"0, 1st step", brut->nom);
  infos->exafs = GRF_creer(brut->nombre_point, GR_EXAFS1, GR_NORMAL,
			   titre, "Wave number", "chi(k)", UNIT_1_ANG, UNIT_ARB);
#endif
  if (infos->exafs == NULL)
  {
    free(infos);
    ferme_fenetre(fenetre);
    return;
  }

  sprintf(titre, MU"0 "o_CIRC"t"e_AIGU" "a_GRAVE" %s, 1re "e_AIGU"tape", brut->nom);
  infos->mu_0 = GRF_creer(brut->nombre_point, GR_MU0, GR_NORMAL, titre, "Nombre d'onde", MU"0", UNIT_1_ANG, UNIT_ARB);
  if (infos->mu_0 == NULL)
  {
    GRF_efface(infos->exafs, VRAI, VRAI);
    free(infos);
    ferme_fenetre(fenetre);
    return;
  }

  brut->drapeaux.protege = VRAI; /* Calcul en cours sur ce spectre -> rien de faisable */
  if ( OBJ_choisit(ELIM_MU0_1, ELIM_MU0_RAPIDE, QUESTION) == VRAI )
    infos->methode = ELIM_MU0_RAPIDE;
  if ( OBJ_choisit(ELIM_MU0_1, ELIM_MU0_LENT  , QUESTION) == VRAI )
    infos->methode = ELIM_MU0_LENT;

  /* On cherche la valeur de E0 la plus plausible */
  infos->e_0 = cherche_e0 (brut, FAUX);                   /* E0 : maximum de la drive             */
  infos->e_2 = plus_proche(brut, infos->e_0 + 100, VRAI); /* E2 : on se place 100 eV aprs le seuil */
  /* Dans EDAEES : utilise la valeur en 2*x_min_avant_seuil - x_max */

  /* On calcule la fonction  soustraire */

  infos->coefs[0] = 0; /* On met  zro les coefficients */
  infos->coefs[1] = 0; infos->coefs[2] = 0; infos->coefs[3] = 0; infos->coefs[4] = 0;
  infos->coefs[5] = 0; infos->coefs[6] = 0; infos->coefs[7] = 0; infos->coefs[8] = 0;
  infos->coefs[9] = 0;
  infos->degre       = prend_valeur(ELIM_MU0_1, DEGRE_VALEUR);
  infos->deg_affiche = prend_valeur(ELIM_MU0_1, MU01_DEG_AFFICHE);

  infos->exafs->drapeaux.protege = VRAI; /* On protge le graphe de toute btise */
  infos->mu_0->drapeaux.protege = VRAI; /* On protge le graphe de toute btise */

  graf_mouse(S_ABEILLE, NULL); /* Souris en abeille  */
  MU1_calc(brut, infos);       /* Calcul du polynme */
  MU1_extrait(brut, infos);    /* Extraction         */
  graf_mouse(S_FLECHE, NULL);  /* Souris normale     */

  fenetre->drapeaux.redessin = VRAI;
  fenetre->drapeaux.pointeur = VRAI;
  fenetre->msg_masque.fermeture    = VRAI; /* Messages traits par MSG_mu01 */
  fenetre->msg_masque.taille       = VRAI;
  fenetre->msg_masque.premier_plan = VRAI;
  
  met_fvaleur(ELIM_MU0_1, VALEUR_E0, infos->e_0);
  met_fvaleur(ELIM_MU0_1, VALEUR_E2, infos->e_2);
}

/* ~~~~~~~~~~~~~ Gestion de la fentre d'limination de mu0, premire tape ~~~~~~~~~~~~~~ */

/* ------------- Redessin de la fentre d'limination de mu matrice -----------------------\
 |   Le clipping est dj install correctement lors de l'appel                            |
 |
 | fenetre    : structure associe  la fentre considre
 | x, y       : coordonnes de la zone de redessin
 | larg, haut : dimensions de la zone de redessin
 \---------------------------------------------------------------------------------------- */

static void RDS_mu01(Fenetre *fenetre, int x, int y, int larg, int haut)
{
  Graphe *graphe;
  MU1_infos *infos;
  DES_graphe param_dessin;
  double x_echelle, y_echelle;
  int y_trace, haut_trace;

  vsl_type (_PRG_station->handle, 1);
  vswr_mode(_PRG_station->handle, MD_REMPLACE);
  vsf_color(_PRG_station->handle, BLANC); /* couleur de remplissage: blanc */

  points[0] = x;            points[1] = y;
  points[2] = x + larg - 1; points[3] = y + haut + 1;
  v_bar(_PRG_station->handle, points); /* On met le fond au blanc */

  y_trace    = fenetre->w_y    + OB_HEIGHT(arbres[ELIM_MU0_1].adresse, ROOT);
  haut_trace = fenetre->w_haut - OB_HEIGHT(arbres[ELIM_MU0_1].adresse, ROOT);

  graphe = (Graphe *)    fenetre->pointeur;
  infos  = (MU1_infos *) fenetre->adresse;
  x_echelle = ( (double) fenetre->w_larg ) / (graphe->x_max - graphe->x_min);
  y_echelle = ( (double) haut_trace )      / (graphe->y_max - graphe->y_min);

  /* On trace la courbe exprimentale - en noir */

  param_dessin.graphe  = graphe;
  param_dessin.couleur = NOIR;
  param_dessin.style   = 1;
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	     graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);

  /* On trace la courbe  soustraire - en bleu */

  vsl_color(_PRG_station->handle, BLEU);
  vsl_type (_PRG_station->handle, 1);
  TRC_poly(graphe->nombre_point, infos->degre, infos->coefs, graphe->x,
	   x_echelle, y_echelle, fenetre->w_x, y_trace,
	   graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max);

  /* On met les barres */

  vsl_color(_PRG_station->handle, ROUGE); /* Barre rouge : E0 */
  points[0] = fenetre->w_x + x_echelle * (infos->e_0 - graphe->x_min);
  points[1] = y_trace;
  points[2] = points[0];
  points[3] = y_trace + haut_trace;
  v_pline(_PRG_station->handle,2,points);

  vsl_color(_PRG_station->handle, VERT); /* Barre verte : E2 */
  points[0] = fenetre->w_x + x_echelle * (infos->e_2 - graphe->x_min);
  points[1] = y_trace;
  points[2] = points[0];
  points[3] = y_trace + haut_trace;
  v_pline(_PRG_station->handle,2,points);

  /* On trace le graphe EXAFS extrait, en gris - en fin, car chelle diffrente */

  x_echelle = ( (double) fenetre->w_larg ) / (infos->exafs->x_max - infos->exafs->x_min);
  y_echelle = ( (double) haut_trace      ) / (infos->exafs->y_max - infos->exafs->y_min);

  param_dessin.graphe  = infos->exafs;
  param_dessin.couleur = GRIS_FONCE;
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace,
	     infos->exafs->x_min, infos->exafs->x_max, infos->exafs->y_min, infos->exafs->y_max,
	     x_echelle, y_echelle);

  /* On trace l'axe des X associ, en violet */

  vsl_color (_PRG_station->handle, VIOLET);
  points[0] = fenetre->w_x;
  points[1] = y_trace + y_echelle * infos->exafs->y_max;
  points[2] = fenetre->w_x + fenetre->w_larg;
  points[3] = points[1];
  v_pline(_PRG_station->handle, 2, points);
}

/* -------------- Messages reus par la fentre d'limination de mu_0, 1re tape ---------- */

static boolean MSG_mu01(Fenetre *fenetre)
{
  Graphe *brut, *exafs, *mu_0;
  MU1_infos *infos;

  brut = (Graphe *) fenetre->pointeur;
  infos = (MU1_infos *) fenetre->adresse;

  if ( buffer[0] == WM_CLOSED ) /* On veut fermer la fentre */
  {
    int retour;

    if ( (buffer[1] == application) && (buffer[7] != 0) ) /* On force la fermeture */
    {
#ifdef _DEVELOPPE_
      printf("On demande la fermeture FORCE de la fentre\n");
#endif
      retour = 2;
    }
    else
      retour = EXF_ERREUR(FIN_MODULE);

    if (retour == 3) /* On annule la fermeture */
    {
      return VRAI;
    }

    /* De toute faon, on ferme */

    exafs = infos->exafs;
    mu_0 = infos->mu_0;
    ferme_fenetre(fenetre);

    if (retour == 2) /* On ferme sans soustraire */
    {
      GRF_efface(exafs, VRAI, VRAI);
      GRF_efface(mu_0, VRAI, VRAI);

      brut->drapeaux.protege = FAUX;
      GRF_aller(brut);                /* On va au spectre de dpart */

      free(infos);
      return VRAI;
    }
    
    /* On stocke les informations dans l'arbre de "rptition" */

    met_fvaleur(EXAFS_GENERAL, EXF_VALEUR_E0  , infos->e_0);
    met_fvaleur(EXAFS_GENERAL, EXF_VALEUR_E2  , infos->e_2);
    met_valeur(EXAFS_GENERAL, EXV_DEGRE_MU0_I, infos->degre);
    met_valeur(EXAFS_GENERAL, EXF_VALEUR_K, infos->deg_affiche);
    if (infos->methode == ELIM_MU0_RAPIDE)
      OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_RAPIDE, FORCE_OUI);
    else if (infos->methode == ELIM_MU0_LENT)
      OBJ_choisit(EXAFS_GENERAL, EXF_MU0_I_LENT, FORCE_OUI);

    /* On veut extraire le spectre */

    exafs->drapeaux.sauve = FAUX;   /* Graphe non sauv            */
    exafs->drapeaux.protege = FAUX; /* Graphe non protg a priori */
    GRF_aller(exafs); /* On va  ce graphe */
    mu_0->drapeaux.sauve = FAUX;   /* Graphe non sauv            */
    mu_0->drapeaux.protege = FAUX; /* Graphe non protg a priori */

    brut->drapeaux.protege = FAUX;

    if (Histoire) /* On ralise donc l'historique */
    {
      int i;

      fprintf(Histoire, "\n--- "E_AIGU"limination de l'absorption atomique ---\n");
      fprintf(Histoire, "Spectre trait"e_AIGU" : %s\n", brut->nom);
      fprintf(Histoire, "Signal extrait : %s\n", exafs->nom);
      fprintf(Histoire, "                 (correspond  k^%d * "KHI"(k)\n", infos->deg_affiche);
      fprintf(Histoire, "M"e_AIGU"thode        : soustraction d'un polyn"o_CIRC"me\n");
      fprintf(Histoire, "                 obtenu par lissage d'un spectre complet au-del"a_GRAVE" de E2\n");
      fprintf(Histoire, "                 extrapol"e_AIGU" entre E0 et E2\n");
      fprintf(Histoire, "                 (premi"e_GRAVE"re "e_AIGU"tape du logiciel EDA)\n");
      fprintf(Histoire, "Polyn"o_CIRC"me       : ");
      if ( infos->methode == ELIM_MU0_RAPIDE )
	fprintf(Histoire, "meilleur polyn"o_CIRC"me au sens des moindres carr"e_AIGU"s\n");
      else if ( infos->methode == ELIM_MU0_LENT )
	fprintf(Histoire, "obtenu par lissage [algorithme d'EDA]\n");
      fprintf(Histoire, "           degr"e_AIGU" : %d (A + B * X + ...)\n", infos->degre);
      for (i = 0 ; i <= infos->degre ; i++)
	{
	  fprintf(Histoire, "           %c = ", 'A' + i);
	  if ( infos->coefs[i] < .0001 )
	    fprintf(Histoire, "%e\n", infos->coefs[i]);
	  else
	    fprintf(Histoire, "%f\n", infos->coefs[i]);
	}
      fprintf(Histoire, "Seuil       E0 = %f eV\n", infos->e_0);
      fprintf(Histoire, "E2             = %f eV\n", infos->e_2);
      fprintf(Histoire, "Polyn"o_CIRC"me utilis"e_AIGU" : %s\n", mu_0->nom);
    }

    free(infos);
    
    if (TT_exafs) /* On veut faire le traitement de tout l'EXAFS d'un coup */
      MU2_ouvre(exafs, mu_0); /* On passe  la deuxime tape */
    return VRAI;
  }

  return VRAI;
}

/* -------------- Clic sur la fentre d'limination de mu0, 1re tape ------------------------ */

static boolean CLC_mu01(Fenetre *fenetre, int bouton, int n_clic, int etat, int x_clic, int y_clic)
{
	MU1_infos *infos;
	Graphe *graphe;
	double x_echelle;

#if GEM
	int handle;

	wind_get(0, WF_TOP, &handle, &n_clic, &etat, &_rien_);
	if (handle != fenetre->handle)
	{
		wind_set(fenetre->handle, WF_TOP);
		return VRAI;
	}
#endif

	if ( (x_clic < fenetre->w_x) || (x_clic > fenetre->w_x + fenetre->w_larg)
	  || (y_clic < fenetre->w_y) || (y_clic > fenetre->w_y + fenetre->w_haut) )
	{
		return VRAI; /* Clic en-dehors de la zone de travail */
	}

	infos  = (MU1_infos *) fenetre->adresse;
	graphe = (Graphe *)    fenetre->pointeur;

	x_echelle = (graphe->x_max - graphe->x_min) / ( (double) (fenetre->w_larg) );
	if (x_echelle)
	{
		graf_mouse(S_ABEILLE, NULL); /* Souris en abeille  */
		if (bouton == 1) /* Clic du bouton gauche -> E0 */
		{
			infos->e_0 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
			met_fvaleur(ELIM_MU0_1, VALEUR_E0, infos->e_0);
		}
		else /* Bouton droit -> E2 */
		{
			infos->e_2 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
			met_fvaleur(ELIM_MU0_1, VALEUR_E2, infos->e_2);

			MU1_calc(graphe, infos); /* On recalcule le polynome */
		}

		MU1_extrait(graphe, infos); /* On rextrait l'EXAFS */
		graf_mouse(S_FLECHE, NULL);
		FEN_redessin (fenetre);     /* On redessine         */
	}

	return VRAI;
}



