/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1996 - 1998, Emmanuel CURIS
   Ce fichier : 2 octobre 1998

   Tout ce qui concerne l'extraction du signal EXAFS  partir du signal d'absorption brut

   Troisime tape de la soustraction de mu_0 (spline avec contrle)
   Rcupr de ../EXAFS.c et ../EXF_GEM.c
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "ERREURS.H"
#include "EXTRAIT.H"
#include "FOURIER.H" /* Pour TF_ouvre() dans traitement auto. */
#include "SCIENCE.H"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void    RDS_mu03(Fenetre *fenetre, int x, int y, int larg, int haut);
static boolean MSG_mu03(Fenetre *fenetre);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* ~~~~~~~~~~~~~~~~~ Calculs effectifs de la correction  soustraire ~~~~~~~~~~~~~~~~~~~~~ */

#define avec_p_p /* Si dfini, on utilise tableaux de type double **, sinon de type double *tableaux[10]
		    Sur Hades 40, compilateur = Turbo C 1.0 : mthode avec double ** un poil plus rapide

		    Sur alpha 433/au, compilateur = cc : indiscernable 
		 */

/* ----------------- Calcul de la courbe servant  modliser au mieux mu_0 ----------------\
 |  ne renvoie rien                                                                        |
 |  Le pas du graphe est suppos constant (<=> en-dehors du graphe, x = celui dans le graphe)
 |  Le graphe est suppos nul en-dehors des points dfinis                                 |
 |  Le graphe doit tre tri et sans doublon                                               |
 |                                                                                         |
 | brut  : le spectre brut dont on veut extraire l'EXAFS                                   |
 | infos : les informations ncessaires  la modlisation de mu_0                          |
 \---------------------------------------------------------------------------------------- */

void MU3_calc(Graphe *brut, MU3_infos *infos)
{
  register double *y_brut,*y_mu0;
  double d_x1, d_x2, y1, y2, p = 0.0, controle;
  unsigned long index, nbr_pts;
  boolean sortie = FAUX;

#ifdef _DEVELOPPE_
  unsigned long temps; /* Juste pour les oprations de chronomtrage */
#endif

#ifdef avec_p_p

  double **tableaux;
  tableaux = (double **) calloc(10, sizeof( double *) );
  if (tableaux == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return;
  }

#else

  double *tableaux[10];

#endif

#ifdef _DEVELOPPE_
  temps = clock();
#endif

  nbr_pts = brut->nombre_point + 1;

  for (index = 0 ; index < 10 ; index++)
  {
    tableaux[index] = (double *) calloc( 1 + nbr_pts, sizeof(double) );
    if (tableaux[index] == NULL)
    {
      int i;

      ERREUR(MANQUE_MEMOIRE);
      for (i = 0 ; i < index ; i++ )
	free(tableaux[i]);

#ifdef avec_p_p      
      free(tableaux);
#endif
      return;
    }
  }

  tableaux[3][0] = 0; tableaux[3][1] = 0;
  tableaux[3][nbr_pts - 1] = 0;
  tableaux[4][0] = 0; tableaux[4][1] = 0;
  tableaux[5][0] = 0; tableaux[5][1] = 0;
  tableaux[5][nbr_pts - 1] = 0;
  tableaux[6][0] = 0; tableaux[6][1] = 0;
  tableaux[7][0] = 0; tableaux[7][1] = 0;
  tableaux[8][0] = 0; tableaux[8][1] = 0; tableaux[8][2] = 0;
  tableaux[8][nbr_pts] = 0; tableaux[8][nbr_pts - 1] = 0;

  d_x1 = (brut->x)[1] - (brut->x)[0];
  if (d_x1 == 0)
  {
    EXF_ERREUR(ERR_DOUBLONS);
    for (index = 0 ; index < 10 ; index++) /* On libre la mmoire */
      free(tableaux[index]);
#ifdef avec_p_p      
    free(tableaux);
#endif
    return;
  }
  y1 = ( (brut->y)[1] - (brut->y)[0] ) / d_x1;

  tableaux[3][1] = 1/d_x1; /* On aura besoin de cette valeur : vaut tab[5][0] en fait... */

  for (index = 2 ; index < nbr_pts - 1 ; index++) /* Brut[i] : 0 <= i < nbr_pts - 1 */
  {
    d_x2 = d_x1; y2 = y1;
    d_x1 = (brut->x)[index] - (brut->x)[index - 1];
    if (d_x1 == 0)
    {
      EXF_ERREUR(ERR_DOUBLONS);
      for (index = 0 ; index < 10 ; index++) /* On libre la mmoire */
	free(tableaux[index]);
#ifdef avec_p_p      
      free(tableaux);
#endif
      return;
    }

    y1 = ( (brut->y)[index] - (brut->y)[index - 1] ) / d_x1;
    (infos->mu_0->y)[index - 1] = y1 - y2;
    tableaux[6][index] = 2 * (d_x1 + d_x2) / 3;
    tableaux[7][index] = d_x1 / 3;
    tableaux[3][index] = 1 / d_x1;
    tableaux[4][index] = - (tableaux[3][index - 1] + tableaux[3][index]);
  }

  /* On a rempli tab[   4, 5, 6, 7][           ] de 2  nbr_pts - 2
                 tab[3            ][           ] de 1  nbr_pts - 2
		 tab[3            ][0          ] = 0 toujours
		 tab[   4, 5, 6, 7][0, 1       ] = 0 toujours
		 tab[3, 4, 5, 6, 7][nbr_pts - 1] = 0
		 tab[3, 4, 5, 6, 7][nbr_pts    ] non dfinis     */

  /* Ici, on a besoin de tab[3][1 ... nbr_pts - 2]
	                 tab[4][2 ... nbr_pts - 1] ==> ok */

  for (index = 2 ; index <= nbr_pts - 2 ; index++)
  {
    tableaux[0][index] = tableaux[3][index] * tableaux[3][index]
	               + tableaux[4][index] * tableaux[4][index]
		       + tableaux[3][index - 1] * tableaux[3][index - 1];
    tableaux[1][index] = tableaux[3][index] * tableaux[4][index + 1]
                       + tableaux[4][index] * tableaux[3][index];
    tableaux[2][index] = tableaux[3][index] * tableaux[3][index + 1];
    tableaux[8][index] = 0; /* Suppos nul - rien  ce sujet dans le source de Juris */
  }
  /* On a rempli tab[0, 1, 2][                    ] de 2  nbr_pts - 2
	         tab[0, 1, 2][0          , 1      ] non dfinis
	         tab[0, 1, 2][nbr_pts - 1, nbr_pts] non dfinis        */
	
  controle = - (infos->parametre);

  tableaux[3][1] = 0; /* Remise  cette valeur pour avoir le rsultat correct */

  while (sortie == FAUX)
  {
    /* Ici, on a besoin de tab[3][0 ... nbr_pts - 4] ==> ok
	                   tab[0][2 ... nbr_pts - 2] ==> ok
	                   tab[1][2 ... nbr_pts - 2] ==> ok
			   tab[2][2 ... nbr_pts - 2] ==> ok
			   tab[6][2 ... nbr_pts - 2] ==> ok
			   tab[7][2 ... nbr_pts - 2] ==> ok
			   tab[8][0 ... nbr_pts - 4] ==> ok */

    for (index = 2 ; index <= nbr_pts - 2 ; index++)
    {
      tableaux[4][index - 1] = y1   * tableaux[3][index - 1];
      tableaux[5][index - 2] = d_x2 * tableaux[3][index - 2];
      tableaux[3][index] = 1 / ( p * tableaux[0][index]     +        tableaux[6][index]
			      - y1 * tableaux[4][index - 1] - d_x2 * tableaux[5][index - 2]);
      tableaux[8][index] = (infos->mu_0->y)[index - 1] - tableaux[4][index - 1] * tableaux[8][index - 1]
	                 - tableaux[5][index - 2] * tableaux[8][index - 2];
      y1 = p * tableaux[1][index] + tableaux[7][index] - d_x1 * tableaux[4][index - 1];
      d_x2 = d_x1;
      d_x1 = tableaux[2][index] * p;
    }
    /* On a rempli tab[4][ ] de 1  nbr_pts - 3
                   tab[5][ ] de 0  nbr_pts - 4
		   tab[3][ ] de 2  nbr_pts - 2
		   tab[8][ ] de 2  nbr_pts - 2 */

    tableaux[4][nbr_pts - 2] = y1   * tableaux[3][nbr_pts - 2]; /* Valeurs non dfinies dans le code de Juris   */
    tableaux[5][nbr_pts - 3] = d_x2 * tableaux[3][nbr_pts - 3]; /* Semble pas modifier le rsultat, mais bon... */
    tableaux[5][nbr_pts - 2] = d_x1 * tableaux[3][nbr_pts - 2];

    /* On a besoin de tab[3][2 ... nbr_pts - 2] ==> ok
                      tab[4][2 ... nbr_pts - 2] ==> ?? pour nbr_pts-2 - modifi
		      tab[5][2 ... nbr_pts - 2] ==> ?? pour nbr_pts-2 et -3 - modifi
		      tab[8][2 ... nbr_pts    ] */

    for (index = nbr_pts - 2 ; index > 1 ; index--)
    {
      tableaux[8][index] = tableaux[3][index] * tableaux[8][index]
	                 - tableaux[4][index] * tableaux[8][index + 1]
			 - tableaux[5][index] * tableaux[8][index + 2];
    }
    /* On a rempli tab[8][ ] de 2  nbr_pts - 2 */

    /* On a besoin de tab[8][1 ... nbr_pts - 1] ==> ?? pour 1 et nbr_pts - 1, pris nuls */

    y2 = 0; d_x1 = 0;
    for (index = 1 ; index <= nbr_pts - 2 ; index++)
    {
      d_x2 = d_x1;
      d_x1 = ( tableaux[8][index + 1] - tableaux[8][index] )
	   / ( (brut->x)[index] - (brut->x)[index - 1] );
      tableaux[9][index] = (d_x1 - d_x2);
      y2 += tableaux[9][index] * (d_x1 - d_x2);
    }
    
    /* On a rempli tab[9][ ] de 1  nbr_pts - 2 */
    tableaux[9][nbr_pts - 1] = -d_x1;

    d_x2 = -d_x1;
    y2  -= d_x2 * d_x1;
    d_x2 = controle;
    controle = y2 * p * p;
    if ( (controle >= infos->parametre) || (controle <= d_x2) )
    {
      sortie = VRAI; /* On a fini la boucle... */
    }
    else
    {
      y1   = 0;

      /* On a besoin de tab[9][1 ... nbr_pts - 1]
	                tab[3][0 ... nbr_pts - 2]
			tab[4][1 ... nbr_pts - 3]
			tab[5][0 ... nbr_pts - 4] */
			                  
      d_x1 = ( tableaux[9][2] - tableaux[9][1] )
	   / ( (brut->x)[1] - (brut->x)[0] );
      for (index = 2 ; index <= nbr_pts - 2 ; index++)
      {
	d_x2  = d_x1;
	d_x1  = ( tableaux[9][index + 1] - tableaux[9][index] )
	      / ( (brut->x)[index] - (brut->x)[index - 1] );
	d_x2  = d_x1 - d_x2;
	d_x2 -= tableaux[4][index - 1] * tableaux[3][index - 1];
	d_x2 -= tableaux[5][index - 2] * tableaux[3][index - 2];
	y1   += d_x2 * d_x2 * tableaux[3][index];
	tableaux[3][index] = d_x2;
      }

      /* On a rempli tab[3][2 ... nbr_pts - 2] */

      d_x1 = y2 - p * y1;
      if (d_x1 <= 0 ) 
	sortie = VRAI; /* On sort de la boucle */
      else
	p += (infos->parametre - controle) / ( d_x1 * ( sqrt(infos->parametre / y2) + p ) );
    }
  }

  /* Fin du calcul */
  y_brut = brut->y;
  y_mu0 = infos->mu_0->y;	
  for (index = 0 ; index < brut->nombre_point ; index++, y_mu0++, y_brut++)
  {
    *y_mu0 = (*y_brut) - p * tableaux[9][index + 1];
  }

  for (index = 0 ; index < 10 ; index++) /* On libre la mmoire */
    free(tableaux[index]);

#ifdef avec_p_p      

  free(tableaux);

# ifdef _DEVELOPPE_

  printf("Temps "e_AIGU"coul"e_AIGU", avec double ** : %ld \n", clock() - temps);

# endif /* def _DEVELOPPE_ */

#else

# ifdef _DEVELOPPE_

  printf("Temps "e_AIGU"coul"e_AIGU", avec double *[] : %ld \n", clock() - temps);

# endif /* def _DEVELOPPE_ */

#endif /* def avec_p_p */

  GRF_bornes(infos->mu_0, ORDONNE, DESORDONNE);
}

/* ----------------- Extraction proprement dite du spectre E.X.A.F.S. -------------------- */

void MU3_extrait(Graphe *brut, MU3_infos *infos)
{
  register double *brut_y, *exafs_y, *mu0_y;
  unsigned long i;

  brut_y  = brut->y;
  exafs_y = infos->exafs->y;
  mu0_y   = infos->mu_0->y;
	
  for (i = 0 ; i < brut->nombre_point ; i++)
  {
    *exafs_y = *brut_y - *mu0_y;
    brut_y++; exafs_y++; mu0_y++;
  }

  /* Recherche des bornes */
  
  GRF_bornes(infos->exafs, ORDONNE, DESORDONNE); /* Le graphe est normalement tri */ 
}

/* ~~~~~~~~~~~~~~~~~ Gestion de l'interface ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ----------------- Ouverture de la fentre lie  cette opration -----------------------\
 |   Ne renvoie rien                                                                       |
 |                                                                                         |
 | brut : le graphe qui va subir la soustraction                                           |
 | mu_0 : le polynme soustrait utilis  l'tape prcdente (NULL : on en cre un)        |
 \---------------------------------------------------------------------------------------- */

void MU3_ouvre(Graphe *brut, Graphe *mu_0)
{
  Fenetre *fenetre;
  MU3_infos *infos = NULL;
  unsigned long i;
  register double *exafs_x, *brut_x, *mu0_x;
  char titre[100];

  if (brut == NULL) return; /* Vraiment rien  faire */

  if ( arbres[ELIM_MU0_TROIS].ouvert == VRAI ) /* Arbre dj ouvert */
  {
    Graphe *vieux; /* Ancien spectre travaill dans ce module */
    int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

    fenetre = ARB_fenetre( &(arbres[ELIM_MU0_TROIS]) );
    vieux = (Graphe *) fenetre->pointeur;
    if ( (reponse == 1) || (vieux == brut) ) /* On active seulement */
    {
      FEN_premierplan(fenetre); /* Fentre au premier plan */
      return; /* Rien d'autre  faire */
    }
    else if (reponse == 2) /* On rappelle, pour le nouveau graphe */
    {
      if ( brut->drapeaux.protege )
      {
	EXF_ERREUR(GRF_PROTEGE);
	return;
      }

      infos = (MU3_infos *) fenetre->adresse;
      vieux->drapeaux.protege = FAUX; /* On a fini de travailler avec */

      GRF_efface(infos->exafs, VRAI, VRAI);   /* On efface le graphe associ  */
      GRF_efface(infos->mu_0, VRAI, VRAI);

      FEN_premierplan(fenetre); /* Fentre au premier plan */
    }
    else /* On annule l'appel */
      return;
  }
  else /* Arbre non ouvert : on l'ouvre */
  {
    if ( brut->drapeaux.protege )
    {
      EXF_ERREUR(GRF_PROTEGE);
      return;
    }

    fenetre = creer_mixte_fenetre(NAME|CLOSER|FULLER|INFO|SIZER|MOVER, ELIM_MU0_TROIS, HAUT,
				  RDS_mu03,
				  ( boolean (*)(Fenetre *, unsigned int, int) ) NULL,
				  ( boolean (*)(Fenetre *, int, int, int, int, int) ) NULL, 
				  MSG_mu03,
				  ( boolean (*)(Fenetre *, int, int) ) NULL,
				  ( boolean (*)(Fenetre *) ) NULL);
    if (fenetre == NULL) return; /* Rien  faire... */

#ifdef _FRANCAIS_
    FEN_titre(fenetre, "Troisi"e_GRAVE"me "e_AIGU"tape de l'"e_AIGU"limination de l'absorption atomique");
#endif
#ifdef _ENGLISH_
    FEN_titre(fenetre, "Third step of atomic absorption removal");
#endif
  }

  if (infos == NULL) /* Aucune info n'existe dj */
  {
    infos = (MU3_infos *) malloc( (size_t) sizeof(MU3_infos) );
    if (infos == NULL)
    {
      ferme_fenetre(fenetre);
      ERREUR(MANQUE_MEMOIRE);
      return;
    }
  }

#ifdef _FRANCAIS_
  infos->exafs = GRF_creer(brut->nombre_point, GR_EXAFS, GR_NORMAL,
			   "EXAFS en cours d'extraction", "Nombre d'onde", "Signal", UNIT_1_ANG, UNIT_ARB);
#endif
#ifdef _ENGLISH_
  infos->exafs = GRF_creer(brut->nombre_point, GR_EXAFS, GR_NORMAL,
			   "EXAFS while extracting", "Wave number", "Signal", UNIT_1_ANG, UNIT_ARB);
#endif

  if (infos->exafs == NULL)
  {
    free(infos);
    ferme_fenetre(fenetre);
    return;
  }
  fenetre->pointeur = brut;  /* Graphe trait              */
  fenetre->adresse  = infos; /* Infos d'extraction de mu_0 */

  if (mu_0) /* On a envoy un polynme */
  {
    if ( mu_0->nombre_point != brut->nombre_point )
    {
      EXF_ERREUR(NBRPTS_INCORRECT);
      mu_0 = NULL;
    }
  }			

  infos->mu_0_ref = mu_0; /* Le graphe mu_0 de rfrence */

#ifdef _FRANCAIS_
  sprintf(titre, MU"0 soustrait "a_GRAVE" %s", brut->nom);
  infos->mu_0 = GRF_creer(brut->nombre_point, GR_MU0, GR_NORMAL,
			  titre, "Nombre d'onde", MU"0", UNIT_1_ANG, UNIT_ARB);
#endif
#ifdef _ENGLISH_
  sprintf(titre, MU"0 removed from %s", brut->nom);
  infos->mu_0 = GRF_creer(brut->nombre_point, GR_MU0, GR_NORMAL,
			  titre, "Wave number", MU"0", UNIT_1_ANG, UNIT_ARB);
#endif

  if (infos->mu_0 == NULL)
  {
    GRF_efface(infos->exafs, VRAI, VRAI);
    free(infos);
    ferme_fenetre(fenetre);
    return;
  }

  brut->drapeaux.protege = VRAI; /* Calcul en cours sur ce spectre -> rien de faisable */

  /* On prpare les spectres pour gagner du temps */

  memcpy( infos->mu_0->x , brut->x, (size_t) ( brut->nombre_point * sizeof(double) ) );
  memcpy( infos->exafs->x, brut->x, (size_t) ( brut->nombre_point * sizeof(double) ) );

/*  mu0_x = infos->mu_0->x; exafs_x = infos->exafs->x; brut_x = brut->x;
  for (i = 0 ; i < brut->nombre_point ; i++)
  {
    *mu0_x   = *brut_x;
    *exafs_x = *brut_x;
    mu0_x++; exafs_x++; brut_x++;
  } */
  infos->mu_0->x_min = brut->x_min; infos->mu_0->x_max = brut->x_max;
  infos->exafs->x_min = brut->x_min; infos->exafs->x_max = brut->x_max;

  /* On calcule la fonction  soustraire */

  infos->parametre = prend_fvaleur(ELIM_MU0_TROIS, EL3_BOUTON);
  
  infos->exafs->drapeaux.protege = VRAI; /* On protge le graphe de toute btise */
  infos->mu_0->drapeaux.protege  = VRAI; /* On protge le graphe de toute btise */

  graf_mouse(S_ABEILLE, NULL); /* Souris en abeille  */
  MU3_calc(brut, infos);       /* Calcul du polynme */
  MU3_extrait(brut, infos);    /* Extraction         */
  graf_mouse(S_FLECHE, NULL);  /* Souris normale     */

  fenetre->drapeaux.redessin    = VRAI;
  fenetre->msg_masque.fermeture = VRAI; /* Messages traits par MSG_mu01 */
}

/* ------------- Redessin de la fentre d'limination de mu_0, troisime tape ------------\
 |   Le clipping est dj install correctement lors de l'appel                            |
 |                                                                                         |
 | fenetre    : structure associe  la fentre considre                                 |
 | x, y       : coordonnes de la zone de redessin                                         |
 | larg, haut : dimensions de la zone de redessin                                          |
 \---------------------------------------------------------------------------------------- */

static void RDS_mu03(Fenetre *fenetre, int x, int y, int larg, int haut)
{
  Graphe *graphe;
  MU3_infos *infos;
  DES_graphe param_dessin;
  double x_echelle, y_echelle;
  int y_trace, haut_trace, rien;

  vsl_type (_PRG_station->handle, 1);
  vswr_mode(_PRG_station->handle, MD_REMPLACE);
  vsf_color(_PRG_station->handle, BLANC); /* couleur de remplissage: blanc */

  points[0] = x;            points[1] = y;
  points[2] = x + larg - 1; points[3] = y + haut + 1;
  v_bar(_PRG_station->handle, points); /* On met le fond au blanc */

  OBJ_taille(ELIM_MU0_TROIS, ROOT, -1, -1, &rien, &y_trace);
  haut_trace = fenetre->w_haut - y_trace;
  y_trace   += fenetre->w_y;

  graphe = (Graphe *)    fenetre->pointeur;
  infos  = (MU3_infos *) fenetre->adresse;
  x_echelle = ( (double) fenetre->w_larg ) / (graphe->x_max - graphe->x_min);
  y_echelle = ( (double) haut_trace )      / (graphe->y_max - graphe->y_min);

  /* On trace la courbe exprimentale - en noir */

  param_dessin.graphe  = graphe;
  param_dessin.couleur = NOIR;
  param_dessin.style   = 1;
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	     graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);

  /* On trace la courbe  soustraire - en bleu */	

  param_dessin.graphe  = infos->mu_0;
  param_dessin.couleur = BLEU;
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	     graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);

  /* On trace le graphe EXAFS extrait, en bordeaux - en fin, car chelle diffrente */

  if ( infos->exafs->y_max == infos->exafs->y_min ) /* Le signal extrait est nul ! */
  {
#ifdef _FRANCAIS_
    v_gtext(_PRG_station->handle, fenetre->w_x + fenetre->w_larg >> 1, y_trace + haut_trace >> 1,
	    "Le signal EXAFS obtenu est nul");
#endif
#ifdef _ENGLISH_
    v_gtext(_PRG_station->handle, fenetre->w_x + fenetre->w_larg >> 1, y_trace + haut_trace >> 1,
	    "The EXAFS obtained is nul");
#endif
    return;
  }

  y_echelle = ( (double) haut_trace ) / (infos->exafs->y_max - infos->exafs->y_min);

  param_dessin.graphe  = infos->exafs;
  param_dessin.couleur = GRIS_FONCE;
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace,
	     infos->exafs->x_min, infos->exafs->x_max, infos->exafs->y_min, infos->exafs->y_max,
	     x_echelle, y_echelle);

  /* On trace l'axe des X associ, en violet */

  vsl_color (_PRG_station->handle, VIOLET);
  points[0] = fenetre->w_x;
  points[1] = y_trace + y_echelle * infos->exafs->y_max;
  points[2] = fenetre->w_x + fenetre->w_larg;
  points[3] = points[1];
  v_pline(_PRG_station->handle, 2, points);
}

/* -------------- Messages reus par la fentre d'limination de mu_0, 3e tape ---------- */

static boolean MSG_mu03(Fenetre *fenetre)
{
  int retour;
  Graphe *brut, *exafs, *mu_0, *mu_ref;
  MU3_infos *infos;

  brut  = (Graphe *)    fenetre->pointeur;
  infos = (MU3_infos *) fenetre->adresse;

  switch ( buffer[0] ) /* Nature du message */
  {
  case WM_CLOSED: /* On veut fermer la fentre */
    retour = EXF_ERREUR(FIN_MODULE);
    if (retour == 3) /* On annule la fermeture */
    {
      return VRAI;
    }

    /* De toute faon, on ferme */

    exafs  = infos->exafs;
    mu_0   = infos->mu_0;
    mu_ref = infos->mu_0_ref;

    ferme_fenetre(fenetre);

    if (mu_ref) /* Existe bien... */
      mu_ref->drapeaux.protege = FAUX;
    brut->drapeaux.protege   = FAUX;

    if (retour == 2) /* On ferme sans soustraire */
    {
      GRF_efface(exafs, VRAI, VRAI);
      GRF_efface(mu_0,  VRAI, VRAI);
      GRF_aller(brut);                /* On va au spectre de dpart */
      free(infos);
      return VRAI;
    }

    /* On veut extraire le spectre --> il faut faire la somme de mu0_ref et mu_0 */

    met_fvaleur(EXAFS_GENERAL, EXF_PARAM_LISSAG, infos->parametre);

    sprintf(_intermediaire, "%s - %c0, derni"e_GRAVE"re "e_AIGU"tape%c", brut->nom, 230, 0);
    GRF_titre(exafs, _intermediaire);

    if (Histoire)
    {
#ifdef _FRANCAIS_
      fprintf(Histoire, "\n--- "E_AIGU"limination de l'absorption atomique ---\n");
      fprintf(Histoire, "    (Correction de l'extraction : 3e "e_AIGU" de l'extraction\n");
      fprintf(Histoire, "Spectre trait"e_AIGU"   : %s\n", brut->nom);
      fprintf(Histoire, "Signal extrait   : %s\n", exafs->nom);
      fprintf(Histoire, "M"e_AIGU"thode          : soustraction d'une courbe\n");
      fprintf(Histoire, "                   obtenue par lissage du spectre complet\n");
      fprintf(Histoire, "                   (troisi"e_GRAVE"me "e_AIGU"tape du logiciel EDA)\n");
#endif
#ifdef _ENGLISH_
      fprintf(Histoire, "\n--- Atomic absorption removal ---\n");
      fprintf(Histoire, "    (Extraction correction: third step\n");
      fprintf(Histoire, "Spectrum treated: %s\n", brut->nom);
      fprintf(Histoire, "Signal extracted: %s\n", exafs->nom);
      fprintf(Histoire, "Methode         : removing a courbe\n");
      fprintf(Histoire, "                  calculated by complete signal smoothing with constraint\n");
#endif
    }

    if (mu_ref) /* Mu_ref existe */
    {
      unsigned long i;
      register double *exafs_y, *mu_0_y, *mu_ref_y;

      for (i = 0, exafs_y = exafs->y, mu_0_y = mu_0->y, mu_ref_y = mu_ref->y ;
	   i < mu_0->nombre_point ;
	   i++, exafs_y++, mu_0_y++, mu_ref_y++)
      {
	*mu_0_y += *mu_ref_y;

	if ( mu_0_y )
	  *exafs_y /= *mu_0_y; /* On normalise */
	else
	{
	  EXF_ERREUR(DIV_ZERO);
	}
      }
      GRF_bornes(exafs, ORDONNE, DESORDONNE); /* Bornes ont pu changer... */
      GRF_bornes(mu_0 , ORDONNE, DESORDONNE);

      if (Histoire)
      {
#ifdef _FRANCAIS_
	fprintf(Histoire, "                 (le signal est normalis"e_AIGU"\n");
#endif
#ifdef _ENGLISH_
	fprintf(Histoire, "                 (signal is normalised\n");
#endif
      }

      /* Il faudrait diviser mu_ref par k^n, n affichage, pour bien faire * /
	 EXF_k_p_n(mu_ref, -n_affichage); */
    }
    else /* On propose une normalisation thorique */
    {
      Graphe *norm = norm_exafs(exafs);

      if (norm)
	exafs = norm;
      else
	EXF_ERREUR(NORM_IMPOSSIBLE);
    }

    if (Histoire)
    {
#ifdef _FRANCAIS_
      fprintf(Histoire, "Param"e_GRAVE"tre de lissage : %.10g\n", infos->parametre);

      if (mu_ref)
	fprintf(Histoire, "Absorption atomique compl"e_GRAVE"te : %s\n", mu_0->nom);
      else
	fprintf(Histoire, "Ce qui a "e_AIGU"t"e_AIGU" soustrait        : %s\n", mu_0->nom);
#endif
#ifdef _ENGLISH_
      fprintf(Histoire, "Smoothing parameter: %.10g\n", infos->parametre);
      if (mu_ref)
	fprintf(Histoire, "Complete atomic absorption: %s\n", mu_0->nom);
      else
	fprintf(Histoire, "What has been removed     : %s\n", mu_0->nom);
#endif
    }

    exafs->drapeaux.protege = FAUX; /* On n'utilise plus ces graphes... */
    mu_0->drapeaux.protege  = FAUX;
    exafs->drapeaux.sauve   = FAUX; /* Graphe non sauv            */
    mu_0->drapeaux.sauve    = FAUX;
    GRF_aller(exafs); /* On va  ce graphe */

    free(infos);

    if (TT_exafs) /* On veut faire le traitement de tout l'EXAFS d'un coup */
    {
      Graphe *retour = magnifier(exafs); /* Pour la suite */

      if (retour)
	TF_ouvre(retour);
      else
	TF_ouvre(exafs); /* On fait la TF de ce graphe... */
    }
    return VRAI;
  }

  return VRAI;
}
