/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
   Tout ce qui concerne la gestion des arbres GEM, non automatique
   (c) Emmanuel CURIS 1996-1997
   
   Problmes connus : plusieurs x_souris et y_souris inutiliss (normal...)

   1er avril 1998
   * Modifi liste des graphes : nom affich plus long ; affichage de tous/aucun
   * GER_xnorm : bouton nouveau graphe oui/non actif

   10, 12 & 13, 16 & 17, 25, 31 mars 1998
   * Remplac des printf() par des botes d'alerte ; supprim quelques printf() de dbogage
   * Inclusion de "systeme.h" au dbut
   * Inclus "extrait.h" (pour extraction du signal EXAFS/XANES)
   * Commenc gestion de l'arbre d'exafs : GER_reproduit
   * Inclus "erreurs.h" (pour l'exafs)
   * Termin l'adaptation de GER_extrait() aux nouvelles fonctions
   * Commenc routines de gestion du paramtrage de la dispersion : GER_dispersion()
   * Intgr "exf_stat.h"
   * Modifi GRF_par_type() : accepte les types spciaux < 0
   * Insertion de la date au dbut de l'historique -> insertion de gem_tos.h
   * Modifi GER_extrait() : paramtrage du type de moyenne en absorption
   * Lgrement modifi GER_extrait() : possibilit d'avoir 2 chambres d'ionisation
   
   5 & 6 fvrier 1998
   * Adapt GER_extrait()  la nouvelle version de l'arbre
   * Gestion de l'historique d'une sance commence
   * Remplac les OB_STATE par OBJ_choisit dans GER_infos()
   
   12 janvier 1998
   * Inclus les messages d'erreur GEM
   
   21 au 28 novembre ; 2 & 3, 5 dcembre 1997
   * Cration de la fonction GER_decompose() [fentre de dcomposition...]
   * Cration de cet historique
   * Rorganisation de l'interface
   - options appeles depuis la liste des graphes : normalisation
   - bote d'infos : choix du menu affich...
   * Cr GER_tronque()
   * Complt GER_legende() : choix du style de grille fonctionne
   * Modifi  GER_graphes() : - utilisation de GRF_titre(), GRF_legende(), lit_chaine()
   - redessin du style de ligne fonctionne (mauvais objet redessin...)
   * Ajout l'inclusion de gem_aes.h -> dfinitions de ED_xxx disponibles mme pour GCC
   * Modifi les boucles de graphes dans GER_fusion -> plus d'avertissement avec GCC (?)
   * Dplac les messages d'erreur dans erreurs.c
   * Adaptation de la bote de prparation de la TF selon qu'on choisit TF simple ou rapide
   TF_simple -> seul le pas est modifiable, plus r_min et r_max
*/

#define _EXAFS_GEM_ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "ERREURS.H"
#include "EXF_STAT.H"
#include "EXTRAIT.H"
#include "FOURIER.H"

#include "LISSAGE.H"

/* *************** Quelques variables locales de ce module *************************/

const char *TXT_informations[] = 
{
  "                 *** LASE ***",
  "      Logiciel d'analyse des spectres EXAFS",
  "",
  " "COPYRIGHT" Emmanuel CURIS, 1995 - 1998",
  " Version 1.2, dat"e_AIGU"e du 6 octobre 1998",
#ifdef _UNIX_
  "        ------ Version X-Window -------",
#else
  "         -------- Version GEM --------",
#endif
  "",
  " Programm"e_AIGU" sur Hades 40 / TOS 3.06",
  "    avec Interface 2.21 et Turbo C 1.1",
  "           sur Sparc 4 / SunOS",
  "    avec emacs 19.2 et gcc 2.7.1",
  "           sur Alpha 433au / Digital Unix",
  "    avec emacs 19 et cc/digital",
  "",
  " Une partie du code est adapt"e_AIGU"e des",
  " programmes EDA d'Alexei Kuzmin - consultez la notice",
  " pour les r"e_AIGU"f"e_AIGU"rences exactes.",
  "",
  " Merci "a_GRAVE" : Jean-Hughes Agondanou",
  "           Bernard Amadon (essais Linux/Alpha)",
  "           Simone B"e_AIGU"nazeth",
  "           Dominique B"e_AIGU"r"e_AIGU"ziat",
  "           Bruno Blanchet",
  "           Christophe Curis",
  "           Fran"c_CEDILLE"ois Le Coat (essais Hades 60)",
  "           Christian Marrillat (essais Linux/Falcon)",
  "           Ioannis Nicolis",
  "           Dominique Quatravaux",
  "  pour leurs essais, suggestions et corrections",
  "",
  " Pour toute erreur, question ou suggestion :",
  "    curis@lure.u-psud.fr"
};

#define NBR_LIGNES_D_INFOS 32 /* Lignes de texte  afficher */

/***************** Quelques macros spcifiques de ce programme *********************/

/* ~~~~~~~~~~~~~~~ Tout ce qui concerne la ralisation d'un slecteur de graphes ~ */

/* --------------- Prparation de la liste de tous les graphes du mme type -------\
 |   Renvoie un pointeur sur la liste cre ( librer aprs utilisation)          |
 |           NULL si la cration de la liste a choue                             |
 |                                                                                 |
 | type   : le type de graphe  prendre en compte                                  |
 |   (type < -100 : types spciaux, pour slections plus vastes. Voir graphes.h    |
 | alerte : VRAI ==> on prvient si aucun graphe n'a t trouv                    |
 \-------------------------------------------------------------------------------- */

GRF_liste *GRF_par_type(int type, boolean alerte)
{
  GRF_liste *liste;
  Graphe *index;
  unsigned long i;

  liste = (GRF_liste *) malloc( (size_t) sizeof(GRF_liste) );
  if (liste == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return NULL;
  }

  liste->nbr_graphe = 0; /* Pour l'instant, aucun graphe marqu comme convenant */
  if ( (type >= 0) || (type == GR_AUTRETYPE) ) /* Seuls les graphes du mme type conviennent */
  {
    for (index = premier_graphe ; index ; index = index->suivant)
    {
      if ( (index->type == type) || (index->type == GR_SSTYPE) )
	liste->nbr_graphe++;
    }
  }
  else if ( (type == GR_SSTYPE) || (type == GR_TOUS) )
  {
    for ( index = premier_graphe ; index ; index = index->suivant )
    {
      if ( index->type != GR_INTERNE ) /* Les graphes internes ne sont JAMAIS affichs */
	liste->nbr_graphe++;
    }
  }
  else if ( type == GR_TT_EXAFS ) /* Tous les spectres de type EXAFS */
  {
    for ( index = premier_graphe ; index ; index = index->suivant )
    {
      if ( (index->type == GR_SSTYPE) /* Les graphes sans type sont toujours affichs */
	   || (index->drapeaux.exafs == VRAI) )

	liste->nbr_graphe++;
    }
  }

  if(liste->nbr_graphe == 0) /* Aucun graphe ne convient */
  {
    if (alerte == VRAI) /* On prvient l'utilisateur */
      form_alert(1,"[3][ Aucun graphe du type d"e_AIGU"sir"e_AIGU" | n'est en m"e_AIGU"moire ][ Annuler ]");
    free(liste);
    return NULL;
  }

  liste->graphes = (Graphe **) calloc( liste->nbr_graphe, sizeof (Graphe *) );
  if (liste->graphes == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    free(liste);
    return NULL;
  }

  liste->prem_graphe = 0; /* On commence au premier graphe, en haut de la liste */
  liste->num_select  = 0;

  i = 0;
  for (index = premier_graphe ; index ; index = index->suivant )
  {
    switch (type)
    {
    case GR_TT_EXAFS:
      if ( (index->type == GR_SSTYPE) /* Les graphes sans type sont toujours affichs */
	   || (index->drapeaux.exafs == VRAI) )
      {
	liste->graphes[i] = index;
	i++; /* On prpare le suivant */
      }
      break;

    case GR_TOUS:
    case GR_SSTYPE:
      if ( index->type != GR_INTERNE )
      {
	liste->graphes[i] = index;
	i++; /* On prpare le suivant */
      }
      break;
      
    default:		
      if ( ( index->type == type ) || ( index->type == -1 ) )
      {
	liste->graphes[i] = index;
	i++; /* On prpare le suivant */
      }
    }
  }

  return liste; /* La liste est prte, normalement */
}

/* --------------- Prpare le slecteur avec les infos de la liste ----------------\
 |   Pour l'instant, ne renvoie rien (-> toujours VRAI)                            |
 |
 | liste : liste des graphes  afficher, avec numro du graphe slectionn et tout |
 | arbre : numro de l'arbre dans lequel se trouve le slecteur
 | prem_nom : numro d'objet de la chane affichant le premier nom
 | der_nom  : ---------------------------------------- dernier nom
 | avant    : numro d'objet de la flche vers le haut
 | apres    : ----------------------------------- bas
 | fond     : numro d'objet du fond d'ascenseur
 | bouton   : numro d'objet du curseur de l'ascenseur
 \-------------------------------------------------------------------------------- */

boolean GRF_affiche(GRF_liste *liste, int arbre, int prem_nom, int der_nom, int avant, int apres, int fond, int bouton)
{
	OBJECT *adresse;
	float inter;    /* Pour les ascenseurs */
	int nbr_noms,i; /* Nombre de graphes affichable simultanment par le slecteur */

	if (liste == NULL) /* Aucune info de liste */
		return FAUX;

	adresse = arbres[arbre].adresse; /* Adresse de l'arbre o on travaille */
	nbr_noms = der_nom - prem_nom + 1;
	if (nbr_noms <= 0) /* Incohrence... */
		return FAUX;

	if ( nbr_noms >= liste->nbr_graphe ) /* Moins de graphes que de nom affichables */
	{
		OBJ_active(arbre, avant , FORCE_NON); /* On dsactive l'ascenseur */
		OBJ_active(arbre, fond  , FORCE_NON);
		OBJ_active(arbre, bouton, FORCE_NON);
		OBJ_active(arbre, apres , FORCE_NON);

		OB_Y(adresse,bouton) = 0; /* On positionne l'ascenseur */
		OB_HEIGHT(adresse,bouton) = OB_HEIGHT(adresse,fond);

		for (i = 0 ; i < liste->nbr_graphe ; i++)
		{
			OB_STATE(adresse,prem_nom + i) &= ~DISABLED; /* Actif pour la slection */
			if (i == liste->num_select)
				OB_STATE(adresse,prem_nom + i) |= SELECTED;
			else
				OB_STATE(adresse,prem_nom + i) &= ~SELECTED;
			OB_SPEC(adresse,prem_nom + i) = (long) ( (liste->graphes[i])->nom );
		}
		/* On continue la prparation */
		for ( ; i < nbr_noms ; i++)
		{
			OB_STATE(adresse,prem_nom + i) &= ~SELECTED;
			OB_STATE(adresse,prem_nom + i) |=  DISABLED; /* Inactif pour la slection */
			OB_SPEC(adresse,prem_nom + i) = (long) " ";
		}

		return VRAI; /* Fini */
	}

	/* Plus de graphes slectionns que de graphes affichables */

	OB_STATE(adresse,avant)  &= ~DISABLED;
	OB_STATE(adresse,fond)   &= ~DISABLED; /* Ascenseur actif */
	OB_STATE(adresse,bouton) &= ~DISABLED;
	OB_STATE(adresse,apres)  &= ~DISABLED;

	inter = (float) (nbr_noms) / (float) (liste->nbr_graphe);
	inter *= OB_HEIGHT(adresse,fond);
	if (inter < 5) inter = 5;                /* Au moins 5 pixels de haut       */
	OB_HEIGHT(adresse,bouton) = (int) inter; /* Taille du bouton de l'ascenseur */

	inter  = OB_HEIGHT(adresse,fond) - OB_HEIGHT(adresse,bouton);
	inter *= liste->prem_graphe;
	inter /= (liste->nbr_graphe - nbr_noms);
	OB_Y(adresse,bouton) = (int) inter; /* Position de l'ascenseur */

	for (i = 0 ; i < nbr_noms ; i++)
	{
		OB_STATE(adresse,prem_nom + i) &= ~DISABLED; /* Actif pour la slection */
		OBJ_choisit(arbre, prem_nom + i, i + liste->prem_graphe == liste->num_select);
		OB_SPEC(adresse,prem_nom + i) = (long) ( (liste->graphes[i + liste->prem_graphe])->nom );
	}

	if (liste->prem_graphe == 0)
	{
		OB_STATE(adresse,avant) |= DISABLED;  /* Pas de graphe avant              */
	}

  if (liste->prem_graphe + nbr_noms - 1 == liste->nbr_graphe)
  {
  	OB_STATE(adresse,apres) |= DISABLED;  /* Pas de graphe aprs              */
  }

	return VRAI;
}

/* --------------- Gestion d'un slecteur (un seul graphe parmi une liste) --------\
 |    Renvoie VRAI si tout s'est bien pass, FAUX sinon
 |
 | num_objet : l'objet sur lequel on a cliqu
 | x, y      : coordonnes de la souris lors du clic (ncessaire pour l'ascenseur) |
 |    Les autres paramtres sont comme ci_dessus
 \-------------------------------------------------------------------------------- */

boolean GRF_selecteur(GRF_liste *liste, int arbre, int prem_nom, int der_nom, 
                      int avant, int apres, int fond, int bouton, int num_objet, int x, int y)
{
	OBJECT *adresse;
	int y_obj;   /* Aussi pour l'ascenseur */
	unsigned long nbr_noms, i;
	float inter; /* Pour les ascenseurs    */

	if (liste == NULL) return FAUX; /* Pas de liste de graphes... */

	nbr_noms = der_nom - prem_nom + 1;
	if (nbr_noms <= 0) return FAUX; /* Absurde...                 */

	adresse = arbres[arbre].adresse;

	if ( (num_objet >= prem_nom) && (num_objet <= der_nom) )
	{
		i = liste->num_select - liste->prem_graphe; /* Position du graphe choisi par rapport au premier nom visible */
		if (num_objet - prem_nom == i) /* Clic sur le graphe dj choisi */
			return VRAI; /* Rien  faire */

		if ( i < nbr_noms ) /* Graphe avant choisi est visible */
		{
			OBJ_choisit(arbre, (int) (prem_nom + i), FORCE_NON); /* On dslectionne */
		}

		liste->num_select = liste->prem_graphe + (num_objet - prem_nom); /* Nouveau graphe choisi */
		OBJ_choisit(arbre, num_objet, FORCE_OUI); /* On slectionne */

		return VRAI;
	}

	if (num_objet == avant) /* Clic sur la flche remontant d'un cran la liste des graphes */
	{
		if (liste->prem_graphe == 0) /* Pas normal de recevoir ce clic */
		{
			OBJ_active(arbre, avant, FORCE_NON);
		  return FAUX;
		}

		for (i = nbr_noms - 1 ; i > 0 ; i--)
		{
			OB_STATE(adresse,prem_nom + i) = OB_STATE(adresse,prem_nom + i - 1 );
			OB_SPEC(adresse,prem_nom + i) = OB_SPEC(adresse,prem_nom + i -1);
		}
		liste->prem_graphe--;
		OB_SPEC(adresse,prem_nom) = (long) ( (liste->graphes[liste->prem_graphe])->nom );
		OB_STATE(adresse,prem_nom) &= ~DISABLED;
		if (liste->num_select == liste->prem_graphe)
			OB_STATE(adresse,prem_nom) |= SELECTED;
		else
		  OB_STATE(adresse,prem_nom) &= ~SELECTED;

		if (liste->prem_graphe == 0) /* Pas de graphe avant */
		{
			OBJ_active(arbre, avant, FORCE_NON);
		}

		OBJ_active(arbre, apres, FORCE_OUI);
		dessine_objet(arbre,prem_nom - 1); /* On dessine la liste des noms */

		inter  = OB_HEIGHT(adresse,fond) - OB_HEIGHT(adresse,bouton);
		inter *= liste->prem_graphe;
		inter /= (liste->nbr_graphe - nbr_noms);
		OB_Y(adresse,bouton) = (int) inter; /* Position de l'ascenseur */
		dessine_objet(arbre,fond);

		return VRAI; /* Plus rien  faire */
	}

	if (num_objet == apres) /* Clic sur la flche descendant d'un cran la liste des graphes */
	{
		if (liste->prem_graphe == liste->nbr_graphe - nbr_noms) /* Pas de graphe aprs */
		{
			OB_STATE(adresse,apres) |= DISABLED;
			dessine_objet(arbre,apres);
			return FAUX;
		}

		for (i = 0 ; i < nbr_noms - 1 ; i++)
		{
			OB_STATE(adresse,prem_nom + i) = OB_STATE(adresse,prem_nom + i + 1 );
			OB_SPEC(adresse,prem_nom + i) = OB_SPEC(adresse,prem_nom + i + 1);
		}
		liste->prem_graphe++;
		OB_SPEC(adresse,der_nom) = (long) ( (liste->graphes[liste->prem_graphe + nbr_noms - 1])->nom );
		OB_STATE(adresse,der_nom) &= ~DISABLED;
		if (liste->num_select == liste->prem_graphe + nbr_noms - 1)
			OB_STATE(adresse,der_nom) |= SELECTED;
		else
		  OB_STATE(adresse,der_nom) &= ~SELECTED;

		if (liste->prem_graphe == liste->nbr_graphe - nbr_noms) /* Pas de graphe aprs */
		{
			OB_STATE(adresse,apres) |= DISABLED;
			dessine_objet(arbre,apres);
		}

		OB_STATE(adresse,avant) &= ~DISABLED;
		dessine_objet(arbre,avant);
		dessine_objet(arbre,prem_nom - 1);
		
		inter  = OB_HEIGHT(adresse,fond) - OB_HEIGHT(adresse,bouton);
		inter *= liste->prem_graphe;
		inter /= (liste->nbr_graphe - nbr_noms);
		OB_Y(adresse,bouton) = (int) inter; /* Position de l'ascenseur */
		dessine_objet(arbre,fond);

		return VRAI;		
	}

	if (num_objet == fond) /* Clic sur l'ascenseur, avant ou aprs le bouton */
	{
		objc_offset(adresse,fond,&x,&y_obj);
		y -= y_obj; /* On prend la position de la souris par rapport au fond */
		if ( y < OB_Y(adresse,bouton) ) /* Clic avant le curseur -> on monte de nbr_noms - 1 */
		{
			if (liste->prem_graphe > nbr_noms - 1)
				liste->prem_graphe -= nbr_noms - 1;
			else
				liste->prem_graphe = 0;
		}
		else
		{
			liste->prem_graphe += nbr_noms - 1;
			if (liste->prem_graphe > liste->nbr_graphe - nbr_noms)
				liste->prem_graphe = liste->nbr_graphe - nbr_noms;
		}
		GRF_affiche(liste,arbre,prem_nom,der_nom,avant,apres,fond,bouton);
		dessine_objet(arbre,prem_nom - 1);
		dessine_objet(arbre,avant);
		dessine_objet(arbre,apres);
		dessine_objet(arbre,fond);
	}

	if (num_objet == bouton) /* Clic sur le bouton -> on le fait glisser */
	{
    inter  = graf_slidebox(adresse,fond,bouton,1); /* Position en pour-mille  - vertical */
		inter *= liste->nbr_graphe - nbr_noms;
		inter /= 1000; /* OB_HEIGHT(adresse,fond) - OB_HEIGHT(adresse,bouton); */
		liste->prem_graphe = (unsigned long) inter;
		if ( (signed) (liste->prem_graphe) < 0 )
			liste->prem_graphe = 0;
		if ( liste->prem_graphe > liste->nbr_graphe - nbr_noms )
			liste->prem_graphe = liste->nbr_graphe - nbr_noms;

		GRF_affiche(liste,arbre,prem_nom,der_nom,avant,apres,fond,bouton);
		dessine_objet(arbre,prem_nom - 1);
		dessine_objet(arbre,avant);
		dessine_objet(arbre,apres);
		dessine_objet(arbre,fond);			
	}
	
	return FAUX; /* Trs trange : appel alors que pas de clic rel... */
}

/***************** Routines de dessin des objets USERDEF ***************************
 * int cdecl USR_xxxxx(USR_FONCTION *parametres)                                   *
 *
 * parametres : contient les paramtres ncessaires au redessin                    *
 ***********************************************************************************/

/* ~~~~~~~~~~~~~ Affichage d'une ligne traversant horizontalement le bouton ~~~~~~ \
 |  (paramtre interne  l'objet pb_parm : style de ligne  utiliser               |
 \-------------------------------------------------------------------------------- */

int cdecl USR_style(USR_FONCTION *adresse)
{
  PARMBLK *parametres;
  int style_ligne,i;

  parametres = CONVERT_PARMBLK adresse; /* Bizarre, d  la lib. gem du gcc */

  points[0] = parametres->pb_xc;      /* Zone  dessiner */
  points[1] = parametres->pb_yc;
  points[2] = parametres->pb_xc + parametres->pb_wc - 1;
  points[3] = parametres->pb_yc + parametres->pb_hc - 1;
  vs_clip(_PRG_station->handle, 1, points);

  style_ligne = (int) parametres->pb_parm; /* Le paramtre qui est pass */
  if (style_ligne < 7) /* Style de ligne */
  {
    vsl_color(_PRG_station->handle, NOIR);
    vsl_type (_PRG_station->handle, style_ligne); 
		
    points[0] = parametres->pb_x + 1;
    points[1] = parametres->pb_y + parametres->pb_h/2;
    points[2] = parametres->pb_x + parametres->pb_w - 1;
    points[3] = points[1];
    v_pline(_PRG_station->handle, 2, points);
  }
  else
  {
    vsm_color(_PRG_station->handle, NOIR);
    vsm_type (_PRG_station->handle,style_ligne - 6);
    vsm_height(_PRG_station->handle,parametres->pb_h - 2);
		
    for (i = 0 ; i < parametres->pb_w/5 ; i += 2)
    {
      points[i]   = parametres->pb_x + i * 5 + 5;
      points[i+1] = parametres->pb_y + parametres->pb_h/2;
    }
    v_pmarker(_PRG_station->handle, i/2, points);
  }

  return parametres->pb_currstate; /* tat courant du bouton */	
}

/* ~~~~~~~~~~~~~ Affichage d'une fonte (criture de son nom avec elle) ~~~~~~~~~~~ \
\   (paramtre interne  l'objet usr_parm : index GDOS de la fonte  utiliser     */

int cdecl USR_fonte(USR_FONCTION *adresse)
{
	PARMBLK *parametres;
	char nom_fonte[33];
	int  num_fonte;
	int  points[8];

	parametres = CONVERT_PARMBLK adresse; /* Bizarre, mais c'est d  la lib. du GCC ! */

	num_fonte = vqt_name(_PRG_station->handle,(int) ( parametres->pb_parm),nom_fonte);
	points[0] = parametres->pb_xc;      /* Zone  effectivement dessiner */
	points[1] = parametres->pb_yc;
	points[2] = parametres->pb_xc + parametres->pb_wc - 1;
	points[3] = parametres->pb_yc + parametres->pb_hc - 1;
	vs_clip(_PRG_station->handle,1,points);

	vst_font(_PRG_station->handle,num_fonte);
	vst_alignment(_PRG_station->handle, ALI_LEFT, ALI_BOTTOM, &_rien_, &_rien_);
	v_gtext(_PRG_station->handle, parametres->pb_x+5, parametres->pb_y + parametres->pb_h - 4,nom_fonte);

	return parametres->pb_currstate; /* tat courant du bouton */	
}

/***************** Dfinition commune  toutes les fonctions des arbres ************
 *                                                                                 *
 * boolean GER_xxx(int num_objet, int x_souris, int y_souris, int etat)            *
 * Gestion de l'arbre en rponse  un clic
 *   Renvoie : -> FAUX : rien de spcial
 *
 * num_objet : numro de l'objet cliqu
 * x_souris  : X de la souris lors du clic
 * y_souris  : Y de la souris lors du clic
 * etat      : tat des touches spciales lors du clic
 *
 * boolean CLV_xxx(
 * Gestion de l'arbre en rponse  l'action du clavier
 *   Renvoie :
 *
 ***********************************************************************************/

/* ~~~~~~~~~~~~~~ Paramtrages divers sur le graphe courant ~~~~~~~~~~~~~~~~~~~~~~ */

void refait_liste(void)
{
	Graphe *graphe;
	int i;

	graphe = liste[0];
	for (i = 0 ; i < 14 ; i++)
	{
		liste[i] = graphe;
		if (graphe)
			graphe = graphe->suivant;
	}
}

/* -------------- Choix du graphe courant, des couleurs et styles,... ------------ */

boolean GER_graphes(int num_objet, int x_souris, int y_souris)
{
  OBJECT *adresse = arbres[CHOIX_GRAPHE].adresse;
  int nouv_objet, i;
  Graphe *compteur;
  double *inverser,echange;
  unsigned long num_graphe = 0; /* Numro du premier graphe visible... */

  if (premier_graphe == NULL) /* Aucun graphe -> cette boite est sans intret */
    return VRAI;

  /* Numro du premier graphe affich */
  for (compteur = premier_graphe ; compteur != liste[0] ; compteur = compteur->suivant)
    num_graphe++;
  
  if ( (num_objet >= GRAPHE_1) && (num_objet <= GRAPHE_14) )
  {
    if (liste[num_objet - GRAPHE_1] == NULL)
      return FAUX; /* Clic l o il n'y a pas de graphe */

    if ( (_etat & K_RSHIFT) || (_etat & K_LSHIFT) ) /* Shift-clic : change aspect */
    {
      liste[num_objet - GRAPHE_1]->trace = !(liste[num_objet - GRAPHE_1]->trace);
      if ( liste[num_objet - GRAPHE_1]->trace == VRAI )
	OB_STATE(adresse,num_objet) |= CHECKED;
      else
	OB_STATE(adresse,num_objet) &= ~CHECKED;
      dessine_objet(CHOIX_GRAPHE,num_objet);

      return FAUX; /* Rien  changer : on ne fait que modifier visible/invisible */
    }
				
    /* On enregistre les changements du graphe courant */
		
    GRF_titre  (graphe_courant, lit_chaine(CHOIX_GRAPHE, GR_NOM, 0, NULL, TXT_REMPLACE) );
    GRF_legende(graphe_courant, AXE_X, lit_chaine(CHOIX_GRAPHE, LEGENDE_X, 0, NULL, TXT_REMPLACE), graphe_courant->unit_x);
    GRF_legende(graphe_courant, AXE_Y, lit_chaine(CHOIX_GRAPHE, LEGENDE_Y, 0, NULL, TXT_REMPLACE), graphe_courant->unit_y);

    GRF_aller( liste[num_objet-GRAPHE_1] );

    return FAUX;
  }
	
  switch (num_objet)
  {
  case GRAPHE_AVANT:
    for (i = 13 ; i > 0 ; i--)
    {
      liste[i] = liste[i - 1];
    }
    liste[0] = liste[0]->precedent;
    prep_geregraphe(FAUX);
    if ( nbr_graphes > 14 )
    {
      float pos_asc;

      num_graphe--; /* Premier graphe : celui d'avant */
      
      pos_asc  = OB_HEIGHT(adresse,GRAPHE_FOND) - OB_HEIGHT(adresse,GRAPHE_BOUTON);
      pos_asc /= nbr_graphes - 14;
      pos_asc *= num_graphe;
      OBJ_position(CHOIX_GRAPHE, GRAPHE_BOUTON, -1, (int) pos_asc, &_rien_, &_rien_);
      dessine_objet(CHOIX_GRAPHE, GRAPHE_FOND);
    }
    dessine_objet(CHOIX_GRAPHE, 1);
    dessine_objet(CHOIX_GRAPHE, GRAPHE_APRES);
    dessine_objet(CHOIX_GRAPHE, GRAPHE_AVANT);
    break;

  case GRAPHE_FOND:
  {
    float position; int x, y;
    
    objc_offset(adresse, GRAPHE_FOND, &x, &y);
    y_souris -= y;
    if ( y_souris < OB_Y(adresse, GRAPHE_BOUTON) ) /* Clic _avant_ le curseur */
    {
      if (num_graphe > 14)
      {
	for (x = 0 ; x < 14 ; x++, num_graphe--)
	  liste[0] = liste[0]->precedent;
	OBJ_active(CHOIX_GRAPHE, GRAPHE_AVANT, FORCE_OUI);
      }
      else
      {
	liste[0] = premier_graphe;
	num_graphe = 0;
	OBJ_active(CHOIX_GRAPHE, GRAPHE_AVANT, FORCE_NON);
      }
    }
    else
    {
      OB_STATE(adresse,GRAPHE_APRES) &= ~DISABLED;
      for (x = 0 ; x < 14 ; x++, num_graphe++)
      {
	if (liste[13]->suivant == NULL)
	{
	  OB_STATE(adresse,GRAPHE_APRES) |= DISABLED;
	  break;
	}
	liste[0] = liste[0]->suivant; /* On remonte de 14 graphes */
	refait_liste();
      }
    }

    position  = OB_HEIGHT(adresse,GRAPHE_FOND) - OB_HEIGHT(adresse,GRAPHE_BOUTON);
    position /= nbr_graphes - 14;
    position *= num_graphe;
    OBJ_position(CHOIX_GRAPHE, GRAPHE_BOUTON, -1, (int) position, &_rien_, &_rien_);
    /* OB_Y(adresse,GRAPHE_BOUTON) = (int) (position); */

    refait_liste();

    prep_geregraphe(FAUX);  /* Pas besoin de refaire la liste, juste son affichage */
    dessine_objet(CHOIX_GRAPHE,1);
    dessine_objet(CHOIX_GRAPHE,GRAPHE_APRES);
    dessine_objet(CHOIX_GRAPHE,GRAPHE_AVANT);
    dessine_objet(CHOIX_GRAPHE,GRAPHE_FOND);
  }
  break;

  case GRAPHE_BOUTON: /* On fait glisser l'ascenseur */
  {
    float position;

    position  = graf_slidebox(adresse, GRAPHE_FOND, GRAPHE_BOUTON, 1); /* Position en pour milles */
    OBJ_active(CHOIX_GRAPHE, GRAPHE_AVANT, position > 0);
    OBJ_active(CHOIX_GRAPHE, GRAPHE_APRES, position < 1000);
		
    position *= nbr_graphes - 14;
    position /= 1000;
	
    liste[0] = premier_graphe;
    for (num_graphe = 0 ; num_graphe < position ; num_graphe++)
    {
      liste[0] = liste[0]->suivant;
    }

    position  = OB_HEIGHT(adresse,GRAPHE_FOND) - OB_HEIGHT(adresse,GRAPHE_BOUTON);
    position /= nbr_graphes - 14;
    position *= num_graphe;
    OBJ_position(CHOIX_GRAPHE, GRAPHE_BOUTON, -1, (int) position, &_rien_, &_rien_);

    refait_liste();

    prep_geregraphe(FAUX);  /* Pas besoin de refaire la liste, juste son affichage */
    dessine_objet(CHOIX_GRAPHE,1);
/*			dessine_objet(CHOIX_GRAPHE,GRAPHE_APRES);
			dessine_objet(CHOIX_GRAPHE,GRAPHE_AVANT); */
    dessine_objet(CHOIX_GRAPHE,GRAPHE_FOND);
  }
  break;

  case GRAPHE_APRES:
    for (i = 0 ; i < 13 ; i++)
    {
      liste[i] = liste[i + 1];
    }
    liste[13] = liste[13]->suivant;
    prep_geregraphe(FAUX);
    if ( nbr_graphes > 14 )
    {
      int nv_y; /* Nouvelle position de l'ascenseur */
      float pos_asc;
      num_graphe++;

      pos_asc  = OB_HEIGHT(adresse,GRAPHE_FOND) - OB_HEIGHT(adresse,GRAPHE_BOUTON);
      pos_asc /= nbr_graphes - 14;
      pos_asc *= num_graphe;
      OBJ_position(CHOIX_GRAPHE, GRAPHE_BOUTON, -1, (int) pos_asc, &_rien_, &nv_y);
      /*      OB_Y(adresse,GRAPHE_BOUTON) = (int) (echange); */
      if ( nv_y /*OB_Y(adresse, GRAPHE_BOUTON)*/ >  OB_HEIGHT(adresse,GRAPHE_FOND) - OB_HEIGHT(adresse,GRAPHE_BOUTON) )
      {
	OBJ_position(CHOIX_GRAPHE, GRAPHE_BOUTON, -1, OB_HEIGHT(adresse,GRAPHE_FOND) - OB_HEIGHT(adresse,GRAPHE_BOUTON), &_rien_, &_rien_);
      }
      dessine_objet(CHOIX_GRAPHE,GRAPHE_FOND);
    }
    dessine_objet(CHOIX_GRAPHE,1);
    dessine_objet(CHOIX_GRAPHE,GRAPHE_APRES);
    dessine_objet(CHOIX_GRAPHE,GRAPHE_AVANT);
    break;

  case COULEUR_GRAPHE: /* Choix de la couleur du trac */
    objc_offset(adresse, COULEUR_GRAPHE, &x_souris, &y_souris);
    y_souris -= (graphe_courant->couleur - 1) * OB_HEIGHT(arbres[CHX_COULEUR].adresse, 1);
    nouv_objet = GER_ssmenu(CHX_COULEUR, 0, x_souris, y_souris);
    OB_SPEC(adresse, COULEUR_GRAPHE) &= 0xFFFFFFF0L; /* On met les 4 derniers bits  0 */
    OB_SPEC(adresse, COULEUR_GRAPHE) |= nouv_objet;
    dessine_objet(CHOIX_GRAPHE, COULEUR_GRAPHE);
    graphe_courant->couleur = nouv_objet;
    break;

  case LIGNE_GRAPHE:   /* Choix du style de ligne     */
    objc_offset(adresse, LIGNE_GRAPHE, &x_souris, &y_souris);
    y_souris -= (graphe_courant->style) * OB_HEIGHT(arbres[CHOIX_STYLE].adresse, 1);
    nouv_objet = GER_ssmenu(CHOIX_STYLE, 0, x_souris, y_souris) - 1;
    graphe_courant->style = nouv_objet;
    ( (USERBLK *) OB_SPEC(adresse, LIGNE_GRAPHE))->ub_parm = nouv_objet;
    dessine_objet(CHOIX_GRAPHE, LIGNE_GRAPHE);
    break;

  case GR_APPLIQUER:
    GRF_titre  (graphe_courant, lit_chaine(CHOIX_GRAPHE, GR_NOM, 0, NULL, TXT_REMPLACE) );
    GRF_legende(graphe_courant, AXE_X, lit_chaine(CHOIX_GRAPHE, LEGENDE_X, 0, NULL, TXT_REMPLACE), graphe_courant->unit_x);
    GRF_legende(graphe_courant, AXE_Y, lit_chaine(CHOIX_GRAPHE, LEGENDE_Y, 0, NULL, TXT_REMPLACE), graphe_courant->unit_y);

		FEN_titre(principale, graphe_courant->nom);

		prep_geregraphe(VRAI); /* On refait l'ascenseur : titre modifi */
		FEN_redessin(principale);
		FEN_redessin(trace_fenetre);
		dessine_objet(CHOIX_GRAPHE, GRAPHE_1 - 1);
		break;
	
	case EFFACER_GRAPHE:
		GRF_efface(graphe_courant, FAUX, FAUX);
		break;
	
	case INVERSER_GRAPHE:
		inverser = graphe_courant->x;
		graphe_courant->x = graphe_courant->y;
		graphe_courant->y = inverser;
		echange = graphe_courant->x_min;
		graphe_courant->x_min = graphe_courant->y_min;
		graphe_courant->y_min = echange;
		echange = graphe_courant->x_max;
		graphe_courant->x_max = graphe_courant->y_max;
		graphe_courant->y_max = echange;

		GRF_aller(graphe_courant);
/*		FEN_redessin(principale); */
		break;

	case COPIE_GRAPHE:
		copier_graphe(graphe_courant);
		break;

	case GRF_NORMALISE: /* Normaliser le graphe slectionn */
	  i = form_alert(1,"[2][ Normaliser le spectre : | - extrema  0 et 1 | - sur un point ][ Extrema | Point ]");
	  if (i == 1) /* Normalisation sur un point */
	  {
	    compteur = normalise(graphe_courant);
	  }
	  else
	  {
	    form_alert(1,"[2][ Dsol, pas encore ralis ][ Tant pis ]");
	  }
	  if (compteur != NULL)
	    GRF_aller(compteur);
	  break;

	case GRF_LISSER:    /* Lisser le graphe slectionn (pour l'instant, impossible si complexe) */
	  compteur = lisse_graphe(graphe_courant);
		if (compteur != NULL)
			GRF_aller(compteur);
		break;

	case GRF_DECOMPOSE: /* Dcomposition d'un graphe complexe */
		CMP_decompose(graphe_courant);
		break;

	case GRF_TRIER:     /* Tri du graphe courant, par ordre croissant, + traitement des doublons */
		tri_graphe(graphe_courant);
		break;

	case GRF_TS_TRACER: /* Tous les graphes marqus comme devant tre trac */
		for (compteur = premier_graphe ; compteur ; compteur = compteur->suivant)
			compteur->trace = VRAI; /* On force  tracer */
		prep_geregraphe(FAUX);
		break;

	case GRF_TRACER_AUCUN: /* Marquer tous les graphes comme invisibles */
		for (compteur = premier_graphe ; compteur ; compteur = compteur->suivant)
			compteur->trace = FAUX;
		prep_geregraphe(FAUX);
		break;
	}

	return FAUX;
}

/* ~~~~~~~~~~ Tri d'un graphe ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

boolean GER_tri(int num_objet, int x, int y)
{
	OBJECT *adresse = arbres[ORDONNE_GRAPHE].adresse;

	switch (num_objet)
	{
	case TRI_ALEATOIRE:
		OB_STATE(adresse,TRI_NBR_BLOCS) |= DISABLED;
		form_button(adresse, num_objet, 1, &y);
		arbres[ORDONNE_GRAPHE].premier_edit = TRI_GROUPE_PARAM;
		if ( arbres[ORDONNE_GRAPHE].edit_courant == TRI_ALEATOIRE )
		{
			x = arbres[ORDONNE_GRAPHE].pos_curseur;
			Objc_edit(adresse, TRI_NBR_BLOCS, 0, x, ED_END, &x); /* Plus de curseur */
			arbres[ORDONNE_GRAPHE].edit_courant = TRI_GROUPE_PARAM;
			Objc_edit(adresse, TRI_GROUPE_PARAM, 0, x, ED_INIT, &x);
			arbres[ORDONNE_GRAPHE].pos_curseur = x;
		}
		dessine_objet(ORDONNE_GRAPHE,TRI_NBR_BLOCS);
		break;

	case TRI_BLOCS:
		OB_STATE(adresse,TRI_NBR_BLOCS) &= ~DISABLED;
		form_button(adresse, num_objet, 1, &y);
		arbres[ORDONNE_GRAPHE].premier_edit = TRI_NBR_BLOCS;
		dessine_objet(ORDONNE_GRAPHE,TRI_NBR_BLOCS);
		break;

	case TRI_TRI_OK:     /* Activation des diffrentes options */
	case TRI_DOUBLON_OK:
	case TRI_DOUBLES_OK:
	case TRI_GROUPER_OK:
		if ( OB_STATE(adresse, num_objet) & CHECKED )
			OB_STATE(adresse, num_objet) &= ~CHECKED;
		else
			OB_STATE(adresse, num_objet) |= CHECKED;
		dessine_objet(ORDONNE_GRAPHE, num_objet);
		break;

	default:
		form_button(adresse, num_objet, 1, &y);
	}

	return VRAI; /* Trait -> rien  faire */
}

/* ~~~~~~~~~~ Fusion de graphes ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

typedef struct inf_asc
{
  unsigned long index_dep;
	boolean *a_fondre;
	Graphe *prem_graphe;
} inf_asc;

boolean GER_fusion(int num_objet, int x_souris, int y_souris)
{
	OBJECT *adresse = arbres[FUSION_GRAPHES].adresse;
	inf_asc *infos_ascenseur;
	int nouvel_objet;
	unsigned long index;
	float pos_asc;
	Graphe *graphe;

	infos_ascenseur = (inf_asc *) (arbres[FUSION_GRAPHES].pointeur);

	if ( (num_objet >= FUSION_GR1) && (num_objet <= FUSION_GR11) )
	{
		form_button(adresse,num_objet,1,&nouvel_objet);
		index = infos_ascenseur->index_dep + num_objet - FUSION_GR1;

		if (OB_STATE(adresse,num_objet) & SELECTED)
		{
			(infos_ascenseur->a_fondre)[index] = VRAI;
		}
		else
		{
			(infos_ascenseur->a_fondre)[index] = FAUX;
		}
		return FAUX; /* Traitement termin */
	}

	switch (num_objet)
	{
	case FUSION_AVANT:
		if ( infos_ascenseur->index_dep ) /* Il y a des graphes avant */
		{
			infos_ascenseur->prem_graphe = infos_ascenseur->prem_graphe->precedent;
			infos_ascenseur->index_dep--; /* Le premier graphe est celui d'avant */
			/* On dcale les 10 derniers graphes vers le bas */
			for (index = 10 ; index > 0 ; index--)
			{
				OB_STATE(adresse,FUSION_GR1 + index) = OB_STATE(adresse,FUSION_GR1 + index - 1);
				strncpy( (char *) OB_SPEC(adresse,FUSION_GR1 + index),
				         (char *) OB_SPEC(adresse,FUSION_GR1 + index - 1),50);
			}
			if (infos_ascenseur->a_fondre[infos_ascenseur->index_dep] == VRAI)
				OB_STATE(adresse,FUSION_GR1) |= SELECTED;
			else
				OB_STATE(adresse,FUSION_GR1) &= ~SELECTED;
			strncpy( (char *) OB_SPEC(adresse,FUSION_GR1), infos_ascenseur->prem_graphe->nom, 50);

			pos_asc  = infos_ascenseur->index_dep;
			pos_asc *= OB_HEIGHT(adresse,FUSION_FOND) - OB_HEIGHT(adresse,FUSION_BOUTON);
			pos_asc /= nbr_graphes - 11; /* 11 graphes visibles */
			OB_Y(adresse,FUSION_BOUTON) = (int) (pos_asc);

			if ( infos_ascenseur->index_dep == 0 ) /* Premier graphe en haut */
			{
				OB_STATE(adresse,FUSION_AVANT) |= DISABLED;
				dessine_objet(FUSION_GRAPHES,FUSION_AVANT);
			}
			OB_STATE(adresse,FUSION_APRES) &= ~DISABLED;
			dessine_objet(FUSION_GRAPHES,5);
			dessine_objet(FUSION_GRAPHES,FUSION_APRES);
			dessine_objet(FUSION_GRAPHES,FUSION_FOND);
		}
		break;

	case FUSION_APRES:
		if ( infos_ascenseur->index_dep + 11 < nbr_graphes ) /* On n'est pas sur le dernier */
		{
			Graphe *dernier;

			infos_ascenseur->index_dep++; /* Le premier graphe est celui d'aprs */
			dernier = infos_ascenseur->prem_graphe;
			infos_ascenseur->prem_graphe = infos_ascenseur->prem_graphe->suivant;
			for (index = 0 ; index < 11 ; index++)
			{
				OB_STATE(adresse,FUSION_GR1 + index) = OB_STATE(adresse,FUSION_GR1 + index + 1);
				strncpy( (char *) OB_SPEC(adresse,FUSION_GR1 + index),
				         (char *) OB_SPEC(adresse,FUSION_GR1 + index + 1), 50);
				dernier = dernier->suivant;
			}
			if (infos_ascenseur->a_fondre[infos_ascenseur->index_dep + 10] == VRAI)
				OB_STATE(adresse,FUSION_GR11) |= SELECTED;
			else
				OB_STATE(adresse,FUSION_GR11) &= ~SELECTED;
			strncpy( (char *) OB_SPEC(adresse,FUSION_GR11), dernier->nom, 50);

			pos_asc  = infos_ascenseur->index_dep;
			pos_asc *= OB_HEIGHT(adresse,FUSION_FOND) - OB_HEIGHT(adresse,FUSION_BOUTON);
			pos_asc /= nbr_graphes - 11; /* 11 graphes visibles */
			OB_Y(adresse,FUSION_BOUTON) = (int) (pos_asc);

			if ( infos_ascenseur->index_dep + 11 == nbr_graphes )
			{
				OB_STATE(adresse,FUSION_APRES) |= DISABLED;
				dessine_objet(FUSION_GRAPHES,FUSION_APRES);
			}
			OB_STATE(adresse,FUSION_AVANT) &= ~DISABLED;
			dessine_objet(FUSION_GRAPHES,5);
			dessine_objet(FUSION_GRAPHES,FUSION_AVANT);
			dessine_objet(FUSION_GRAPHES,FUSION_FOND);
		}
		break;

	case FUSION_FOND: /* Fond de l'ascenseur */
		objc_offset(adresse,FUSION_FOND,&x_souris,&nouvel_objet);
		y_souris -= nouvel_objet;
		if ( y_souris < OB_Y(adresse,FUSION_FOND) ) /* On descend de 10 graphes */
		{
			OB_STATE(adresse,FUSION_APRES) &= ~DISABLED;
			if ( infos_ascenseur->index_dep > 10 )
			{
				OB_STATE(adresse,FUSION_AVANT) &= ~DISABLED;
				infos_ascenseur->index_dep -= 10;
			}
			else
			{
				OB_STATE(adresse,FUSION_AVANT) |= DISABLED;
				infos_ascenseur->index_dep = 0;
			}
		}
		else /* On monte de 10 graphes */
		{
			OB_STATE(adresse,FUSION_AVANT) &= ~DISABLED;
			if (infos_ascenseur->index_dep + 21 < nbr_graphes ) /* Assez pour descendre de 10 graphes */
			{
				OB_STATE(adresse,FUSION_APRES) &= ~DISABLED;
				infos_ascenseur->index_dep += 10;
			}
			else
			{
				OB_STATE(adresse,FUSION_APRES) |= DISABLED;
				infos_ascenseur->index_dep = nbr_graphes - 11;
			}
		}

		/* On reconstruit la liste des affichables */
		infos_ascenseur->prem_graphe = premier_graphe;
		for (index = 0 ; index < infos_ascenseur->index_dep ; index++)
			infos_ascenseur->prem_graphe = infos_ascenseur->prem_graphe->suivant;

		graphe = infos_ascenseur->prem_graphe;
		for (index = 0 ; index < 11 ; index++)
		{
			if (infos_ascenseur->a_fondre[infos_ascenseur->index_dep + index] == VRAI)
				OB_STATE(adresse, FUSION_GR1 + index) |= SELECTED;
			else
				OB_STATE(adresse, FUSION_GR1 + index) &= ~SELECTED;

			strncpy( (char *) OB_SPEC(adresse,FUSION_GR1 + index),
			         graphe->nom, 50);

			graphe = graphe->suivant;
		}

		/* On positionne l'ascenseur */
		pos_asc  = infos_ascenseur->index_dep;
		pos_asc *= OB_HEIGHT(adresse,FUSION_FOND) - OB_HEIGHT(adresse,FUSION_BOUTON);
		pos_asc /= nbr_graphes - 11; /* 11 graphes visibles */
		OB_Y(adresse,FUSION_BOUTON) = (int) (pos_asc);

		/* On redessine */
		dessine_objet(FUSION_GRAPHES,5);
		dessine_objet(FUSION_GRAPHES,FUSION_AVANT);
		dessine_objet(FUSION_GRAPHES,FUSION_FOND);
		dessine_objet(FUSION_GRAPHES,FUSION_APRES);
		break;

	case FUSION_BOUTON: /* On fait glisser l'ascenseur */
		pos_asc  = graf_slidebox(adresse,FUSION_FOND,FUSION_BOUTON,1);
		pos_asc *= nbr_graphes - 11;
		pos_asc /= 1000;

		infos_ascenseur->index_dep = (int) pos_asc;
		
		/* On reconstruit la liste des affichables */
		infos_ascenseur->prem_graphe = premier_graphe;
		for (index = 0 ; index < infos_ascenseur->index_dep ; index++)
			infos_ascenseur->prem_graphe = infos_ascenseur->prem_graphe->suivant;

		graphe = infos_ascenseur->prem_graphe;
		for (index = 0 ; index < 11 ; index++)
		{
			if (infos_ascenseur->a_fondre[infos_ascenseur->index_dep + index] == VRAI)
				OB_STATE(adresse,FUSION_GR1 + index) |= SELECTED;
			else
				OB_STATE(adresse,FUSION_GR1 + index) &= ~SELECTED;

			strncpy( (char *) OB_SPEC(adresse,FUSION_GR1 + index),
			         graphe->nom, 50);

			graphe = graphe->suivant;
		}

		/* On positionne l'ascenseur */
		pos_asc  = infos_ascenseur->index_dep;
		pos_asc *= OB_HEIGHT(adresse,FUSION_FOND) - OB_HEIGHT(adresse,FUSION_BOUTON);
		pos_asc /= nbr_graphes - 11; /* 11 graphes visibles */
		OB_Y(adresse,FUSION_BOUTON) = (int) (pos_asc);

		/* On redessine */
		dessine_objet(FUSION_GRAPHES,FUSION_GR1 - 1);
		dessine_objet(FUSION_GRAPHES,FUSION_AVANT);
		dessine_objet(FUSION_GRAPHES,FUSION_FOND);
		dessine_objet(FUSION_GRAPHES,FUSION_APRES);

		break;

	default: /* Les radio-boutons d'en bas */
		form_button(adresse,num_objet,1,&nouvel_objet);
	}

	return FAUX;
}

/* ~~~~~~~~~~ Extraction du signal exprimental depuis un fichier ~~~~~ */

boolean GER_extrait(int num_objet, int x, int y)
{
	boolean etat;
	
	switch (num_objet)
	{
	case TRANSMISSION: /* Type de manipulation */
	case FLUORESCENCE:
		OBJ_choisit(TYPE_LIGNE, num_objet, FORCE_OUI);

		etat  = OBJ_choisit(TYPE_LIGNE, TRANSMISSION, QUESTION);
		etat &= OBJ_choisit(TYPE_LIGNE, MULTI_ELEMENTS, QUESTION);
		etat &= ( OBJ_choisit(TYPE_LIGNE, MULTI_ELEM_MOYEN, QUESTION)
		       || OBJ_choisit(TYPE_LIGNE, MULTI_ELEM_SOMME, QUESTION) ); /* On fait une somme */
		OBJ_active(TYPE_LIGNE, MELM_SIGMA_LOG, etat);
		OBJ_active(TYPE_LIGNE, MELM_LOG_SIGMA, etat);

		etat  = OBJ_choisit(TYPE_LIGNE, TRANSMISSION, QUESTION);
		etat &= OBJ_choisit(TYPE_LIGNE, LURE_2, QUESTION);
		etat &= OBJ_choisit(TYPE_LIGNE, MULTI_SPECTRES, QUESTION);
		etat &= ( OBJ_choisit(TYPE_LIGNE, MSPR_MOYENNE, QUESTION)
		       || OBJ_choisit(TYPE_LIGNE, MSPR_SOMME, QUESTION) ); /* On fait une somme */
		OBJ_active(TYPE_LIGNE, MSPR_SIGMA_LOG, etat);
		OBJ_active(TYPE_LIGNE, MSPR_LOG_SIGMA, etat);
		break;

	case LURE_1: /* Type ligne EXAFS III du LURE */
		OBJ_choisit(TYPE_LIGNE, LURE_1, FORCE_OUI);      /* C'est cette ligne qui compte */

		met_valeur(TYPE_LIGNE, LIGNES_SAUT, 16); /* 16 lignes le plus souvent */
		met_valeur(TYPE_LIGNE,ENERGIE,1);
		met_valeur(TYPE_LIGNE,COLONNE_I0,2);
		met_valeur(TYPE_LIGNE,COLONNE_I1,3);

		OBJ_active(TYPE_LIGNE, MULTI_SPECTRES , FORCE_NON); /* Il ne peut pas y avoir x spectres  la suite */
		OBJ_active(TYPE_LIGNE, NOMBRE_SPECTRES, FORCE_NON);
		OBJ_active(TYPE_LIGNE, MSPR_SOMME  , FORCE_NON);
		OBJ_active(TYPE_LIGNE, MSPR_MOYENNE, FORCE_NON);
		OBJ_active(TYPE_LIGNE, MSPR_SEPARER, FORCE_NON);
		OBJ_active(TYPE_LIGNE, MSPR_FUSION , FORCE_NON);
		OBJ_active(TYPE_LIGNE, MSPR_TOUS     , FORCE_NON);
		OBJ_active(TYPE_LIGNE, MSPR_SELECTION, FORCE_NON);
		OBJ_active(TYPE_LIGNE, MSPR_SIGMA_LOG, FORCE_NON);
		OBJ_active(TYPE_LIGNE, MSPR_LOG_SIGMA, FORCE_NON);
		break;

	case LURE_2: /* Type ligne EXAFS II  du LURE */
		OBJ_choisit(TYPE_LIGNE, LURE_2, FORCE_OUI);      /* C'est cette ligne qui compte */

		met_valeur(TYPE_LIGNE,LIGNES_SAUT,2); /* 2 lignes d'en-tte */
		met_valeur(TYPE_LIGNE,ENERGIE,1);
		met_valeur(TYPE_LIGNE,COLONNE_I0,2);
		met_valeur(TYPE_LIGNE,COLONNE_I1,3);

		OBJ_active(TYPE_LIGNE, MULTI_SPECTRES , FORCE_OUI); /* Il peut y avoir x spectres  la suite */
		etat = OBJ_choisit(TYPE_LIGNE, MULTI_SPECTRES, QUESTION); /* On est effectivement dans ce cas */
		OBJ_active(TYPE_LIGNE, NOMBRE_SPECTRES, etat);
		OBJ_active(TYPE_LIGNE, MSPR_SOMME  , etat);
		OBJ_active(TYPE_LIGNE, MSPR_MOYENNE, etat);
		OBJ_active(TYPE_LIGNE, MSPR_SEPARER, etat);
		OBJ_active(TYPE_LIGNE, MSPR_FUSION , etat);
		OBJ_active(TYPE_LIGNE, MSPR_TOUS     , etat);
		OBJ_active(TYPE_LIGNE, MSPR_SELECTION, etat);
		etat &= OBJ_choisit(TYPE_LIGNE, TRANSMISSION, QUESTION);  /* On est en transmission */
		etat &= ( OBJ_choisit(TYPE_LIGNE, MSPR_MOYENNE, QUESTION) || OBJ_choisit(TYPE_LIGNE, MSPR_SOMME, QUESTION) ); /* On fait une somme */
		OBJ_active(TYPE_LIGNE, MSPR_LOG_SIGMA, etat); /* Si tout cela est vrai : option intressante */
		OBJ_active(TYPE_LIGNE, MSPR_SIGMA_LOG, etat);
		break;

	case ITALIE:
		break;
	case AUTRE_LIGNE:
		break;
	
	case MULTI_ELEMENTS:
		etat = OBJ_choisit(TYPE_LIGNE,MULTI_ELEMENTS , ECHANGE);
		OBJ_active(TYPE_LIGNE, NOMBRE_ELEMENTS, etat);
		OBJ_active(TYPE_LIGNE, MULTI_ELEM_SOMME, etat);
		OBJ_active(TYPE_LIGNE, MULTI_ELEM_MOYEN, etat);
		OBJ_active(TYPE_LIGNE, MULTI_ELEM_SEPAR, etat);
		OBJ_active(TYPE_LIGNE, MULTI_ELEM_TOUS , etat);
		OBJ_active(TYPE_LIGNE, MULTI_ELEM_CHOIX, etat);
		etat &= OBJ_choisit(TYPE_LIGNE, TRANSMISSION, QUESTION);  /* On est en transmission */
		etat &= ( OBJ_choisit(TYPE_LIGNE, MULTI_ELEM_MOYEN, QUESTION)
		       || OBJ_choisit(TYPE_LIGNE, MULTI_ELEM_SOMME, QUESTION) ); /* On fait une somme */
		OBJ_active(TYPE_LIGNE, MELM_SIGMA_LOG, etat); /* Si tout est vrai : option judicieuse */
		OBJ_active(TYPE_LIGNE, MELM_LOG_SIGMA, etat);
		break;

	case MULTI_ELEM_SOMME: /* Diffrentes faons de traiter la chose */
	case MULTI_ELEM_MOYEN:
	case MULTI_ELEM_SEPAR:
		OBJ_choisit(TYPE_LIGNE, num_objet, FORCE_OUI); /* On active celui que l'on a cliqu */
		etat = ! ( OBJ_choisit(TYPE_LIGNE, MULTI_ELEM_SEPAR, QUESTION) ); /* Si on ne spare pas */
		OBJ_active(TYPE_LIGNE, MULTI_ELEM_TOUS, etat);
		OBJ_active(TYPE_LIGNE, MULTI_ELEM_CHOIX, etat);
		etat &= OBJ_choisit(TYPE_LIGNE, TRANSMISSION, QUESTION); /* et si on est en transmission */
		OBJ_active(TYPE_LIGNE, MELM_SIGMA_LOG, etat); /* Sans intrt si on spare */
		OBJ_active(TYPE_LIGNE, MELM_LOG_SIGMA, etat);
		break;

	case MULTI_SPECTRES:
		etat = OBJ_choisit(TYPE_LIGNE, MULTI_SPECTRES, ECHANGE);
		OBJ_active(TYPE_LIGNE, NOMBRE_SPECTRES, etat);
		OBJ_active(TYPE_LIGNE, MSPR_SOMME  , etat);
		OBJ_active(TYPE_LIGNE, MSPR_MOYENNE, etat);
		OBJ_active(TYPE_LIGNE, MSPR_SEPARER, etat);
		OBJ_active(TYPE_LIGNE, MSPR_FUSION , etat);
		OBJ_active(TYPE_LIGNE, MSPR_TOUS     , etat);
		OBJ_active(TYPE_LIGNE, MSPR_SELECTION, etat);
		etat &= OBJ_choisit(TYPE_LIGNE, TRANSMISSION, QUESTION);  /* On est en transmission */
		etat &= ( OBJ_choisit(TYPE_LIGNE, MSPR_MOYENNE, QUESTION) || OBJ_choisit(TYPE_LIGNE, MSPR_SOMME, QUESTION) ); /* On fait une somme */
		OBJ_active(TYPE_LIGNE, MSPR_LOG_SIGMA, etat); /* Si tout cela est vrai : option intressante */
		OBJ_active(TYPE_LIGNE, MSPR_SIGMA_LOG, etat);
		break;

	case MSPR_SOMME:
	case MSPR_MOYENNE:
	case MSPR_SEPARER:
	case MSPR_FUSION:
		OBJ_choisit(TYPE_LIGNE, num_objet, FORCE_OUI);
		etat = ! OBJ_choisit(TYPE_LIGNE, MSPR_SEPARER, QUESTION);
		OBJ_active(TYPE_LIGNE, MSPR_TOUS, etat); /* Sans intrt si on spare */
		OBJ_active(TYPE_LIGNE, MSPR_SELECTION, etat);
		etat &= ! OBJ_choisit(TYPE_LIGNE, MSPR_FUSION, QUESTION);   /* ni fusion, ni sparation */
		etat &=   OBJ_choisit(TYPE_LIGNE, TRANSMISSION, QUESTION);  /* transmission */
		/* Si on reoit a, les boutons sont actifs -> pas besoin de vrifier le reste */
		OBJ_active(TYPE_LIGNE, MSPR_LOG_SIGMA, etat); /* Si tout cela est vrai : option intressante */
		OBJ_active(TYPE_LIGNE, MSPR_SIGMA_LOG, etat);
		break;			

	case LGN_2_CHAMBRES:
		etat = OBJ_choisit(TYPE_LIGNE, LGN_2_CHAMBRES, ECHANGE);
		OBJ_active(TYPE_LIGNE, LGN_GARDE_I2, etat);
		break;

	case EXTRAIT_OK:
	case EXTRAIT_ANNULER:
		return VRAI; /* Dialogue termin */

	case MULTI_ELEM_TOUS:
	case MULTI_ELEM_CHOIX:
	case MSPR_TOUS:
	case MSPR_SELECTION:
		OBJ_choisit(TYPE_LIGNE, num_objet, FORCE_OUI); /* Boutons radio... */
		return VRAI;

	default:
		OBJ_choisit(TYPE_LIGNE, num_objet, ECHANGE); /* Clic change l'tat, par dfaut */
		break;
	}
	
	return VRAI;
}

/* --------------- Paramtrage du calcul de dispersion standard -------------------------- */

boolean GER_dispersion(int num_objet, int x, int y)
{
	DISP_infos *infos;

	infos = arbres[BARRE_ERREUR].pointeur; /* On rcupre les infos dj existentes */

	if ( num_objet == BERR_AJOUTER ) /* On ajoute un spectre  la liste */
	{
		unsigned long i;
		Graphe *choisi, *index = NULL;

		if (infos->nbr_choisis == 0) /* Aucun spectre choisi */
		{
			choisi = GRF_choisir(GR_TOUS); /* Tous les types visibles */
			if (choisi == NULL)
				return VRAI; /* On n'a rien choisi */
		}
		else
		{
			int type = GR_AUCUN;
			unsigned long i;

			for ( i = 0, choisi = premier_graphe ; choisi ; choisi = choisi->suivant, i++)
			{
				if (infos->liste_graphes[i] == VRAI) /* Graphe de la liste */
				{
					type = choisi->type; /* Donc on prend ce type */
					if ( (type != GR_SSTYPE) && (type != GR_INTERNE) ) /* Rellement un type */
						break; /* On s'arrte donc l */
				}
			}

			if ( type == GR_AUCUN ) /* Aucun graphe ne convient !!! */
			{
				EXF_ERREUR(ERR_STRUCT_GRF);

				type = GR_TOUS;
				infos->nbr_choisis = 0; /* Il y a eu un problme quelque part */
			}

			choisi = GRF_choisir(type);
			if (choisi == NULL)
				return VRAI; /* Aucun graphe choisi : on ne fait rien */
		}

		for ( i = 0, index = premier_graphe ;
		     	choisi != index ; index = index->suivant, i++ ); /* Suppose qu'il y a bien un choisi dans le chanage des cadres... */
		if (infos->liste_graphes[i] == FAUX) /* Le graphe n'tait pas slectionn */
		{
			infos->nbr_choisis++; /* Un graphe de plus */
			infos->liste_graphes[i] = VRAI; /* Graphe slectionn */

			DISP_prep_asc(); /* On refait l'ascenseur */
		}
		else
			EXF_ERREUR(GRF_DEJA_PRESENT);

		return VRAI;
	}

	if ( num_objet == BERR_EFFACER ) /* On efface le graphe actuellement slectionn */
	{
		unsigned long i, j = 0;
		unsigned long num_graphe = infos->prem_affiche + infos->num_choisi + 1; /* 1...n */

		for (i = 0 ; i < nbr_graphes ; i++)
		{
			if (infos->liste_graphes[i] == VRAI)
			{
				j++;
				if (j == num_graphe) /* C'est que le graphe i est le bon */
					break;
			}
		}

		infos->liste_graphes[i] = FAUX; /* Le graphe n'est plus dans la liste */
		infos->nbr_choisis--;           /* donc un graphe en moins            */
		infos->num_choisi = -1; /* et plus de graphe choisi */
		DISP_prep_asc();
		return VRAI;
	}

	if ( num_objet == BERR_TT_AJOUTER ) /* On ajoute _tous_ les spectres de mme type */
	{
		Graphe *choisi;
		int type = GR_AUCUN; /* Le type qu'il faut utiliser */
		unsigned long i;

		for ( i = 0, choisi = premier_graphe ; choisi ; choisi = choisi->suivant, i++)
		{
			if (infos->liste_graphes[i] == VRAI) /* Graphe de la liste */
			{
				type = choisi->type; /* Donc on prend ce type */
				if ( (type != GR_SSTYPE) && (type != GR_INTERNE) ) /* Rellement un type */
					break; /* On s'arrte donc l */
			}
		}
		if ( type == GR_AUCUN ) /* Option sans intrt */
			return VRAI;

		for (i = 0, choisi = premier_graphe ; choisi ; choisi = choisi->suivant, i++)
		{
			if ( (choisi->type == type)              /* Graphe de mme type _exactement_ */
			  && (infos->liste_graphes[i] == FAUX) ) /* mais pas encore dans la liste    */
			{
				infos->liste_graphes[i] = VRAI;
				infos->nbr_choisis++;
			}
		}
		DISP_prep_asc();
		return VRAI;
	}

	if ( num_objet == BERR_TT_EFFACER ) /* On efface _tous_ les graphes de la liste */
	{
		unsigned long i;
		for (i = 0 ; i < nbr_graphes ; i++)
			infos->liste_graphes[i] = FAUX;
		infos->nbr_choisis = 0;
		infos->num_choisi = -1;

		DISP_prep_asc();
		return VRAI;
	}

	if ( (num_objet >= BERR_GRF_PREMNOM) && (num_objet <= BERR_GRF_DER_NOM) ) /* Clic sur un graphe */
	{
		DISP_gere_asc(num_objet, x, y);
		return VRAI;
	}

	switch (num_objet)
	{
		boolean etat;

	case BERR_GRF_AVANT:
	case BERR_GRF_ASC_FND: /* Tout ce qui concerne l'ascenseur */
	case BERR_GRF_ASC_BOU:
	case BERR_GRF_APRES:
		DISP_gere_asc(num_objet, x, y);
		break;

	case BERR_X_IGNORE:
	case BERR_X_MOYENNE:
	case BERR_S_MOYEN:
	case BERR_GARDE_MOYEN: /* Tout a, c'est de la gestion classique de base */
		OBJ_choisit(BARRE_ERREUR, num_objet, ECHANGE);
		break;

	case BERR_MUL_SIGMA: /* a aussi, presque */
		etat = OBJ_choisit(BARRE_ERREUR, BERR_MUL_SIGMA, ECHANGE);
		OBJ_active(BARRE_ERREUR, BERR_MUL_SIGMA_V, etat);
		OBJ_active(BARRE_ERREUR, BERR_MUL_SIGMA_S, etat);
		break;
	}

	return VRAI; /* On a trait tout comme il faut */
}

/* ~~~~~~~~~~~~~~~ Arbres en fentres mixtes (travail sur un spectre) ~~~~~~~~~~~~~~~~~~~~ *
 *  Si appelle, c'est qu'on est en premier plan => redessin possible                      */

/* --------------- Intgration du prseuil d'un graphe ----------------------------------- */

boolean GER_intg(int num_objet, int x, int y)
{
	Fenetre *fenetre; /* Fentre courante               */
	Graphe *graphe;   /* Graphe associ  cette fentre */
	double resultat;
	OBJECT *adresse = arbres[CHOIX_VALEUR].adresse;
	char *chaine;

	fenetre = ARB_fenetre(arbres + INTG_XANES); /* Fentre courante */
	graphe  = (Graphe *) (fenetre->adresse);

	switch (num_objet)
	{
	case INTG_XMIN: /* On demande X min  l'utilisateur */
		chaine = (char *) OB_SPEC(adresse, CHXVAL_UNITE);
		chaine[0] = 0; /* Pas d'unit dfinie pour l'instant */

		chaine = ( (TEDINFO *) OB_SPEC(adresse, CHXVAL_TITRE) )->te_ptext;
		sprintf(chaine, "Valeur de X min");

		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, INTG_min);

		ext_formdo(CHOIX_VALEUR);

		INTG_min = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		met_fvaleur(INTG_XANES,INTG_XMIN,INTG_min);

		resultat = XAN_intg_calcule(graphe);
		met_fvaleur(INTG_XANES, INTG_RESULTAT, resultat);

		FEN_redessin(fenetre); /* On fait redessiner */
		return VRAI;

	case INTG_XMAX:	/* On demande X max  l'utilisateur */
		chaine = (char *) OB_SPEC(adresse, CHXVAL_UNITE);
		chaine[0] = 0; /* Pas d'unit dfinie pour l'instant */

		chaine = ( (TEDINFO *) OB_SPEC(adresse, CHXVAL_TITRE ) )->te_ptext;
		sprintf(chaine, "Valeur de X max");

		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, INTG_max);

		ext_formdo(CHOIX_VALEUR);

		INTG_max = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		met_fvaleur(INTG_XANES,INTG_XMIN,INTG_max);

		resultat = XAN_intg_calcule(graphe);
		met_fvaleur(INTG_XANES, INTG_RESULTAT, resultat);

		FEN_redessin(fenetre); /* On fait redessiner */
	
		return VRAI; /* On a fini */
	}

	return VRAI + 0*(x+y);
}

/* --------------- limination d'une aberrance ------------------------------------------- */

boolean GER_aberrance(int num_objet, int x, int y)
{
	Fenetre *fenetre; /* Fentre courante               */
	ABER_param *param_aber;
	OBJECT *adresse = arbres[ELIM_ABERRANCE].adresse;
	int nouvel_objet;
	char *chaine;

	fenetre = ARB_fenetre(arbres + ELIM_ABERRANCE);
	param_aber = (ABER_param *) fenetre->adresse;

	switch (num_objet)
	{
	case ABER_XMIN1: /* Bornes de l'intervale d'avant */
	case ABER_XMAX1:
	case ABER_XMIN2: /* Bornes de l'intervale d'aprs */
	case ABER_XMAX2:
		chaine = (char *) OB_SPEC(adresse, CHXVAL_UNITE);
		chaine[0] = 0; /* Pas d'unit dfinie pour l'instant */

		if ( num_objet == ABER_XMIN1 )
		{
			met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "X 1", TXT_REMPLACE);
			met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, param_aber->x_borne[0]);
		}
		else if ( num_objet == ABER_XMAX1 )
		{
			met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "X 2", TXT_REMPLACE);
			met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, param_aber->x_borne[1]);
		}
		else if ( num_objet == ABER_XMIN2 )
		{
			met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "X 3", TXT_REMPLACE);
			met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, param_aber->x_borne[2]);
		}
		else
		{
			met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "X 4", TXT_REMPLACE);
			met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, param_aber->x_borne[3]);
		}

		ext_formdo(CHOIX_VALEUR);

		if ( num_objet == ABER_XMIN1 )
			param_aber->x_borne[0] = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		else if ( num_objet == ABER_XMAX1 )
			param_aber->x_borne[1] = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		else if ( num_objet == ABER_XMIN2 )
			param_aber->x_borne[2] = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		else
			param_aber->x_borne[3] = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);

		/* On vrifie l'ordre des bornes, on l'amliore si ncessaire */
		for (x = 0 ; x < 4 ; x++)
			for (y = x + 1 ; y < 4 ; y++)
				if (param_aber->x_borne[y] < param_aber->x_borne[x]) /* On intervertit les deux */
				{
					int vx = param_aber->x_borne[y];
					param_aber->x_borne[y] = param_aber->x_borne[x];
					param_aber->x_borne[x] = vx;
				}

		/* On met les nouvelles valeurs */

		met_fvaleur(ELIM_ABERRANCE, ABER_XMIN1, param_aber->x_borne[0]);
		met_fvaleur(ELIM_ABERRANCE, ABER_XMAX1, param_aber->x_borne[1]);
		met_fvaleur(ELIM_ABERRANCE, ABER_XMIN2, param_aber->x_borne[2]);
		met_fvaleur(ELIM_ABERRANCE, ABER_XMAX2, param_aber->x_borne[3]);

		/* On recommence tout avec ces valeurs */

		FEN_redessin(fenetre);
		return VRAI;

	case ABER_MOINS:  /* Ascenseur de choix du degr */
	case ABER_FOND:
	case ABER_VALEUR:
	case ABER_PLUS:
		param_aber->degre = gere_ascenseur(ELIM_ABERRANCE,ABER_MOINS,1,9,1,FAUX,num_objet,x,y);
		FEN_redessin(fenetre);
		return VRAI;

	case ABER_POLY1: /* Type de courbe d'interpolation */
	case ABER_POLY2:
	case ABER_SPLINE:
		form_button(adresse,num_objet,1,&nouvel_objet);
		if ( OB_STATE(adresse,ABER_SPLINE) & SELECTED )
		{
			OB_STATE(adresse, ABER_MOINS)  |= DISABLED;
			OB_STATE(adresse, ABER_FOND)   |= DISABLED;
			OB_STATE(adresse, ABER_VALEUR) |= DISABLED;
			OB_STATE(adresse, ABER_PLUS)   |= DISABLED;
		}
		else
		{
			OB_STATE(adresse, ABER_MOINS)  &= ~DISABLED;
			OB_STATE(adresse, ABER_FOND)   &= ~DISABLED;
			OB_STATE(adresse, ABER_VALEUR) &= ~DISABLED;
			OB_STATE(adresse, ABER_PLUS)   &= ~DISABLED;
		}

		FEN_redessin(fenetre);
		return VRAI;
	}

	return VRAI; /* On a trait tous les cas intressants */
}

/* --------------- Dcomposition d'un spectre sur une base de 2 spectres ---------------------------------- */

boolean GER_decompose(int num_objet, int x, int y)
{
	Fenetre *fenetre;
	Graphe  *spectre;
	DCMP_infos *infos;

	fenetre = ARB_fenetre(arbres + DECOMP_GRAPHE); /* Fentre actuelle */
	infos   = (DCMP_infos *) fenetre->adresse;
	spectre = (Graphe *) fenetre->pointeur;

	switch (num_objet)
	{
		Graphe *nv_ref;

	case DCMP_SPECTRE_A: /* Choix du premier spectre de la base */
		nv_ref = GRF_choisir(spectre->type); /* Seulement les graphes de mme type... */
		if ( (nv_ref) && (nv_ref != infos->ref_1) )
		{
			double retour;

			free(infos->a_1); free(infos->b_1); free(infos->c_1); /* Ne servent plus */
			retour = SPL_tt_prep(nv_ref, &(infos->a_1), &(infos->b_1), &(infos->c_1) );
			if (retour < 0)
				infos->ref_1 = NULL;
			else
				infos->ref_1 = nv_ref;
		}
		if (infos->ref_1)
			met_chaine(DECOMP_GRAPHE, DCMP_SPECTRE_A, 40, infos->ref_1->nom, TXT_COPIER);
		else
			met_chaine(DECOMP_GRAPHE, DCMP_SPECTRE_A, 40, "Pas de spectre choisi", TXT_COPIER);
		DCMP_calc(spectre, infos); /* On fait le calcul */
		FEN_redessin(fenetre);
		break;

	case DCMP_SPECTRE_B: /* Choix du second spectre de la base */
		nv_ref = GRF_choisir(spectre->type); /* Seulement les graphes de mme type... */
		if ( (nv_ref) && (nv_ref != infos->ref_2) )
		{
			double retour;

			free(infos->a_2); free(infos->b_2); free(infos->c_2); /* Ne servent plus */
			retour = SPL_tt_prep(nv_ref, &(infos->a_2), &(infos->b_2), &(infos->c_2) );
			if (retour < 0)
				infos->ref_2 = NULL;
			else
				infos->ref_2 = nv_ref;
		}
		if (infos->ref_2)
			met_chaine(DECOMP_GRAPHE, DCMP_SPECTRE_B, 40, infos->ref_2->nom, TXT_COPIER);
		else
			met_chaine(DECOMP_GRAPHE, DCMP_SPECTRE_B, 40, "Pas de spectre choisi", TXT_COPIER);
		DCMP_calc(spectre, infos); /* On fait le calcul */
		FEN_redessin(fenetre);
		break;

	case DCMP_XMIN:
		met_chaine (CHOIX_VALEUR, CHXVAL_TITRE, 0,
#ifdef _FRANCAIS_
                "X minimal",
#endif
#ifdef _ENGLISH_
                "Minimal X",
#endif
                TXT_REMPLACE);
		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->x_min);
		ext_formdo(CHOIX_VALEUR);
		infos->x_min = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		if (infos->x_min < spectre->x_min)
			infos->x_min = spectre->x_min;
		if (infos->x_min > infos->x_max)
		{
			double x = infos->x_min; infos->x_min = infos->x_max; infos->x_max = x;
			met_fvaleur(DECOMP_GRAPHE, DCMP_XMAX, infos->x_max);
		}
		met_fvaleur(DECOMP_GRAPHE, DCMP_XMIN, infos->x_min);
		DCMP_calc(spectre, infos); /* On fait le calcul */
		FEN_redessin(fenetre);
		break;

	case DCMP_XMAX:
		met_chaine (CHOIX_VALEUR, CHXVAL_TITRE, 0,
#ifdef _FRANCAIS_
                "X maximal",
#endif
#ifdef _ENGLISH_
                "Maximal X",
#endif
                TXT_REMPLACE);
		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->x_max);
		ext_formdo(CHOIX_VALEUR);
		infos->x_max = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		if (infos->x_max > spectre->x_max)
			infos->x_max = spectre->x_max;
		if (infos->x_min > infos->x_max)
		{
			double x = infos->x_min; infos->x_min = infos->x_max; infos->x_max = x;
			met_fvaleur(DECOMP_GRAPHE, DCMP_XMIN, infos->x_min);
		}
		met_fvaleur(DECOMP_GRAPHE, DCMP_XMAX, infos->x_max);
		DCMP_calc(spectre, infos); /* On fait le calcul */
		FEN_redessin(fenetre);
	}

	return VRAI;
}


/* --------------- Extraction d'un [mu]_0 dj calcul ----------------------------------------- */

boolean GER_mu0(int num_objet, int x_souris, int y_souris)
{
	OBJECT *adresse = arbres[OTE_MU0_CALC].adresse;
	MU0_infos *infos;
	Graphe *brut;
	Fenetre *fenetre;
	char *chaine;

	fenetre = ARB_fenetre(arbres + OTE_MU0_CALC); /* Fentre actuelle */
	infos = (MU0_infos *) fenetre->adresse;
	brut  = (Graphe *)    fenetre->pointeur;

	switch (num_objet)
	{
	case MU0CALC_TXT_E0: /* Demande de la valeur de E0  la main */
	case MU0CALC_E0:
		chaine = (char *) OB_SPEC(arbres[CHOIX_VALEUR].adresse, CHXVAL_UNITE);
		sprintf(chaine, "eV"); /* Unit... */

		met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, 
#ifdef _FRANCAIS_
		           "Valeur de E0", TXT_REMPLACE);
#endif
#ifdef _ENGLISH_
               "E0 value", TXT_REMPLACE);
#endif

		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->e_0);

		ext_formdo(CHOIX_VALEUR);

		infos->e_0 = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		met_fvaleur(OTE_MU0_CALC, MU0CALC_E0, infos->e_0);
		graf_mouse(S_ABEILLE, NULL);
		MU0_extrait(brut, infos);   /* On refait l'extraction */
		graf_mouse(S_FLECHE, NULL);
		FEN_redessin(fenetre);
		return VRAI;

	case MU0CALC_ECH_K:  /* chelle du polynme [mu]_0 utilis */
	case MU0CALC_ECH_EV:
		infos->methode = num_objet;
		form_button(adresse, num_objet, 1, &x_souris);
		graf_mouse(S_ABEILLE, NULL);
		MU0_extrait(brut, infos);   /* On refait l'extraction */
		graf_mouse(S_FLECHE, NULL);
		FEN_redessin(fenetre);
		return VRAI;

	case MU0CALC_NOUVEAU: /* On veut changer de polynme */
  	{
  		Graphe *nv_mu0 = GRF_choisir(GR_MU0);
  		if (nv_mu0 == NULL) return VRAI; /* Rien d'autre  faire */

			graf_mouse(S_ABEILLE, NULL);
			infos->mu_0->drapeaux.protege = FAUX; /* On n'utilise plus celui-ci */
			infos->mu_0 = nv_mu0;
			infos->mu_0->drapeaux.protege = VRAI;
			MU0_prepare(infos);
			MU0_extrait(brut, infos);
			graf_mouse(S_FLECHE, NULL);
			FEN_redessin(fenetre);
		}
		return VRAI;

	case MU0CALC_DEGRE: /* Degr de k^n * khi(k) pour le spectre final */
		{
			int vx_degre = infos->degre;

			chaine = (char *) OB_SPEC(arbres[CHOIX_VALEUR].adresse, CHXVAL_UNITE);
			sprintf(chaine, " "); /* Unit... */

		met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, 
#ifdef _FRANCAIS_
		           "Facteur de pond"e_AIGU"ration", TXT_REMPLACE);
#endif
#ifdef _ENGLISH_
               "Weighting factor", TXT_REMPLACE);
#endif

			met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, (double) (infos->degre) );

			ext_formdo(CHOIX_VALEUR);

			infos->degre = (int) prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
			met_valeur(OTE_MU0_CALC, MU0CALC_DEGRE, infos->degre);
			graf_mouse(S_ABEILLE, NULL);
			EXF_k_p_n(infos->exafs, infos->degre - vx_degre); /* Juste  modifier l'affichage */
			graf_mouse(S_FLECHE, NULL);
			FEN_redessin(fenetre);
		}
	}

	return VRAI + 0 * y_souris; /* Pour viter un bte avertissement  la compilation */
}

/* --------------- Extraction de mu_0, premire tape ------------------------------------------ */

boolean GER_mu01(int num_objet, int x, int y)
{
	OBJECT *adresse = arbres[ELIM_MU0_1].adresse;
	MU1_infos *infos;
	Graphe *brut;
	Fenetre *fenetre;
/*	char *chaine; */

	fenetre = ARB_fenetre(arbres + ELIM_MU0_1); /* Fentre actuelle */
	infos = (MU1_infos *) fenetre->adresse;
	brut  = (Graphe *)    fenetre->pointeur;

	switch (num_objet)
	{
	case ELIM_MU0_RAPIDE: /* Mthode utilise pour approcher l'absorption atomique */
	case ELIM_MU0_LENT:
		if (num_objet != infos->methode) /* On change de mthode */
		{
			form_button(adresse, num_objet, 1, &y);
			infos->methode = num_objet;
			graf_mouse(S_ABEILLE, NULL);
			MU1_calc   (brut, infos); /* On recalcule  */
			MU1_extrait(brut, infos); /* Et on extrait */
			graf_mouse(S_FLECHE, NULL);
			FEN_redessin(fenetre);
		}
		break;

	case VALEUR_E0: /* Valeur  utiliser pour E0 */
		met_chaine(CHOIX_VALEUR, CHXVAL_UNITE, 6, "eV", TXT_COPIER);
		met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, 
#ifdef _FRANCAIS_
		           "Valeur de E0", TXT_REMPLACE);
#endif
#ifdef _ENGLISH_
               "E0 value", TXT_REMPLACE);
#endif
		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->e_0);

		ext_formdo(CHOIX_VALEUR);

		infos->e_0 = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		met_fvaleur(ELIM_MU0_1, VALEUR_E0, infos->e_0);
		graf_mouse(S_ABEILLE, NULL);
		MU1_extrait(brut, infos); /* On refait l'extraction - pas de recalcul du polynome */
		graf_mouse(S_FLECHE, NULL);
		FEN_redessin(fenetre);
		return VRAI;

	case VALEUR_E2: /* Valeur  utiliser pour E2 */
		met_chaine(CHOIX_VALEUR, CHXVAL_UNITE, 6, "eV", TXT_COPIER);
		met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, 
#ifdef _FRANCAIS_
		           "Valeur de E2", TXT_REMPLACE);
#endif
#ifdef _ENGLISH_
               "E2 value", TXT_REMPLACE);
#endif

		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->e_2);

		ext_formdo(CHOIX_VALEUR);

		infos->e_2 = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		met_fvaleur(ELIM_MU0_1, VALEUR_E2, infos->e_2);
		graf_mouse(S_ABEILLE, NULL);
		MU1_calc   (brut, infos); /* On recalcule le polynome */
		MU1_extrait(brut, infos); /* Et on extrait            */
		graf_mouse(S_FLECHE, NULL);
		FEN_redessin(fenetre);
		break;

	case DEGRE_MOINS:
	case ELIM_MU0_FOND:
	case DEGRE_VALEUR:
	case DEGRE_PLUS:
		infos->degre = gere_ascenseur(ELIM_MU0_1, DEGRE_MOINS, 1, 9, 1, FAUX, num_objet, x, y);
		graf_mouse(S_ABEILLE, NULL);
		MU1_calc   (brut, infos); /* On recalcule    */
		MU1_extrait(brut, infos); /* Et on soustrait */
		FEN_redessin(fenetre);
		graf_mouse(S_FLECHE, NULL);
		return VRAI;

	case MU01_DEG_AFFICHE: /* Degr pour k^n * khi(k) */
		{
			int vx_degre = infos->deg_affiche;

			met_chaine(CHOIX_VALEUR, CHXVAL_UNITE, 6, " ", TXT_COPIER);
			met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0,
#ifdef _FRANCAIS_
                 "Facteur de pond"e_AIGU"ration", TXT_REMPLACE);
#endif
#ifdef _ENGLISH_
                 "Weighting factor", TXT_REMPLACE);
#endif
			met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, (double) (infos->deg_affiche) );

			ext_formdo(CHOIX_VALEUR);

			infos->deg_affiche = (int) prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
			met_valeur(ELIM_MU0_1, MU01_DEG_AFFICHE, infos->deg_affiche);
			graf_mouse(S_ABEILLE, NULL);
			EXF_k_p_n(infos->exafs, infos->deg_affiche - vx_degre);
			EXF_k_p_n(infos->mu_0 , infos->deg_affiche - vx_degre);
			graf_mouse(S_FLECHE, NULL);
			FEN_redessin(fenetre);
		}
		break;
	
	}
	return VRAI;
}

/* --------------- Extraction de mu_0, deuxime tape ----------------------------------------- */

boolean GER_mu02(int num_objet, int x, int y)
{
	OBJECT *adresse = arbres[ELIM_MUO_DEUX].adresse;
	MU2_infos *infos;
	Graphe *brut;
	Fenetre *fenetre;

	fenetre = ARB_fenetre(arbres + ELIM_MUO_DEUX); /* Fentre actuelle */
	infos = (MU2_infos *) fenetre->adresse;
	brut  = (Graphe *)    fenetre->pointeur;

	switch (num_objet)
	{
	case ELIM2_RAPIDE: /* Mthode utilise pour approcher l'absorption atomique */
	case ELIM2_PRECISE:
		if (num_objet != infos->methode) /* On change de mthode */
		{
			form_button(adresse, num_objet, 1, &y);
			infos->methode = num_objet;
			graf_mouse(S_ABEILLE, NULL);
			MU2_calc   (brut, infos); /* On recalcule  */
			MU2_extrait(brut, infos); /* Et on extrait */
			graf_mouse(S_FLECHE, NULL);
			FEN_redessin(fenetre);
		}
		break;

	case ELMU2_MOINS:
	case ELMU2_FOND:
	case ELMU2_VALEUR:
	case ELMU2_PLUS:
		infos->degre = gere_ascenseur(ELIM_MUO_DEUX, ELMU2_MOINS, 1, 9, 1, FAUX, num_objet, x, y);
		graf_mouse(S_ABEILLE, NULL);
		MU2_calc   (brut, infos); /* On recalcule    */
		MU2_extrait(brut, infos); /* Et on soustrait */
		FEN_redessin(fenetre);
		graf_mouse(S_FLECHE, NULL);
		return VRAI;

	case ELMU2_CHOIX: /* Ajouter le choix du mu0 de rfrence */
		{
			Graphe *nv_mu0 = GRF_choisir(GR_MU0);
			if (nv_mu0 == NULL) return VRAI;
			
			if (nv_mu0->nombre_point != brut->nombre_point)
			{
				EXF_ERREUR(NBRPTS_INCORRECT);
				return VRAI;
			}
			if (nv_mu0->drapeaux.protege == VRAI)
			{
				EXF_ERREUR(GRF_PROTEGE);
				return VRAI;
			}

			infos->mu_0_ref->drapeaux.protege = FAUX;
			infos->mu_0_ref = nv_mu0;
			nv_mu0->drapeaux.protege = VRAI;
	  }
		return VRAI;
	}
	return VRAI;
}

/* --------------- Extraction de mu_0, troisime tape ----------------------------------------- */

boolean GER_mu03(int num_objet, int x, int y)
{
	OBJECT *adresse = arbres[ELIM_MU0_TROIS].adresse;
	MU3_infos *infos;
	Graphe *brut;
	Fenetre *fenetre;

	fenetre = ARB_fenetre(arbres + ELIM_MU0_TROIS); /* Fentre actuelle */
	infos = (MU3_infos *) fenetre->adresse;
	brut  = (Graphe *)    fenetre->pointeur;

	switch (num_objet)
	{
	case EL3_MOINS:
	case EL3_FOND:
	case EL3_BOUTON:
	case EL3_PLUS:
		infos->parametre = gere_fascenseur(ELIM_MU0_TROIS, EL3_MOINS, 0.0, 9999.9, 0.1, FAUX, num_objet, x, y);
		graf_mouse(S_ABEILLE, NULL);
		MU3_calc   (brut, infos); /* On recalcule    */
		MU3_extrait(brut, infos); /* Et on soustrait */
		FEN_redessin(fenetre);
		graf_mouse(S_FLECHE, NULL);
		return VRAI;

	case EL3_VALEUR: /* Demande manuelle du paramtre de lissage */
		met_chaine(CHOIX_VALEUR, CHXVAL_UNITE, 6, " ", TXT_COPIER);
		met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 50, "Param"e_GRAVE"tre de lissage", TXT_COPIER);
		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->parametre);

		ext_formdo (CHOIX_VALEUR);

		infos->parametre = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		if (infos->parametre < 0)     infos->parametre = 0;   /* On ne sort pas des bornes... */
		if (infos->parametre > 10000) infos->parametre = 9999.9999;

		/* On repositionne l'ascenseur */
		met_fvaleur(ELIM_MU0_TROIS, EL3_BOUTON, infos->parametre);
		{
			float bouton = OB_WIDTH(adresse, EL3_FOND) - OB_WIDTH(adresse, EL3_BOUTON);
		  OB_X(adresse,EL3_BOUTON) = (int) (infos->parametre / 9999.9999 * (float) bouton);
		}
		OBJ_active(ELIM_MU0_TROIS, EL3_MOINS, infos->parametre > 0);
		OBJ_active(ELIM_MU0_TROIS, EL3_PLUS , infos->parametre < 9999);

		/* On recalcule tout */
		graf_mouse(S_ABEILLE, NULL);
		MU3_calc   (brut, infos); /* On recalcule    */
		MU3_extrait(brut, infos); /* Et on soustrait */
		FEN_redessin(fenetre);
		graf_mouse(S_FLECHE, NULL);
		return VRAI;

	case CHX_MU0_DEUX: /* Choix du mu0 de rfrence */
		{
			Graphe *nv_mu0 = GRF_choisir(GR_MU0);
			if (nv_mu0 == NULL) return VRAI;
			
			if (nv_mu0->nombre_point != brut->nombre_point)
			{
				EXF_ERREUR(NBRPTS_INCORRECT);
				return VRAI;
			}
			if (nv_mu0->drapeaux.protege == VRAI)
			{
				EXF_ERREUR(GRF_PROTEGE);
				return VRAI;
			}

			infos->mu_0_ref->drapeaux.protege = FAUX;
			infos->mu_0_ref = nv_mu0;
			nv_mu0->drapeaux.protege = VRAI;
	  }
		return VRAI;
	}
	return VRAI;
}

/* --------------- Fentre de troncature des graphes ------------------------------------------------------ */

boolean GER_tronque(int num_objet, int x_souris, int y_souris)
{
	TRQ_infos *infos;
	Fenetre *fenetre;
	Graphe *graphe;

	fenetre = ARB_fenetre( &(arbres[TRONQUE_GRAPHE]) );
	infos   = (TRQ_infos *) fenetre->adresse;
	graphe  = (Graphe *)    fenetre->pointeur;

	switch (num_objet)
	{
	case TRQ_XMIN_TXT:
	case TRQ_XMIN:
		met_chaine(CHOIX_VALEUR, CHXVAL_UNITE,  6, "u. a.", TXT_COPIER);
		met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 50,
		           "Borne inf"e_AIGU"rieure", TXT_REMPLACE );
		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->x_min);

		ext_formdo(CHOIX_VALEUR);

		infos->x_min = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		if (infos->x_min < graphe->x_min)
			infos->x_min = graphe->x_min;
		if (infos->x_min > graphe->x_max)
			infos->x_min = graphe->x_max;

		if ( infos->x_max < infos->x_min ) /* On s'assure que min < max */
		{
			double x = infos->x_max; infos->x_max = infos->x_min; infos->x_min = x;
		}

		met_fvaleur(TRONQUE_GRAPHE, TRQ_XMIN, infos->x_min);
		FEN_redessin(fenetre);
		break;

	case TRQ_XMAX_TXT:
	case TRQ_XMAX:
		met_chaine(CHOIX_VALEUR, CHXVAL_UNITE,  6, "u. a.", TXT_COPIER);
		met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 50,
		           "Borne sup"e_AIGU"rieure", TXT_REMPLACE );
		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->x_max);

		ext_formdo(CHOIX_VALEUR);

		infos->x_max = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		if (infos->x_max < graphe->x_min)
			infos->x_max = graphe->x_min;
		if (infos->x_max > graphe->x_max)
			infos->x_max = graphe->x_max;

		if ( infos->x_max < infos->x_min ) /* On s'assure que min < max */
		{
			double x = infos->x_max; infos->x_max = infos->x_min; infos->x_min = x;
		}

		met_fvaleur(TRONQUE_GRAPHE, TRQ_XMAX, infos->x_max);
		FEN_redessin(fenetre);
		break;

	case TRONQUE_EXT:
	case TRONQUE_INT:
		infos->type = num_objet;
		form_button(arbres[TRONQUE_GRAPHE].adresse, num_objet, 1, &_rien_);
		FEN_redessin(fenetre);
	}

	return VRAI;
}

/* ~~~~~~~~~~~~~~~ Arbres non en fentre, forcs en premier plan ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* --------------- Slection d'un graphe d'un type donn -------------------------------------------------- */

boolean GER_choix(int num_objet, int x, int y)
{
  GRF_liste *liste = (GRF_liste *) (arbres[CHX_GRAPHE].pointeur);

  if ( ( (num_objet >= CHXG_NOM1 ) && (num_objet <= CHX_NOM10 ) )   /* Clic sur les noms */
    || ( (num_objet >= CHXG_AVANT) && (num_objet <= CHXG_APRES) ) ) /* Clic sur l'ascenseur */
  {
    return GRF_selecteur(liste, CHX_GRAPHE, CHXG_NOM1, CHX_NOM10, 
			 CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR,
			 num_objet, x, y);
  }

  return VRAI; /* Autrement, c'est bon */
}

/* ~~~~~~~~~~~~~~~~~ Arbres des calculs thoriques ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ----------------- Calcul des phases --------------------------------------------------------------------- */

boolean GER_phase(int num_obj, int x_souris, int y_souris)
{
	OBJECT *adresse = arbres[CALC_PHASE].adresse;
	PHA_infos *inf_phases;
	int nv_obj;

	inf_phases = (PHA_infos *) arbres[CALC_PHASE].pointeur;

	if ( (num_obj >= PHA_CTR_1) && (num_obj <= PHA_CTR_APRES) ) 
	/* Slecteur de graphe des phases de l'atome central */
	{
		GRF_selecteur(inf_phases->centre, CALC_PHASE, PHA_CTR_1, PHA_CTR_4, PHA_CTR_AVANT, PHA_CTR_APRES, 
		                                  PHA_CTR_FOND, PHA_CTR_BOUTON, num_obj, x_souris, y_souris);
		return VRAI;
	}

	if ( (num_obj >= PHA_EXP_1) && (num_obj <= PHA_EXP_APRES) ) 
	/* Slecteur de graphe des phases de l'atome rflecteur */
	{
		GRF_selecteur(inf_phases->reflecteur, CALC_PHASE, PHA_EXP_1, PHA_EXP_4, PHA_EXP_AVANT, PHA_EXP_APRES, 
		                                      PHA_EXP_FOND, PHA_EXP_BOUTON, num_obj, x_souris, y_souris);
		return VRAI;
	}

	switch (num_obj)
	{
	case PHA_CTR_THEORIE:
	case PHA_CTR_EXP:
		form_button(adresse, num_obj, 1, &nv_obj);
		if ( OB_STATE(adresse, PHA_CTR_THEORIE) & SELECTED )
		{
			OB_FLAGS(adresse,PHA_CTR_1 - 1) |= HIDETREE; /* Slecteur invisible */
			OB_FLAGS(adresse,PHA_CTR_AVANT) |= HIDETREE;
			OB_FLAGS(adresse,PHA_CTR_FOND)  |= HIDETREE;
			OB_FLAGS(adresse,PHA_CTR_APRES) |= HIDETREE;

			OB_FLAGS(adresse,PHA_CTR_NATURE) &= ~HIDETREE; /* Choix de l'lment possible */
			arbres[CALC_PHASE].premier_edit = PHA_CTR_NATURE;
		}
		else
		{
			OB_FLAGS(adresse,PHA_CTR_1 - 1) &= ~HIDETREE; /* Slecteur visible */
			OB_FLAGS(adresse,PHA_CTR_AVANT) &= ~HIDETREE;
			OB_FLAGS(adresse,PHA_CTR_FOND)  &= ~HIDETREE;
			OB_FLAGS(adresse,PHA_CTR_APRES) &= ~HIDETREE;

			OB_FLAGS(adresse,PHA_CTR_NATURE) |= HIDETREE; /* Choix de l'lment invisible */
			if ( OB_STATE(adresse, PHA_THEORIE) & SELECTED )
				arbres[CALC_PHASE].premier_edit = PHA_EXP_NATURE;
			else
				arbres[CALC_PHASE].premier_edit = CALC_PHA_DEBUT;
			if ( arbres[CALC_PHASE].edit_courant == PHA_CTR_NATURE )
				arbres[CALC_PHASE].edit_courant = arbres[CALC_PHASE].premier_edit;
		}
		dessine_objet(CALC_PHASE, PHA_CTR_1 - 1);
		dessine_objet(CALC_PHASE, PHA_CTR_AVANT);
		dessine_objet(CALC_PHASE, PHA_CTR_FOND);
		dessine_objet(CALC_PHASE, PHA_CTR_APRES);
		dessine_objet(CALC_PHASE, PHA_CTR_NATURE);

		return VRAI;

	default:
		form_button(adresse, num_obj, 1, &nv_obj);
	}

	return VRAI;
}

/* ~~~~~~~~~ Divers arbres ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* --------- Paramtrage de l'export au format GEM -------------------- */

boolean GER_svgem(int num_objet, int x, int y)
{
	if (num_objet == GEM_A4_PORTRAIT) /* On demande du A4, sens habituel */
	{
		met_fvaleur(PARAM_GEM, GEM_LARGEUR, 210.0);
		met_fvaleur(PARAM_GEM, GEM_HAUTEUR, 297.0);
		dessine_objet(PARAM_GEM, GEM_LARGEUR);
		dessine_objet(PARAM_GEM, GEM_HAUTEUR);
		return VRAI;
	}

  if (num_objet == GEM_A4_PAYSAGE) /* On demande du A4  l'italienne */
	{
		met_fvaleur(PARAM_GEM, GEM_LARGEUR, 297.0);
		met_fvaleur(PARAM_GEM, GEM_HAUTEUR, 210.0);
		dessine_objet(PARAM_GEM, GEM_LARGEUR);
		dessine_objet(PARAM_GEM, GEM_HAUTEUR);
		return VRAI;
	}

	if ( (num_objet == GEM_ANNULE) || (num_objet == GEM_OK) )
		return _QUITTER; /* Clic pour sortir */

	return VRAI + 0*(x + y); /* On a fini ! */
}

/* --------- Paramtrage de l'import des fichiers FEFF --------------- */

boolean GER_feff(int num_objet, int x, int y)
{
	OBJECT *adresse = arbres[IMPORTE_FEFF].adresse;

	form_button(adresse, num_objet, 1, &x); /* De toute faon... */

	switch (num_objet)
	{
	case FEFF_FEFF: /* Fichier FEFF de type FEFFnnnn.DAT */	
		OB_STATE(adresse, FEFF_K) |= DISABLED; /* Lecture en abscisse : forcment k */
		OB_STATE(adresse, FEFF_K) |= SELECTED;
		OB_STATE(adresse, FEFF_ENERGIE) |= DISABLED;
		OB_STATE(adresse, FEFF_ENERGIE) &= ~SELECTED;
		OB_STATE(adresse, FEFF_E_RELATIVE) |=  DISABLED;
		OB_STATE(adresse, FEFF_E_RELATIVE) &= ~SELECTED;
		dessine_objet(IMPORTE_FEFF, FEFF_K);
		dessine_objet(IMPORTE_FEFF, FEFF_ENERGIE);
		dessine_objet(IMPORTE_FEFF, FEFF_E_RELATIVE);

		OB_STATE(adresse, FEFF_REPHC)  &= ~DISABLED; /* Type d'informations lisibles */
		OB_STATE(adresse, FEFF_AMP)    &= ~DISABLED; /* Type d'informations lisibles */
		OB_STATE(adresse, FEFF_REDFAC) &= ~DISABLED; /* Type d'informations lisibles */
		OB_STATE(adresse, FEFF_LIBRE)  &= ~DISABLED; /* Type d'informations lisibles */
		OB_STATE(adresse, FEFF_PHASE)  &= ~DISABLED; /* Type d'informations lisibles */
		OB_STATE(adresse, FEFF_RE_PH)  &= ~DISABLED; /* Type d'informations lisibles */
		OB_STATE(adresse, FEFF_EXAFS)  |=  DISABLED; /* Pas dans ces fichiers */
		OB_STATE(adresse, FEFF_PHIM2KR) |= DISABLED;
		OB_STATE(adresse, FEFF_ABSORBE) |= DISABLED;
		dessine_objet(IMPORTE_FEFF, FEFF_REPHC ); /* On redessine le fond : plus rapide */
		dessine_objet(IMPORTE_FEFF, FEFF_AMP);
		dessine_objet(IMPORTE_FEFF, FEFF_REDFAC);
		dessine_objet(IMPORTE_FEFF, FEFF_LIBRE);
		dessine_objet(IMPORTE_FEFF, FEFF_PHASE);
		dessine_objet(IMPORTE_FEFF, FEFF_RE_PH);
		dessine_objet(IMPORTE_FEFF, FEFF_PHIM2KR);
		dessine_objet(IMPORTE_FEFF, FEFF_EXAFS);
		dessine_objet(IMPORTE_FEFF, FEFF_ABSORBE);
		break;

	case FEFF_CHIP: /* Fichier FEFF de type CHIPnnnn.DAT */
	case FEFF_CHI:  /* Fichier FEFF de type CHI.DAT      */
		OB_STATE(adresse, FEFF_K) |= DISABLED; /* Lecture en abscisse : forcment k */
		OB_STATE(adresse, FEFF_K) |= SELECTED;
		OB_STATE(adresse, FEFF_ENERGIE) |=  DISABLED;
		OB_STATE(adresse, FEFF_ENERGIE) &= ~SELECTED;
		OB_STATE(adresse, FEFF_E_RELATIVE) |=  DISABLED;
		OB_STATE(adresse, FEFF_E_RELATIVE) &= ~SELECTED;
		dessine_objet(IMPORTE_FEFF, FEFF_K);
		dessine_objet(IMPORTE_FEFF, FEFF_ENERGIE);
		dessine_objet(IMPORTE_FEFF, FEFF_E_RELATIVE);

		OB_STATE(adresse, FEFF_REPHC)   |=  DISABLED; /* Informations absentes */
		OB_STATE(adresse, FEFF_REDFAC)  |=  DISABLED;
		OB_STATE(adresse, FEFF_LIBRE)   |=  DISABLED;
		OB_STATE(adresse, FEFF_RE_PH)   |=  DISABLED;
		OB_STATE(adresse, FEFF_ABSORBE) |=  DISABLED;
		OB_STATE(adresse, FEFF_AMP)     &= ~DISABLED; /* Informations lisibles */
		OB_STATE(adresse, FEFF_PHASE)   &= ~DISABLED;
		OB_STATE(adresse, FEFF_EXAFS)   &= ~DISABLED;
		if (num_objet == FEFF_CHIP)
			OB_STATE(adresse, FEFF_PHIM2KR) &= ~DISABLED; /* Seulement dans les CHIPnnnn.DAT */
		else
			OB_STATE(adresse, FEFF_PHIM2KR) |= DISABLED;
		dessine_objet(IMPORTE_FEFF, FEFF_REPHC);
		dessine_objet(IMPORTE_FEFF, FEFF_AMP);
		dessine_objet(IMPORTE_FEFF, FEFF_REDFAC);
		dessine_objet(IMPORTE_FEFF, FEFF_LIBRE);
		dessine_objet(IMPORTE_FEFF, FEFF_PHASE);
		dessine_objet(IMPORTE_FEFF, FEFF_RE_PH);
		dessine_objet(IMPORTE_FEFF, FEFF_PHIM2KR);
		dessine_objet(IMPORTE_FEFF, FEFF_EXAFS);
		dessine_objet(IMPORTE_FEFF, FEFF_ABSORBE);
		break;

	case FEFF_XMU: /* Fichier FEFF de type XMU.DAT */
		OB_STATE(adresse, FEFF_K) &= ~DISABLED; /* Lecture en abscisse : k ou E */
		OB_STATE(adresse, FEFF_ENERGIE) &= ~DISABLED;
		OB_STATE(adresse, FEFF_E_RELATIVE) &= ~DISABLED;
		dessine_objet(IMPORTE_FEFF, FEFF_K);
		dessine_objet(IMPORTE_FEFF, FEFF_ENERGIE);
		dessine_objet(IMPORTE_FEFF, FEFF_E_RELATIVE);

		OB_STATE(adresse, FEFF_REPHC)   |=  DISABLED; /* Informations absentes */
		OB_STATE(adresse, FEFF_REDFAC)  |=  DISABLED;
		OB_STATE(adresse, FEFF_LIBRE)   |=  DISABLED;
		OB_STATE(adresse, FEFF_RE_PH)   |=  DISABLED;
		OB_STATE(adresse, FEFF_AMP)     |=  DISABLED;
		OB_STATE(adresse, FEFF_PHASE)   |=  DISABLED;
		OB_STATE(adresse, FEFF_PHIM2KR) |=  DISABLED;
		OB_STATE(adresse, FEFF_EXAFS)   &= ~DISABLED; /* Informations lisibles */
		OB_STATE(adresse, FEFF_ABSORBE) &= ~DISABLED;
		dessine_objet(IMPORTE_FEFF, FEFF_REPHC);
		dessine_objet(IMPORTE_FEFF, FEFF_AMP);
		dessine_objet(IMPORTE_FEFF, FEFF_REDFAC);
		dessine_objet(IMPORTE_FEFF, FEFF_LIBRE);
		dessine_objet(IMPORTE_FEFF, FEFF_PHASE);
		dessine_objet(IMPORTE_FEFF, FEFF_RE_PH);
		dessine_objet(IMPORTE_FEFF, FEFF_PHIM2KR);
		dessine_objet(IMPORTE_FEFF, FEFF_EXAFS);
		dessine_objet(IMPORTE_FEFF, FEFF_ABSORBE);
		break;		
	}

	return VRAI + 0*y; /* Trait */
}

/* --------------- Paramtrage des lgendes -------------------------------------------------- */

boolean GER_legende(int num_objet, int x_souris, int y_souris)
{
	OBJECT *adresse = arbres[PARAM_LEGENDE].adresse;
	int nv_objet;

	switch (num_objet)
	{
	case LEG_NONGRADUE:
		form_button(adresse, num_objet, 1, &nv_objet);
		OB_STATE(adresse, LEG_CENT) |= DISABLED;
		OB_STATE(adresse, LEG_DIX ) |= DISABLED;
		dessine_objet(PARAM_LEGENDE, LEG_CENT);
		dessine_objet(PARAM_LEGENDE, LEG_DIX);
		break;

	case LEG_GRAD_DECIMAL:
	case LEG_GRAD_LOG:
		form_button(adresse, num_objet, 1, &nv_objet);
		OB_STATE(adresse, LEG_CENT) &= ~DISABLED;
		OB_STATE(adresse, LEG_DIX ) &= ~DISABLED;
		dessine_objet(PARAM_LEGENDE, LEG_CENT);
		dessine_objet(PARAM_LEGENDE, LEG_DIX);
		break;

	case LEG_STYLE:       /* Style de dessin de la grille */
		objc_offset(adresse, num_objet, &x_souris, &y_souris);
		nv_objet  = (int) ( (USERBLK *) OB_SPEC(adresse, LEG_STYLE) )->ub_parm;
		y_souris -= (nv_objet - 1) * OB_HEIGHT(adresse, LEG_STYLE);
		nv_objet  = GER_ssmenu(CHOIX_STYLE, 0, x_souris, y_souris);
		( (USERBLK *) OB_SPEC(adresse, LEG_STYLE) )->ub_parm = nv_objet - 1;
		dessine_objet(PARAM_LEGENDE, LEG_STYLE);
		break;

	case LEG_FNT_AVANT:
	case LEG_FNT_NOM:
	case LEG_FNT_APRES:
		break;

	case LEG_TXT_COUL:    /* Couleur des textes   */
	case LEG_GRAD_COUL:   /* Couleur de la grille */
		objc_offset(adresse, num_objet, &x_souris, &y_souris);
		nv_objet  = (int) ( OB_SPEC(adresse, num_objet) & 0xFL ) - 1;
		y_souris -= nv_objet * OB_HEIGHT(adresse, num_objet);
		nv_objet  = GER_ssmenu(CHX_COULEUR, 0, x_souris, y_souris);
		OB_SPEC(adresse, num_objet) &= 0xFFFFFFF0L; /* On met les 4 derniers bits  0 */
		OB_SPEC(adresse, num_objet) |= nv_objet;
		dessine_objet(PARAM_LEGENDE, num_objet); /* On dessine */
		break;

	default:
		form_button(adresse, num_objet, 1, &nv_objet);
	}

	return VRAI;
}

/* --------------- Paramtrage de la TF inverse ---------------------------------------------- */

boolean GER_paramtfi(int num_objet, int x_souris, int y_souris)
{
  OBJECT *adresse  = arbres[PARAM_TFI].adresse;
  TFI_infos *infos = (TFI_infos *) arbres[PARAM_TFI].pointeur;
  int nv_objet;
  boolean etat;
  extern const char *Types_fenetres[];

  if ( (num_objet >= TFI_FENETRE_1) && (num_objet <= TFI_FENETRE_5) )
  {
    unsigned long nv_fenetre   = (unsigned long) (infos->num_choisi);
    unsigned long prem_visible = (unsigned long) (infos->num_depart);

    GDFL_textes(PARAM_TFI, TFI_FENETRE_1, TFI_FENETRE_5, TFI_FEN_AVANT, TFI_FEN_APRES, TFI_FOND_ASC, TFI_BOUTON_ASC,
		NOMBRE_FENETRES, (char **) Types_fenetres, NULL, &nv_fenetre, 15,
		&prem_visible, num_objet, y_souris);

    if (infos->num_choisi == (int) nv_fenetre)
      return VRAI; /* Rien  faire */

    infos->num_choisi = (int) nv_fenetre;
    etat = (nv_fenetre == FEN_HANNING) || (nv_fenetre == FEN_HAMMING)
      || (nv_fenetre == FEN_BARTLETT); /* VRAI -> asymtrie possible */
    OBJ_active(PARAM_TFI, TF_FEN_SYMETRIK, etat);
    OBJ_active(PARAM_TFI, TFI_K_1, etat);
    
    etat &= ! (OBJ_choisit(PARAM_TFI, TF_FEN_SYMETRIK, QUESTION));
    OBJ_active(PARAM_TFI, TFI_K_2, etat);

    etat = (nv_fenetre == FEN_HAMMING) || (nv_fenetre == FEN_NORTON_F3)
      || (nv_fenetre == FEN_BESSEL)  || (nv_fenetre == FEN_GAUSSIENNE);
    OBJ_active(PARAM_TFI, TFI_PARAMETRE, etat); /* Seule fentre avec un paramtre... */

    return VRAI;
  }

	switch (num_objet)
	{
	case TFI_FEN_AVANT :
	case TFI_FEN_APRES : /* Gestion de la liste de fentres */
	case TFI_FOND_ASC  :
	case TFI_BOUTON_ASC:
  	{
		  unsigned long nv_fenetre   = (unsigned long) (infos->num_choisi);
		  unsigned long prem_visible = (unsigned long) (infos->num_depart);
	
		  GDFL_textes(PARAM_TFI, TFI_FENETRE_1, TFI_FENETRE_5, TFI_FEN_AVANT, TFI_FEN_APRES, TFI_FOND_ASC, TFI_BOUTON_ASC,
		              NOMBRE_FENETRES, (char **) Types_fenetres, NULL, &nv_fenetre, 15,
	  	            &prem_visible, num_objet, y_souris);
  		infos->num_depart = (int) prem_visible;
		}
		break;

	case TF_FEN_SYMETRIK:
		etat = OBJ_choisit(PARAM_TFI, TF_FEN_SYMETRIK, ECHANGE);
		OBJ_active(PARAM_TFI, TFI_K_2, !etat); /* Inutile si symtrique... */
		return VRAI;

	default:
		form_button(adresse, num_objet, 1, &nv_objet);
	}

	return VRAI;
}

/* --------------- EXAFS : extraction du spectre en une seule tape -------------------------- */

boolean GER_reproduit(int objet, int x, int y)
{
  switch (objet)
  {
  case EXF_MU_MATRICE:
  case EXF_MU0_CALC:
  case EXF_MU0_REFAIT:
  case EXF_LISSER:
  case EXF_TF:
  case EXF_TFI:
    OBJ_choisit(EXAFS_GENERAL, objet, ECHANGE);
    break;

  case EXF_MUM_DROITE:
  case EXF_MUM_VICT1:
  case EXF_MUM_VICT2:
  case EXF_MUM_LANS:
  case EXF_MU0_I_RAPIDE:
  case EXF_MU0_I_LENT:
  case EXF_MU0_II_RAPID:
  case EXF_MU0_II_LENT:
    OBJ_choisit(EXAFS_GENERAL, objet, FORCE_OUI);
    break;	
  }

  return VRAI;
}

/* --------------- Bote d'informations : choix du menu que l'on utilise --------------------- */

boolean GER_infos(int num_objet, int x_souris, int y_souris)
{
  unsigned long i = NBR_LIGNES_D_INFOS, prem_ligne; /* Pour les infos */

  switch (num_objet)
  {
  case TRAITE_SPECTRES:
    OBJ_choisit(INFORMATIONS, TRAITE_SPECTRES, FORCE_OUI);
    installe_menu(MENU, MEN_extrait);
    return VRAI;

#ifndef _LIMITE_ /* Seulement dans la version complte */

  case EXF_AJUSTEMENTS:
    OBJ_choisit(INFORMATIONS, EXF_AJUSTEMENTS, FORCE_OUI);
    installe_menu(MEN_AJUSTE, MEN_ajuste);
    return VRAI;

  case CALC_THEORIQUES:
    OBJ_choisit(INFORMATIONS, CALC_THEORIQUES, FORCE_OUI);
    installe_menu(MEN_THEORIE, MEN_theorie);
    return VRAI;

#endif

  case HISTORIQUE:
    if ( OBJ_choisit(INFORMATIONS, HISTORIQUE, ECHANGE) ) /* On demande un historique */
    {
      Histoire = fopen("HISTOIRE.TXT", "a");
      if (Histoire == NULL)
      {
	ERREUR(ERREUR_ECRITURE);
	OBJ_choisit(INFORMATIONS, HISTORIQUE, FORCE_NON);
      }
      else
      {
	Date_Gemdos date = TGetdate();
	
	fprintf(Histoire," ********** S"e_AIGU"ance de travail du %d %s %d **********\n",
		date.jour, _Mois_[date.mois - 1], date.annee + 1980);
      }
    }
    else
    {
      fclose(Histoire); /* On ferme l'historique */
      Histoire = NULL; /* Plus de fichier associ */
    }
    return VRAI;

  case INFOS_AVANT:
  case INFOS_APRES:
  case INFOS_FOND:
  case INFOS_FOND + 1:
    prem_ligne = (unsigned long) arbres[INFORMATIONS].pointeur;

    GDFL_textes(INFORMATIONS, INFOS_PREM_TEXTE, INFOS_DERN_TEXTE,
		INFOS_AVANT, INFOS_APRES, INFOS_FOND, INFOS_FOND + 1,
		NBR_LIGNES_D_INFOS, (char **) TXT_informations, 
		NULL, &i, 55, &prem_ligne, num_objet, y_souris);	

    arbres[INFORMATIONS].pointeur = (void *) prem_ligne;
  }
  return VRAI;
}

/* ~~~~~~~~~~~~~~~ Macros de gestion des fentres ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* --------------- Trac d'un pointeur (vertical) dans une fentre ----------------------------\
 |    Renvoie VRAI si tout s'est pass comme il faut (vnement trait)                        |
 |    Suppose que fenetre->pointeur contienne l'adresse du graphe en cours dans la fentre     |
 |                                                                                             |
 | fenetre : structure associe  la fentre dans laquelle on travaille                        |
 | x_souris, y_souris : coordonnes de la souris                                               |
 \-------------------------------------------------------------------------------------------- */

boolean RCT_pointeur(Fenetre *fenetre, int x_souris, int y_souris)
{
  Graphe  *graphe;
  double x_echelle,x_reel;
  int y_trace;

  graphe = (Graphe *) fenetre->pointeur;
  if (graphe == NULL) return VRAI;

  if (graphe->x_min == graphe->x_max) return VRAI; /* Pas de pointeur si bornes identiques... */
  if (graphe->y_min == graphe->y_max) return VRAI;

  y_trace = fenetre->w_y;

  if (fenetre->arbre) /* Il y a un arbre associ  la fentre */
    y_trace += OB_HEIGHT( fenetre->arbre->adresse, ROOT);

  points[0] = fenetre->w_x;
  points[1] = y_trace;
  points[2] = fenetre->w_x + fenetre->w_larg;
  points[3] = fenetre->w_y + fenetre->w_haut;
  
  wind_update(BEG_UPDATE);     /* On coupe l'AES le temps du dessin */
  wind_update(BEG_MCTRL);      /* On coupe l'AES le temps du dessin */
  graf_mouse(M_OFF,NULL);

  vswr_mode(_PRG_station->handle, MD_INVERSE);
  vsl_color(_PRG_station->handle, NOIR);
  vsl_type (_PRG_station->handle, 1); /* Ligne noire, continue */

  vs_clip(_PRG_station->handle,1,points); /* On met le clipping */

  /* On efface l'ancien pointeur */

  if ( (_x_pointeur >= points[0]) && (_x_pointeur <= points[2])
    && (_y_pointeur >= points[1]) && (_y_pointeur <= points[3]) )
  {
    points[0] = _x_pointeur; points[2] = _x_pointeur;
    v_pline(_PRG_station->handle,2,points);
  }

  points[0] = fenetre->w_x;
  points[2] = fenetre->w_x + fenetre->w_larg;

  if ( (y_souris < points[1]) || (y_souris > points[3])  /* Souris hors de la partie intressante */
    || (x_souris < points[0]) || (x_souris > points[2]) )
  {
    graf_mouse (M_ON,NULL);      /* Souris visible */
    wind_update(END_UPDATE);     /* On remet l'AES */
    wind_update(END_MCTRL);      /* On remet la souris */
    return VRAI;
  }

  x_echelle = ( (double) fenetre->w_larg ) / (graphe->x_max - graphe->x_min);
  x_reel = graphe->x_min + ( (double) (x_souris - fenetre->w_x) ) / x_echelle;

  points[0] = x_souris; points[2] = x_souris;
  v_pline(_PRG_station->handle, 2, points);

  switch (graphe->unit_x) /* En fonction de l'unit */
  {
  case SS_UNITE:
    sprintf(_intermediaire,"Souris : X = %f",x_reel);
    break;

  case UNIT_EV:
    sprintf(_intermediaire,"Souris : X = %f eV",x_reel);
    break;

  case UNIT_ANG:
    sprintf(_intermediaire,"Souris : X = %f "ANGSTROM, x_reel);
    break;

  case UNIT_1_ANG:
    sprintf(_intermediaire,"Souris : X = %f "ANGSTROM"^-1", x_reel);
    break;

  case UNIT_ARB:
    sprintf(_intermediaire,"Souris : X = %f (unit"e_AIGU" arbitraire)",x_reel);
    break;

  case UNIT_NORM:
    sprintf(_intermediaire,"Souris : X = %f (unit"e_AIGU" normalis"e_AIGU"e)",x_reel);
    break;

  default:
    sprintf(_intermediaire,"Souris : X = %f (unit"e_AIGU" inconnue)",x_reel);
  }
  FEN_infos(fenetre, _intermediaire);

  vswr_mode  (_PRG_station->handle, MD_REMPLACE);     /* Mode recouvrir */
  graf_mouse (M_ON,NULL);
  wind_update(END_UPDATE);     /* On remet l'AES */
  wind_update(END_MCTRL);

  return VRAI; /* On a trait l'vnement */
}
