/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
							Toutes les fonctions de traitement statistique de l'E.X.A.F.S.
   (c) Emmanuel CURIS 1998

  23 mars 1998
    * Utilisation de "systeme.h" : plus portable
    * Affiche les graphes choisis mme si > au nombre affichable en 1 fois
    * Ascenseur marche partout sauf dplacement du poussoir directement
    * Corrig calcul de l'cart type (donnait la variance...) Merci Bruno

  16 mars 1998
    * Inclusion de erreurs.h (pour l'exafs)
    * Prparation de ce qu'il faut pour calculer le spectre moyen + calcul de ce spectre
    * Calcul des barres d'erreur fonctionne (y compris erreur moyenne et correction multiplicative)

	13 mars 1998
		* Cration du fichier
    * Dclaration de ERR_dispersion() : paramtrages des calculs de dispersion standard
            interface fonctionne entirement, sauf l'ascenseur
*/

#define _EXAFS_STATS_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "ERREURS.H"
#include "EXF_STAT.H"

/* ~~~~~~~~~~~~~~~~ Calcul de la dispersion standard de spectres E.X.A.F.S. ~~~~~~~~~~~~~~~~~ */

/* ---------------- Bote de dialogue permettant de paramtrer tout a -----------------------\
 |      Ne prend aucun paramtre, n'utilise rien, ne renvoie rien                             |
 \------------------------------------------------------------------------------------------- */

void ERR_dispersion(void)
{
  DISP_infos infos = {0, -1, 0, NULL};
  TRI_infos *infos_moyenne;
  int reponse = 0 ;
  unsigned long i, j = 0;
  double *x_moyen = NULL, *y_moyen = NULL;
  double disp_moyenne; /* cart type moyen sur tous les points */
  Graphe *dispersion, modele, *moyen = NULL;

  infos.liste_graphes = (boolean *) malloc( sizeof(boolean) * nbr_graphes );
  if (infos.liste_graphes == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return;
  }

	for (i = 0 ; i < nbr_graphes ; i++)
		infos.liste_graphes[i] = FAUX; /* Aucun graphe slectionn */

	arbres[BARRE_ERREUR].pointeur = &infos;

	DISP_prep_asc();

	OBJ_active(BARRE_ERREUR, BERR_EFFACER, FORCE_NON);    /* Rien  effacer */
	OBJ_active(BARRE_ERREUR, BERR_TT_AJOUTER, FORCE_NON); /* actif seulement si un type de graphe dfini */
	OBJ_active(BARRE_ERREUR, BERR_TT_EFFACER, FORCE_NON); /* Rien  effacer */	

	do
	{
		reponse = ext_formdo(BARRE_ERREUR);
		if (reponse == BERR_ANNULER)
		{
			free(infos.liste_graphes);
			return;
		}

		if (reponse == BERR_MUL_SIGMA_S) /* Pour la suggestion... */
		{
			if (infos.nbr_choisis == 3)
				met_fvaleur(BARRE_ERREUR, BERR_MUL_SIGMA_V, 2.2);
			if (infos.nbr_choisis == 4)
				met_fvaleur(BARRE_ERREUR, BERR_MUL_SIGMA_V, 1.5);
			if (infos.nbr_choisis == 5)
				met_fvaleur(BARRE_ERREUR, BERR_MUL_SIGMA_V, 1.2);
			if (infos.nbr_choisis == 6)
				met_fvaleur(BARRE_ERREUR, BERR_MUL_SIGMA_V, 1.1);
			if (infos.nbr_choisis >= 7)                         /* Au-del de 7 spectres, suppos correct */
				met_fvaleur(BARRE_ERREUR, BERR_MUL_SIGMA_V, 1.0);
		}
	} while (reponse != BERR_OK);

	/* On est l : on veut le calcul effectif */

	/* On commence par reprer le type des graphes
	               et vrifier qu'ils ont le mme nombre de points */
	/* On en profite pour prparer les informations pour le calcul de la moyenne */

	infos_moyenne = (TRI_infos *) malloc( sizeof(TRI_infos) * infos.nbr_choisis);
	if (infos_moyenne == NULL)
	{
		ERREUR(MANQUE_MEMOIRE);
		free(infos.liste_graphes);
		return;
	}

	modele.type = GR_AUCUN;
	modele.nombre_point = 0;

	for (dispersion = premier_graphe, i = 0 ; i < nbr_graphes ; i++, dispersion = dispersion->suivant)
	{
		if ( infos.liste_graphes[i] == VRAI ) /* Ce graphe doit tre utilis */
		{
			if ( modele.type == GR_AUCUN ) /* C'est le premier graphe que l'on rencontre */
			{
				modele.type = dispersion->type;
				modele.nombre_point = dispersion->nombre_point;
				modele.drapeaux = dispersion->drapeaux;
				strcpy(modele.axe_x, dispersion->axe_x);
				strcpy(modele.axe_y, dispersion->axe_y);
				modele.unit_x = dispersion->unit_x;
				modele.unit_y = dispersion->unit_y;
			}
			else if ( modele.type == GR_SSTYPE )
				modele.type = dispersion->type; /* Type du graphe */

			if ( modele.nombre_point != dispersion->nombre_point )
			{
				EXF_ERREUR(NBRPTS_INCORRECT);
				free(infos.liste_graphes);
				free(infos_moyenne);
				return;
			}

			/* On remplit les infos pour la moyenne */

			infos_moyenne[j].xpoint_depart = dispersion->x; /* les x */
			infos_moyenne[j].ypoint_depart = dispersion->y; /* les y */
			infos_moyenne[j].position = 0;
			infos_moyenne[j].nbr_points = dispersion->nombre_point;
			j++; /* On passe aprs aux infos suivantes */
		}
	}

	free(infos.liste_graphes); /* On n'a plus besoin de a */

	/* Prparation des zones o l'on va garder le spectre moyen */

	if ( OBJ_choisit(BARRE_ERREUR, BERR_GARDE_MOYEN, QUESTION) ) /* On veut garder le spectre moyen */
	{
		char titre[300];

		sprintf(titre, "Spectre moyen de %ld spectres, utilis pour la dispersion", infos.nbr_choisis);
		moyen = GRF_creer(modele.nombre_point, modele.type, modele.drapeaux.complexe, titre, modele.axe_x, modele.axe_y, modele.unit_x, modele.unit_y);
		if (moyen == NULL)
		{
			free(infos_moyenne);
			return;
		}

		x_moyen = moyen->x; /* Ou stocke-t-on le spectre moyen ? */
		y_moyen = moyen->y;
	}
	else
	{
		x_moyen = (double *) malloc( sizeof(double) * modele.nombre_point );
		y_moyen = (double *) malloc( sizeof(double) * modele.nombre_point );
		if ( (x_moyen == NULL) || (y_moyen == NULL) )
		{
			ERREUR(MANQUE_MEMOIRE);
			free(x_moyen); free(y_moyen);
			free(infos_moyenne);
			return;
		}
	}

	/* On calcule le spectre moyen, avant toute chose */

	i = MOY_blocs(x_moyen, y_moyen, infos_moyenne, infos.nbr_choisis, modele.nombre_point);
	if (i != modele.nombre_point)
	{
		printf("Problme survenu... Appuyez sur une touche pour continuer\n");
		evnt_keybd();
	}

	/* On calcule maintenant l'cart type pour chaque point + l'cart type moyen */

	{
		double correction = 0, *moyenne = NULL;

		if ( OBJ_choisit(BARRE_ERREUR, BERR_S_MOYEN, QUESTION) ) /* On veut l'erreur moyenne */
			moyenne = &disp_moyenne;

		if ( OBJ_choisit(BARRE_ERREUR, BERR_MUL_SIGMA, QUESTION) ) /* On veut corriger l'erreur */
			correction = prend_fvaleur(BARRE_ERREUR, BERR_MUL_SIGMA_V);

		dispersion = DISP_calc(x_moyen, y_moyen, infos_moyenne, infos.nbr_choisis,
		                       modele.nombre_point, correction, moyenne);

		if (moyenne) /* On a calcul la dispersion moyenne */
		{
			printf("Valeur de la dispersion standard moyenne : %f (%e)\n", disp_moyenne, disp_moyenne);
			printf("Appuyez sur une touche pour continuer\n");
			evnt_keybd();
		}
	}

	free(infos_moyenne);

	if (moyen)
	{
	  GRF_bornes(moyen, DESORDONNE, ORDONNE); /* On recalcule les bornes */
	  GRF_aller(moyen); /* On va sur ce spectre */
	  /* et on lie les erreurs en y au graphe de dispersion */
	}
	else
	  GRF_aller(dispersion);
}

/* ----------- Gestion du slecteur de la bote de dialogue --------------------- */

void DISP_gere_asc(int num_objet, int x, int y)
{
  DISP_infos *infos = arbres[BARRE_ERREUR].pointeur;
  const unsigned int nbr_visibles = BERR_GRF_DER_NOM - BERR_GRF_PREMNOM + 1;

  if ( (num_objet >= BERR_GRF_PREMNOM) && (num_objet <= BERR_GRF_DER_NOM) ) /* Clic sur un graphe */
  {
    int num_clique = num_objet - BERR_GRF_PREMNOM;

    if (num_clique == infos->num_choisi) /* On dsactive ce graphe, en fait */
    {
      infos->num_choisi = -1; /* Aucun objet choisi */
    }
    else
    {
      if (infos->num_choisi >= 0) /* Il y a dj un graphe slectionn */
	OBJ_choisit(BARRE_ERREUR, infos->num_choisi + BERR_GRF_PREMNOM, FORCE_NON); /* On le dsactive */

      infos->num_choisi = num_clique; /* Nouvel objet choisi */
    }

    OBJ_choisit(BARRE_ERREUR, num_objet, ECHANGE); /* De toute faon */
    OBJ_active(BARRE_ERREUR, BERR_EFFACER, infos->num_choisi >=0 );
    return; /* Fini */
  }

  if (num_objet == BERR_GRF_AVANT)
  {
    infos->prem_affiche--;
  }

  if (num_objet == BERR_GRF_ASC_FND)
  {
    int x_bouton, y_bouton;

    objc_offset(arbres[BARRE_ERREUR].adresse, BERR_GRF_ASC_BOU, &x_bouton, &y_bouton);
    if ( y < y_bouton ) /* Clic avant l'ascenseur */
    {
      if (infos->prem_affiche > 10)
	infos->prem_affiche -= 10;
      else
	infos->prem_affiche = 0;
    }
    else /* Clic aprs le poussoir */
    {
      if (infos->prem_affiche + nbr_visibles + 10 < infos->nbr_choisis)
	infos->prem_affiche += 10;
      else
	infos->prem_affiche = infos->nbr_choisis - nbr_visibles;
    }
  }

  if (num_objet == BERR_GRF_ASC_BOU)
  {
  }

  if (num_objet == BERR_GRF_APRES)
  {
    infos->prem_affiche++;
  }

  DISP_prep_asc();
}

/* ----------- Prparation du slecteur dans la bote de dialogue --------------- */

void DISP_prep_asc(void)
{
	const DISP_infos *infos = arbres[BARRE_ERREUR].pointeur;
	const unsigned int nbr_visibles = BERR_GRF_DER_NOM - BERR_GRF_PREMNOM + 1;
	unsigned long i;

	OBJ_active(BARRE_ERREUR, BERR_AJOUTER, infos->nbr_choisis < nbr_graphes ); /* Il en reste  ajouter */
	OBJ_active(BARRE_ERREUR, BERR_EFFACER, infos->num_choisi >=0 );
	OBJ_active(BARRE_ERREUR, BERR_TT_AJOUTER , infos->nbr_choisis > 0); /* actif seulement si un type de graphe dfini */
	OBJ_active(BARRE_ERREUR, BERR_TT_EFFACER , infos->nbr_choisis > 0);
	OBJ_active(BARRE_ERREUR, BERR_MUL_SIGMA_S, infos->nbr_choisis > 2); /* Pas de sens sinon */
	OBJ_active(BARRE_ERREUR, BERR_OK         , infos->nbr_choisis > 1); /* Pas de sens sinon */	

	if (infos->nbr_choisis <= nbr_visibles)
	{
		unsigned int objet = BERR_GRF_PREMNOM;
		Graphe *graphe = premier_graphe;
		int haut;

		OBJ_active(BARRE_ERREUR, BERR_GRF_AVANT  , FORCE_NON); /* Ascenseur inutile */
		OBJ_active(BARRE_ERREUR, BERR_GRF_ASC_FND, FORCE_NON);
		OBJ_active(BARRE_ERREUR, BERR_GRF_ASC_BOU, FORCE_NON);
		OBJ_active(BARRE_ERREUR, BERR_GRF_APRES  , FORCE_NON);

		OBJ_position(BARRE_ERREUR, BERR_GRF_ASC_BOU, 0, 0, &_rien_, &_rien_);
		OBJ_taille(BARRE_ERREUR, BERR_GRF_ASC_FND, -1, -1, &_rien_, &haut);
		OBJ_taille(BARRE_ERREUR, BERR_GRF_ASC_BOU, -1, haut, &_rien_, &_rien_);

		for	(i = 0 ; i < nbr_graphes ; i++, graphe = graphe->suivant)
		{
			if (infos->liste_graphes[i] == VRAI) /* Ce graphe est choisi */
			{
				OBJ_active (BARRE_ERREUR, objet, FORCE_OUI); /* On active */
				OBJ_choisit(BARRE_ERREUR, objet, objet == (infos->num_choisi + BERR_GRF_PREMNOM - 1) );
				met_chaine (BARRE_ERREUR, objet, 42, graphe->nom, TXT_COPIER);
				objet++;
			}
		}
		for ( ; objet <= BERR_GRF_DER_NOM ; objet++)
		{
			OBJ_active (BARRE_ERREUR, objet, FORCE_NON); /* On active */
			OBJ_choisit(BARRE_ERREUR, objet, FORCE_NON);
			met_chaine (BARRE_ERREUR, objet, 42, "", TXT_COPIER);			
		}
	}
	else
	{
		unsigned int objet = BERR_GRF_PREMNOM;
		unsigned long num_graphe = 0;
		Graphe *graphe = premier_graphe;
		float inter;
		int haut;

		for (i = 0 ; (i < nbr_graphes) && (objet <= BERR_GRF_DER_NOM) ; i++, graphe = graphe->suivant)
		{
			if ( infos->liste_graphes[i] == VRAI )
			{
				if (num_graphe >= infos->prem_affiche) /* On doit l'afficher */
				{
					OBJ_active (BARRE_ERREUR, objet, FORCE_OUI); /* On active */
					OBJ_choisit(BARRE_ERREUR, objet, objet == (infos->num_choisi + BERR_GRF_PREMNOM - 1) );
					met_chaine (BARRE_ERREUR, objet, 42, graphe->nom, TXT_COPIER);
					objet++;
				}
				num_graphe++;
			}
		}

		OBJ_active(BARRE_ERREUR, BERR_GRF_AVANT, infos->prem_affiche > 0);
		OBJ_active(BARRE_ERREUR, BERR_GRF_APRES, infos->prem_affiche + nbr_visibles < infos->nbr_choisis);

		OBJ_taille(BARRE_ERREUR, BERR_GRF_ASC_FND, -1, -1, &_rien_, &haut);

		inter  = (float) infos->nbr_choisis - nbr_visibles;
		inter /= (float) infos->nbr_choisis;
		inter  = (float) haut * ( 1 - inter );
		OBJ_taille(BARRE_ERREUR, BERR_GRF_ASC_BOU, -1, (int) inter, &_rien_, &_rien_);

		inter  = (float) infos->prem_affiche;
		inter /= (float) (infos->nbr_choisis - nbr_visibles);
		inter = (float) (haut - _rien_) * inter;
		OBJ_position(BARRE_ERREUR, BERR_GRF_ASC_BOU, 0, (int) inter, &_rien_, &_rien_);

		OBJ_active(BARRE_ERREUR, BERR_GRF_ASC_BOU, FORCE_OUI);
		OBJ_active(BARRE_ERREUR, BERR_GRF_ASC_FND, FORCE_OUI);
	}
}

/* ~~~~~~~~~~~~~~ Calcul effectif de la dispersion standard ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* -------------- Routine principale -----------------------------------------------------------\
 |    Renvoie le graphe contenant la dispersion sur les Y                                       |
 |
 | x_moyen     : contient les X du spectre moyen
 | y_moyen     : contient les Y du spectre moyen
 | infos       : contient les infos sur tous les spectres ayant servi au calcul                 |
 |               (voir graphes.h pour la structure TRI_infos)                                   |
 | nbr_graphes : nombre de graphes ayant servi au calcul                                        |
 | nbr_points  : nombre de points dans chaque graphe                                            |
 | correction  : facteur multiplicatif  appliquer sur chaque point pour corriger               |
 | moyenne     : valeur moyenne de l'cart type sur l'ensemble du spectre (NULL : pas de calcul)|
 \--------------------------------------------------------------------------------------------- */

Graphe *DISP_calc(double *x_moyen, double *y_moyen, TRI_infos *infos, unsigned long nbr_graphes,
                  unsigned long nbr_points, double correction, double *moyenne)
{
  Graphe *x_disperse, *y_disperse;
  unsigned long i, j;

  if ( (x_moyen == NULL) || (y_moyen == NULL) || (infos == NULL) )
    return NULL;

  if (nbr_graphes < 2)
  {
    EXF_ERREUR(DIV_ZERO);
    return NULL;
  }

  y_disperse = GRF_creer(nbr_points, GR_AUTRETYPE, GR_NORMAL, "Dispersion standard (cart type)", "Unit inconnue", "Unit inconnue", SS_UNITE, SS_UNITE);
  if (y_disperse == NULL) return NULL; /* Sert  rien du tout */

  for (i = 0 ; i < nbr_points ; i++) /* Pour chaque point */
  {
    double somme = 0;
    for (j = 0 ; j < nbr_graphes ; j++)
    {
      double carre = ( y_moyen[i] - (infos[j].ypoint_depart)[i] );
      somme += carre * carre;
    }
    somme /= (nbr_graphes - 1);
    (y_disperse->x)[i] = x_moyen[i];
    (y_disperse->y)[i] = sqrt(somme); /* cart type sur ce point = dispersion standard */
  }

  if ( (correction > 0) && (correction != 1) )
  {
    register double *ecart_type = y_disperse->y;

    for (i = 0 ; i < nbr_points ; i++, ecart_type++)
    {
      *ecart_type *= correction;
    }
  }

  if (moyenne) /* On demande la dviation standard moyenne */
  {
    double dev_moyenne = 0;
    register double *ecart_type = y_disperse->y;
    
    for (i = 0 ; i < nbr_points ; i++, ecart_type++)
    {
      dev_moyenne += 1/ (*ecart_type * *ecart_type); /* Somme des sigma2(i) */
    }
    dev_moyenne = nbr_points / dev_moyenne;
    *moyenne = sqrt(dev_moyenne);
  }

  GRF_bornes(y_disperse, DESORDONNE, DESORDONNE);
  return y_disperse;
}
