/* Traitement de l'E.X.A.F.S. sur Atari
                  Tout ce qui concerne l'export des graphes en images (GEM, PS, CVG, IMG)
   (c) 1997 - Emmanuel CURIS

 26 mars 1998
   * Nouveau mode d'appel des fonctions de trac : utilise les stations vdi internes, pas leur handle

 25 mars 1998
   * Utilisation de "systeme.h"

 12 janvier 1998
   * Export GEM : conversion pixels/mm plus correcte

  6 janvier 1998
   * Cration du fichier
   * Repris les fonctions concernes dans fichiers.c
   * Modifi export_img() : utilise Station_VDI
                            sauvegarde de la fentre de trac fonctionne! Joie immense!
   * Modifi export_gem() : utilise Station_VDI
                            sauvegarde de la fentre de trac intgre & fonctionne!
   * Supprim EXP_gem() car intgre dans export_gem()
*/

#define _EXPORT_ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"
#include "EXPORT.H"

/* ---------------- Format du type d'image  sauvegarder --------------------------------------\
 |   Ne renvoie rien
 |
 | type : doit-on exporter le graphe courant (FAUX) ou la fentre de trac (VRAI)              |
 \-------------------------------------------------------------------------------------------- */

void EXP_image(boolean type)
{
	switch ( ext_formdo(TYPE_IMAGE) )
	{
	case IMG_GEM: /* Image GEM vectorielle */
		export_gem(type);
		return;

	case IMG_IMG: /* Image GEM bitmap */
		export_img(type);
		return;

	case IMG_PS:  /* Fichier Postscript */
		export_ps(type);
		return;
	}
}

/* --------------------- Export d'une image vectorielle GEM ---------------------------------\
 |       Utilise le pilote 31 de GDOS / SpeedoGDOS / NVDI
 |
 | type : VRAI = on superpose les graphes; FAUX = on trace seulement le graphe courant
 \------------------------------------------------------------------------------------------ */

void export_gem(boolean type)
{
  Station_VDI *Station_GEM;
  int larg_page, haut_page;    /* Dimensions de la page (en 1/10 mm);            */
  int PX_haut;
  double epaisseur;            /* paisseur de trait, en mm */
  double x_echelle, y_echelle;
  char *nom_fichier;
	
  if (ext_formdo(PARAM_GEM) == GEM_ANNULE) return;

  { /* Vrification du format de la page */
    long larg, haut;

    larg = (long) 10.0 * prend_fvaleur(PARAM_GEM, GEM_LARGEUR); /* Dimensions en 1/10 mm */
    haut = (long) 10.0 * prend_fvaleur(PARAM_GEM, GEM_HAUTEUR);

    if ( (larg <= 0) || (haut <= 0) )
    {
      form_alert(1,"[2][ Dimensions de page incorrectes. | Sauvegarde impossible ! ][ Annuler ]");
      return;
    }
    if ( (larg > 32767) || (haut > 32767) )
    {
      form_alert(1,"[2][ Page trop grande ! | 3276 mm max ][ Bien ]");
      return;
    }

    larg_page = (int) larg; /* Dimensions finales - en 1/10 mm */
    haut_page = (int) haut;
  }

#ifdef _FRANCAIS_
  nom_fichier = SV_selecteur("*.GEM", "Cr"e_AIGU"er une image GEM");
#endif
#ifdef _ENGLISH_
  nom_fichier = SV_selecteur("*.GEM", "Create a GEM image");
#endif
  if (nom_fichier == NULL)
    return; /* Erreur lors de l'ouverture du fichier */

  Station_GEM = VDI_ouvre(31); /* Pilote 31 = meta.sys : fichier GEM */

  vm_filename   (Station_GEM->handle, nom_fichier); /* On fait ouvrir le fichier voulu */
  v_meta_extents(Station_GEM->handle, 0, 0, Station_GEM->larg, Station_GEM->haut); /* Zone de dessin */
  vm_pagesize   (Station_GEM->handle, larg_page, haut_page); /* Dimension de la page (en 1/10 mm) */

  epaisseur  = prend_fvaleur(PARAM_GEM, GEM_EPAISSEUR); /* paisseur en mm */
  epaisseur /= Station_GEM->x_pixel; /* paisseur en pixels */
  if ( epaisseur < 1) epaisseur = 1;                 /* paisseur minimale (en pixels) */
  PX_haut = (int) ( 10.0 * ( 0.3759 / Station_GEM->x_pixel ) ); /* Taille de la fonte en pixels - 10 points */

  vsl_width(Station_GEM->handle, (int) epaisseur); /* paisseur demande     */
/*	vst_font      (gem_handle, 1);     /* On utilise la premire fonte */
  vst_height(Station_GEM->handle, PX_haut, &_rien_, &_rien_, &_rien_, &_rien_);

  if (type == FAUX) /* Trac du graphe courant */
  {
    Rect_graphe rect_gem;        /* Rectangle qui contiendra le dessin  exporter  */

    /* Positionnement du rectangle de trac */

    rect_gem.x1 = (int) ( prend_fvaleur(PARAM_GEM, GEM_GAUCHE) / Station_GEM->x_pixel );
    rect_gem.y1 = (int) ( prend_fvaleur(PARAM_GEM, GEM_HAUTE ) / Station_GEM->y_pixel );
    rect_gem.x2 = Station_GEM->larg - (int) ( prend_fvaleur(PARAM_GEM, GEM_DROITE) / Station_GEM->x_pixel );
    rect_gem.y2 = Station_GEM->haut - (int) ( prend_fvaleur(PARAM_GEM, GEM_BASSE ) / Station_GEM->y_pixel );

    rect_gem.x_min = RCT_courant.x_min; 	/* Bornes de dessin  utiliser */
    rect_gem.y_min = RCT_courant.y_min;
    rect_gem.x_max = RCT_courant.x_max;
    rect_gem.y_max = RCT_courant.y_max;

    rect_gem.axe_x = RCT_courant.axe_x; 	/* Axes  utiliser */
    rect_gem.axe_y = RCT_courant.axe_y;

    rect_gem.nbr_graphes = 1;
    rect_gem.prem_graphe = RCT_courant.prem_graphe;

    strncpy(rect_gem.axe_x.legende, graphe_courant->axe_x, 50);
    strncpy(rect_gem.axe_y.legende, graphe_courant->axe_y, 50);

    /* On trace un cadre (noir) dlimitant la zone de dessin (+ axes, lgendes,...) */
    TRC_cadre(Station_GEM, &rect_gem, 0, 0);

    x_echelle = (double) (rect_gem.x2 - rect_gem.x1) / (rect_gem.x_max - rect_gem.x_min);
    y_echelle = (double) (rect_gem.y2 - rect_gem.y1) / (rect_gem.y_max - rect_gem.y_min);

    /* On trace le graphe (de sa couleur) */
    rect_gem.prem_graphe->graphe  = graphe_courant;
    rect_gem.prem_graphe->couleur = graphe_courant->couleur;
    rect_gem.prem_graphe->style   = graphe_courant->style;

    TRC_graphe(Station_GEM, rect_gem.prem_graphe, rect_gem.x1, rect_gem.y1,
	       rect_gem.x_min, rect_gem.x_max, rect_gem.y_min, rect_gem.y_max,
      	       x_echelle, y_echelle);
  }
  else /* Superposition de divers graphes */
  {
    Cadre *rect;
    float x_facteur, y_facteur; /* Facteurs d'chelle */

    x_facteur = ( (float) Station_GEM->larg ) / ( (float) trace_fenetre->w_larg );
    y_facteur = ( (float) Station_GEM->haut ) / ( (float) trace_fenetre->w_haut );

    for (rect = prem_rectangle ; rect ; rect = rect->apres)
    {
      rect->x *= x_facteur;    rect->y *= y_facteur;
      rect->larg *= x_facteur; rect->haut *= y_facteur;

      switch (rect->type)
      {
      case REC_GRAPHE: /* Contient des graphes - seul cas possible pour l'instant... */
	{
	  Rect_graphe *a_trace = (Rect_graphe *) rect->pointeur;
	
          a_trace->x1 *= x_facteur; a_trace->y1 *= y_facteur;
          a_trace->x2 *= x_facteur; a_trace->y2 *= y_facteur;

          DES_Rgraphe(Station_GEM, NULL, rect);

          a_trace->x1 /= x_facteur; a_trace->y1 /= y_facteur;
          a_trace->x2 /= x_facteur; a_trace->y2 /= y_facteur;
	}
	break;
      }

      rect->x    /= x_facteur; rect->y    /= y_facteur;
      rect->larg /= x_facteur; rect->haut /= y_facteur;
    }
  }

  /* On a fini */
  v_updwk(Station_GEM->handle);
  VDI_ferme(Station_GEM);
  free(nom_fichier);
}

/* ---------------- Export d'une image GEM bitmap (IMG) ---------------------------------------\
 |    Utilise le pilote IMG.SYS (91) de GDOS / SpeedoGDOS / NVDI pour cela...                  |
 |    Malheureusement, seulement en noir et blanc semble-t-il :-(                              |
 |
 | type : FAUX si trac du graphe courant, VRAI si trac de la fentre de tracs               |
 \-------------------------------------------------------------------------------------------- */

void export_img(boolean type)
{
  Station_VDI *Station_IMG; /* Station VDI associe */
  int x, y, larg, haut; /* Dimensions */
  int PX_haut;
  double x_echelle, y_echelle;

  Station_IMG = VDI_ouvre(91); /* Pilote IMG.SYS en 91, normalement */
  if (Station_IMG == NULL)
    return; /* Erreur survenue  l'ouverture de la station VDI */

  /* Dimensions de dessin */

  larg = (int) ( 0.85 * (double) Station_IMG->larg ); /* 9/10 de la largeur de la feuille */
  haut = (int) ( 0.80 * (double) Station_IMG->haut ); /* 4/5  de la hauteur de la feuille */
  x = (Station_IMG->larg - larg) / 2; /* Marges X : 1/2 - 1/2 */
  y = (Station_IMG->haut - haut) / 4; /* Marges Y : 1/4 - 3/4 */

  /* On trace le graphe */
  PX_haut = (int) ( 10.0 * 0.3759 / Station_IMG->x_pixel );
  /*	printf("Taille de fonte choisie : %d pixels\n", PX_haut); */

  vst_height(Station_IMG->handle, PX_haut, &_rien_, &_rien_, &_rien_, &_rien_);

  if ( type == FAUX ) /* Trac seulement du graphe courant */
  {
    Rect_graphe rect_gem; /* Rectangle qui contiendra le dessin  exporter  */

    /* Positionnement du rectangle de trac */

    rect_gem.x1 = x;        rect_gem.y1 = y;
    rect_gem.x2 = x + larg; rect_gem.y2 = y + haut;

    rect_gem.x_min = RCT_courant.x_min; 	/* Bornes de dessin  utiliser */
    rect_gem.y_min = RCT_courant.y_min;
    rect_gem.x_max = RCT_courant.x_max;
    rect_gem.y_max = RCT_courant.y_max;

    rect_gem.axe_x = RCT_courant.axe_x; 	/* Axes  utiliser */
    rect_gem.axe_y = RCT_courant.axe_y;

    rect_gem.nbr_graphes = 1;
    rect_gem.prem_graphe = RCT_courant.prem_graphe;

    strncpy(rect_gem.axe_x.legende, graphe_courant->axe_x, 50);
    strncpy(rect_gem.axe_y.legende, graphe_courant->axe_y, 50);

    /* On trace un cadre (noir) dlimitant la zone de dessin (+ axes, lgendes,...) */
    TRC_cadre(Station_IMG, &rect_gem, 0, 0);

    /* On trace le graphe (de sa couleur) */
    x_echelle = (double) (rect_gem.x2 - rect_gem.x1) / (rect_gem.x_max - rect_gem.x_min);
    y_echelle = (double) (rect_gem.y2 - rect_gem.y1) / (rect_gem.y_max - rect_gem.y_min);
    
    rect_gem.prem_graphe->graphe  = graphe_courant;
    rect_gem.prem_graphe->couleur = graphe_courant->couleur;
    rect_gem.prem_graphe->style   = graphe_courant->style;

    TRC_graphe(Station_IMG, rect_gem.prem_graphe, rect_gem.x1, rect_gem.y1,
	       rect_gem.x_min, rect_gem.x_max, rect_gem.y_min, rect_gem.y_max,
      	       x_echelle, y_echelle);
  }
  else /* Trac de la fentre de trac */
  {
    Cadre *rect;
    float x_facteur, y_facteur; /* Facteurs d'chelle */

    x_facteur = ( (float) Station_IMG->larg ) / ( (float) trace_fenetre->w_larg );
    y_facteur = ( (float) Station_IMG->haut ) / ( (float) trace_fenetre->w_haut );

    for (rect = prem_rectangle ; rect ; rect = rect->apres)
    {
      rect->x *= x_facteur;    rect->y *= y_facteur;
      rect->larg *= x_facteur; rect->haut *= y_facteur;

      switch (rect->type)
      {
      case REC_GRAPHE: /* Contient des graphes - seul cas possible pour l'instant... */
	{
	  Rect_graphe *a_trace = (Rect_graphe *) rect->pointeur;
					
          a_trace->x1 *= x_facteur; a_trace->y1 *= y_facteur;
          a_trace->x2 *= x_facteur; a_trace->y2 *= y_facteur;

          DES_Rgraphe(Station_IMG, NULL, rect);

          a_trace->x1 /= x_facteur; a_trace->y1 /= y_facteur;
          a_trace->x2 /= x_facteur; a_trace->y2 /= y_facteur;
	}
	break;
      }

      rect->x    /= x_facteur; rect->y    /= y_facteur;
      rect->larg /= x_facteur; rect->haut /= y_facteur;
    }
  }

  /* On a fini */
  v_updwk(Station_IMG->handle);
  VDI_ferme(Station_IMG); /* On ferme la station VDI */
}

