/* LASE - Logiciel d'analyse des spectres EXAFS
   (C) Emmanuel CURIS, 1998 - Cration du fichier le 14 juin 1998

   Sauvegarde d'un graphe au format postscript
   L'unit de base est le 1/100 de mm = taille d'un point donc 
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <SYSTEME.H>

#include <ERREURS.H>
#include <EXAFS.H>
#include <GRAPHES.H>

#include "PS_VDI.H"

/* ***** Les fonctions locales *************************************************************** */

static void PS_unique(void);
static void PS_cadre (Rect_graphe *rectangle);
static void PS_graphe(DES_graphe *graphe, int x, int y, 
		      double x_min, double x_max, double y_min, double y_max, 
		      double x_echelle, double y_echelle);

/* ***** Les variables locales *************************************************************** */

static FILE *fichier;
static float larg_page, haut_page; /* Dimensions de la page, en mm */

/* ---------------- Export d'un fichier Postscript --------------------------------------------\
 |   Routine gnrale : appel du slecteur et tout et tout                                     |
 |   Hypothse: on est sur une page A4 (portrait), en attendant mieux                         |
 |                                                                                             |
 | type : VRAI = la fentre de trac des graphes                                               |
 |        FAUX = juste la fentre contenant le graphe courant                                  |
 \-------------------------------------------------------------------------------------------- */

void export_ps(boolean type)
{
  char *nom_fichier;

  if ( ext_formdo(PARAM_GEM) == GEM_ANNULE )
    return; /* On annule l'impression */

  larg_page = (float) prend_fvaleur(PARAM_GEM, GEM_LARGEUR);
  haut_page = (float) prend_fvaleur(PARAM_GEM, GEM_HAUTEUR);

#ifdef _FRANCAIS_
  nom_fichier = SV_selecteur("*.ps", "Export d'un fichier Postscript");
#endif
#ifdef _ENGLISH_
  nom_fichier = SV_selecteur("*.ps", "Export a PostScript file");
#endif
  if (nom_fichier == NULL) /* Erreur  la sauvegarde */
    return;

  fichier = fopen(nom_fichier, "w"); /* Postscript : fichier texte, fins de lignes quelconques */
  if (fichier == NULL)
  {
    ERREUR(ERREUR_ECRITURE);
    return;
  }

  /* ---- On s'occupe de l'en-tte ---- */

  fprintf(fichier, "%%!PS-Adobe-minimally-conforming\n");
  fprintf(fichier, "%%%%Creator: LASE v. %.2f - (C) 1998, Emmanuel CURIS\n", VERSION / 10.0);
  if (type)
    fprintf(fichier, "%%%%Title: Superposition de divers graphes\n");
  else
    fprintf(fichier, "%%%%Title: Graphe %s\n", graphe_courant->nom);
  fprintf(fichier, "%%%%Pages: (atend)\n");
  fprintf(fichier, "%%%%DocumentFonts: Helvetica\n");
  fprintf(fichier, "%%%%EndComments\n");

  fprintf(fichier, "/Helvetica findfont\n"); /* On rcupre une fonte helvetica       */
  fprintf(fichier, "setfont\n");             /* On la choisit comme fonte courante    */

  /* --- On positionne l'origine --- */

  /*  fprintf(fichier, "100 100 translate -90 rotate\n"); /* Dplacement de l'origine */

  fprintf(fichier, "90 rotate\n"); /* Axe des x sur le grand ct */
  fprintf(fichier, "2.8346457e-2 2.8346457e-2 scale\n"); /* On passe en 1/100 mm [pour rester dans 1 int] */
  fprintf(fichier, "0 %f translate\n", - larg_page * 100); /* Origine : coin infrieur droit, page en paysage */

  /* ---- On va maintenant dessiner ---- */

  if (type == FAUX) /* Graphe seul pour l'instant */
  {
    PS_unique();
  }
  else
  {
  }

  /* ---- On a fini d'exporter ---- */

  fclose(fichier);
}

static void PS_unique(void)
{
  DES_graphe graphe;
  Rect_graphe rectangle;
  double x_echelle, y_echelle;
  float haut, bas, gauche, droite;

  haut = (float) prend_fvaleur(PARAM_GEM, GEM_HAUTE);
  bas  = (float) prend_fvaleur(PARAM_GEM, GEM_BASSE);
  gauche = (float) prend_fvaleur(PARAM_GEM, GEM_GAUCHE);
  droite = (float) prend_fvaleur(PARAM_GEM, GEM_DROITE);

  rectangle = RCT_courant;

  rectangle.x1 = gauche * 100; rectangle.y1 =  bas * 100;     /* Bord de 1cm */
  rectangle.x2 = (haut_page - droite) * 100; rectangle.y2 = (larg_page - haut) * 100;

  printf("Intervalle en X : %d %d\n", rectangle.x1, rectangle.x2);

  graphe.graphe  = graphe_courant;
  graphe.suivant = NULL;
  graphe.couleur = graphe_courant->couleur;
  graphe.style   = graphe_courant->style;

  rectangle.prem_graphe = &graphe;
  rectangle.nbr_graphes = 1;

  /* ----- On trace un cadre autour du graphe ----- */

  PS_cadre (&rectangle); /* On commence  1 cm du bord */
  if ( (rectangle.x_min == rectangle.x_max) /* Problme... */
    || (rectangle.y_min == rectangle.y_max) )
    return;

  /* ----- On trace le graphe associ ------------- */

  x_echelle = (double) (rectangle.x2 - rectangle.x1) / fabs(rectangle.x_max - rectangle.x_min);
  y_echelle = (double) (rectangle.y2 - rectangle.y1) / fabs(rectangle.y_max - rectangle.y_min);

  PS_graphe(&graphe, rectangle.x1, rectangle.y1, rectangle.x_min, rectangle.x_max,
	    rectangle.y_min, rectangle.y_max, x_echelle, y_echelle);

  fprintf(fichier, "showpage\n"); /* On a fini avec ce dessin... Sinon, imprime pas ! */
}

/* ----------- Trac d'un cadre, gradu ----------- */

static void PS_cadre(Rect_graphe *rectangle)
{
  double x_echelle, y_echelle;
  float haut_texte;
  int x, y;

  fprintf(fichier, "0 0 0 setrgbcolor "); /* Cadre en noir */
  fprintf(fichier, "10 setlinewidth\n");  /* paisseur de 1/10 mm */

  /* Le cadre proprement dit */

  fprintf(fichier, "%d %d moveto ", rectangle->x1, rectangle->y1);
  fprintf(fichier, "%d %d lineto ", rectangle->x1, rectangle->y2);
  fprintf(fichier, "%d %d lineto ", rectangle->x2, rectangle->y2);
  fprintf(fichier, "%d %d lineto ", rectangle->x2, rectangle->y1);
  fprintf(fichier, "closepath stroke\n");

  /* Vrifications des problmes ventuels */

  if ( (rectangle->x_min == rectangle->x_max) /* Problme... */
    || (rectangle->y_min == rectangle->y_max) )
  {
    fprintf(fichier, "%d %d moveto ", rectangle->x1, rectangle->y1);
    fprintf(fichier, "%d %d lineto stroke\n", rectangle->x2, rectangle->y2);
    fprintf(fichier, "%d %d moveto ", rectangle->x2, rectangle->y1);
    fprintf(fichier, "%d %d lineto stroke\n", rectangle->x1, rectangle->y2);
    return;
  }

  /* Facteurs d'chelle */

  x_echelle = (double) (rectangle->x2 - rectangle->x1) / fabs(rectangle->x_max - rectangle->x_min);
  y_echelle = (double) (rectangle->y2 - rectangle->y1) / fabs(rectangle->y_max - rectangle->y_min);

  /* Traitement de l'axe des X */

  if ( is_inf(rectangle->x_min) ) return; /* Une des bornes du graphe est infinie  */
  if ( is_inf(rectangle->x_max) ) return; /* On sort donc, sinon risque de planter */
  if ( is_nan(rectangle->x_min) ) return; /* Idem pour les "Not a Number"          */
  if ( is_nan(rectangle->x_max) ) return;

  if (rectangle->axe_x.graduation.gradue == GRADUE_DECIMAL) /* Graduation dcimale */
  {
    int lg_tiret; /* Dimension du tiret de graduation */
    int degre;    /* On travaille sur un intervalle de cette puissance de 10 l en largeur */
    double pas, x_dep;
    int y_2;

    PS_haut_texte(fichier, rectangle->axe_x.taille_txt / 100, &haut_texte); /* Hauteur du texte en points */
    PS_st_alignment(fichier, ALI_CENTER, ALI_TOP, NULL, NULL);

    lg_tiret = (rectangle->y2 - rectangle->y1) / 50;  /* 1/50 de la hauteur          */
    y_2 = rectangle->y1 + lg_tiret;       /* Position du haut du tiret    */

    degre = (int) ( log10( fabs(rectangle->x_max - rectangle->x_min) ) );
    pas = pow(10, degre); /* On commence par les graduations en unit */
    x_dep = floor(rectangle->x_min / pas) * pas;
    if (x_dep + pas == x_dep)
      return;
    while ( x_dep < rectangle->x_max )
    {
      if ( x_dep > rectangle->x_min )
      {
	int x_1 = rectangle->x1 + x_echelle * (x_dep - rectangle->x_min);
	sprintf(_intermediaire, "%.2f%c", x_dep,0);
	PS_gtext(fichier, x_1, rectangle->y1 + 5, _intermediaire);
	fprintf(fichier, "%d %d moveto %d %d lineto stroke\n", x_1, rectangle->y1, x_1, y_2);
      }
      x_dep += pas;
    }
    
    if ( rectangle->axe_x.graduation.dixieme ) /* On veut aussi les diximes */
    {
      int tic_5; /* Le dixime par lequel on commence */
	
      tic_5 = (int) ( ( rectangle->x_min - floor(rectangle->x_min / pas) * pas ) / (pas / 10) );
      pas /= 10;    /* On passe aux diximes */
      lg_tiret /= 4;
      x_dep = floor(rectangle->x_min / pas) * pas;
      if (x_dep + pas == x_dep)
	return;
      while ( x_dep < rectangle->x_max )
      {
	if ( x_dep > rectangle->x_min )
	{
	  int x_1 = rectangle->x1 + x_echelle * (x_dep - rectangle->x_min);
	  if ( tic_5 % 10 ) /* Ce n'est pas un multiple de 10 */
	  {
	    if ( (tic_5 % 5) == 0 ) /* Multiple de 5 */
	      y_2 = rectangle->y1 + 2 * lg_tiret;
	    else
	      y_2 = rectangle->y1 + lg_tiret;
	    fprintf(fichier, "%d %d moveto %d %d lineto stroke\n", x_1, rectangle->y1, x_1, y_2);
	  }
	}
	x_dep += pas;
	tic_5++; /* Graduation suivante */
      }

      if ( rectangle->axe_x.graduation.centieme ) /* On veut aussi les centimes */
      {
	tic_5 = (int) ( ( rectangle->x_min - floor(rectangle->x_min / (10 * pas) ) * (10 * pas) ) / (pas / 10) );
	pas /= 10;    /* On passe aux centimes */
	lg_tiret /= 2;
	x_dep = floor(rectangle->x_min / pas) * pas;
	if (x_dep + pas == x_dep)
	  return;
	while ( x_dep < rectangle->x_max )
	{
	  if ( x_dep > rectangle->x_min )
	  {
	    int x_1 = rectangle->x1 + x_echelle * (x_dep - rectangle->x_min);
	    if ( tic_5 % 10 ) /* Ce n'est pas un multiple de 10 */
	    {
	      if ( (tic_5 % 5) == 0 ) /* Multiple de 5 */
		y_2 = rectangle->y1 + 2 * lg_tiret;
	      else
		y_2 = rectangle->y1 + lg_tiret;
	      fprintf(fichier, "%d %d moveto %d %d lineto stroke\n", x_1, rectangle->y1, x_1, y_2);
	    }
	  }
	  x_dep += pas;
	  tic_5++; /* Graduation suivante */
	}
      }
    }
  }

  PS_st_alignment(fichier, ALI_CENTER, ALI_TOP, &_rien_, &_rien_);

  sprintf(_intermediaire, "%.2f%c", rectangle->x_min, 0);
  PS_gtext(fichier, rectangle->x1, rectangle->y1 - haut_texte, _intermediaire);

  sprintf(_intermediaire, "%.2f%c", rectangle->x_max, 0);
  PS_gtext(fichier, rectangle->x2, rectangle->y1 - haut_texte, _intermediaire);

  PS_gtext(fichier, rectangle->x1 + (rectangle->x2 - rectangle->x1) / 2, rectangle->y1 - haut_texte,
	   rectangle->axe_x.legende);

  /* On met les bornes & graduations en Y */		

  if ( is_inf(rectangle->y_min) ) return; /* Si elles sont infinies, plante... boucle sans fin */
  if ( is_inf(rectangle->y_max) ) return;
  if ( is_nan(rectangle->y_min) ) return; /* Idem pour les NaN */
  if ( is_nan(rectangle->y_max) ) return;

  if ( rectangle->axe_y.graduation.grille ) /* On veut une grille visible */
  {
    /*    TRC_grille(station, rectangle, x, y, AXE_Y, NON_TOURNE); */
  }

  /*  PS_st_color(fichier, rectangle->axe_y.couleurs.texte); /* Couleur des textes */
  /*  PS_sl_color(fichier, NOIR); */
  PS_sl_type (fichier, 0);

  if ( rectangle->axe_y.graduation.gradue == GRADUE_DECIMAL ) /* On veut graduer en Y */
  {
    int lg_tiret; /* Dimension du tiret de graduation */
    int degre;    /* On travaille sur un intervalle de cette puissance de 10 l en largeur */
    double pas, y_dep;
    int x_1, x_2;

    PS_st_alignment(fichier, ALI_RIGHT, ALI_HALF, &_rien_, &_rien_);

    lg_tiret = (rectangle->x2 - rectangle->x1) / 50;  /* 1/50 de la largeur           */
    x_1 = rectangle->x1;  /* Position de la base du tiret */

    degre = (int) ( log10( fabs(rectangle->y_max - rectangle->y_min) ) );
    pas = pow(10, degre); /* On commence par les graduations en unit */
    y_dep = floor(rectangle->y_min / pas) * pas;
    if (y_dep + pas == y_dep) /* Zoom trop grand -> prcision insuffisante */
    {
      rectangle->y_min = -2;
      rectangle->y_max = -2;
      return;
    }

    while ( y_dep < rectangle->y_max )
    {
      if ( y_dep > rectangle->y_min )
      {
	int y = rectangle->y2 - y_echelle * (y_dep - rectangle->y_min);
	sprintf(_intermediaire, "%.2f%c", y_dep,0);
	PS_gtext(fichier, points[0] - 5, points[1], _intermediaire);
	fprintf(fichier, "%d %d moveto %d %d lineto stroke\n", x_1, y, x_1 + lg_tiret, y);
      }
      y_dep += pas;
    }

    if ( rectangle->axe_y.graduation.dixieme ) /* On veut aussi les diximes */
    {
      int tic_5;

      tic_5 = (int) ( ( rectangle->y_min - floor(rectangle->y_min / pas) * pas ) / (pas / 10) );
      pas /= 10;    /* On passe aux diximes */
      lg_tiret /= 4;
      y_dep = floor(rectangle->y_min / pas) * pas;
      if (y_dep + pas == y_dep) /* Zoom trop grand -> prcision insuffisante */
      {
	rectangle->axe_y.graduation.dixieme = FAUX; /* On dsactive */
	return;
      }
      while ( y_dep < rectangle->y_max )
      {
	if ( y_dep > rectangle->y_min )
	{
	  int y = rectangle->y2 - y_echelle * (y_dep - rectangle->y_min);
	  if ( tic_5 % 10 ) /* Ce n'est pas un multiple de 10 */
	  {
	    if ( (tic_5 % 5) == 0 ) /* Multiple de 5 */
	      fprintf(fichier, "%d %d moveto %d %d lineto stroke\n", x_1, y, x_1 + 2 * lg_tiret, y);
	    else
	      fprintf(fichier, "%d %d moveto %d %d lineto stroke\n", x_1, y, x_1 + lg_tiret, y);
	  }
	}
	y_dep += pas;
	tic_5++; /* Graduation suivante */
      }

      if ( rectangle->axe_y.graduation.centieme ) /* On veut aussi les centimes */
      {
	tic_5 = (int) ( ( rectangle->y_min - floor(rectangle->y_min / (10 * pas)) * (10 * pas) ) / (pas / 10) );
	pas /= 10;    /* On passe aux centimes */
	lg_tiret /= 2;
	y_dep = floor(rectangle->y_min / pas) * pas;
	if (y_dep + pas == y_dep) /* Zoom trop grand -> prcision insuffisante */
	{
	  rectangle->axe_y.graduation.centieme = FAUX; /* On dsactive */
	  return;
	}
	while ( y_dep < rectangle->y_max )
	{
	  if ( y_dep > rectangle->y_min )
	  {
	    int y = rectangle->y2 - y_echelle * (y_dep - rectangle->y_min);
	    if ( tic_5 % 10 ) /* Ce n'est pas un multiple de 10 */
	    {
	      if ( (tic_5 % 5) == 0 ) /* Multiple de 5 */
		fprintf(fichier, "%d %d moveto %d %d lineto stroke\n", x_1, y, x_1 + 2*lg_tiret, y);
	      else
		fprintf(fichier, "%d %d moveto %d %d lineto stroke\n", x_1, y, x_1 + lg_tiret, y);
	    }
	  }
	  y_dep += pas;
	  tic_5++; /* Graduation suivante */
	}
      }
    }
  }

  /* Les bornes (aprs pour tre sr qu'elles soient visibles) */

  PS_st_alignment(fichier,ALI_RIGHT,ALI_HALF,&_rien_,&_rien_);

  sprintf(_intermediaire, "%.2f%c", rectangle->y_min, 0);
  PS_gtext(fichier, rectangle->x1 - 50, rectangle->y1, _intermediaire);

  sprintf(_intermediaire, "%.2f%c", rectangle->y_max, 0);
  PS_gtext(fichier, rectangle->x1 - 50, rectangle->y2, _intermediaire);
}

/* ------------ Dessine un graphe en postscript ----------- */

static void PS_graphe(DES_graphe *graphe, int x, int y, 
		      double x_min, double x_max, double y_min, double y_max, 
		      double x_echelle, double y_echelle)
{
  Graphe *courbe; /* Adresse du graphe  dessiner */
  unsigned long i = 0;
  register double *zone_x;
  int compteur = 0;

  courbe = graphe->graphe;
  zone_x = courbe->x;

  if ( is_inf(x_min) || is_inf(x_max) || is_inf(y_min) || is_inf(y_max) ) return;
  if ( is_nan(x_min) || is_nan(x_max) || is_nan(y_min) || is_nan(y_max) ) return;

  if (x_max < courbe->x_min) return; /* Graphe hors de la zone choisie */
  if (x_min > courbe->x_max) return;
  if (y_max < courbe->y_min) return;
  if (y_min > courbe->y_max) return;

  /* Style de dessin */

  if (graphe->style >= 7)
  {
    /*    vsm_color (fichier, graphe->couleur);
    vsm_height(fichier, 5); /* 5 pixels de haut pour le polymark... */
    /*    vsm_type  (fichier, graphe->style - 6); */
  }
  else
  {
    /*    vsl_color(fichier, graphe->couleur);
    vsl_type (fichier, graphe->style); */
  }

  if ( courbe->drapeaux.complexe == GR_NORMAL ) /* Graphe normal : cas facile... */
  {
    register double *zone_y = courbe->y;

    while (i < courbe->nombre_point) /* Tant qu'il reste des points */
    {
      while ( (*zone_x < x_min) || (*zone_x > x_max) /* On dborde */
	   || (*zone_y < y_min) || (*zone_y > y_max) )
      {
	zone_x++; zone_y++; /* On passe au point suivant */
	i++;
	if (i >= courbe->nombre_point) break; /* On s'arrte */
      }

      if (i >= courbe->nombre_point) break; /* On s'arrte */

      compteur = 0; /* On commence un nouveau trac */
      while ( (*zone_x >= x_min) && (*zone_x <= x_max) /* On dborde */
	   && (*zone_y >= y_min) && (*zone_y <= y_max) )
      {
	points[compteur]   = x + (*(zone_x) - x_min) * x_echelle;
	points[compteur+1] = y + (*(zone_y) - y_min) * y_echelle;
	compteur += 2;
	if (compteur >= 256) /* Tableau intin plein */
	{
	  if (graphe->style < 7)
	    PS_pline (fichier, 128, points); /* On trace ces 128 points */
	  else
	    PS_pmarker(fichier, 128, points);
	  points[0] = points[254];
	  points[1] = points[255];
	  compteur = 2;
	}
	zone_x++; zone_y++;
	i++; /* Point suivant */
	if (i >= courbe->nombre_point)
	{
	  if (compteur < 256)
	  {
	    if (graphe->style < 7)
	      PS_pline  (fichier, (int) (compteur - 1)/2, points); /* On trace ces 128 points */
	    else
	      PS_pmarker(fichier, (int) (compteur - 1)/2, points);
	  }
	  break;
	}
      }

      /* On trace s'il reste des points */
      if ( (compteur < 256) && (compteur > 0) )
      {
	if (graphe->style < 7)
	  PS_pline  (fichier, (compteur - 1)/2, points); /* On trace ce qui reste */
	else
	  PS_pmarker(fichier, (compteur - 1)/2, points);
      }
    }

    return; /* On a fini */
  }

  if (  ( courbe->drapeaux.complexe == GR_IM_MOD )
     || ( courbe->drapeaux.complexe == GR_RE_IM  ) ) /* Graphe complexe : tracer la partie imaginaire */
  {
    register Complexe *zone_y = (Complexe *) courbe->y;

    while (i < courbe->nombre_point) /* Tant qu'il reste des points */
    {
      while ( (*zone_x     < x_min) || (*zone_x     > x_max) /* On dborde */
           || ( zone_y->im < y_min) || ( zone_y->im > y_max) )
      {
	zone_x++; zone_y++; /* On passe au point suivant */
	i++;
	if (i >= courbe->nombre_point) break; /* On s'arrte */
      }

      if (i >= courbe->nombre_point) break; /* Mme remarque */

      compteur = 0; /* On commence un nouveau trac */
      while ( (*zone_x     >= x_min) && (*zone_x     <= x_max) /* On dborde */
	   && ( zone_y->im >= y_min) && ( zone_y->im <= y_max) )
      {
	points[compteur]   = x + (*(zone_x) -  x_min) * x_echelle;
	points[compteur+1] = y + (zone_y->im - y_min) * y_echelle;
	compteur += 2;
	if (compteur >= 256) /* Tableau intin plein */
	{
	  if (graphe->style < 7)
	    PS_pline  (fichier, 128, points); /* On trace ces 128 points */
	  else
	    PS_pmarker(fichier, 128, points);
	  points[0] = points[254];
	  points[1] = points[255];
	  compteur = 2;
	}
	zone_x++; zone_y++;
	i++; /* Point suivant */
	if (i >= courbe->nombre_point)
	{
	  if (compteur < 256)
	  {
	    if (graphe->style < 7)
	      PS_pline  (fichier, (int) (compteur - 1)/2, points); /* On trace ces 128 points */
	    else
	      PS_pmarker(fichier, (int) (compteur - 1)/2, points);
	  }
	  break;
	}
      }

      /* On trace s'il reste des points */
      if ( (compteur < 256) && (compteur > 0) )
      {
	if (graphe->style < 7)
	  PS_pline  (fichier, (compteur - 1)/2, points); /* On trace ce qui reste */
	else
	  PS_pmarker(fichier, (compteur - 1)/2, points);
      }
    }
  }

  if ( ( courbe->drapeaux.complexe == GR_IM_MOD )
    || ( courbe->drapeaux.complexe == GR_RE_MOD ) ) /* Graphe complexe : trac du module */
  {
    register Complexe *y_courbe = (Complexe *) courbe->y;
    double module;
    
    /*    vsl_color(fichier, graphe->couleur + 1); /* On utilise la couleur suivante */
    /*    vsm_color(fichier, graphe->couleur + 1); */

    zone_x = courbe->x;
    i = 0; compteur = 0; /* Remise des compteurs  0 */
    
    if ( y_max <= 0 ) /* Le module tant forcment positif, rien  tracer... */
      return;
    
    while (i < courbe->nombre_point) /* Tant qu'il reste des points */
    {
      module = sqrt( y_courbe->re * y_courbe->re + y_courbe->im * y_courbe->im );

      while ( (*zone_x < x_min) || (*zone_x > x_max) /* On dborde */
	   || ( module < y_min) || ( module > y_max) )
      {
	zone_x++; y_courbe++; /* On passe au point suivant */
	i++;
	if (i > courbe->nombre_point) break; /* On s'arrte */
	
	module = sqrt( y_courbe->re * y_courbe->re + y_courbe->im * y_courbe->im );
      }
      if (i > courbe->nombre_point) break; /* On s'arrte */

      compteur = 0; /* On commence un nouveau trac */
      while ( (*zone_x >= x_min) && (*zone_x <= x_max) /* On dborde */
	   && ( module >= y_min) && ( module <= y_max) )
      {
	points[compteur]   = x + (*(zone_x) -  x_min) * x_echelle;
	points[compteur+1] = y + (  module  - y_min) * y_echelle;
	compteur += 2;
	if (compteur >= 256) /* Tableau intin plein */
	{
	  if (graphe->style < 7)
	    PS_pline  (fichier, 128, points); /* On trace ces 128 points */
	  else
	    PS_pmarker(fichier, 128, points);
	  points[0] = points[254];
	  points[1] = points[255];
	  compteur = 2;
	}
	zone_x++; y_courbe++;
	i++; /* Point suivant */
	if (i >= courbe->nombre_point)
	{
	  if (compteur < 256)
	  {
	    if (graphe->style < 7)
	      PS_pline  (fichier, (int) (compteur - 1)/2, points); /* On trace ces 128 points */
	    else
	      PS_pmarker(fichier, (int) (compteur - 1)/2, points);
	  }
	  break;
	}
	
	module = sqrt( y_courbe->re * y_courbe->re + y_courbe->im * y_courbe->im );
      }

      /* On trace s'il reste des points */
      if ( (compteur < 256) && (compteur > 0) )
      {
	if (graphe->style < 7)
	  PS_pline  (fichier, (compteur - 1)/2, points); /* On trace ce qui reste */
	else
	  PS_pmarker(fichier, (compteur - 1)/2, points);
      }
    }
    
    return; /* On a fini */
  }
}
