/* Traitement de l'E.X.A.F.S. sur ATARI
   Extraction du signal EXAFS / XANES
   (c) 1998 - Emmanuel CURIS

   FICHIER  INCLURE

 22 mai 1998
   * Refait les types de courbes pour modliser mum_matrice

 17 mars 1998
   * Rcupr les dfinitions dans graphe.h
   * Dfinition de la fonction EXF_reproduire()
*/

#ifndef _EXTRACTION_H_

#define _EXTRACTION_H_ 

/* ---------- Extraction du signal X.A.N.E.S. ------------------- *
 *      (Module XANES.C)                                          *
 ******************************************************************/

typedef struct
{
  Graphe *brut;     /* Signal brut   */
  Graphe *ms_mumat; /* Brut moins mu matrice */
  Graphe *mu_0;     /* mu0 soustrait */
  double e_0;       /* Valeur de E0  */
  double e_1;       /* Valeur de E1 (pour ter mu matrice)     */
  double e_2;       /* Valeur de E2 (pout ter mu0, 1re tape) */
  int degre;        /* Degr de k^n * khi(k)                   */
} EXF_extrait; /* Tout pour l'extraction du signal EXAFS */

/* ---------- Intgration du prseuil --------------------------- */

#ifndef _XANES_

extern double INTG_min, INTG_max;

#endif

boolean XAN_integre(Graphe *signal);
double  XAN_intg_calcule(Graphe *graphe);

void    RDS_intg(Fenetre *fen, int x, int y, int larg, int haut);
boolean MSG_intg(Fenetre *fen);
boolean GER_intg(int num_objet,int x, int y);
boolean CLC_intg(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

/* ---------- Extraction du signal E.X.A.F.S. ------------------- *
 *      (Module EXAFS.C)                                          *
 ******************************************************************/

/* ---------- Reproduction de l'extraction  l'identique -------- */

void EXF_reproduit(Graphe *graphe);

boolean GER_reproduit(int objet, int x, int y);

void EXF_chparams(void); /* Charger - sauver params. d'extraction */
void EXF_svparams(void);

/* ---------- limination de [mu]0, premire tape -------------- */

typedef struct
{
  double e_0;       /* Valeur de E0                     */
  double e_2;       /* Valeur de E2                     */
  double coefs[11]; /* Coefficients du polynome utilis */
  int degre;        /* Degr du polynme utilis        */
  int methode;      /* Type de calcul du polynome       */
  Graphe *exafs;    /* Le signal EXAFS extrait          */
  Graphe *mu_0;     /* Le mu_0 utilis                  */
  int deg_affiche;  /* Degr pour affichage : k^n * spectre */
} MU1_infos;

void    MU1_ouvre (Graphe *brut);
boolean GER_mu01  (int objet, int x, int y);

void MU1_calc   (Graphe *brut, MU1_infos *infos);
void MU1_extrait(Graphe *brut, MU1_infos *infos);

/* ---------- limination de [mu]0, deuxime tape -------------- */

typedef struct
{
  double coefs[11]; /* Coefficients du polynme utilis     */
  int degre;        /* Degr du polynme utilis            */
  int methode;      /* Mthode de calcul du polynme        */
  Graphe *exafs;    /* Le signal EXAFS extrait              */
  Graphe *mu_0;     /* Le polynme soustrait                */
  Graphe *mu_0_ref; /* Le polynme soustrait, tape d'avant */
} MU2_infos;

void MU2_ouvre(Graphe *brut, Graphe *mu_0);

void    RDS_mu02(Fenetre *fen, int x, int y, int larg, int haut);
boolean MSG_mu02(Fenetre *fen);
boolean GER_mu02(int num_objet,int x, int y);

void MU2_calc   (Graphe *brut, MU2_infos *infos);
void MU2_extrait(Graphe *brut, MU2_infos *infos);

/* ---------- limination de [mu]0, troisime tape ------------- */

typedef struct
{
  double parametre; /* Valeur du paramtre de lissage       */
  Graphe *exafs;    /* Le signal EXAFS extrait              */
  Graphe *mu_0;     /* Le polynme soustrait                */
  Graphe *mu_0_ref; /* Le polynme soustrait, tape d'avant */
} MU3_infos;

void MU3_ouvre(Graphe *brut, Graphe *mu_0);

boolean GER_mu03(int num_objet,int x, int y);

void MU3_calc   (Graphe *brut, MU3_infos *infos);
void MU3_extrait(Graphe *brut, MU3_infos *infos);

/* ---------- limination d'un [mu]0 calcul par ailleurs ------- */

typedef struct
{
  double e_0;        /* nergie de seuil         */
  int methode;       /* Mthode de calcul du [mu]0 (en nergie, en nombre d'onde ?) */
  Graphe *mu_0;      /* Le graphe choisi...      */
  Graphe *exafs;     /* Le spectre EXAFS extrait */
  double *b, *c, *d; /* Tableaux de coefficients pour le lissage par splines */
  char degre;        /* Puissance  l'affichage  */
} MU0_infos;

void MU0_ouvre(Graphe *brut, Graphe *mu_0);

void    RDS_mu0(Fenetre *fen, int x, int y, int larg, int haut);
boolean MSG_mu0(Fenetre *fen);
boolean GER_mu0(int num_objet,int x, int y);
boolean CLC_mu0(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

void MU0_extrait(Graphe *brut, MU0_infos *infos);
void MU0_prepare(MU0_infos *infos);

/* ---------- Normalisation d'un spectre exafs ------------------ */

Graphe *norm_exafs(Graphe *source);

/* ---------- Diverses "macros" de calcul ----------------------- */

Graphe *magnifier   (Graphe *spectre); /* Multiplier par x^n */
Graphe *lisse_graphe(Graphe *spectre);
void    corr_energie(Graphe *spectre);
Graphe *convert     (Graphe *spectre); /* Conversion k <-> E */
double  cherche_e0  (Graphe *spectre, boolean affiche);

int EXF_e_v_k(Graphe *spectre, double e_0); /* Conversion E (eV)  -> k (1/) */
int EXF_k_v_e(Graphe *spectre, double e_0); /* Conversion k (1/) -> E (eV)  */
int EXF_k_p_n(Graphe *spectre, int degre); /* Multiplication des Y par X^n */

#define E_V_K_PAS_GRAPHE   -1 /* Pas de graphe pass            */
#define E_V_K_E0_FAUX      -2 /* Valeur de E0 transmise absurde */
#define E_V_K_VAL_NEGATIVE -3 /* On a pris E0 > x_min -> on a des k ngatifs ! */

#endif /* _EXTRACTION_H_ */
