/* Traitement de l'E.X.A.F.S. sur Atari
	 Tout ce qui concerne le calcul du spectre d'absorption  partir des fichiers exprimentaux
   (c) 1996  1998, Emmanuel CURIS - Cration du fichier : 27 mai 1998

	Calcul de l'absorption  partir de I0 et de I1 se trouvant dj en mmoire
	Rcupr de ../XANES.C
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SYSTEME.H>

#include <EXAFS.H>
#include <GRAPHES.H>

#include <ERREURS.H>
#include <EXTRAIT.H>
#include <SCIENCE.H>

#include "XT_CALC.H"

/* ------------- Conversion transmission <-> fluorescence -------------------------------- *
 *******************************************************************************************/

/* ------------- Calcul d'un spectre  partir de I0 et I1 -------------------------------- *
 *******************************************************************************************/

Graphe *XAN_extrait(void)
{
	EXT_infos infos;
	Graphe *spectre = NULL, *gr_i0, *gr_i1;
	int retour;
	unsigned long nbr_pts, index;
	boolean fluo = FAUX; /* VRAI si fluorescence */

	arbres[RECOMP_BRUT].pointeur = &infos;

	infos.i0 = GRF_par_type(GR_I0, VRAI); /* Liste des graphes de type I0 */
	infos.i1 = GRF_par_type(GR_I1, VRAI);
	if ( (infos.i0 == NULL) || (infos.i1 == NULL) )
	{
		free(infos.i1);
		free(infos.i0);
		return NULL;
	}

	GRF_affiche(infos.i0, RECOMP_BRUT, RCP_BRT_I0_1, RCP_BRT_I0_4,
	                    RCP_BRT_I0_AV, RCP_BRT_I0_AP, RCP_BRT_I0_ASC, RCP_BRT_I0_BOUT);
	GRF_affiche(infos.i1, RECOMP_BRUT, RCP_BRT_I1_1, RCP_BRT_I1_4,
	                    RCP_BRT_I1_AV, RCP_BRT_I1_AP, RCP_BRT_I1_ASC, RCP_BRT_I1_BOUT);

	retour = ext_formdo(RECOMP_BRUT);
	if (retour == RCP_BRT_ANNULE)
	{
		free(infos.i1->graphes);
		free(infos.i1);
		free(infos.i0->graphes);
		free(infos.i0);
		return NULL;
	}

	gr_i0 = (infos.i0->graphes)[infos.i0->num_select]; /* On rcupre les graphes choisis */
	gr_i1 = (infos.i1->graphes)[infos.i1->num_select];

	free(infos.i1->graphes);  /* On libre _toute_ la mmoire : plus besoin de a */
	free(infos.i1);
	free(infos.i0->graphes);
	free(infos.i0);

	nbr_pts = gr_i0->nombre_point;
	if (nbr_pts != gr_i1->nombre_point ) /* Pas le mme nombre de points */
	{
		EXF_ERREUR(NBRPTS_INCORRECT);
		return NULL;
	}

	if ( (gr_i0->x_min >= gr_i1->x_max) || (gr_i1->x_min >= gr_i0->x_max) )
	{
		EXF_ERREUR(ERR_DISJOINTS); /* Graphes sans X communs ! */
		return NULL;
	}

	fluo = OBJ_choisit(RECOMP_BRUT, RCP_BRT_FLUO, QUESTION);
	if ( fluo ) /* On est en fluorescence */
		sprintf(_intermediaire, "Signal brut (fluorescence)");
	else
		sprintf(_intermediaire, "Signal brut (absorption)");

	spectre = GRF_creer(nbr_pts, GR_BRUT, GR_NORMAL, _intermediaire, E_AIGU"nergie", "Absorbance", UNIT_EV, UNIT_ARB);
	if (spectre == NULL)
	{
		return NULL;
	}

	for (index = 0 ; index < nbr_pts ; index++)
	{
		(spectre->x)[index] = ( (gr_i0->x)[index] + (gr_i1->x)[index] ) / 2;

		if ( (gr_i0->y)[index] <= 0)
		{
			retour = EXT_ERREUR(EXT_IO_NUL, 0);
			if ( (retour == 3) || (index < 3) ) /* On annule le calcul */
			{
				GRF_efface(spectre, VRAI, VRAI);
				return NULL;
			}
			else if ( retour == 1 ) /* On interrompt le calcul */
			{
			  spectre->nombre_point = index;
			  break;
			}
			else
			{
			  (spectre->y)[index] = 0; /* Arbitraire... */
			}
		}
		else
		{
		  if ( fluo ) /* Spectre de fluorescence */
		  {
		    (spectre->y)[index] = (gr_i1->y)[index] / (gr_i0->y)[index];
		  }
		  else
		  {
		    if ( (gr_i1->y)[index] <= 0 )
		    {
		      retour = EXT_ERREUR(EXT_I1_NUL, 0); /* I1 incorrect pour de l'absorbance */
		      if ( (retour == 3) || (index < 3) ) /* On annule le calcul */
		      {
			GRF_efface(spectre, VRAI, VRAI);
			return NULL;
		      }
		      else if (retour == 1) /* On interrompt le calcul */
		      {
			spectre->nombre_point = index;
			break;
		      }
		      else
			(spectre->y)[index] = 0; /* Arbitraire... */
		    }
		    else
		    {
		      (spectre->y)[index] = log( gr_i0->y[index] / gr_i1->y[index] );
		    }
		  }
		}
	}

	GRF_bornes(spectre, DESORDONNE, DESORDONNE);

	return spectre;
}
