/* Traitement de l'E.X.A.F.S. sur Atari
	 Tout ce qui concerne le calcul du spectre d'absorption  partir des fichiers exprimentaux
   (c) 1996  1998, Emmanuel CURIS - Cration du fichier : 27 mai 1998

	Fichiers provenant du LURE : acquvisu (exafs III) ou logiciel de Robert Cortes (exafs II)
	                             traitement des en-ttes, analyse, lancement de la conversion
	Rcupr de ../FICHIERS.C
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SYSTEME.H>

#include <EXAFS.H>
#include <GRAPHES.H>

#include <ERREURS.H>
#include <EXTRAIT.H>
#include <FICHIERS.H>
#include <SCIENCE.H>

/* ~~~~~~~~~~~~~~~ Fichier du L.U.R.E., type ligne E.X.A.F.S. II ~~~~~~~~~~~~~~~~~~~~~~~~~ */

boolean ligne_l2(char *nom, MOD_extrait *type)
{
	FILE *fichier;
	boolean retour = VRAI;
	unsigned int index, nbr_spectres;
	unsigned long nbr_points, nbr_lignes = 0, en_tete;

	fichier = fopen(nom,"r");
	if (fichier == NULL)
	{
		form_alert(1,"[2][ Erreur au chargement du fichier | On passe au suivant ][ Suite ]");
		return FAUX;
	}

	/* On vrifie le format */
	fscanf(fichier, "%ld", &en_tete);
	if (en_tete == 0)
	{
		form_alert(1,"[3][ Format de fichier incorrect | Chargement annul ][ Suite ]"); 
	  return FAUX;
  }
	en_tete = 0;

	/* On passe les deux premire lignes */

	while( fgetc(fichier) != 10)
		en_tete++;
	en_tete+=2; /* Les 2 codes de fin de ligne */
	while( fgetc(fichier) != 10)
		en_tete++;
	en_tete+=2; /* Les 2 codes de fin de ligne */

	/* On compte le nombre de lignes */

	while ( feof(fichier) == FAUX ) /* Tant que l'on n'est pas au bout */
	{
		if (fgetc(fichier) == 10) /* Fin de ligne */
			nbr_lignes++;
	}

	/* On se place aprs l'en-tte pour commencer  lire */

	fseek(fichier, en_tete, SEEK_SET);

	/* On fait le chargement proprement dit */

	if (type->multi_spectres == FAUX) /* Un seul graphe dans le spectre */
	{
		boolean retour;

		if ( type->deux_chambres )                    /* Spectre  2 chambres d'ionisation */
			retour = DCI_2chambres(nom, fichier, nbr_lignes, type);
		else if ( type->multi_elements == FAUX )      /* Un seul lment dans le spectre   */
		  retour = DCI_mono(nom, fichier, nbr_lignes, type);
		else if ( (type->ajouter_elements  == VRAI)   /* On fait la somme des lments     */
		       || (type->moyenner_elements == VRAI) ) /* On fait la moyenne : mme chose   */
			return DCI_somme(nom, fichier, nbr_lignes, type);
		else
			retour = DCI_separe(nom, fichier, nbr_lignes, type);

		fclose(fichier); /* les DCI_??? ne ferment pas le fichier */
		return retour;
	}

	/* Si on est l, c'est que le fichier contient plusieurs spectres  la suite */

	nbr_spectres = prend_valeur(TYPE_LIGNE, NOMBRE_SPECTRES); /* Nombre de spectres */
	nbr_points   = nbr_lignes / nbr_spectres;       /* Nombre de points par spectre */
	if ( nbr_lignes % nbr_spectres ) /* Reste de la division non nul */
	{
		printf("Erreur : incohrence entre le nombre de lignes et le nombre de spectres\n");
		printf("Appuyez sur une touche\n");
		evnt_keybd();

		fclose(fichier);
		return FAUX;
	}

	if ( (type->ajouter_spectres  == VRAI)   /* On veut faire la somme de tous les spectres */
	  || (type->moyenner_spectres == VRAI) ) /* ou leur moyenne */
	{
		if (Histoire)
		{
			fprintf(Histoire, "              on ralise la somme de ces spectres\n");
			fprintf(Histoire, "   lments : 1\n");
		}

		return LII_somme(nom, fichier, nbr_spectres, nbr_lignes / nbr_spectres, type);
	}

	if (type->fondre_spectres == VRAI) /* On veut fondre tous les spectres */
	{
		if (Histoire)
		{
			fprintf(Histoire, "              on ralise la fusion de ces spectres\n");
			fprintf(Histoire, "   lments : 1\n");
		}

		return LII_fondre(nom, fichier, nbr_lignes, type);
	}

	/* Si on est l, c'est que l'on veut sparer tous les spectres : cas le plus simple */

	for (index = 0 ; index < nbr_spectres ; index++)
	{
		char nom_2[300];

		sprintf(nom_2, "Spectre %d de %s",index + 1, nom);
		if ( type->deux_chambres )                    /* Spectre  2 chambres d'ionisation */
			retour &= DCI_2chambres(nom_2, fichier, nbr_points, type);
		else if ( type->multi_elements == FAUX )      /* Un seul lment dans le spectre   */
	  	retour &= DCI_mono(nom_2, fichier, nbr_points, type);
		else if ( (type->ajouter_elements  == VRAI)   /* On fait la somme des lments     */
		       || (type->moyenner_elements == VRAI) ) /* On fait la moyenne : mme chose   */
			return DCI_somme(nom_2, fichier, nbr_points, type);
		else
			retour &= DCI_separe(nom_2, fichier, nbr_points, type);

		if (retour == FAUX) /* Erreur survenue */
		{
			break;
		}
	}
		
	fclose(fichier);
	return retour;
}

/* -------------- Fichier du L.U.R.E., type acquvisu [ligne E.X.A.F.S. I, III & XIII aussi] ----------------------------\
 | Format suppos : x lignes commenant par # -> commentaires, seulement en dbut du fichier
 |                  y lignes de donnes aprs : nergie i_0 (i_1 // i_1 i_2 // i1_1 i1_2 ... i1_n)
 |
 | nom  : le nom du fichier  charger
 | type : paramtres pour le type de signal enregistr
 \--------------------------------------------------------------------------------------------------------------------- */ 

boolean ligne_l3(char *nom, MOD_extrait *type)
{
	FILE *fichier;
	int c;
	unsigned long nbr_lignes = 0;
	unsigned long debut_donnees; /* Dbut des donnes = fin de l'en-tte */
	boolean nv_ligne = VRAI, retour, fin_en_tete = FAUX;

	fichier = fopen(nom, "rb"); /* Mode binaire pour tre tranquille avec les fins de ligne */
	if (fichier == NULL)
	{
		form_alert(1,"[2][ Erreur au chargement du fichier | On passe au suivant ][ Suite ]");
		return FAUX;
	}

	/* On passe l'en-tte */

	do
	{
		c = fgetc(fichier);
		if ( (c != '#') && (c != 13) && (c != 10) ) /* Tout sauf # en dbut de ligne */
		{
			debut_donnees = ftell(fichier) - 1; /* On ne compte pas le caractre dernirement lu */
			fin_en_tete = VRAI;
		}
		else if (c == '#') /* # = commentaire -> on ignore la fin de la ligne */
		{
			do
			{
				c = fgetc(fichier);
			} while ( (c != 13) && (c != 10) ); /* Fin de ligne */
		}
	} while (fin_en_tete == FAUX); /* Tant que l'on est dans l'en-tte */

	/* On compte le nombre de lignes */	

	do
	{
		c = fgetc(fichier);
		if ( ( (c == 13) || (c == 10) ) /* Code de fin de ligne */
		  && ( nv_ligne == FAUX ) )     /* Et on n'est pas au milieu d'une ligne */
		{
			nv_ligne = VRAI; /* On commence une nouvelle ligne */
			nbr_lignes++; /* Une ligne de donnes de plus */
		}
		else if ( (c != 13) && (c != 10) ) /* Autre chose qu'une fin de ligne */
		{
			nv_ligne = FAUX;
			if ( ( (c < '0') || (c > '9') ) /* Ce n'est pas un chiffre */
			  && ( (c != '.') && (c != 'e') && (c != 'E') && (c > 32) )
			    && (c != '+') && (c != '-') ) /* Ni un ., e, E, +, -, code de contrle ou espace */
			{
				if (EXT_ERREUR( CAR_INCONNU, c) == 1) /* On annule le chargement */
				{
				  fclose(fichier);
				  return FAUX;
				}
			}
		}
	}	while (c != EOF); /* Tant que l'on n'est pas en fin de fichier */

	if (nbr_lignes == 0)
	{
		printf("Aucune donne trouve dans ce fichier ! Chargement annul\n");
		fclose(fichier);
		return FAUX;
	}

	if (debut_donnees == 0)
	{
		if ( EXT_ERREUR(PAS_D_EN_TETE, 0) == 2) /* On interrompt le chargement */
		{
			fclose(fichier);
			return FAUX;
		}
	}

	/* On se place aprs l'en-tte pour commencer  lire */

	fseek(fichier, debut_donnees, SEEK_SET);

	if (type->deux_chambres) /* Travail avec deux chambres d'ionisation */
		retour = DCI_2chambres(nom, fichier, nbr_lignes, type);
	else if (type->multi_elements == FAUX) /* Un seul graphe dans le spectre */
		retour = DCI_mono(nom, fichier, nbr_lignes, type);
	else if ( (type->ajouter_elements == VRAI)    /* On veut faire la somme de tous les spectres */
	       || (type->moyenner_elements == VRAI) ) /*   ou en faire la moyenne */
	{
		retour = DCI_somme(nom, fichier, nbr_lignes, type);
	}
	else
	{
		retour = DCI_separe(nom, fichier, nbr_lignes, type);
	}
	fclose(fichier); /* DCI_??? ne ferment pas le fichier */

	return retour;
}
