/* Traitement de l'E.X.A.F.S. sur Atari
	 Tout ce qui concerne le calcul du spectre d'absorption  partir des fichiers exprimentaux
   (c) 1996  1998, Emmanuel CURIS - Cration du fichier : 27 mai 1998

	Fichiers multilments : sparation des spectres de tous les lments
	Fichiers multispectres : sparation de tous les spectres [pas ici...]
	Rcupr de ../FICHIERS.C

	 faire : - sauter fin du fichier si I1 = 0 et utilisateur le demande
	          - pas demander 36 fois si I0 = 0 et utilisateur demande d'ignorer
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SYSTEME.H>

#include <EXAFS.H>
#include <GRAPHES.H>

#include <ERREURS.H>
#include <EXTRAIT.H>
#include <FICHIERS.H>
#include <SCIENCE.H>

/* #include "ELEMENTS.H" */

/* --------------- Fichiers multi-lments en colonnes : sparation des spectres ----------\
 |     Renvoie VRAI si tout s'est bien pass, FAUX en cas d'erreur                         |
 |             les tableaux passs en paramtres remplis avec le spectre                   |
 |     Suppose un fichier de lignes E I0 I1_1 I1_2 ... I1_nbr_elements
 |                                                                                         |
 | fichier      : le fichier  lire, positionn au dbut des donnes pour ce spectre       |
 | nbr_points   : le nombre de points dans chacun des spectres                             |
 | nbr_elements : le nombre d'lments dans le fichier                                     |
 | type         : informations pour l'extraction (fluorescence ou absorption, en gros      |
 | x            : tableau qui contiendra les nergies (x) pour le spectre lu               |
 | y            : y[0] ... y[n-1] contiennent les n spectres des lments                  |
 | i_0          : contient la valeur de i_0 (commune  tous les lments)                  |
 | i_1          : comme y, mais contient les valeurs de i_1                                |
 \---------------------------------------------------------------------------------------- */

boolean ELEM_separe(FILE *fichier, unsigned long nbr_points, unsigned int nbr_elements,
		    MOD_extrait *type,
                    double *x, double **y, double *i_0, double **i_1)
{
  double I0, I1;            /* Les valeurs lues dans le fichier */
  unsigned long index;      /* Point en cours de lecture        */
  unsigned int num_element; /* lment en cours de lecture      */

  if ( (x == NULL) || ( (y == NULL) && (i_0 == NULL) && (i_1 == NULL) ) )
  {
    return FAUX; /* Rien pour enregistrer des donnes ! */
  }

  /* On lit le fichier */

  for (index = 0 ; index < nbr_points ; index++, x++)
  {
    fscanf(fichier,"%lf%lf", x, &I0); /* Valeurs communes  tous les spectres */

    if (i_0) /* On charge les I0 */
    {
      *i_0 = I0;
      i_0++;
    }

    if (I0 == 0) /* Valeur aberrante -> problme lors de l'enregistrement ? */
    {
      int reponse = EXT_ERREUR(EXT_IO_NUL, 0);
      if ( (reponse == 3) || (index < 2) )/* On annule le chargement */
      {
	return FAUX; /* On annule le chargement */
      }
      else if (reponse == 1) /* On saute toutes les lignes suivantes - fin du spectre  0 */
      {
	for ( ; index < nbr_points ; index++ )
	{
	  for (num_element = 0 ; num_element < nbr_elements ; num_element++) /* Fin de la ligne */
	  {
	    fscanf(fichier, "%lf", &I1);
	    if ( (y) && ( y[num_element] ) ) /* On veut cette valeur du spectre */
	      y[num_element][index] = 0; /* Indfinie --> on prend 0 pour simplifier */
	    if ( (i_1) && (i_1[num_element]) ) /* On veut i_1 pour ce spectre */
	      i_1[num_element][index] = I1;
	  }

	  x++; /* On enregistre le x suivant */

	  fscanf(fichier, "%lf%lf", x, &I0); /* Dbut de ligne suivante */
	  if (i_0)
	  {
	    *i_0 = I0;
	    i_0++;
	  }
	}
	for (num_element = 0 ; num_element < nbr_elements ; num_element++)
	{
	  fscanf(fichier, "%lf", &I1); /* On lit la fin de la dernire ligne */

	  if ( (y) && ( y[num_element] ) ) /* On veut ce spectre */						
	    y[num_element][index] = 0; /* Indfinie --> on prend 0 pour simplifier */

	  if ( (i_1) && (i_1[num_element]) ) /* On veut i_1 pour ce spectre */
	    i_1[num_element][index] = I1;
	}
					 
	return VRAI;
      }
      else /* On continue en "ignorant" ce point malencontreux */
      {
	for (num_element = 0 ; num_element < nbr_elements ; num_element++)
	{
	  fscanf(fichier, "%lf", &I1); /* On lit la fin de la dernire ligne */

	  if ( (y) && ( y[num_element] ) ) /* On veut ce spectre */						
	    y[num_element][index] = 0; /* Indfinie --> on prend 0 pour simplifier */

	  if ( (i_1) && (i_1[num_element]) ) /* On veut i_1 pour ce spectre */
	    i_1[num_element][index] = I1;
	}
      }
    }
    else
    {
      /* On est l dans le cas normal (I0 non nul) ou demande de continuer */

      for (num_element = 0 ; num_element < nbr_elements ; num_element++)
      {
	fscanf(fichier, "%lf", &I1); /* Valeur de I1 pour cet lment */
	if ( (y) && (y[num_element]) ) /* On veut calculer ce spectre */
	{
	  if (type->fluoresce) /* On est en fluorescence */
	  {
	    y[num_element][index] = I1 / I0;
	  }
	  else                 /* On est en absorbance   */
	  {
	    if (I1 == 0)
	    {
	      int reponse = EXT_ERREUR(EXT_I1_NUL, 0);
	      if (reponse == 3) /* On demande d'arrter le chargement */
		return FAUX;
	      if (reponse == 1) /* On demande de sauter la suite */
	      {
		/* Pas encore ralis de faon propre... */
	      }
	      y[num_element][index] = 0;
	    }
	    else   	
	      y[num_element][index] = log( I0 / I1 ); /* logarithme nprien */
	  }
	}
	
	if ( (i_1) && (i_1[num_element]) ) /* On veut garder la valeur de I1 pour cet lment */
	{
	  i_1[num_element][index] = I1;
	}
      }
    }
  }
  
  return VRAI; /* Chargement a eu lieu dans problme */
}

/* ------------------ Ligne du DCI, 1 spectre, n lments, sparation des spectres de chacun -----------------\
 |     Renvoie VRAI si tout s'est bien pass, FAUX sinon                                                      |
 |  ATTENTION : ne ferme pas le fichier, quoi qu'il arrive [car utilis plusieurs fois si graphes  la suite] |
 |                                                                                                            |
 | nom        : le nom du fichier trait                                                                      |
 | fichier    : le fichier trait (positionn au dbut des donnes  lire)                                    |
 | nbr_lignes : nombre de lignes  traiter                                                                    |
 | type       : type de traitement  leur faire subir                                                         |
 \----------------------------------------------------------------------------------------------------------- */

boolean DCI_separe(char *nom, FILE *fichier, unsigned long nbr_lignes, MOD_extrait *type)
{
  Graphe **spectres = NULL, *gr_i0 = NULL, **gr_i1 = NULL;
  double *x = NULL, *I0 = NULL, **y = NULL, **I1 = NULL; /* Endroits o on lit les donnes */
  unsigned int nbr_elements, num_element;
  boolean reponse;

  nbr_elements = prend_valeur(TYPE_LIGNE, NOMBRE_ELEMENTS); /* Nombre d'lments utiliss */
  if (nbr_elements <= 0)
  {
    EXT_ERREUR(AUCUN_ELEMENT, 0);
    return FAUX;
  }

  /* On prpare tous les spectres ncessaires */
  
  if ( type->garde_i0 )
  {
    char titre[300];

    sprintf(titre, "%s (I0 - commun)", nom);
    gr_i0 = GRF_creer(nbr_lignes, GR_I0, GR_NORMAL, titre, E_AIGU"nergie", "Nombre de coups", UNIT_EV, SS_UNITE);
    if (gr_i0 == NULL)
      type->garde_i0 = FAUX; /* Impossible de conserver I0 en plus */
    else
    {
      x = gr_i0->x;
      I0 = gr_i0->y;
      if ( Histoire )
      {
	fprintf(Histoire, "   I0       : %s (%ld points)\n", gr_i0->nom, gr_i0->nombre_point);
      }
    }
  }

  if ( type->garde_signal )
  {
    spectres = (Graphe **) malloc( (size_t) ( sizeof(Graphe *) * nbr_elements ) );
    if (spectres == NULL)
    {
      ERREUR(MANQUE_MEMOIRE);
      type->garde_signal = FAUX;
    }
    else /* On rserve les spectres qui reoivent le signal */
    {
      y = (double **) malloc( sizeof(double *) * nbr_elements );
      if (y == NULL)
      {
	ERREUR(MANQUE_MEMOIRE);
	type->garde_signal = FAUX;
	free(spectres);
	spectres = NULL;
      }
    }
  }
  if ( type->garde_signal ) /* On peut effectivement garder les spectres */
  {
    char titre[300];
    int i;

    for (i = 0 ; i < nbr_elements ; i++)
    {
      sprintf(titre, "%s - "e_AIGU"l"e_AIGU"ment %d", nom, i + 1);
#ifdef _FRANCAIS_
      spectres[i] = GRF_creer(nbr_lignes, GR_BRUT, GR_NORMAL, titre,
			      E_AIGU"nergie", "Absorbance", UNIT_EV, UNIT_ARB);
#endif
#ifdef _ENGLISH_
      spectres[i] = GRF_creer(nbr_lignes, GR_BRUT, GR_NORMAL, titre,
			      "Energy", "Absorbance", UNIT_EV, UNIT_ARB);
#endif

      if ( spectres[i] )
      {
	if (x == NULL)
	  x = spectres[i]->x;  /* Zone de lecture des X */
	y[i] = spectres[i]->y; /* Zone de lecture des Y */
	spectres[i]->couleur = i + 1;
      }
      else
      {
	y[i] = NULL; /* On n'enregistre pas ce spectre */
      }
    }
  }

  if ( type->garde_i1 )
  {
    gr_i1 = (Graphe **) malloc( (size_t) ( sizeof(Graphe *) * nbr_elements ) );
    if (gr_i1 == NULL)
    {
      ERREUR(MANQUE_MEMOIRE);
      type->garde_i1 = FAUX; /* Impossible de conserver I1 en plus */
    }
    else
    {
      I1 = (double **) malloc ( (size_t) ( sizeof( double * ) * nbr_elements ) );
      if (I1 == NULL)
      {
	ERREUR(MANQUE_MEMOIRE);
	type->garde_i1 = FAUX;
	free(gr_i1);
	gr_i1 = NULL;
      }
    }
  }
  if ( type->garde_i1 ) /* On peut effectivement garder I1 */
  {
    char titre[300];
    int i;

    for (i = 0 ; i < nbr_elements ; i++)
    {
      sprintf(titre, "%s - I1, "e_AIGU"l"e_AIGU"ment %d", nom, i + 1);
      gr_i1[i] = GRF_creer(nbr_lignes, GR_I1, GR_NORMAL, titre, E_AIGU"nergie", "Nombre de coups", UNIT_EV, SS_UNITE);
      if ( gr_i1[i] )
      {
	if (x == NULL)
	  x = gr_i1[i]->x;  /* Zone de lecture des X */
	I1[i] = gr_i1[i]->y; /* Zone de lecture des Y */
	gr_i1[i]->couleur = i + 1;
      }
      else
      {
	I1[i] = NULL; /* On n'enregistre pas ce spectre */
      }
    }
  }
  
  /* On lit le fichier, tranquillement */

  reponse = ELEM_separe(fichier, nbr_lignes, nbr_elements, type, x, y, I0, I1);
  if (reponse == FAUX) /* Une erreur est survenue -> on efface les graphes */
  {
    int i;

    GRF_efface(gr_i0, VRAI, VRAI);
    for (i = 0 ; i < nbr_elements ; i++)
    {
      if (spectres)
	GRF_efface(spectres[i], VRAI, VRAI); /* On efface quoi qu'il arrive */
      if (gr_i1)
	GRF_efface(gr_i1[i], VRAI, VRAI);
    }
    free(spectres); /* Plus besoin de a */
    free(gr_i1);
    free(y);
    free(I1);
    
    return FAUX; /* Problme... */
  }

  /* On recopie les x avant de faire quoi que ce soit, sinon rsultat trange ! */

  if (spectres)
    for (num_element = 0 ; num_element < nbr_elements ; num_element++)
    {
      if (spectres[num_element]) /* Ce spectre a bien t utilis */
	if ( x!= spectres[num_element]->x ) /* Il faut installer les X */
	  memcpy(spectres[num_element]->x, x, sizeof(double) * nbr_lignes);	
    }

  if (gr_i1)
    for (num_element = 0 ; num_element < nbr_elements ; num_element++)
    {
      if (gr_i1[num_element]) /* Ce spectre a bien t utilis */
	if ( x!= gr_i1[num_element]->x ) /* Il faut installer les X */
	  memcpy(gr_i1[num_element]->x, x, sizeof(double) * nbr_lignes);	
    }

  /* On termine l'init. des graphes */

  reponse = FAUX; /* On n'a pas prvenu l'utilisateur en cas de doublons */
  
  if ( gr_i0 ) /* Il y a un graphe contenant les I0 */
  {
    if (type->tri) /* On veut raliser un tri avant */
      tri_en_place(gr_i0);
	
    if (type->doublons) /* On limine les doublons au vol */
    {
      unsigned long nbr_doublons = tri_doublons(gr_i0, VRAI);
      if (nbr_doublons)
      {
	reponse = VRAI; /* On ne prvient qu'une seule fois ! */
	sprintf(_intermediaire,"[3][ On a moyenn | %ld doublons ][ Compris ]", nbr_doublons);
	form_alert(1,_intermediaire);
	if ( Histoire )
	  fprintf(Histoire, "              contenait %ld doublons (moyenns)\n", nbr_doublons);
      }
    }
    GRF_bornes(gr_i0, DESORDONNE, DESORDONNE);
  }

  for (num_element = 0 ; num_element < nbr_elements ; num_element++)
  {
    if ( Histoire )
      fprintf(Histoire, "   --> "E_AIGU"l"e_AIGU"ment num"e_AIGU"ro %d\n", num_element + 1);

    if ( (spectres) && (spectres[num_element]) ) /* Le spectre existe bien */
    {
      if (Histoire)
	fprintf(Histoire, "   Signal   : %s (%ld points)\n", spectres[num_element]->nom, spectres[num_element]->nombre_point);

      if (type->tri)      /* On trie le spectre au vol */
	tri_en_place(spectres[num_element]);
	    
      if (type->doublons) /* On limine les doublons au vol */
      {
	unsigned long nbr_doublons;
	nbr_doublons = tri_doublons(spectres[num_element], VRAI); /* On les moyenne */
	if (nbr_doublons)
	{
	  if ( !reponse )
	  {
	    reponse = VRAI; /* On ne prvient qu'une seule fois ! */
	    sprintf(_intermediaire,"[3][ On a moyenn"e_AIGU" | %ld doublons ][ Compris ]", nbr_doublons);
	    form_alert(1,_intermediaire);
	  }

	  if (Histoire)
	    fprintf(Histoire, "              contenait %ld doublons (moyenn"e_AIGU"s)\n", nbr_doublons);
	}
      }			
      
      GRF_bornes(spectres[num_element], DESORDONNE, DESORDONNE); /* Les bornes */
    }
    else
    {
      if (Histoire)
	fprintf(Histoire, "   Signal   : ignor"e_AIGU"\n");
    }

    if ( (gr_i1) && (gr_i1[num_element]) ) /* Le graphe des I1 existe bien */
    {
      if (Histoire)
	fprintf(Histoire, "   I1       : %s (%ld points)\n", gr_i1[num_element]->nom, gr_i1[num_element]->nombre_point);

      if (type->tri)      /* On trie le spectre au vol */
	tri_en_place(gr_i1[num_element]);
      
      if (type->doublons) /* On limine les doublons au vol */
      {
	unsigned long nbr_doublons = tri_doublons(gr_i1[num_element], VRAI); /* On les moyenne */
	if (nbr_doublons)
	{
	  sprintf(_intermediaire,"[3][ On a moyenn | %ld doublons ][ Compris ]", nbr_doublons);
	  form_alert(1,_intermediaire);

	  if (Histoire)
	    fprintf(Histoire, "              contenait %ld doublons (moyenn"e_AIGU"s)\n", nbr_doublons);
	}
      }			
      
      GRF_bornes(gr_i1[num_element], DESORDONNE, DESORDONNE); /* Les bornes */
    }
    else if (type->garde_i1) /* On a pourtant demand de garder I1 */
    {
      if (Histoire)
	fprintf(Histoire, "   I1       : ignor"e_AIGU"\n");
    }		
  }

  /* On met les bornes, les titres,... */

  if (spectres)
    GRF_aller(spectres[0]);

  if ( type->garde_i1 ) free( gr_i1 );
  free(spectres);
  free(y); free(I1);

  return VRAI;
}
