/* Traitement de l'E.X.A.F.S. sur Atari
	 Tout ce qui concerne le calcul du spectre d'absorption  partir des fichiers exprimentaux
   (c) 1996  1998, Emmanuel CURIS - Cration du fichier : 27 mai 1998

	Fichiers multilments : somme de tous les lments
	Fichiers multispectres : somme de tous les spectres _par lment_, le traitement par lment est fait _avant_
	Rcupr de ../FICHIERS.C

   faire : - interrompre proprement si I0 = 0 et utilisateur demande de sauter
            - cas de la somme de spectres de multi-lments si : > somme / moyenne des lments
                                                                 > sparation des lments
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SYSTEME.H>

#include <EXAFS.H>
#include <GRAPHES.H>

#include <ERREURS.H>
#include <EXTRAIT.H>
#include <FICHIERS.H>
#include <SCIENCE.H>

/* ****** Fonctions internes  ce module *************************************************** */

static boolean SOM_un_element(const char *nom, FILE *fichier, unsigned int nbr_spectres, unsigned long nbr_lignes, MOD_extrait *type);

/* ****** Traitement des fichiers multi-lments, 1 spectre *********************************
 *                                                                                          *
 * Format suppos : energie i_0 i_element_1 ... i_element_n                              *
 *                  une ligne par point exprimental, pas de ligne de commentaire entre                       *
 **************************************************************************************************************/

/* --------------- Lignes du DCI, un seul spectre dans le fichier, n lments : on les somme -----------------\
 |    Renvoie VRAI si le chargement s'est bien pass, FAUX sinon                                              |
 |    Suppose un fichier de format ENERGIE IO I1_1 I1_2... pour charger correctement                          |
 |                                                                                                            |
 | nom        : le nom du fichier charg                                                                      |
 | fichier    : le fichier  lire, positionn au dbut des donnes                                            |
 | nbr_lignes : le nombre de lignes  lire (contiennent les donnes)                                          |
 | type       : infos pour le chargement                                                                      |
 \----------------------------------------------------------------------------------------------------------- */

boolean DCI_somme(char *nom, FILE *fichier, unsigned long nbr_lignes, MOD_extrait *type)
{
	Graphe *spectre = NULL, *gr_i0 = NULL, *gr_i1 = NULL; /* Graphes contenant le rsultat */
	double *x = NULL, *I0 = NULL, *I1 = NULL, *y = NULL;  /* Valeurs des zones de lecture  */
	unsigned int nbr_elements;
	boolean reponse, *util_element;
	char methode[13]; /* Type de calcul ralis */

	/* On prpare pour les lments */

	nbr_elements = prend_valeur(TYPE_LIGNE, NOMBRE_ELEMENTS); /* Nombre d'lments utiliss */
	if (nbr_elements <= 0)
	{
		form_alert(1,"[3][ Vous avez oubli de spcifier | le nombre d'lments ! ][ Annuler ]");
		return FAUX;
	}

	util_element = (boolean *) malloc( sizeof(boolean) * nbr_elements);
	if (util_element == NULL)
	{
		ERREUR(MANQUE_MEMOIRE);
		return FAUX;
	}

	if ( OBJ_choisit(TYPE_LIGNE, MULTI_ELEM_TOUS, QUESTION) == VRAI ) /* On les utilise tous */
	{
		unsigned int i;

		for (i = 0 ; i < nbr_elements ; i++)
			util_element[i] = VRAI;
	}
	else /* On veut une slection */
	{
		int reponse;
		unsigned int i;

		for (i = 0 ; i < 22 ; i++) /* On prpare ce qu'il faut... */
		{
			OBJ_active (CHOIX_SPECTRE, EXTRAIT_1 + i, (i < nbr_elements) );
			OBJ_choisit(CHOIX_SPECTRE, EXTRAIT_1 + i, (i < nbr_elements) ); /* Par dfaut, tous actifs... */
		}

		do
		{
			reponse = ext_formdo(CHOIX_SPECTRE); /* Quels spectres prendre en compte ? */
			if (reponse == SOMME_ANULLE)
			{
				free(util_element);
				return FAUX; /* On annule le chargement */
			}
			
			if (reponse == EXTRAIT_TOUS) /* On active tous les boutons */
			{
				for (i = 0 ; (i < 22) && (i < nbr_elements) ; i++)
					OBJ_choisit(CHOIX_SPECTRE, EXTRAIT_1 + i, FORCE_OUI );
			}

			if (reponse == EXTRAIT_AUCUN) /* On dsactive tous les lments */
			{
				for (i = 0 ; (i < 22) && (i < nbr_elements) ; i++)
					OBJ_choisit(CHOIX_SPECTRE, EXTRAIT_1 + i, FORCE_NON );
			}
		} while (reponse != SOMME_OK);

		for (i = 0 ; i < nbr_elements ; i++)
		{
			if ( i >= 22 ) /* Plus de 22 lments -> on garde tous ceux d'aprs */
				util_element[i] = VRAI;
			else
				util_element[i] = OBJ_choisit(CHOIX_SPECTRE, EXTRAIT_1 + i, QUESTION);
		}
	}

	/* On rserve le graphe correspondant */

	if ( type->moyenner_elements )
		strcpy(methode, "Moyenne");
	else
		strcpy(methode, "Somme");

	if ( type->garde_signal)
	{
		char titre[300];

		sprintf(titre, "%s (%s)", nom, methode);
		spectre = GRF_creer(nbr_lignes, GR_BRUT, GR_NORMAL, titre, "nergie", "Absorbance", UNIT_EV, UNIT_ARB);
		if (spectre == NULL)
		{
			type->garde_signal = FAUX; /* On ne garde pas le spectre, finalement */
		}
		else
		{
			x = spectre->x;    /* On crit les X dans x... */
			y = spectre->y;    /* On crit les Y dans y... */

			if (Histoire)
			{
				fprintf(Histoire, "   Signal   : %s (%ld points)\n", spectre->nom, spectre->nombre_point);
			}
		}
	}

	if ( type->garde_i0 ) /* On veut garder i0 */
	{
		char titre[300];

		sprintf(titre, "%s (I0)", nom);
		gr_i0 = GRF_creer(nbr_lignes, GR_I0, GR_NORMAL, titre, "nergie", "Nombre de coups", UNIT_EV, SS_UNITE);
		if (gr_i0 == NULL)
		{
			type->garde_i0 = FAUX; /* On ne peut pas le garder... */
		}
		else
		{
			if (x == NULL) /* Pas encore de x dfini */
				x = gr_i0->x;
			I0 = gr_i0->y; /* On enregistre I0 */

			if (Histoire)
			{
				fprintf(Histoire, "   I0       : %s (%ld points)\n", gr_i0->nom, gr_i0->nombre_point);
			}
		}
	}

	if ( type->garde_i1 ) /* On veut garder i1 */
	{
		char titre[300];

		sprintf(titre, "%s (I1 - %s)", nom, methode);
		gr_i1 = GRF_creer(nbr_lignes, GR_I1, GR_NORMAL, titre, "nergie", "Nombre de coups", UNIT_EV, SS_UNITE);
		if (gr_i1 == NULL)
		{
			type->garde_i1 = FAUX; /* On ne peut pas le garder... */
		}
		else
		{
			if (x == NULL)
				x = gr_i1->x; /* Si zone de lecture des x pas encore dfinie */
			I1 = gr_i1->y; /* Adresse de lecture de I1 */

			if (Histoire)
			{
				fprintf(Histoire, "   I1       : %s (%ld points)\n", gr_i1->nom, gr_i1->nombre_point);
			}
		}
	}

	if ( x == NULL ) /* Aucune information n'est charge */
	{
		printf("Aucune information  charger ! Chargement annul\n");
		printf(" ------- Appuyer sur une touche --------\n");
		evnt_keybd();
		free(util_element);
		return FAUX;
	}

	/* On lit le fichier */

	reponse = ELEM_somme(fichier, nbr_lignes, nbr_elements, type, x, y, I0, I1, util_element);
	if (reponse == FAUX) /* Une erreur est survenue -> on efface les graphes */
	{
		GRF_efface(spectre, VRAI, VRAI); /* On efface quoi qu'il arrive */
		GRF_efface(gr_i0, VRAI, VRAI);
		GRF_efface(gr_i1, VRAI, VRAI);

		free(util_element);

		return FAUX; /* Problme... */
	}

	/* Lecture termine */
	/* On recopie les X dans les spectres qui ne les ont pas */

	if ( (gr_i0) && (x != gr_i0->x) ) /* Le spectre existe et ne contient pas les X */
	{
		memcpy(gr_i0->x, x, sizeof(double) * nbr_lignes);
	}

	if ( (gr_i1) && (x != gr_i1->x) )
	{
		memcpy(gr_i1->x, x, sizeof(double) * nbr_lignes);
	}

	/* Si demand, on tri tout a */

	if (type->tri)
	{
		tri_en_place(spectre);
		tri_en_place(gr_i0);
		tri_en_place(gr_i1);
	}

	/* limination des ventuels doublons */

	if (type->doublons) /* On limine les doublons au vol */
	{
		if (spectre)
		{
			unsigned long nbr_doublons = tri_doublons(spectre, VRAI); /* On les moyenne */
			if (nbr_doublons)
			{
				sprintf(_intermediaire,"[3][ On a moyenn | %ld doublons ][ Compris ]", nbr_doublons);
				form_alert(1,_intermediaire);
			}
			else
				form_alert(1,"[3][ Aucun doublon n'a t | mis en vidence ][ Bien ]");
		}

		if (type->garde_i0)
			tri_doublons(gr_i0, VRAI);
		if (type->garde_i1)
			tri_doublons(gr_i1, VRAI);
	}

	GRF_bornes(spectre, DESORDONNE, DESORDONNE); /* On met les bornes - aprs car doublons peuvent modifier */
	GRF_bornes(gr_i0, DESORDONNE, DESORDONNE);   /* NULL : revient directement -> pas de problme...     */
	GRF_bornes(gr_i1, DESORDONNE, DESORDONNE);

	GRF_aller(spectre);

	free(util_element);
	return VRAI;
}

/* --------------- Fichiers multi-lments en colonnes : somme des spectres ---------------\
 |     Renvoie VRAI si tout s'est bien pass, FAUX en cas d'erreur                         |
 |             les tableaux passs en paramtres remplis avec le spectre                   |
 |     Suppose un fichier de lignes E I0 I1_1 I1_2 ... I1_nbr_elements                     |
 |                                                                                         |
 | fichier      : le fichier  lire, positionn au dbut des donnes pour ce spectre       |
 | nbr_points   : le nombre de points dans chacun des spectres                             |
 | nbr_elements : le nombre d'lments dans le fichier                                     |
 | type         : informations pour l'extraction (fluorescence ou absorption, en gros      |
 | x            : tableau qui contiendra les nergies (x) pour le spectre lu               |
 | y            : comme x, mais pour la somme des lments lus                             |
 | i_0          : contient la valeur de i_0 (commune  tous les lments)                  |
 | i_1          : contient la valeur de la somme des i_1                                   |
 | util_element : ...[i] contient VRAI si on doit utiliser l'lment i, FAUX sinon         |
 \---------------------------------------------------------------------------------------- */

boolean ELEM_somme(FILE *fichier, unsigned long nbr_points, unsigned int nbr_elements, MOD_extrait *type,
                    double *x, double *y, double *i_0, double *i_1, boolean *util_element)
{
	double I0, I1;            /* Les valeurs lues dans le fichier */
	unsigned long index;      /* Point en cours de lecture        */
	unsigned int num_element; /* lment en cours de lecture      */
	unsigned int total_elements = 0; /* Nombre d'lments rellement utiles */

	if ( (x == NULL) || ( (y == NULL) && (i_0 == NULL) && (i_1 == NULL) ) )
	{
		return FAUX; /* Rien pour enregistrer des donnes ! */
	}

	/* On met  zro la rgion qui va recevoir les sommes */
	
	for (index = 0 ; index < nbr_points ; index++)
	{
		if (y)     y[index] = 0;
		if (i_1) i_1[index] = 0;
	}

	/* On compte combien d'lments servent rellement  faire le calcul */

	if ( type->moyenner_elements)
	{
		for (num_element = 0 ; num_element < nbr_elements ; num_element++)
		{
			if ( util_element[num_element] == VRAI ) /* On utilise cet lment */
				total_elements++;
		}
	}

	/* On lit le fichier */

	for (index = 0 ; index < nbr_points ; index++, x++)
	{
		fscanf(fichier,"%lf%lf", x, &I0); /* Valeurs communes  tous les spectres */

		if (i_0) /* On charge les I0 */
		{
			*i_0 = I0; i_0++;
		}

		if (I0 == 0) /* Valeur aberrante -> problme lors de l'enregistrement ? */
		{
			int reponse = EXT_ERREUR(EXT_IO_NUL, 0);
			if ( (reponse == 3) || (index < 2) )/* On annule le chargement */
			{
				return FAUX; /* On annule le chargement */
			}
			else if (reponse == 1) /* On saute toutes les lignes suivantes - fin du spectre mise  zro */
			{
				for (num_element = 0 ; num_element < nbr_elements ; num_element++)
				{
					fscanf(fichier, "%lf", &I1); /* On lit la fin de la ligne */
					if ( (i_1) && (util_element[num_element]) ) /* On veut i_1 pour ce spectre */
						i_1[index] += I1;
				}
				for ( index++ ; index < nbr_points ; index++, x++ )
				{
					fscanf(fichier, "%lf%lf", x, &I0);
					if (i_0)
					{
						*i_0 = I0;
						i_0++;
					}

					for (num_element = 0 ; num_element < nbr_elements ; num_element++)
					{
						fscanf(fichier, "%lf", &I1); /* On lit la fin de la ligne */
						if ( (i_1) && (util_element[num_element]) ) /* On veut i_1 pour ce spectre */
							i_1[index] += I1;
					}
				}
				 
				return VRAI;
			}
		}

		/* On est l dans le cas normal (I0 non nul) */

		for (num_element = 0 ; num_element < nbr_elements ; num_element++)
		{
			fscanf(fichier, "%lf", &I1); /* Valeur de I1 pour cet lment */
			if ( (y) && (util_element[num_element]) ) /* On veut calculer le spectre avec cet lment */
			{
				if (type->fluoresce) /* On est en fluorescence */
				{
				  y[index] += I1 / I0; /* y = (I1 + ... + In) / I0 = <I> / I0 */
				}
				else                 /* On est en transmission */
			  {
				  if (I1 == 0)
				  {
				  	int reponse = EXT_ERREUR(EXT_I1_NUL, 0);
				  	if (reponse == 3)
				  	{
				  		return FAUX; /* On annule le chargement */
				  	}
				  }
				  else
				  {
				 	  if ( type->elm_sigma_log ) /* On fait la somme des logarithmes [npriens] */
					    y[index] += log( I0 / I1 ); /* y = (ln I0/I1 + ... + ln I0/In) = < ln I0/I > */
					  else
							y[index] += I1 / I0; /* Moyenne des I1  : y = (I1 + ... + In) / I0 */
					}
				}
			}

			if ( (i_1) && (util_element[num_element]) ) /* On veut garder la valeur de I1 pour cet lment */
			{
				i_1[index] += I1;
			}
		}

		if ( type->moyenner_elements ) /* On fait les moyennes, pas la somme */
		{
			if (y)
				y[index] /= (double) total_elements; /* signal = (I1 + ... + In) / I0 */

			if (i_1)
				i_1[index] /= (double) total_elements;
		}

		if ( (type->fluoresce == FAUX) && (type->elm_sigma_log == FAUX) )
			y[index] = -log( y[index] ); /* y = ln [ I0 / (I1 + ... + In) ] = ln (I0 / <I>) */
	}

	return VRAI; /* Chargement a eu lieu dans problme */
}

/* ************** Traitement des fichiers multi-spectres, 1 spectre *******************************************
 *                                                                                                            *
 * Format suppos : energie i_0 i_element_1 ... i_element_n                               } par spectre       *
 *                  une ligne par point exprimental, pas de ligne de commentaire entre   }                   *
 *                  n spectres  la suite dans le fichier                                                     *
 **************************************************************************************************************/

/* -------------- Prparation du travail + rpartition entre les diffrentes fonctions internes --------------\
 |  Renvoie VRAI si tout s'est bien pass                                                                     |
 |                                                                                                            |
 | nom          =                                                                                             |
 | fichier      = Le fichier  charger                                                                        |
 | nbr_ligne    = nombre de lignes  lire _par spectre_                                                       |
 | nbr_spectres = nombre de spectres  la suite                                                               |
 | type         = conditions exprimentales + oprations  effectuer                                          |
 \----------------------------------------------------------------------------------------------------------- */

boolean LII_somme(char *nom, FILE *fichier, unsigned int nbr_spectres, unsigned long nbr_ligne, MOD_extrait *type)
{
	unsigned int nbr_elements;

  if ( type->multi_elements ) /* C'est en plus un fichier multi-lments */
  {
		nbr_elements = prend_valeur(TYPE_LIGNE, NOMBRE_SPECTRES); /* Nombre d'lments utiliss */
		if (nbr_elements == 0)
		{
			form_alert(1,"[3][ Vous avez oubli de spcifier | le nombre d'lments ! ][ Annuler ]");
			fclose(fichier);
			return FAUX;
		}
	}
	else
		nbr_elements = 1; /* Un seul "lment" */

	if ( nbr_elements == 1 ) /* Un seul lment */
	{
		boolean retour = SOM_un_element((const char *) nom, fichier, nbr_spectres, nbr_ligne, type);
		fclose(fichier);
		return retour;
	}

	/* Ici, cas hautement plus complexe : il y a plusieurs lments -> que fait-on ? */

	return FAUX; /* Rien pour l'instant... */
}

/* -------------- Cas le plus simple : il n'y a qu'un seul lment, plusieurs spectres successifs ------------\
 |  Renvoie VRAI si tout s'est bien pass - NE ferme PAS le fichier avant de sortir                           |
 |                                                                                                            |
 | nom          : le nom du fichier [pour mettre le titre du graphe...]                                       |
 | fichier      : le fichier  charger                                                                        |
 | nbr_ligne    : nombre de lignes  lire _par spectre_                                                       |
 | nbr_spectres : nombre de spectres  la suite                                                               |
 | type         : conditions exprimentales + oprations  effectuer                                          |
 \----------------------------------------------------------------------------------------------------------- */

static boolean SOM_un_element(const char *nom, FILE *fichier,
                              unsigned int nbr_spectres, unsigned long nbr_lignes, MOD_extrait *type)
{
	char titre[300];
	Graphe *spectre, *gr_i0, *gr_i1; /* Les spectres obtenus */
	unsigned int i; /* Index sur les spectres */

	if ( type->choix_spectres ) /* On veut ne garder que certains spectres */
	{
		int retour;

 		for (i = 0 ; i < 22 ; i++)
			OBJ_active(CHOIX_SPECTRE, EXTRAIT_1 + i, i < nbr_spectres); /* Actif seulement si le spectre existe */

		do
		{
			retour = ext_formdo(CHOIX_SPECTRE);

			if (retour == SOMME_ANULLE) /* Chargement annul */
				return FAUX;

			if (retour == EXTRAIT_TOUS)
			{
				for (i = 0 ; (i < 22) && (i < nbr_spectres) ; i++)
					OBJ_choisit(CHOIX_SPECTRE, EXTRAIT_1 + i, FORCE_OUI);
			}

			if (retour == EXTRAIT_AUCUN)
			{
				for (i = 0 ; (i < 22) && (i < nbr_spectres) ; i++)
					OBJ_choisit(CHOIX_SPECTRE, EXTRAIT_1 + i, FORCE_NON);
			}
		} while (retour != SOMME_OK); /* Tant que l'on ne confirme pas */
  }

	if (type->garde_signal)
	{
#ifdef _FRANCAIS_
  	sprintf(titre, "%s (somme des spectres)", nom);
    spectre = GRF_creer(nbr_lignes, GR_BRUT, GR_NORMAL, titre, E_AIGU"nergie", "Absorbance", UNIT_EV, SS_UNITE);
#endif
#ifdef _ENGLISH_
  	sprintf(titre, "Sum of spectra of %s", nom);
    spectre = GRF_creer(nbr_lignes, GR_BRUT, GR_NORMAL, titre, "Energy", "Absorbance", UNIT_EV, SS_UNITE);
#endif

  	if (spectre == NULL)
  		type->garde_signal = FAUX;
		else
		{
			register double *y;
			unsigned long i;

      for (i = 0, y = spectre->y ; i < nbr_lignes ; i++, y++) /* On met  zro pour recevoir la somme */
      	*y = 0;		
		}
	}

	if ( type->garde_i0 )
	{
		sprintf(titre, "%s (somme des I0)", nom);
		gr_i0 = GRF_creer(nbr_lignes, GR_I0, GR_NORMAL, titre, E_AIGU"nergie", "Nombre de coups", UNIT_EV, SS_UNITE);
		if (gr_i0 == NULL)
			type->garde_i0 = FAUX;
		else
		{
			register double *y;
			unsigned long i;

      for (i = 0, y = gr_i0->y ; i < nbr_lignes ; i++, y++) /* On met  zro pour recevoir la somme */
      	*y = 0;		
		}
	}

	if ( type->garde_i1 )
	{
		sprintf(titre, "%s (somme des I1)", nom);
		gr_i1 = GRF_creer(nbr_lignes, GR_I1, GR_NORMAL, titre, E_AIGU"nergie", "Nombre de coups", UNIT_EV, SS_UNITE);
		if (gr_i1 == NULL)
			type->garde_i1 = FAUX;
		else
		{
			register double *y;
			unsigned long i;

      for (i = 0, y = gr_i1->y ; i < nbr_lignes ; i++, y++) /* On met  zro pour recevoir la somme */
      	*y = 0;		
		}
	}

	if ( (spectre == NULL) && (gr_i0 == NULL) && (gr_i1 == NULL) )
	{
		printf("Aucune donne n'a t demande ! Aucun chargement  effectuer\n");
		return FAUX;
	}

  /* On lit les spectres, uniquement si l'utilisateur veut les lire */

	for (i = 0 ; i < nbr_spectres ; i++)
	{
		unsigned long j;
		register double *x = NULL, *y_signal = NULL, *y_i0 = NULL, *y_i1 = NULL;

		if ( type->garde_signal ) /* On fait le calcul */
		{
			x = spectre->x; y_signal = spectre->y;
		}
		if ( type->garde_i0 )
		{
			if (x == NULL) x = gr_i0->x;
			y_i0 = gr_i0->y;
		}
		if ( type->garde_i1 )
		{
			if (x == NULL) x = gr_i1->x;
			y_i1 = gr_i1->y;
		}

		for (j = 0 ; j < nbr_lignes ; j++, x++) /* On lit pour ce spectre */
		{
			double energie, i_0, i_1;

			fscanf(fichier, "%lf %lf %lf", &energie, &i_0, &i_1);

			if ( (type->choix_spectres == FAUX) || (i > 22) /* Seulement 22 boutons dans la bote, pour l'instant... */
			  || ( OBJ_choisit(CHOIX_SPECTRE, EXTRAIT_1 + i, QUESTION) == VRAI ) )
			{
				*x = energie;

				if (y_signal)
				{
					if (i_0 == 0) /* Gros problme */
					{
						int reponse = EXT_ERREUR(EXT_IO_NUL, 0);
						if (reponse == 3) /* On annule le chargement */
						{
							GRF_efface(spectre, VRAI, VRAI);
							GRF_efface(gr_i0, VRAI, VRAI); /* Si pas utilis, NULL -> pas de problme */
  						GRF_efface(gr_i1, VRAI, VRAI);

	  					return FAUX;
		  			}
		  			if (reponse == 1) /* On demande de sauter la suite */
		  			{
		  			}
		  		}
	  			else
	  			{
						if (type->fluoresce == FAUX) /* En transmission */
						{
							if ( i_1 == 0 )
							{
								int reponse = EXT_ERREUR(EXT_I1_NUL, 0);
								if (reponse == 3) /* On annule le chargement */
								{
     							GRF_efface(spectre, VRAI, VRAI);
			     				GRF_efface(gr_i0, VRAI, VRAI); /* Si pas utilis, NULL -> pas de problme */
  					    	GRF_efface(gr_i1, VRAI, VRAI);

    	  					return FAUX;
								}
							}
							else
								*y_signal += log( i_0 / i_1);
						}
						else
							*y_signal += i_1  / i_0;
					}
					y_signal++;
			  }
				
				if (type->garde_i0)
				{
					*y_i0 += i_0;
					y_i0++;
				}
				if (type->garde_i1)
				{
					*y_i1 += i_1;
					y_i1++;
				}
			}
		}
	}

  /* On met les X si ncessaire */

  if ( type->garde_i0 )
  {
  	if (spectre)
  		memcpy(gr_i0->x, spectre->x, sizeof(double) * gr_i0->nombre_point);
  }
  if ( type->garde_i1 )
  {
  	if (spectre)
  		memcpy(gr_i1->x, spectre->x, sizeof(double) * gr_i1->nombre_point);
  	else if (gr_i0)
  		memcpy(gr_i1->x, gr_i0->x  , sizeof(double) * gr_i1->nombre_point);
  }

	/* Si demand, on fait la moyenne des spectres somms */

	if ( type->moyenner_spectres ) /* On veut la moyenne */
	{
		unsigned int nbr_util = 0; /* Nombre rel de spectres utiliss dans le fichier */

		/* On compte le nombre de spectres effectivement pris en comptes */
		if ( type->choix_spectres == FAUX ) /* On utilise tous les spectres */
		{
			nbr_util = nbr_spectres;
		}
		else
		{
			for (i = 0 ; i < 22 ; i++)
			{
				if ( OBJ_choisit(CHOIX_SPECTRE, EXTRAIT_1 + i, QUESTION) == VRAI )
					nbr_util++;
			}
			if (nbr_spectres > 22)
				nbr_util += nbr_spectres - 22;
		}

		/* On divise tout le spectre par le nombre d'lments */

		if (type->garde_signal)
		{
			register double *y = spectre->y;
			unsigned long j;

   		for (j = 0 ; j < spectre->nombre_point ; j++, y++)
		  {
			  *y /= nbr_util;
	  	}
	  }
		if ( type->garde_i0 )
		{
			register double *y = gr_i0->y;
			unsigned long j;

   		for (j = 0 ; j < gr_i0->nombre_point ; j++, y++)
		  {
			  *y /= nbr_util;
	  	}
		}
		if ( type->garde_i1 )
		{
			register double *y = gr_i1->y;
			unsigned long j;

   		for (j = 0 ; j < gr_i1->nombre_point ; j++, y++)
		  {
			  *y /= nbr_util;
	  	}
		}
	}

	/* limination des ventuels doublons */

	if (type->doublons) /* On limine les doublons au vol */
	{
	  unsigned long nbr_doublons = tri_doublons(spectre, VRAI); /* On les moyenne */

	  if (type->garde_signal)
	  {
	    if (nbr_doublons)
	    {
	      sprintf(_intermediaire,"[3][ On a moyenn | %ld doublons ][ Compris ]", nbr_doublons);
	      form_alert(1, _intermediaire);
	    }
	    else
	      form_alert(1,"[3][ Aucun doublon n'a t | mis en vidence ][ Bien ]");
	  }

	  if (type->garde_i0)
	    tri_doublons(gr_i0, VRAI);
	  if (type->garde_i1)
	    tri_doublons(gr_i1, VRAI);
	}

	/* Recherche des bornes */

	if (type->garde_signal)
	  GRF_bornes(spectre, ORDONNE, DESORDONNE);
	if (type->garde_i0)
	  GRF_bornes(gr_i0, ORDONNE, DESORDONNE);
	if (type->garde_i1)
	  GRF_bornes(gr_i1, ORDONNE, DESORDONNE);

	GRF_aller(spectre);

	return VRAI; /* Charg */
}
