/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
				   Tout ce qui concerne le paramtrage de FEFF (shell en particulier)
				   Fichier d'en-tte : toutes les dclarations ncessaires
   (c) Emmanuel CURIS 1996 - 1997

 16 janvier 1998
   * Modifi enregistrement des paramtres de CONTROL et PRINT : une seule variable

 12 janvier 1998
   * Ajout dfinition de FEFF_termine() : comme son nom l'indique...

  9 janvier 1998
   * Dclaration de nombreuses fonctions
   * Ajout le champ "nom" dans les atomes FEFF : 6 lettres maximum

  6 janvier 1998
   * Prise en compte des dclarations d'atomes

  5, 9 dcembre 1997
   * Cration de ce fichier
*/

#ifndef _INC_FEFF_ /* Le fichier n'a pas dj t inclus */

#define _INC_FEFF_ 

#include "ATOMES.H" /* Ncessaire dans les dclarations */

/* ------------ Toutes les infos ncessaires  FEFF ---------------------------- */

typedef struct atome_feff
{
  struct atome_feff *suivant; /* NULL : on est sur le dernier                                 */
  Atome *atome;  /* L'atome utilis : x, y, z, charge, symbole, nom                           */
  int potentiel; /* Le potentiel FEFF associ  cet atome (0 = absorbeur, 1...8 = rflecteur) */
  char nom[7];   /* Nom d'un atome FEFF (6 lettres max. )                                     */
} FEFF_atome;

typedef struct ss_feff
{
  struct ss_feff *suivante;
  int index;          /* Couche que l'on considre         */
  int potentiel;      /* Potentiel des atomes de la couche */
  int degenerescence; /* Dgnrescence de la couche       */
  float distance;     /* Distance  l'atome central        */
} FEFF_SS; /* Commandes de rflexion simple */

typedef struct overlap_feff
{
  struct overlap_feff *suivante;
  int potentiel;           /* Potentiel de l'atome avec lequel on recouvre */
  unsigned int nbr_atomes; /* Nombre d'atomes de ce genre                  */
  float distance;          /* Distance de ces atomes                       */
} FEFF_OVERLAP;

/* ------------ Commandes possibles dans FEFF.INP ----------------------------- */

#define ABSENT -1 /* Pour les commandes qui doivent ne pas tre prsentes */
#define NON_LANCE -1 /* Module de FEFF non lanc au dmarrage */

#define POTPH  0 /* Les modules FEFF utilisables */
#define PATHS  1
#define GENFMT 2
#define FF2CHI 3

typedef struct
{
  FEFF_atome  *absorbeur;   /* Atome absorbeur : forcment un seul, pot. 0 */
  unsigned int nbr_atomes;  /* On limite  65536 atomes... devrait suffire */
  FEFF_atome  *prem_atome;  /* Liste des atomes des autres couches         */

  int CONTROL[4]; /* NON_LANCE : non lanc, sinon lanc avec sortie indique */

  struct
  {
    float temperature; /* Temprature          */
    float t_debye;     /* Temprature de Debye */
  } DEBYE;             /* Nulles -> option non utilise */

  struct
  {
    int type_lacune;   /* Type de lacune : 0 = aucune, 1 = K, 2 = L1, 3 = L2, 4 = L3 */
    float s0_2;        /* S_0^2 */
  } HOLE;

  float SIG2; /* Facteur de Debye-Waller */

  char *titre[10]; /* Les lignes de titre */

  struct
  {
    double decalage;
    double elargissement;
  } CORRECTIONS;
  
  struct
  {
    float critcw, critpw;
  } CRITERIA;

  struct
  {
    int circulaire; /* FAUX : linaire, VRAI : circulaire, ABSENT : on s'en fiche */
    double x, y, z; /* Vecteur d'ellipticit */
  } ELLIPTICITY;

  int NEMAX; /* Intervalle de calcul */

  int NLEG; /* Nombre maximal de rflexions */

  boolean NOGEOM; /* Commande NOGEOM prsente ? */
  int  XANES;  /* Commande XANES prsente, si oui valeur ? */
  boolean AFOLP;  /* Commande AFOLP  prsente ? */

  struct
  {
    float conservation, tas;
  } PCRITERIA;

  struct
  {
    int presente;   /* FAUX : linaire, VRAI : circulaire, ABSENT : on s'en fiche */
    double x, y, z; /* Vecteur de polarisation */
  } POLARIZATION;

  float RMAX; /* Longueur maximale des chemins (0 : absent...) */

  float RMULTIPLIER; /* Facteur multiplicatif des distances */

  unsigned int nbr_ss; /* Nombre de commandes SS */
  FEFF_SS *prem_ss;    /* Paramtres de la premire commande SS */

  int potentiels[9]; /* Numro atomique associ au potentiel 0...8   */
  float FOLP[9];     /* Recouvrements pour chacun des potentiels     */
  float ION[9];      /* Charge porte par les atomes au potentiel... */

  struct
  {
    int type; /* 0 : H-L, 1 : D-H, 2 : fondamental, 3 : D-H/H-L */
    float decalage;
    float p_imag;
  } EXCHANGE;

  FEFF_OVERLAP *prem_couche[8]; /* Pour instruction OVERLAP */

  /* -------- Options ajoutes par Bruce Raven pour xanes ---------- */

  boolean Utiliser_Xanes; /* VRAI si on veut utiliser les options de XANES */

  struct
  {
    int nombre;
    float position[5]; /* Dfinition des intervalles pour la grille */
    float delta_x[5];  /* Ceci ira en fait dans EMESH */
  } EGRID;
  float VINTFIX; /* Dmarrage du calcul, relatif au 0 des potentiel  muffin-tin */

} FEFF_INP; /* Tout ce qu'il faut pour un fichier FEFF.INP */

#ifndef _FEFF_SHELL_

extern FEFF_INP params_feff;

#endif

void FEFF_init   (void); /* Dmarrer le module : arbres en particulier... */
void FEFF_termine(void); /* Librer la mmoire occupe par le modle FEFF */

/* --------------- Fonctions d'entre-sortie ------------------------------------ */

void FEFF_sauve (void); /* Sauver FEFF.INP  */
void FEFF_charge(void); /* Charger FEFF.INP */

/* --------------- Fonctions de paramtrage proprement dites -------------------- */

void FEF_gene  (void); /* Paramtres gnraux    */
void FEF_struct(void); /* Paramtres structuraux */
void FEF_modele(void); /* Paramtres du modle   */

void xanes_feff(void); /* Commandes FEFF pour XANES */

/* --------------- Fonctions de gestion des formulaires (du SHELL donc) --------- */

boolean FEFF_general  (int num_objet, int x, int y);
boolean FEFF_structure(int num_objet, int x, int y);
boolean FEFF_modele   (int num_objet, int x, int y);

/* --------------- Messages d'erreur du shell FEFF ----------------------------- */

int FEFF_ERREUR(int code_erreur, int parametre); /* Codes dans sh_feff.h */

#endif /* _INC_FEFF_ */
