/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
           Relecture des fichiers FEFF.INP
   (c) Emmanuel CURIS 1998

 Commandes de dbogage : _D_FEFF_

 30 mars 1998
   * Chargement des fichiers FEFF : ligne TITLE vide provoquait un dcalage -> chargement foireux...

 25 mars 1998
   * Corrig les fins de ligne - certaines taient en unix ! Empchait gcc de compiler.

 16 mars 1998
   * Remplac graphes.h par erreurs.h : pour dfinition de is_nan()

 28 & 29 janvier 1998
   * Utilisation de FEFF_defauts = paramtres par dfaut
   * Efface les atomes dj existants au chargement
   * Chargement : compteur de lignes tait trop incrment... (2 fois par tour !)
   * Cration du fichier
   * Fonction d'appel du dcodage d'une ligne (reprise  lit_ccdb)
   * Dcodage d'une ligne : reconnat tout sauf OVERLAP (les paramtres)
*/

#define _LIT_FEFF_ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef _UNIX_

#include "../SYSTEME.H"
#include "../ATOMES/ATOMES.H"
#include "../EXAFS.H"   /* Dfinitions des arbres      */
#include "../DEBUG.H"
#include "../ERREURS.H" /* Pour is_nan() uniquement... */

#else

#include "..\SYSTEME.H"
#include "..\ATOMES\ATOMES.H"
#include "..\EXAFS.H"   /* Dfinitions des arbres      */
#include "..\DEBUG.H"
#include "..\ERREURS.H" /* Pour is_nan() uniquement... */

#endif

#include "FEFF.H"
#include "FEFF_GEM.H"
#include "F_STRUCT.H"
#include "SH_FEFF.H"


extern FEFF_INP params_feff,  /* Dfinie dans FEFF.C */
                FEFF_defauts; /* Valeurs par dfaut des paramtres */
extern int _rien_;           /* Dfinie quelque par en EXAFS... */

int FEFF_fichier(char *nom_fichier);
int FEFF_ligne(FILE *fichier, int *mode);

/* ************** Appel du slecteur : choix du fichier  charger *********** */

#define TOUT_VA_BIEN   0 /* Retours possibles de la fonction de chargement */
#define NOM_VIDE       1

void FEFF_charge(void)
{
	char **liste_noms;
	int nbr_fichiers, index;

	liste_noms = CH_selecteur("FEFF.INP", "Fichier de configuration de FEFF", 0, NULL, &nbr_fichiers);
	if (liste_noms == NULL) return; /* Aucun fichier  traiter */

	if (nbr_fichiers > 1)
	{
		int i;

		ERREUR(TROP_FICHIERS);
		for (i = 0 ; i < nbr_fichiers ; i++)
		{
			free(liste_noms[i]);
		}
		free(liste_noms);

		return; /* Chargement annul */
	}

	for (index = 0 ; index < nbr_fichiers ; index++)
	{
		int retour;

		sprintf(_intermediaire, "%s%s%c", nom_fichier, liste_noms[index], 0);

		retour = FEFF_fichier(_intermediaire);
		if (retour == TOUT_VA_BIEN)
		{
		}
		else
		{
			ERREUR(retour); /* Erreur lors du chargement */
		}
		free(liste_noms[index]);
	}
	free(liste_noms);
}

/* ************** Lecture proprement dite du fichier ************************ */

#define FEFF_NORMAL     0 /* Lecture normale d'un fichier FEFF           */
#define FEFF_ATOMS      1 /* Liste de coordonnes atomiques              */
#define FEFF_POTENTIALS 2 /* Liste des potentiels */
#define FEFF_OVERLAP    3 /* Liste de couches pour l'instruction OVERLAP - 3 : potentiel 0
                                                                           4 : potentiel 1 ... */

#define COMMENTAIRE    1
#define BLAH_BLAH     -2
#define SEPARATEUR    -3
#define DONNEES       -4
#define LIGNE_VIDE    -5

int FEFF_fichier(char *nom_fichier)
{
  unsigned long nbr_lignes = 1;
  int type_ligne, mode = FEFF_NORMAL;
  FILE *fichier; /* le fichier que l'on va lire */

  if (nom_fichier == NULL)
    return NOM_VIDE;

  fichier = fopen(nom_fichier,"r");
  if (fichier == NULL)
    return ERREUR_LECTURE;

  /* Il faut remettre  zro le modle FEFF dj en mmoire ! */

  FEFF_termine(); /* On efface toutes les structures associes */
  params_feff = FEFF_defauts;

  /* Doit-on effacer les atomes dj existant, ou ajouter les nouveau ? */

  type_ligne = FEFF_ERREUR(EFFACER_ATOMES, 0);
  if (type_ligne == 1) /* On efface les atomes */
  {
    while (ATM_premier)
      ATM_efface(ATM_premier); /* Mthode la plus simple... */
  }

  /* On charge maintenant le nouveau fichier */

  type_ligne = FEFF_ligne(fichier, &mode);
  while ( type_ligne != EOF )
  {
    nbr_lignes++; /* On passe  la ligne suivante */

    if (type_ligne != LIGNE_VIDE)
    {
      /* On ignore la fin de la ligne */

      int car_lu = fgetc(fichier);

#ifdef _D_FEFF_
      fprintf(debogage, "On ignore la fin de la ligne (ligne %ld)\n", nbr_lignes);
#endif

      while (car_lu != '\n')
      {
				car_lu = fgetc(fichier);
#ifdef _D_FEFF_
				fprintf(debogage, "%d (car : %c)\n", car_lu, (char) car_lu);
#endif
				if ( (car_lu == EOF) || ( (char) car_lu == 10) || ( (char) car_lu == 13) ) /* Pour voir */
				{
					car_lu = '\n';
				}
      }
    }
#ifdef _D_FEFF_
	 else
	 {
	 	 fprintf(debogage, "On vient de lire une ligne vide\n");
	 }
#endif

  /*  nbr_lignes++; */

    /* On passe  la ligne suivante */

    type_ligne = FEFF_ligne(fichier, &mode);
  }

#ifdef _D_FEFF_
  fprintf(debogage, "Fin du fichier atteinte - %ld lignes\n", nbr_lignes);
#endif

  fclose(fichier);

  return TOUT_VA_BIEN;
}

/* ----------- Lecture d'une ligne d'un fichier FEFF.INP et dcodage associ ----------------------\
 |   Renvoie le type de ligne lue                                                                  |
 |                                                                                                 |
 | fichier : le fichier que l'on est en train de traiter                                           |
 | mode    : le mode dans lequel on est (FEFF_NORMAL, FEFF_ATOMS ou FEFF_OVERLAP)                  |
 \------------------------------------------------------------------------------------------------ */

int FEFF_ligne(FILE *fichier, int *mode)
{
	long pos_debut; /* Position du dbut de la ligne : pour cause de plantage de fseek(,<0,SEEK_CUR) avec Turbo C ! */
	char commande[20]; /* La commande  traiter */
	int retour;

	pos_debut = ftell(fichier); /* Position du dbut de la ligne */

	retour = fscanf(fichier, "%s", commande); /* On lit le premier mot de la ligne */
	if (retour == EOF) /* Fin de fichier */
		return EOF;

	/* - - - - Mode normal -> dcodage des instructions FEFF - - - - */

#ifdef _D_FEFF_
	fprintf(debogage, "** Nouvelle ligne - commande lue : %s (longueur : %ld)\n", commande, strlen(commande));
	fprintf(debogage, " -*- Position dans le fichier : %ld\n", ftell(fichier) );
#endif

	if ( commande[0] == '*' ) return COMMENTAIRE; /* Ligne de commentaires -> on ignore */

	if ( strcmp(commande, "AFOLP") == 0 )       /* commande AFOLP */
  {
  	*mode = FEFF_NORMAL;
  	params_feff.AFOLP = VRAI;
  	return COMMENTAIRE;
  }

	if ( strcmp(commande, "ATOMS") == 0 )       /* commande ATOMS */
	{
		*mode = FEFF_ATOMS; /* On passe en mode atomes */
		return COMMENTAIRE;
	}

	if ( strcmp(commande, "CONTROL") == 0 )     /* commande CONTROL */
	{
		int i, lance;

		*mode = FEFF_NORMAL;
		for( i = 0 ; i < 4 ; i++)
		{
			fscanf(fichier, "%d", &lance);
			if (lance == 0)
				params_feff.CONTROL[i] = NON_LANCE; /* Sinon, on garde la valeur existante */
		}
		return COMMENTAIRE; /* On ignore la fin de la ligne */
	}

	if ( strcmp(commande, "CORRECTIONS") == 0 ) /* commande CORRECTIONS */
	{
		*mode = FEFF_NORMAL;
		fscanf(fichier, "%lf%lf", &params_feff.CORRECTIONS.decalage, &params_feff.CORRECTIONS.elargissement);
		return COMMENTAIRE;
	}

	if ( strcmp(commande, "CRITERIA") == 0 )    /* commande CRITERIA */
	{
		*mode = FEFF_NORMAL;
		fscanf(fichier, "%f%f", &params_feff.CRITERIA.critcw, &params_feff.CRITERIA.critpw );
		return COMMENTAIRE;
	}

	if ( strcmp(commande, "DEBYE") == 0 )       /* commande DEBYE */
	{
		*mode = FEFF_NORMAL;
		fscanf(fichier, "%f%f", &params_feff.DEBYE.temperature, &params_feff.DEBYE.t_debye);
		return COMMENTAIRE;
	}

	if ( strcmp(commande, "ELLIPTICITY") == 0 ) /* commande ELLIPTICITY */
	{
		*mode = FEFF_NORMAL;
		fscanf(fichier, "%d%lf%lf%lf", &params_feff.ELLIPTICITY.circulaire, &params_feff.ELLIPTICITY.x,
		                               &params_feff.ELLIPTICITY.y, &params_feff.ELLIPTICITY.z );
    return COMMENTAIRE;
	}

	if ( strcmp(commande, "END") == 0 )         /* commande END */
	{
		printf("Fin du fichier : ce qui suit est normalement ignor. Continuer o/n ?\n");
		return COMMENTAIRE;
	}

	if ( strcmp(commande, "EXCHANGE") == 0 )    /* commande EXCHANGE */
	{
		*mode = FEFF_NORMAL;
		fscanf(fichier, "%d%f%f", &params_feff.EXCHANGE.type, &params_feff.EXCHANGE.decalage,
		                          &params_feff.EXCHANGE.p_imag);
		return COMMENTAIRE;
	}

	if ( strcmp(commande, "FOLP") == 0 )        /* commande FOLP */
	{
	  int potentiel;
	  float recouvre;

	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%d%f", &potentiel, &recouvre);
	  if ( (potentiel < 0) || (potentiel > 8) )
	  {
	    FEFF_ERREUR(POTENTIEL_FAUX, potentiel);
	  }
	  else
	    params_feff.FOLP[potentiel] = recouvre - 1;
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "HOLE") == 0 )        /* commande HOLE */
	{
	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%d%f", &params_feff.HOLE.type_lacune, &params_feff.HOLE.s0_2);
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "ION") == 0 )         /* commande ION */
	{
	  int potentiel;
	  float charge;

	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%d%f", &potentiel, &charge);
	  if ( (potentiel < 0) || (potentiel > 8) )
	  {
	    FEFF_ERREUR(POTENTIEL_FAUX, 0);
	  }
	  else
	    params_feff.ION[potentiel] = charge;
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "IORDER") == 0 )      /* commande IORDER */
	{
	  printf("Commande IORDER non gre par ce programme. Si vous y tenez, il faudra la rajouter manuellement dans le fichier FEFF.INP produit\n");
	  evnt_keybd();
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "NEMAX") == 0 )       /* commande NEMAX */
	{
	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%d", &params_feff.NEMAX);
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "NLEG") == 0 )        /* commande NLEG */
	{
	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%d", &params_feff.NLEG);
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "NOGEOM") == 0 )      /* commande NOGEOM */
	{
	  *mode = FEFF_NORMAL;
	  params_feff.NOGEOM = VRAI;
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "OVERLAP") == 0 )     /* commande OVERLAP */
	{
	  int potentiel;

	  fscanf(fichier, "%d", &potentiel);
	  if ( (potentiel >= 0) && (potentiel <= 7) )
	    *mode = FEFF_OVERLAP + potentiel; /* Pour mmoriser le potentiel sur lequel on travaille */
	  else
	  {
	    *mode = FEFF_NORMAL;
	    FEFF_ERREUR(POTENTIEL_FAUX, potentiel);
	  }
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "PCRITERIA") == 0 )   /* commande PCRITERIA */
	{
	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%f%f", &params_feff.PCRITERIA.conservation, &params_feff.PCRITERIA.tas );
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "POLARIZATION") == 0 ) /* commande ELLIPTICITY */
	{
	  *mode = FEFF_NORMAL;
	  params_feff.POLARIZATION.presente = FAUX;
	  fscanf(fichier, "%lf%lf%lf", &params_feff.POLARIZATION.x, &params_feff.POLARIZATION.y, &params_feff.POLARIZATION.z );
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "POTENTIALS") == 0)   /* Commande POTENTIALS */
	{
	  *mode = FEFF_POTENTIALS;
	  return COMMENTAIRE; /* On ne lit pas la fin de la ligne */
	}

	if ( strcmp(commande, "PRINT") == 0 ) /* Commande PRINT */
	{
	  int i;
	  
	  *mode = FEFF_NORMAL;
	  for( i = 0 ; i < 4 ; i++)
	  {
	    fscanf(fichier, "%d", &params_feff.CONTROL[i]);
	  }

	  return COMMENTAIRE; /* On ignore la fin de la ligne */		
	}

	if ( strcmp(commande, "RMAX") == 0 ) /* Commande RMAX */
	{
	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%f", &params_feff.RMAX);
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "RMULTIPLIER") == 0 ) /* Commande RMULTIPLIER */
	{
	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%f", &params_feff.RMULTIPLIER);
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "SIG2") == 0 ) /* Commande SIG2 */
	{
	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%f", &params_feff.SIG2);
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "SS") == 0 ) /* commande SS */
	{
	  unsigned int index;
	  int potentiel, degenerescence;
	  float distance;

	  *mode = FEFF_NORMAL;
	  fscanf(fichier, "%d%d%d%f", &index, &potentiel, &degenerescence, &distance);
	  SS_creer(index, potentiel, degenerescence, distance);
	  return COMMENTAIRE;
	}

	if ( strcmp(commande, "TITLE") == 0 ) /* commande TITLE */
	{
	  int num_titre = 0, car_lu;
	  unsigned long lg_titre = 0;

	  *mode = FEFF_NORMAL;
	  while (params_feff.titre[num_titre])
	  {
	    num_titre++;
	    if (num_titre > 9) /* Dix commandes TITLE au maximum */
	    {
	      printf("Ce fichier contient trop de commandes TITLE\n");
	      return COMMENTAIRE;
	    }
	  }

	  car_lu = fgetc(fichier);
	  while ( (car_lu != '\n') && (car_lu != 10) && (car_lu != 13) ) /* Pas la fin de ligne */
	  {
	    if ( ! ( (car_lu == ' ') && (lg_titre == 0) ) ) /* Espaces en dbut de ligne ignors */
	      lg_titre++; /* Un caractre de plus dans le titre */
	    car_lu = fgetc(fichier);
	  }
	  if (lg_titre > 0) /* Titre de longueur non-nulle */
	  {
	    params_feff.titre[num_titre] = malloc( (size_t) (lg_titre + 1) ); /* Rserver la place */
	    if ( params_feff.titre[num_titre] == NULL)
	    {
	      ERREUR(MANQUE_MEMOIRE); /* Mal parti pour la suite de toute faon... */
	      return COMMENTAIRE; /* On passe  la ligne suivante */
	    }
	    fseek(fichier, -lg_titre-1, SEEK_CUR); /* On recule */
	    fread(params_feff.titre[num_titre], sizeof(char), lg_titre, fichier); /* Lire le titre */
	    params_feff.titre[num_titre][lg_titre] = 0; /* Fin de chane */
	    return COMMENTAIRE; /* Lecture de la ligne normalement termine */
	  }
	  else
	    return LIGNE_VIDE;
	}

	if ( strcmp(commande, "XANES") == 0 ) /* commande XANES */
	{
	  *mode = FEFF_NORMAL;
	  params_feff.XANES = VRAI;
	  return COMMENTAIRE;
	}

	/* Autre commande -> on regarde si on n'est pas dans un mode spcial... */

	if ( *mode == FEFF_NORMAL ) /* En mode normal */
	{
	  printf("Autre commande : %s\n", commande); /* Commande inconnue = commentaire */
	  return COMMENTAIRE;
	}

	/* Mode spcial -> on annule la lecture que l'on vient de faire */

	fseek(fichier, pos_debut, 0); /* On se remet au dbut de la ligne, en mode absolu */
	/*  fseek(fichier, -strlen(commande), SEEK_CUR); ne tourne pas avec Turbo C : erreur dans la lib. ? */

#ifdef _D_FEFF_
	fprintf(debogage, " -*- Position dans le fichier : %ld\n", ftell(fichier) );
	retour = fscanf(fichier, "%s", commande); /* On lit le premier mot de la ligne */
	fprintf(debogage, "~~ Paramtres d'une instruction type ATOMS. Si on relit : on trouve %s\n", commande);
	fprintf(debogage, " -*- Position dans le fichier : %ld\n", ftell(fichier) );
	fseek(fichier, -strlen(commande), SEEK_CUR);
#endif

	if (*mode == FEFF_ATOMS) /* Lecture des coordonnes atomiques */
	{
	  float x, y, z;
	  int potentiel;

	  retour = fscanf(fichier,"%f", &x); /* On lit la premire coordonne */
	  if (retour == EOF) /* Fin du fichier atteinte */
	    return EOF;
	  else if ( is_nan(x) ) /* ce n'est pas un nombre */
	  {
	    *mode = FEFF_NORMAL;
	    fseek(fichier, -retour, SEEK_CUR); /* On annule la lecture */
	  }
	  else
	  {
	    retour = fscanf(fichier,"%f%f%d", &y, &z, &potentiel);
#ifdef _D_FEFF_
	    printf("Coordonnes (potentiel %d) : %f %f %f\n", potentiel, x, y, z);
	    fprintf(debogage, "Coordonnes atomiques : x = %f, y = %f, z = %f - Potentiel : %d\n", x, y, z, potentiel);
#endif

	    if ( (potentiel < 0) || (potentiel > 7) )
	    {
	      FEFF_ERREUR(POTENTIEL_FAUX, potentiel);
	    }
	    else
	    {
	      Atome *nv_atome;
	      nv_atome = ATM_creer(params_feff.potentiels[potentiel], x, y, z, 0, NULL, 0, 0);
	      if ( nv_atome ) /* On a pu le crer */
	      {
		FATM_creer(nv_atome, potentiel, NULL);
	      }
	    }

	    return COMMENTAIRE; /* Aprs, on ignore */
	  }
	}

	if (*mode >= FEFF_OVERLAP)
	{
	  int pot_traite = *mode - FEFF_OVERLAP; /* Le potentiel dont on construit les recouvrements */
		
	  /* Paramtres de la fonction OVERLAP */
	}

	if (*mode == FEFF_POTENTIALS) /* Lecture des potentiels */
	{
	  int potentiel, num_atomique;

	  fscanf(fichier, "%d%d", &potentiel, &num_atomique); /* Si 6 caractres aprs: nom */
	  if ( (potentiel >= 0) && (potentiel <= 7) ) /* Potentiels autoriss */
	  {
	    params_feff.potentiels[potentiel] = num_atomique;
	  }
	  else
	  {
	    FEFF_ERREUR(POTENTIEL_FAUX, potentiel);
	  }
		
	  return COMMENTAIRE;
	}

	*mode = FEFF_NORMAL; /* Si on est l, c'est qu'on tait dans un mode farfelu et que les donnes ne conviennent pas */
	return COMMENTAIRE; /* Commande inconnue... */
}
