/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
   Tout ce qui concerne le paramtrage de FEFF (shell en particulier)
   (c) Emmanuel CURIS 1997 - 1998 ** Cration : 5 dcembre 1997

  3 mai 1998
   * Adapt aux nouveaux arbres dans le ressource
   * Corrig petite erreur dans entre des potentiels par symboles chimiques (sautait une ligne...)

 19 avril 1998
   * Sauver feff.inp : choix du type de fins de ligne (feff/unix comprend pas les feff.inp/pc)

 19 & 27 mars 1998
   * Clarification des #include
   * Cration ex nihilo d'un atome FEFF

 6 & 7, 9, 12 & 13, 16, 20, 28 & 29 janvier 1998
   * Cration de FEFF_defauts pour les paramtres par dfaut; utilisation  l'initialisation
   * Effacement d'un modle FEFF : rendu plus propre
   * Sauvegarde des couches de rflexion simple (SS) dans FEFF.INP
   * Continu interface pour SS : cration et tout plus logiques
                                  tout ce qui est affichage du dfileur fonctionne
                                  le clic sur les couches fonctionne
                                  effacement d'une couche fonctionne
   * En quittant le module, efface les couches SS cres
   * Modifi la gestion de CONTROL et PRINT : plus qu'une seule variable/module.
   * Ajout FEFF_init() -> utilisation d'arbres intgrs au ressource
   * Corrig sauvegarde FEFF : sauvegarde du potentiel absorbeur ne marchait plus !
   * Init de l'ascenseur des atomes avant l'appel de l'arbre
   * Paramtrage des potentiels : on peut utiliser le symbole de l'lment
   * Corrig l'effacement des atomes : effaait pas le bon !
   * Liste des atomes FEFF sauve dans FEFF.INP
   * Conversion de tous les atomes en atomes FEFF fonctionne
   * Squelette de gestion de l'arbre de structure
   * Gestion des options RMULTIPLIER, NLEG et RMAX fonctionne
   * Remplac gem_vdi.h par gem_var.h : plus logique !
   * Rorganis l'historique
   * Inclusion de erreurs.h et atomes.h
   * Squelette de l'appel de l'arbre de structure fonctionne

  5, 9-11 dcembre 1997
   * Termin interface pour config. des paramtres gnraux (instruction PRINT)
   * Commenc la cration sur disque de FEFF.INP
*/

#define _FEFF_SHELL_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"
#include "GRAPHES.H"
#include "EXAFS.H"
#include "ATOMES.H"

#include "FEFF_GEM.H"
#include "SH_FEFF.H"
#include "FEFF.H"

/* ~~~~~~~~~~~~~~~~~~~~~ Structure de donnes de FEFF, utilise par le programme ~~~~~~~~~~~ *
 *           Initialise avec les valeurs par dfaut donnes dans la notice                  *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

FEFF_INP params_feff,   /* Paramtres effectivement utiliss */
         FEFF_defauts = /* Paramtres par dfaut             */
{
  NULL, 0, NULL,  /* Pas d'atomes dfinis au dbut - ATOMS   */
  { 0, 0, 0, 0},  /* On lance tous les modules     - CONTROL */
                  /*     en sortie minimale        - PRINT   */
  { 0, 0 },       /* Option absente...             - DEBYE   */
  { 1, 1 },       /* Lacune de type K, S_0^2 = 1   - HOLE    */
  0,              /* Option absente...             - SIG2    */
  { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }, /* Aucun titre - TITLE */
  { 0, 0 },       /* Option absente...             - CORRECTIONS */
  { 4, 2.5 },     /* Critres de slection         - CRITERIA    */
  { ABSENT, 0, 0, 0 }, /* Ellipticit : aucune     - ELLIPTICITY */
  49,             /* Conditionne l'intervalle      - NEMAX */
  8,              /* Nombre maximal de rflexions  - NLEG  */
  FAUX,           /* Option absente                - NOGEOM    */
  ABSENT,         /* Option absente                - XANES     */
  FAUX,           /* Option absente                - AFOLP     */
  { 0, 0 },       /* Option absente                - PCRITERIA */
  { ABSENT, 0, 0, 0 }, /* Polarisation : aucune    - POLARIZATION */
  0,              /* Option absente                - RMAX */
  1,              /* Option absente                - RMULTIPLIER */
  0, NULL,        /* Aucune commande               - SS */
  {0, 0, 0, 0, 0, 0, 0, 0, 0}, /* Aucun potentiel dfini - POTENTIALS */
  {-1, -1, -1, -1, -1, -1, -1, -1, -1}, /* Aucune commande - FOLP */
  {0, 0, 0, 0, 0, 0, 0, 0, 0}, /* Aucune commande  - ION  */
  {-1, 0, 0},     /* Option absente                - EXCHANGE */
  { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }, /* Aucune commande - OVERLAP */
  FAUX,           /* On n'utilise pas les extensions Xanes */
  { 0, {0.0, 0.0, 0.0, 0.0, 0.0}, {0.0, 0.0, 0.0, 0.0, 0.0} }, /* EGRID et EMESH */
  0               /* Aucun dcalage d'origine      - VINTFIX */
};

/* ~~~~~~~~~~~~~~~~~~~~~ Vrification de la cohrence du modle propos ~~~~~~~~~~~~~~~~~~~~ */

void FEFF_verifie(void)
{
  int i;
  boolean pot_util[8] = {0, 0, 0, 0, 0, 0, 0, 0};
  FEFF_atome *atome = NULL;

  if ( params_feff.absorbeur == NULL )
  {
    FEFF_ERREUR(PAS_ABSORBEUR, 0);
    return;
  }

  if ( params_feff.potentiels[0] == 0 )
  {
    FEFF_ERREUR(PAS_ABSORBEUR, 0);
    return;
  }

  if ( params_feff.prem_atome == NULL )
  {
    FEFF_ERREUR(FEF_AUCUN_ATOME, 0);
    return;
  }

  /* On vrifie que chaque potentiel dfini est utilis */

  for (atome = params_feff.prem_atome ; atome ; atome = atome->suivant)
  {
    pot_util[atome->potentiel] = VRAI; /* Il y a au moins un atome */
  }

  /* Pour bien faire, il faudrait vrifier aussi SS et OVERLAP */

  for (i = 1 ; i < 8 ; i++)
  {
    if ( (pot_util[i] == 0) && (params_feff.potentiels[i] > 0) )
    {
      int reponse = FEFF_ERREUR(POT_SS_ATOME, i);

      if (reponse == 1) /* On corrige */
	params_feff.potentiels[i] = 0;
      else
	return; /* On arrte ici la vrification */
    }
  }
}

/* ~~~~~~~~~~~~~~~~~~~~~ Paramtres gnraux de FEFF ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* --------------------- Appel et enregistrement de ces paramtres --------------------------\
 |    Ne renvoie rien, ne prend aucun paramtre
 |    Suppose que titre[0] = compos tudi et titre[1] = utilisateur
 \------------------------------------------------------------------------------------------ */

void FEF_gene(void)
{
  boolean retour = FAUX;

  /* On prpare l'affichage de l'arbre */

  retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_POTPHA,  params_feff.CONTROL[POTPH] != NON_LANCE );
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD1_MIN , retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD1_MIN , params_feff.CONTROL[POTPH] == 0);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD1_AFF1, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD1_AFF1, params_feff.CONTROL[POTPH] == 1);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD1_AFF2, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD1_AFF2, params_feff.CONTROL[POTPH] == 2);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD1_AFF5, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD1_AFF5, params_feff.CONTROL[POTPH] == 5);
  
  retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_CHEMINS, params_feff.CONTROL[PATHS] != NON_LANCE ); /* Doit-on lancer paths ?  */
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD2_MIN , retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD2_MIN , retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD2_AFF1, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD2_AFF1, params_feff.CONTROL[PATHS] == 1);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD2_AFF2, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD2_AFF2, params_feff.CONTROL[PATHS] == 2);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD2_AFF3, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD2_AFF3, params_feff.CONTROL[PATHS] == 3);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD2_AFF5, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD2_AFF5, params_feff.CONTROL[PATHS] == 5);

  retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_GENFMT,  params_feff.CONTROL[GENFMT] != NON_LANCE ); /* Doit-on lancer genfmt ? */
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD3_LIMITE , retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD3_LIMITE, params_feff.CONTROL[GENFMT] == 0);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD3_AFF1, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD3_AFF1, params_feff.CONTROL[GENFMT] == 1);
  
  retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_FF2CHI,  params_feff.CONTROL[FF2CHI] != NON_LANCE); /* Doit-on lancer ff2chi ? */
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD4_MIN , retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD4_MIN , params_feff.CONTROL[FF2CHI] == 0);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD4_AFF1, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD4_AFF1, params_feff.CONTROL[FF2CHI] == 1);
  OBJ_active (FEFF_PARAM_GENE, FEFF_MOD4_AFF2, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD4_AFF2, params_feff.CONTROL[FF2CHI] == 2);

  if ( params_feff.titre[0] == NULL )
#ifdef _FRANCAIS_
    met_chaine(FEFF_PARAM_GENE, FEFF_COMPOSE, 0, "Nom du compos"e_AIGU" inconnu", TXT_COPIER);
#endif
#ifdef _ENGLISH_
    met_chaine(FEFF_PARAM_GENE, FEFF_COMPOSE, 0, "Compound name unknown", TXT_COPIER);
#endif
  else
    met_chaine(FEFF_PARAM_GENE, FEFF_COMPOSE, 0, params_feff.titre[0], TXT_COPIER);

  if ( params_feff.titre[1] == NULL )
    met_chaine(FEFF_PARAM_GENE, FEFF_UTILISATEUR, 0, "Nom de l'utilisateur inconnu", TXT_COPIER);
  else
    met_chaine(FEFF_PARAM_GENE, FEFF_UTILISATEUR, 0, params_feff.titre[1], TXT_COPIER);

  OBJ_choisit(FEFF_PARAM_GENE, FEFF_NOGEOM, params_feff.NOGEOM);
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_XANES, params_feff.XANES >= 0);
  met_valeur(FEFF_PARAM_GENE, FEFF_NEMAXE, params_feff.NEMAX);

  retour = ( params_feff.POLARIZATION.presente != ABSENT );
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_POLARISE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_X_POLARISE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_Y_POLARISE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_Z_POLARISE, retour);
  met_fvaleur(FEFF_PARAM_GENE, FEFF_X_POLARISE, params_feff.POLARIZATION.x);
  met_fvaleur(FEFF_PARAM_GENE, FEFF_Y_POLARISE, params_feff.POLARIZATION.y);
  met_fvaleur(FEFF_PARAM_GENE, FEFF_Z_POLARISE, params_feff.POLARIZATION.z);

  retour = ( params_feff.ELLIPTICITY.circulaire != ABSENT );
  OBJ_choisit(FEFF_PARAM_GENE, FEFF_ELIPTICITE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_X_ELLIPSE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_Y_ELLIPSE, retour);
  OBJ_active (FEFF_PARAM_GENE, FEFF_Z_ELLIPSE, retour);
  OBJ_choisit(FEFF_PARAM_GENE, FEF_POLAR_RECT, params_feff.ELLIPTICITY.circulaire == FAUX);
  OBJ_choisit(FEFF_PARAM_GENE, FEF_POLAR_CIRC, params_feff.ELLIPTICITY.circulaire == VRAI);
  met_fvaleur(FEFF_PARAM_GENE, FEFF_X_ELLIPSE, params_feff.ELLIPTICITY.x);
  met_fvaleur(FEFF_PARAM_GENE, FEFF_Y_ELLIPSE, params_feff.ELLIPTICITY.y);
  met_fvaleur(FEFF_PARAM_GENE, FEFF_Z_ELLIPSE, params_feff.ELLIPTICITY.z);

  /* On lance l'arbre */

  if (ext_formdo(FEFF_PARAM_GENE) == FEFF_GENE_ANNULE) /* On annule : rien d'autre  faire */
    return;

  /* On lit les valeurs des paramtres nouvellement dfinis */

  if (OBJ_choisit(FEFF_PARAM_GENE, FEFF_POTPHA,  QUESTION) == FAUX) /* Dsactiv */
    params_feff.CONTROL[POTPH] = -1;
  else
  {
    if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD1_MIN, QUESTION) == VRAI )
      params_feff.CONTROL[POTPH] = 0;
    else if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD1_AFF1, QUESTION) == VRAI )
      params_feff.CONTROL[POTPH] = 1;
    else if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD1_AFF2, QUESTION) == VRAI )
      params_feff.CONTROL[POTPH] = 2;
    else
      params_feff.CONTROL[POTPH] = 5;
  }

  if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_CHEMINS, QUESTION) == FAUX ) /* Dsactiv */
    params_feff.CONTROL[PATHS] = -1;
  else
  {
    if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD2_MIN, QUESTION) == VRAI )
      params_feff.CONTROL[PATHS] = 0;
    else if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD2_AFF1, QUESTION) == VRAI )
      params_feff.CONTROL[PATHS] = 1;
    else if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD2_AFF2, QUESTION) == VRAI )
      params_feff.CONTROL[PATHS] = 2;
    else if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD2_AFF3, QUESTION) == VRAI )
      params_feff.CONTROL[PATHS] = 3;
    else
      params_feff.CONTROL[PATHS] = 5;
  }

  if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_GENFMT, QUESTION) == FAUX ) /* Dsactiv */
    params_feff.CONTROL[GENFMT] = -1 ;
  else
  {
    if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD3_AFF1, QUESTION) == VRAI )
      params_feff.CONTROL[GENFMT] = 1;
    else
      params_feff.CONTROL[GENFMT] = 0;
  }

  if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_FF2CHI, QUESTION) == FAUX )
    params_feff.CONTROL[FF2CHI] = -1;
  else
  {
    if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD4_MIN, QUESTION) == VRAI )
      params_feff.CONTROL[FF2CHI] = 0;
    else if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD4_AFF1, QUESTION) == VRAI )
      params_feff.CONTROL[FF2CHI] = 1;
    else if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_MOD4_AFF2, QUESTION) == VRAI )
      params_feff.CONTROL[FF2CHI] = 2;
  }

  if ( params_feff.titre[0] == NULL )
  {
    params_feff.titre[0] = (char *) malloc( sizeof(char) * 75 );
    if ( params_feff.titre[0] == NULL )
      ERREUR(MANQUE_MEMOIRE);
  }
  lit_chaine(FEFF_PARAM_GENE, FEFF_COMPOSE, 75, params_feff.titre[0], TXT_COPIER);

  if ( params_feff.titre[1] == NULL )
  {
    params_feff.titre[1] = (char *) malloc( sizeof(char) * 75 );
    if ( params_feff.titre[1] == NULL )
      ERREUR(MANQUE_MEMOIRE);
  }
  lit_chaine(FEFF_PARAM_GENE, FEFF_UTILISATEUR, 75, params_feff.titre[1], TXT_COPIER);

  params_feff.NOGEOM = OBJ_choisit(FEFF_PARAM_GENE, FEFF_NOGEOM, QUESTION);
  if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_XANES, QUESTION) )
    params_feff.XANES = 0;
  else
    params_feff.XANES = ABSENT;
  params_feff.NEMAX = prend_valeur(FEFF_PARAM_GENE, FEFF_NEMAXE);

  if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_ELIPTICITE, QUESTION) == VRAI )
  {
    if ( OBJ_choisit(FEFF_PARAM_GENE, FEF_POLAR_RECT, QUESTION) == VRAI )
    /* Polarisation rectiligne */
    {
      params_feff.ELLIPTICITY.circulaire = FAUX;
    }
    else
    {
      params_feff.ELLIPTICITY.circulaire = VRAI;
    }
  }
  else
    params_feff.ELLIPTICITY.circulaire = ABSENT;

  if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_POLARISE, QUESTION) == VRAI )
    params_feff.POLARIZATION.presente = VRAI;
  else
    params_feff.POLARIZATION.presente = ABSENT;

  params_feff.POLARIZATION.x = prend_fvaleur(FEFF_PARAM_GENE, FEFF_X_POLARISE);	
  params_feff.POLARIZATION.y = prend_fvaleur(FEFF_PARAM_GENE, FEFF_Y_POLARISE);	
  params_feff.POLARIZATION.z = prend_fvaleur(FEFF_PARAM_GENE, FEFF_Z_POLARISE);	
  params_feff.ELLIPTICITY.x = prend_fvaleur(FEFF_PARAM_GENE, FEFF_X_ELLIPSE);	
  params_feff.ELLIPTICITY.y = prend_fvaleur(FEFF_PARAM_GENE, FEFF_Y_ELLIPSE);	
  params_feff.ELLIPTICITY.z = prend_fvaleur(FEFF_PARAM_GENE, FEFF_Z_ELLIPSE);	
}

/* --------------------- Procdure de gestion de cet arbre ----------------------------------\
 |    Renvoie VRAI si la gestion a pu tre faite, FAUX sinon
 |
 | numero_objet : l'objet sur lequel on a cliqu
 | x, y : position de la souris lors du clic
 \------------------------------------------------------------------------------------------ */

boolean FEFF_general(int numero_objet, int x, int y)
{
  OBJECT *adresse = arbres[FEFF_PARAM_GENE].adresse;
  boolean retour;

  switch (numero_objet)
  {
  case FEFF_POTPHA:  /* Quels modules doit-on lancer ? */
    retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_POTPHA, ECHANGE);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD1_MIN , retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD1_AFF1, retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD1_AFF2, retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD1_AFF5, retour);
    return VRAI;

  case FEFF_CHEMINS:
    retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_CHEMINS, ECHANGE); /* Doit-on lancer paths ?  */
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD2_MIN , retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD2_AFF1, retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD2_AFF2, retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD2_AFF3, retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD2_AFF5, retour);
    return VRAI;

  case FEFF_GENFMT:
    retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_GENFMT, ECHANGE); /* Doit-on lancer genfmt ? */
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD3_LIMITE , retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD3_AFF1, retour);
    return VRAI;

  case FEFF_FF2CHI:
    retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_FF2CHI, ECHANGE); /* Doit-on lancer ff2chi ? */
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD4_MIN , retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD4_AFF1, retour);
    OBJ_active(FEFF_PARAM_GENE, FEFF_MOD4_AFF2, retour);
    return VRAI;

  case FEFF_ELIPTICITE:
    retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_ELIPTICITE, ECHANGE);
    OBJ_active (FEFF_PARAM_GENE, FEFF_X_ELLIPSE, retour);
    OBJ_active (FEFF_PARAM_GENE, FEFF_Y_ELLIPSE, retour);
    OBJ_active (FEFF_PARAM_GENE, FEFF_Z_ELLIPSE, retour);
    return VRAI;

  case FEFF_POLARISE:
    retour = OBJ_choisit(FEFF_PARAM_GENE, FEFF_POLARISE, ECHANGE);
    OBJ_active (FEFF_PARAM_GENE, FEFF_X_POLARISE, retour);
    OBJ_active (FEFF_PARAM_GENE, FEFF_Y_POLARISE, retour);
    OBJ_active (FEFF_PARAM_GENE, FEFF_Z_POLARISE, retour);
    break;

  default:
    form_button(adresse, numero_objet, 1, &x);
  }

  return VRAI;
}

/* ~~~~~~~~~~~~~~~~~~~~~~~~~ Paramtrage du modle de FEFF (potentiels et autres) ~~~~~ */

void FEF_modele(void) /* On suppose les objets ordonns en colonne... */
{
  boolean test;
  int i;

  /* -~-~-~-~-~ On installe les paramtres actuel -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~ */

  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_AFOLP, params_feff.AFOLP);

  met_valeur(FEFF_PARAM_MODEL, FEFF_POT_0, params_feff.potentiels[0]); /* Atome absorbeur */
  met_fvaleur(FEFF_PARAM_MODEL, FEFF_ION_0, params_feff.ION[0]);

  for (i = 0 ; i < 7 ; i++) /* Les autres atomes */
  {
    met_valeur(FEFF_PARAM_MODEL, FEFF_POT_1 + i, params_feff.potentiels[i + 1]);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_FLP_1 + i, !(params_feff.AFOLP) );
    if ( params_feff.potentiels[i + 1] > 0 ) /* Le potentiel est dfini */
    {
      met_fvaleur(FEFF_PARAM_MODEL, FEFF_ION_1 + i, params_feff.ION[i + 1]);
      if ( (params_feff.FOLP[i + 1] >= 0) && (params_feff.AFOLP == FAUX) )
      {
	OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FLP_1 + i, FORCE_OUI);
	OBJ_active (FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, FORCE_OUI);
	met_fvaleur(FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, params_feff.FOLP[i + 1]);
      }
      else
      {
	OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FLP_1 + i, FORCE_NON);
	OBJ_active (FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, FORCE_NON);
	met_fvaleur(FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, 0);
      }
    }
    else
    {
      met_fvaleur(FEFF_PARAM_MODEL, FEFF_ION_1 + i, 0.0);
      OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FLP_1 + i, FORCE_NON);
      OBJ_active (FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, FORCE_NON);
      met_fvaleur(FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + i, 0.0);
    }
  }

  test = ( params_feff.DEBYE.temperature > 0 ) && ( params_feff.DEBYE.t_debye > 0 );
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_DEBYE, test);
  met_fvaleur(FEFF_PARAM_MODEL, FEFF_DEBYE_T , params_feff.DEBYE.temperature);
  met_fvaleur(FEFF_PARAM_MODEL, FEFF_DEBYE_TD, params_feff.DEBYE.t_debye);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_SIG2, !test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_VAL_SIGMA, !test);

  test = (params_feff.SIG2 > 0);
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_SIG2, test);
  met_fvaleur(FEFF_PARAM_MODEL, FEFF_VAL_SIGMA, params_feff.SIG2);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_DEBYE, !test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_DEBYE_T, !test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_DEBYE_TD, !test);

  met_fvaleur(FEFF_PARAM_MODEL, FEFF_DECALAGE, params_feff.CORRECTIONS.decalage);
  met_fvaleur(FEFF_PARAM_MODEL, FEFF_ELARGIT, params_feff.CORRECTIONS.elargissement);

  test = params_feff.EXCHANGE.type >= 0;
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_EXCHANGE, test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_HEDIN, test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_DIRAC, test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_FONDAMENTAL, test);
  OBJ_active (FEFF_PARAM_MODEL, FEFF_MIXTE, test);

  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_K , params_feff.HOLE.type_lacune == 1);
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L1, params_feff.HOLE.type_lacune == 2);
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L2, params_feff.HOLE.type_lacune == 3);
  OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L3, params_feff.HOLE.type_lacune == 4);

  met_fvaleur(FEFF_PARAM_MODEL, FEFF_S0_2, params_feff.HOLE.s0_2);

  /* -~-~-~-~-~ On appelle l'arbre, on gre et tout et tout -~-~-~-~-~-~-~-~-~-~-~-~-~ */

  if ( ext_formdo(FEFF_PARAM_MODEL) == FEFF_MODELE_ANN )
    return;

  /* -~-~-~-~-~ On enregistre les nouveaux paramtres ~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- */

  params_feff.AFOLP = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_AFOLP, QUESTION);

  params_feff.potentiels[0] = prend_valeur(FEFF_PARAM_MODEL, FEFF_POT_0);
  if ( params_feff.potentiels[0] == 0 ) /* Potentiel nul */
  {
    params_feff.potentiels[0] = ATM_num_atomique( lit_chaine( FEFF_PARAM_MODEL, FEFF_POT_0, 0, NULL, TXT_REMPLACE ) );
  }
  params_feff.ION[0] = prend_fvaleur(FEFF_PARAM_MODEL, FEFF_ION_0);
  
  for (i = 1 ; i < 8 ; i++) /* Autres potentiels existants */
  {
    params_feff.potentiels[i] = prend_valeur(FEFF_PARAM_MODEL, FEFF_POT_1 + i - 1);
    if ( params_feff.potentiels[i] == 0 )
    {
      /* On vrifie que l'utilisateur n'a pas entr le symbole de l'lment */

      params_feff.potentiels[i] = ATM_num_atomique( lit_chaine( FEFF_PARAM_MODEL, FEFF_POT_1 + i - 1, 0, NULL, TXT_REMPLACE ) );
    }

    if ( params_feff.potentiels[i] == 0 )
    {
      params_feff.FOLP[i] = -1; /* Pas de recouvrement */
      params_feff.ION[i] = 0; /* Pas de charge */
    }
    else
    {
      params_feff.ION[i] = prend_fvaleur(FEFF_PARAM_MODEL, FEFF_ION_1 + i - 1);
      if ( (params_feff.AFOLP) || (OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FLP_1 + i - 1, QUESTION) == FAUX ) )
	params_feff.FOLP[i] = -1; /* Pas de recouvrement */
      else
	params_feff.FOLP[i] = prend_fvaleur(FEFF_PARAM_MODEL, FEFF_FLP_1 + i - 1);
    }
  }

  if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_DEBYE, QUESTION) == VRAI )
  {
    params_feff.DEBYE.temperature = prend_fvaleur(FEFF_PARAM_MODEL, FEFF_DEBYE_T);
    params_feff.DEBYE.t_debye     = prend_fvaleur(FEFF_PARAM_MODEL, FEFF_DEBYE_TD);
    params_feff.SIG2 = 0;
  }
  else
  {
    params_feff.DEBYE.temperature = 0;
    params_feff.DEBYE.t_debye     = 0;
    if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_SIG2, QUESTION) == VRAI )
      params_feff.SIG2 = prend_fvaleur(FEFF_PARAM_MODEL, FEFF_VAL_SIGMA);
    else
      params_feff.SIG2 = 0;
  }

  params_feff.CORRECTIONS.decalage = prend_fvaleur(FEFF_PARAM_MODEL, FEFF_DECALAGE);
  params_feff.CORRECTIONS.elargissement = prend_fvaleur(FEFF_PARAM_MODEL, FEFF_ELARGIT);

  if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_EXCHANGE, QUESTION) == VRAI )
  {
    params_feff.EXCHANGE.decalage = params_feff.CORRECTIONS.decalage;
    params_feff.EXCHANGE.p_imag   = params_feff.CORRECTIONS.elargissement;
    if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_HEDIN, QUESTION) == VRAI )
      params_feff.EXCHANGE.type = 1;
    else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_DIRAC, QUESTION) == VRAI )
      params_feff.EXCHANGE.type = 2;
    else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_FONDAMENTAL, QUESTION) == VRAI )
      params_feff.EXCHANGE.type = 3;
    else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_MIXTE, QUESTION) == VRAI )
      params_feff.EXCHANGE.type = 4;
  }
  else
  {
    params_feff.EXCHANGE.type = -1; /* On n'utilise pas */
    params_feff.EXCHANGE.decalage = 0;
    params_feff.EXCHANGE.p_imag   = 0;
  }

  if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_K, QUESTION) == VRAI )
    params_feff.HOLE.type_lacune = 1;
  else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L1, QUESTION) == VRAI )
    params_feff.HOLE.type_lacune = 2;
  else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L2, QUESTION) == VRAI )
    params_feff.HOLE.type_lacune = 3;
  else if ( OBJ_choisit(FEFF_PARAM_MODEL, FEFF_LACUNE_L3, QUESTION) == VRAI )
    params_feff.HOLE.type_lacune = 4;
  else
    params_feff.HOLE.type_lacune = 0; /* Aucune lacune - devrait pas arriver */

  params_feff.HOLE.s0_2 = prend_fvaleur(FEFF_PARAM_MODEL, FEFF_S0_2);
}

/* ------------------------- Gestion effective de la bote de paramtrage ------------- */

boolean FEFF_modele(int num_objet, int x, int y)
{
  int nv_etat, i;

  switch (num_objet)
  {
  case FEFF_EXCHANGE:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_EXCHANGE, ECHANGE);
    OBJ_active (FEFF_PARAM_MODEL, FEFF_HEDIN, nv_etat);
    OBJ_active (FEFF_PARAM_MODEL, FEFF_DIRAC, nv_etat);
    OBJ_active (FEFF_PARAM_MODEL, FEFF_FONDAMENTAL, nv_etat);
    OBJ_active (FEFF_PARAM_MODEL, FEFF_MIXTE, nv_etat);
    break;

  case FEFF_FLP_1:
  case FEFF_FLP_2:
  case FEFF_FLP_3:
  case FEFF_FLP_4:
  case FEFF_FLP_5:
  case FEFF_FLP_6:
  case FEFF_FLP_7:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, num_objet, ECHANGE);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_RECOUVRE_1 + num_objet - FEFF_FLP_1, nv_etat);
    break;

  case FEFF_DEBYE:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_DEBYE, ECHANGE);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_SIG2, !nv_etat);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_VAL_SIGMA, !nv_etat);
    break;
    
  case FEFF_SIG2:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_SIG2, ECHANGE);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_DEBYE, !nv_etat);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_DEBYE_T, !nv_etat);
    OBJ_active(FEFF_PARAM_MODEL, FEFF_DEBYE_TD, !nv_etat);
    break;

  case FEFF_AFOLP:
    nv_etat = OBJ_choisit(FEFF_PARAM_MODEL, FEFF_AFOLP, ECHANGE);
    for (i = 0 ; i < 7 ; i++)
    {
      OBJ_active(FEFF_PARAM_MODEL, FEFF_FLP_1 + i , !nv_etat);
    }
    break;
    
  default:
    form_button(arbres[FEFF_PARAM_MODEL].adresse, num_objet, 1, &x);
  }
  return VRAI;
}

/* ~~~~~~~~~~~~~~~~~~~~~~~~~ Modle structural utilis par FEFF ~~~~~~~~~~~~~~~~~~~~~~~ */

/* --------------------- Appel et enregistrement de ces paramtres --------------------------\
 |    Ne renvoie rien, ne prend aucun paramtre
 \------------------------------------------------------------------------------------------ */

void FEF_struct(void)
{
  boolean retour = FAUX;
  FEF_infos infos;

  /* On prpare l'affichage de l'arbre */

  met_fvaleur(FEFF_PAR_STRUCT, FEF_RMAX,  params_feff.RMAX);
  met_fvaleur(FEFF_PAR_STRUCT, FEF_RMULT, params_feff.RMULTIPLIER);
  met_valeur (FEFF_PAR_STRUCT, FEFF_NLEG, params_feff.NLEG);

  retour = ( ATM_premier != NULL ); /* VRAI s'il y a des atomes de dfinis */
  OBJ_active(FEFF_PAR_STRUCT, FEF_AJOUTER,      retour);
  OBJ_active(FEFF_PAR_STRUCT, FEF_TOUT_CHOISIR, retour);

  if ( params_feff.absorbeur )
    infos.prem_atome = -1;
  else
    infos.prem_atome = 0;
  infos.atm_choisi = 1; /* On commence ascenseur en haut, 1er atome choisi */
  
  FATM_init_defil(&infos); /* On prpare le dfileur d'atomes */

  infos.prem_ss = 0;    /* On commence en haut */
  infos.ss_choisie = 1; /* Et c'est celle-ci qui est slectionne */

  SS_init_defil(&infos);

  /* On lance l'arbre */

  arbres[FEFF_PAR_STRUCT].pointeur = (void *) &infos;

  if (ext_formdo(FEFF_PAR_STRUCT) == FEF_STRCT_ANNULE) /* On annule : rien d'autre  faire */
    return;

  /* Atome ventuellement modifi */

  if (infos.atm_choisi > 0) /* Il y a un atome choisi */
    FATM_gere_defil(&infos, FEF_NOM_PREM_ATM + infos.atm_choisi - 1, 0, 0);
  
  /* On lit les valeurs des paramtres nouvellement dfinis */
  
  params_feff.RMAX        = prend_fvaleur(FEFF_PAR_STRUCT, FEF_RMAX);
  params_feff.RMULTIPLIER = prend_fvaleur(FEFF_PAR_STRUCT, FEF_RMULT);
  params_feff.NLEG        = prend_valeur (FEFF_PAR_STRUCT, FEFF_NLEG);
}

/* ------------------------- Gestion effective de la bote de paramtrage ------------- */

boolean FEFF_structure(int num_objet, int x, int y)
{
  FEF_infos *infos;
  int nv_etat, i;
  unsigned int nbr_objets, nbr_visibles;

  infos = (FEF_infos *) arbres[FEFF_PAR_STRUCT].pointeur;

  if ( (num_objet >= FEF_NOM_PREM_ATM) && (num_objet <= FEF_NOM_DERN_ATM) )
  {
    FATM_gere_defil(infos, num_objet, x, y);
    return VRAI;
  }

  if ( (num_objet >= FEF_PREM_SS) && (num_objet <= FEF_DERN_SS) )
  {
    SS_gere_defil(infos, num_objet, x, y);
    return VRAI;
  }

  switch (num_objet)
  {
  case FEF_ATM_AVANT:  /* Faire dfiler d'un atome vers le haut */
  case FEF_ATM_FND:    /* Faire dfiler de 10 atomes */
  case FEF_ATM_BOUTON: /* Faire dfiler : ascenseur glisse */
  case FEF_ATM_APRES:  /* Faire dfiler d'un atome vers le bas */
    FATM_gere_defil(infos, num_objet, x, y);
    break;

  case FEF_AJOUTER:    /* Ajouter un atome  la liste, aprs l'atome slectionn */
    break;

  case FEF_EFFACER:    /* Effacer l'atome choisi */
    i = infos->prem_atome + (infos->atm_choisi - 1); /* Atome  effacer */

    nbr_visibles = FEF_NOM_DERN_ATM - FEF_NOM_PREM_ATM + 1;
    nbr_objets = params_feff.nbr_atomes;
    if (params_feff.absorbeur)
      nbr_objets++;

    if (i == -1) /* On a choisi l'atome absorbeur */
    {
      FATM_effacer(params_feff.absorbeur, FAUX);
      infos->prem_atome = 0; /* Plus d'atome absorbeur */
    }
    else
    {
      FEFF_atome *atome = params_feff.prem_atome;
      unsigned int j = 0;

      while ( (j < i) && (atome) )
      {
	j++; atome = atome->suivant;
      }
      if (atome->suivant == NULL) /* On efface le dernier */
	infos->atm_choisi--; /* L'atome choisi est celui d'avant, sinon c'est le suivant */
      FATM_effacer(atome, FAUX);
    }

    i = infos->prem_atome + nbr_visibles;
    if (params_feff.absorbeur)
      i++;

    if (i >= nbr_objets)
      i = nbr_objets - 1;
    
    infos->prem_atome = i - nbr_visibles;
    if (params_feff.absorbeur)
      infos->prem_atome--;
    if (infos->prem_atome < 0)
    {
      if (params_feff.absorbeur)
	infos->prem_atome = -1;
      else
	infos->prem_atome = 0;
    }

    FATM_init_defil(infos);
    break;

  case FEF_TT_EFFACE:  /* Effacer tous les atomes choisis */
    if (FEFF_ERREUR(FATM_TT_EFFACER, 0) == 2)
      return VRAI;
  	
    FATM_effacer(params_feff.absorbeur, VRAI);
    while (params_feff.prem_atome) /* Tant qu'il y a un atome */
      FATM_effacer(params_feff.prem_atome, VRAI);
    
    FATM_init_defil(infos);
    break;

  case FEF_TOUT_CHOISIR: /* Tous les atomes en mmoire deviennent des atomes FEFF */
    if (ATM_premier == NULL)
    {
      FEFF_ERREUR(AUCUN_ATOME, 0);
      return VRAI;
    }

    if (params_feff.prem_atome) /* Dj des atomes -> que faire ? */
    {
      int retour = FEFF_ERREUR(FF_TOUT_CHOISIR, 0);
      if (retour == 3) /* On annule l'opration */
	return VRAI;

      if (retour == 2) /* On efface les atomes dj existants */
      {
	FATM_effacer(params_feff.absorbeur, VRAI);
	
	while (params_feff.prem_atome) /* Tant qu'il y a un atome */
	  {
	    FATM_effacer(params_feff.prem_atome, VRAI);
	  }
      }
    }
    
    {
      Atome *atome;
      for ( atome = ATM_dernier ; atome ; atome = atome->precedent )
      {
	FEFF_atome *Fatome = NULL;
	int num_atomique = atome->num_atomique; /* Le numro atomique */
	int pot; /* Le potentiel que l'on doit lui attribuer */

	/* On cherche s'il existe un potentiel pour les atomes de mme Z */

	if ( params_feff.potentiels[0] == num_atomique ) /* Un potentiel d'absorbeur est dfini */
	{
	  if ( params_feff.absorbeur == NULL )   /* Pas d'atome absorbeur */
	  {
	    Fatome = FATM_creer(atome, 0, NULL); /* -> On en cre un */
	    continue;
	  }
	}

	for (pot = 1 ; pot < 8 ; pot++)
	{
	  if ( params_feff.potentiels[pot] == num_atomique ) /* Potentiel dj dfini */
	  {
	    Fatome = FATM_creer(atome, pot, NULL);
	    break;
	  }
	}
				
	if (Fatome == NULL) /* Aucun potentiel dj dfini trouv -> on essaye d'en crer un */
	{
	  /* On cherche le premier potentiel vide (i.e. de numro atomique 0) */
				
	  for (pot = 0 ; pot < 8 ; pot++)
	  {
	    if (params_feff.potentiels[pot] <= 0) /* Potentiel non-dfini */
	    {
	      params_feff.potentiels[pot] = num_atomique; /* On le dfinit donc */
	      Fatome = FATM_creer(atome, pot, NULL); /* Et on cr l'atome associ */
	      break;
	    }
	  }
					
	  if (Fatome == NULL) /* a a chou aussi -> impossible de lui attribuer un potentiel */
	  {
	    if ( FEFF_ERREUR(PLUS_POTENTIEL, 0) == 2 )
	      return VRAI; /* On abandonne le chargement */
	  }
	}
      }
    }

		if ( params_feff.absorbeur ) /* On refait ce qu'il faut pour que ca colle */
			infos->prem_atome = -1;
		else
			infos->prem_atome = 0;
		infos->atm_choisi = 1; /* On commence ascenseur en haut, 1er atome choisi */

	 	FATM_init_defil(infos); /* On affiche ce qu'on vient de faire - en commenant au premier atome */
		
    break;

  case FEF_CREER:      /* Crer un atome et l'ajouter  la liste */
   	if ( ext_formdo(FEF_CREER_ATOME) == FATM_CREER )
   	{
   		Atome *atome;
    	int num_atomique;
    	float x, y, z;
    	int potentiel;

    	num_atomique = prend_valeur(FEF_CREER_ATOME, FATM_NUM_ATOMIK);
    	if (num_atomique == 0) /* Compltement farfelu */
    	{
    		num_atomique = ATM_num_atomique( lit_chaine(FEF_CREER_ATOME, FATM_NUM_ATOMIK, 0, NULL, TXT_REMPLACE ) );
    	}

			potentiel = prend_valeur(FEF_CREER_ATOME, FATM_POTENTIEL);
			if (num_atomique == 0) /* Aucun numro atomique correct de dfini */
			{
				num_atomique = params_feff.potentiels[potentiel]; /* Numro associ  ce potentiel */
			}

			x = prend_fvaleur(FEF_CREER_ATOME, FATM_X);
			y = prend_fvaleur(FEF_CREER_ATOME, FATM_Y);
			z = prend_fvaleur(FEF_CREER_ATOME, FATM_Z);

			atome = ATM_creer(num_atomique, x, y, z, 0, NULL, 0, 0);
			FATM_creer(atome, potentiel, NULL);

		 	FATM_init_defil(infos); /* On affiche ce qu'on vient de faire - en commenant au premier atome */
    }
    break;

  case FEF_COPIER:     /* Copier l'atome FEFF courant */
    break;

	/* -~-~-~-~-~- On passe aux recouvrements (OVERLAP) -~-~-~-~-~-~ */

	case FEF_OVERLAP_OK: /* Utilise-t-on cette option ? */
		nv_etat = OBJ_choisit(FEFF_PAR_STRUCT, FEF_OVERLAP_OK, ECHANGE);
		break;

  case FEF_NV_OVL:     /* Nouveau recouvrement pour le potentiel la couche trait */
    break;

  case FEF_EFF_OVL:    /* Effacer le recouvrement slectionn */
    break;

	case FEF_OVL_AVANT:  /* Recouvrement prcdent */
		break;

	case FEF_OVL_FOND:   /* Recouvrement : dfilement de +/- 10 */
		break;

	case FEF_OVL_BOUTON: /* Recouvrement : dfilement par le bouton */
		break;

  case FEF_OVL_APRES:  /* Recouvrement suivant */
  	break;

	case FEF_OVL_POT_AV: /* Potentiel  traiter prcdent */
		break;

	case FEF_OVL_POT_FND:/* Potentiel : dfilement de +/- 10 */
		break;

	case FEF_OVL_CHX_POT:/* Potentiel : dfilement par le bouton */
		break;

  case FEF_OVL_POT_AP: /* Potentiel  traiter suivant */
  	break;

	/* -~-~-~-~-~- On passe aux rflexions simples (SS) -~-~-~-~-~-~ */

	case FEF_SS_OK:      /* Utilise-t-on cette option ? */
		nv_etat = OBJ_choisit(FEFF_PAR_STRUCT, FEF_SS_OK, ECHANGE);
		if ( nv_etat ) /* On veut crer une structure de rflexions simple */
		{
			SS_nouvelle();
			infos->prem_ss = 0;    /* On affiche la premire en haut */
			infos->ss_choisie = 1; /* Et elle est slectionne */
		}
		else /* Au contraire, on dtruit celle existant dj */
		{
			while (params_feff.prem_ss)
				SS_effacer(params_feff.prem_ss, VRAI); /* On efface toutes les couches */
			infos->prem_ss    = -1; /* Aucune couche */
			infos->ss_choisie =  0; /* Donc aucune slectionne */
		}
		SS_init_defil(infos);
		break;

  case FEF_NV_SS:      /* Crer une nouvelle couche de rflexions simples */
    SS_nouvelle();
    if (infos->prem_ss < 0)
    {
    	infos->prem_ss = 0;
    	infos->ss_choisie = 1;
    }
		SS_init_defil(infos);
		break;

	case FEF_SS_AVANT:   /* Couche prcdente */
	case FEF_SS_FOND:    /* Couche : dfilement de +/- 10 */
	case FEF_SS_BOUTON:  /* Couche : dfilement par le bouton */
  case FEF_SS_APRES:   /* Couche suivante */
    SS_gere_defil(infos, num_objet, x, y);
    break;

  case FEF_EFF_SS:     /* Effacer la couche courante */
    i = infos->prem_ss + (infos->ss_choisie - 1); /* Couche  effacer */
    nbr_visibles = FEF_DERN_SS - FEF_PREM_SS + 1;
    nbr_objets = params_feff.nbr_ss;

    {
      FEFF_SS *couche = params_feff.prem_ss;
      unsigned int j = 0;

      while ( (j < i) && (couche) )
      {
	j++; couche = couche->suivante;
      }
      SS_effacer(couche, FAUX);
    }

    i = infos->prem_ss + nbr_visibles;
    
    if (i >= nbr_objets)
      i = nbr_objets - 1;

    infos->prem_ss = i - nbr_visibles;
    if (infos->prem_ss < 0)
      infos->prem_ss = 0;

    SS_init_defil(infos);
    break;
  }

  return VRAI; /* Trait convenablement */
}

/* ~~~~~~~~~~~~~~~~~~~~~~ Initialisation du module ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

void FEFF_init(void)
{
  OBJECT *adresse;

  params_feff = FEFF_defauts; /* On met les paramtres par dfaut */

  /* On s'arrange pour que les arbres soient plus jolis */

  /* FEFF, paramtrages gnraux */
  adresse = arbres[FEFF_PARAM_GENE].adresse;
  OB_Y(adresse, FEFF_MODULES_TXT) += OB_HEIGHT(adresse, FEFF_MODULES_TXT) / 2;
  OB_HEIGHT(adresse, FEFF_MODS_FOND) += 3;
  OB_Y(adresse, FEFF_CHEMINS) += 1;
  OB_Y(adresse, FEFF_GENFMT)  += 2;
  OB_Y(adresse, FEFF_FF2CHI)  += 3;
  OB_WIDTH(adresse, FEFF_AFF1_FOND) += 3; /* Ligne pour FEFF_POTPHA  */
  OB_X(adresse, FEFF_MOD1_AFF1) += 1;
  OB_X(adresse, FEFF_MOD1_AFF2) += 2;
  OB_X(adresse, FEFF_MOD1_AFF5) += 3;
  OB_WIDTH(adresse, FEFF_AFF2_FOND) += 4; /* Ligne pour FEFF_CHEMINS */
  OB_Y(adresse, FEFF_AFF2_FOND) += 1;
  OB_X(adresse, FEFF_MOD2_AFF1) += 1;
  OB_X(adresse, FEFF_MOD2_AFF2) += 2;
  OB_X(adresse, FEFF_MOD2_AFF3) += 3;
  OB_X(adresse, FEFF_MOD2_AFF5) += 4;
  OB_WIDTH(adresse, FEFF_AFF3_FOND) += 1; /* Ligne pour FEFF_GENFMT */
  OB_Y(adresse, FEFF_AFF3_FOND) += 2;
  OB_X(adresse, FEFF_MOD3_AFF1) += 1;
  OB_WIDTH(adresse, FEFF_AFF4_FOND) += 2; /* Ligne pour FEFF_FF2CHI */
  OB_Y(adresse, FEFF_AFF4_FOND) += 3;
  OB_X(adresse, FEFF_MOD4_AFF1) += 1;
  OB_X(adresse, FEFF_MOD4_AFF2) += 2;
  OB_Y(adresse, FEFF_POLARISE) += 1;
  arbres[FEFF_PARAM_GENE].fonction.arbre = FEFF_general;

  /* FEFF, paramtrages des potentiels */
  adresse = arbres[FEFF_PARAM_MODEL].adresse;
  arbres[FEFF_PARAM_MODEL].fonction.arbre = FEFF_modele;
  OB_WIDTH(adresse, FEFF_LACUNE_FOND) += 3;
  OB_X(adresse, FEFF_LACUNE_L1) += 1;
  OB_X(adresse, FEFF_LACUNE_L2) += 2;
  OB_X(adresse, FEFF_LACUNE_L3) += 3;

  /* FEFF, paramtrage de la structure tudie */
  arbres[FEFF_PAR_STRUCT].fonction.arbre = FEFF_structure;
}

/* ~~~~~~~~~~~~~~~~~~~~~~ Libration de la mmoire occupe par le modle FEFF ~~~~~~~~~ *
 *
 * Le fait de faon propre (remise  zro de params_feff), car appele avant le
 * chargement d'un fichier aussi...
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

void FEFF_termine(void)
{
	int i;

	/* Libration des titres du modle FEFF */

	for (i = 0 ; i < 10 ; i++)
	{
		free(params_feff.titre[i]); /* On libre la mmoire ventuellement utilise par le modle pour FEFF */
  	params_feff.titre[i] = NULL; /* Plus de titre */
	}

  /* Libration des atomes FEFF : on force l'effacement */

	FATM_effacer(params_feff.absorbeur, VRAI);
	while (params_feff.prem_atome)
		FATM_effacer(params_feff.prem_atome, VRAI);

	params_feff.absorbeur = NULL;
	params_feff.prem_atome = NULL;

  /* Libration des couches SS FEFF : on force l'effacement */
 
 	while (params_feff.prem_ss)
 		SS_effacer(params_feff.prem_ss, VRAI);

	params_feff.prem_ss = NULL;
}
