/* LASE - Traitement de l'E.X.A.F.S. sous TOS, MagiC, ... & X-Window
   Interface graphique pour le paramtrage de FEFF

   Gestion des objets ncessaires aux descriptions structurales
   (c) Emmanuel CURIS 1996 - 1998

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SYSTEME.H>
#include <GRAPHES.H>
#include <EXAFS.H>
#include <ATOMES.H>

#include "FEFF_GEM.H"
#include "SH_FEFF.H"
#include "FEFF.H"
#include "F_STRUCT.H"

/* ---------------------- Routines de gestion des recouvrements FEFF ------------------- *
 * Cration et effacement d'un recouvrement FEFF                                         *
 *****************************************************************************************/

/* ---------------------- Cration d'un recouvrement FEFF -------------------------------\
 | Renvoie l'adresse du recouvrement cr (NULL si erreur)                               |
 |                                                                                       |
 | potentiel    : le potentiel pour lequel il faut crer le recouvrement                 |
 | pot_recouvre : potentiel de l'atome avec lequel il se recouvre                        |
 | nbr_atomes   : nombre d'atomes de ce type avec lesquels il se recouvre                |
 | distance     : distance de ces atomes                                                 |
 \-------------------------------------------------------------------------------------- */

FEFF_OVERLAP *OVL_creer(int potentiel, int pot_recouvre, int nbr_atomes, float distance)
{
  FEFF_OVERLAP *Overlap;

  if ( (potentiel < 0) || (potentiel > 8) )
  {
    FEFF_ERREUR(POTENTIEL_FAUX, potentiel);
    return NULL;
  }
  if ( (pot_recouvre < 0) || (pot_recouvre > 8) )
  {
    FEFF_ERREUR(POTENTIEL_FAUX, pot_recouvre);
    return NULL;
  }
  if ( nbr_atomes < 1 )
  {
    printf("Il faut au moins un atome d'un autre type pour dfinir un recouvrement!\n");
    return NULL;
  }
  if ( distance <= 0 )
  {
    FEFF_ERREUR(FF_DIST_NEGATIVE, (int) distance);
    return NULL;
  }

  Overlap = (FEFF_OVERLAP *) malloc( sizeof(FEFF_OVERLAP) );
  if (Overlap == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return NULL;
  }

  /* La couche de recouvrement a pu tre dfinie */

  Overlap->potentiel  = pot_recouvre; /* Paramtres du recouvrement */
  Overlap->nbr_atomes = nbr_atomes;
  Overlap->distance   = distance;

  /* On la place en dbut de liste */

  Overlap->suivante = (params_feff.prem_couche)[potentiel];
  (params_feff.prem_couche)[potentiel] = Overlap;

  return Overlap;
}

/* ---------------------- Routines de gestion des atomes FEFF -------------------------- *
 * Cration et effacement d'un atome FEFF                                                *
 *****************************************************************************************/

/* ---------------------- Cration d'un nouvel atome FEFF -------------------------------\
 |   Crer un atome FEFF. Si potentiel = 0 ==> atome absorbeur (unique)                  |
 |                        Si potentiel < 8 et > 0 ==> ajout en dbut de liste           |
 |   Renvoi l'adresse de cet atome - NULL si erreur                                      |
 |                                                                                       |
 | atome     : l'atome (rel) qui doit tre utilis pour la modlisation                 |
 | potentiel : le potentiel FEFF auquel on doit l'assimiler (0 : absorbeur)              |
 | nom       : nom  donner  l'atome FEFF (NULL -> nom de l'lment)                    |
 \-------------------------------------------------------------------------------------- */

FEFF_atome *FATM_creer(Atome *atome, int potentiel, char *nom)
{
  FEFF_atome *Fatome;

  if (atome == NULL) return NULL; /* Pas d'atome pour aller avec ! */

  if ( (potentiel < 0) || (potentiel > 7) ) /* Potentiel absurde */
  {
    FEFF_ERREUR(POTENTIEL_FAUX, potentiel);
    return NULL;
  }

  /* On vrifie qu'il n'y a pas de conflit avec l'absorbeur */

  if ( (potentiel == 0) && (params_feff.absorbeur) ) /* Atome absorbeur dj dfini */
  {
    if (params_feff.absorbeur->atome == atome)
    {
      FEFF_ERREUR(FATM_EXISTE_DEJA, 0);
      return NULL;
    }

    if (FEFF_ERREUR(ABSORBEUR_DEFINI, 0) == 1) /* On ne veut pas le remplacer */
    {
      return NULL;
    }
  }

  /* On vrifie qu'aucun atome FEFF n'existe ayant le mme atome comme modle */

  for (Fatome = params_feff.prem_atome ; Fatome ; Fatome = Fatome->suivant)
  {
    if (Fatome->atome == atome)
    {
      FEFF_ERREUR(FATM_EXISTE_DEJA, 0);
      return NULL;
    }
  }

  Fatome = (FEFF_atome *) malloc( sizeof(FEFF_atome) );
  if (Fatome == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return NULL;
  }

  Fatome->atome     = atome;     /* Infos sur l'atome pris en compte */
  Fatome->potentiel = potentiel; /* Potentiel FEFF  utiliser        */

  if ( nom == NULL )
  {
    if ( (atome->num_atomique > 0) && (atome->num_atomique < 130) )
      nom = ATM_donnees[atome->num_atomique - 1].nom;
    else
      nom = "Inconnu";
  }

  { /* On installe le nom de l'atome FEFF */
    unsigned long i, lgn = strlen(nom);
		
    for (i = 0 ; i < 6 ; i++)
    {
      if ( i >= lgn )
	Fatome->nom[i] = 0;
      else
	Fatome->nom[i] = nom[i];
    }
    Fatome->nom[6] = 0;
  }

  if (potentiel == 0) /* Atome absorbeur */
  {
    if (params_feff.absorbeur) /* Il y a dj un atome absorbeur */
      FATM_effacer(params_feff.absorbeur, VRAI); /* On l'efface donc */
    
    params_feff.absorbeur = Fatome;
  }
  else
  {
    Fatome->suivant = params_feff.prem_atome;
    params_feff.prem_atome = Fatome;
    params_feff.nbr_atomes++;
  }

  return Fatome;
}

/* ------------------ On efface un atome FEFF -------------------------------------------\
 |   Renvoie VRAI si on a pu effacer
 |                                                                                       |
 | atome  : l'atome qu'il faut supprimer
 | forcer : VRAI -> on ne demande pas confirmation, FAUX : on demande                    |
 \-------------------------------------------------------------------------------------- */

boolean FATM_effacer(FEFF_atome *atome, boolean forcer)
{
  FEFF_atome *Fatome = params_feff.prem_atome, *avant = NULL;

  if (atome == NULL) return VRAI;

  if (forcer == FAUX)
  {
    if ( FEFF_ERREUR(FATM_EFFACER, 0) == 2 )
      return FAUX; /* On abandonne l'effacement */
  }

  if (atome == params_feff.absorbeur)
  {
    free(params_feff.absorbeur);  /* On libre la mmoire          */
    params_feff.absorbeur = NULL; /* Plus d'atome absorbeur dfini */

    return VRAI;
  }

  if (Fatome == NULL) /* Chose vraiment trange, mais sait-on jamais... */
  {
    printf("Incoh"e_AIGU"rence ! Un atome FEFF n'est pas dans la liste !\n");
    printf("Il vaut mieux quitter le programme et v"e_AIGU"rifier l'int"e_AIGU"grit"e_AIGU" de vos donn"e_AIGU"es\n");
    printf("Appuyez sur une touche pour continuer\n");
    evnt_keybd();

    return FAUX;
  }

  if (Fatome == atome) /* Premier atome de la liste */
  {
    params_feff.prem_atome = atome->suivant;
    free(atome);
    params_feff.nbr_atomes--;

    return VRAI;
  }

  avant = Fatome; Fatome = Fatome->suivant;
  
  while (Fatome != atome)
  {
    avant = Fatome;
    Fatome = Fatome->suivant;
    if (Fatome == NULL)
    {
      printf("Incohrence ! Un atome FEFF n'est pas dans la liste !\n");
      printf("Il vaut mieux quitter le programme et vrifier l'intgrit de vos donnes\n");
      printf("Appuyez sur une touche pour continuer\n");
      evnt_keybd();

      return FAUX;
    }
  }

  avant->suivant = atome->suivant; /* On reforme la chane */
  free(atome); /* On efface l'atome */
  params_feff.nbr_atomes--; /* Un atome de moins dans la liste */

  return VRAI;
}

/* ---------------------- Routines de gestion des couches de rflexion simple FEFF ----- *
 * Cration et effacement d'une couche de rflexion simple pour la modlisation par FEFF *
 *****************************************************************************************/

/* ---------------------- Paramtrage d'une couche de rflexion simple ------------------\
 |    Ne renvoie rien
 |    Correspond juste  l'interface GEM avant appel de SS_creer
 \-------------------------------------------------------------------------------------- */

void SS_nouvelle(void)
{
	int index, deg, potentiel;
	float distance;

	if ( ext_formdo(SS_NOUVELLE) == NVSS_ANNULER )
		return;

	index     = prend_valeur (SS_NOUVELLE, NVSS_INDEX    );
	potentiel = prend_valeur (SS_NOUVELLE, NVSS_POTENTIEL);
	deg       = prend_valeur (SS_NOUVELLE, NVSS_DEGENERE );
	distance  = prend_fvaleur(SS_NOUVELLE, NVSS_DIST     );

	SS_creer(index, potentiel, deg, distance);
}

/* ---------------------- Cration d'une nouvelle couche FEFF ---------------------------\
 |   Crer une couche de rflexion simple (SS) FEFF                                      |
 |   Renvoi l'adresse de cette couche - NULL si erreur                                   |
 |                                                                                       |
 | index          : le numro du fichier FEFFnnnn.DAT associ  cette couche             |
 | potentiel      : le potentiel des atomes de la couche                                 |
 | degenerescence : le nombre d'atomes dans cette couche (= dgnrescence du chemin)    |
 | distance       : distance de cette couche  l'atome central (en )                    |
 \-------------------------------------------------------------------------------------- */

FEFF_SS *SS_creer(int index, int potentiel, int degenerescence, float distance)
{
  FEFF_SS *couche;
  
  if (index <= 0)  /* Pas de sens */
  {
    FEFF_ERREUR(FSS_FAUX_NUMERO, index);
    return NULL;
  }

  if ( (potentiel < 1) || (potentiel > 7) )
  {
    FEFF_ERREUR(POTENTIEL_FAUX, potentiel);
    return NULL;
  }

  if (degenerescence < 1) /* Impossible aussi */
  {
    FEFF_ERREUR(FSS_DEG_FAUSSE, degenerescence);
    return NULL;
  }

  if (distance < 0) /* Pas trs raliste non plus */
  {
    FEFF_ERREUR(FF_DIST_NEGATIVE, 0);
    return NULL;
  }

  if ( params_feff.prem_ss ) /* Il y a dj des couches */
  {
    for (couche = params_feff.prem_ss ; couche ; couche = couche->suivante)
    {
      if ( index == couche->index )
      {
	FEFF_ERREUR(FSS_EXISTE_DEJA, index);
	return NULL;
      }
    }
  }
  
  couche = (FEFF_SS *) malloc( sizeof(FEFF_SS) );
  if (couche == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return NULL;
  }

  couche->index = index;
  couche->potentiel = potentiel;
  couche->degenerescence = degenerescence;
  couche->distance = distance;

  couche->suivante = params_feff.prem_ss; /* On l'insre en dbut de liste */
  params_feff.prem_ss = couche;
  params_feff.nbr_ss++; /* Une couche de plus */

  return couche;
}

/* ---------------------- Effacement d'une couche SS ------------------------------------\
 |     Renvoie FAUX si l'effacement n'a pas eu lieu, VRAI sinon                          |
 |                                                                                       |
 | couche : la couche  effacer                                                          |
 | force  : VRAI -> on ne demande pas de confirmation                                    |
 \-------------------------------------------------------------------------------------- */

boolean SS_effacer(FEFF_SS *couche, boolean force)
{
  FEFF_SS *avant;

  if ( couche == NULL ) return FAUX;

  if (force == FAUX)
  {
    if ( FEFF_ERREUR(FSS_EFFACER, couche->index) == 2 )
      return FAUX; /* On annule l'effacement */
  }

  if ( couche == params_feff.prem_ss ) /* On efface la premire */
  {
    params_feff.prem_ss = params_feff.prem_ss->suivante;
    free(couche);
    params_feff.nbr_ss--; /* Une couche de moins */
    return VRAI;
  }

  avant = params_feff.prem_ss;
  while ( (avant) && (avant->suivante != couche) )
  {
    avant = avant->suivante;
  }

  if (avant == NULL) /* Dernire couche et on n'a pas trouv la couche  effacer */
  {
    printf("Incohrence ! Une sous-couche FEFF n'est pas dans la liste !\n");
    printf("Il vaut mieux quitter le programme et vrifier l'intgrit de vos donnes\n");
    printf("Appuyez sur une touche pour continuer\n");
    evnt_keybd();

    return FAUX; /* On quitte */
  }

  avant->suivante = couche->suivante; /* On reforme la chane */
  free(couche);
  params_feff.nbr_ss--; /* Une couche de moins */
  
  return VRAI;
}
