/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
   Sauvegarde des fichiers FEFF.INP
   (c) Emmanuel CURIS 1998 - Cration : 25 mai 1998
*/

#include <stdio.h>
#include <stdlib.h>

#include "SYSTEME.H"

#include "ATOMES.H"
#include "EXAFS.H"
#include "FEFF.H"
#include "SH_FEFF.H"

extern FEFF_INP params_feff; /* Dans FEFF_SH.c */

/* -------------------- Sauvegarde d'un fichier FEFF.INP -------------------------------\
 |    Ne renvoie rien, ne prend aucun paramtre                                         |
 \------------------------------------------------------------------------------------- */

void FEFF_sauve(void)
{
  char fin_ligne[3] = {0, 0, 0}; /* Pour les fins de ligne max/pc/unix */
  FILE *fichier;
  unsigned int i;
  char *nom_fichier;

#ifdef _FRANCAIS_
  nom_fichier = SV_selecteur("feff.inp", "Sauver fichier FEFF.INP");
#endif
#ifdef _ENGLISH_
  nom_fichier = SV_selecteur("feff.inp", "Save file feff.inp");
#endif

  if ( nom_fichier == NULL )
  {
    ERREUR(ERREUR_ECRITURE); /* Erreur lors de la cration du fichier */
    return;
  }

  fichier = fopen(nom_fichier, "wb"); /* On ouvre le fichier en criture */
  free(nom_fichier); /* On n'en a plus besoin */
  if (fichier == NULL)
  {
    ERREUR(ERREUR_ECRITURE); /* Erreur lors de la cration du fichier */
    return;
  }

  if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_FL_PC, QUESTION) )
  {
    fin_ligne[0] = 13;
    fin_ligne[1] = 10;
  }
  else if ( OBJ_choisit(FEFF_PARAM_GENE, FEFF_FL_MAC, QUESTION) )
  {
    fin_ligne[0] = 13;
  }
  else
    fin_ligne[0] = 10; /* Unix par dfaut */

  /* On signe ce que l'on ralise */
  fprintf(fichier, "* Fichier r"e_AIGU"alis"e_AIGU" "a_GRAVE" partir du programme EDA/GEM - "COPYRIGHT" Emmanuel CURIS, janvier-mai 1998%s", fin_ligne);

  /* On dmarre le fichier par la liste des titres */
  for (i = 0 ; i < 10 ; i++)
  {
    if ( params_feff.titre[i] ) /* Le titre existe */
    {
      fprintf(fichier, "TITLE %s%s", params_feff.titre[i], fin_ligne);
    }
  }

  /* On prcise quels modules on veut utiliser */
  fprintf(fichier, "%s* 1 = on lance le module, 0 = on ne le lance pas%s", fin_ligne, fin_ligne);
  if ( params_feff.CONTROL[POTPH] == -1)
    fprintf(fichier, "CONTROL 0");
  else
    fprintf(fichier, "CONTROL 1");
  if ( params_feff.CONTROL[PATHS] == -1)
    fprintf(fichier, " 0");
  else
    fprintf(fichier, " 1");
  if ( params_feff.CONTROL[GENFMT] == -1)
    fprintf(fichier, " 0");
  else
    fprintf(fichier, " 1");
  if ( params_feff.CONTROL[FF2CHI] == -1)
    fprintf(fichier, " 0 ! Resp. potph, paths, genfmt et ff2chi%s", fin_ligne);
  else
    fprintf(fichier, " 1 ! Resp. potph, paths, genfmt et ff2chi%s", fin_ligne);
		
  /* On prcise ce que doivent sortir ces modules */
  fprintf(fichier, "%s* Param"e_AIGU"trage des fichiers sauv"e_AIGU"s par chacun des modules - voir la notice%s", fin_ligne, fin_ligne);
  if ( params_feff.CONTROL[POTPH] < 1 )
    fprintf(fichier, "PRINT 0");
  else
    fprintf(fichier, "PRINT %d", params_feff.CONTROL[POTPH]);
  if ( params_feff.CONTROL[PATHS] < 1 )
    fprintf(fichier, " 0");
  else
    fprintf(fichier, " %d", params_feff.CONTROL[PATHS]);
  if ( params_feff.CONTROL[GENFMT] < 1 )
    fprintf(fichier, " 0");
  else
    fprintf(fichier, " %d", params_feff.CONTROL[GENFMT]);
  if ( params_feff.CONTROL[FF2CHI] < 1 )
    fprintf(fichier, " 0 ! M"e_CIRC"me ordre que ci-dessus%s", fin_ligne);
  else
    fprintf(fichier, " %d ! M"e_CIRC"me ordre que ci-dessus%s", params_feff.CONTROL[FF2CHI], fin_ligne);
  
  /* On dfinit ici les potentiels utiliss */

  fprintf(fichier,"%s* D"e_AIGU"finition des diff"e_AIGU"rents types d'atome "a_GRAVE" prendre en compte%s", fin_ligne, fin_ligne);
  fprintf(fichier,"POTENTIALS%s", fin_ligne);
  if ( params_feff.potentiels[0] > 0 )
  {
    char *symbole = "(?)";

    if ( params_feff.potentiels[0] < 130 )
      symbole = ATM_donnees[params_feff.potentiels[0] - 1].symbole;
    fprintf(fichier, "0 %d %s ! Atome absorbant%s", params_feff.potentiels[0], symbole, fin_ligne);
  }
  else
  {
    FEFF_ERREUR(PAS_ABSORBEUR, 0);
    fclose(fichier);
    return;
  }
  for (i = 1 ; i < 9 ; i++)
  {
    if ( params_feff.potentiels[i] > 0 )
    {
      char *symbole = "(?)";
      
      if ( params_feff.potentiels[i] < 130 )
	symbole = ATM_donnees[params_feff.potentiels[i] - 1].symbole;
      fprintf(fichier,"%d %d %s%s", i, params_feff.potentiels[i], symbole, fin_ligne);
    }
  }

  /* On prcise ici quels sont les atomes  utiliser */

  if ( (params_feff.absorbeur) || (params_feff.nbr_atomes > 0) )
  {
    FEFF_atome *Fatome;

    fprintf(fichier, "%s* Liste des coordonn"e_AIGU"es atomiques%s", fin_ligne, fin_ligne);
    fprintf(fichier, "ATOMS ! x, y, z, potentiel, nom%s", fin_ligne);
    if ( params_feff.absorbeur )
    {
      Fatome = params_feff.absorbeur;
      fprintf(fichier, "%f %f %f 0 %s (Absorbeur)%s%s", Fatome->atome->x, Fatome->atome->y,
	      Fatome->atome->z, Fatome->nom,
	      fin_ligne, fin_ligne);
    }

    for (Fatome = params_feff.prem_atome ; Fatome ; Fatome = Fatome->suivant)
    {
      if ( params_feff.potentiels[Fatome->potentiel] > 0 )
      {
	fprintf(fichier, "%f %f %f %d %s%s", Fatome->atome->x, Fatome->atome->y, Fatome->atome->z,
		Fatome->potentiel, Fatome->nom, fin_ligne);
      }
    }
  }

  /* Quelques autres dtails de gomtrie */

  if ( (params_feff.RMULTIPLIER > 0) && (params_feff.RMULTIPLIER != 1) )
  {
    fprintf(fichier, "%s* Toutes les distances doivent "e_CIRC"tre multipli"e_AIGU"es par...%s",
	    fin_ligne, fin_ligne);
    fprintf(fichier, "RMULTIPLIER %f ! %s", params_feff.RMULTIPLIER, fin_ligne);
  }

  /* Ensuite, description de la lacune et de S0^2 */
  fprintf(fichier, "%s* Type de transitions mod"e_AIGU"lis"e_AIGU"e - Facteur S_0^2%s",
	  fin_ligne, fin_ligne);
  fprintf(fichier, "HOLE %d %f%s", params_feff.HOLE.type_lacune, params_feff.HOLE.s0_2, fin_ligne);

  /* Quelques options gnrales */
  if ( params_feff.XANES >= 0 )
    fprintf(fichier, "%sXANES %d ! On demande de calculer le spectre xanes%s",
	    fin_ligne, params_feff.XANES, fin_ligne); /* Pour tre compatible avec feff/xanes */
  if ( params_feff.NOGEOM == VRAI )
    fprintf(fichier, "%sNOGEOM ! On demande de ne pas cr"e_AIGU"er le fichier geom.dat %s",
	    fin_ligne, fin_ligne);

  if ( params_feff.ELLIPTICITY.circulaire != ABSENT ) /* On a dfini une polarisation */
  {
    fprintf(fichier,"%s* Prise en compte de l'ellipticit"e_AIGU" du faisceau%s",
	    fin_ligne, fin_ligne);
    fprintf(fichier,"ELLIPTICITY %d %f %f %f%s", params_feff.ELLIPTICITY.circulaire,
	    params_feff.ELLIPTICITY.x, params_feff.ELLIPTICITY.y, params_feff.ELLIPTICITY.z,
	    fin_ligne);
  }

  if ( params_feff.POLARIZATION.presente != ABSENT ) /* On prend en compte la polarisation */
  {
    fprintf(fichier,"%s* Prise en compte de la polarisation du faisceau%s", fin_ligne, fin_ligne);
    fprintf(fichier,"POLARIZATION %f %f %f%s", params_feff.POLARIZATION.x, 
	    params_feff.POLARIZATION.y, params_feff.POLARIZATION.z, fin_ligne);
  }

  /* Options pour le modle considr */

  if ( params_feff.NEMAX != 49 ) /* Si valeur par dfaut, non sauv : vite un avertissement FEFF */
  {
    fprintf(fichier, "%s* Limitation de l'intervalle du signal%s", fin_ligne, fin_ligne);
    fprintf(fichier, "NEMAX %d ! Valeur par d"e_AIGU"faut : 49%s", params_feff.NEMAX, fin_ligne);
  }

  if ( (params_feff.DEBYE.temperature > 0) && (params_feff.DEBYE.t_debye > 0) )
  {
    fprintf(fichier, "%s* Param"e_GRAVE"tres de mod"e_AIGU"lisation du facteur de Debye-Waller%s",
	    fin_ligne, fin_ligne);
    fprintf(fichier, "DEBYE %f %f%s", params_feff.DEBYE.temperature, params_feff.DEBYE.t_debye,
	    fin_ligne);
  }

  if ( params_feff.SIG2 > 0)
  {
    fprintf(fichier, "%s* Facteur de Debye-Waller "a_GRAVE" utiliser%s", fin_ligne, fin_ligne);
    fprintf(fichier, "SIG2 %f%s", params_feff.SIG2, fin_ligne);
  }

  if ( (params_feff.CORRECTIONS.decalage != 0) && (params_feff.CORRECTIONS.elargissement > 0) )
  {
    fprintf(fichier, "%s* Corrections d'"e_AIGU"nergie  appliquer - utilis"e_AIGU"e seulement par ff2chi%s", fin_ligne, fin_ligne);
    fprintf(fichier, "CORRECTIONS %f %f%s", params_feff.CORRECTIONS.decalage,
	    params_feff.CORRECTIONS.elargissement, fin_ligne);
  }

  if ( (params_feff.EXCHANGE.type >= 0) && (params_feff.EXCHANGE.p_imag > 0) )
  {
    fprintf(fichier, "%s* Corrections d'"e_AIGU"nergie "a_GRAVE" appliquer - utilis"e_AIGU"e partout%s", fin_ligne, fin_ligne);
    fprintf(fichier, "EXCHANGE %d %f %f%s", params_feff.EXCHANGE.type, params_feff.EXCHANGE.decalage, params_feff.EXCHANGE.p_imag, fin_ligne);
  }

  if ( params_feff.AFOLP == VRAI )
    fprintf(fichier, "%sAFOLP ! On d"e_AIGU"finit le recouvrement maximal pour tout%s",
	    fin_ligne, fin_ligne);
  else
  {
    fprintf(fichier, "%s* Recouvrements entr"e_AIGU"s manuellement%s", fin_ligne, fin_ligne);
    for (i = 0 ; i < 9 ; i++)
    {
      if (params_feff.FOLP[i] >= 0) /* \045 = % */
	fprintf(fichier, "FOLP %d %f ! %f%c %s", i, 1.0 + params_feff.FOLP[i]/100.0,
		params_feff.FOLP[i], '%', fin_ligne);
    }
  }

  fprintf(fichier,"%s* "E_AIGU"tat d'ionisation des atomes%s", fin_ligne, fin_ligne);
  for (i = 0 ; i < 9 ; i++)
  {
    if (params_feff.ION[i] != 0)
    {
      fprintf(fichier, "ION %d %f%s", i, params_feff.ION[i], fin_ligne);
    }
  }

  /* Critres de slection des chemins */

  fprintf(fichier, "%s* Nombre maximal de r"e_AIGU"flexions autoris"e_AIGU"es%s",
	  fin_ligne, fin_ligne);
  fprintf(fichier, "NLEG %d%s", params_feff.NLEG, fin_ligne);

  if ( params_feff.RMAX > 0 )
  {
    fprintf(fichier, "%s* Longueur maximale autoris"e_AIGU"e pour un chemin%s",
	    fin_ligne, fin_ligne);
    fprintf(fichier, "RMAX %f%s", params_feff.RMAX, fin_ligne);
  }

  if ( (params_feff.CRITERIA.critcw >= 0) && (params_feff.CRITERIA.critpw >= 0) )
  {
    fprintf(fichier, "%s* "E_AIGU"liminer les chemins d'amplitude trop faible, APR"E_GRAVE"S calcul de d"e_AIGU"g"e_AIGU"n"e_AIGU"rescence - voir notice%s", fin_ligne, fin_ligne);
    fprintf(fichier, "CRITERIA %f %f%s", params_feff.CRITERIA.critcw, params_feff.CRITERIA.critpw,
	    fin_ligne);
  }

  if ( (params_feff.PCRITERIA.conservation >= 0) && (params_feff.PCRITERIA.tas >= 0) )
  {
    fprintf(fichier, "%s* "E_AIGU"liminer les chemins d'amplitude trop faible, AVANT calcul de d"e_AIGU"g"e_AIGU"n"e_AIGU"rescence - voir notice%s", fin_ligne, fin_ligne);
    fprintf(fichier, "PCRITERIA %f %f%s", params_feff.PCRITERIA.conservation,
	    params_feff.PCRITERIA.tas, fin_ligne);
  }

  /* Modle des rflexions simples */

  if ( params_feff.prem_ss ) /* Il y a des couches de rflexion simple */
  {
    FEFF_SS *couche = params_feff.prem_ss;

    fprintf(fichier, "%s* Les couches de r"e_AIGU"flexion simple%s", fin_ligne, fin_ligne);
    for (couche = params_feff.prem_ss ; couche ; couche = couche->suivante)
    {
      fprintf(fichier, "SS %d %d %d %f%s", couche->index, couche->potentiel, 
	      couche->degenerescence, couche->distance, fin_ligne);
    }
  }

  /* Ajouts de FEFF modifi pour le programme xanes */

  if ( params_feff.Utiliser_Xanes == VRAI )
  {
    if (params_feff.VINTFIX > 0)
    {
      fprintf(fichier, "%s* D"e_AIGU"calage de l'origine avant le 0 des potentiels muffin-tin%s",
	      fin_ligne, fin_ligne);
      fprintf(fichier, "VINTFIX %f ! Exprim"e_AIGU" en eV%s", params_feff.VINTFIX, fin_ligne);
    }

    if (params_feff.EGRID.nombre > 0)
    {
      int i;

      fprintf(fichier, "%s* D"e_AIGU"finition d'une grille pour calculer les valeurs%s",
	      fin_ligne, fin_ligne);
      fprintf(fichier, "EGRID %d ", params_feff.EGRID.nombre);
      for (i = 0 ; i < params_feff.EGRID.nombre ; i++)
	fprintf(fichier, "%f ", params_feff.EGRID.position[i]);
      fprintf(fichier, "%sEMESH ", fin_ligne);
      for (i = 0 ; i < params_feff.EGRID.nombre ; i++)
	fprintf(fichier, "%f ", params_feff.EGRID.delta_x[i]);

      fprintf(fichier, "%s", fin_ligne);
    }
  }

  fclose(fichier); /* On a fini d'crire le fichier */
}
