/* Traitement de l'E.X.A.F.S. sur Atari
                  Tout ce qui concerne le chargement/la sauvegarde de fichiers rutilisables
                  Fichier des dclarations locales  ce module
   (c) 1998 - Emmanuel CURIS

  18 mars 1998
    * Ajout import_ascii() : import des fichiers en ASCII brut

  12 mars 1998
    * Supprim L13_fondre()  : aucune raison d'exister
               L13_somme()   : remplace par DCI_somme()
               LII_separer() : remplace par les DCI_xxx()
               LII_mono()    : remplace par les DCI_xxx()
    * Supprim type.fondre_elements : aucun sens quand les x sont identiques de fusionner...

  11 mars 1998
    * Ajout 2 champs dans MOD_extrait : type de moyenne  faire en absorption

	10 mars 1998
	  * Modifi MOD_extrait : possibilit d'utiliser deux chambres d'ionisation (I1 & I2)
	  * Cration des routines de chargement pour 2 chambres :
	          routine gnrale       = ELEM_2chambres()
	          routine d'appel simple = DCI_2chambres()

  6, 9 & 10 fvrier 1998
    * L13_separe() devient DCI_separe()
    * Dclaration de ELEM_separe : un spectre par lment, sur une mme colonne
    * Rcupr les dclarations dans graphes.h
    * Modifi MOD_extrait : champ tri, VRAI si on veut trier
                            champs sur extraction mono/multi spectres/lments
*/

/* ~~~~~~~~~~~~~~~ Chargement et sauvegarde au format interne ~~~~~~~~~~~~~~~ */


Graphe *charger(char *nom);      /* Charger un graphe */
Graphe *ch_interne(char *nom);   /* Routine effective */

boolean sauver (Graphe *graphe); /* Sauver le graphe  */

/* ~~~~~~~~~~~~~~~ Import et export de fichiers ASCII ~~~~~~~~~~~~~~~~~~~~~~~ */

void    importer   (void);
boolean import_ascii(char *nom, int nbr_col, int x, int y, unsigned int lg_en_tete, char commentaire);
boolean import_eda (char *nom);
boolean import_gnu (void);
boolean import_feff(void);

boolean fef_feff(char *nom);               /* Fichiers FEFFnnnn.DAT            */
boolean fef_chip(char *nom, boolean mode); /* Fichiers CHIPnnnn.DAT et CHI.DAT */
boolean fef_xmu (char *nom);               /* Fichiers XMU.DAT                 */

void exporter  (void);
void export_eda(void);

boolean convertit   (FILE *fichier, unsigned long nbr_lignes); /* Conversion effective */

/* ~~~~~~~~~~~~~~~ Extraction du spectre d'un fichier exprimental ~~~~~~~~~~~~~~~~~~~~~ */

typedef struct
{
	unsigned fluoresce        : 1; /* VRAI -> on bosse en fluorescence                */
	unsigned multi_spectres   : 1; /* VRAI -> plusieurs spectres dans le fichier      */
	unsigned ajouter_spectres : 1; /* VRAI -> on veut sommer les graphes              */
	unsigned moyenner_spectres: 1; /* VRAI -> on moyenne les graphes                  */
	unsigned fondre_spectres  : 1; /* VRAI -> on veut fondre les graphes              */
	unsigned choix_spectres   : 1; /* VRAI -> on veut ne garder que certains graphes  */
	unsigned multi_elements   : 1; /* VRAI -> plusieurs lments dans le fichier      */
	unsigned ajouter_elements : 1; /* VRAI -> on veut sommer les graphes              */
	unsigned moyenner_elements: 1;
	unsigned choix_elements   : 1; /* VRAI -> on veut ne garder que certains lments */
	unsigned deux_chambres    : 1; /* VRAI -> il y a deux chambres d'ionisation       */
	unsigned garde_i0         : 1; /* VRAI -> on garde I0                             */
	unsigned garde_i1         : 1; /* VRAI -> on garde I1                             */
	unsigned garde_i2         : 1; /* VRAI -> on garde I2 (s'il existe)               */
	unsigned garde_signal     : 1; /* VRAI -> on garde le signal                      */
	unsigned doublons         : 1; /* VRAI -> on vrifie qu'il n'y a pas de doublons  */
	unsigned tri              : 1; /* VRAI -> on trie le spectre final                */
	unsigned spr_sigma_log    : 1; /* VRAI -> moyenne des log ; FAUX -> log de la moyenne */
	unsigned elm_sigma_log    : 1; /* idem, mais pour les lments cette fois (en absorption...) */
} MOD_extrait;

/* --------------- Routines gnrales -------------------------------------------------- */

void    extraire    (void); /* Conversion I1/I0 = f(E) ==> signal */
boolean GER_extrait (int num_objet, int x, int y);

boolean ELEM_2chambres(FILE *fichier, unsigned long nbr_points, MOD_extrait *type,
                       double *x, double *y_1, double *y_2, double *i_0, double *i_1, double *i_2);
boolean ELEM_separe   (FILE *fichier, unsigned long nbr_points, unsigned int nbr_elements, MOD_extrait *type,
                       double *x, double **y, double *i_0, double **i_1);
boolean ELEM_somme    (FILE *fichier, unsigned long nbr_points, unsigned int nbr_elements, MOD_extrait *type,
                       double *x, double *y, double *i_0, double *i_1, boolean *util_element);

/* --------------- Routines communes aux deux lignes de DCI ------------------------------\
 | Fonctionnent pour EXAFS XIII (Acquvisu) et EXAFS II si un seul spectre dans le fichier |
 |         ou EXAFS II si on spare les spectres du fichier -> NE ferme PAS le fichier... |
 |
 | DCI_2chambres : deux chambres d'ionisation successives sur la ligne...                 |
 | DCI_mono      : un seul lment dans le fichier (cas le plus simple)                   |
 | DCI_separe    : le fichier a n lments, on fait un spectre pour chaque                |
 | DCI_somme     : on fait la somme de n lments (ventuellement moyenne)               |
 \--------------------------------------------------------------------------------------- */

boolean DCI_2chambres(char *nom, FILE *fichier, unsigned long nbr_ligne, MOD_extrait *type);
boolean DCI_mono     (char *nom, FILE *fichier, unsigned long nbr_ligne, MOD_extrait *type);
boolean DCI_separe   (char *nom, FILE *fichier, unsigned long nbr_ligne, MOD_extrait *type);
boolean DCI_somme    (char *nom, FILE *fichier, unsigned long nbr_ligne, MOD_extrait *type);

/* --------------- Fichier de la ligne Exafs II ---------------------------------------- */

boolean ligne_l2   (char *nom, MOD_extrait *type);
boolean LII_fondre (char *nom, FILE *fichier, unsigned long nbr_ligne, MOD_extrait *type);
boolean LII_somme  (char *nom, FILE *fichier, unsigned int nbr_spectres, unsigned long nbr_ligne, MOD_extrait *type);

/* --------------- Fichier de la ligne Exafs III (maintenant XIII) --------------------- */

boolean ligne_l3   (char *nom, MOD_extrait *type); /* Type EXAFS III (LURE)  */

boolean ligne_italie(boolean fluorescence);         /* Type Frascati (Italie) */

/* ~~~~~~~~~~~~~~~ Routines de gestion GEM associes ~~~~~~~~~~~~~~~~~~~~~~~~~ */

boolean GER_feff(int num_objet, int x_souris, int y_souris);
boolean GER_svgem(int num_objet, int x_souris, int y_souris);
