/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1998, Emmanuel CURIS - Cration de ce fichier: 23 juin 1998

   Module de filtrage des spectres (essentiellement par transforme de Fourier)

   Paramtrage de la transforme de Fourier inverse
   Source original rcupr de ../FOURIER.c et ../EXF_GEM.c
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "ERREURS.H"
#include "GRAPHES.H"
#include "FOURIER.H"

/* ******** Variables locales ************************ */

static int num_tfi_fenetre = FEN_HANNING; /* Fentre  l'ouverture du module */

extern const char *Types_fenetres[];      /* Dans fourier/fourier.c : noms des fentres */

/* ******** Fonctions locales (gestion de l'interface) ******** */

static void RDS_tfi(Fenetre *, int, int, int, int);
static boolean MSG_tfi(Fenetre *fen);
static boolean GER_tfi(int num_objet, int x, int y);
static boolean CLC_tfi(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

static void met_les_r(TFI_infos *infos);

/* ******** Ralisation pratique des fonctions **************** */

/* -------------- Ouverture de la fentre ----------------------------------------------- */

void TFI_ouvre(Graphe *tf)
{
  Fenetre *GEM_fenetre;
  TFI_infos *infos;

  if (tf == NULL) return; /* Rien  faire */
  if (tf->drapeaux.protege == VRAI)
  {
    EXF_ERREUR(GRF_PROTEGE); /* Graphe protg -> appel impossible */
    return;
  }

  GEM_fenetre = creer_mixte_fenetre(NAME|CLOSER|FULLER|INFO|SIZER|MOVER, PARAM_FOUR_INVER, HAUT,
				    RDS_tfi, ( boolean (*)(Fenetre *, unsigned int, int) ) NULL, CLC_tfi,
				    MSG_tfi, RCT_pointeur, ( boolean (*)(Fenetre *) ) NULL);
  if (GEM_fenetre == NULL) return; /* Rien  faire... */

  arbres[PARAM_FOUR_INVER].fonction.arbre = GER_tfi; /* Fonction grant la partie arbre GEM */

  GEM_fenetre->msg_masque.fermeture = VRAI; /* Messages traits par MSG_tf */
  GEM_fenetre->drapeaux.redessin    = VRAI;
  GEM_fenetre->drapeaux.pointeur    = VRAI;

  infos = (TFI_infos *) GEM_fenetre->adresse;
  if (infos) /* Dj des infos sur une TF -> on les perd */
  {
    if (infos->graphe == tf) /* Mme graphe -> pas  se fouler */
      return;
    else /* On efface ces infos pour en crer de nouvelles */
    {
      num_tfi_fenetre = infos->type_fenetre;
      GRF_efface(infos->fenetre, VRAI, VRAI);
      infos->graphe->drapeaux.protege = VRAI; /* Pas utilis -> on peut l'effacer */
      free(infos);
    }
  }

  infos = (TFI_infos *) malloc( sizeof(TFI_infos) ); /* On rserve les infos de TF inverse */
  if (infos == NULL)
  {
    ferme_fenetre(GEM_fenetre); /* On ferme la fentre : sert  rien... */
    ERREUR(MANQUE_MEMOIRE);
    return;
  }
  GEM_fenetre->adresse  = infos;
  GEM_fenetre->pointeur = tf;

#ifdef _FRANCAIS_
  infos->fenetre = GRF_creer(tf->nombre_point, GR_INTERNE, GR_IM_MOD,
			     "Fen"e_CIRC"tre pour le calcul de la transform"e_AIGU"e de Fourier inverse",
			     "Longueur du chemin", "Pond"e_AIGU"ration", UNIT_ANG, UNIT_NORM);
#endif
#ifdef _ENGLISH_
  infos->fenetre = GRF_creer(tf->nombre_point, GR_INTERNE, GR_IM_MOD,
			     "Window for back Fourier transform computation",
			     "Path length", "Weighting factor", UNIT_ANG, UNIT_NORM);
#endif

  if (infos->fenetre == NULL) /* Mmoire insuffisante */
  {
    free(infos);
    ferme_fenetre(GEM_fenetre);
    return;
  }
  infos->fenetre->couleur = BLEU;
  infos->fenetre->drapeaux.protege = VRAI; /* Protg -> ineffaable */

  infos->graphe = tf; /* Le graphe sur lequel on travaille */
  tf->drapeaux.protege = VRAI;  /* Il est donc ineffaable... */
  
  /* Paramtres de la TFI (fentre en particulier) */

  infos->r_min = tf->x_min + 1; /* Intervalle pour lequel on calculera */
  infos->r_max = tf->x_max - 1;
  infos->r_1 = infos->r_min + 1;
  infos->r_2 = infos->r_max - 1;
  met_fvaleur(PARAM_FOUR_INVER, TF_INV_RMIN, infos->r_min);
  met_fvaleur(PARAM_FOUR_INVER, PRPTFI_R1  , infos->r_1  );
  met_fvaleur(PARAM_FOUR_INVER, PRPTFI_R_2 , infos->r_2  );
  met_fvaleur(PARAM_FOUR_INVER, TF_INV_RMAX, infos->r_max);
  infos->symetrique = OBJ_choisit(PARAM_FOUR_INVER, PRPTFI_SYMETRIK, QUESTION);

  /* Calcul de la TF-inverse : fentre */

  infos->type_fenetre = num_tfi_fenetre;
  infos->num_depart   = FEN_RECTANGLE; /* Affichage  partir de la premire */
  PDFL_textes(PARAM_FOUR_INVER, TF_INV_FEN1, TF_INV_FEN_4,
	      TF_INV_FEN_AVANT, TF_INV_FEN_APRES, TF_INV_FEN_FOND, TF_INV_FEN_BOUT,
	      NOMBRE_FENETRES, (char **) Types_fenetres, NULL, num_tfi_fenetre, 15, FEN_RECTANGLE);

  infos->parametre = prend_fvaleur(PARAM_FOUR_INVER, TF_INV_PARAM);
  TF_fenetre(infos->fenetre, tf, infos->type_fenetre, infos->r_min, infos->r_max, infos->r_1, infos->r_2,
	     infos->parametre, infos->symetrique); /* On calcule la fentre */
}

/* ----------- Redessin de cette fentre ------------------------------------------------ */

static void RDS_tfi(Fenetre *fenetre, int x, int y, int larg, int haut)
{
  Graphe *graphe;
  DES_graphe param_dessin = {NULL, NULL, 1, 0};
  TFI_infos *infos;
  double x_echelle, y_echelle;
  int y_trace, haut_trace; /* Zone de dessin du graphe */

  infos = (TFI_infos *) fenetre->adresse;

  vswr_mode(_PRG_station->handle,MD_REMPLACE);

  /* Fond blanc */

  vsf_color(_PRG_station->handle, BLANC);
  points[0] = x;        points[1] = y;
  points[2] = x + larg; points[3] = y + haut;
  v_bar(_PRG_station->handle,points);

  graphe = infos->graphe;

  /* Zone de dessin du graphe */

  y_trace    = fenetre->w_y    + OB_HEIGHT(arbres[PARAM_FOUR_INVER].adresse,ROOT);
  haut_trace = fenetre->w_haut - OB_HEIGHT(arbres[PARAM_FOUR_INVER].adresse,ROOT);

  /* Calcul des facteurs d'chelle */

  x_echelle = (double) (fenetre->w_larg) / (graphe->x_max - graphe->x_min);
  y_echelle = (double) (haut_trace)      / (graphe->y_max - graphe->y_min);

  /* Le graphe dont on fait la TF  */

  param_dessin.graphe  = graphe;
  param_dessin.couleur = NOIR;  /* Graphe normal en noir */
  param_dessin.style   = 1;     /* Ligne continue        */
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace,
	     graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max,
	     x_echelle, y_echelle);

  /* Dessin de la fentre */

  param_dessin.graphe  = infos->fenetre;
  param_dessin.couleur = BLEU - 1;  /* Fentre en bleu, si possible */
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace,
             infos->fenetre->x_min, infos->fenetre->x_max, infos->fenetre->y_min, infos->fenetre->y_max,
             x_echelle, haut_trace);

  /* Dessin du graphe multipli par la fentre  - en violet */
  vsl_color(_PRG_station->handle, VIOLET);
  vsl_type (_PRG_station->handle, 1);

  if (graphe->drapeaux.complexe) /* Graphe dont on fait la TF inverse complexe */
  {
    unsigned long i;
    int compteur = 0;
    register double   *x     = graphe->x;
    register Complexe *y_fen = (Complexe *) (infos->fenetre->y),
                      *y     = (Complexe *) (graphe->y);

    /* Trac du module */

    for (i = 0 ; i < graphe->nombre_point ; i++)
    {
      double module = sqrt(y->re * y->re + y->im * y->im);
      points[compteur]   = fenetre->w_x + x_echelle * (*x - graphe->x_min);
      points[compteur+1] = y_trace      + y_echelle * (graphe->y_max - module * y_fen->re);
      compteur += 2;

      x++; y++; y_fen++;

      if (compteur >= 256) /* Tableau de points complet */
      {
	v_pline(_PRG_station->handle,128,points);
	points[0] = points[254];
	points[1] = points[255];
	compteur = 2;
      }
    }
    v_pline(_PRG_station->handle,(int) (compteur-1)/2,points);
		
    /* Trac de la partie imaginaire */

    compteur = 0;
    x = graphe->x; y = (Complexe *) graphe->y; y_fen = (Complexe *) infos->fenetre->y;
    for (i = 0 ; i < graphe->nombre_point ; i++)
    {
      points[compteur]   = fenetre->w_x + x_echelle * (*x - graphe->x_min);
      points[compteur+1] = y_trace      + y_echelle * (graphe->y_max - y->im * y_fen->re);
      compteur += 2;
      
      x++; y++; y_fen++;
      
      if (compteur >= 256) /* Tableau de points complet */
      {
	v_pline(_PRG_station->handle,128,points);
	points[0] = points[254];
	points[1] = points[255];
	compteur = 2;
      }
    }
    v_pline(_PRG_station->handle, (int) (compteur-1)/2, points);
  }
  else /* Trac du graphe * la fentre, si c'est un graphe rel */
  {
    unsigned long index;
    int compteur = 0;
    register double   *x     = graphe->x,
                      *y     = graphe->y;
    register Complexe *y_fen = (Complexe *) (infos->fenetre->y);

    x = graphe->x; y = graphe->y; y_fen = (Complexe *) infos->fenetre->y;
    for (index = 0 ; index < graphe->nombre_point ; index++)
    {
      points[compteur]   = fenetre->w_x + x_echelle * (*x - graphe->x_min);
      points[compteur+1] = y_trace      + y_echelle * (graphe->y_max - *y * y_fen->re);
      compteur += 2;

      x++; y++; y_fen++;
      
      if (compteur >= 256) /* Tableau de points complet */
      {
	v_pline(_PRG_station->handle,128,points);
	points[0] = points[254];
	points[1] = points[255];
	compteur = 2;
      }
    }
    v_pline(_PRG_station->handle, compteur >> 1, points);
  }

  /* Mise en place des barres pour k_min (rouge) et k_max (vert) */

  vsl_color(_PRG_station->handle, ROUGE);
  points[0] = fenetre->w_x + x_echelle * (infos->r_min - graphe->x_min);
  points[1] = y_trace;
  points[2] = points[0];
  points[3] = y_trace + haut_trace;
  v_pline(_PRG_station->handle,2,points);
	
  vsl_color(_PRG_station->handle, VERT);
  points[0] = fenetre->w_x + x_echelle * (infos->r_max - graphe->x_min);
  points[2] = points[0];
  v_pline(_PRG_station->handle,2,points);

  /* Mise en place des barres pour k_1 et k_2, si la fentre s'y prte */
  
  if ( (infos->type_fenetre == FEN_HANNING) || (infos->type_fenetre == FEN_HAMMING)
    || (infos->type_fenetre == FEN_BARTLETT) )
  {
    double r_2;

    vsl_color(_PRG_station->handle, ROUGE_FONCE);
    points[0] = fenetre->w_x + x_echelle * (infos->r_1 - graphe->x_min);
    points[2] = points[0];
    v_pline(_PRG_station->handle, 2, points);	

    if (infos->symetrique)
      r_2 = infos->r_max - (infos->r_1 - infos->r_min);
    else
      r_2 = infos->r_2;

    vsl_color(_PRG_station->handle, VERT_FONCE);
    points[0] = fenetre->w_x + x_echelle * (r_2 - graphe->x_min);
    points[2] = points[0];
    v_pline(_PRG_station->handle, 2, points);	
  }
}

/* ------------ Traitement des messages reus par la fentre ----------------------------------------------- */

static boolean MSG_tfi(Fenetre *fenetre)
{
  Graphe *graphe, *tf_fenetre, *fourier;
  TFI_infos *infos; /* Les infos pour la fentre, pour la TF */

  infos      = (TFI_infos *) fenetre->adresse;
  graphe     = infos->graphe;
  tf_fenetre = infos->fenetre;

  if ( buffer[0] == WM_CLOSED ) /* On demande la fermeture de la fentre */
  {
    int reponse = EXF_ERREUR(FIN_MODULE);
    if (reponse == 3) return VRAI; /* On annule */

    num_tfi_fenetre = infos->type_fenetre;
    if (reponse == 2) /* On quitte sans calculer la TF */
    {
      GRF_efface(tf_fenetre, VRAI, VRAI); /* On efface ce graphe          */
      graphe->drapeaux.protege = FAUX;
      free(infos);                /* On libre la mmoire         */
      ferme_fenetre(fenetre);     /* On ferme la fentre          */
      return VRAI;
    }

    /* On ferme la fentre : les choses srieuses commencent... */

    FEN_ferme(fenetre, FAUX); /* vite un sale message de redessin */

    fourier = TFI_init(infos); /* On prpare la TF inverse */
    if (fourier == NULL) /* Calcul annul (utilisateur ou problme) */
    {
      FEN_reouvre(fenetre);
      return VRAI;
    }

    ferme_fenetre(fenetre); /* On ferme entirement la fentre */

    fourier->drapeaux.protege = VRAI;

    /* Calcul de la transforme de Fourier - le spectre  TFer est dans tf_fenetre */

    if ( OBJ_choisit(PARAM_TFI, TFI_RAPIDE, QUESTION) ) /* On veut la mthode rapide */
    {
      TRQ_infos infos_troncature;
      Graphe *nv_fourier;

      TFI_rapide(tf_fenetre, fourier); /* Calcul effectif */

      infos_troncature.x_min = prend_fvaleur(PARAM_TFI, TFI_KMIN);
      infos_troncature.x_max = prend_fvaleur(PARAM_TFI, TFI_KMAX);
      infos_troncature.type = TRONQUE_EXT; /* On garde l'intrieur */
      nv_fourier = TRQ_calc(fourier, &infos_troncature);
      if (nv_fourier)
      {
	GRF_efface(fourier, VRAI, VRAI);
	fourier = nv_fourier;
	fourier->drapeaux.protege = VRAI;
      }
    }
    else
      TFI_simple(tf_fenetre, fourier);

    /* On dconvolue la fentre utilise lors du calcul de la TF inverse */
    TFI_fenetre(fourier, (TF_infos *) graphe->pointeur);
    GRF_bornes(fourier, ORDONNE, DESORDONNE); /* Les bornes du spectre ainsi obtenu */

    /* On rcupre le spectre EXAFS proprement dit == la partie relle du spectre transform... */

    if ( OBJ_choisit(PARAM_TFI, TFI_SPECTRE, QUESTION) ) /* On garde le signal EXAFS */
    {
      char titre[100];
      Graphe *exafs;

#ifdef _FRANCAIS_
      sprintf(titre, "EXAFS filtr"e_AIGU" de %s", infos->graphe->nom);
      exafs = GRF_creer(fourier->nombre_point, GR_EXAFS, GR_NORMAL,
			titre, "Nombre d'onde", "khi(k)", UNIT_1_ANG, UNIT_NORM);
#endif
#ifdef _ENGLISH_
      sprintf(titre, "EXAFS filtered from %s", infos->graphe->nom);
      exafs = GRF_creer(fourier->nombre_point, GR_EXAFS, GR_NORMAL,
			titre, "Wave number", "chi(k)", UNIT_1_ANG, UNIT_NORM);
#endif

      if (exafs)
      {
	CMP_reelle  (fourier, exafs);  /* Extraction de la partie relle      */
	exafs->drapeaux.protege = FAUX;
      }
    }

    if ( OBJ_choisit(PARAM_TFI,TFI_AMPLITUDE, QUESTION) ) /* On veut garder l'amplitude = module */
    {
      char titre[100];
      Graphe *module;

#ifdef _FRANCAIS_
      sprintf(titre, "Amplitude du signal EXAFS de %s", infos->graphe->nom);
      module = GRF_creer(fourier->nombre_point, GR_EXAFSAMP, GR_NORMAL,
			 titre, "Nombre d'onde", "|khi(k)|", UNIT_1_ANG, UNIT_NORM);
#endif
#ifdef _ENGLISH_
      sprintf(titre, "EXAFS amplitude, from %s", infos->graphe->nom);
      module = GRF_creer(fourier->nombre_point, GR_EXAFSAMP, GR_NORMAL,
			 titre, "Wave number", "|chi(k)|", UNIT_1_ANG, UNIT_NORM);
#endif

      if (module)
      {
	CMP_module  (fourier, module); /* Extraction du module, si demand */
	module = corr_amplitude(module, 0, 0, 0); /* Demande des corrections d'amplitude */
	met_fvaleur(CHOIX_PHASE, PHA_LONGUEUR, prend_fvaleur(CHOIX_AMPLITUDE, AMP_LONGUEUR));
      }
    }

    if ( OBJ_choisit(PARAM_TFI, TFI_PHASE, QUESTION) ) /* On veut le dphasage = argument */
    {
      char titre[100];
      Graphe *phase;
      
#ifdef _FRANCAIS_
      sprintf(titre, "D"e_AIGU" du signal EXAFS de %s", infos->graphe->nom);
      phase = GRF_creer(fourier->nombre_point, GR_EXAFSPHA, GR_NORMAL,
			titre, "Nombre d'onde", "Arg(khi(k))", UNIT_1_ANG, UNIT_NORM);
#endif
#ifdef _ENGLISH_
      sprintf(titre, "EXAFS phase, from %s", infos->graphe->nom);
      phase = GRF_creer(fourier->nombre_point, GR_EXAFSPHA, GR_NORMAL,
			titre, "Wave number", "Arg(chi(k))", UNIT_1_ANG, UNIT_NORM);
#endif

      if (phase)
      {
	CMP_argument(fourier, phase); /* Extraction de la phase, si demande */
	phase = corr_phase(phase, 0);
	/* Ajouter les corrections de dphasage */
      }
    }

    /* On peut supposer que l'on a fini ici... */

    GRF_efface(tf_fenetre, VRAI, VRAI); /* Plus besoin de la fentre */
    fourier->drapeaux.protege = FAUX; /* On a fini de calculer sur ces graphes */
    graphe->drapeaux.protege  = FAUX;
    GRF_aller(fourier); /* On met ce graphe en graphe courant */

    return VRAI;
  }

  return VRAI; /* On a trait tout ce que l'on voulait traiter */
}

/* --------- Gestion d'un clic sur la fentre de TF inverse --------------------------------- */

static boolean CLC_tfi(Fenetre *fenetre, int bouton, int n_clic, int etat, int x, int y)
{
  TFI_infos *infos;
  double x_echelle, x_clic;

  if (y < fenetre->w_y + OB_HEIGHT(arbres[PARAM_FOUR_INVER].adresse,ROOT) ) return VRAI;

  infos = (TFI_infos *) fenetre->adresse;
  x_echelle = ( (double) fenetre->w_larg ) / (infos->graphe->x_max - infos->graphe->x_min);
  x_clic  = x - fenetre->w_x;
  x_clic /= x_echelle;
  x_clic += infos->graphe->x_min;

  if (bouton == 1) /* Bouton gauche */
  {
    if ( (etat & K_LSHIFT) || (etat & K_RSHIFT) ) /* On appuie sur shift en mme temps */
    {
      if ( Autorise_Symetrique( infos->type_fenetre ) )
	infos->r_1 = x_clic;
      else
	infos->r_min = x_clic;
    }
    else
      infos->r_min = x_clic;
  }
  else /* Bouton droit */
  {
    if ( (etat & K_LSHIFT) || (etat & K_RSHIFT) ) /* On appuie sur shift en mme temps */
    {
      if ( Autorise_Symetrique( infos->type_fenetre ) && (infos->symetrique == FAUX) )
	infos->r_2 = x_clic;
      else
	infos->r_max = x_clic;
    }
    else
      infos->r_max = x_clic;
  }

  met_les_r(infos);

  if (infos->r_min > infos->r_max) /* On vrifie la cohrence */
  {
    x_clic = infos->r_min;
    infos->r_min = infos->r_max;
    infos->r_max = x_clic;
  }

  met_fvaleur(PARAM_FOUR_INVER, FOUR_KMIN, infos->r_min);
  met_fvaleur(PARAM_FOUR_INVER, FOUR_KMAX, infos->r_max);

  TF_fenetre(infos->fenetre, infos->graphe, infos->type_fenetre, infos->r_min, infos->r_max,
	     infos->r_1, infos->r_2, infos->parametre, infos->symetrique);

  FEN_redessin(fenetre); /* On redessine : fentre a chang... */
  
  return VRAI + 0 * etat + 0 * n_clic;
}


/* --------------- Prparation de la TFI (choix de la fentre principalement) ----------------------------- */

static boolean GER_tfi(int num_objet, int x_souris, int y_souris)
{
  Fenetre *fen_cour;
  Graphe *fenetre;
  TFI_infos *infos;
  extern const char *Types_fenetres[];

  fen_cour = ARB_fenetre(arbres + PARAM_FOUR_INVER); /* Fentre actuelle */
  infos = (TFI_infos *) fen_cour->adresse;
  fenetre = infos->fenetre;

  if ( (num_objet >= TF_INV_FEN1) && (num_objet <= TF_INV_FEN_4) )
  {
    unsigned long nv_fenetre = infos->type_fenetre; /* Nouvelle fentre choisie */
    unsigned long prem_visible = infos->num_depart;
    
    GDFL_textes(PARAM_FOUR_INVER, TF_INV_FEN1, TF_INV_FEN_4,
		TF_INV_FEN_AVANT, TF_INV_FEN_APRES, TF_INV_FEN_FOND, TF_INV_FEN_BOUT,
		NOMBRE_FENETRES, (char **) Types_fenetres, NULL, &nv_fenetre, 15,
		&prem_visible, num_objet, y_souris);

    if ( nv_fenetre != infos->type_fenetre )
    {
      boolean etat;
      
      etat = (nv_fenetre == FEN_HANNING) || (nv_fenetre == FEN_HAMMING)
	  || (nv_fenetre == FEN_BARTLETT); /* VRAI -> asymtrie possible */
      OBJ_active(PARAM_FOUR_INVER, PRPTFI_SYMETRIK, etat);
      OBJ_active(PARAM_FOUR_INVER, PRPTFI_R1, etat);
      
      etat &= ! (OBJ_choisit(PARAM_FOUR_INVER, PRPTFI_SYMETRIK, QUESTION));
      OBJ_active(PARAM_FOUR_INVER, PRPTFI_R_2, etat);
      
      etat = (nv_fenetre == FEN_HAMMING) || (nv_fenetre == FEN_NORTON_F3)
	  || (nv_fenetre == FEN_BESSEL)  || (nv_fenetre == FEN_GAUSSIENNE);
      OBJ_active(PARAM_FOUR_INVER, TF_INV_PARAM, etat); /* Seule fentre avec un paramtre... */

      infos->type_fenetre = (int) nv_fenetre; /* De toute faon, il n'y aura jamais max_int fentres ! */
      TF_fenetre(fenetre, infos->graphe, infos->type_fenetre, infos->r_min, infos->r_max,
		 infos->r_1, infos->r_2, infos->parametre, infos->symetrique); /* On calcule la fentre */
      FEN_redessin(fen_cour);
    }
  }

  switch (num_objet)
  {
  case TF_INV_FEN_FOND:  /* Dfileur des fentres */
  case TF_INV_FEN_BOUT:
  case TF_INV_FEN_APRES:
  case TF_INV_FEN_AVANT:
    {
      unsigned long nv_fenetre = infos->type_fenetre,
      prem_visible = infos->num_depart;

      GDFL_textes(PARAM_FOUR_INVER, TF_INV_FEN1, TF_INV_FEN_4,
		  TF_INV_FEN_AVANT, TF_INV_FEN_APRES, TF_INV_FEN_FOND, TF_INV_FEN_BOUT,
		  NOMBRE_FENETRES, (char **) Types_fenetres, NULL, &nv_fenetre, 15,
		  &prem_visible, num_objet, y_souris);
		  infos->num_depart = (int) prem_visible;
    }
    break;

  case TF_INV_RMIN: /* Demande de R min  l'utilisateur */
    met_chaine(CHOIX_VALEUR, CHXVAL_UNITE , 0, ANGSTROM, TXT_REMPLACE);
    met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "D"e_AIGU"but de la fen"e_CIRC"tre", TXT_REMPLACE);
    met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->r_min);
    
    ext_formdo(CHOIX_VALEUR);
    
    infos->r_min = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
    met_fvaleur(PARAM_FOUR_INVER, TF_INV_RMIN, infos->r_min);
    
    TF_fenetre(fenetre, infos->graphe, infos->type_fenetre, infos->r_min, infos->r_max,
	       infos->r_1, infos->r_2, infos->parametre, infos->symetrique); /* On calcule la fentre */
    FEN_redessin(fen_cour);
    return VRAI;

  case PRPTFI_R1:
    met_chaine(CHOIX_VALEUR, CHXVAL_UNITE , 0, ANGSTROM, TXT_REMPLACE);
    met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "Fin de mont"e_AIGU"e de la fen"e_CIRC"tre", TXT_REMPLACE);
    met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->r_1);

    ext_formdo(CHOIX_VALEUR);
    
    infos->r_1 = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
    met_fvaleur(PARAM_FOUR_INVER, PRPTFI_R1, infos->r_1);
    
    TF_fenetre(fenetre, infos->graphe, infos->type_fenetre, infos->r_min, infos->r_max,
	       infos->r_1, infos->r_2, infos->parametre, infos->symetrique); /* On calcule la fentre */
    FEN_redessin(fen_cour);
    return VRAI;
    
  case PRPTFI_R_2:
    met_chaine(CHOIX_VALEUR, CHXVAL_UNITE , 0, ANGSTROM, TXT_REMPLACE);
    met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "D"e_AIGU"but de descente de la fen"e_CIRC"tre", TXT_REMPLACE);
    met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->r_2);

    ext_formdo(CHOIX_VALEUR);

    infos->r_2 = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
    met_fvaleur(PARAM_FOUR_INVER, PRPTFI_R_2, infos->r_2);
    
    TF_fenetre(fenetre, infos->graphe, infos->type_fenetre, infos->r_min, infos->r_max,
	       infos->r_1, infos->r_2, infos->parametre, infos->symetrique); /* On calcule la fentre */
    FEN_redessin(fen_cour);
    return VRAI;

  case TF_INV_RMAX: /* Demande de k max  l'utilisateur */
    met_chaine(CHOIX_VALEUR, CHXVAL_UNITE , 0, ANGSTROM, TXT_REMPLACE);
    met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "Fin de la fen"e_CIRC"tre", TXT_REMPLACE);
    met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->r_max);

    ext_formdo(CHOIX_VALEUR);

    infos->r_max = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
    met_fvaleur(PARAM_FOUR_INVER, TF_INV_RMAX, infos->r_max);

    TF_fenetre(fenetre, infos->graphe, infos->type_fenetre, infos->r_min, infos->r_max,
	       infos->r_1, infos->r_2, infos->parametre, infos->symetrique); /* On calcule la fentre */
    FEN_redessin(fen_cour);
    break;

  case TF_INV_PARAM:
    met_chaine(CHOIX_VALEUR, CHXVAL_UNITE, 0, NULL, TXT_REMPLACE);
    met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "Param"e_GRAVE"tre de fen"e_CIRC"tre", TXT_REMPLACE);
    met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->parametre);

    ext_formdo(CHOIX_VALEUR);
    
    infos->parametre = prend_fvaleur(CHOIX_VALEUR,CHXVAL_VALEUR);
    met_fvaleur(PARAM_FOUR_INVER, TF_INV_PARAM, infos->parametre);
    
    TF_fenetre(fenetre, infos->graphe, infos->type_fenetre, infos->r_min, infos->r_max,
	       infos->r_1, infos->r_2, infos->parametre, infos->symetrique); /* On calcule la fentre */
    FEN_redessin(fen_cour);
    break;

  case PRPTFI_SYMETRIK:
    infos->symetrique = OBJ_choisit(PARAM_FOUR_INVER, PRPTFI_SYMETRIK, ECHANGE);
    OBJ_active(PARAM_FOUR_INVER, PRPTFI_R_2, !(infos->symetrique) );
    TF_fenetre(fenetre, infos->graphe, infos->type_fenetre, infos->r_min, infos->r_max,
	       infos->r_1, infos->r_2, infos->parametre, infos->symetrique); /* On calcule la fentre */
    FEN_redessin(fen_cour);
  }
  
  return VRAI;
}

/* -------- Met les valeurs des R aprs tri (pour la cohrence) -------- */

static void met_les_r(TFI_infos *infos)
{
  if (infos->r_min > infos->r_max) /* On vrifie la cohrence */
  {
    double r = infos->r_min;
    infos->r_min = infos->r_max;
    infos->r_max = r;
  }
  if ( Autorise_Symetrique( infos->type_fenetre ) )
  {
    if (infos->symetrique)
    {
      if ( infos->r_1 < infos->r_min )
      {
	double r = infos->r_min;
	infos->r_min = infos->r_1;
	infos->r_1 = r;
      }
      if ( infos->r_1 > infos->r_max )
      {
	double r = infos->r_max;
	infos->r_max = infos->r_1;
	infos->r_1 = r;
      }
    }
    else
    {
      double r[4];
      int i, j;

      r[0] = infos->r_min;
      r[1] = infos->r_1;
      r[2] = infos->r_2;
      r[3] = infos->r_max;

      for (i = 0 ; i < 4 ; i++)
	for (j = i ; j < 4 ; j++)
	  if ( r[i] > r[j] )
	  {
	    double l = r[i];
	    r[i] = r[j]; r[j] = l;
	  }

      infos->r_min = r[0];
      infos->r_1   = r[1];
      infos->r_2   = r[2];
      infos->r_max = r[3];
    }
  }

  met_fvaleur(PARAM_FOUR_INVER, TF_INV_RMIN, infos->r_min);
  if ( Autorise_Symetrique( infos->type_fenetre ) )
  {
    met_fvaleur(PARAM_FOUR_INVER, PRPTFI_R1  , infos->r_1  );
    if (infos->symetrique == FAUX)
      met_fvaleur(PARAM_FOUR_INVER, PRPTFI_R_2 , infos->r_2  );
  }
  met_fvaleur(PARAM_FOUR_INVER, TF_INV_RMAX, infos->r_max);
}

/* ~~~~~~~~ Paramtres de calcul pour la TFI, autres que la fentre : type de mthode,... ~~~~~~~~ */

/* ----------------- Initialisation du calcul de la TF inverse ----------------------------\
 |     Renvoie le graphe qui contiendra la TF inverse                                      |
 |                                                                                         |
 | infos        : les informations de calcul de la TF inverse                              |
 \---------------------------------------------------------------------------------------- */

Graphe *TFI_init(TFI_infos *infos)
{
  TF_infos *param_tf;
  Graphe *resultat;
  boolean etat;

  /* On installe les infos de la TF, si on les a */

  infos->num_depart = FEN_RECTANGLE;
  if ( infos->graphe->pointeur ) /* Infos de TF inverse prsentes */
  {
    param_tf = (TF_infos *) (infos->graphe->pointeur);

    met_fvaleur(PARAM_TFI, TFI_PARAMETRE, param_tf->param_fenetre);
    met_fvaleur(PARAM_TFI, TFI_KMIN,      param_tf->k_min);
    met_fvaleur(PARAM_TFI, TFI_KMAX,      param_tf->k_max);
    met_fvaleur(PARAM_TFI, TFI_KMIN_FEN,  param_tf->k_min);
    met_fvaleur(PARAM_TFI, TFI_KMAX_FEN,  param_tf->k_max);
    met_fvaleur(PARAM_TFI, TFI_K_1,       param_tf->k_1  );
    met_fvaleur(PARAM_TFI, TFI_K_2,       param_tf->k_2  );
    OBJ_choisit(PARAM_TFI, TF_FEN_SYMETRIK, param_tf->symetrique);

    infos->num_choisi = param_tf->type_fenetre;
  }
  else
  {
    param_tf = (TF_infos *) malloc( sizeof(TF_infos) );
    if (param_tf == NULL)
    {
      ERREUR(MANQUE_MEMOIRE);
      printf("FILTRAGE/F_INV.c : dans TFI_init, cration de param_tf\n");
      return NULL;
    }

    infos->graphe->pointeur = param_tf; /* On installe les paramtres de fentre
					   associ  un graphe -> sera libr quand le graphe sera effac... */
    infos->num_choisi = FEN_RECTANGLE;
    OBJ_choisit(PARAM_TFI, TF_FEN_SYMETRIK, FAUX);
    param_tf->symetrique = FAUX;
  }

  PDFL_textes(PARAM_TFI, TFI_FENETRE_1, TFI_FENETRE_5, TFI_FEN_AVANT, TFI_FEN_APRES, TFI_FOND_ASC, TFI_BOUTON_ASC,
	      NOMBRE_FENETRES, (char **) Types_fenetres, NULL, infos->num_choisi, 15, FEN_RECTANGLE);
  
  etat = (infos->num_choisi == FEN_HANNING) | (infos->num_choisi == FEN_HAMMING)
    | (infos->num_choisi == FEN_BARTLETT);
  OBJ_active(PARAM_TFI, TF_FEN_SYMETRIK, etat); /* Seules fentres pour lesquelles a a un sens */
  OBJ_active(PARAM_TFI, TFI_K_1, etat);
  OBJ_active(PARAM_TFI, TFI_K_2, etat & (param_tf->symetrique));

  arbres[PARAM_TFI].pointeur = infos;

  if ( ext_formdo(PARAM_TFI) == TFI_ANNULE) /* On annule le calcul */
    return NULL;

  /* On prpare tout pour la Transforme de Fourier inverse */

  if ( OBJ_choisit(PARAM_TFI, TFI_RAPIDE, QUESTION) == VRAI )
  {
    resultat = TFI_prep_rapide(infos, prend_fvaleur(PARAM_TFI, TFI_KMIN), prend_fvaleur(PARAM_TFI, TFI_KMAX),
			       prend_fvaleur(PARAM_TFI, TFI_PAS));
  }
  else
  {
    resultat = TFI_prep_normal(infos, prend_fvaleur(PARAM_TFI, TFI_KMIN), prend_fvaleur(PARAM_TFI, TFI_KMAX),
			       prend_fvaleur(PARAM_TFI, TFI_PAS));
  }
  if (resultat == NULL) /* Erreur dans la prparation */
    return NULL;

  /* Paramtres de la fentre ayant servi lors du calcul de la TF directe */

  param_tf->k_min         = prend_fvaleur(PARAM_TFI, TFI_KMIN_FEN);
  param_tf->k_max         = prend_fvaleur(PARAM_TFI, TFI_KMAX_FEN);
  param_tf->param_fenetre = prend_fvaleur(PARAM_TFI, TFI_PARAMETRE);
  param_tf->type_fenetre  = infos->num_choisi;

  return resultat; /* On a fini l'initialisation */
}
