/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1998, Emmanuel CURIS - Cration de ce fichier: 23 juin 1998

   Module de filtrage des spectres (essentiellement par transforme de Fourier)

   Initialisation du calcul de la transforme de Fourier inverse
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "ERREURS.H"
#include "GRAPHES.H"
#include "FOURIER.H"
#include "LISSAGE.H"
#include "SCIENCE.H"

/* ~~~~~~~~ Prparation dans le cas d'un calcul par algorithme rapide simple ~~~~~~~~~~

   Mthode : utilise l'intervalle demand pour le spectre d'arriv et le pas pour connatre le nombre de points
             calcule alors le R max correspondant
	     lisse et extrapole  zro le spectre dont on veut la TFI ensuite           */

Graphe *TFI_prep_rapide(TFI_infos *infos, double k_min, double k_max, double pas)
{
  Graphe *source, *resultat;
  unsigned long nbr_pts, i;
  int bit_1 = 0;
  double pas_r, k;
  double *nv_x, *x;
  Complexe *nv_y, *y;

  if (infos == NULL) return NULL;

  if (k_max < k_min)
  {
    double k = k_max; k_max = k_min; k_min = k;
  }
  if (k_max == k_min)
  {
    EXF_ERREUR(BORNES_FAUSSES);
    return NULL;
  }

  if ( (pas <= 0) || ( pas > (k_max - k_min) ) )
  {
    EXF_ERREUR(PAS_FAUX);
    return NULL;
  }

  source = infos->graphe;

  /* Calcul du nombre de points dsir = nombre de points entre 0 et k_max (on coupera de 0  k_min...) */

  pas_r = (source->x_max - source->x_min) / (source->nombre_point - 1);

  nbr_pts = 1 + floor( M_PI / (pas * pas_r) ); /* Nombre de points qu'imposent les pas choisis */

  /* On arrondit  la puissance de 2 suprieure, si ncessaire */

  while ( nbr_pts >> bit_1 )
    bit_1++;

  if ( nbr_pts != (1 << bit_1) )
    nbr_pts = 1 << (bit_1 + 1);

  pas_r = M_PI / ( (double) (nbr_pts - 1) * pas ); /* Pas ncessaire en R */

  nv_x = (double *) calloc( nbr_pts, sizeof(double) );
  if (nv_x == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return NULL;
  }

  nv_y = (Complexe *) calloc( nbr_pts, sizeof(Complexe) );
  if (nv_y == NULL)
  {
    free(nv_x);
    ERREUR(MANQUE_MEMOIRE);
    return NULL;
  }

  free(infos->fenetre->x); /* On libre les anciens points */
  free(infos->fenetre->y);
  infos->fenetre->x = nv_x;
  infos->fenetre->y = (double *) nv_y; /* On installe les nouveaux points */
  infos->fenetre->nombre_point = nbr_pts;

  /* On calcule la nouvelle courbe dont on veut la TF */

  if ( source->drapeaux.complexe == GR_NORMAL ) /* Graphe normal */
  {
    double retour, x_fen = 0;
    double *a, *b, *c; /* Tableau pour le lissage par splines */

    retour = SPL_tt_prep(source, &a, &b, &c);
    if (retour == -1) /* Erreur dans le calcul... */
      return NULL;

    x = nv_x;
    y = nv_y;
    retour = pas_r * sqrt( 2 / M_PI );
    for (i = 0 ; i < nbr_pts ; i++, x++, y++)
    {
      *x = x_fen; /* Valeur pour laquelle on calcule */
      if ( (x_fen < source->x_min) || (x_fen > source->x_max) )
	y->re = 0;
      else			/* On multiplie par la fentre au passage */
      {
	double y_fen = TF_calcfen(x_fen, infos->type_fenetre, infos->r_min, infos->r_max,
				  infos->r_1, infos->r_2, infos->type_fenetre, infos->symetrique);
	y->re = y_fen * SPL_tt_calc(source, x_fen, a, b, c);

	y->re *= retour; /* Pour la normalisation de la TFI */
      }
      y->im = 0;
	
      x_fen += pas_r;
    }

    free(a); free(b); free(c); /* Libration des tableaux de la spline, inutiles... */
  }
  else
  {
    double retour, x_fen = 0;
    Complexe *a, *b, *c; /* Pour la spline cubique */

    retour = CSPL_tt_prep(source, &a, &b, &c);
    if (retour < 0) /* Erreur dans le calcul... */
      return NULL;

    retour = pas_r * sqrt( 2 / M_PI );
    for (i = 0, x = nv_x, y = nv_y ; i < nbr_pts ; i++, x++, y++)
    {
      *x = x_fen; /* Valeur pour laquelle on calcule */
      if ( (x_fen < source->x_min) || (x_fen > source->x_max) )
      {
	y->re = 0;
	y->im = 0;
      }
      else			/* On multiplie par la fentre au passage */
      {
	double y_fen = TF_calcfen(x_fen, infos->type_fenetre, infos->r_min, infos->r_max,
				  infos->r_1, infos->r_2, infos->type_fenetre, infos->symetrique);
	*y = CSPL_tt_calc(source, x_fen, a, b, c);
	y->re *= y_fen * retour;
	y->im *= y_fen * retour;
      }
      x_fen += pas_r;
    }

    free(a); free(b); free(c); /* Libration des tableaux de la spline, inutiles... */
  }
  k_max = *(nv_x + nbr_pts - 1);
  
  attend_evenement(MU_TIMER,          /* vnements attendus */
		   0, 0, 0, 0, 0,     /* rectangle 2 : aucun */
		   0,
		   &_rien_, &_rien_, &_rien_, &_rien_, (unsigned int *) (&_rien_), &_rien_);

  /* On prpare maintenant le graphe contenant la TF ... */

#ifdef _FRANCAIS_
  resultat = GRF_creer(nbr_pts, GR_TF, GR_IM_MOD,
		       "TF inverse (calcul en cours)", "Nombre d'onde", "Signal EXAFS", UNIT_ANG, UNIT_ARB);
#endif
#ifdef _ENGLISH_
  resultat = GRF_creer(nbr_pts, GR_TF, GR_IM_MOD,
		       "Inverse FT, beeing computed", "Wave number", "Amplitude", UNIT_ANG, UNIT_ARB);
#endif

  if (resultat == NULL) return NULL;

  resultat->drapeaux.protege = VRAI; /* Graphe en cours de calcul : on le protge... */
  resultat->x_min = 0;
  resultat->x_max = k_max;
  resultat->y_min = -1; /*  l'usage, suffit le plus souvent */
  resultat->y_max =  1;

  x = resultat->x;
  y = (Complexe *) resultat->y;
  for (i = 0, k = 0 ; i < resultat->nombre_point ; i++, x++, y++)
  {
    *x = k;
    y->re = 0; y->im = 0; /* On met  zro la TF - juste pour le dessin... */

    k += pas;
  }

  return resultat;
}

/* ~~~~~~~~ Prparation dans le cas d'un calcul par algorithme rapide simple ~~~~~~~~~~

   Mthode : calcule le nombre de points ncessaire pour conciler au mieux k_min, k_max et le pas en k
             calcule les X pour chacun de ces points et prpare le graphe multipli par la fentre      */

Graphe *TFI_prep_normal(TFI_infos *infos, double k_min, double k_max, double pas)
{
  Graphe *resultat;
  double k;
  register double *x;
  register Complexe *y;
  register unsigned long i;
  unsigned long nbr_pts;

  /* On prpare le spectre dont on veut la TF inverse brutale : multiplication par la fentre */

  x = infos->fenetre->x;
  y = (Complexe *) infos->fenetre->y;
  if ( infos->graphe->drapeaux.complexe == GR_NORMAL ) /* Spectre rel */
  {
    register double *vx_y;

    vx_y = infos->graphe->y;
    for (i = 0 ; i < infos->fenetre->nombre_point ; i++, y++, vx_y++)
    {
      y->re *= *vx_y;
      y->im = 0;
    }
  }
  else /* Graphe complexe - cas le plus frquent en fait */
  {
    register Complexe *vx_y;

    vx_y = (Complexe *) infos->graphe->y;
    for (i = 0 ; i < infos->fenetre->nombre_point ; i++, y++, vx_y++)
    {
      y->im = y->re * vx_y->im;
      y->re *= vx_y->re;
    }
  }

  /* On cre le graphe de la TF inverse calcule et on l'initialise pour les X */

  nbr_pts = floor((k_max - k_min) / pas) + 1;

#ifdef _FRANCAIS_
  resultat = GRF_creer(nbr_pts, GR_TF, GR_IM_MOD,
		       "TF inverse (calcul en cours)", "Nombre d'onde", "Signal EXAFS", UNIT_ANG, UNIT_ARB);
#endif
#ifdef _ENGLISH_
  resultat = GRF_creer(nbr_pts, GR_TF, GR_IM_MOD,
		       "Inverse FT, beeing computed", "Wave number", "Amplitude", UNIT_ANG, UNIT_ARB);
#endif

  if (resultat == NULL) return NULL;

  resultat->drapeaux.protege = VRAI; /* Graphe en cours de calcul : on le protge... */
  resultat->x_min = k_min;
  resultat->x_max = k_max;
  resultat->y_min = -1; /*  l'usage, suffit le plus souvent */
  resultat->y_max =  1;

  x = resultat->x;
  y = (Complexe *) resultat->y;
  for (i = 0, k = k_min ; i < resultat->nombre_point ; i++, x++, y++)
  {
    *x = k;
    y->re = 0; y->im = 0; /* On met  zro la TF - juste pour le dessin... */

    k += pas;
  }

  return resultat;
}
