/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
            Tout ce qui concerne les transformes de Fourier
            Fichier H pour les dfinitions et autre
   (c) Emmanuel CURIS, 1998 - Cration : 4 mai 1998

  4 mai 1998
   * Rcupr les dfinitions dans GRAPHES.H
*/

#ifndef _FOURIER_H_

#define _FOURIER_H_ 

/* ~~~~~~~~~ Transforme de Fourier ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* --------- Infos ncessaires au calcul... --------------------- */

typedef struct
{
  Graphe *graphe;        /* Graphe dont on veut la TF                */
  Graphe *fenetre;       /* La fentre, calcule                     */
  int type_fenetre;      /* Le type de la fentre                    */
  int num_depart;        /* Pour affichage du slecteur              */
  double param_fenetre;  /* Paramtre pour le calcul de la fentre   */
  double k_min, k_max;   /* Les extrmes pour la fentre             */
  double k_1, k_2;       /* Les internes, pour fentre asymtrique   */
  boolean symetrique;    /* VRAI si la fentre est symtrique        */
  int degre;             /* Multiplier le graphe par k^degr         */
  GRF_liste *amplitude;  /* Choix de la correction d'amplitude       */
  GRF_liste *phase;      /* Choix de la correction des phases        */
} TF_infos;

/* --------- Paramtrage de la faon de faire la TF ------------- */

void    TF_ouvre(Graphe *graphe); /* Le graphe dont on veut la TF */

/* --------- Paramtrage des corrections de la TF --------------- */

boolean GER_debTF(int num_objet,int x, int y);

/* --------- Initialisation de la TF ---------------------------- */

Graphe *TF_init       (TF_infos *infos_tf);
Graphe *TF_prep_normal(TF_infos *infos_tf, double r_min, double r_max, double pas); /* TF normale */
Graphe *TF_prep_rapide(TF_infos *infos_tf, double r_min, double r_max, double pas); /* TF rapide  */

/* --------- Corrections avant la TF ---------------------------- */

boolean TF_coramp(Graphe *graphe, Graphe *correct);                 /* d'amplitude */
boolean TF_corpha(Graphe *graphe, Graphe *correct, boolean reelle); /* de phase    */

/* --------- Calcul de la TF proprement dit -------------------- */

Complexe TF_complexe(Graphe *graphe, double x); /* D'un graphe complexe                      */
Complexe TF_reelle  (Graphe *graphe, double x); /* D'un graphe rel : un poil plus rapide... */

void TF_simple(Graphe *source, Graphe *fourier); /* Mthode bte et mchante */
void TF_rapide(Graphe *source, Graphe *fourier); /* Mthode de la TF rapide  */

/* ~~~~~~~~~ Transforme de Fourier inverse ~~~~~~~~~~~~~~~~~~~~ */

typedef struct
{
  Graphe *graphe,      /* Le graphe dont on veut une TF inverse   */
         *fenetre;     /* La fentre utilise pour le calcul      */
  double r_min, r_max; /* Bornes externes de calcul de la fentre */
  double r_1, r_2;     /* Bornes internes de calcul de la fentre */
  double parametre;    /* Paramtre pour la fentre               */
  boolean symetrique;  /* VRAI si fentre symtrique, FAUX sinon  */
  int    type_fenetre; /* Type de la fentre  utiliser           */
  int    num_depart,   /* Pour le choix de la fentre de la TF    */
         num_choisi;
} TFI_infos;

/* --------- Paramtrage de la faon de faire la TFI ----------- */

void TFI_ouvre(Graphe *graphe);

/* --------- Initialisations du calcul ------------------------- */

Graphe *TFI_init       (TFI_infos *infos);
Graphe *TFI_prep_rapide(TFI_infos *infos, double k_min, double k_max, double pas);
Graphe *TFI_prep_normal(TFI_infos *infos, double k_min, double k_max, double pas);

boolean GER_paramtfi(int num_objet, int x, int y); /* Gestion de l'arbre */

/* --------- Calcul effectif de la TF inverse ------------------ */

void TFI_simple(Graphe *source, Graphe *fourier); /* Mthode simple */
void TFI_rapide(Graphe *source, Graphe *fourier); /* Mthode rapide */

Complexe TFI_calc(Graphe *fourier, double x); /* Calcul normal en un point */

void TFI_fenetre(Graphe *fourier, TF_infos *infos); /* On dconvolue la fentre */

/* ~~~~~~~~~ Calcul des fentres pour la TF et la TFI ~~~~~~~~~~ */

/* Les fentres de TF utilisables - index du tableau Types_fenetres dans fourier.c */

#define PAS_DE_FENETRE 0 /* Basique...    */
#define FEN_RECTANGLE  1 /* mme remarque */
#define FEN_BESSEL     2 /* Kaiser-Bessel, symtrique, sans plateau */
#define FEN_GAUSSIENNE 3 /* Gaussienne, symtrique, sans plateau    */
#define FEN_HANNING    4 /* Cas particulier de Hamming, pour alpha = 0,5 */
#define FEN_HAMMING    5 /* Symtrique ou asymtrique */
#define FEN_NORTON_F3  6 /* Norton-Beer, F3, symtrique */
#define FEN_BARTLETT   7 /* Linaire, symtrique ou asymtrique */
#define FEN_BLACKMAN   8 /* ??? */

#define NOMBRE_FENETRES 8 /* Nombre total de fentres disponibles */

/* ~~~~~~~~~ Quelques macros pour simplifier les choses ~~~~~~~~~~~~ */

#define Autorise_Symetrique(i) ( (i == FEN_HANNING) || (i == FEN_HAMMING) \
                              || (i == FEN_BARTLETT) )

void   TF_fenetre(Graphe *fenetre, Graphe *source, int type, double min, double max,
                  double k_1, double k_2, double parametre, boolean symetrique); /* Complte */

double TF_calcfen(double x, int type_fenetre, double min, double max, double k_1, double k_2,
                  double parametre, boolean symetrique); /* En un point */

	/* Macros de calcul des fentres un peu complexes... */

int TFEN_bartlett(Graphe *fenetre, double k_1, double k_2, double k_3, double k_4);
int TFEN_hamming (Graphe *fenetre, double k_1, double k_2, double k_3, double k_4, double param);
int TFEN_hanning (Graphe *fenetre, double k_1, double k_2, double k_3, double k_4);

double bessel  (double x); /* J0 de Bessel         */
double bessel2 (double x); /* I0 de Bessel         */

#endif /* _FOURIER_H_ */
