/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
            Tout ce qui concerne les transformes de Fourier
   (c) Emmanuel CURIS 1996

	tat : - compil. Turbo C : nickel
	       - compil. GCC     : l. 788 - 792, zone_y non initialis
	                           l. 1699, nbr_points non initialis
	                           l. 1701, k_min, k_max, k, pas non initialiss

  16, 25 mars 1998
    * Utilisation de "systeme.h" et de "erreurs.h" (pour l'exafs)

  12, 16 janvier 1998
    * TFI : graphes en cours de calcul maintenant protgs contre l'effacement...
    * Intgr messages d'erreur GEM (erreurs.h)

  15 dcembre 1997
    * Pris en compte changement de type de graphe->pointeur (long -> void *)

  21, 28 novembre 1997
    * Cration de l'historique (il tait temps...)
    * Suppression de RCT_tf() : on utilise RCT_pointeur  la place
    * TF inverse : corrig le calcul des X -> rsultat plus correct...
                   adapt le nombre de points au rsultat intressant
                   amlior l'interface : corrig pb. de redessin du pointeur
                                          dessine graphe * fentre aussi si graphe rel
    * Remplac tous les effacer_graphe par GRF_efface()
    * Commenc remplacement de _PRG_station->handle par _PRG_station->handle
	  * la TF rapide pour un nombre quelconque de points fonctionne
	  * chang quelques affectations de noms en GRF_titre
	  * corrig le test de nbr_points = 2^k dans la TF inverse (appel de l'arbre)
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"
#include "ERREURS.H"
#include "FOURIER.H"
#include "SCIENCE.H"

#include <LISSAGE.H> /* Pour les splines */

extern const char *Types_fenetres[]; /* Dans fourier/fourier.c */

/* ----------------------- Calcul de la TF d'un spectre E.X.A.F.S. ---------------------- *
 ******************************************************************************************/

/* ~~~~~~~~~~~~~~~~~~~~ Calcul de la TF : mthode brutale ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */


/* ----------------- TF d'un graphe, mthode brutale --------------------------------------\
 |    Ne renvoie rien
 |    Suppose que les X ont dj t calculs et sont dans fourier-x...
 |
 | infos_tf : le graphe dont on veut la TF
 | fourier  : le graphe recevant la TF calcule
 | pas      : le pas du calcul
 \---------------------------------------------------------------------------------------- */

void TF_simple(Graphe *source, Graphe *fourier)
{
  OBJECT *adresse = arbres[DEBUT_TF].adresse;
  register double   *zone_x;
  register Complexe *zone_y;
  unsigned long index, temps; /* Pour le dcompte du temps de calcul */

  zone_x = fourier->x;
  zone_y = (Complexe *) fourier->y;

  temps = clock();
  if ( OB_STATE(adresse, TF_COR_PHASE) & SELECTED ) /* Correction de phase -> TF totale */
  {
    for (index = 0 ; index < fourier->nombre_point ; index++)
    {
      *(zone_y) = TF_complexe(source, *zone_x); /* On calcule en ce point */

      zone_x++; zone_y++; /* Point suivant */
    }
  }
  else
  {
    for (index = 0 ; index < fourier->nombre_point ; index++)
    {
      *(zone_y) = TF_reelle(source, *zone_x);   /* On calcule en ce point */

      zone_x++; zone_y++; /* Point suivant */

      if (index % 20) /* Tous les 20 points : on regarde ce qui se passe */
      {
	unsigned int touche;

	int retour = attend_evenement(MU_TIMER|MU_KEYBD, /* vnements attendus */
				      0, 0, 0, 0, 0,     /* rectangle 2 : aucun */
				      0,              /* Compteur  1, sinon plantage sur TOS 1.0 */
				      &_rien_, &_rien_, &_rien_, &_rien_, &touche, &_rien_);

	if (retour & MU_KEYBD) /* On a appuy sur une touche */
	{
	  /* Vrifier - ^C ==> on arrte le calcul */
	}
	if ( retour == _FIN_PRG ) /* On demande la fin du programme */
	{
	  fourier->nombre_point = index;
	  return;
	}
      }
    }
  }
  temps = clock() - temps;

  sprintf(_intermediaire,"[3][ --- TF termin"e_AIGU"e --- | Temps mis : %f s ][ Ok ]", (float) (temps / CLK_TCK) );
  form_alert(1,_intermediaire);
}

/* ----------------- TF d'une fonction complexe pure (mthode "brutale") ------------------\
 |   Renvoie la valeur calcule au point considr                                         |
 |
 | graphe : le graphe dont on veut calculer la TF - doit tre de type complexe...          |
 | x      : le point pour lequel on calcule la TF                                          |
 \---------------------------------------------------------------------------------------- */

Complexe TF_complexe(Graphe *graphe, double x)
{
	Complexe calcul = {0.0 , 0.0}; /* Le rsultat de l'intgration */
	register double *zone_x;
	register Complexe *zone_y;
	double dx_x_r,reelle,imag,delta_x;
	double inter;
	double cosinus, sinus;
	unsigned long index;

	x *= 2; /* Toujours par la suite... */

	zone_x = graphe->x;
	zone_y = (Complexe *) graphe->y; /* On se place au dbut du graphe */

	dx_x_r = x * *(zone_x);
	cosinus = cos( dx_x_r );
	sinus   = sin( dx_x_r );

	reelle =  cosinus * zone_y->re + sinus   * zone_y->im;
	imag   =  -sinus  * zone_y->re + cosinus * zone_y->im;

	for (index = 0 ; index < graphe->nombre_point - 1 ; index++)
	{
		delta_x = *(zone_x + 1) - *zone_x;
		zone_x++; zone_y++;

		dx_x_r = x * *zone_x;
		cosinus = cos( dx_x_r );
		sinus   = sin( dx_x_r );
			
		inter = cosinus * zone_y->re + sinus * zone_y->im;
		reelle    += inter;
		calcul.re += reelle * delta_x;
		reelle     = inter;

		inter = -sinus * zone_y->re + cosinus * zone_y->im;
		imag      += inter;
		calcul.im += imag * delta_x;
		imag       = inter;
	}

	calcul.re *= 0.3989422804; /* racine (2/pi) / 2 [ /2 == mthode des trapzes pour l'intgration... ] */
	calcul.im *= 0.3989422804;

	return calcul;
}

/* ----------------- TF d'une fonction relle pure (mthode "brutale") --------------------\
 |   Renvoie la valeur calcule au point considr                                         |
 |
 | graphe : le graphe dont on veut calculer la TF
 | x      : le point pour lequel on calcule la TF
 \---------------------------------------------------------------------------------------- */

Complexe TF_reelle(Graphe *graphe, double x)
{
  Complexe calcul = {0.0 , 0.0}; /* Le rsultat de l'intgration */
  Complexe *zn_y;
  const double coefficient = 1 / sqrt(2 * M_PI);
  double dx_x_r,reelle,imag,delta_x;
  double inter_reel, inter_imag, y;
  register double *zone_x, *zone_y;
  unsigned long index;
  boolean reel;

  x *= 2; /* Toujours par la suite... */

  zone_x = graphe->x;
  reel = (graphe->drapeaux.complexe == GR_NORMAL);
  if (reel)
  {
    zone_y = graphe->y;
    y = *zone_y;
  }
  else
  {
    zn_y = (Complexe *) graphe->y;
    y = zn_y->re;
  }

  dx_x_r = x * *(zone_x);

  reelle =  cos( dx_x_r ) * y;
  imag   =  -sin( dx_x_r ) * y;

  for (index = 0 ; index < graphe->nombre_point - 1 ; index++)
  {
    delta_x = *(zone_x + 1) - *zone_x;
    zone_x++;
    dx_x_r = x * *zone_x;

    if (reel)
    {
      zone_y++;
      y = *zone_y;
    }
    else
    {
      zn_y++;
      y = zn_y->re;
    }

    inter_reel = y * cos( dx_x_r );
    reelle    += inter_reel;
    calcul.re += reelle * delta_x;
    reelle     = inter_reel;

    inter_imag = - y * sin( dx_x_r );
    imag      += inter_imag;
    calcul.im += imag * delta_x;
    imag       = inter_imag;
  }

  calcul.re *= coefficient;
  calcul.im *= coefficient;

  return calcul;
}

/* ~~~~~~ Quelques calculs autour de la transformation de Fourier ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ----------------- Calcul de la fentre --------------------------------------------------------------------- */

/* ----------------- Calcul de la fentre d'un seul coup, dans un graphe dj dfini ---------------------------\
 |    Ne renvoie rien
 |
 | fenetre      : le graphe contenant la fentre
 | source       : le graphe prcisant les X  utiliser
 | type_fenetre : le type de fentre utilis (dfini dans...)
 | min, max, k_1, k_2 : bornes pour le calcul de la fentre
 | parametre    : parametre de calcul de la fentre
 | symetrique   : VRAI si symtrique, FAUX sinon [seulement dans le cas o a a un sens...]
 \------------------------------------------------------------------------------------------------------------- */

void TF_fenetre(Graphe *fenetre, Graphe *source, int type_fenetre, double min, double max, double k_1, double k_2,
                double parametre, boolean symetrique)
{
	double moyen;
	double inter, coefficient, commun;
	register double *zone_x,*zone_y;
	unsigned long index;
	char inc = 1;

	if (source == NULL) return;  /* Pas de graphe source... */
	if (fenetre == NULL) return; /* Pas de graphe fentre   */
	if (fenetre->drapeaux.complexe)
	{
		inc = 2;
		for (index = 0 ; index < fenetre->nombre_point ; index++)
		{
			(( (Complexe *) (fenetre->y) )[index]).im = 0.0; /* Partie imaginaire nulle */
		}
	}

	/* Toutes les fentres sont comprises entre 0 et 1 */
	/* pour le dessin : on triche en disant qu'elles sont comme la courbe */

	memcpy(fenetre->x, source->x, fenetre->nombre_point * sizeof(double)); /* On copie les X */

	fenetre->x_min = source->x_min; /* On met les bornes */
	fenetre->x_max = source->x_max;
	fenetre->y_min = 0;
	fenetre->y_max = 1;

	zone_x = fenetre->x;
	zone_y = fenetre->y;

	moyen = (min + max) / 2; /* Valeur moyenne, sur laquelle est centre la fentre */

	switch (type_fenetre)
	{
	case PAS_DE_FENETRE: /* Aucun fentre utilise */
		min = source->x_min - 1;
		max = source->x_max + 1; /* Sr d'tre au-del du graphe */
	case FEN_RECTANGLE: /* Fentre rectangulaire */
		for (index = 0 ; index < fenetre->nombre_point ; index++)
		{
			*(zone_x) = *(source->x + index);
			if ( (*zone_x >= min) && (*zone_x <= max) )
				*(zone_y) = 1;
			else
				*(zone_y) = 0;
			
			zone_x++; zone_y += inc;
		}
		break;
	
	case FEN_BESSEL: /* Kaiser-Bessel */
		inter = M_PI * parametre;
		coefficient = 1/bessel2(inter);
		for (index = 0 ; index < fenetre->nombre_point ; index++)
		{
			*(zone_x) = *(source->x + index);
			if ( ( *(zone_x) < 0 ) || ( *(zone_x) > (min + max) ) )
				*(zone_y) = coefficient * bessel2(0);
			else
			{
  				commun  = 1 - *(zone_x) / moyen;
				*(zone_y) = coefficient * bessel2( inter * sqrt(1 - commun * commun) );
			}
			
			zone_x++; zone_y += inc;
		}
		break;
	
	case FEN_GAUSSIENNE:  /* Gaussienne */
		coefficient = - M_PI_2 * parametre;
		for (index = 0 ; index < fenetre->nombre_point ; index++)
		{
			*(zone_x) = *(source->x + index);
			inter = 1 - *(zone_x) / moyen;
			*(zone_y) = exp( coefficient * inter * inter );

			zone_x++; zone_y += inc;
		}
		break;
	
	case FEN_HAMMING: /* Hamming */
		if (symetrique)
			k_2 = max - (k_1 - min);

		TFEN_hamming(fenetre, min, k_1, k_2, max, parametre);
		break;
	
	case FEN_HANNING: /* Normalement : hanning = hamming (0,5) */
		if (symetrique)
			k_2 = max - (k_1 - min);

		TFEN_hanning(fenetre, min, k_1, k_2, max);
		break;
	
	case FEN_NORTON_F3:   /* Norton-Beer F3 */
		for (index = 0 ; index < fenetre->nombre_point ; index++)
		{
			*(zone_x) = *(source->x + index);
			if ( (*zone_x < 0) || (*zone_x > min + max) )
				inter = 0;
			else
			{
				inter  = 1 - *(zone_x) / moyen;
				inter *= inter;
				inter  = 1 - inter;
				inter *= inter;    /* Donc inter = [1-(1-k/km)^2]^2 */
			}

			*(zone_y) = 0.045335 + 0.554883 * inter + 0.399782 * inter * inter;

			zone_x++; zone_y += inc;
		}
		break;
	
	case FEN_BARTLETT:
		if (symetrique == VRAI) /* Fentre symtrique */
			k_2 = max - (k_1 - min);

		TFEN_bartlett(fenetre, min, k_1, k_2, max);
		break;

	case FEN_BLACKMAN: /* Fentre de Blackman : pas encore faite */
		break;
	}
	
	return;
}

/* -------------- Sous fonctions du calcul des fentre en une fois ---------------------- */

int TFEN_bartlett(Graphe *fenetre, double k_1, double k_2, double k_3, double k_4)
{
	unsigned long i;
	double pente_1, pente_2;
	register double *x;

	pente_1 = (k_2 - k_1);
	pente_2 = (k_3 - k_4);
	if ( (pente_1 == 0) || (pente_2 == 0) )
	{
		EXF_ERREUR(DIV_ZERO);
		return -DIV_ZERO;
	}

	if (fenetre->drapeaux.complexe == GR_NORMAL)
	{
		register double *y = fenetre->y;

		for (i = 0, x = fenetre->x ; i < fenetre->nombre_point ; i++, x++, y++)
		{
			if ( (*x <= k_1) || (*x >= k_4) )
				*y = 0;
			else if ( (*x > k_1) && (*x < k_2) )
				*y = (*x - k_1) / pente_1;
			else if ( (*x >= k_2) && (*x <= k_3) )
				*y = 1;
			else
				*y = (*x - k_4) / pente_2;
		}
	}
	else
	{
		register Complexe *y = (Complexe *) fenetre->y;

		for (i = 0, x = fenetre->x ; i < fenetre->nombre_point ; i++, x++, y++)
		{
			y->im = 0; /* Toujours */
			if ( (*x <= k_1) || (*x >= k_4) )
				y->re = 0;
			else if ( (*x > k_1) && (*x < k_2) )
				y->re = (*x - k_1) / pente_1;
			else if ( (*x >= k_2) && (*x <= k_3) )
				y->re = 1;
			else
				y->re = (*x - k_4) / pente_2;
		}
	}

	return 0; /* Pas de problme */
}

int TFEN_hamming(Graphe *fenetre, double k_1, double k_2, double k_3, double k_4, double parametre)
{
	double coef_1, coef_2;
	unsigned long i;
	register double *x = fenetre->x;

	if ( (k_1 == k_2) || (k_3 == k_4) )
	{
		EXF_ERREUR(DIV_ZERO);
		return -DIV_ZERO;
	}

	coef_1 = M_PI / (k_2 - k_1);
	coef_2 = M_PI / (k_3 - k_4);

	if ( fenetre->drapeaux.complexe == GR_NORMAL )
	{
		register double *y = fenetre->y;

		for (i = 0 ; i < fenetre->nombre_point ; i++, x++, y++)
		{
			if ( (*x <= k_1) || (*x >= k_4) )
				*y = 2 * parametre - 1;
			else if ( (*x > k_1) && (*x < k_2) )
			{
				*y = parametre + (parametre - 1) * cos( (*x - k_1) * coef_1 );
			}
			else if ( (*x >= k_2) && (*x <= k_3) )
				*y = 1;
			else
			{
				*y = parametre + (parametre - 1) * cos( (*x - k_4) * coef_2 );
			}
		}
	}
	else
	{
		register Complexe *y = (Complexe *) fenetre->y;

		for (i = 0 ; i < fenetre->nombre_point ; i++, x++, y++)
		{
			y->im = 0;
			if ( (*x <= k_1) || (*x >= k_4) )
				y->re = 2 * parametre - 1;
			else if ( (*x > k_1) && (*x < k_2) )
			{
				y->re =  parametre + (parametre - 1) * cos( (*x - k_1) * coef_1 );
			}
			else if ( (*x >= k_2) && (*x <= k_3) )
				y->re = 1;
			else
			{
				y->re =  parametre + (parametre - 1) * cos( (*x - k_4) * coef_2 );
			}
		}
	}

	return 0; /* Pas de problme */
}

int TFEN_hanning(Graphe *fenetre, double k_1, double k_2, double k_3, double k_4)
{
	double coef_1, coef_2;
	unsigned long i;
	register double *x = fenetre->x;

	if ( (k_1 == k_2) || (k_3 == k_4) )
	{
		EXF_ERREUR(DIV_ZERO);
		return -DIV_ZERO;
	}

	coef_1 = M_PI_2 / (k_2 - k_1);
	coef_2 = M_PI_2 / (k_3 - k_4);

	if ( fenetre->drapeaux.complexe == GR_NORMAL )
	{
		register double *y = fenetre->y;

		for (i = 0 ; i < fenetre->nombre_point ; i++, x++, y++)
		{
			double inter;

			if ( (*x <= k_1) || (*x >= k_4) )
				*y = 0;
			else if ( (*x > k_1) && (*x < k_2) )
			{
				inter = sin( (*x - k_2) * coef_1 );
				*y = 1 - inter * inter;
			}
			else if ( (*x >= k_2) && (*x <= k_3) )
				*y = 1;
			else
			{
				inter = sin( (*x - k_3) * coef_2 );
				*y = 1 - inter * inter;
			}
		}
	}
	else
	{
		register Complexe *y = (Complexe *) fenetre->y;

		for (i = 0 ; i < fenetre->nombre_point ; i++, x++, y++)
		{
			double inter;
			
			y->im = 0;
			if ( (*x <= k_1) || (*x >= k_4) )
				y->re = 0;
			else if ( (*x > k_1) && (*x < k_2) )
			{
				inter = sin( (*x - k_2) * coef_1 );
				y->re = 1 - inter * inter;
			}
			else if ( (*x >= k_2) && (*x <= k_3) )
				y->re = 1;
			else
			{
				inter = sin( (*x - k_3) * coef_2 );
				y->re = 1 - inter * inter;
			}
		}
	}

	return 0; /* Pas de problme */
}

/* -------------- Calcul de la fentre en un point prcis --------------------------------\
 |   Renvoie la valeur de la fentre en ce point
 |
 | x            : point o calculer la fentre
 | type_fenetre : type de la fentre  utiliser
 | k_min, k_max : bornes dlimitant la fentre
 | k_1, k_2     : bornes "internes" dfinissant la monte et la descente de la fentre
 | parametre    : parametre servant  dfinir la fentre
 \--------------------------------------------------------------------------------------- */

double TF_calcfen(double x, int type_fenetre, double k_min, double k_max, double k_1, double k_2,
                  double parametre, boolean symetrique)
{
	double moyen, coefficient, inter;
	double pente_1, pente_2;

	if (symetrique)
		k_2 = k_max - (k_1 - k_min); /* On symtrise tout a... */

	moyen = (k_min + k_max) / 2; /* K_moyen, revient souvent... */

	switch (type_fenetre)
	{
	case FEN_RECTANGLE:         /* Rectangulaire : la plus simple... */
		if (x < k_min) return 0;
		if (x > k_max) return 0;
		return 1;

  case FEN_GAUSSIENNE:        /* Gaussienne    */
		coefficient = - M_PI_2 * parametre;
		x = 1 - 2 * x / (k_min + k_max);
		return exp( coefficient * x * x );

	case FEN_BESSEL:            /* Kaiser-Bessel */
		inter = M_PI * parametre;
		coefficient = 1/bessel2(inter);

		if (x < 0)             return bessel2(0) / bessel2( inter );
		if (x > k_min + k_max) return bessel2(0) / bessel2( inter );

		x = 1 - x / moyen;
		return bessel2( M_PI * parametre * sqrt(1 - x * x) ) / bessel2( inter );

	case FEN_HANNING: /* Hanning = Hamming (0,5) en ralit... */
		parametre = 0.5;
	case FEN_HAMMING: /* Hamming */

		if ( (k_min == k_1) || (k_max == k_2) )
		{
			EXF_ERREUR(DIV_ZERO);
			return 0;
		}

		pente_1 = M_PI / (k_1 - k_min);
		pente_2 = M_PI / (k_2 - k_max);

		if ( (x <= k_min) || (x >= k_max) )
			return	2 * parametre - 1;
		if ( (x > k_min) && (x < k_1) )
			return parametre + (parametre - 1) * cos( (x - k_min) * pente_1 );
		if ( (x >= k_1) && (x <= k_2) )
			return 1;
		return parametre + (parametre - 1) * cos( (x - k_max) * pente_2 );

	case FEN_NORTON_F3:   /* Norton-Beer F3 */
		if ( x < 0 )             return 0;
		if ( x > k_min + k_max ) return 0;

		inter  = 1 - x / moyen;
		inter *= inter;
		inter  = 1 - inter;
		inter *= inter;    /* Donc inter = [1-(1-k/km)^2]^2 */

		return 0.045335 + 0.554883 * inter + 0.399782 * inter * inter;

	case FEN_BARTLETT:
		pente_1 = (k_1 - k_min);
		pente_2 = (k_2 - k_max);

		if ( (pente_1 == 0) || (pente_2 == 0) )
		{
			EXF_ERREUR(DIV_ZERO);
			return 0;
		}

		if ( (x <= k_min) || (x >= k_max) )
			return 0;
		if ( (x > k_min) && (x < k_1) )
			return (x - k_min) / pente_1;
		if ( (x >= k_1) && (x <= k_2) )
			return 1;
		return (x - k_max) / pente_2;

	case FEN_BLACKMAN: /* Fentre de Blackman : pas encore faite */
		break;
	}

	return 1; /* On est incapable de calculer la fentre */
}

/* -------------- Calcul de la transforme de Fourier inverse --------------------------- *
 ******************************************************************************************/

/* ~~~~~~~~~~~~~~~~~ Calcul proprement dit de la Transforme de Fourier ~~~~~~~~~~~~~~~~~~ */


/* ----------------- TF inverse d'un graphe, mthode brutale ------------------------------\
 |    Ne renvoie rien
 |    Suppose que les X ont dj t calculs et sont dans fourier->x...
 |
 | source   : le graphe dont on veut la TF
 | fourier  : le graphe recevant la TF calcule
 \---------------------------------------------------------------------------------------- */

void TFI_simple(Graphe *source, Graphe *fourier)
{
  register double   *zone_x;
  register Complexe *zone_y;
  unsigned long index, temps; /* Pour le dcompte du temps de calcul */

  zone_x = fourier->x;
  zone_y = (Complexe *) fourier->y;

  temps = clock();

  for (index = 0 ; index < fourier->nombre_point ; index++)
  {
    *(zone_y) = TFI_calc(source, *zone_x); /* On calcule en ce point */

    zone_x++; zone_y++; /* Point suivant */

    if (index % 20) /* Tous les 20 points : on regarde ce qui se passe */
    {
      unsigned int touche;

      int retour = attend_evenement(MU_TIMER|MU_KEYBD, /* vnements attendus */
				    0, 0, 0, 0, 0,     /* rectangle 2 : aucun */
				    0,                 /* Compteur  1 -> pas de plantage sur TOS 1.0 */
				    &_rien_, &_rien_, &_rien_, &_rien_, &touche, &_rien_);

      if (retour & MU_KEYBD) /* Appuy sur une touche */
      {
	/* Autoriser l'interruption du calcul... */
      }
    }
  }
  temps = clock() - temps;

  sprintf(_intermediaire,"[3][ --- TFI termine --- | Temps mis : %f s ][ Ok ]", (float) (temps / CLK_TCK) );
  form_alert(1,_intermediaire);
}

/* ----------------- TFI d'une fonction complexe (mthode "brutale") ----------------------\
 |   Renvoie la valeur calcule au point considr                                         |
 |
 | graphe : le graphe dont on veut calculer la TF - doit tre de type complexe...          |
 | x      : le point pour lequel on calcule la TF                                          |
 \---------------------------------------------------------------------------------------- */

Complexe TFI_calc(Graphe *graphe, double x)
{
  Complexe calcul = {0.0 , 0.0}; /* Le rsultat de l'intgration */
  register Complexe *zone_y;
  double dx_x_r, reelle, imag, delta_x;
  double inter;
  double cosinus, sinus;
  register double *zone_x;
  unsigned long i;

  x *= 2; /* Toujours par la suite... */

  zone_x = graphe->x; zone_y = (Complexe *) graphe->y; /* On se place au dbut du graphe */

  dx_x_r = x * *(zone_x);
  cosinus = cos( dx_x_r );
  sinus   = sin( dx_x_r );

  reelle =  cosinus * zone_y->re -   sinus * zone_y->im;
  imag   =    sinus * zone_y->re + cosinus * zone_y->im;

  for (i = 0 ; i < graphe->nombre_point - 1 ; i++)
  {
    delta_x = *(zone_x + 1) - *zone_x;
    zone_x++;
    zone_y++;
    dx_x_r = x * *zone_x;
    cosinus = cos( dx_x_r );
    sinus   = sin( dx_x_r );
			
    inter = cosinus * zone_y->re -   sinus * zone_y->im;
    reelle    += inter;
    calcul.re += reelle * delta_x;
    reelle     = inter;

    inter =   sinus * zone_y->re + cosinus * zone_y->im;
    imag      += inter;
    calcul.im += imag * delta_x;
    imag       = inter;
  }

  calcul.re *= 0.3989422804; /* racine (2/pi) / 2 [ /2 == mthode des trapzes pour l'intgration... ] */
  calcul.im *= 0.3989422804;

  return calcul;
}

/* ----------------- TFI : dconvolution de la fentre de calcul de la TF -----------------\
 |   Ne renvoie rien                                                                       |
 |
 | fourier : le graphe contenant la TF inverse - de type complexe obligatoirement          |
 | infos   : les infos sur la fentre  dconvoluer [NULL -> on ne fait rien...]           |
 \---------------------------------------------------------------------------------------- */

void TFI_fenetre(Graphe *fourier, TF_infos *infos)
{
  register Complexe *y; /* Les Y de la TFI */
  register double   *x; /* Les X de la TFI */
  unsigned long index;
  boolean erreur = FAUX;

  if (infos   == NULL) return;
  if (fourier == NULL) return;

  /* On divise par la fentre ayant servie au calcul de la TF */

  x = fourier->x;
  y = (Complexe *) fourier->y;
  for (index = 0 ; index < fourier->nombre_point ; index++, y++, x++)
  {
    double y_fen = TF_calcfen(*x, infos->type_fenetre, infos->k_min, infos->k_max,
			      infos->k_1, infos->k_2, infos->param_fenetre, infos->symetrique);
    if ( y_fen == 0 )
    {
      if (erreur == FAUX)
      {
	EXF_ERREUR(DIV_ZERO);
	erreur = VRAI;
      }
    }
    else
    {
      y->re /= y_fen; /* On divise par la fentre */
      y->im /= y_fen;
    }
  }
}

/* -------------- Quelques petites routines mathmatiques utiles ------------------------ *
 ******************************************************************************************/

/* -------------- Fonction de Bessel J0 ------------------------------------------------- */

double bessel(double x)
{
  double x_abs,x_carre,y1,y2,inter;
  double retour;
	
  x_abs = fabs(x); /* Valeur absolue */

  if ( x_abs < 8) /* Approximation polynomiale */
  {
    x *= x;
    retour  = 57568490574.0 + x * ( -13362590354.0 + x * ( 651619640.7 
	                    + x * ( -11214424.18   + x * ( 77392.33017
		            - x * 184.9052456 ) ) ) );
    retour /= 57568490411.0 + x * (  1029532985.0  + x * ( 9494680.718
		            + x * (   59272.64853  + x * ( 267.8532712
		            + x ) ) ) );
  }
  else
  {
    x_carre  = 8 / x_abs;
    x_carre *= x_carre;
    inter = x_abs - 0.785398164;
    y1 = 1 + x_carre * ( -0.1098628627e-2 + x_carre * ( 0.2734510407e-4
		       + x_carre * ( -0.2073370639e-5 + x_carre * ( 0.2093887211e-6 ) ) ) );
    y2 = -0.1562499995e-1 + x_carre * ( 0.1430488765e-3 + x_carre * ( -0.6911147651e-5
			  + x_carre * ( 0.7621095161e-6 - x_carre *    0.934935152e-7 ) ) );
    retour  = sqrt( 0.636619772 / x_abs );
    retour *= cos(inter) * y1 - 8 / x_abs * sin(inter) * y2;
  }
	
  return retour;
}

/* -------------- Fonction de Bessel modifie I0 ---------------------------------------- */

double bessel2(double x)
{
  double ans;
  double y;
	
  x = fabs(x);
  if ( x < 3.75 )
  {
    y  = x / 3.75;
    y *= y;

    ans = 1 + y * (3.5156229 + y * (3.0899424 + y * ( 1.2067492 + y * ( 0.2659732
					+ y * (0.360768e-1 + y * 0.45813e-2) ) ) ) );
  }
  else
  {
    y = 3.75 / x;
    ans  = 0.39894228 + y * (0.1328592e-1 + y * ( 0.225319e-2  + y * (-0.157565e-2
					+ y * (-0.157565e-2 + y * ( 0.916281e-2  + y * (-0.2057706e-1
					+ y * (0.2635537e-1 + y * (-0.1647633e-1 + y * 0.392377e-2))))))));
    ans *= exp(x) / sqrt(x);
  }
	
  return ans;
}
