/* LASE : logiciel d'analyse des spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS
   Ce fichier : 29 septembre 1998

   Module de  Calcul formel 

   Dclarations des fonctions
*/

#ifndef _CALCUL_FORMEL_H_

#define _CALCUL_FORMEL_H_ /* Protection contre l'inclusion multiple */

/* -------- Qu'est-ce qu'une expression ? ------------------------------- */

struct variable;

typedef struct expr
{
  int type;
  int code;

  union
  {
    double valeur;
    struct expr *expression;
    struct variable *variable;
  } arg_1;
  union
  {
    double valeur;
    struct expr *expression;
    struct variable *variable;
  } arg_2;

} Expression;

/* Types d'expression rencontres */

#define EXP_CTE_CTE    0 /* Les deux arguments sont des nombres    */
#define EXP_CTE_ARG    1 /* Le premier argument est une expression, le second un nombre (toujours) */
#define EXP_ARG_ARG    2 /* Les deux arguments sont des relations  */
#define EXP_CTE_VAR    3 /* Premier argument est une variable, le second un nombre */
#define EXP_VAR_VAR    4 /* Les deux arguments sont des variables */
#define EXP_ARG_VAR    5 /* Le premier argument est une expression, le second une variable */

/* Oprations  deux arguments, symtriques */

#define EXP_ADDITION     1
#define EXP_SOUSTRACTION 2
#define EXP_PRODUIT      3
#define EXP_DIVISION     4

/* Oprations  un seul argument */

#define EXP_SINUS       10
#define EXP_COSINUS     11
#define EXP_TANGENTE    12
#define EXP_COTANGENTE  13
#define EXP_SECANTE     14
#define EXP_COSECANTE   15

#define EXP_COSH        20
#define EXP_SINH        21
#define EXP_TANH        22
#define EXP_COTANH      23
#define EXP_SECH        24
#define EXP_COSECH      25

#define EXP_EXP         30
#define EXP_EXP10       31
#define EXP_LOG         32
#define EXP_LOG10       33

#define EXP_RACINE      40
#define EXP_CARRE       41
#define EXP_CUBE        42
#define EXP_INVERSE     43

#define EXP_ASIN        50
#define EXP_ACOS        51
#define EXP_ATAN        52

#define EXP_ASINH       60
#define EXP_ACOSH       61
#define EXP_ATANH       62

/* -------- Qu'est-ce qu'une grandeur que l'on peut faire varier ? ------ */

typedef struct variable
{
  double valeur_initiale;
  double min, max;        /* Intervalle de variation */
  double delta;           /* Pas minimal de variation (0 = au choix) */
  int type;               /* Voir les types possibles ci-dessous     */
  union
  {
    double valeur;             /* C'est une valeur numrique */
    struct variable *original; /* Fantme                    */
    Expression expression;     /* Dfinie par une relation   */
  } contenu;
} Variable;

#define VAR_CONSTANTE  1 /* Fausse variable... = valeur initiale, toujours */
#define VAR_LIBRE      2 /* Variable simple    */
#define VAR_COPIE      3 /* Variable fantme, toujours gale  une autre */
#define VAR_EXPRESSION 4 /* Valeur = expression(autres variables), calcule APRES ajustement */
#define VAR_RELATION   5 /* Idem, mais intervient pour chercher le minimum (+ raliste mais + complexe... */

double VAR_prend_valeur(Variable *variable);

#endif
