/* Traitement de l'E.X.A.F.S. sur ATARI
   Dfinitions gnrales au programme d'EXAFS sous GEM
   (c) 1996 - Emmanuel CURIS

    4 mai 1998
     * Report les dclarations pour la TF dans fourier.h

    3 mai 1998
     * Plus d'infos dans TF_infos : plus de fentres possibles...

   10, 16 & 17, 24, 26 & 27 mars 1998
     * Report quelques dfinitions de base dans gem_sys.h
     * Suppression dfinitive de effacer_graphe()
     * Remplac int handle par Station_VDI *station
     * Supprim inclusion de gem_var.h : report dans "systeme.h"
     * Report dclaration de la DTA dans gem_tos.h
     * Dplac tout ce qui concerne l'extraction (XANES / EXAFS) dans "extrait.h"
     * Dplac les messages d'erreur et associs dans erreurs.h
     * Ajout, dans la structure de graphe :
           - drapeaux "doit-on tracer les barres d'erreur ?"
           - pointeur sur le graphe des erreurs en X
           - pointeur sur le graphe des erreurs en Y
           - pointeur sur un autre graphe, au cas o
     * Ajout quelques types spciaux de graphes
     * Ajout le prototype de tri_en_place()

    5 & 6 fvrier 1998
     * Dplac les procdures de gestion des fichiers vers fichier.h
     * Dclar Histoire : le fichier contenant la description de la sance de travail

   13 janvier 1998
     * Report les routines d'impression dans "imprime.h"

   1 & 2, 15 dcembre 1997
     * Modifi type de graphe->pointeur : devenu void *, plus propre
     * Remplac adapte_menus() par MEN_adapte()
     * Version 1.1 du programme (le format de fichier a chang...)
     * Ajout dfinitions pour diffrentes mthodes d'intgration
     * Dfinitions pour les nouvelles fonctions de calcul des drives
     * Ajout message d'erreur GRAPHE_COMPLEXE : le graphe choisi est complexe !

   22, 24-28 novembre 1997
     * Cration de cet historique
     * Dfinit TOURNE et NON_TOURNE indiquant le sens de trac du graphe (imprimante/cran)
     * Dfinition de TRC_gradue(), qui s'occupe du trac des graduations des axes
                  de TRC_grille() (trac d'une grille en fond)
                  des fonctions pour l'change d'un menu  l'autre
                  des fonctions is_inf() et is_nan()
		  des fonctions de troncature et de la structure associe (TRQ_infos)
     * Supprim GR_MOINSMAT  GR_LIBREP : redfinis dans le resource
       dfinition de RCT_tronque() [inexistante] et RCT_aberrance() [efface]
     * Inclusion de gem_var.h, ncessaire partout
     * Protection contre l'inclusion multiple
     * Nouvelle rpartition des fonctions de traitement des graphes
            graphes.c   --> tout ce qui est cration, effacement,...
	    grf_math.c  --> traitement mathmatique (drivation, intgration,...)
	    grf_stat.c  --> traitement statistique  (tronquer, dcomposer,...)
*/

#ifndef _GRAPHES_H_

#define _GRAPHES_H_ 

#define VERSION 12 /* Numro de version : 10 = 1.0 */

/*********** Les graphes dans leur ensemble (dfinitions) *************
 **********************************************************************/

/* --------- Structures de donnes pour les graphes ----------------- */

typedef struct
{
	double re;
	double im;
} Complexe; /* Nombre complexe */

/* --------- Le graphe lui-mme ------------------------------------- */

typedef struct graphe
{
  struct graphe *precedent;       /* Le graphe prcdent en mmoire  */
  struct graphe *suivant;         /* Le graphe suivant en mmoire    */
  int            type;            /* Type du graphe                  */
  unsigned long  nombre_point;    /* Nombre de points dans le graphe */
  double        *x;               /* Position des x en mmoire       */
  double        *y;               /* Position des y en mmoire       */
  boolean        trace;           /* VRAI s'il faut le dessiner      */
  double         x_min,x_max;     /* Bornes de l'intervalle en X     */
  double         y_min,y_max;     /* Bornes de l'intervalle en Y     */
  char           nom[60];         /* Nom du graphe (57 car. max)     */
  int            style;           /* Style de dessin (1  6 : pointills - 7  ... : polymark */
  int 		 couleur;         /* Couleur  utiliser              */
  char           axe_x[50];       /* Lgende de l'axe des X          */
  char           axe_y[50];       /* Lgende de l'axe des Y          */
  int            unit_x, unit_y;  /* Units utilises sur les axes   */
  double         decal_x,decal_y; /* Dcalage en X/Y pour le trac multiple */
  struct
  {
    unsigned sauve        : 1; /* VRAI -> le spectre a t sauv   */
    unsigned exafs        : 1; /* VRAI -> spectre de type EXAFS    */
    unsigned complexe     : 2; /* 0 -> rien de spcial
				  1 -> en Y, des complexes - tracer Im + module
				  2 -> en Y, des complexes - tracer Re + module
				  3 -> en Y, des complexes - tracer Re + Im */
    unsigned parametrique : 1; /* VRAI -> en Y, des couples (X, Y) */
    unsigned trois_d      : 1; /* VRAI -> graphe en 3 dimensions   */
    unsigned protege      : 1; /* VRAI -> graphe protg, ineffaable */
    unsigned trace_erreur : 1; /* VRAI -> tracer les barres d'erreur, si erreur_x/_y dfinis */
  } drapeaux;
  void *pointeur;       /* Pointeur sur des infos supplmentaires (ex : param de calcul des TF) */
  struct graphe *erreur_x; /* Graphe contenant l'erreur sur les X */
  struct graphe *erreur_y; /* Graphe contenant l'erreur sur les Y */
  struct graphe *grf_lie;  /* Autre graphe li, au cas o         */
} Graphe;

#define GR_NORMAL  0  /* Les diffrents types de reprsentation des graphes complexes */
#define GR_IM_MOD  1
#define GR_RE_MOD  2
#define GR_RE_IM   3
#define GR_PARAM   4  /* Graphe paramtrique */
#define GR_3D      5  /* Graphe en 3D        */

#define LONG_LEGENDE 49 /* Nombre de caractres au maximum dans la lgende */
#define SS_UNITE   0    /* Sans unit                  */
#define UNIT_EV    1    /* Unit : l'lectron-volt, eV */
#define UNIT_ANG   2    /* Unit : angstrom,          */
#define UNIT_1_ANG 3    /* Unit : 1/ (nombre d'onde) */
#define UNIT_ARB   4    /* Unit arbitraire            */
#define UNIT_NORM  5    /* Unit arbitraire, normalis */

/* --------- Les types de graphe spciaux reconnus ------------------------------- *
 *  types pour des slections plus souples dans le slecteur de graphe ( < -100)   *
 *  types spciaux  usage interne
 *  types courants > 0
 * ------------------------------------------------------------------------------- */

#define GR_TOUS     -100 /* On choisit tous les graphes */
#define GR_TT_EXAFS -101 /* On choisit tous les graphes pour lesquels exafs == VRAI */

#define GR_INTERNE   -5 /* Graphe  usage interne -> utilisateur peut rien faire... */
#define GR_AUCUN     -3 /* Aucun graphe ne convient  ce que l'on veut faire        */
#define GR_AUTRETYPE -2 /* Graphe d'un type donn, diffrent des types courants     */
#define GR_SSTYPE    -1 /* Graphe sans type (type inconnu en fait)                  */

#define GR_BRUT       0 /* Graphe brut, soit A = f(E) exprimental                  */
#define GR_MOINSMAT   1
#define GR_DERIVEE    2
#define GR_DERIVEE2   3
#define GR_EXAFS1     4
#define GR_EXAFS2     5
#define GR_EXAFS      6
#define GR_TF         7
#define GR_TFI        8
#define GR_EXAFSAMP   9
#define GR_EXAFSPHA  10
#define GR_AJUSTE    11
#define GR_MU0       12
#define GR_ECART     13
#define GR_RDF       14
#define GR_COMPREEL  15
#define GR_COMPIMA   16
#define GR_COMPMOD   17
#define GR_COMPARG   18
#define GR_I0        19
#define GR_I1        20
#define GR_LIBREP    21
#define GR_FENETRE   22
#define GR_DEVIATION 23
#define GR_FAC_RED   24

#define LG_TITRE    56 /* Longueur maximale des titres                    */

/* Note : pour GR_TF, y pointe en fait sur un champ de nombres complexes  */

/* --------------- Pour la mise en page avant impression -> divers rectangles */

typedef struct rect
{
  struct rect *avant; /* Rectangle prcdent */
  struct rect *apres; /* Rectangle suivant   */
  int           x, y,  /* Position et dimensions du rectangle */
          larg, haut;
  unsigned int  type; /* Type de contenu du rectangle */
  long      pointeur; /* Pointeur sur une structure contenant plus d'infos */
} Cadre;

typedef struct
{
  char legende[50]; /* Texte de la lgende */
  char unite[7];    /* Unit utilise sur l'axe */

  struct
  {
    unsigned gradue   : 2; /* 0 -> aucune, 1 -> dcimale, 2 -> logarithmique */
    unsigned dixieme  : 1; /* VRAI -> on marque les diximes  */
    unsigned centieme : 1; /* VRAI -> on marque les centimes */
    unsigned science  : 1; /* VRAI -> notation scientifique   */
    unsigned grille   : 1; /* VRAI -> on trace une grille     */
    unsigned visible  : 1; /* VRAI -> l'axe vrai est dessin s'il est visible */
    unsigned reserve  : 1; /* Le complment  8 */
  } graduation;

  struct
  {
    unsigned texte  : 4; /* Couleur des textes   */
    unsigned grille : 4; /* Couleur de la grille */
  } couleurs;

  char styl_grille; /* Style de ligne pour le trac de la grille */
  int  taille_txt;  /* Taille du texte, en centimes de points   */
  int  fnt_texte;   /* Fonte utilise pour le texte (index GDOS) */
} Axe; /* Toutes les infos sur le trac d'un axe */

#define PAS_GRADUE     0 /* Types de graduation possible */
#define GRADUE_DECIMAL 1
#define GRADUE_LOG     2

typedef struct dess_graphe
{
  struct dess_graphe *suivant; /* Pointeur sur le graphe suivant    */
  Graphe *graphe;              /* Pointeur sur le graphe  dessiner */
  int couleur;                 /* Couleur de dessin                 */
  int style;                   /* Style de ligne  utiliser         */
} DES_graphe;

typedef struct
{
  int    x1,y1,   /* Coin suprieur gauche, par rapport au rectangle principal    */
	 x2,y2;   /* Coin infrieur droit (mme chose) ==> zone de trac effectif */

  double x_min, x_max, /* Les bornes de trac dans ce rectangle */
         y_min, y_max;

  Axe  axe_x; /* Infos pour l'axe des X */
  Axe  axe_y; /* .................... Y */

  unsigned long nbr_graphes; /* Nombre de graphes  afficher (0->graphe marqus visibles) */
  DES_graphe *prem_graphe;   /* Premier graphe  dessiner                                 */
} Rect_graphe; /* Rectangle contenant des graphes */

/* ----------- Les diffrents types de rectangles possibles ---------- */

#define REC_GRAPHE  1 
#define REC_TEXTE   2
#define REC_LEGENDE 3
#define REC_SURFACE 4
#define REC_LIGNE   5
#define REC_IMAGE   6

/* ----------- Position du clic dans un rectangle -------------------- */

typedef struct
{
  unsigned centre     :1; /* VRAI si on est dans la position dsigne */
  unsigned bord_haut  :1;
  unsigned bord_bas   :1;
  unsigned bord_gauche:1;
  unsigned bord_droit :1;
  unsigned legende_x  :1;
  unsigned legende_y  :1;
  unsigned reserve    :1;  
} Position;

/* ~~~~~~~~~~~ Les variables globales ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

#ifndef _VARIABLES_ /* Elles sont toutes dclares dans VARIABLE.C : voir ce fichier pour leur rle */

extern FILE *Histoire;

extern Fenetre *principale; /* Fentre principale... */

extern const char *extensions[];
extern const char *Type_graphe[];
extern char titre[];

extern boolean TT_xanes, TT_exafs;

extern Graphe   *premier_graphe;  /* Premier graphe en memoire      */
extern Graphe   *dernier_graphe;
extern Graphe   *graphe_courant;  /* Graphe sur lequel on travaille */
extern unsigned long nbr_graphes; /* Nombre de graphes en mmoire   */
extern unsigned long num_graphe;  /* Numro du graphe courant       */

extern Graphe   *liste[];         /* Pour l'ascenseur de l'arbre correspondant */

extern Fenetre *trace_fenetre;
extern int trace_handle;  /* Fentre de superposition des graphes   */
extern Cadre *prem_rectangle;

extern Rect_graphe RCT_courant; /* Paramtres de dessin pour fen. principale */
extern DES_graphe DES_courant;

extern int points[], compteur;    /* Trac... */

extern Graphe *mu0_soustrait; /* mu0 approch en cours d'laboration */
extern Graphe *intermediaire; /* Sert de temps  autre... */

extern int _rien_; /* Pour les fonctions renvoyant des valeurs inutiles... */

#endif /* _VARIABLES_ */

/********** Routines spcifiques des graphes complexes *****************
 *     (Module COMPLEXE.C)
 ***********************************************************************/

/* ------------ Dcomposition d'un graphe en ses lments rels ------ */

void CMP_decompose (Graphe *complexe);

void CMP_reelle    (Graphe *complexe, Graphe *reelle);
void CMP_imaginaire(Graphe *complexe, Graphe *imag  );
void CMP_module    (Graphe *complexe, Graphe *module);
void CMP_argument  (Graphe *complexe, Graphe *phase );

/* ------------ Calculs usuels, mais sur un graphe complexe ---------- */

Graphe *CMP_derive (Graphe *complexe, int methode);

/********** Routines GEM gnrales mais propres au programme ***********
 *     (Module EXF_GEM.C)                                              *
 ***********************************************************************/

void prep_arbres   (void); /* Comme son nom l'indique */

/* ~~~~~~~~ Raccourcis claviers ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* -------- Menu ----------------------------------------------------- */

boolean CLA_menu       (unsigned int touche, int etat, int *renvoi);
boolean CLV_men_theorie(unsigned int touche, int etat, int *renvoi);

/* ~~~~~~~~ Gestion des arbres ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* -------- Gnralit pour les arbres ("macros") -------------------- */

/* -------- Gestion d'un slecteur de graphes, version simple ---------\
 |   savoir : slecteur pour choisir un seul graphe...                |
 \-------------------------------------------------------------------- */

typedef struct
{
  unsigned long nbr_graphe;  /* Nombre de graphes dans cette liste          */
  unsigned long prem_graphe; /* Le premier graphe affich dans le slecteur */
  unsigned long num_select;  /* Numro du graphe slectionn                */
  Graphe **graphes;          /* Les graphes  afficher dans le slecteur    */
} GRF_liste;

GRF_liste *GRF_par_type(int type, boolean alerte); /* Prparation de la liste : graphes d'un mme type */

/* Gestion proprement dite du slecteur */

boolean GRF_affiche(GRF_liste *liste, int arbre,
                    int prem_nom, int der_nom,
                    int avant, int apres, int fond, int bouton);

boolean GRF_selecteur(GRF_liste *liste, int arbre,
                      int prem_nom, int der_nom, 
                      int avant, int apres, int fond, int bouton,
                      int num_objet, int x, int y);

/* ~~~~~~~~ Objets Userdef (routines de dessin) ~~~~~~~~~~~~~~~~~~~~~~ */

int cdecl USR_style(USR_FONCTION *parametres); /* Affichage du style de ligne  */
int cdecl USR_fonte(USR_FONCTION *parametres); /* Affichage du nom d'une fonte */

/* -------- Menus ---------------------------------------------------- */

boolean GER_infos(int num_objet, int x, int y);
boolean MEN_ajuste (int option); /* Menu des options d'ajustement               */
boolean MEN_extrait(int option); /* Menu des options de traitement exprimental */
boolean MEN_theorie(int option); /* Menu des options de calcul thorique        */

/* -------- Type de graphe ------------------------------------------- */

boolean GER_graphes (int num_objet, int x_souris, int y_souris);

/* -------- Paramtrages de trac ------------------------------------ */

boolean GER_legende (int num_objet, int x_souris, int y_souris);
boolean GER_bornes  (int num_objet, int x_souris, int y_souris);

void redessine_travail(Fenetre *fen, int x, int y, int larg, int haut);
void    ASC_principale(Fenetre *fen, int type, int ss_type, int position);
boolean CLC_principale(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);
boolean CLV_principale(Fenetre *fen, unsigned int touche, int etat);
boolean MSG_principale(Fenetre *fen);
boolean RCT_principale(Fenetre *fen, int x, int y);

/* -------- Fentres volues spcifiques au programme --------------- */

boolean RCT_pointeur(Fenetre *fen, int x, int y);

/* ---------- Gestion des graphes ------------------------------- *
 *    Tout ce qui est cration, effacement, paramtrage           *
 *     (Module GRAPHES.C)                                         *
 ******************************************************************/

void MEN_adapte  (Graphe *graphe); /* Adapte le menu en fonction du type du graphe courant */

Graphe *copier_graphe  (Graphe *graphe);
Graphe *creer_graphe   (unsigned long nbr_points, int type);

boolean GRF_tteffacer(void);           /* Effacer tous les graphes en mmoire [quand on quitte, par exemple...] */
boolean GRF_efface   (Graphe *graphe, boolean force_sauve, boolean force_protege);
Graphe *GRF_creer    (unsigned long nbr_pts, int type, int mode,
		      char *nom, char *leg_x, char *leg_y, int unit_x, int unit_y);
Graphe *GRF_choisir  (int type);       /* Slecteur de graphes d'un type donn */

#define NE_PAS_FAIRE 0
#define ORDONNE      1 /* Valeurs pour en_x et en_y */
#define DESORDONNE   2

void    GRF_bornes   (Graphe *graphe, int en_x, int en_y); /* Bornes du graphe */

void    GRF_aller    (Graphe *graphe); /* Mettre graphe en graphe courant                */

void    GRF_legende(Graphe *graphe, int axe, char *texte, int unite); /* Met une lgende */
void    GRF_titre  (Graphe *graphe, char *titre);                     /* Met un titre    */

#define AXE_X        1  /* Lgende pour l'axe des X */
#define AXE_Y        2  /* Lgende pour l'axe des Y */

boolean GER_choix(int num_objet, int x_souris, int y_souris);

double  plus_proche    (Graphe *graphe, double valeur, boolean mode);

void    prep_geregraphe(boolean mode);

/* ----------- Tri et apparents ---------------------------------------------------------- */

unsigned long tri_doublons (Graphe *graphe, boolean moyenner);
unsigned long DBL_diff     (Graphe *graphe, boolean moyenner, double difference);
unsigned long DBL_groupe   (Graphe *graphe, boolean moyenner, int nbr_x);

boolean       tri_graphe   (Graphe *graphe);
void          tri_en_place (Graphe *graphe);
void          tri_fusion   (Graphe *graphe, double *nv_x, double *nv_y,
			    unsigned long i1, unsigned long i2, unsigned long i3);
unsigned long tri_partition(Graphe *graphe, unsigned long deb, unsigned long fin);

typedef struct /* Infos ncessaires pour le tri par bloc */
{
  double *xpoint_depart;    /* Pointeur de dpart - X                 */
  double *ypoint_depart;    /* Pointeur de dpart - Y                 */
  unsigned long position;   /* Point que l'on est en train de traiter */
  unsigned long nbr_points; /* Nombre de points dans le bloc          */
} TRI_infos;

Graphe *fondre  (void); /* Fusion  - renvoi le graphe obtenu */
Graphe *moyenner(void); /* Moyenne - mme chose              */
void tri_blocs(double *x_tri, double *y_tri, TRI_infos *blocs, unsigned long nbr_blocs, unsigned long total);
unsigned long MOY_blocs(double *x_tri, double *y_tri, TRI_infos *blocs, unsigned long nbr_blocs, unsigned long total);

boolean GER_tri   (int num_objet, int x_souris, int y_souris);
boolean GER_fusion(int num_objet, int x_souris, int y_souris);

/* ---------- Gestion des graphes ------------------------------- *
 *    Tout ce qui est traitement mathmatique                     *
 *     (Module GRF_MATH.C)                                        *
 ******************************************************************/

/* ------------ Oprations mathmatiques sur un seul graphe ------------------------------- */

Graphe *ajuste         (Graphe *experience);
Graphe *normalise      (Graphe *source);

/* Drivation  */

#define DER_RAPIDE     1 /* Drive rapide : delta_y/delta_x */
#define DER_SPLINE     2 /* Drive par calcul de la spline */

Graphe *GRF_derive (Graphe *graphe, int methode);
Graphe *DER_simple (Graphe *graphe);
Graphe *DER_lisse  (Graphe *graphe, unsigned long nbr_pts, const double *b, const double *c, const double *d);
Graphe *DER_seconde(Graphe *graphe, unsigned long nbr_pts, const double *b, const double *c, const double *d);

/* Intgration */

#define INTG_RECTANGLE 0
#define INTG_TRAPEZE   1
#define INTG_SIMPSON   2
#define INTG_SPLINE    3

Graphe *GRF_integre    (Graphe *graphe);
Graphe *INTG_trapeze   (Graphe *graphe, double constante);
Graphe *INTG_rectangle (Graphe *graphe, double constante);
Graphe *INTG_simpson   (Graphe *graphe, double constante);
Graphe *INTG_spline    (Graphe *graphe, double constante);

Graphe *GRF_normalise  (Graphe *graphe, int methode);

/* ------------ Oprations mathmatiques sur deux graphes --------------------------------- */

boolean GRF_clin(Graphe *gr_1, double coef_1, Graphe *gr_2, double coef_2, Graphe *resultat);

/* ---------- Gestion des graphes ------------------------------- *
 *    Tout ce qui est traitement "statistique"                    *
 *     (Module GRF_STAT.C)                                        *
 ******************************************************************/

/* ------------ Dcomposer un graphe sur une base de 2 ---------- */

typedef struct
{
  Graphe *ref_1;           /* Graphe de rfrence 1          */
  double *a_1, *b_1, *c_1; /* Pour la spline associe        */
  Graphe *ref_2;
  double *a_2, *b_2, *c_2;
  double coef_1, coef_2;   /* Les deux coefficients          */
  Graphe *residu;          /* Le graphe montrant l'erreur... */
  Graphe *c_lineaire;      /* Graphe contenant a*ref_1 + b*ref_2 */
  double erreur;
  double x_min, x_max;     /* Intervalle de travail          */
} DCMP_infos;

void    GRF_decompose(Graphe *graphe);
void    RDS_decompose(Fenetre *fen, int x, int y, int larg, int haut);
boolean MSG_decompose(Fenetre *fen);
boolean GER_decompose(int num_objet,int x, int y);
boolean CLC_decompose(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

int DCMP_calc(Graphe *spectre, DCMP_infos *infos); /* Calcul effectif */

/* ------------ Tronquer un graphe ------------------------------ */

typedef struct
{
  double x_min, x_max; /* Les limites de troncature */
  int type;            /* Mode (on perd  l'extrieur ou  l'intrieur ? */
} TRQ_infos;

void GRF_tronque(Graphe *graphe);
void    RDS_tronque(Fenetre *fen, int x, int y, int larg, int haut);
boolean MSG_tronque(Fenetre *fen);
boolean GER_tronque(int num_objet,int x, int y);
boolean CLC_tronque(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

Graphe *TRQ_calc(Graphe *graphe, TRQ_infos *infos);

/* ------------ liminer un/des points aberrants ---------------- */

typedef struct ab_param
{
  struct ab_param *suivant; /* La suivante, si travail avec plusieurs fentres simultanment */
  Graphe *graphe;    /* Le graphe sur lequel on travaille */
  double x_borne[4]; /* Les bornes pour les intervalles de lissage : x1min, x1max, x2min, x2max */
  char degre;        /* Degr du polynme de lissage */
  double coefs[11];  /* Coefficients de ce polynme  */
} ABER_param;

void aberrance(Graphe *graphe);

void    RDS_aberrance(Fenetre *fen, int x, int y, int larg, int haut);
boolean MSG_aberrance(Fenetre *fen);
boolean GER_aberrance(int num_objet,int x, int y);
boolean CLC_aberrance(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

/* ---------- Superposition de plusieurs graphes ---------------- *
 *      (Module TRACE.C)                                          *
 ******************************************************************/

/* ---------- Gestion de la fentre contenant cela ------------------- */

void    TRC_ouvre(void);  /* On ouvre la fentre de trac de plusieurs graphes */
void    RDS_trace(Fenetre *, int, int, int, int); /* Redessin de cette fentre */
boolean RCT_trace(Fenetre *fen, int x, int y);
boolean MSG_trace(Fenetre *fen);
boolean CLC_trace(Fenetre *fen, int bouton, int n_clic, int etat, int x, int y);

/* ---------- Tout ce qui concerne la gestion des rectangles --------- */

Cadre *REC_copie  (Cadre *source);
Cadre *REC_creer  (int type, int x1, int y1, int x2, int y2);
void       REC_efface (Cadre *rect);

boolean CLC_Rgraphe(Fenetre *fen, Cadre *rect, int x, int y, int n_clic, int etat);
void    REC_bouge  (Cadre *rect, Position *lieu_clic, int x, int y, int x_max, int y_max);

void REC_bornes(Rect_graphe *rect);

void DES_Rgraphe(Station_VDI *station, Fenetre *fenetre, Cadre *rect);

/* ---------- Les fonctions de base de dessin ------------------------ */

#define NON_TOURNE 0 /* Sens normal : type cran ou imprimante, portrait */
#define TOURNE     1 /* Tourn de 90 degrs : type imprimante, paysage   */

void TRC_poly  (unsigned long nbr_pts, char degre, double *coefs, double *x,
                double x_echelle, double y_echelle, double x_trace, double y_trace,
                double x_min, double x_max, double y_min, double y_max);
void TRC_graphe(Station_VDI *station, DES_graphe *graphe, int x, int y, 
                double x_min, double x_max, double y_min, double y_max, /* Les bornes de trac...
									   -> vite de dborder + acclre */
                double x_echelle, double y_echelle);

void TRC_cadre (Station_VDI *station, Rect_graphe *rect, int x, int y);
void TRC_gradue(Station_VDI *station, Rect_graphe *rect, int x, int y, int type, int sens);
void TRC_grille(Station_VDI *station, Rect_graphe *rect, int x, int y, int type, int sens);

/* ---------- Le paramtrage du dessin ------------------------------- */

void chg_legende(Axe *axe);          /* Les axes en X & en Y */
void chg_bornes (Rect_graphe *rect); /* Les bornes de trac  */

/* ---------- Ajustement des spectres thoriques ---------------- *
 *      (Module AJUSTE.C)                                         *
 ******************************************************************/

/* ---------- Correction des phases et amplitudes thoriques ---- */

Graphe *corr_phase    (Graphe *graphe, double longueur); /* Correction de la phase */
Graphe *corr_amplitude(Graphe *graphe, double n_voisin, double sigma, double longueur);

/* ---------- Calcul de divers graphes par utilisation de la thorie *
 *      Module THEORIE.C                                             *
 *      Prfixe gnral : CTH pour Calcul THorique                  * 
 *********************************************************************/

Graphe *CTH_mu0(void); /* Calcul d'un mu0 thorique */

typedef struct
{
  GRF_liste *centre;
  GRF_liste *reflecteur;
} PHA_infos;

Graphe *CTH_phase(void);   /* Calcul d'un dphasage pour une couple d'atomes */
boolean GER_phase(int num_obj, int x_souris, int y_souris); /* Arbre associ */

#endif
