/* Traitement de l'E.X.A.F.S. sur ATARI
						Gestion gnrale des graphes (cration, effacement,...)
	 (c) 1996-1998 - Emmanuel CURIS

  1er avril 1998
    * Modifi prep_geregraphe() : utilisation des OBJ_??? et prise en compte tous/aucun

  10, 11, 13, 16, 25 mars 1998
    * Utilisation de "systeme.h"
    * Cration des graphes : init. des nouveaux champs (pointeurs sur les graphes contenant les erreurs)
    * Effacement de graphe : supprime les liens qui pourraient exister vers le graphe
    * Inclus "erreurs.h" pour l'exafs
    * Adapt GRF_choisir : titres quand slection plus souple que par type
    * Corrig le tri sur place : bouclait sans fin si non tri...
    * Fonction de tri sur place d'un graphe, par permutations : tri_en_place()
    * Inclus fichiers.h, pour le prototype de sauver()

  12, 19 janvier 1998
    * Titres de menu "erreur" et "affichage" inactifs si aucun graphe en mmoire
    * Intgr les messages d'erreur GEM : erreurs.h

   2 dcembre 1997
    * Affichage des menus quand ils changent plus correct
    * Cration de MEN_adapte() : nouvelle version de l'adaptation des menus
      suppression de adapte_menus()
    * Dplac les calculs de primitive vers grf_math.c
    * Choix d'un graphe dans un slecteur : le type du graphe dsir s'affiche maintenant

  25 - 28 novembre 1997
    * prise en compte de la nouvelle interface de la liste de graphes
    * prep_geregraphes() : les types des graphes utilisent les chanes dans le ressource
    * GRF_aller() : mme chose
    * Suppression des dfinitions des noms de types (maintenant dans le ressource...)
    * Commenc remplacement de effacer_graphe() par GRF_efface(), plus souple d'emploi
    * Adapt prep_geregraphes() : utilisation de met_chaine()
    * Cration de cet historique
    * Rorganis la recherche des bornes d'un graphe
    * Corrig, dans GRF_bornes, erreur : bornes en X d'un graphe complexe taient fausses
                              si cherches par parcours complet (prises gales
			      au X du premier point toujours...)
*/

#define _GRAPHES_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "GRAPHES.H"

#include "ERREURS.H"
#include "EXAFS.H"
#include "FICHIERS.H"

/* --------------- Gestion gnrale des graphes : cration, effacement,...    *
 ******************************************************************************/

/* --------------- Cration d'un nouveau graphe, aprs le graphe courant -----\
 |      Renvoie l'adresse de ce graphe (NULL si erreur)                       |
 |      Les valeurs du graphe sont mises  zro par calloc                    |
 |                                                                            |
 | nbr_pts : nombre de points dans le graphe                                  |
 | type    : le type de graphe cr (GR_...)                                  |
 | mode    : la nature du graphe (GR_NORMAL, GR_IM_MOD, ...)                  |
 | nom     : le nom  donner au graphe                                        |
 | leg_x   : la lgende en X                                                  |
 | leg_y   : la lgende en Y                                                  |
 | unit_x  : unit de l'axe des abscisses                                     |
 | unit_y  : unit de l'axe des ordonnes                                     |
 \--------------------------------------------------------------------------- */

Graphe *GRF_creer(unsigned long nbr_pts, int type, int mode, char *nom, 
		  char *leg_x, char *leg_y, int unit_x, int unit_y)
{
  Graphe *graphe;

  if (nbr_pts < 2)
  {
    EXF_ERREUR(TROP_PEU_POINTS);
    return NULL;
  }

  /* On rserve la mmoire pour les graphes */

  graphe = (Graphe *) malloc( sizeof(Graphe) );
  if (graphe == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return NULL;
  }

  graphe->x = (double *) calloc( nbr_pts,  sizeof(double) );
  if (graphe->x == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    free(graphe);
    return NULL;
  }

  if (mode) /* Graphe complexe <=> mode > 0 */
  {
    Complexe *y = (Complexe *) calloc( nbr_pts, sizeof(Complexe) );
    if (y == NULL)
    {
      ERREUR(MANQUE_MEMOIRE);
      free(graphe); free(graphe->x);
      return NULL;
    }
    graphe->y = (double *) y; /* Ce n'est pas trs beau, mais bon... */
  }
  else
  {
    graphe->y = (double *) calloc( nbr_pts, sizeof(double) );
    if (graphe->y == NULL)
    {
      ERREUR(MANQUE_MEMOIRE);
      free(graphe); free(graphe->x);
      return NULL;
    }
  }

  /* On ralise le chanage des graphes (liste chane dans les 2 sens) */

  if (premier_graphe == NULL) /* donc on cre le premier graphe */
  {
    OBJECT *adr_menu = arbres[MENU].adresse;

    premier_graphe = graphe;
    dernier_graphe = graphe;
    graphe_courant = graphe; /* Forcment... */
		
    graphe->precedent = NULL;
    graphe->suivant   = NULL;
    nbr_graphes = 1;
    num_graphe  = 1;
		
    menu_ienable(adr_menu, SAUVER,1);				/* Active les options de menu */
    menu_ienable(adr_menu, EXPORTER,1);
    if ( gdos ) /* GDOS est install */
    {
      menu_ienable(adr_menu, IMP_GRAPHE,1);
    }

    menu_ienable(adr_menu, TIT_GRAPHES,   1);
    menu_ienable(adr_menu, TIT_EXTRACTION,1);
    menu_ienable(adr_menu, TIT_FILTRAGE,  1); 
    menu_ienable(adr_menu, TIT_ERREURS,   1);
    menu_ienable(adr_menu, TIT_TRACE,     1);

    if (_PRG_menu == MENU) /* On travaille bien sur ce menu l */
      menu_bar(adr_menu,1);
  }
  else /* On insre le graphe aprs le graphe courant */
  {
    int pos_ascenseur, taille_ascenseur;
    graphe->precedent       = graphe_courant;
    graphe->suivant         = graphe_courant->suivant;
    graphe_courant->suivant = graphe;
		
    if (graphe->suivant) /* Il y a un graphe aprs */
    {
      (graphe->suivant)->precedent = graphe;
    }
    else
      dernier_graphe = graphe; /* Nouveau dernier graphe */

    nbr_graphes++; /* Un graphe de plus                                   */
    num_graphe++;  /* Et on est plac juste aprs l'ancien graphe courant */

    pos_ascenseur = ( (float) ( 1000 * (num_graphe - 1) ) / (float) (nbr_graphes - 1) );
    taille_ascenseur = (int) ( 1000.0 / (float) (nbr_graphes)	);
    FEN_ascenseur(principale, ASC_VERTICAL, pos_ascenseur, taille_ascenseur);
  }

  /* On initialise le graphe */

  graphe->nombre_point      = nbr_pts; /* Nombre de points du graphe  */
  graphe->type              = type;    /* Type du graphe              */
  graphe->drapeaux.complexe = mode;    /* Genre de graphe             */
  graphe->drapeaux.protege  = FAUX;    /* Graphe non protg a priori */
  graphe->drapeaux.sauve    = FAUX;    /* Pas encore sauv            */
  graphe->trace             = VRAI;    /* On le trace                 */
  graphe->style             = 1;       /* Ligne continue              */
  graphe->couleur           = NOIR;    /*                noire        */
  graphe->pointeur          = NULL;    /* Pas d'info complmentaire   */
  graphe->erreur_x          = NULL;    /* Pas de graphe contenant l'erreur en X */
  graphe->erreur_y          = NULL;    /*                               ni en Y */
  graphe->grf_lie           = NULL;    /* Pas d'autre graphe associ  celui-ci */

  graphe->x_min = 0; graphe->x_max = 0; /* Tranquille pour le dessin... */
  graphe->y_min = 0; graphe->y_max = 0;

  if ( (type == GR_EXAFS1) || (type == GR_EXAFS2) || (type == GR_EXAFS)
    || (type == GR_TFI)    || (type == GR_AJUSTE) )
    graphe->drapeaux.exafs  = VRAI;    /* C'est un signal E.X.A.F.S.  */
  else
    graphe->drapeaux.exafs  = FAUX;    /* Ce n'est pas un signal E.X.A.F.S. */

  GRF_titre  (graphe, nom);                  /* On met le nom */
  GRF_legende(graphe, AXE_X, leg_x, unit_x); /* Lgende en X  */
  GRF_legende(graphe, AXE_Y, leg_y, unit_y); /* Lgende en Y  */

  prep_geregraphe(VRAI);
  FEN_redessin( ARB_fenetre(arbres + CHOIX_GRAPHE) );

  return graphe; /* On a fini */
}

/* --------------- Ancienne routine, pour compatibilit en attendant -------- */

Graphe *creer_graphe(unsigned long nbr_points, int type)
{
  switch (type)
  {
  case GR_EXAFS :
  case GR_EXAFS1: /* Tous les spectres de genre EXAFS */
  case GR_EXAFS2:
  case GR_TFI   :
  case GR_AJUSTE:
    return GRF_creer(nbr_points, type, GR_NORMAL, "Sans nom", "Nombre d'onde k", "khi(k)", UNIT_1_ANG, UNIT_ARB);
	
  case GR_TF:
    return GRF_creer(nbr_points, GR_TF, GR_IM_MOD, "Sans nom", "Longueur du chemin", "Contribution", UNIT_ANG, UNIT_ARB);

  case GR_MU0:
    return GRF_creer(nbr_points, GR_MU0, GR_NORMAL, "Sans nom", "Nombre d'onde k", MU"0", UNIT_1_ANG, UNIT_ARB);
		
  case GR_BRUT:
  case GR_MOINSMAT:
    return GRF_creer(nbr_points, type, GR_NORMAL, "Sans nom", E_AIGU"nergie", "Absorbance", UNIT_EV, UNIT_ARB);

  case GR_EXAFSAMP:
    return GRF_creer(nbr_points, GR_EXAFSAMP, GR_NORMAL, "Sans nom", "Nombre d'onde k", "Amplitude", UNIT_1_ANG, UNIT_ARB);
		
  case GR_EXAFSPHA:
    return GRF_creer(nbr_points, GR_EXAFSPHA, GR_NORMAL, "Sans nom", "Nombre d'onde k", "D"e_AIGU"phasage", UNIT_1_ANG, UNIT_ARB);

  default:
    return GRF_creer(nbr_points, type, GR_NORMAL, "Sans nom", "Sans unit"e_AIGU, "Sans unit"e_AIGU, SS_UNITE, SS_UNITE);
  }
}

/* -------------- Effacer un graphe, avec ou sans contrle --------------------------------\
 |    Renvoie VRAI si le graphe a t effac, FAUX sinon                                   |
 |                                                                                         |
 | graphe        : le graphe que l'on veut effacer                                         |
 | force_sauve   : VRAI : on efface mme si le graphe n'a pas t sauv                    |
 | force_protege : VRAI = on efface mme si le graphe est protg                          |
 \---------------------------------------------------------------------------------------- */

boolean GRF_efface(Graphe *graphe, boolean force_sauve, boolean force_protege)
{
	Graphe *graphe_lie;
	boolean redessin = FAUX; /* VRAI si on doit redessiner */
	OBJECT *adr_menu = arbres[MENU].adresse;

	if (graphe == NULL) return VRAI; /* Pas de problme */

	if ( (graphe->drapeaux.protege == VRAI) && (force_protege == FAUX) )
	{
		EXF_ERREUR(GRF_PROTEGE);
		return FAUX;	
	}

	if ( (graphe->drapeaux.sauve == FAUX) && (force_sauve == FAUX) )
	{
		int reponse = EXF_ERREUR(NON_SAUVE);

		if (reponse == 3) return FAUX;
		if (reponse == 1) sauver(graphe); /* On sauve le graphe */
	}

	if (graphe == graphe_courant) /* Seul cas o le redessin est obligatoire */
		redessin = VRAI;

	free(graphe->x); /* On efface les donnes */
	free(graphe->y);
	free(graphe->pointeur); /* Y compris les ventuelles complmentaires */

	if (graphe->precedent == NULL) /* Premier graphe */
	{
	  if (graphe->suivant == NULL) /* Dernier graphe en mmoire */
	  {
	    premier_graphe = NULL;
	    dernier_graphe = NULL;
	    graphe_courant = NULL;
	    num_graphe = 0;
			
	    menu_ienable(adr_menu, SAUVER,   0); /* Dsactive les options de menu */
	    menu_ienable(adr_menu, EXPORTER, 0);

	    menu_ienable(adr_menu, TIT_GRAPHES,    0); /* Dsactive les titres */
	    menu_ienable(adr_menu, TIT_EXTRACTION, 0);
	    menu_ienable(adr_menu, TIT_FILTRAGE,   0); 
	    menu_ienable(adr_menu, TIT_ERREURS,    0);
	    menu_ienable(adr_menu, TIT_TRACE,      0);

	    if (_PRG_menu == MENU) /* On travaille bien sur ce menu l */
	      menu_bar(adr_menu,1);
	  }
	  else
	  {
	    premier_graphe = graphe->suivant;
	    premier_graphe->precedent = NULL; /* Le suivant devient le premier */
	    if (graphe == graphe_courant)
	    {
	      graphe_courant = premier_graphe;
	    }
	    num_graphe = 1;
	  }
	}
	else if (graphe->suivant == NULL) /* Dernier graphe */
	{
		dernier_graphe = graphe->precedent;
		dernier_graphe->suivant = NULL;
		if (graphe == graphe_courant)
		{
			graphe_courant = dernier_graphe;
		}
		num_graphe = nbr_graphes-1;
	}
	else
	{
		(graphe->precedent)->suivant = graphe->suivant;   /* On reforme la chaine */
		(graphe->suivant)->precedent = graphe->precedent;
		if (graphe == graphe_courant)
		{
			graphe_courant = graphe->suivant;
		}
	}

	nbr_graphes--; /* Un graphe de moins  */
	if (graphe == mu0_soustrait) /* pour viter des mauvaises surprises */
		mu0_soustrait = NULL;

	/* On vrifie qu'aucun graphe n'est reli  celui-ci */

	if (premier_graphe) /* Il reste un graphe au moins en mmoire */
	{
		for (graphe_lie	= premier_graphe ; graphe_lie->suivant != NULL ; graphe_lie = graphe_lie->suivant)
		{
			if (graphe_lie->erreur_x == graphe) graphe_lie->erreur_x = NULL;
			if (graphe_lie->erreur_y == graphe) graphe_lie->erreur_y = NULL;
			if (graphe_lie->grf_lie  == graphe) graphe_lie->grf_lie  = NULL;
		}
	}

	free(graphe);  /* On efface le graphe */

	if (nbr_graphes < 2) /* 0 ou 1 graphe */
	{
		FEN_ascenseur(principale, ASC_VERTICAL, 0, 1000);
	}
	else
	{
		int pos_asc, taille_asc;

		pos_asc = (int) ( ( 1000 * (num_graphe - 1) ) / (long) (nbr_graphes - 1) );
		taille_asc = (int) ( 1000.0 / (float) (nbr_graphes)	);
		FEN_ascenseur(principale, ASC_VERTICAL, pos_asc, taille_asc);
	}

	if (redessin == VRAI)  /* On vient d'effacer le graphe courant */
	{
		GRF_aller(graphe_courant);
	}
	if (nbr_graphes == 0)  /* Plus aucun graphe en mmoire */
		FEN_redessin(principale);

	prep_geregraphe(VRAI); /* On refait la liste des graphes */
	FEN_redessin( ARB_fenetre(arbres + CHOIX_GRAPHE) );

	return VRAI; /* On a effac le graphe */
}

/* -------------- Effacer tous les graphes en mmoire -------------------------------------\
 |   Renvoie FAUX si l'utilisateur n'a finalement pas voulu tout effacer                   |
 \---------------------------------------------------------------------------------------- */

boolean GRF_tteffacer()
{
  Graphe *graphe = premier_graphe;
  int reponse;
  boolean tt_sauve = FAUX, tt_perdre = FAUX;

  if (premier_graphe == NULL) /* Aucun graphe  effacer */
    return VRAI;

  /* On commence par tous les graphes utilisateur */

  graphe = premier_graphe;
  while (graphe)
  {
    Graphe *tmp = graphe->suivant;

    if ( (graphe->type == GR_INTERNE) /* Graphe  usage interne -> on efface seulement  la fin */
	 || (graphe->drapeaux.protege == VRAI) ) /* Graphe utilis pour un calcul               */
    {
      graphe = tmp; /* On saute directement au suivant */
      continue;
    }
	  
    if (tt_perdre) /* L'utilisateur a demand  tout perdre */
      graphe->drapeaux.sauve = VRAI; /* Donc le graphe est considr sauv */
	  
    if (graphe->drapeaux.sauve == FAUX) /* Le graphe n'est pas sauv */
    {
      if (tt_sauve) /* On veut tous les sauver */
      {
	sauver(graphe); /* On sauve le graphe */
      }
      else
      {
	met_chaine(ARB_QUITTER, QUIT_NOM_GRAPHE, 0, graphe->nom, TXT_REMPLACE);
	reponse = ext_formdo(ARB_QUITTER);
	switch (reponse)
	{
	case QUIT_RESTER:   /* Finalement, on ne veut plus effacer les graphes... */
	  return FAUX;
		      
	case QUIT_TTSAUVER: /* On sauve tous les graphes non sauvs               */
	  tt_sauve = VRAI;
	case QUIT_SAUVER:   /* On sauve ce graphe                                 */
	  sauver(graphe);
	  break;
		      
	case QUIT_TTPERDRE: /* On perd tous les graphes                           */
	  tt_perdre = VRAI;
	}
      }
      graphe->drapeaux.sauve = VRAI; /* De toute faon, on le perd... */
    }
	  
    GRF_efface(graphe, VRAI, FAUX); /* On efface ce graphe */
    graphe = tmp;
  }

  if (premier_graphe) /* Il reste des graphes : graphes internes... */
  {
    while (premier_graphe)
      GRF_efface(premier_graphe, VRAI, VRAI);
  }

  return VRAI; /* On a pu effacer tous les graphes */
}

/* -------------- Copier un graphe ------------------------------------------------------- */

Graphe *copier_graphe(Graphe *source)
{
	Graphe *copie;
	unsigned long index;
	register double   *zone_x,*zone_y;

	if (source == NULL) return NULL;
	sprintf(_intermediaire, "Copie de %s", source->nom);

	copie = GRF_creer(source->nombre_point, source->type, source->drapeaux.complexe, _intermediaire,
	                  source->axe_x, source->axe_y, source->unit_x, source->unit_y);
	if (copie == NULL) return NULL; /* Copie a choue */

	copie->x_min = source->x_min;
	copie->x_max = source->x_max;
	copie->y_min = source->y_min;
	copie->y_max = source->y_max;
	
	zone_x = source->x;
	zone_y = copie->x;
	for (index = 0 ; index < source->nombre_point ; index++)
	{
		*zone_y = *zone_x;
		zone_x++; zone_y++;
	}
	
	if (source->drapeaux.complexe) /* Graphe complexe */
	{
		register Complexe *source_ft = (Complexe *) source->y;
		register Complexe *copie_ft  = (Complexe *) copie->y;
		for (index = 0 ; index < source->nombre_point ; index++)
		{
			*copie_ft = *source_ft;
			source_ft++; copie_ft++;
		}
	}
	else
	{
		zone_x = source->y;
		zone_y = copie->y;
		for (index = 0 ; index < source->nombre_point ; index++)
		{
			*zone_y = *zone_x;
			zone_x++; zone_y++;
		}
	}
	
	copie->drapeaux = source->drapeaux; /* Mme drapeaux */
	copie->drapeaux.protege = FAUX; /* Par dfaut, n'est pas utilis dans un calcul... */

	return copie; /* la copie a t effectue */
}

/* -------------- Aller  un graphe (le mettre comme graphe courant) ----------------------\
 |   Renvoie rien
 |
 | graphe : le graphe que l'on veut mettre comme graphe courant                            |
 \---------------------------------------------------------------------------------------- */

void GRF_aller(Graphe *graphe)
{
  if (graphe == NULL) return;

  if (premier_graphe == NULL)
  {
    printf("Erreur vraiment trange : demande d'aller  un graphe possible \n alors qu'il n'y a aucun graphe en mmoire !");
    return;
  }

  if (nbr_graphes == 1) /* Un seul graphe -> de toute faon, pas le choix... */
  {
    if (graphe != premier_graphe)
    {
      form_alert(1,"[3][ Ce graphe n'existe pas ou plus ][ Annuler ]");
      graphe = premier_graphe;
    }
  }
  else
  {
    Graphe *index = premier_graphe;
    float pos_asc;

    num_graphe = 0;
    for ( index = premier_graphe ; index != graphe ; index = index->suivant )
    {
      num_graphe++;
      if ( num_graphe >= nbr_graphes ) /* Problme... */
      {
	form_alert(1,"[3][ Ce graphe n'existe pas | ou a "e_AIGU"t"e_AIGU" "e_AIGU"ffac"e_AIGU" ][ Annuler ]");
	graphe = dernier_graphe;
	num_graphe = nbr_graphes - 1;
	break;
      }
    }
    
    /* On positionne l'ascenseur */
    pos_asc  = num_graphe * 1000;
    pos_asc /= nbr_graphes - 1;

    FEN_ascenseur(principale, ASC_VERTICAL, (int) pos_asc, -1);
  }

  graphe_courant = graphe;

  /* Titre de la fentre */
  FEN_titre(principale, graphe_courant->nom);

  /* Ligne d'informations de la fentre */
  if (graphe_courant->type >= 0)
    FEN_infos(principale, (char *) Type_graphe[graphe_courant->type] );
/*  else if ( graphe_courant->type == GR_BRUT    )
    FEN_infos(principale, "Spectre exp"e_AIGU"rimental brut"); */
  else if ( graphe_courant->type == GR_INTERNE )
    FEN_infos(principale, "Graphe "a_GRAVE" usage interne");
  else
    FEN_infos(principale, "Sans type");

  /* Paramtres de trac */

  RCT_courant.x_min = graphe->x_min;
  RCT_courant.x_max = graphe->x_max;
  RCT_courant.y_min = graphe->y_min;
  RCT_courant.y_max = graphe->y_max;

  switch (graphe_courant->unit_x) /* On adapte l'unit des axes */
  {
  case SS_UNITE:
    strcpy(RCT_courant.axe_x.unite, "Ss. u.");
    break;
    
  case UNIT_EV:
    strcpy(RCT_courant.axe_x.unite, "eV");
    break;
    
  case UNIT_ANG:
    strcpy(RCT_courant.axe_x.unite, ANGSTROM);
    break;
    
  case UNIT_1_ANG:
    strcpy(RCT_courant.axe_x.unite, "1/"ANGSTROM);
    break;
    
  case UNIT_ARB:
    strcpy(RCT_courant.axe_x.unite, "u. arb");
    break;
    
  case UNIT_NORM:
    strcpy(RCT_courant.axe_x.unite, "norm"e_AIGU);
    break;
    
  default:
    strcpy(RCT_courant.axe_x.unite, "???");
    break;
  }
  
  switch (graphe_courant->unit_y)
  {
  case SS_UNITE:
    strcpy(RCT_courant.axe_y.unite,"Ss. u.");
    break;
    
  case UNIT_EV:
    strcpy(RCT_courant.axe_y.unite,"eV");
    break;

  case UNIT_ANG:
    strcpy(RCT_courant.axe_y.unite, ANGSTROM);
    break;

  case UNIT_1_ANG:
    strcpy(RCT_courant.axe_y.unite, "1/"ANGSTROM);
    break;

  case UNIT_ARB:
    strcpy(RCT_courant.axe_y.unite,"u. arb");
    break;

  case UNIT_NORM:
    strcpy(RCT_courant.axe_y.unite,"norm"e_AIGU);
    break;

  default:
    strcpy(RCT_courant.axe_y.unite,"???");
    break;
  }

  FEN_redessin( principale );
  prep_geregraphe(VRAI);
  FEN_redessin( ARB_fenetre( arbres + CHOIX_GRAPHE ) );
  FEN_redessin( trace_fenetre );

  MEN_adapte( graphe );
}

/* -------------- Choisir un graphe d'un type donn ---------------------------------------\
 |   Renvoie le graphe choisi (NULL == erreur)                                             |
 |                                                                                         |
 | type : le type de graphes que l'on veut afficher                                        |
 |   (reconnat quelques types spciaux : si type <= -100 - cf. graphes.h                  |
 \---------------------------------------------------------------------------------------- */

Graphe *GRF_choisir(int type)
{
	GRF_liste *liste; /* La liste contenant les choix possibles */
	Graphe *retour = NULL;

	liste = GRF_par_type(type, VRAI);
	if (liste == NULL) return NULL; /* Problme de slection */

	arbres[CHX_GRAPHE].pointeur = liste; /* On affecte la liste  l'arbre */

	GRF_affiche(liste, CHX_GRAPHE, CHXG_NOM1, CHX_NOM10, 
		                 CHXG_AVANT, CHXG_APRES, CHXG_FOND_ASC, CHXG_POUSSOIR);

/*	if (type == GR_BRUT)
		met_chaine(CHX_GRAPHE, CHXG_TITRE, 40, "Spectre exp"e_AIGU"rimental brut", TXT_COPIER);
	else */ if (type == GR_EXAFS)
		met_chaine(CHX_GRAPHE, CHXG_TITRE, 40, "Tous les spectres de type EXAFS", TXT_COPIER);
	else if (type == GR_TOUS)
		met_chaine(CHX_GRAPHE, CHXG_TITRE, 40, "Tous les spectres en m"e_AIGU"moire", TXT_COPIER);
	else if (type >= 0)
		met_chaine(CHX_GRAPHE, CHXG_TITRE, 40, (char *) Type_graphe[type], TXT_COPIER);

	if (ext_formdo(CHX_GRAPHE) == CHXG_ANNULE)
	{
		free(liste); /* On libre */
		return NULL;
	}

	retour = (liste->graphes)[liste->num_select];
	free(liste);
	return retour;
}

/* -------------- Modification de la lgende d'un des axes d'un graphe --------------------\
 |
 | graphe  : le graphe  modifier
 | axe     : l'axe sur lequel on travaille (AXE_X ou AXE_Y)
 | legende : pointeur de la chane contenant la lgende
 | unite   : unit utilise sur cet axe (code...)
 \---------------------------------------------------------------------------------------- */

void GRF_legende(Graphe *graphe, int axe, char *legende, int unite)
{
	char *dest_legende;
	int  i;
	unsigned long long_legende;

	if (graphe == NULL) return;     /* Pas de graphe... */
	if (axe == AXE_X)
	{
		graphe->unit_x = unite;       /* On met l'unit */
		dest_legende = graphe->axe_x;
	}
	else if (axe == AXE_Y)
	{
		graphe->unit_y = unite;
		dest_legende = graphe->axe_y;
	}

	long_legende = strlen(legende);
	for (i = 0 ; i < LONG_LEGENDE ; i++) /* Avec cette mthode, sr de ne pas boucler  l'infini... */
	{
		if ( i < long_legende )
			dest_legende[i] = legende[i];
		else
			dest_legende[i] = 0;
	}
	dest_legende[LONG_LEGENDE] = 0;
}

/* -------------- Modification du titre d'un graphe ---------------------------------------\
 |
 | graphe : le graphe  modifier
 | titre  : pointeur de la chane contenant le titre  installer
 \---------------------------------------------------------------------------------------- */

void GRF_titre(Graphe *graphe, char *titre)
{
	int i;
	unsigned long long_titre;

	if (graphe == NULL) return;     /* Pas de graphe... */

/*	printf("LG_TITRE = %ld, chaine de longueur %ld\n", (long) LG_TITRE, strlen(titre) ); */

	long_titre = strlen(titre);
	for (i = 0 ; i < LG_TITRE ; i++) /* Avec cette mthode, sr de ne pas boucler  l'infini... */
	{
		if ( i < long_titre )
			(graphe->nom)[i] = titre[i];
		else
			(graphe->nom)[i] = 0;
	}
	(graphe->nom)[LG_TITRE] = 0;
}

/* -------------- Adaptation des menus au type de graphe --------------------------------- */

void MEN_adapte(Graphe *graphe)
{
  OBJECT *adr_menu = arbres[MENU].adresse;
  boolean absorbe = FAUX, exafs = FAUX, autre = FAUX, derive = FAUX, module = FAUX, phase = FAUX;

  /* Options gnrales au traitement des graphes */

  if ( graphe->drapeaux.complexe == GR_NORMAL ) /* Options spcifiques des graphes normaux */
  {
    menu_ienable(adr_menu, TIT_EXTRACTION, 1); /* Extraction d'un signal E.X.A.F.S    */
    menu_ienable(adr_menu, RECOMP_TF, 1);      /* Recomposition d'un spectre complexe */
    menu_ienable(adr_menu, ABERRANCE, 1);      /* limination d'une aberrance         */
  }
  else /* Options spcifiques des graphes complexes */
  {
    /* On dsactive donc les options spcifiques des graphes rels */
    menu_ienable(adr_menu, TIT_EXTRACTION, 0);
    menu_ienable(adr_menu, RECOMP_TF, 0);
    menu_ienable(adr_menu, ABERRANCE, 0);
  }

  /* Options spcifiques  l'extraction d'un signal E.X.A.F.S. */

  if (graphe->type == -1) autre = VRAI; /* Graphe sans type : tout est possible... */

  absorbe = ( (graphe->type == GR_BRUT)     || (graphe->type == GR_MOINSMAT) );
  derive  = ( (graphe->type == GR_DERIVEE)  || (graphe->type == GR_DERIVEE2) );
  exafs   = ( (graphe->type == GR_EXAFS1)   || (graphe->type == GR_EXAFS2)
	   || (graphe->type == GR_EXAFS)    || (graphe->drapeaux.exafs == VRAI) );
  module  = ( (graphe->type == GR_EXAFSAMP) || (graphe->type == GR_COMPMOD ) );
  phase   = ( (graphe->type == GR_EXAFSPHA) || (graphe->type == GR_COMPARG ) );

  menu_ienable(adr_menu, MATRICE, autre || (graphe->type == GR_BRUT)); /* Soustration du terme de la matrice */
  menu_ienable(adr_menu, XANES_TOUT,    autre|absorbe); /* Traitement complet du XANES          */
  menu_ienable(adr_menu, CHERCHE_E0,    autre|absorbe|derive); /* Recherche d'un E0             */
  menu_ienable(adr_menu, DERIVEE_UN,    autre|absorbe|derive); /* Calcul de la drive premire */
  menu_ienable(adr_menu, DERIVEE_DEUX,  autre|absorbe); /* Calcul direct de la drive seconde  */
  menu_ienable(adr_menu, XAN_NORMALISE, autre|absorbe); /* Normalisation du X.A.N.E.S.          */
  menu_ienable(adr_menu, INT_PRESEUIL,  autre|absorbe); /* Intgration du prseuil              */
  menu_ienable(adr_menu, EXAFS_TOUT,    autre|absorbe); /* Traitement complet de l'E.X.A.F.S.   */
  menu_ienable(adr_menu, OTE_MU0_1,     autre|absorbe); /* Soustraction de mu0, premire tape  */
  menu_ienable(adr_menu, OTE_MU0_2,     autre|exafs);   /* Soustraction de mu0, deuxime tape  */
  menu_ienable(adr_menu, OTE_MU0_3,     autre|exafs);   /* Soustraction de mu0, troisime tape */
  menu_ienable(adr_menu, MEN_MU0_TOTAL, autre|absorbe); /* Soustraction de mu0, dj calcul    */
  menu_ienable(adr_menu, OTE_MU0_3,     autre|exafs);   /* Soustraction de mu0, troisime tape */
  menu_ienable(adr_menu, OTE_MU0_3,     autre|exafs);   /* Soustraction de mu0, troisime tape */
  menu_ienable(adr_menu, CORR_AMPLITUDE, autre|module); /* Correction d'une amplitude */
  menu_ienable(adr_menu, CORR_PHASE,     autre|phase);  /* Correction d'une phase     */

  if ( (graphe->unit_x == UNIT_EV) || (graphe->unit_x == UNIT_1_ANG) )
    menu_ienable(adr_menu, CONVERSION, 1); /* Conversion nergie (eV) <-> nombre d'onde (1/) */
  else
    menu_ienable(adr_menu, CONVERSION, 0);
		
  if (_PRG_menu == MENU) /* Menu d'extraction est le menu actif */
    menu_bar(adr_menu, 1); /* On rinstalle la barre de menu modifie */
}

/* -------------- Quelques oprations mathmatiques sur les graphes ---------------------- *
 *******************************************************************************************/

/* -------------- Normaliser le graphe : minimum  0, maximum  1 ------------------------ */

Graphe *normalise(Graphe *graphe)
{
	double y,facteur;
	unsigned long index;
	
	facteur = (graphe->y_max - graphe->y_min);
	if (facteur == 0)
	{
		form_alert(1,"[3][ Normalisation impossible | Les bornes sont identiques ][ Annuler ]");
		return graphe;
	}

	for (index = 0 ; index < graphe->nombre_point ; index++)
	{
		y = *(graphe->y + index);
		*(graphe->y + index) = (y - graphe->y_min) / facteur;
	}
	
	graphe->y_min = 0.0;
	graphe->y_max = 1.0;

	sprintf(_intermediaire, "%s (Normalis"e_AIGU")", graphe->nom);
	if ( strlen(_intermediaire) > LG_TITRE )
		_intermediaire[LG_TITRE] = 0;
	strncpy(graphe->nom,_intermediaire,LG_TITRE);
	
	return graphe;
}

/* ---------------- Rechercher une valeur dans un graphe ---------------------------------
        Renvoie le nombre le plus proche de valeur trouv dans graphe
        mode = VRAI => chercher dans les x, FAUX => chercher dans les y                     */

double plus_proche(Graphe *graphe, double valeur, boolean mode)
{
	double erreur,retour;
	unsigned long index;

	if (mode == VRAI) /* Quel est le x enregistr le plus proche de valeur ? */
	{
		if (valeur <= graphe->x_min) return graphe->x_min; /* On est au-del des bornes */
		if (valeur >= graphe->x_max) return graphe->x_max;

		erreur = fabs( valeur - *(graphe->x) );
		retour = *(graphe->x);
		for (index = 1 ; index < graphe->nombre_point ; index++)
		{
			if ( fabs( valeur - *(graphe->x + index) ) < erreur )
			{
				retour = *(graphe->x + index);
				erreur = fabs(valeur-retour);
			}
			if (erreur == 0.0) /* La valeur est contenue dans le graphe */
				return retour;
		}
	}
	else /* Quel est le y enregistr le plus proche de valeur ? */
	{
		if (valeur <= graphe->y_min) return graphe->y_min; /* On est au-del des bornes */
		if (valeur >= graphe->y_max) return graphe->y_max;

		erreur = fabs( valeur - *(graphe->y) );
		retour = *(graphe->y);
		for (index = 1 ; index < graphe->nombre_point ; index++)
		{
			if ( fabs( valeur - *(graphe->y + index) ) < erreur )
			{
				retour = *(graphe->y + index);
				erreur = fabs(valeur-retour);
			}
		}
	}
	
	return retour;
}

/* -------------- Calculer les bornes d'un graphe -----------------------------------------\
 |
 | graphe : le graphe  traiter
 | en_x   : NE_PAS_FAIRE = on ne les cherche pas
 |          ORDONNE      = ils sont tris par ordre croissant
 |          DESORDONNE   = ils ne sont pas tris
 | en_y   : mme chose mais pour les Y
 \---------------------------------------------------------------------------------------- */

void GRF_bornes(Graphe *graphe, int en_x, int en_y)
{
  unsigned long i;
  register double *x;
  register Complexe *y;
  double module;

  if (graphe == NULL)    return; /* Pas de graphe */
  if (graphe->x == NULL) return; /* Pas de points dans le graphe */
  if (graphe->y == NULL) return;

  x = graphe->x;

  if ( graphe->drapeaux.complexe == GR_NORMAL ) /* Graphe rel, normal donc */
  {
    double y_min, y_max, x_min, x_max;
    register double *y = graphe->y;

    if (en_x == DESORDONNE) /* Les X ne sont pas tris */
    {
      if (en_y == ORDONNE) /* Les Y sont tris */
      {
	x_min = *x; x_max = *x; x++;
	for (i = 1 ; i < graphe->nombre_point ; i++, x++, y++)
	{
	  if ( *x < x_min ) x_min = *x;
	  if ( *x > x_max ) x_max = *x;
	}
	graphe->x_min = x_min;
	graphe->x_max = x_max;

	graphe->y_min = y[0];
	graphe->y_max = y[graphe->nombre_point - 1];
      }
      else if (en_y == DESORDONNE) /* Ni les X, ni les Y ne sont tris */
      {
	x_min = *x; x_max = *x; x++;
	y_min = *y; y_max = *y; y++;
	for (i = 1 ; i < graphe->nombre_point ; i++, x++, y++)
	{
	  if ( *x < x_min ) x_min = *x;
	  if ( *x > x_max ) x_max = *x;
	  if ( *y < y_min ) y_min = *y;
	  if ( *y > y_max ) y_max = *y;
	}
	graphe->x_min = x_min;
	graphe->x_max = x_max;
	graphe->y_min = y_min;
	graphe->y_max = y_max;
      }
    }
    else /* Les X sont dj tris, o on ne s'en occupe pas */
    {
      if ( en_x == ORDONNE ) /* Les X sont dj tris */
      {
	graphe->x_min = x[0];
	graphe->x_max = x[graphe->nombre_point - 1];
      }

      if ( en_y == ORDONNE ) /* Les Y sont aussi prtris */
      {
	graphe->y_min = *y;
	graphe->y_max = *(y + graphe->nombre_point - 1);
      }
      else if (en_y == DESORDONNE)
      {
	y_min = *y; y_max = y_min; y++;
	for (i = 1 ; i < graphe->nombre_point ; i++, y++)
	{
	  if ( *y < y_min ) y_min = *y;
	  if ( *y > y_max ) y_max = *y;
	}
	graphe->y_min = y_min;
	graphe->y_max = y_max;
      }
    }
	
    return; /* Fini */
  }

  /* Si on est l, c'est que le graphe est complexe */

  y = (Complexe *) graphe->y;

  if (en_x == DESORDONNE) /* Les X ne sont pas tris */
  {
    if (en_y == ORDONNE) /* Les Y sont tris, pas les X */
    {
      y += graphe->nombre_point - 1;
      graphe->y_max = sqrt( y->re * y->re + y->im * y->im );
      graphe->y_min = -graphe->y_max;
      y++;

      graphe->x_min = graphe->x[0]; graphe->x_max = graphe->x[0]; x++;
      for (i = 1 ; i < graphe->nombre_point ; i++, x++)
      {
	if (graphe->x_min > *x) graphe->x_min = *x;
	if (graphe->x_max < *x) graphe->x_max = *x;
      }
    }
    else if (en_y == DESORDONNE) /* X et Y ne sont pas tris */
    {
      graphe->x_min = *x; graphe->x_max = graphe->x_min; x++;
      graphe->y_max = y->re * y->re + y->im * y->im;     y++;

      for (i = 1 ; i < graphe->nombre_point ; i++, x++, y++)
      {
	module = y->re * y->re + y->im * y->im;
	if (module > graphe->y_max) graphe->y_max = module;
	if (*x < graphe->x_min) graphe->x_min = *x;
	if (*x > graphe->x_max) graphe->x_max = *x;
      }
      graphe->y_max = sqrt(graphe->y_max);
      graphe->y_min = -graphe->y_max;
    }
  }
  else /* X tris ou ne nous intressent pas */
  {
    if (en_x == ORDONNE) /* Les X sont dj tris */
    {
      graphe->x_min = x[0];
      graphe->x_max = x[ graphe->nombre_point - 1];
    }

    if (en_y == ORDONNE) /* Les Y sont aussi dj tris */
    {
      y += graphe->nombre_point - 1;
      graphe->y_max = sqrt( y->re * y->re + y->im * y->im );
      graphe->y_min = -graphe->y_max;
    }
    else if (en_y == DESORDONNE) /* Les Y ne sont pas tris */
    {
      graphe->y_max = y->re * y->re + y->im * y->im;
      y++;
      for (i = 1 ; i < graphe->nombre_point ; i++, y++)
      {
	module = y->re * y->re + y->im * y->im;
	if (module > graphe->y_max) graphe->y_max = module;
      }
      graphe->y_max = sqrt(graphe->y_max);
      graphe->y_min = -graphe->y_max;
    }

    return;
  }
}

/* -------------- Tri de graphes, fusion de graphes,... ---------------------------------- *
 *******************************************************************************************/

/* -------- Appel des fonctions de tri, pour un seul graphe ------------------------------ */

boolean tri_graphe(Graphe *graphe)
{
  OBJECT    *adresse = arbres[ORDONNE_GRAPHE].adresse;
  TRI_infos *blocs;
  short int  nbr_blocs, index;
  double    *x_tri,*y_tri; /* Position des points tris */
  int        retour;
  boolean    redessin = FAUX; /* VRAI -> on a fait quelque chose */

  if (graphe == NULL) return FAUX; /* Pas de graphe */

  retour = ext_formdo(ORDONNE_GRAPHE);
  if (retour == TRI_ANNULE)
    return FAUX;                   /* On annule     */

  if ( OB_STATE(adresse, TRI_TRI_OK) & CHECKED ) /* On veut trier le graphe */
  {
    x_tri = (double *) malloc( sizeof(double) * graphe->nombre_point);
    y_tri = (double *) malloc( sizeof(double) * graphe->nombre_point);

    if ( (x_tri == NULL) || (y_tri == NULL) )
    {
      ERREUR(MANQUE_MEMOIRE);
      free(x_tri);
      free(y_tri);
      return FAUX;
    }

    if ( OB_STATE(adresse,TRI_ALEATOIRE) & SELECTED ) /* Tri complet */
    {
      unsigned long index = 1, i;
      double *temp;

      while (index < graphe->nombre_point)
      {
	for (i = 0 ; i < graphe->nombre_point ; i++)
	{
	  tri_fusion(graphe, x_tri, y_tri, i, i+index, i+2*index);
	}
	index *= 2;

	temp = x_tri; x_tri = graphe->x; graphe->x = temp;
	temp = y_tri; y_tri = graphe->y; graphe->y = temp;
      }

      free(x_tri); free(y_tri); /* Plus besoin de cela */
      redessin = VRAI;          /* On redessine        */
    }
    else /* Tri par blocs */
    {
      unsigned long nbr_pts; /* Nombre de points par bloc */

      nbr_blocs = prend_valeur(ORDONNE_GRAPHE,TRI_NBR_BLOCS);
      if (graphe->nombre_point % nbr_blocs)
	/* % = reste de la division; non-nul => pas assez de points pour avoir tous ces blocs */
      {
	form_alert(1,"[3][ Le graphe ne peut pas contenir | autant de blocs ! Il n'y a pas | le bon nombre de points ][ Annuler ]");
	free(y_tri);
	free(x_tri);
	return FAUX;
      }
      nbr_pts = graphe->nombre_point / nbr_blocs;

      blocs = (TRI_infos *) malloc( sizeof(TRI_infos) * nbr_blocs );
      if (blocs == NULL)
      {
	ERREUR(MANQUE_MEMOIRE);
	free(x_tri);
	free(y_tri);
	return FAUX;
      }

      for (index = 0 ; index < nbr_blocs ; index++)
      {
	blocs[index].xpoint_depart = graphe->x + index * nbr_pts;
	blocs[index].ypoint_depart = graphe->y + index * nbr_pts;
	blocs[index].position = 0;
	blocs[index].nbr_points = nbr_pts;
      }

      tri_blocs(x_tri,y_tri,blocs,nbr_blocs,graphe->nombre_point);
      
      free(blocs);        /* On libre ce que l'on a utilis    */
      
      free(graphe->x);    /* On efface l'ancienne version       */
      free(graphe->y);
      graphe->x = x_tri;  /* On met la version trie  la place */
      graphe->y = y_tri;
      redessin = VRAI; /* On redessine le graphe */
    }
  }

  /* ~~~~~~~~~~~~~~~ limination des doublons ~~~~~~~~~~~~~~~~~~~~~~~~~~ */

  /* Cas des regroupements par paquet de n X */

  if ( OB_STATE(adresse, TRI_GROUPER_OK) & CHECKED )
  {
    int nbr_x;

    nbr_x = prend_valeur(ORDONNE_GRAPHE, TRI_GROUPE_PARAM);
    DBL_groupe(graphe, OB_STATE(adresse,TRI_MOY_DOUBLON) & SELECTED, nbr_x);

    GRF_bornes(graphe, DESORDONNE, DESORDONNE);
    redessin = VRAI;
  }

  /* Cas des doublons dfinis par x < qq chose */

  if ( OB_STATE(adresse,TRI_DOUBLES_OK) & CHECKED )
  {
    unsigned long nbr_doublons;
    double difference;

    difference = prend_fvaleur(ORDONNE_GRAPHE, TRI_PARAM_DOUBLE);
    nbr_doublons = DBL_diff(graphe, OB_STATE(adresse,TRI_MOY_DOUBLON) & SELECTED, difference);
    if (nbr_doublons > 0) /* Des doublons => les bornes ne sont plus bonnes */
    {
      sprintf(_intermediaire,"[3][ On a limin (moyenn) | %ld points | carts de moins de | %f en X ][ Bien ]", nbr_doublons, difference);
      form_alert(1,_intermediaire);
      
      GRF_bornes(graphe, DESORDONNE, DESORDONNE); /* Bornes ont pu changer... */
      redessin = VRAI;
    }
    else if (nbr_doublons == 0)
    {
      form_alert(1,"[3][ Ce graphe ne contenait | aucun doublon ][Chouette]");
    }
  }

  /* Cas des doublons simples */

  if ( OB_STATE(adresse,TRI_DOUBLON_OK) & CHECKED )
  {
    unsigned long nbr_doublons;

    nbr_doublons = tri_doublons(graphe, OB_STATE(adresse,TRI_MOY_DOUBLON) & SELECTED );
    if (nbr_doublons > 0) /* Des doublons => les bornes ne sont plus bonnes */
    {
      sprintf(_intermediaire,"[3][ On a limin (moyenn) | %ld doublons ][ Bien ]", nbr_doublons);
      form_alert(1,_intermediaire);

      GRF_bornes(graphe, DESORDONNE, DESORDONNE); /* Bornes ont pu changer... */
      redessin = VRAI;
    }
    else
    {
      form_alert(1,"[3][ Ce graphe ne contenait | aucun doublon ][Chouette]");
    }
  }

  if (redessin == VRAI) /* On a modifi le graphe */
  {
    FEN_redessin(principale);
    FEN_redessin(trace_fenetre);
  }

  return VRAI; /* Fini */
}

/* -------------- Tri total d'un graphe, sur place, mthode de permutations ------------------\
 |      Suppose que le graphe est presque tri pour tre efficace                             |
 |                                                                                            |
 | graphe : le graphe que l'on veut trier                                                     |
 \------------------------------------------------------------------------------------------- */

void tri_en_place(Graphe *graphe)
{
  unsigned long index;
  register double *x, *x_suivant;
  boolean non_trie;

  if (graphe == NULL) return; /* Pas de tri si pas de graphe... */

  do
  {
    non_trie = FAUX; /* On recommence un tour... */
    x = graphe->x; x_suivant = x + 1;
    for (index = 1 ; index < graphe->nombre_point ; index++, x++, x_suivant++)
    {
      if (*x_suivant < *x) /* Problme de tri */
      {
	double ech_x = *x_suivant;

	*x_suivant = *x; *x = ech_x; /* On change les X */
	non_trie = VRAI; /* Le spectre n'tait pas tri */

	if (graphe->drapeaux.complexe) /* Graphe complexe */
	{
	  Complexe y = ((Complexe *) graphe->y)[index];
					
	  ((Complexe *) graphe->y)[index]     = ((Complexe *) graphe->y)[index - 1];
	  ((Complexe *) graphe->y)[index - 1] = y;
	}
	else
	{
	  double y = (graphe->y)[index];
	  (graphe->y)[index] = (graphe->y)[index - 1];
	  (graphe->y)[index - 1] = y;
	}
      }
    }
  } while (non_trie == VRAI);
}

/* -------------- Tri total d'un graphe (mthode du tri par fusion) --------------------------\
 |    Code adapt de l'exemple fourni par Bruno Blanchet                                      |
 |
 | graphe     = le graphe  trier 											                                      |
 | nv_x, nv_y = adresse o crire les donnes tries                                          |
 | i1, i2  = bornes du premier morceau  fusionner (fusionne [i1, i2[ et [i2, i3[)            |
 | i2, i3  = bornes du second morceau  fusionner                                             |
 \------------------------------------------------------------------------------------------- */

void tri_fusion(Graphe *graphe, double *x_tri, double *y_tri, unsigned long i1, unsigned long i2, unsigned long i3)
{
  unsigned long i,j,k;
  double x_1, x_2, y_1, y_2;

  /* On vrifie qu'on ne dpasse pas le tableau */
  if (i2 > graphe->nombre_point) i2=graphe->nombre_point;
  if (i3 > graphe->nombre_point) i3=graphe->nombre_point;
 
  i = i1; j = i2;
  if (i < i2)
  {
    x_1 = graphe->x[i];
    y_1 = graphe->y[i];
  }
  else
  {
    x_1 = HUGE_VAL;
    y_1 = HUGE_VAL;
  }
  if (j < i3)
  {
    x_2 = graphe->x[j];
    y_2 = graphe->y[j];    
  }
  else
  {
    x_2 = HUGE_VAL;
    y_2 = HUGE_VAL;
  }

  for (k = i1 ; k < i3 ; k++)
  {
    if (x_1 < x_2)
		{
		  x_tri[k] = x_1;
		  y_tri[k] = y_1;
		  if (++i < i2)
		  {
		    x_1 = graphe->x[i];
    		y_1 = graphe->y[i];
		  }
	  	else
		  {
		    x_1 = HUGE_VAL;
    		y_1 = HUGE_VAL;
		  }
		}
		else
		{
	  	x_tri[k] = x_2;
	  	y_tri[k] = y_2;
		  if (++j<i3)
		  {
    		x_2 = graphe->x[j];
		    y_2 = graphe->y[j];    
		  }
	  	else
		  {
		    x_2 = HUGE_VAL;
    		y_2 = HUGE_VAL;
		  }
		}
    k++;
  }
}

/* -------------- Fusion de n graphes dj en mmoire (appel) ---------------------------- */

Graphe *fondre(void)
{
  Graphe *resultat,parametres;
  OBJECT *adresse = arbres[FUSION_GRAPHES].adresse;
  TRI_infos *infos_tri;
  int renvoi;
  unsigned long total_points = 0;
  unsigned long index,compteur;
  unsigned long nbr_a_fondre = 0; /* Nombre de graphes  fusionner */
  struct { unsigned long index_dep;
	   boolean *a_fondre;
	   Graphe  *prem_graphe; /* Premier graphe visible dans la liste */
	 } infos_ascenseur;

  infos_ascenseur.a_fondre = (boolean *) malloc( sizeof(boolean) * nbr_graphes );
  if (infos_ascenseur.a_fondre == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return NULL; /* Aucun graphe cr */
  }

  /* Prparation de l'ascenseur (aucun graphe n'est slectionn) */
  infos_ascenseur.index_dep = 0; /* On commence au graphe 1 */
  for (index = 0 ; index < nbr_graphes ; infos_ascenseur.a_fondre[index++] = FAUX);
  
  resultat = premier_graphe;
  infos_ascenseur.prem_graphe = resultat;
  for (index = 0 ; index < 11 ; index++)
  {
    if (resultat) /* Tant que ce n'est pas le dernier... */
    {
      strncpy((char *) OB_SPEC(adresse,FUSION_GR1 + index),resultat->nom,50);
      resultat = resultat->suivant;
      OB_STATE(adresse,FUSION_GR1 + index) &= ~DISABLED;
    }
    else
    {
      ((char *) OB_SPEC(adresse,FUSION_GR1 + index))[0] = 0;
      OB_STATE(adresse, FUSION_GR1 + index) |=  DISABLED;
    }
    OB_STATE(adresse,FUSION_GR1 + index) &= ~SELECTED; /* Non choisi par dfaut */
  }

  OB_STATE(adresse,FUSION_AVANT)  |= DISABLED;
  OB_Y(adresse,FUSION_BOUTON) = 0; /* Ascenseur en haut pour dmarrer                  */
  if (nbr_graphes <= 11)           /* Moins de 11 graphes -> pas besoin de l'ascenseur */
  {
    OB_STATE(adresse,FUSION_APRES)  |= DISABLED;
    OB_STATE(adresse,FUSION_FOND)   |= DISABLED;
    OB_HEIGHT(adresse,FUSION_BOUTON) = OB_HEIGHT(adresse,FUSION_FOND);
    OB_STATE(adresse,FUSION_BOUTON) |= DISABLED;
  }
  else /* Mise en place de l'ascenseur */
  {
    float taille_asc;

    OB_STATE(adresse,FUSION_APRES)  &= ~DISABLED;
    OB_STATE(adresse,FUSION_FOND)   &= ~DISABLED;
    OB_STATE(adresse,FUSION_BOUTON) &= ~DISABLED;

    taille_asc  = OB_HEIGHT(adresse,FUSION_FOND) * 11;
    taille_asc /= nbr_graphes;
    OB_HEIGHT(adresse,FUSION_BOUTON) = (int) taille_asc;
  }

  arbres[FUSION_GRAPHES].pointeur = &infos_ascenseur;

#ifdef _FRANCAIS_
  met_chaine(FUSION_GRAPHES, FUSION_TITRE, 0, "Choisir les spectres "a_GRAVE" fusionner", TXT_REMPLACE);
  met_chaine(FUSION_GRAPHES, FUSION_OK, 0, "Fusionner", TXT_REMPLACE);
#endif

#ifdef _ENGLISH_
  met_chaine(FUSION_GRAPHES, FUSION_TITRE, 0, "Choose the spectra you want to merge", TXT_REMPLACE);
  met_chaine(FUSION_GRAPHES, FUSION_OK, 0, "Merge", TXT_REMPLACE);
#endif

  OB_FLAGS(adresse, FUS_DOUBLE_NOM)  &= ~HIDETREE;
  OB_FLAGS(adresse, FUS_DOUBLE_FOND) &= ~HIDETREE;

  renvoi = ext_formdo(FUSION_GRAPHES);
  if (renvoi == FUSION_ANNULE) /* On annule la fusion */
  {
    free(infos_ascenseur.a_fondre); /* On libre la mmoire */
    return NULL;
  }

  for (index = 0 ; index < nbr_graphes ; index++)
  {
    if ( (infos_ascenseur.a_fondre)[index] == VRAI)
      nbr_a_fondre++;
  }

  if (nbr_a_fondre < 2)
  {
    form_alert(1,"[3][ La fusion de moins de 2 graphes | ne peut tre ralise. Calcul annul ][ Logique ]");
    free(infos_ascenseur.a_fondre); /* On libre la mmoire */
    return NULL;
  }

  infos_tri = (TRI_infos *) malloc( sizeof(TRI_infos) * nbr_a_fondre );
	if (infos_tri == NULL)
	{
	  ERREUR(MANQUE_MEMOIRE);
	  free(infos_ascenseur.a_fondre); /* On libre la mmoire */
	  return NULL;
	}

	/* On prpare les blocs  trier */
	
	resultat = premier_graphe;
	compteur = 0; /* Les graphes slectionns */
	index = 0;
	while ( (infos_ascenseur.a_fondre)[index] == FAUX) /* Recherche du premier graphe  fondre */
	{
	  index++;
	  resultat = resultat->suivant;
	  if (resultat == NULL) /* Dernier graphe */
	  {
	    form_alert(1,"[2][ Aucun graphe slectionn ! | Fusion impossible ][ Annuler ]");
	    free(infos_ascenseur.a_fondre); /* On libre la mmoire */
	    free(infos_tri);
	    return NULL;
	  }
	}
	parametres.x_min = resultat->x_min; parametres.x_max = resultat->x_max;
	parametres.y_min = resultat->y_min; parametres.y_max = resultat->y_max;
	parametres.type = resultat->type;
	parametres.drapeaux.complexe = resultat->drapeaux.complexe;
	strcpy(parametres.axe_x, resultat->axe_x);
	strcpy(parametres.axe_y, resultat->axe_y);

	while ( index < nbr_graphes )
	{
	  if ( (infos_ascenseur.a_fondre)[index] == VRAI)
	  {
	    total_points += resultat->nombre_point; /* Nombre de points du total */
	    infos_tri[compteur].xpoint_depart = resultat->x;
	    infos_tri[compteur].ypoint_depart = resultat->y;
	    infos_tri[compteur].position   = 0;
	    infos_tri[compteur].nbr_points = resultat->nombre_point;
	    compteur++;
			
	    if (parametres.x_min > resultat->x_min) parametres.x_min = resultat->x_min;
	    if (parametres.x_max < resultat->x_max) parametres.x_max = resultat->x_max;
	    if (parametres.y_min > resultat->y_min) parametres.y_min = resultat->y_min;
	    if (parametres.y_max < resultat->y_max) parametres.y_max = resultat->y_max;

	    if ( resultat->drapeaux.complexe != parametres.drapeaux.complexe )
	    {
	      printf("Fusion de graphes complexes et rels impossible. Opration annule\n");
	      free(infos_ascenseur.a_fondre);
	      free(infos_tri);
	      return NULL;
	    }

	    if ( ( parametres.type != resultat->type ) && (resultat->type != -1) )
	    {
	      if (parametres.type == -1) 
		parametres.type = resultat->type;
	      else
	      {
		renvoi = form_alert(1,"[2][ Les graphes ne sont pas tous | de mme type !  Le rsultat | pourra tre trange... ][Calculer|Annuler]");
		if (renvoi == 2)
		{
		  free(infos_ascenseur.a_fondre); /* On libre la mmoire */
		  free(infos_tri);
		  return NULL;
		}
	      }
	    }
	  }
	  resultat = resultat->suivant;
	  index++;
	}

	sprintf(_intermediaire,"Fusion de %ld graphes%c",nbr_a_fondre,0);

	resultat = GRF_creer(total_points, parametres.type, parametres.drapeaux.complexe, _intermediaire,
			     parametres.axe_x, parametres.axe_y, parametres.unit_x, parametres.unit_y);
	if (resultat == NULL)
	{
	  free(infos_ascenseur.a_fondre); /* On libre la mmoire */
	  free(infos_tri);
	  return NULL;
	}

	tri_blocs(resultat->x,resultat->y,infos_tri,nbr_a_fondre,total_points);

	/* Reste l'limination des doublons */
	if ( !( OB_STATE(adresse,FUS_GARDER) & SELECTED ) )
	{
	  compteur = tri_doublons(resultat, OB_STATE(adresse,FUS_MOYENNE) & SELECTED );

	  if (compteur > 0) /* Des doublons => les bornes calcules ne conviennent pas */
	  {
	    /* On recherche les nouvelles bornes en X et en Y */

	    parametres.x_min = *(resultat->x);
	    parametres.x_max = *(resultat->x);
	    parametres.y_min = *(resultat->y);
	    parametres.y_max = *(resultat->y);
			
	    for (index = 1 ; index < resultat->nombre_point ; index++)
	    {
	      if (parametres.x_min > *(resultat->x + index) ) parametres.x_min = *(resultat->x + index);
	      if (parametres.x_max < *(resultat->x + index) ) parametres.x_max = *(resultat->x + index);
	      if (parametres.y_min > *(resultat->y + index) ) parametres.y_min = *(resultat->y + index);
	      if (parametres.y_max < *(resultat->y + index) ) parametres.y_max = *(resultat->y + index);
	    }
	  }
	}

	resultat->x_min = parametres.x_min;
	resultat->x_max = parametres.x_max;
	resultat->y_min = parametres.y_min;
	resultat->y_max = parametres.y_max;


	strcpy(resultat->axe_x,parametres.axe_x);
	strcpy(resultat->axe_y,parametres.axe_y);

	free(infos_ascenseur.a_fondre); /* On libre la mmoire */
  free(infos_tri);
	return resultat;
}

/* -------------- La fusion effective (aussi utilise pour le tri par blocs) ------------- */

void tri_blocs(double *x_tri, double *y_tri, TRI_infos *blocs, unsigned long nbr_blocs, unsigned long total)
{
  double x_min;
  unsigned long index, num_bloc, num_mini;

  if ( (x_tri == NULL) || (y_tri == NULL) )
  {
    form_alert(1,"[2][ Pas de zone destination prcise | La fusion (le tri) est impossible ][ Annuler ]");
    return;
  }

  for (index = 0 ; index < total ; index++)
  {
    /* On cherche le premier bloc non termin */
    num_mini = 0;
    while ( blocs[num_mini].position >= blocs[num_mini].nbr_points )
    {
      num_mini++;
      if (num_mini >= nbr_blocs)
      {
	form_alert(1,"[2][ Erreur survenue pendant la fusion | Le graphe obtenu est imprvisible ][ Ah ? ]");
	return;
      }
    }

    /* Point courant du premier bloc non-termin */
    
    x_min = *(blocs[num_mini].xpoint_depart + blocs[num_mini].position);

    /* On recherche le plus petit point des blocs restants */

    for (num_bloc = num_mini ; num_bloc < nbr_blocs ; num_bloc++)
    {
      if ( blocs[num_bloc].position < blocs[num_bloc].nbr_points ) /* Il reste des points  lire */
      {
	if ( *(blocs[num_bloc].xpoint_depart  + blocs[num_bloc].position) < x_min ) /* Premier point du bloc n */
	{
	  num_mini = num_bloc;
	  x_min = *(blocs[num_bloc].xpoint_depart + blocs[num_bloc].position);
	}
      }
    }
    
    /* Maintenant, x_min contient la valeur minimale et num_mini le numro du bloc associ */

    *x_tri = x_min;                          /* On enregistre le premier point */
    *y_tri = *(blocs[num_mini].ypoint_depart + blocs[num_mini].position);
    blocs[num_mini].position++; /* On a lu un point de ce bloc */
	
    x_tri++; y_tri++; /* On va s'intresser au point suivant */
  }
}

/* -------------- L'limination des doublons ------------------------------------------- *\
|    Suppose le graphe tri, donc les doublons forcment  la suite                       |
|    Renvoi le nombre de doublons trouvs                                                 |
|                                                                                         |
| graphe   : le graphe dont on veut liminer les doublons                                 |
| moyenner : VRAI si on veut moyenner les doublons, FAUX si on veut juste les liminer    |
\* ------------------------------------------------------------------------------------- */

unsigned long tri_doublons(Graphe *graphe, boolean moyenner)
{
  register double *x_lu,*x_ecrit;
  double *y_lu,*y_ecrit,x_ref,y_ref;
  unsigned long i = 1,        /* Index sur les points _lus_, le premier l'est toujours... */
                nbr_doublons;
  unsigned long nbr_points,total_doublons = 0;

  nbr_points = graphe->nombre_point;
  x_ref = *(graphe->x); y_ref = *(graphe->y); /* Premier point du graphe */

  if ( moyenner == FAUX ) /* On limine purement et simplement les doublons -> trs simple */
  {
    x_lu = graphe->x + 1;    y_lu = graphe->y + 1;      /* Second point  lire    */
    x_ecrit = graphe->x + 1;  y_ecrit = graphe->y + 1;  /* Premier point  crire - le premier est conserv */

    while (i < graphe->nombre_point)
    {
      while ( *x_lu != x_ref ) /* Tant que le point est diffrent */
      {
	*x_ecrit = *x_lu; /* On recopie le point  sa place */
	*y_ecrit = *y_lu;
	x_ref = *x_lu;    /* On se dcale d'un cran         */
	x_lu++; y_lu++;   /* On se place pour lire le point suivant */
	x_ecrit++; y_ecrit++;
	i++;
	if (i >= graphe->nombre_point)
	{
	  graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
	  return total_doublons;
	}
      }
			
      /* Si on est l, c'est que l'on a trouv un doublon */
      total_doublons++;
			
      while (*x_lu == x_ref)
      {
	x_lu++; y_lu++; /* On lit le point suivant */
	nbr_points--;   /* Un point en moins       */
	i++;
	if (i >= graphe->nombre_point)
	{
	  graphe->nombre_point = nbr_points; /* L'idal serait ensuite de librer la mmoire en trop */
	  return total_doublons;
	}
      }
    }

    graphe->nombre_point = nbr_points; /* L'idal serait ensuite de librer la mmoire en trop */
    return total_doublons; /* On a fini */
  }

  x_lu    = graphe->x + 1;   y_lu = graphe->y + 1;
  x_ecrit = graphe->x;    y_ecrit = graphe->y;
  i = 1; /* Index : compteur sur ce qui est _lu_ -> commence  UN */
  while (i < graphe->nombre_point)
  {
    while (*x_lu != x_ref) /* Tant que le point est diffrent */
    {
      *x_ecrit = x_ref; /* On recopie le point  sa place */
      *y_ecrit = y_ref;

      x_ref = *x_lu;    /* On recopie le nouveau point comme rfrence */
      y_ref = *y_lu;

      x_ecrit++; y_ecrit++; /* On se place pour crire le point suivant */
      i++;

      if (i >= graphe->nombre_point)
      {
	*x_ecrit = x_ref;
	*y_ecrit = y_ref;
	graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
	return total_doublons;
      }

      x_lu++; y_lu++;   /* On se place pour lire le point suivant, s'il y en a un... */
    }

    /* Si on est l, c'est que l'on a trouv un doublon */
    total_doublons++;
    nbr_doublons = 1;
    while (*x_lu == x_ref)
    {
      nbr_points--;   /* Un point en moins       */
      nbr_doublons++; /* Mais une copie du doublon en plus */
      y_ref += *y_lu;
      x_lu++; y_lu++; /* On lit le point suivant */
      i++;
      if (i >= graphe->nombre_point)
      {
	*x_ecrit = x_ref;
	*y_ecrit = y_ref / ((double) nbr_doublons); /* On fait la moyenne */
	graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
	return total_doublons;
      }
    }

    *x_ecrit = x_ref;
    *y_ecrit = y_ref / ((double) nbr_doublons); /* On fait la moyenne */

    x_ecrit++; y_ecrit++; /* On passe au point suivant */
    x_ref = *x_lu;
    y_ref = *y_lu;
    x_lu++; y_lu++;
    i++; /* On a lu encore un point... */
  }

  *x_ecrit = x_ref;
  *y_ecrit = y_ref; /* On crit le doublon - ici, arrive que si dernier point unique et 1 doublon juste avant */

  graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
  return total_doublons;
}

/* -------------- limination des doublons : avec critre de diffrence ---------------- *\
|    Suppose le graphe tri, donc les "doublons" forcment  la suite                     |
|    Renvoi le nombre de doublons trouvs                                                 |
|                                                                                         |
| difference : si Dx < cette valeur, alors c'est un doublon                               |
| graphe     : le graphe dont on veut liminer les doublons                               |
| moyenner   : VRAI si on veut moyenner les doublons, FAUX si on veut juste les liminer  |
\* ------------------------------------------------------------------------------------- */

unsigned long DBL_diff(Graphe *graphe, boolean moyenner, double difference)
{
  register double *x_lu,*x_ecrit;
  double *y_lu,*y_ecrit,x_ref,y_ref,x;
  unsigned long i = 1,nbr_doublons;
  unsigned long nbr_points,total_doublons = 0;

  if (graphe == NULL) return -1; /* Forcment... */
  if (difference < 0) difference *= -1; /* On travaille en valeur absolue */
  
  if (difference == 0) return tri_doublons(graphe, moyenner); /* Plus rapide... */

  if (difference >= (graphe->x_max - graphe->x_min) )
  {
    form_alert(1,"[3][ Le critre choisi conduirait |  regrouper tous les points | en un seul ! ][Annuler]");
    return -2;
  }

  nbr_points = graphe->nombre_point;
  x_ref = *(graphe->x); y_ref = *(graphe->y); /* Premier point du graphe */

  if ( moyenner == FAUX ) /* On limine purement et simplement les doublons -> trs simple */
  {
    x_lu = graphe->x + 1;    y_lu = graphe->y + 1;     /* Second point  lire */
    x_ecrit = graphe->x + 1;  y_ecrit = graphe->y + 1;

    while (i < graphe->nombre_point)
    {
      while ( fabs( (graphe->x)[i] -= x_ref ) > difference ) /* Tant que le point est diffrent */
      {
	*x_ecrit = *x_lu; /* On recopie le point  sa place */
	*y_ecrit = *y_lu;
	x_ref = (graphe->x)[i];    /* On se dcale d'un cran         */
	x_lu++; y_lu++;   /* On se place pour lire le point suivant */
	x_ecrit++; y_ecrit++;
	i++;
	if (i >= graphe->nombre_point)
	{
	  graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
	  return total_doublons;
	}
      }
      
      /* Si on est l, c'est que l'on a trouv un doublon */
      total_doublons++;
			
      while ( fabs(*x_lu - x_ref) <= difference )
      {
	x_lu++; y_lu++; /* On lit le point suivant */
	nbr_points--;   /* Un point en moins       */
	i++;
	if (i >= graphe->nombre_point)
	{
	  graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
	  return total_doublons;
	}
      }
    }

    graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
    return total_doublons; /* On a fini */
  }

  /* Si on est l, c'est qu'on veut moyenner - donc en X et en Y... */
  
  x_lu    = graphe->x + 1;   y_lu = graphe->y + 1;
  x_ecrit = graphe->x;    y_ecrit = graphe->y;
  i = 1;
  while (i < graphe->nombre_point)
  {
    while ( fabs(*x_lu - x_ref) > difference ) /* Tant que le point est diffrent */
    {
      *x_ecrit = x_ref; /* On recopie le point  sa place */
      *y_ecrit = y_ref;
      x_ref = *x_lu;    /* On se dcale d'un cran         */
      y_ref = *y_lu;
      x_lu++; y_lu++;   /* On se place pour lire le point suivant */
      x_ecrit++; y_ecrit++;
      i++;
      if (i >= graphe->nombre_point)
      {
	*x_ecrit = x_ref;
	*y_ecrit = y_ref;
	graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
	return total_doublons;
      }
    }
    
    /* Si on est l, c'est que l'on a trouv un doublon */
    total_doublons++;
    nbr_doublons = 1;
    x = x_ref;
    while ( fabs(*x_lu - x_ref) <= difference ) /* Appartient toujours au doublon */
    {
      nbr_points--;   /* Un point en moins       */
      nbr_doublons++; /* Mais une copie du doublon en plus */
      x += *x_lu;      /* Pour la moyenne des X aussi */
      y_ref += *y_lu;
      x_lu++; y_lu++; /* On lit le point suivant */
      i++;
      if (i >= graphe->nombre_point)
      {
	*x_ecrit =     x / ((double) nbr_doublons);
	*y_ecrit = y_ref / ((double) nbr_doublons); /* On fait la moyenne */
	graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
	return total_doublons;
      }
    }
    
    *x_ecrit =     x / ((double) nbr_doublons);
    *y_ecrit = y_ref / ((double) nbr_doublons); /* On fait la moyenne */
    x_ecrit++; y_ecrit++;
    x_ref = *x_lu;
    y_ref = *y_lu;
    x_lu++; y_lu++;
    i++;
  }

  graphe->nombre_point = nbr_points; /* Idal serait ensuite de librer la mmoire en trop */
  return total_doublons;
}

/* -------------- limination des doublons : groupe par paquets de n les X ------------- *\
|    Suppose le graphe tri, donc les "doublons" forcment  la suite                     |
|    Renvoi le nombre de doublons trouvs                                                 |
|                                                                                         |
| nbr_x    : il faut regrouper tous les nbr_x X conscutifs                               |
| graphe   : le graphe dont on veut liminer les doublons                                 |
| moyenner : VRAI si on veut moyenner les doublons, FAUX si on veut juste les liminer    |
\* ------------------------------------------------------------------------------------- */

unsigned long DBL_groupe(Graphe *graphe, boolean moyenner, int nbr_x)
{
  register double *x_lu,*x_ecrit;
  double *y_lu,*y_ecrit,x,y;
  unsigned long i = 0, nbr_points = 0;
  int k;

  if (graphe == NULL) return -1; /* Forcment... */
  if (nbr_x < 2)
  {
    EXF_ERREUR(VALEUR_ABSURDE);
    return -2;
  }

  if (nbr_x >= graphe->nombre_point) /* On regrouperait tous les points en un seul ! */
  {
    EXF_ERREUR(VALEUR_ABSURDE);
    return -3;
  }

  if (moyenner == FAUX) /* On ne veut pas moyenner : cas le plus simple... */
  {
    x_ecrit = graphe->x; y_ecrit = graphe->y;
    x_lu = graphe->x; y_lu = graphe->y;
    while (i < graphe->nombre_point)
    {
      *x_ecrit = *x_lu; /* On copie le premier point du couple */
      *y_ecrit = *y_lu;
      for (k = 0 ; k < nbr_x ; k++) /* On saute les points suivants */
      {
	x_lu++; y_lu++;
	i++;
      }
      x_ecrit++; y_ecrit++;
      nbr_points++; /* Un point de plus */
    }
  }
  else /* On veut moyenner les points -> plus compliqu */
  {
    x_ecrit = graphe->x; y_ecrit = graphe->y;
    x_lu = x_ecrit; y_lu = y_ecrit;
    while ( i < graphe->nombre_point )
    {
      x = 0; y = 0;
      for (k = 0 ; k < nbr_x ; k++)
      {
	x += *x_lu; y += *y_lu; /* On lit le point suivant */
	i++; /* On passe au point suivant */
	x_lu++; y_lu++;
	if (i >= graphe->nombre_point) /* On est  la fin */
	{
	  k++; /* Comme on ne repasse pas, k n'est pas correct sinon... */
	  break;
	}
      }

      if (k > 0)
      {
	*x_ecrit = x / k; /* On met la nouvelle valeur */
	*y_ecrit = y / k;
      }
      else
      {
	EXF_ERREUR(DIV_ZERO);
      }
      
      x_ecrit++; y_ecrit++; /* On passe au point suivant */
      nbr_points++;
    }
  }
  
  graphe->nombre_point = nbr_points; 
  return nbr_points;              /* Nouveau nombre de points */
}

/* -------------- Oprations sur plusieurs graphes --------------------------------------- *
 *******************************************************************************************/

/* ~~~~~~~~~~~~~~ Moyenne de plusieurs graphes ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* -------------- Moyenne : procdure d'appel, avec arbre GEM et tout et tout ------------ */

Graphe *moyenner(void)
{
	OBJECT *adresse = arbres[FUSION_GRAPHES].adresse;
	TRI_infos *infos_tri;
	Graphe *resultat = NULL;
	int renvoi, type;
	unsigned long total_points = 0;
	unsigned long i,compteur;
	unsigned long nbr_a_moyenner = 0; /* Nombre de graphes  moyenner */
	struct { unsigned long index_dep;
					 boolean *a_fondre;
	         Graphe  *prem_graphe; /* Premier graphe visible dans la liste */
	       } infos_ascenseur;

	infos_ascenseur.a_fondre = (boolean *) malloc( sizeof(boolean) * nbr_graphes );
	if (infos_ascenseur.a_fondre == NULL)
	{
		ERREUR(MANQUE_MEMOIRE);
		return NULL; /* Aucun graphe cr */
	}

	/* Prparation de l'ascenseur (aucun graphe n'est slectionn) */
	infos_ascenseur.index_dep = 0; /* On commence au graphe 1 */
	for (i = 0 ; i < nbr_graphes ; infos_ascenseur.a_fondre[i++] = FAUX);

	resultat = premier_graphe;
	infos_ascenseur.prem_graphe = resultat;
	for (i = 0 ; i < 11 ; i++)
	{
		if (resultat) /* Tant que ce n'est pas le dernier... */
		{
			strncpy((char *) OB_SPEC(adresse,FUSION_GR1 + i),resultat->nom,50);
			resultat = resultat->suivant;
			OB_STATE(adresse,FUSION_GR1 + i) &= ~DISABLED;
		}
		else
		{
	  	((char *) OB_SPEC(adresse,FUSION_GR1 + i))[0] = 0;
			OB_STATE(adresse,FUSION_GR1 + i) |= DISABLED;
	  }
		OB_STATE(adresse,FUSION_GR1 + i) &= ~SELECTED; /* Non choisi par dfaut */
	}

	OB_STATE(adresse,FUSION_AVANT)  |= DISABLED;
	OB_Y(adresse,FUSION_BOUTON) = 0; /* Ascenseur en haut pour dmarrer                  */
	if (nbr_graphes <= 11)           /* Moins de 11 graphes -> pas besoin de l'ascenseur */
	{
		OB_STATE(adresse,FUSION_APRES)  |= DISABLED;
		OB_STATE(adresse,FUSION_FOND)   |= DISABLED;
		OB_HEIGHT(adresse,FUSION_BOUTON) = OB_HEIGHT(adresse,FUSION_FOND);
		OB_STATE(adresse,FUSION_BOUTON) |= DISABLED;
	}
	else /* Mise en place de l'ascenseur */
	{
		float taille_asc;

		OB_STATE(adresse,FUSION_APRES)  &= ~DISABLED;
		OB_STATE(adresse,FUSION_FOND)   &= ~DISABLED;
		OB_STATE(adresse,FUSION_BOUTON) &= ~DISABLED;

		taille_asc  = OB_HEIGHT(adresse,FUSION_FOND) * 11;
		taille_asc /= nbr_graphes;
		OB_HEIGHT(adresse,FUSION_BOUTON) = (int) taille_asc;
	}

	arbres[FUSION_GRAPHES].pointeur = &infos_ascenseur;

#ifdef _FRANCAIS_
	OB_SPEC(adresse, FUSION_TITRE) = (long) "Choisir les spectres "a_GRAVE" moyenner";
	met_chaine(FUSION_GRAPHES, FUSION_OK, 0, "Moyenner", TXT_REMPLACE);
#endif
	OB_FLAGS(adresse, FUS_DOUBLE_NOM)  |= HIDETREE;
	OB_FLAGS(adresse, FUS_DOUBLE_FOND) |= HIDETREE;

	renvoi = ext_formdo(FUSION_GRAPHES);
	if (renvoi == FUSION_ANNULE) /* On annule la fusion */
	{
		free(infos_ascenseur.a_fondre); /* On libre la mmoire */
		return NULL;
	}

	for (i = 0 ; i < nbr_graphes ; i++)
	{
		if ( (infos_ascenseur.a_fondre)[i] == VRAI)
			nbr_a_moyenner++;
	}

	if (nbr_a_moyenner < 2)
	{
		form_alert(1,"[3][ Moyenner moins de 2 graphes | ne peut tre ralis. | Le calcul est annul. ][ Logique ]");
		free(infos_ascenseur.a_fondre); /* On libre la mmoire */
		return NULL;
	}

	infos_tri = (TRI_infos *) malloc( sizeof(TRI_infos) * nbr_a_moyenner );
	if (infos_tri == NULL)
	{
		ERREUR(MANQUE_MEMOIRE);
		free(infos_ascenseur.a_fondre); /* On libre la mmoire */
	  return NULL;
	}

	/* On prpare les blocs  moyenner */

	resultat = premier_graphe;
	compteur = 0; /* Les graphes slectionns */
	i = 0;
	while ( (infos_ascenseur.a_fondre)[i] == FAUX) /* Recherche du premier graphe  moyenner */
	{
		i++;
		resultat = resultat->suivant;
		if (resultat == NULL) /* Dernier graphe */
		{
			form_alert(1,"[2][ Aucun graphe slectionn ! | Moyenne impossible ][ Annuler ]");
			free(infos_ascenseur.a_fondre); /* On libre la mmoire */
	  	free(infos_tri);
			return NULL;
		}
	}
	type = resultat->type; /* Juste vrification des types... */
	total_points = resultat->nombre_point;

	while ( i < nbr_graphes )
	{
		if ( (infos_ascenseur.a_fondre)[i] == VRAI)
		{
			if (resultat->nombre_point > total_points)
				total_points = resultat->nombre_point; /* Nombre de points du total */

			infos_tri[compteur].xpoint_depart = resultat->x;
			infos_tri[compteur].ypoint_depart = resultat->y;
			infos_tri[compteur].position   = 0;
			infos_tri[compteur].nbr_points = resultat->nombre_point;
			compteur++;
			
			if ( ( type != resultat->type ) && (resultat->type != -1) )
			{
				if (type == -1) 
					type = resultat->type;
				else
				{
					renvoi = form_alert(1,"[2][ Les graphes ne sont pas tous | de mme type !  Le rsultat | pourra tre trange... ][Calculer|Annuler]");
					if (renvoi == 2)
					{
						free(infos_ascenseur.a_fondre); /* On libre la mmoire */
	  				free(infos_tri);
						return NULL;
					}
				}
			}
		}
		resultat = resultat->suivant;
		i++;
	}

	sprintf(_intermediaire,"Moyenne de %ld graphes", compteur);

	resultat = GRF_creer(total_points, type, GR_NORMAL, _intermediaire, "", "", 0, 0);
	if (resultat == NULL)
	{
	  free(infos_ascenseur.a_fondre); /* On libre la mmoire */
	  free(infos_tri);
	  return NULL;
	}

	/* On fait la moyenne effective */

	i = MOY_blocs(resultat->x, resultat->y, infos_tri, compteur, resultat->nombre_point);
	if (i > 0) resultat->nombre_point = i; /* Nombre de points rel */
	GRF_bornes(resultat, DESORDONNE, DESORDONNE);


	return resultat;
}

/* -------------- Moyennage des diffrents blocs ------------------------------------------\
 |   Renvoie le nombre de points effectivement moyenns                                    |
 |   Ne modifie pas le contenu des TRI_infos                                               |
 |                                                                                         |
 | x_moyen       = adresse o crire les X moyenns                                        |
 | y_moyen       = adresse o crire les Y moyenns                                        |
 | infos_moyenne = toutes les infos ncessaire au calcul                                   |
 | nbr_blocs     = nombre de blocs  moyenner                                              |
 | total_pts     = nombre total de points au maximum dans x_moyen/y_moyen                  |
 \---------------------------------------------------------------------------------------- */

unsigned long MOY_blocs(double *x_moyen, double *y_moyen, TRI_infos *infos_moyenne, unsigned long nbr_blocs, unsigned long total_pts)
{
	unsigned long index, compteur;
	unsigned long reel_nbr_blocs; /* Nombre rel de blocs dans la moyenne */
	unsigned long vx_blocs;
	double x, y;

	vx_blocs = nbr_blocs; /* Nombre attendu de blocs */
	for (index = 0 ; index < total_pts ; index++, x_moyen++, y_moyen++)
	{
		x = 0; y = 0;
		reel_nbr_blocs = 0; /* Aucun bloc ne convient */
		for (compteur = 0 ; compteur < nbr_blocs ; compteur++)
		{
			if ( index < infos_moyenne[compteur].nbr_points ) /* Ce bloc convient encore */
			{
				x += (infos_moyenne[compteur].xpoint_depart)[index];
				y += (infos_moyenne[compteur].ypoint_depart)[index];
				reel_nbr_blocs++;
			}
		}

		if ( reel_nbr_blocs == 0)
		{
			EXF_ERREUR(DIV_ZERO);
			return index; /* On s'arrte ici... */
		}
		else
		{
			if (vx_blocs != reel_nbr_blocs)
			{
				int i = form_alert(1,"[3][ Un des spectres est fini | avant les autres... | Que faire ? ][Continuer|Arrter]");
				if (i == 2) /* On arrte ici */
					return index;
			}
			*x_moyen = x / reel_nbr_blocs;
			*y_moyen = y / reel_nbr_blocs;
			vx_blocs = reel_nbr_blocs;
		}
	}

	return total_pts; /* On les a tous utiliss */
}

/* -------------- Paramtrisations diverses sur le trac ------------------------- *
 ***********************************************************************************/
 
/* ------------------ Prparation de l'affichage pour le choix des graphes --------------- */

void prep_geregraphe(boolean mode)
{
  OBJECT *adresse = arbres[CHOIX_GRAPHE].adresse;
  Graphe *graphe;
  char   *chaine;
  int     i;
  boolean etat;

  if (mode == VRAI) /* Il faut positionner l'ascenseur pour avoir le graphe courant visible */
  {
    if (nbr_graphes < 15) /* Moins de graphes que de ligne */
    {
      graphe = premier_graphe;
      for (i = 0 ; i < 14 ; i++ )
      {
      	if (graphe != NULL)
	      {
      	  liste[i] = graphe;
      	  graphe = graphe->suivant;
       	}
      	else
	        liste[i] = NULL;
      }
      /* Ascenseur vide... */
      OB_Y(adresse, GRAPHE_BOUTON) = 0;
      OB_HEIGHT(adresse, GRAPHE_BOUTON) = OB_HEIGHT(adresse, GRAPHE_FOND);
    }
    else
    {
      float inter;
      Graphe *graphe;
      unsigned long num_graphe; /* Numro du graphe courant */

      for (num_graphe = 1, graphe = premier_graphe ;
   	    graphe != graphe_courant ;
	    graphe = graphe->suivant, num_graphe++);

      /* Taille de l'ascenseur adapte */

      inter  = OB_HEIGHT(adresse, GRAPHE_FOND) * 14;
      inter /= nbr_graphes;
      OB_HEIGHT(adresse, GRAPHE_BOUTON) = (int) inter;
      
      if (num_graphe + 1 < 14) /* On est prs du dbut */
      {
	inter = 0; /* Ascenseur tout en haut */
		    
	graphe = premier_graphe;
	for (i = 0 ; i < 14 ; i++)
	{
	  liste[i] = graphe;
	  graphe = graphe->suivant;
	}
      }
      else if ( (num_graphe + 15) > nbr_graphes ) /* On est prs de la fin */
      {
	inter = OB_HEIGHT(adresse, GRAPHE_FOND) - OB_HEIGHT(adresse, GRAPHE_BOUTON);
		    
	graphe = dernier_graphe;
	for (i = 13 ; i >= 0 ; i--)
	{
	  liste[i] = graphe;
	  graphe = graphe->precedent;
	}
      }
      else
      {
	graphe = graphe_courant;
	for (i = 6 ; i >= 0 ; i--)
	{
	  liste[i] = graphe;
	  graphe = graphe->precedent;
	}
	graphe = graphe_courant->suivant;
	for (i = 7 ; i < 14 ; i++)
	{
	  liste[i] = graphe;
	  graphe = graphe->suivant;
	}
				
	inter = 0;
	for (graphe = premier_graphe ; graphe != liste[0] ; graphe = graphe->suivant)
	  inter++;
	inter *= ( OB_HEIGHT(adresse, GRAPHE_FOND) - OB_HEIGHT(adresse, GRAPHE_BOUTON) );
	inter /= (nbr_graphes - 14);
      }
      
      OB_Y(adresse, GRAPHE_BOUTON) = (int) inter; /* On positionne le bouton */
    }
  }
  
  /* On met  jour les chanes et les tats */
	
  for (i = GRAPHE_1 ; i <= GRAPHE_14 ; i++ )
  {
    chaine = (char *) (OB_SPEC(adresse, i)) + 2; /* Chane associe - 2 espaces pour le crochet */

    if (liste[i - GRAPHE_1] == NULL) /* Pas de graphe associ */
    {
      OBJ_active (CHOIX_GRAPHE, i, FORCE_NON);
      OBJ_choisit(CHOIX_GRAPHE, i, FORCE_NON);
      OB_STATE(adresse, i) &= ~CHECKED;
      chaine[0] = 0;
    }
    else
    {
      strncpy(chaine, liste[i - GRAPHE_1]->nom, 20);
						
      if (liste[i - GRAPHE_1]->trace == VRAI)
	OB_STATE(adresse, i) |= CHECKED;
      else
	OB_STATE(adresse, i) &= ~CHECKED;

      OBJ_active (CHOIX_GRAPHE, i, liste[i - GRAPHE_1]->type != GR_INTERNE );
      OBJ_choisit(CHOIX_GRAPHE, i, liste[i - GRAPHE_1] == graphe_courant);
    }
  }
	
  if (liste[0] == NULL) /* Aucun graphe en mmoire */
  {
    OBJ_active(CHOIX_GRAPHE, GRAPHE_AVANT, FORCE_NON);
    OBJ_active(CHOIX_GRAPHE, GRAPHE_APRES, FORCE_NON);
		
    return;
  }

  OBJ_active(CHOIX_GRAPHE, GRAPHE_AVANT, liste[0]->precedent != NULL);

  if (liste[13] == NULL)
    OB_STATE(adresse, GRAPHE_APRES) |= DISABLED;
  else if (liste[13]->suivant == NULL)
    OB_STATE(adresse, GRAPHE_APRES) |= DISABLED;
  else
    OB_STATE(adresse, GRAPHE_APRES) &= ~DISABLED;

  if (graphe_courant->type == -1)
    met_chaine(CHOIX_GRAPHE, TYPE_GRAPHE, 20, "Sans type", TXT_COPIER);
  else if (graphe_courant->type == GR_INTERNE) /* Graphe interne */
    met_chaine(CHOIX_GRAPHE, TYPE_GRAPHE, 20, "Usage intere", TXT_COPIER);
  else if (graphe_courant->type >= 0)
    met_chaine(CHOIX_GRAPHE, TYPE_GRAPHE, 20, (char *) Type_graphe[graphe_courant->type], TXT_COPIER);

  met_chaine(CHOIX_GRAPHE, GR_NOM,    0, graphe_courant->nom  , TXT_COPIER);
  met_chaine(CHOIX_GRAPHE, LEGENDE_X, 0, graphe_courant->axe_x, TXT_COPIER);
  met_chaine(CHOIX_GRAPHE, LEGENDE_Y, 0, graphe_courant->axe_y, TXT_COPIER);
	
  OB_SPEC(adresse, COULEUR_GRAPHE) &= 0xFFFFFFF0L; /* On met les 4 derniers bits  0 */
  OB_SPEC(adresse, COULEUR_GRAPHE) |= graphe_courant->couleur;
  
  ( (USERBLK *) OB_SPEC(adresse,LIGNE_GRAPHE))->ub_parm = graphe_courant->style;

  etat = (graphe_courant->drapeaux.complexe == GR_NORMAL); /* VRAI si graphe normal, FAUX sinon */
  OBJ_active(CHOIX_GRAPHE, GRF_TRIER    ,  etat);
  OBJ_active(CHOIX_GRAPHE, GRF_NORMALISE,  etat);
  OBJ_active(CHOIX_GRAPHE, GRF_DECOMPOSE, !etat);

  etat = (graphe_courant->type == GR_INTERNE);
  OBJ_active(CHOIX_GRAPHE, EFFACER_GRAPHE , !etat);
  OBJ_active(CHOIX_GRAPHE, COPIE_GRAPHE   , !etat);
  OBJ_active(CHOIX_GRAPHE, INVERSER_GRAPHE, !etat);

  etat = (graphe_courant->type != GR_INTERNE) && (graphe_courant->drapeaux.complexe == GR_NORMAL);
  OBJ_active(CHOIX_GRAPHE, INVERSER_GRAPHE,  etat);	
}
