/* Traitement de l'E.X.A.F.S. sur ATARI
					 Traitement mathmatique gnral des graphes (drive, intgrale,...)
	 (c) 1996 - Emmanuel CURIS

	27 mars 1998
	  * Utilisation de systeme.h
	  * Noms de fichiers en majuscules

   2 dcembre 1997
    * Calcul direct de la drive seconde fonctionne : DER_seconde()
    * Si drive d'une drive : corrige le titre en "Drive seconde de"
    * Rcupr GRF_integre() de graphes.c (calcul de la primitive d'un graphe)
          Ajout le paramtre permettant de choisir la mthode de calcul

	1er dcembre 1997
   * Cration du fichier
   * Rcupration et ragencement des fonctions de drivation de XANES.C :
	           - cration de DER_simple() et DER_lisse()
	           - pour DER_simple(), drive de graphes complexes possible

*/

#define _GRF_MATHS_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "ERREURS.H"
#include "SCIENCE.H"

#include <LISSAGE.H>

/* ----------------------- Calculs de la drive d'un graphe ----------------------------------- *
 *************************************************************************************************/

/* ----------------------- Procdure gnrale : choisit la mthode et tout et tout --------------\
 |   Renvoie le graphe contenant la drive (NULL : problme)
 |   Suppose que le graphe soit tri par X croissants (ou dcroissants ventuellement...)
 |
 | graphe  : le graphe dont on veut la drive
 | methode : mthode de calcul de la drive (DER_RAPIDE ou DER_PRECISE)
 \------------------------------------------------------------------------------------------------ */

Graphe *GRF_derive(Graphe *graphe, int methode)
{
	Graphe *derive;
	double *b, *c, *d; /* Tableaux pour le lissage */
	double retour;

	if (methode == DER_RAPIDE) /* Drive rapide */
	{
		return DER_simple(graphe);
	}

	/* On veut une drive par lissage */

	retour = SPL_tt_prep(graphe, &b, &c, &d);
	if (retour == -1)
		return NULL; /* Problme survenu */

	derive = DER_lisse(graphe, graphe->nombre_point, b, c, d);
	free(b);
	free(c);
	free(d);

	return derive;
}

/* ------------------------- Calcul de la drive par la mthode la plus simple --------------------\
 |   Renvoie le graphe contenant la drive                                                         |
 |   Suppose que le graphe est tri selon les X - Algorithme : y' = y/x                           |
 |
 | graphe : le graphe dont on veut la drive
 \------------------------------------------------------------------------------------------------- */

Graphe *DER_simple(Graphe *graphe)
{
	Graphe *derivee;
	unsigned long i;
	register double *x, *x_der;

	if (graphe == NULL) return NULL; /* Rien de spcial  faire... */
	if (graphe->nombre_point < 3)
	{
		EXF_ERREUR(TROP_PEU_POINTS); /* Au moins trois point pour que a ait un vague intrt... */
		return NULL;
	}

	/* Cration du graphe qui contiendra la drive */
	{
		char titre[100];
		int type = graphe->type;

		sprintf(titre,"Drive de %s", graphe->nom);

		if (type == GR_DERIVEE)
		{
			type = GR_DERIVEE2;
			printf("Comparaison : %d\n", strncmp(graphe->nom, "Drive", 7) );
		}
		else
			type = GR_DERIVEE;

		derivee = GRF_creer(graphe->nombre_point - 1, type, graphe->drapeaux.complexe,
		                                              titre, graphe->axe_x, graphe->axe_y,
		                                              graphe->unit_x, UNIT_ARB);
	  if (derivee == NULL)
	  	return NULL;
	}

	/* Calcul effectif de la drive */

	x = graphe->x;
	x_der = derivee->x;
	if ( graphe->drapeaux.complexe == GR_NORMAL ) /* Graphe rel */
	{
		register double *y = graphe->y, *y_der = derivee->y;

		for (i = 0 ; i < derivee->nombre_point ; i++, x++, y++, x_der++, y_der++)
		{
			double dx = *(x + 1) - *x;
			if (dx == 0)
			{
				EXF_ERREUR(ERR_DOUBLONS); /* Calcul impossible -> on quitte */
				GRF_efface(derivee, VRAI, VRAI);
				return NULL;
			}

			*x_der = ( *(x + 1) + *x ) / 2.0;
			*y_der = ( *(y + 1) - *y ) / dx;
		}
	}
	else /* Le graphe est un graphe complexe -> drive de Re() et Im()... */
	{
		register Complexe *y = (Complexe *) graphe->y, *y_der = (Complexe *) derivee->y;

		for (i = 0 ; i < derivee->nombre_point ; i++, x++, y++, x_der++, y_der++)
		{
			double dx = *(x + 1) - *x;
			if (dx == 0)
			{
				EXF_ERREUR(ERR_DOUBLONS); /* Calcul impossible -> on quitte */
				GRF_efface(derivee, VRAI, VRAI);
				return NULL;
			}

			*x_der    = ( *(x + 1)     + *x     ) / 2.0;
			y_der->re = (  (y + 1)->re -  y->re ) / dx;
			y_der->im = (  (y + 1)->im -  y->im ) / dx;
		}		
	}

	GRF_bornes(derivee, DESORDONNE, DESORDONNE); /* Les nouvelles bornes */
	return derivee;
}

/* ------------------------- Calcul de la drive avec lissage par splines -------------------------\
 |   Renvoie le graphe contenant la drive                                                         |
 |   Suppose que le graphe est tri selon les X et qu'il est REEL                                   |
 |   Mthode : on a la SPLINE -> on drive l'expression, donne ce qu'il faut...                     |
 |
 | graphe  : le graphe dont on veut la drive
 | nbr_pts : le nombre de points dans la drive [bornes : celles du graphe de dpart]
 | b, c, d : tableaux des coefficients du lissage par spline, dj calculs...
 \------------------------------------------------------------------------------------------------- */

Graphe *DER_lisse(Graphe *graphe, unsigned long nbr_pts, const double *b, const double *c, const double *d)
{
	Graphe *derivee;
	register double *x_der, *y_der;
	double x, pas;
	unsigned long i;

	if (graphe == NULL) return NULL;
	if ( (b == NULL) || (c == NULL) || (d == NULL) ) /* Coefficients non dfinis */
		return NULL;

	if (graphe->drapeaux.complexe != GR_NORMAL)
	{
		EXF_ERREUR(GRAPHE_COMPLEXE);
		return NULL;
	}

	/* Cration du graphe qui contiendra la drive */

	{
		char titre[100];
		int type = graphe->type;

		sprintf(titre, "Drive de %s", graphe->nom);

		if (type == GR_DERIVEE)
		{
			type = GR_DERIVEE2;
			if ( strcmp("Drive de", graphe->nom) )
				sprintf(titre, "Drive seconde de %s", graphe->nom + 10);
		}
		else
			type = GR_DERIVEE;

		derivee = GRF_creer(nbr_pts, type, graphe->drapeaux.complexe,
		                             titre, graphe->axe_x, graphe->axe_y,
		                             graphe->unit_x, UNIT_ARB);
	  if (derivee == NULL)
	  	return NULL;
	}

	/* Calcul effectif de la drive */

	x   = graphe->x_min;
	pas = (graphe->x_max - graphe->x_min) / (nbr_pts - 1);

	x_der = derivee->x; y_der = derivee->y;
	for (i = 0 ; i < nbr_pts ; i++, x_der++, y_der++)
	{
		unsigned long index;
		double delta_x;

		*x_der = x;

		/* On recherche le point en x le plus proche */
		if ( x <= graphe->x_min)
		{
			delta_x = x - graphe->x_min;
			index = 0;
		}
		else if (x >= graphe->x_max)
		{
			delta_x = x - graphe->x_max;
			index = graphe->nombre_point - 1;
		}
		else /* Recherche du point le plus proche - par dichotomie */
		{
			register double *zone_x = graphe->x;

			index = 0;
			while (x > *zone_x)
			{
				zone_x++; index++;
			}
			/* Ici, zone_x pointe sur l'lment juste au-dessus de X et on veut celui d'avant... */

			delta_x = x - *(zone_x - 1);
			index--;
		}

		if (fabs(delta_x) > 0)
		{
			*y_der  = delta_x * d[index] * 3;
			*y_der += 2 * c[index];
			*y_der *= delta_x;
		}
		else
		{
			*y_der = 0;
		}
		*y_der += b[index]; /* drive = b + 2*c*X + 3*d*X^2 */

		x += pas; /* On passe au point suivant */
	}

	GRF_bornes(derivee, DESORDONNE, DESORDONNE); /* On cherche les bornes */
	return derivee;

	/* ----------- Ce qui suit : mthode de Juris = utilise y/x avec des points pris sur la SPLINE (!) ---------------------- */
	/*    me semble moins prcis, mais bon... */

/*
		zone_x = derivee->x;
		zone_y = derivee->y;
		pas    = (source->x_max - source->x_min) / (derivee->nombre_point - 1);

		x_derive = source->x_min;
		*zone_x = x_derive;
		for (index = 1 ; index < derivee->nombre_point-1 ; index++)
		{
			zone_x++; zone_y++;
			x_derive += pas;
			*zone_x = x_derive;

			/ * On cherche le x prcdent dans le graphe source * /
			index2 = 0;
			while ( *(source->x + index2) < x_derive )
				index2++;

			index2--;
			dx  = x_derive - (source->x)[index2];
			dx1 = dx - pas; dx2 = dx + pas;

			yd1 = dx2 * (b[index2] + dx2 * (c[index2] + d[index2] * dx2));
			yd2 = dx1 * (b[index2] + dx1 * (c[index2] + d[index2] * dx1));
			y_derive = (yd1 - yd2) / (2 * pas);
			*zone_y = y_derive;			
		}
		(derivee->x)[derivee->nombre_point - 1] = source->x_max;
		(derivee->y)[derivee->nombre_point - 1] = (derivee->y)[derivee->nombre_point - 2];
		*(derivee->x) = source->x_min;
		*(derivee->y) = *(derivee->y + 1);
*/
}

/* ------------------------- Calcul de la drive seconde avec lissage par splines -----------------\
 |   Renvoie le graphe contenant la drive seconde                                                 |
 |   Suppose que le graphe est tri selon les X et qu'il est REEL                                   |
 |   Mthode : on a la SPLINE -> on drive l'expression, donne ce qu'il faut...                     |
 |                                                                                                  |
 | graphe  : le graphe dont on veut la drive                                                      |
 | nbr_pts : le nombre de points dans la drive [bornes : celles du graphe de dpart]              |
 | b, c, d : tableaux des coefficients du lissage par spline, dj calculs...                      |
 \------------------------------------------------------------------------------------------------- */

Graphe *DER_seconde(Graphe *graphe, unsigned long nbr_pts, const double *b, const double *c, const double *d)
{
	Graphe *derivee;
	register double *x_der, *y_der;
	double x, pas;
	unsigned long i;

	if (graphe == NULL) return NULL;
	if ( (b == NULL) || (c == NULL) || (d == NULL) ) /* Coefficients non dfinis */
		return NULL;

	if (graphe->drapeaux.complexe != GR_NORMAL)
	{
		EXF_ERREUR(GRAPHE_COMPLEXE);
		return NULL;
	}

	/* Cration du graphe qui contiendra la drive seconde */

	{
		char titre[100];
		sprintf(titre,"Drive seconde de %s", graphe->nom);

		derivee = GRF_creer(nbr_pts, GR_DERIVEE2, graphe->drapeaux.complexe,
		                             titre, graphe->axe_x, graphe->axe_y,
		                             graphe->unit_x, UNIT_ARB);
	  if (derivee == NULL)
	  	return NULL;
	}

	/* Calcul effectif de la drive */

	x   = graphe->x_min;
	pas = (graphe->x_max - graphe->x_min) / (nbr_pts - 1);

	x_der = derivee->x; y_der = derivee->y;
	for (i = 0 ; i < nbr_pts ; i++, x_der++, y_der++)
	{
		unsigned long index;
		double delta_x;

		*x_der = x;

		/* On recherche le point en x le plus proche */
		if ( x <= graphe->x_min)
		{
			delta_x = x - graphe->x_min;
			index = 0;
		}
		else if (x >= graphe->x_max)
		{
			delta_x = x - graphe->x_max;
			index = graphe->nombre_point - 1;
		}
		else /* Recherche du point le plus proche */
		{
			register double *zone_x = graphe->x;

			index = 0;
			while (x > *zone_x)
			{
				zone_x++; index++;
			}
			/* Ici, zone_x pointe sur l'lment juste au-dessus de X et on veut celui d'avant... */

			delta_x = x - *(zone_x - 1);
			index--;
		}

		if (fabs(delta_x) > 0)
		{
			*y_der  = delta_x * d[index] * 3;
		}
		else
		{
			*y_der = 0;
		}
		*y_der += c[index]; /* drive seconde = 2*c + 2*3*d*X */
		*y_der *= 2;

		x += pas; /* On passe au point suivant */
	}

	GRF_bornes(derivee, DESORDONNE, DESORDONNE); /* On cherche les bornes */
	return derivee;
}

/* ------------------------- Calculs de la primitive d'un graphe --------------------------------- *
 ***************************************************************************************************/

/* ------------------------- Procdure gnrale de calcul, avec choix des paramtres --------------\
 |    Renvoie le graphe contenant la primitive
 |
 | graphe  : le graphe que l'on veut intgrer
 \------------------------------------------------------------------------------------------------ */

Graphe *GRF_integre(Graphe *graphe)
{
	Graphe *somme; /* Graphe somme */
	double constante;
	unsigned long index;
	char titre[100];

	if (graphe == NULL) return NULL;

	met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0,
	           "Constante d'int"e_AIGU"gration", TXT_REMPLACE);
	                                        
	ext_formdo(CHOIX_VALEUR); /* On demande la constante d'intgration */
	constante = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);

	sprintf(titre, "Int"e_AIGU"grale de %s%c", graphe->nom, 0);

	somme = GRF_creer(graphe->nombre_point, -1, graphe->drapeaux.complexe, titre,
	                  graphe->axe_x, "????", graphe->unit_x, UNIT_ARB);
	if (somme == NULL) /* Pas pu crer le graphe */
		return NULL;

	if (graphe->drapeaux.complexe == GR_NORMAL)
	{
		double inter;
		register double *x      = graphe->x, *y      = graphe->y, 
		                *x_intg = somme->x , *y_intg = somme->y;

		/* Mthode des trapzes */

		for (index = 0 ; index < graphe->nombre_point ; index++, x++, y++, x_intg++, y_intg++)
		{
			*x_intg = *x;
			*y_intg = constante;

			/* On calcule pour le point suivant */
			inter  = *(y + 1) + *y;
			inter *= *(x + 1) - *x;
			inter /= 2;
			constante += inter;
		}
	}
	else /* On intgre un graphe complexe -> partie relle et partie imaginaire */
	{
		printf("Intgration d'un graphe complexe : pas encore ralis\n");
	}

	/* On mets les bornes */

	GRF_bornes(somme, DESORDONNE, DESORDONNE);

	return somme; /* On renvoie le graphe somme */
}
