/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
            Tout ce qui concerne les traitements statistiques des graphes
   (c) Emmanuel CURIS 1997-1998 (cration : 27 / 11 / 1997)

	tat : - Compil. parfaite avec Turbo C

	23 avril 1998
		* Adaptation  la nouvelle gestion des rticules
		* Quelques modifs pour stabiliser sous X

  16 & 17, 25, 31 mars 1998
    * Affichage des bornes aprs modification dans GRF_tronque() et autres...
  	* Utilisation de "systeme.h"
    * Inclus extrait.h (pour l'limination des aberrances)
    * Inclus les messages d'erreur exafs (erreurs.h)

  6, 12 janvier 1998
    * Inclus les messages d'erreur GEM (erreurs.h)
    * Annulation de la dcomposition -> effacement des graphes inutiles

  2 & 3, 9, 15 dcembre 1997
    * Rorganis l'historique
    * Troncature d'une TF : copie les infos de TF dans le graphe tronqu
	  * Corrig quelques problmes d'affichage dans la dcomposition de 2 graphes
	  * Dcomposition : le spectre diffrence est calcul
	                    l'erreur relative ( somme(y^2)/somme(y^2) ) est calcule
	  * Modifi DCMP_infos : ajout graphe contenant la combinaison linaire
	  * Corrig MSG_tronque() : mauvais arbre utilis pour redessin du pointeur
	  * MSG_decompose() fonctionne, sauf appel de la routine de dcomposition effective

  22 & 23, 25, 27 novembre 1997
	  * Rcupr le traitement des aberrances
    * Supprim effectivement RCT_aberrance() (inutile) - utilisation de RCT_pointeur fonctionne
    * Dcomposition d'un graphe sur deux spectres : cration des fonctions
            GRF_decompose() = ouverture de la fentre d'interface
            RDS_decompose() = redessin de la fentre
            CLC_decompose() = clic dans la fentre
            MSG_decompose() = messages pour la fentre
	  * Troncature des graphes : cration des fonctions
	          GRF_tronque() = ouverture de la fentre d'interface
	          RDS_tronque() = redessin de cette fentre
	          CLC_tronque() = clic sur cette fentre
	          MSG_tronque() = messages pour cette fentre
	          TRQ_calc()    = ralisation effective de la troncature
*/

#define _GRF_STATS_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "SYSTEME.H"

#include "GRAPHES.H"

#include "ERREURS.H"
#include "EXAFS.H"
#include "EXTRAIT.H"
#include "FOURIER.H"
#include "SCIENCE.H"

#include <LISSAGE.H>

/* ----------------- Dcomposer un graphe sur une base de 2 graphes connus ------------- *
 *****************************************************************************************/

/* ----------------- Ouverture de la fentre d'interface ------------------------------- */

void GRF_decompose(Graphe *graphe)
{
	DCMP_infos *infos = NULL;
	Fenetre *fenetre;

	if (graphe == NULL) return;

	if ( arbres[DECOMP_GRAPHE].ouvert == VRAI ) /* Arbre dj ouvert */
	{
		Graphe *vieux;   /* Ancien spectre travaill dans ce module */
		int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

		fenetre = ARB_fenetre( &(arbres[DECOMP_GRAPHE]) );
		vieux = (Graphe *) fenetre->pointeur;
		if ( (reponse == 1) || (vieux == graphe) ) /* On active seulement */
		{
			FEN_premierplan(fenetre);
			return; /* Rien d'autre  faire */
		}
		else if (reponse == 2) /* On rappelle, pour le nouveau graphe */
		{
			if ( graphe->drapeaux.protege )
			{
				EXF_ERREUR(GRF_PROTEGE);
				return;
			}

			infos = (DCMP_infos *) fenetre->adresse;
			vieux->drapeaux.protege = FAUX;       /* On a fini de travailler avec */

			GRF_efface(infos->residu, VRAI, VRAI); /* Plus besoin de celui-ci */
			infos->residu = NULL;

			FEN_premierplan(fenetre);
		}
		else /* On annule l'appel */
			return;
	}
	else /* Arbre non ouvert : on l'ouvre */
	{
		if ( graphe->drapeaux.protege )
		{
			EXF_ERREUR(GRF_PROTEGE);
			return;
		}

		fenetre = creer_mixte_fenetre(NAME|CLOSER|FULLER|INFO|SIZER|MOVER, DECOMP_GRAPHE, HAUT,
		                        RDS_decompose, ( boolean (*)() ) NULL, CLC_decompose,
	  	                      MSG_decompose, RCT_pointeur, ( boolean (*)() ) NULL);
		if (fenetre == NULL) return; /* Rien  faire... */
	}

	if ( infos == NULL ) /* Aucune information disponible */
	{
		infos = (DCMP_infos *) malloc( sizeof( DCMP_infos ) );
		if (infos == NULL)
		{
			ERREUR(MANQUE_MEMOIRE);
			ferme_fenetre(fenetre);
			return;
		}

		infos->ref_1 = NULL;
		infos->a_1 = NULL; infos->b_1 = NULL; infos->c_1 = NULL;
		infos->ref_2 = NULL;
		infos->a_2 = NULL; infos->b_2 = NULL; infos->c_2 = NULL;
		infos->residu = NULL;
		infos->c_lineaire = NULL;
	}

	fenetre->drapeaux.redessin = VRAI;
	fenetre->drapeaux.pointeur = VRAI;
	fenetre->msg_masque.fermeture    = VRAI; /* Messages traits par MSG_mu01 */
	fenetre->adresse  = infos;  /* Trs tt pour viter problmes sous X (avec les alertes) */
	fenetre->pointeur = graphe;

	infos->x_min = graphe->x_min; /* Positions pour l'intervalle de dcomposition */
	infos->x_max = graphe->x_max;
	met_fvaleur(DECOMP_GRAPHE, DCMP_XMIN, infos->x_min); /* On les affiche */
	met_fvaleur(DECOMP_GRAPHE, DCMP_XMAX, infos->x_max);

	if (infos->residu == NULL)
	{
		unsigned long i;
		register double *x_gr, *x_res;

		infos->residu = GRF_creer(graphe->nombre_point, GR_ECART, GR_NORMAL, "Rsidu de la dcomposition", graphe->axe_x, graphe->axe_y, graphe->unit_x, graphe->unit_y);
		if (infos->residu == NULL)
		{
			free(infos->a_1); free(infos->b_1); free(infos->c_1);
			free(infos->a_2); free(infos->b_2); free(infos->c_2);
			free(infos);
			ferme_fenetre(fenetre);
			GRF_efface(infos->c_lineaire, VRAI, VRAI); /* Au cas o */
			return;
		}

		infos->residu->x_min = graphe->x_min;
		infos->residu->x_max = graphe->x_max;
		x_gr = graphe->x; x_res = infos->residu->x;
		for (i = 0 ; i < graphe->nombre_point ; i++, x_res++, x_gr++)
		{
			*x_res = *x_gr;
		}
	}

	if (infos->c_lineaire == NULL)
	{
		unsigned long i, nbr_pts = 0;
		register double *x_gr, *x_clin;

		infos->c_lineaire = GRF_creer(graphe->nombre_point, graphe->type, GR_NORMAL, "Graphe obtenu par la combinaison linaire trouve", graphe->axe_x, graphe->axe_y, graphe->unit_x, graphe->unit_y);
		if (infos->c_lineaire == NULL)
		{
			free(infos->a_1); free(infos->b_1); free(infos->c_1);
			free(infos->a_2); free(infos->b_2); free(infos->c_2);
			free(infos);
			ferme_fenetre(fenetre);
			GRF_efface(infos->residu, VRAI, VRAI); /* Au cas o */
			return;
		}

		infos->c_lineaire->x_min = infos->x_min;
		infos->c_lineaire->x_max = infos->x_max;
		x_gr = graphe->x; x_clin = infos->c_lineaire->x;
		for (i = 0 ; i < graphe->nombre_point ; i++, x_clin++, x_gr++)
		{
			double x = *x_gr;
			if ( (x >= infos->x_min) && (x<= infos->x_max) )
			{
				*x_clin = x;
				nbr_pts++;
			}
		}	
		infos->c_lineaire->nombre_point = nbr_pts; /* Nombre de points effectif */
	}

	DCMP_calc(graphe, infos); /* On fait le calcul */

	graphe->drapeaux.protege = VRAI;
}

/* ----------------- Redessin de la fentre d'interface (dcomposition d'un spectre) --- */

void RDS_decompose(Fenetre *fenetre, int x, int y, int larg, int haut)
{
	DES_graphe param_dessin;
	DCMP_infos *infos;
	Graphe *graphe;
	double x_echelle, y_echelle, x_min, x_max, y_min, y_max;
	int y_trace, haut_trace;

	/* On met le fond blanc */

	vswr_mode(_PRG_station->handle, MD_REMPLACE);
	vsf_color(_PRG_station->handle, BLANC);
	points[0] = x;
	points[1] = y;
	points[2] = x + larg - 1;
	points[3] = y + haut - 1;
	v_bar(_PRG_station->handle, points);

	y_trace = fenetre->w_y + OB_HEIGHT(arbres[DECOMP_GRAPHE].adresse, ROOT);
	haut_trace = fenetre->w_haut - OB_HEIGHT(arbres[DECOMP_GRAPHE].adresse, ROOT);

	graphe = (Graphe *) fenetre->pointeur;
	if (graphe == NULL) return;

	infos = (DCMP_infos *) fenetre->adresse;
	if (infos == NULL) return;

	/* On calcule l'chelle globale - pour avoir une meilleur superposition */
	
	x_min = graphe->x_min;
	x_max = graphe->x_max;
	if ( (infos->ref_1) && (infos->ref_1->x_min < x_min) )
		x_min = infos->ref_1->x_min;
	if ( (infos->ref_2) && (infos->ref_2->x_min < x_min) )
		x_min = infos->ref_2->x_min;
	if ( (infos->ref_1) && (infos->ref_1->x_max > x_max) )
		x_max = infos->ref_1->x_max;
	if ( (infos->ref_2) && (infos->ref_2->x_max > x_max) )
		x_max = infos->ref_1->x_max;		

	y_min = graphe->y_min;
	y_max = graphe->y_max;
	if ( (infos->ref_1) && (infos->ref_1->y_min < y_min) )
		y_min = infos->ref_1->y_min;
	if ( (infos->ref_2) && (infos->ref_2->y_min < y_min) )
		y_min = infos->ref_2->y_min;
	if ( (infos->ref_1) && (infos->ref_1->y_max > y_max) )
		y_max = infos->ref_1->y_max;
	if ( (infos->ref_2) && (infos->ref_2->y_max > y_max) )
		y_max = infos->ref_1->y_max;		

	x_echelle = ( (double) fenetre->w_larg ) / (x_max - x_min);
	y_echelle = ( (double) haut_trace ) / (y_max - y_min);

	/* On trace le graphe que l'on traite */

	param_dessin.graphe  = graphe;
	param_dessin.couleur = NOIR;
	param_dessin.style   = 1;
	TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	           x_min, x_max, y_min, y_max, x_echelle, y_echelle);

	/* On met les deux index de l'intervalle utilis */

	vsl_color(_PRG_station->handle, ROUGE);
	points[0] = fenetre->w_x + x_echelle * (infos->x_min - x_min);
	points[1] = y_trace;
	points[2] = points[0];
	points[3] = y_trace + haut_trace;
	v_pline(_PRG_station->handle,2,points);
	
	vsl_color(_PRG_station->handle, VERT);
	points[0] = fenetre->w_x + x_echelle * (infos->x_max - x_min);
	points[2] = points[0];
	v_pline(_PRG_station->handle,2,points);

  /* On trace le premier spectre de la base - en rouge fonc, pointill */

	if (infos->ref_1) /* Ce spectre a t choisi */
	{
		param_dessin.graphe  = infos->ref_1;
		param_dessin.couleur = ROUGE_FONCE;
		param_dessin.style   = 2;
		TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	  	         x_min, x_max, y_min, y_max, x_echelle, y_echelle);
	}

	/* On trace le second spectre de la base - en vert fonc, pointill */

	if (infos->ref_2) /* Ce spectre a t choisi */
	{
		param_dessin.graphe  = infos->ref_2;
		param_dessin.couleur = VERT_FONCE;
		param_dessin.style   = 2;
		TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	  	         x_min, x_max, y_min, y_max, x_echelle, y_echelle);
	}

	/* On trace le graphe combinaison linaire des deux spectres de base */

	if (infos->c_lineaire)
	{
		param_dessin.graphe  = infos->c_lineaire;
		param_dessin.couleur = ROUGE;
		param_dessin.style   = 1;
		TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	  	         x_min, x_max, y_min, y_max, x_echelle, y_echelle);
	}
}

/* ----------------- Traitement des messages (dcomposition d'un spectre) -------------- */

boolean MSG_decompose(Fenetre *fenetre)
{
	if ( buffer[0] == WM_CLOSED )
	{
		Graphe *graphe;
		DCMP_infos *infos;
		int reponse = EXF_ERREUR(FIN_MODULE);

		if (reponse == 3) return VRAI; /* Fermeture annule */

		infos  = (DCMP_infos *) fenetre->adresse;
		graphe = (Graphe     *) fenetre->pointeur;

		if (reponse == 2) /* On ferme sans calculer les graphes */
		{
			free(infos);
			ferme_fenetre(fenetre); /* On ferme la fentre */
			GRF_efface(infos->c_lineaire, VRAI, VRAI);   /* Combinaison linaire */
			GRF_efface(infos->residu, VRAI, VRAI); /* Erreur entre les 2 graphes */
			graphe->drapeaux.protege = FAUX;
			return VRAI;
		}

		free(infos);
		ferme_fenetre(fenetre);
		graphe->drapeaux.protege = FAUX;
	}
	return VRAI;
}

/* ----------------- Gestion d'un clic souris (dcomposition d'un spectre) ------------ */

boolean CLC_decompose(Fenetre *fenetre, int bouton, int n_clic, int etat, int x, int y)
{

#if GEM
	wind_get(fenetre->handle, WF_TOP, &n_clic);
	if (n_clic != fenetre->handle)
	{
		wind_set(fenetre->handle, WF_TOP); /* Fentre non active : on l'active donc */
		return VRAI;
	}
#endif

	return VRAI;
}

/* ----------------- Dcomposition sur une base de deux spectres : calcul effectif ------\
 |    Renvoie 0 si tout va bien, -NUM_ERREUR en cas d'erreur                             |
 |    Suppose que les graphes sur lesquels on dcompose soient tris (SPLINES...)        |
 |
 | graphe : le graphe que l'on veut dcomposer                                           |
 | infos  : les infos pour la dcomposition                                              |
 \-------------------------------------------------------------------------------------- */

int DCMP_calc(Graphe *spectre, DCMP_infos *infos)
{
	double sp_moyen = 0, ref1_moyen = 0, ref2_moyen = 0;
	unsigned long i, nbr_pts = 0;
	register double *x, *y;

	if (spectre == NULL) return -1;
	if (infos   == NULL) return -1;
	if (infos->ref_1 == NULL) return 1;
	if (infos->ref_2 == NULL) return 1;

	/* On vrifie d'abord que faire le calcul a un sens */

	if (infos->x_min == infos->x_max)
	{
		EXF_ERREUR(BORNES_FAUSSES);
		return -BORNES_FAUSSES;
	}

	if ( (infos->x_min >= infos->ref_1->x_max) || (infos->x_max <= infos->ref_1->x_min) )
	{
		EXF_ERREUR(ERR_DISJOINTS);
		return -ERR_DISJOINTS;
	}

	if ( (infos->x_min >= infos->ref_2->x_max) || (infos->x_max <= infos->ref_2->x_min) )
	{
		EXF_ERREUR(ERR_DISJOINTS);
		return -ERR_DISJOINTS;
	}

	/* Premire quation : les valeurs moyennes vrifient <spectre> = coef_1 * <ref_1> + coef_2 * <ref_2> */

	/* Moyenne du spectre  dcomposer */

	x = spectre->x; y = spectre->y;
	for (i = 0 ; i < spectre->nombre_point ; i++, x++, y++)
	{
		double x_lu = *x;
		
		if ( (x_lu >= infos->x_min) && (x_lu <= infos->x_max) )
		{
			sp_moyen += *y; /* Le point compte */
			nbr_pts++;
		}
	}
	sp_moyen /= nbr_pts; /* On fait la moyenne */

	/* Moyenne du premier spectre de rfrence - en utilisant les point calculs par spline */

	x = spectre->x;
	for (i = 0 ; i < spectre->nombre_point ; i++, x++)
	{
		double x_lu = *x;
		
		if ( (x_lu >= infos->x_min) && (x_lu <= infos->x_max) )
		{
			double y = SPL_tt_calc(infos->ref_1, x_lu, infos->a_1, infos->b_1, infos->c_1);

			ref1_moyen += y; /* Le point compte */
		}
	}
	ref1_moyen /= nbr_pts; /* On fait la moyenne */

	/* Moyenne du second spectre de rfrence - en utilisant les point calculs par spline */

	x = spectre->x;
	for (i = 0 ; i < spectre->nombre_point ; i++, x++)
	{
		double x_lu = *x;
		
		if ( (x_lu >= infos->x_min) && (x_lu <= infos->x_max) )
		{
			double y = SPL_tt_calc(infos->ref_2, x_lu, infos->a_2, infos->b_2, infos->c_2);

			ref2_moyen += y; /* Le point compte */
		}
	}
	ref2_moyen /= nbr_pts; /* On fait la moyenne */

	/* Calcul effectif des coefficients - moyenne des rsultats obtenus pour chaque point */

	infos->coef_1 = 0; /* On met  zro pour tre tranquille */
	infos->coef_2 = 0;
	x = spectre->x; y = spectre->y;
	for (i = 0 ; i < spectre->nombre_point ; i++, x++, y++)
	{
		double delta, x_lu = *x;
		if ( (x_lu >= infos->x_min) && (x_lu <= infos->x_max) )
		{
			double y_1 = SPL_tt_calc(infos->ref_1, x_lu, infos->a_1, infos->b_1, infos->c_1);
			double y_2 = SPL_tt_calc(infos->ref_2, x_lu, infos->a_2, infos->b_2, infos->c_2);

			delta = y_2 * ref1_moyen - ref2_moyen * y_1;
			if (delta == 0)
			{
				EXF_ERREUR(DIV_ZERO);
			}
			else
			{
				double alpha, beta;

				alpha  = sp_moyen * y_2 - *y * ref2_moyen;
				alpha /= delta;
				beta   = ref1_moyen * *y - y_1 * sp_moyen;
				beta  /= delta;

				infos->coef_1 += alpha;
				infos->coef_2 += beta;
			}
		}
	}
	infos->coef_1 /= nbr_pts; /* On divise par tout a */
	infos->coef_2 /= nbr_pts;

	met_fvaleur(DECOMP_GRAPHE, DCMP_COEF_A, infos->coef_1); /* On affiche les valeurs trouves */
	met_fvaleur(DECOMP_GRAPHE, DCMP_COEF_B, infos->coef_2);

	/* --- Maintenant, on rempli les graphes des rsultats --- */

	{
	  double erreur = 0, somme_y2 = 0;
	  register double *x_clin   = infos->c_lineaire->x, *y_clin   = infos->c_lineaire->y,
	    *x_residu = infos->residu->x,     *y_residu = infos->residu->y;

	  x = spectre->x; y = spectre->y;
	  for (i = 0 ; i < spectre->nombre_point; i++, x++, y++)
	  {
	    double x_lu = *x;
	    if ( (x_lu >= infos->x_min) && (x_lu <= infos->x_max) )
	    {
	      double y_1 = SPL_tt_calc(infos->ref_1, x_lu, infos->a_1, infos->b_1, infos->c_1);
	      double y_2 = SPL_tt_calc(infos->ref_2, x_lu, infos->a_2, infos->b_2, infos->c_2);

	      *x_clin = x_lu; x_clin++;
	      *y_clin = infos->coef_1 * y_1 + infos->coef_2 * y_2;
			  
	      *x_residu = x_lu; x_residu++;
	      *y_residu = *y - *y_clin;

	      somme_y2 += *y * *y;
	      erreur   += *y_residu * *y_residu;

	      y_residu++; y_clin++;
	    }
	  }
	  infos->c_lineaire->nombre_point = nbr_pts; /* Nombre de points effectif */
	  infos->residu->nombre_point = nbr_pts;
	  GRF_bornes(infos->c_lineaire, NE_PAS_FAIRE, DESORDONNE);
	  GRF_bornes(infos->residu, NE_PAS_FAIRE, DESORDONNE);

	  erreur /= somme_y2;
	  met_fvaleur(DECOMP_GRAPHE, DCMP_ERREUR, erreur);
	}

	return 0; /* Pas d'erreur */
}

/* ----------------- Tronquer un graphe ------------------------------------------------ *
 *    Routine ultra-simple : on conserve ce qui faut en le recopiant, on efface l'ancien *
 *  Pas moyen d'tendre l'chelle des X de cette faon...                                *
 *****************************************************************************************/

/* ----------------- Ouverture de la fentre d'interface ------------------------------- */

void GRF_tronque(Graphe *graphe)
{
	TRQ_infos *infos = NULL;
	Fenetre *fenetre;

	if (graphe == NULL) return;

	if ( arbres[TRONQUE_GRAPHE].ouvert == VRAI ) /* Arbre dj ouvert */
	{
		Graphe *vieux;   /* Ancien spectre travaill dans ce module */
		int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

		fenetre = ARB_fenetre( &(arbres[TRONQUE_GRAPHE]) );
		vieux = (Graphe *) fenetre->pointeur;
		if ( (reponse == 1) || (vieux == graphe) ) /* On active seulement */
		{
			FEN_premierplan(fenetre); /* Fentre au premier plan */
			return; /* Rien d'autre  faire */
		}
		else if (reponse == 2) /* On rappelle, pour le nouveau graphe */
		{
			if ( graphe->drapeaux.protege )
			{
				EXF_ERREUR(GRF_PROTEGE);
				return;
			}

			infos = (TRQ_infos *) fenetre->adresse;
			vieux->drapeaux.protege = FAUX;       /* On a fini de travailler avec */

			FEN_premierplan(fenetre); /* Fentre au premier plan */
		}
		else /* On annule l'appel */
			return;
	}
	else /* Arbre non ouvert : on l'ouvre */
	{
		if ( graphe->drapeaux.protege )
		{
			EXF_ERREUR(GRF_PROTEGE);
			return;
		}

		fenetre = creer_mixte_fenetre(NAME|CLOSER|FULLER|INFO|SIZER|MOVER, TRONQUE_GRAPHE, HAUT,
		                        RDS_tronque, ( boolean (*)() ) NULL, CLC_tronque,
	  	                      MSG_tronque, RCT_pointeur, ( boolean (*)() ) NULL);
		if (fenetre == NULL) return; /* Rien  faire... */
	}

	if ( infos == NULL ) /* Aucune information disponible */
	{
		infos = (TRQ_infos *) malloc( sizeof( TRQ_infos ) );
		if (infos == NULL)
		{
			ferme_fenetre(fenetre);
			ERREUR(MANQUE_MEMOIRE);
			return;
		}
	}

	fenetre->adresse  = infos;
	fenetre->pointeur = graphe;

	fenetre->drapeaux.redessin = VRAI;
	fenetre->drapeaux.pointeur = VRAI;
	fenetre->msg_masque.fermeture    = VRAI; /* Messages traits par MSG_mu01 */

	infos->x_min = graphe->x_min; /* Positions pour la troncature */
	met_fvaleur(TRONQUE_GRAPHE, TRQ_XMIN, infos->x_min);
	infos->x_max = graphe->x_max;
	met_fvaleur(TRONQUE_GRAPHE, TRQ_XMAX, infos->x_max);
	if ( OBJ_choisit(TRONQUE_GRAPHE, TRONQUE_EXT, QUESTION) )
		infos->type = TRONQUE_EXT;
	else
		infos->type = TRONQUE_INT;

	graphe->drapeaux.protege = VRAI;
}

/* ----------------- Clic sur la fentre de troncature -----------------------------------\
 |    Renvoie VRAI si l'vnement a t trait, donc toujours VRAI                        |
 |                                                                                        |
 | bouton  : le bouton avec lequel on a cliqu (1 = gauche, 2 = droit)                    |
 | etat    : tat des touches spciales lors du clic (shift, control,...)                 |
 | fenetre : la fentre correspondant  la fentre de troncature                          |
 | n_clic  : le nombre de clics effectus                                                 |
 | x, y    : coordonnes du clic souris                                                   |
 \--------------------------------------------------------------------------------------- */

boolean CLC_tronque(Fenetre *fenetre, int bouton, int n_clic, int etat, int x, int y)
{
	Graphe *graphe;
	TRQ_infos *infos;
	int x_min, x_max;
	double echelle;

#if GEM
	wind_get(fenetre->handle, WF_TOP, &n_clic);
	if (n_clic != fenetre->handle)
	{
		wind_set(fenetre->handle, WF_TOP); /* Fentre non active : on l'active donc */
		return VRAI;
	}
#endif

	if ( (x < fenetre->w_x) || (x > fenetre->w_x + fenetre->w_larg)
	  || (y < fenetre->w_y) || (y > fenetre->w_y + fenetre->w_haut) )
	{
		return VRAI; /* Clic en-dehors de la zone de travail */
	}

	infos  = (TRQ_infos *) fenetre->adresse;
	graphe = (Graphe *)   fenetre->pointeur;

	if (graphe->x_max == graphe->x_min)
		return VRAI;

	echelle = ( (double) fenetre->w_larg) / (graphe->x_max - graphe->x_min);
	x_min = fenetre->w_x + (int) ( (infos->x_min - graphe->x_min) * echelle );
	x_max = fenetre->w_x + (int) ( (infos->x_max - graphe->x_min) * echelle );

	if ( etat & (K_RSHIFT | K_LSHIFT) ) /* Shift-clic => positionne directement la borne la plus proche */
	{
		int d_m = abs(x - x_min), d_M = abs(x - x_max);

		if (d_m > d_M) /* Le clic est plus proche de x_max */
			infos->x_max = graphe->x_min + ( (double) (x - fenetre->w_x) ) / echelle;
		else
			infos->x_min = graphe->x_min + ( (double) (x - fenetre->w_x) ) / echelle;
	}
	else if ( etat & K_CTRL )           /* Ctrl-clic => positionne directement une borne */
	{
		if (bouton == 1)      /* Bouton droit : borne infrieure */
			infos->x_min = graphe->x_min + ( (double) (x - fenetre->w_x) ) / echelle;
		else if (bouton == 2) /* Bouton gauche : borne suprieure */
			infos->x_max = graphe->x_min + ( (double) (x - fenetre->w_x) ) / echelle;
	}
	else                  /* Clic normal => on fait glisser les bornes, normalement...   */
	{
		int d_m = abs(x - x_min), d_M = abs(x - x_max);
		int vx_x, vx_y;
		int bouton, etat, n_clic, retour, boucle = VRAI;

		if ( (d_m > 5) && (d_M > 5) ) /* On est trop loin pour faire glisser */
			return VRAI;

		points[1] = fenetre->w_y + OB_HEIGHT( arbres[TRONQUE_GRAPHE].adresse, ROOT );
		points[3] = fenetre->w_y + fenetre->w_haut - 1;

		graf_mouse(M_OFF, NULL);
		wind_update(BEG_UPDATE);
		wind_update(BEG_MCTRL);

		vswr_mode(_PRG_station->handle, MD_INVERSE);
		vsf_interior(_PRG_station->handle, 1);
		vsf_color(_PRG_station->handle, NOIR);

		if (d_m <= 5)
		{
			vx_x = x_min;
			if (infos->type == TRONQUE_EXT)
			{
				points[0] = fenetre->w_x;
				points[2] = vx_x;
			}
			else
			{
				points[0] = vx_x;
				points[2] = x_max;
			}
		}
		else
		{
			vx_x = x_max;
			if (infos->type == TRONQUE_EXT)
			{
				points[0] = x;
				points[2] = fenetre->w_x + fenetre->w_larg - 1;
			}
			else
			{
				points[0] = x_min;
				points[2] = x;
			}
		}
		v_bar(_PRG_station->handle, points);
		vx_y = y;

		do
		{
			float x_clic;

			retour = Evnt_multi(MU_BUTTON|MU_M1, 258, 3, 0,
	                       1, vx_x, vx_y, 1 ,1,
	                       0, 0, 0, 0, 0,
	                       0, 0, 0,
                         &x, &y, &bouton, &etat, (unsigned int *) &_rien_, &n_clic);

			if (vx_x != x)
			{
				v_bar(_PRG_station->handle, points);

				if (d_m <= 5)
				{
					if (infos->type == TRONQUE_EXT)
					{
						points[2] = x;
					}
					else
					{
   					points[0] = x;
					}
				}
				else
				{
					if (infos->type == TRONQUE_EXT)
					{
						points[0] = x;
					}
					else
					{
						points[2] = x;
					}
				}
				v_bar(_PRG_station->handle, points);

				x_clic = graphe->x_min + (x - fenetre->w_x) / echelle;
				sprintf(_intermediaire, "Souris : X = %f", x_clic);
				FEN_infos(fenetre, _intermediaire);
			}
			vx_x = x; vx_y = y;

			if (retour & MU_BUTTON) /* On sort en cliquant une seconde fois */
				boucle = FAUX;
		}
		while (boucle);

		graf_mouse(M_ON, NULL);
		wind_update(END_UPDATE);
		wind_update(END_MCTRL);

		if (bouton == 1) /* On accepte le changement */
		{
			if (d_m <= 5)
				infos->x_min = graphe->x_min + ( (double) (x - fenetre->w_x) ) / echelle;
			else
				infos->x_max = graphe->x_min + ( (double) (x - fenetre->w_x) ) / echelle;
		}
	}

	if ( infos->x_max < infos->x_min ) /* On s'assure que min < max */
	{
		double x = infos->x_max; infos->x_max = infos->x_min; infos->x_min = x;
	}

	if ( infos->x_max > graphe->x_max )
		infos->x_max = graphe->x_max;
	if ( infos->x_min < graphe->x_min )
		infos->x_min = graphe->x_min;

	met_fvaleur(TRONQUE_GRAPHE, TRQ_XMIN, infos->x_min);
	met_fvaleur(TRONQUE_GRAPHE, TRQ_XMAX, infos->x_max);

	FEN_redessin(fenetre); /* On force le redessin... en attendant mieux */
	return VRAI;
}

/* ----------------- Message destin  la fentre de troncature --------------------------\
 |   Renvoie VRAI si le message a pu tre trait, FAUX sinon                              |
 |                                                                                        |
 | fenetre : la fentre correspondant  la fentre de troncature                          |
 \--------------------------------------------------------------------------------------- */

boolean MSG_tronque(Fenetre *fenetre)
{
	if ( buffer[0] == WM_CLOSED )
	{
		Graphe *graphe, *resultat;
		TRQ_infos *infos;
		int reponse = EXF_ERREUR(FIN_MODULE);

		if (reponse == 3) return VRAI; /* Fermeture annule */

		infos  = (TRQ_infos *) fenetre->adresse;
		graphe = (Graphe    *) fenetre->pointeur;

		if (reponse == 2) /* On ferme sans modifier le graphe */
		{
			free(infos);
			ferme_fenetre(fenetre); /* On ferme la fentre */
			graphe->drapeaux.protege = FAUX;
			return VRAI;
		}

		resultat = TRQ_calc(graphe, infos);
		if ( resultat ) /* Pas d'erreur */
		{
			GRF_aller(resultat); /* On va  ce graphe */
			free(infos);
			ferme_fenetre(fenetre);
			graphe->drapeaux.protege = FAUX;

			if (graphe->pointeur) /* Il y a des infos complmentaires */
			{
				if ( graphe->type == GR_TF ) /* Transforme de Fourier */
				{
					TF_infos *infos = malloc( sizeof(TF_infos) );

					if (infos == NULL)
					{
						ERREUR(MANQUE_MEMOIRE);
					}
					else
					{
						TF_infos *original = (TF_infos *) graphe->pointeur;

						infos->graphe = NULL;
						infos->fenetre = NULL;
						infos->type_fenetre  = original->type_fenetre;
						infos->num_depart = 0;
						infos->param_fenetre = original->param_fenetre;
						infos->k_min = original->k_min;
						infos->k_max = original->k_max;
						infos->degre   = original->degre;
						infos->amplitude = NULL;
						infos->phase = NULL;

						resultat->pointeur = infos; /* On met les nouvelles infos */
					}
				}
			}
		}
	}

	return VRAI;
}

/* ----------------- Redessin de la fentre de troncature --------------------------------\
 |    Ne renvoie rien                                                                     |
 |  Suppose le clipping dj install avant l'appel, ainsi que la souris et l'AES coups  |
 |                                                                                        |
 | fenetre : la fentre qui correspond  la fentre de troncature                         |
 | x, y, larg, haut : coordonnes de la zone de redessin                                  |
 \--------------------------------------------------------------------------------------- */

void RDS_tronque(Fenetre *fenetre, int x, int y, int larg, int haut)
{
	DES_graphe param_dessin;
	TRQ_infos *infos;
	Graphe *graphe;
	int x_min, x_max;
	int y_trace, haut_trace;
	double x_echelle, y_echelle;

	/* On met le fond blanc */

	vswr_mode(_PRG_station->handle, MD_REMPLACE);
	vsf_color(_PRG_station->handle, BLANC);
	points[0] = x;
	points[1] = y;
	points[2] = x + larg - 1;
	points[3] = y + haut - 1;
	v_bar(_PRG_station->handle, points);

	y_trace = fenetre->w_y + OB_HEIGHT(arbres[TRONQUE_GRAPHE].adresse, ROOT);
	haut_trace = fenetre->w_haut - OB_HEIGHT(arbres[TRONQUE_GRAPHE].adresse, ROOT);

	/* On trace le graphe que l'on traite */

	graphe = (Graphe *) fenetre->pointeur;
	if (graphe == NULL)
		return;

	x_echelle = ( (double) fenetre->w_larg ) / (graphe->x_max - graphe->x_min);
	y_echelle = ( (double) haut_trace )      / (graphe->y_max - graphe->y_min);

	param_dessin.graphe  = graphe;
	param_dessin.couleur = NOIR;
	param_dessin.style   = 1;
	TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	           graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);

	/* On hachure la zone qui n'est pas conserve */

	infos = (TRQ_infos *) fenetre->adresse;
	if (infos == NULL)
		return;

	vswr_mode(_PRG_station->handle, MD_TRANSPARENT);
	vsl_color(_PRG_station->handle, NOIR);
	vsf_color(_PRG_station->handle, GRIS_FONCE);
	vsf_interior(_PRG_station->handle, FIS_HATCH); /* On hachure */
	vsf_perimeter(_PRG_station->handle, 0);        /* Pas de bordure */

	points[1] = y; points[3] = y + haut;

	x_min = (int) ( fenetre->w_x + (infos->x_min - graphe->x_min) * x_echelle );
	x_max = (int) ( fenetre->w_x + (infos->x_max - graphe->x_min) * x_echelle );
	if ( infos->type == TRONQUE_INT ) /* On efface ce qui est entre x_min et x_max */
	{
		if ( (x_max > x) || (x_min < x + larg) ) /* Tombe dans la zone  redessiner */
		{
			points[0] = x_min; points[2] = x_max;  /* Le hachurage      */
			v_bar(_PRG_station->handle, points);

			points[2] = x_min;                     /* Limite infrieure */
			v_pline(_PRG_station->handle, 2, points);

			points[0] = x_max; points[2] = x_max;  /* Limite suprieure */
			v_pline(_PRG_station->handle, 2, points);
		}
	}
	else if ( infos->type == TRONQUE_EXT ) /* On efface ce qui est avant x_min et aprs x_max */
	{
		if (x_min > x)
		{
			points[0] = x; points[2] = x_min;
			v_bar(_PRG_station->handle, points);
			
			points[0] = x_min;
			v_pline(_PRG_station->handle, 2, points);
		}
		if (x_max < x + larg)
		{
			points[0] = x_max; points[2] = x + larg;
			v_bar(_PRG_station->handle, points);
			
			points[2] = x_max;
			v_pline(_PRG_station->handle, 2, points);			
		}
	}
}

/* ----------------- Ralisation effective de la troncature -----------------------------\
 |    Renvoie le graphe tronqu  (NULL si erreur quelconque)                             |
 |                                                                                       |
 | graphe : le graphe que l'on veut tronquer                                             |
 | infos  : les infos de troncature                                                      |
 \-------------------------------------------------------------------------------------- */

Graphe *TRQ_calc(Graphe *graphe, TRQ_infos *infos)
{
	Graphe *resultat;
	unsigned long nbr_pts = 0, i;

	if (graphe == NULL) return NULL;
	if (infos  == NULL) return NULL;

	/* Dcompte des points  conserver */

	if (infos->type == TRONQUE_INT)
	{
		register double *x = graphe->x;

		for (i = 0 ; i < graphe->nombre_point ; i++, x++)
		{
			if ( (*x < infos->x_min) || (*x > infos->x_max) )
				nbr_pts++;
		}
	}
	else if (infos->type == TRONQUE_EXT)
	{
		register double *x = graphe->x;

		for (i = 0 ; i < graphe->nombre_point ; i++, x++)
		{
			if ( (*x >= infos->x_min) && (*x <= infos->x_max) )
				nbr_pts++;
		}
	}

	resultat = GRF_creer(nbr_pts, graphe->type, graphe->drapeaux.complexe, 
	                     graphe->nom, graphe->axe_x, graphe->axe_y, graphe->unit_x, graphe->unit_y);
	if (resultat == NULL)
		return NULL;

	/* Reste juste  recopier les points corrects dans le nouveau graphe */

	if ( graphe->drapeaux.complexe == GR_NORMAL ) /* Graphe rel */
	{
	  register double *x = graphe->x, *nv_x = resultat->x;
	  register double *y = graphe->y, *nv_y = resultat->y;
	  unsigned long i;

	  if ( infos->type == TRONQUE_INT ) /* On enlve entre x_min et x_max */
	  {
	    for (i = 0 ; i < graphe->nombre_point ; i++, x++, y++)
	    {
	      if ( (*x < infos->x_min) || (*x > infos->x_max) )
	      {
		*nv_x = *x; nv_x++; /* On recopie le point qui nous intresse */
		*nv_y = *y; nv_y++;
	      }
	    }
	  }
	  else if (infos->type == TRONQUE_EXT) /* On garde entre x_min et x_max */
	  {
	    for (i = 0 ; i < graphe->nombre_point ; i++, x++, y++)
	    {
	      if ( (*x >= infos->x_min) && (*x <= infos->x_max) )
	      {
		*nv_x = *x; nv_x++; /* On recopie le point qui nous intresse */
		*nv_y = *y; nv_y++;
	      }
	    }
	  }
	}
	else                                          /* Graphe complexe */
	{
	  register double   *x =              graphe->x, *nv_x =              resultat->x;
	  register Complexe *y = (Complexe *) graphe->y, *nv_y = (Complexe *) resultat->y;
	  unsigned long i;

	  if ( infos->type == TRONQUE_INT ) /* On enlve entre x_min et x_max */
	  {
	    for (i = 0 ; i < graphe->nombre_point ; i++, x++, y++)
	    {
	      if ( (*x < infos->x_min) || (*x > infos->x_max) )
	      {
		*nv_x = *x; nv_x++; /* On recopie le point qui nous intresse */
		*nv_y = *y; nv_y++;
	      }
	    }
	  }
	  else if (infos->type == TRONQUE_EXT) /* On garde entre x_min et x_max */
	  {
	    for (i = 0 ; i < graphe->nombre_point ; i++, x++, y++)
	    {
	      if ( (*x >= infos->x_min) && (*x <= infos->x_max) )
	      {
		*nv_x = *x; nv_x++; /* On recopie le point qui nous intresse */
		nv_y->re = y->re;
		nv_y->im = y->im;
		nv_y++;
	      }
	    }
	  }
	}
	
	GRF_bornes(resultat, DESORDONNE, DESORDONNE); /* Nouvelles bornes */

	return resultat;
}

/* ----------------- limination d'un point aberrant ----------------------------------- *
 *****************************************************************************************/

ABER_param param_aber, *prem_aber = NULL;

/* ----------------------- Appel de la fentre ------------------------------------------- */

void aberrance(Graphe *graphe)
{
	Fenetre *fenetre;

	if (graphe == NULL) return;    /* Pas de graphe ! */
	if (graphe->nombre_point < 11)
	{
		EXF_ERREUR(TROP_PEU_POINTS);
		return;
	}

	fenetre = creer_mixte_fenetre(NAME|MOVER|CLOSER|FULLER|SIZER|INFO,ELIM_ABERRANCE,HAUT,
	                              RDS_aberrance, ( boolean (*)() ) NULL, CLC_aberrance,
	                              MSG_aberrance, RCT_pointeur, (boolean (*)()) NULL );
	if (fenetre == NULL) return;

	fenetre->adresse  = &param_aber;
	fenetre->pointeur = graphe;
	fenetre->drapeaux.redessin = VRAI;
	fenetre->drapeaux.pointeur = VRAI;
	fenetre->msg_masque.fermeture    = VRAI; /* Message de fermeture...  */

	param_aber.degre = prend_valeur(ELIM_ABERRANCE,ABER_VALEUR);
	param_aber.graphe = graphe_courant;
	param_aber.x_borne[0] = *(graphe->x);
	param_aber.x_borne[1] = *(graphe->x + 10) ;
	param_aber.x_borne[2] = *(graphe->x + graphe->nombre_point - 11);
	param_aber.x_borne[3] = *(graphe->x + graphe->nombre_point - 1);

	met_fvaleur(ELIM_ABERRANCE,ABER_XMIN1,param_aber.x_borne[0]);
	met_fvaleur(ELIM_ABERRANCE,ABER_XMAX1,param_aber.x_borne[1]);
	met_fvaleur(ELIM_ABERRANCE,ABER_XMIN2,param_aber.x_borne[2]);
	met_fvaleur(ELIM_ABERRANCE,ABER_XMAX2,param_aber.x_borne[3]);
}

/* ~~~~~~~~~~~~~~~ Redessin de cette fentre ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

void RDS_aberrance(Fenetre *fenetre, int x_rds, int y_rds, int larg, int haut)
{
	ABER_param *param_aber;
	OBJECT *adresse = arbres[ELIM_ABERRANCE].adresse;
	double x_echelle,y_echelle;
	int x_trace,y_trace,larg_trace,haut_trace,i,compteur = 0;
	double *zone_x,*zone_y,x,y;
	double *b = NULL,*c = NULL,*d = NULL,*tableau = NULL;
	double x_depart,x_fin;
	unsigned long index;
	unsigned long nbr_spline;
	double erreur;

	x_trace = fenetre->w_x;
	y_trace = fenetre->w_y + OB_HEIGHT(adresse, ROOT) + 1;
	larg_trace = fenetre->w_larg;
	haut_trace = fenetre->w_haut - OB_HEIGHT(adresse, ROOT) - 2;

	param_aber = (ABER_param *) fenetre->adresse; /* Adresse du graphe  redessiner */

	vswr_mode(_PRG_station->handle, MD_REMPLACE);
	vsf_color(_PRG_station->handle, BLANC);
	points[0] = x_rds;
	points[1] = y_rds;
	points[2] = x_rds + larg - 1;
	points[3] = y_rds + haut - 1;
	v_bar(_PRG_station->handle,points);     /* Le fond blanc */

	x_echelle = (double) (larg_trace) / (param_aber->graphe->x_max - param_aber->graphe->x_min);
	y_echelle = (double) (haut_trace) / (param_aber->graphe->y_max - param_aber->graphe->y_min);
	x_depart = param_aber->graphe->x_min + (x_rds - x_trace) / x_echelle;
	x_fin = param_aber->graphe->x_min + (x_rds + larg - x_trace) / x_echelle;

	/* On trace la courbe exprimentale - en noir */
	vsl_color(_PRG_station->handle, NOIR);

	zone_x = param_aber->graphe->x; zone_y = param_aber->graphe->y;

	for (index = 0 ; index < param_aber->graphe->nombre_point ; index ++)
	{
		if ( ( (*zone_x) <= x_fin) && ( (*zone_x) >= x_depart ) )
		{
			points[compteur]   = x_trace + x_echelle * ((*zone_x) - param_aber->graphe->x_min);
			points[compteur+1] = y_trace + y_echelle * (param_aber->graphe->y_max - (*zone_y));
			compteur += 2;
		}
		zone_x++;
		zone_y++;

		if (compteur >= 256) /* Tableau de points complet */
		{
			v_pline(_PRG_station->handle,128,points);
			points[0] = points[254];
			points[1] = points[255];
			compteur = 2;
		}
	}
	v_pline(_PRG_station->handle,(int) (compteur+1)/2,points);

	/* On calcule le polynme d'interpolation entre les deux points */
	if ( OBJ_choisit(ELIM_ABERRANCE, ABER_POLY1, QUESTION) )
		erreur = moindre2(param_aber->graphe,param_aber->degre,2,param_aber->x_borne,param_aber->coefs);
	else if ( OBJ_choisit(ELIM_ABERRANCE, ABER_POLY2, QUESTION) )
		erreur = moindrej(param_aber->graphe,param_aber->degre,2,param_aber->x_borne,param_aber->coefs);
	else
		erreur = SPL_prepare(param_aber->graphe,2,param_aber->x_borne,&b,&c,&d,&tableau,&nbr_spline);

	if (erreur >= 0)
	{
		met_fvaleur(ELIM_ABERRANCE, ABER_ERREUR, erreur); /* Erreur calcule */
		dessine_objet(ELIM_ABERRANCE, ABER_ERREUR);
		vsl_color(_PRG_station->handle, BLEU);
		zone_x = param_aber->graphe->x;

		compteur = 0;
		for (index = 0 ; index < param_aber->graphe->nombre_point ; index++)
		{
			x = *(zone_x);
			if ( (x <= x_fin) && (x >= x_depart) )
			{
				if ( (x >= param_aber->x_borne[0]) && (x <= param_aber->x_borne[3]) )
				{
					if ( OBJ_choisit(ELIM_ABERRANCE, ABER_SPLINE, QUESTION) ) /* Calcul par Spline */
					{
						y = SPL_calcule(x,nbr_spline,tableau,b,c,d);
					}
					else
					{
						y = param_aber->coefs[0];
						for (i = 1 ; i <= param_aber->degre ; i++)
						{
							y += x * param_aber->coefs[i];
							x *= *(zone_x);
						}
					}
					points[compteur]   = x_trace + x_echelle * ( *(zone_x) - param_aber->graphe->x_min);
					points[compteur+1] = y_trace + y_echelle * (param_aber->graphe->y_max - y);
					compteur += 2;	
				}
			}
			zone_x++;
			zone_y++;

			if (compteur >= 256) /* Tableau de points complet */
			{
				v_pline(_PRG_station->handle,128,points);
				points[0] = points[254];
				points[1] = points[255];
				compteur = 2;
			}
		}
		v_pline(_PRG_station->handle,(int) (compteur+1)/2,points);
	}

	/* On libre la mmoire ventuellement utilise pour une SPLINE */
	if ( OBJ_choisit(ELIM_ABERRANCE, ABER_SPLINE, QUESTION) )
	{
		free(b); free(c); free(d);
		free(tableau);
	}

	/* On place les quatre curseurs indiquant les limites */
	vswr_mode(_PRG_station->handle, MD_INVERSE); /* Mode inverser */
	vsl_color(_PRG_station->handle, ROUGE);
	vst_color(_PRG_station->handle, BLEU);

	if ( ( param_aber->x_borne[0] >= x_depart) && (param_aber->x_borne[0] <= x_fin) )
	{
		points[0] = x_trace + x_echelle * ( param_aber->x_borne[0] - param_aber->graphe->x_min );
		points[1] = y_trace;
		points[2] = points[0];
		points[3] = y_trace + haut_trace;
		v_pline(_PRG_station->handle,2,points);
		v_gtext(_PRG_station->handle,points[0],y_trace,"1");
	}
	if ( ( param_aber->x_borne[1] >= x_depart) && (param_aber->x_borne[1] <= x_fin) )
	{
		points[0] = x_trace + x_echelle * ( param_aber->x_borne[1] - param_aber->graphe->x_min );
		points[1] = y_trace;
		points[2] = points[0];
		points[3] = y_trace + haut_trace;
		v_pline(_PRG_station->handle,2,points);
		v_gtext(_PRG_station->handle,points[0],y_trace,"2");
	}
	if ( ( param_aber->x_borne[2] >= x_depart) && (param_aber->x_borne[2] <= x_fin) )
	{
		points[0] = x_trace + x_echelle * ( param_aber->x_borne[2] - param_aber->graphe->x_min );
		points[1] = y_trace;
		points[2] = points[0];
		points[3] = y_trace + haut_trace;
		v_pline(_PRG_station->handle,2,points);
		v_gtext(_PRG_station->handle,points[0],y_trace,"3");
	}
	if ( ( param_aber->x_borne[3] >= x_depart) && (param_aber->x_borne[3] <= x_fin) )
	{
		points[0] = x_trace + x_echelle * ( param_aber->x_borne[3] - param_aber->graphe->x_min );
		points[1] = y_trace;
		points[2] = points[0];
		points[3] = y_trace + haut_trace;
		v_pline(_PRG_station->handle,2,points);
		v_gtext(_PRG_station->handle,points[0],y_trace,"4");
	}
}

/* ----------- Gestion du clic dans cette fentre ---------------------------------------- */

boolean CLC_aberrance(Fenetre *fenetre, int bouton, int n_clic, int etat, int x_souris, int y_souris)
{
	ABER_param *param_aber;
	double x_clic;
	double x_echelle;
	int x_borne[4],vieux_x,vieux_y,y_trace,x_mem,index;
	unsigned int rien;
	boolean lieu_clic = 0,boucle = VRAI;
	char numero[2] = { '1', 0 };

#if GEM
	wind_get(0, WF_TOP, &x_mem);
	if (x_mem != fenetre->handle)
	{
		wind_set(fenetre->handle,WF_TOP);
		return VRAI;
	}
#endif

	param_aber = (ABER_param *) (fenetre->adresse);

	y_trace = fenetre->w_y + OB_HEIGHT(fenetre->arbre->adresse,ROOT) + 1;

	x_echelle = (double) (fenetre->w_larg) / (param_aber->graphe->x_max - param_aber->graphe->x_min);

	wind_update(BEG_UPDATE);     /* On coupe l'AES le temps du dessin */
	wind_update(BEG_MCTRL);      /* On coupe l'AES le temps du dessin */
	graf_mouse(M_OFF,NULL); 
	vswr_mode(_PRG_station->handle,3);   /* Mode inverser                             */
	points[0] = fenetre->w_x;                   points[1] = fenetre->w_y;
	points[2] = fenetre->w_x + fenetre->w_larg; points[3] = fenetre->w_y + fenetre->w_haut;
	vs_clip(_PRG_station->handle,1,points);

	points[1] = fenetre->w_y + OB_HEIGHT(fenetre->arbre->adresse,ROOT);
	points[3] = fenetre->w_y + fenetre->w_haut;

	x_borne[0] = (param_aber->x_borne[0] - (double) param_aber->graphe->x_min) * x_echelle + fenetre->w_x;
	x_borne[1] = (param_aber->x_borne[1] - (double) param_aber->graphe->x_min) * x_echelle + fenetre->w_x;
	x_borne[2] = (param_aber->x_borne[2] - (double) param_aber->graphe->x_min) * x_echelle + fenetre->w_x;
	x_borne[3] = (param_aber->x_borne[3] - (double) param_aber->graphe->x_min) * x_echelle + fenetre->w_x;

	for ( index = 0 ; index < 4 ; index++ )
	{
		if ( (x_souris > x_borne[index] - 5) && (x_souris < x_borne[index] + 5) )
		{
			lieu_clic = index + 1;
		}
	}

	/* On attend que le bouton soit relch */
	evnt_button(1,3,0,&vieux_x,&vieux_x,&etat,&n_clic);

	if (lieu_clic == 0) /* On n'a cliqu nulle part */
	{
		graf_mouse(M_ON,NULL);
		wind_update(END_UPDATE);     /* On coupe l'AES le temps du dessin */
		wind_update(END_MCTRL);      /* On coupe l'AES le temps du dessin */
		return VRAI;
	}

	if (lieu_clic > 0)
	{
		vsl_color(_PRG_station->handle, NOIR);
		points[0] = x_souris; /* On efface le pointeur */
		points[2] = x_souris;
		v_pline(_PRG_station->handle,2,points);
		x_mem = x_souris;

		x_souris = x_borne[lieu_clic - 1];
		numero[0] += (lieu_clic - 1);

		vsl_color(_PRG_station->handle, ROUGE);  /* On travaille en rouge, c'est plus joli... */
		vst_color(_PRG_station->handle, BLEU );  /* Avec le numro en bleu                    */

		vieux_x = x_souris; vieux_y = y_souris;
		do
		{
			index = Evnt_multi(MU_BUTTON|MU_M1, 258, 3, 0,
	                       1, vieux_x, vieux_y, 1 ,1,
	                       0, 0, 0, 0, 0,
	                       0, 0, 0,
                         &x_souris, &y_souris, &bouton, &etat, &rien, &n_clic);

			if (vieux_x != x_souris)
			{
				points[0] = vieux_x;
				points[2] = vieux_x;
				v_pline(_PRG_station->handle,2,points);
				v_gtext(_PRG_station->handle,vieux_x,y_trace,numero);

				points[0] = x_souris;
				points[2] = x_souris;
				v_pline(_PRG_station->handle,2,points);
				v_gtext(_PRG_station->handle,x_souris,y_trace,numero);

				x_clic = param_aber->graphe->x_min + (x_souris - fenetre->w_x) / x_echelle;
				sprintf(_intermediaire, "Souris : X = %f", x_clic);
				FEN_infos(fenetre, _intermediaire);
			}
			vieux_x = x_souris; vieux_y = y_souris;

			if (index & MU_BUTTON) /* On sort en cliquant une seconde fois */
				boucle = FAUX;
		}
		while (boucle);

		if (bouton == 1) /* On valide le changement de position du truc */
		{
			x_borne[lieu_clic - 1] = x_souris;
			/* On efface les anciens numros */
			for (index = 0 ; index < 4 ; index++)
			{
				numero[0] = '1' + index;
				v_gtext(_PRG_station->handle,x_borne[index],y_trace,numero);
			}

			/* On enregistre _la_ nouvelle valeur */
			param_aber->x_borne[lieu_clic - 1] = param_aber->graphe->x_min + (x_borne[lieu_clic - 1] - fenetre->w_x) / x_echelle;

			/* On trie les valeurs pour avoir x1_min < x1_max < x2_min < x2_max */
			for (index = 0 ; index < 4 ; index++)
				for (rien = index + 1 ; rien < 4 ; rien++)
					if (param_aber->x_borne[rien] < param_aber->x_borne[index]) /* On intervertit les deux */
					{
						x_clic = param_aber->x_borne[rien];
						vieux_x = x_borne[rien];
						param_aber->x_borne[rien] = param_aber->x_borne[index];
						x_borne[rien] = x_borne[index];
						param_aber->x_borne[index] = x_clic;
						x_borne[index] = vieux_x;
					}

			/* On raffiche les boutons */
			met_fvaleur(ELIM_ABERRANCE, ABER_XMIN1, param_aber->x_borne[0]);
			met_fvaleur(ELIM_ABERRANCE, ABER_XMAX1, param_aber->x_borne[1]);
			met_fvaleur(ELIM_ABERRANCE, ABER_XMIN2, param_aber->x_borne[2]);
			met_fvaleur(ELIM_ABERRANCE, ABER_XMAX2, param_aber->x_borne[3]);
			dessine_objet(ELIM_ABERRANCE,ABER_XMAX2);
			dessine_objet(ELIM_ABERRANCE,ABER_XMAX1);
			dessine_objet(ELIM_ABERRANCE,ABER_XMIN2);
			dessine_objet(ELIM_ABERRANCE,ABER_XMIN1);

			/* On fait redessiner */
			FEN_redessin(fenetre);
		}
		else
		{
			points[0] = x_souris;
			points[2] = x_souris;
			v_pline(_PRG_station->handle,2,points);
			v_gtext(_PRG_station->handle,x_souris,y_trace,numero);

			points[0] = x_borne[lieu_clic - 1];
			points[2] = x_borne[lieu_clic - 1];
			v_pline(_PRG_station->handle,2,points);
			v_gtext(_PRG_station->handle,x_borne[lieu_clic - 1],y_trace,numero);

			vsl_color(_PRG_station->handle, NOIR); /* On remet le pointeur */
			points[0] = x_mem;
			points[2] = x_mem;
			v_pline(_PRG_station->handle,2,points);
		}
	}

	graf_mouse(M_ON,NULL);
	wind_update(END_UPDATE);     /* On coupe l'AES le temps du dessin */
	wind_update(END_MCTRL);      /* On coupe l'AES le temps du dessin */
	return VRAI;
}

/* ----------- Gestion des messages pour cette fentre ----------------------------------- */

boolean MSG_aberrance(Fenetre *fenetre)
{
  ABER_param *param_aber;
  Graphe *resultat,*source;
  double x,y,*b,*c,*d,*tableau;
  register double *zone_x,*zone_y;
  int reponse;
  unsigned long i,nbr_pts;

  switch (buffer[0]) /* Message  considrer */
  {
  case WM_CLOSED: /* Fermeture de la fentre */
    reponse = EXF_ERREUR(FIN_MODULE);
    if (reponse == 3) return VRAI; /* On ne fait rien */

    param_aber = (ABER_param *) (fenetre->adresse);
    source = param_aber->graphe;

    if (reponse == 2) /* On ne veut pas liminer l'aberrance */
    {
      ferme_fenetre(fenetre);
      if (TT_exafs) /* On est dans une chane de traitement d'E.X.A.F.S. */
      {
	MU3_ouvre(source, mu0_soustrait); /* tape suivante */
      }
      return VRAI;
    }

    /* On limine l'aberrance - suppose le graphe tri */
	
    resultat = creer_graphe(source->nombre_point,source->type);
    if (resultat == NULL) /* On quitte sans fermer, pour que l'utilisateur ne perde rien */
      return VRAI;
    
    zone_x = source->x; zone_y = resultat->y;

    if ( OBJ_choisit(ELIM_ABERRANCE, ABER_SPLINE, QUESTION) ) /* On veut une spline */
    {
      SPL_prepare(source,2,param_aber->x_borne,&b,&c,&d,&tableau,&nbr_pts); /* On la prpare */
      for (i = 0 ; i < source->nombre_point ; i++)
      {
	x = *(zone_x);
	*(resultat->x + i) = x;
	if ( ( x < param_aber->x_borne[1] ) || ( x > param_aber->x_borne[2] ) )
	  y = *(source->y + i);
	else
	{
	  y = SPL_calcule(x,nbr_pts,tableau,b,c,d);
	}
	*zone_y = y;
	zone_x++; zone_y++;
      }
      free(b); free(c); free(d); free(tableau); /* On libre tout */
    }
    else /* On veut un polynme, les coefficients en sont dj calculs */
    {
      for (i = 0 ; i < source->nombre_point ; i++)
      {
	x = *(zone_x);
	*(resultat->x + i) = x;
	if ( ( x < param_aber->x_borne[1] ) || ( x > param_aber->x_borne[2] ) )
	  y = *(source->y + i);
	else
	{
	  y = param_aber->coefs[0];
	  for (reponse = 1 ; reponse <= param_aber->degre ; reponse++)
	  {
	    y += param_aber->coefs[reponse] * x;
	    x *= *(zone_x);
	  }
	}
	*zone_y = y;
	zone_x++; zone_y++;
      }
    }

    resultat->x_min = source->x_min; /* Les bornes en X n'ont pas chang */
    resultat->x_max = source->x_max;
    resultat->y_min = *(resultat->y);
    resultat->y_max = *(resultat->y);
    zone_y = resultat->y + 1;
    for (i = 1 ; i < resultat->nombre_point ; i++)
    {
      if ( *(zone_y) < resultat->y_min) resultat->y_min = *zone_y;
      if ( *(zone_y) > resultat->y_max) resultat->y_max = *zone_y;
      zone_y++;
    }
    
    strcpy(resultat->axe_x,source->axe_x); /* Les lgendes ne sont pas changes */
    strcpy(resultat->axe_y,source->axe_y);

    sprintf(_intermediaire,"%s corrig"e_AIGU, source->nom);
    GRF_titre(resultat, _intermediaire);
    
    ferme_fenetre(fenetre); /* On ferme la fentre */

    if (TT_exafs)
    {
      int reponse = EXF_ERREUR(OTE_ABERRANCE);
      if (reponse == 1)
	MU3_ouvre(resultat, mu0_soustrait);
      else
	aberrance(resultat);
    }
  }
  return VRAI;
}
