/* LASE = Logiciel d'analyse des spectres EXAFS
   (c) 1998, Emmanuel CURIS - Cration de ce fichier : 1er juillet 1998
   Dernire modification : 1er juillet 1998

   Tout ce qui concerne l'import de fichiers contenant des graphes  des formats autres

   Import des fichiers sortis par FEFF
   Rcupr de FICHIERS.c
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "GRAPHES.H"
#include "ERREURS.H"
#include "EXAFS.H"

static FILE *fichier; /* transitoire, il va falloir amliorer */

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

static long          passe_en_tete(FILE *fichier);
static unsigned long compte_lignes(FILE *fichier);

/* ****** Ralisation effective des fonctions ******************** */

boolean fef_chip(char *nom, boolean mode);
boolean fef_feff(char *nom);
boolean fef_xmu (char *nom);

/* ------ Paramtrage de l'import de fichier FEFF ----------------- */

boolean import_feff(void)
{
  OBJECT *adresse = arbres[IMPORTE_FEFF].adresse;
  char **liste_noms;
  int retour, index, nbr_fichier = 0;
  boolean redessin = FAUX;

  retour = ext_formdo(IMPORTE_FEFF);
  if (retour == FEFF_ANNULE) return FAUX; /* On annule le chargement */

  if ( OB_STATE(adresse,FEFF_FEFF) & SELECTED )	/* Import d'un fichier FEFFnnnn.DAT */
  {
    liste_noms = CH_selecteur("feff????.DAT", "Charger feffnnnn.dat", 0, NULL, &nbr_fichier);
    if ( OBJ_choisit(IMPORTE_FEFF, FEFF_UNIQUE, QUESTION) ) /* On ne charge que les fichiers choisis */
    {
      unsigned int i;

      if (liste_noms == NULL)
	return FAUX; /* On ne veux charger aucun fichier */

      for (i = 0 ; i < nbr_fichier ; i++)
      {
	sprintf(_intermediaire, "%s%s", nom_fichier, liste_noms[i]);
	redessin |= fef_feff(_intermediaire);
	free(liste_noms[i]);
      }
      free(liste_noms);
    }
    else /* On charge tous les fichiers du rpertoire */
    {
      DTA info_fichier, *prg_dta;
      char *nm_fich = "feff????.dat"; /* Masque de recherche */

      if (liste_noms)
      {
	unsigned int i;
	for (i = 0 ; i < nbr_fichier ; i++)
	  free(liste_noms[i]);
      }
      free(liste_noms); /* Ne sert  rien -> on libre tout */

      prg_dta = (DTA *) Fgetdta();
      Fsetdta(&info_fichier);

      index = Fsfirst(nm_fich, 0x20);
      while (index == 0)
      {
	sprintf(_intermediaire,"%s%s%c",nom_fichier,info_fichier.d_fname,0);
	redessin |= fef_feff(_intermediaire);
	index = Fsnext();
      }

      Fsetdta( CONVERT_DTA prg_dta); /* On remet l'ancienne DTA */
    }
    
    return redessin;
  }

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_CHIP, QUESTION) ) /* Import d'un fichier CHIPnnnn.DAT */
  {
    liste_noms = CH_selecteur("chip????.dat", "Charger chipnnnn.dat", 0, NULL, &nbr_fichier);
    if ( OBJ_choisit(IMPORTE_FEFF, FEFF_UNIQUE, QUESTION) ) /* On ne charge que les fichiers choisis */
    {
      if (liste_noms == NULL)
	return FAUX; /* On ne veux charger aucun fichier */
      
      for (index = 0 ; index < nbr_fichier ; index++)
      {
	sprintf(_intermediaire, "%s%s", nom_fichier, liste_noms[index]);
	redessin |= fef_chip(_intermediaire, VRAI);
	free(liste_noms[index]);
      }
      free(liste_noms);
    }
    else /* On charge tous les fichiers du rpertoire */
    {
      DTA info_fichier, *prg_dta;
      char *nm_fich = "CHIP????.DAT"; /* Masque de recherche */

      if (liste_noms)
      {
	for (index = 0 ; index < nbr_fichier ; index++)
	  free(liste_noms[index]);
      }
      free(liste_noms); /* Ne sert  rien -> on libre tout */
      
      prg_dta = (DTA *) Fgetdta();
      Fsetdta(&info_fichier);

      index = Fsfirst(nm_fich, 0x20);
      while (index == 0)
      {
	sprintf(_intermediaire,"%s%s%c",nom_fichier,info_fichier.d_fname,0);
	redessin |= fef_chip(_intermediaire, VRAI);
	index = Fsnext();
      }

      Fsetdta(CONVERT_DTA prg_dta); /* On remet l'ancienne DTA */
    }

    return redessin;
  }

  if ( OB_STATE(adresse, FEFF_CHI) & SELECTED )
  {
    liste_noms = CH_selecteur("CHI.DAT", "Charger CHI.DAT", 0, NULL, &nbr_fichier);
    if (liste_noms == NULL)
      return FAUX; /* On ne veux charger aucun fichier */

    for (index = 0 ; index < nbr_fichier ; index++)
    {
      sprintf(_intermediaire,"%s%s%c",nom_fichier,liste_noms[index],0);
      redessin |= fef_chip(_intermediaire, FAUX);
      free(liste_noms[index]);
    }
    free(liste_noms);
    
    return redessin;
  }

  if ( OB_STATE(adresse, FEFF_XMU) & SELECTED ) /* Import d'un fichier XMU.DAT */
  {
    liste_noms = CH_selecteur("XMU.DAT", "Charger XMU.DAT (FEFF)", 0, NULL, &nbr_fichier);
    if (liste_noms == NULL)
      return FAUX; /* On ne veux charger aucun fichier */

    for (index = 0 ; index < nbr_fichier ; index++)
    {
      sprintf(_intermediaire,"%s%s%c",nom_fichier,liste_noms[index],0);
      redessin |= fef_xmu(_intermediaire);
      free(liste_noms[index]);
    }
    free(liste_noms);

    return redessin;
  }
	
  return FAUX; /* On n'a rien import */
}

/* ------ Import d'un fichier CHIPnnnn.DAT ------------------------ */

boolean fef_chip(char *nom, boolean mode)
{
  int car_lu, vx_car;
  unsigned long pos_debut;
  Graphe *(graphe[4]) = {NULL, NULL, NULL, NULL};
  unsigned long i, nombre_lignes = 0;
  boolean debut_ligne = VRAI;
  float k,val_lue[4];

  fichier = fopen(nom, "rb"); /* Ouverture en binaire pour les fins de ligne */
  if (fichier == NULL)
    return FAUX; /* Erreurs  l'ouverture */

  /* On passe l'en-tte */

  vx_car = fgetc(fichier);
  car_lu = fgetc(fichier); /* Deux premiers caractres */
  while ( (vx_car != '@') || (car_lu != '#') )
  {
    vx_car = car_lu;
    car_lu = fgetc(fichier);
    if (car_lu == EOF)
    {
      printf("Import FEFF : ERREUR - fin de l'en-tte non dtecte, fin de fichier atteinte");
      printf("Fichier tronqu ou fichier de format autre que FEFF\n");
      fclose(fichier);
      return FAUX;
    }
  }
  while ( (car_lu = fgetc(fichier)) >= 32 ); /* On va  la fin de la ligne */
  while ( car_lu < 32 ) /* Code de contrle */
    car_lu = fgetc(fichier);

  pos_debut = ftell(fichier); /* On marque la fin de l'en-tte */

  /* Dcompte du nombre de lignes de donnes */

  while	( feof(fichier) == FAUX ) /* Tant que l'on n'est pas  la fin */
  {
    int car = fgetc(fichier);
    if ( (car == 10) || (car == 13) )
    {
      if (debut_ligne == FAUX) /* Code de fin de ligne ailleurs qu'en dbut de ligne */
      {
	debut_ligne = VRAI;
	nombre_lignes++;
      }
    }
    else if ( ( (car < '0') || (car > '9') )
	    && (car != ' ') && (car != '.') && (car != '+') && (car != '-')
	    && (car != 9  ) && (car != 'e') && (car != 'E') && (car != EOF) )
    {
      if ( EXT_ERREUR(CAR_INCONNU, car) == 1 ) /* On arrte - sinon, on ignore */
      {
	fclose(fichier);
	return FAUX;
      }
    }
    else
      debut_ligne = FAUX;
  }
  if (nombre_lignes < 2) /* Trop peu de points */
  {
    ERREUR(TROP_PEU_POINTS);
    fclose(fichier);
    return FAUX;
  }

  /* On revient au dbut des donnes */

  fseek(fichier, pos_debut, SEEK_SET);
	
  /* On cre le graphe correspondant */

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_EXAFS, QUESTION) ) /* On veut conserver le signal EXAFS */
  {
    char titre[100];

    sprintf(titre, "%s - Signal E.X.A.F.S.", nom_fichier); /* Premire colonne : EXAFS */
    graphe[0] = GRF_creer(nombre_lignes, GR_EXAFS, GR_NORMAL, titre, "Nombre d'onde", "khi(k)",
			  UNIT_1_ANG, UNIT_NORM);
  }

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_AMP, QUESTION) )  /* On veut conserver l'amplitude */
  {
    char titre[100];

    sprintf(titre, "%s - Amplitude du signal E.X.A.F.S.", nom_fichier);
    graphe[1] = GRF_creer(nombre_lignes, GR_EXAFSAMP, GR_NORMAL, titre,
			  "Nombre d'onde", "Amplitude",
			  UNIT_1_ANG, UNIT_NORM);
  }

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_PHASE, QUESTION) )   /* On veut conserver la phase */
  {
    char titre[100];

    sprintf(titre, "%s - Phase du signal E.X.A.F.S.", nom_fichier);
    graphe[2] = GRF_creer(nombre_lignes, GR_EXAFSPHA, GR_NORMAL, titre,
			  "Nombre d'onde", "D"e_AIGU"phasage",
			  UNIT_1_ANG, UNIT_NORM);
  }

  if ( ( OBJ_choisit(IMPORTE_FEFF, FEFF_PHIM2KR, QUESTION) ) /* On veut garder phi - 2kr */
    && (mode == VRAI) )                                /* Qui ne se trouve que dans CHIPnnnn.DAT */
  {
    char titre[100];

    sprintf(titre, "%s - D"e_AIGU"phasage du signal E.X.A.F.S.", nom_fichier);
    graphe[3] = GRF_creer(nombre_lignes, GR_EXAFSPHA, GR_NORMAL, titre,
			  "Nombre d'onde", "D"e_AIGU"phasage",
			  UNIT_1_ANG, UNIT_NORM);
  }

  /* On lit maintenant le fichier */

  for (i = 0 ; i < nombre_lignes ; i++)
  {
    int a;

    if (mode == VRAI)
      fscanf(fichier,"%f %f %f %f %f",&k,&val_lue[0],&val_lue[1],&val_lue[2],&val_lue[3]);
    else
      fscanf(fichier,"%f %f %f %f", &k, &val_lue[0], &val_lue[1], &val_lue[2]);

    for (a = 0 ; a < 4 ; a++)
    {
      if ( graphe[a] ) /* Le graphe existe */
      {
	*(graphe[a]->x + i) = (double) k;
	*(graphe[a]->y + i) = (double) val_lue[a];
      }
    }
  }
	
  /* Recherche des bornes */

  for (i = 0 ; i < 4 ; i++) /* En X : les fichiers sont tris... */
    GRF_bornes(graphe[i], ORDONNE, DESORDONNE); /* Pas de pb. si NULL */

  return VRAI; /* Fini, pas de problme */
}

/* ------------------ Import d'un fichier FEFFnnnn.DAT -------------------------\
 |   Renvoie VRAI si le fichier a pu tre charg
 |
 |  nom : nom du fichier  charger
 \----------------------------------------------------------------------------- */

boolean fef_feff(char *nom)
{
  int car[2], a;
  Graphe *graphe[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
  long lg_entete; /* Longueur de l'en-tte   */
  unsigned long i, nbr_lignes = 0;
  boolean fin=FAUX, compter;
  boolean debut_ligne = VRAI;
  float k, val_lue[6];
  /*  OBJECT *adresse = arbres[IMPORTE_FEFF].adresse; */

  fichier = fopen(nom, "rb");
  if (fichier == NULL) return FAUX; /* Erreur  l'ouverture */

  /* On cherche le dbut des donnes */

  car[0] = 0; car[1] = 0;
  while ( ! ( (car[0] == '@') && (car[1] == '#') ) )
  {
    car[0] = car[1];
    car[1] = fgetc(fichier);

    if (car[1] == EOF)
    {
      printf("Erreur : fin de fichier atteinte, marque @# non trouve !\n");
      printf("tes-vous sr qu'il s'agisse d'un fichier FEFF ?\n");
      fclose(fichier);
      return FAUX;
    }
  }

  lg_entete = ftell(fichier); /* On repre le dbut des donnes */

  /* On compte le nombre de lignes */

  do
  {
    car[0] = fgetc(fichier); /* On lit un caractre */
	
    if ( debut_ligne == VRAI )
    {
      if ( ( car[0] == 10 ) || ( car[0] == 13 ) )
      /* Ignore les lignes vides -> permet de lire le format PC : 13/10 */
      {
	compter = FAUX; /* On ne compte pas la ligne */
      }
      else
	compter = VRAI; /* On compte la ligne */

      debut_ligne = FAUX;
    }

    if ( (car[0] == 10) || (car[0] == 13) ) /* Fin de ligne, normalement tout type */
    {
      debut_ligne = VRAI; /* On va commencer une ligne */

      if (compter) /* On utilise cette ligne */
      {
	nbr_lignes++; /* Une ligne de plus */
      }
    }
  } while (car[0] != EOF); /* Tant qu'il y a des donnes */

  if (nbr_lignes < 2) /* Trop peu de donnes */
  {
    printf("Le fichier ne contient pas assez de lignes\n");
    printf("Appuyez sur une touche pour continuer\n");
    evnt_keybd();

    fclose(fichier);
    return FAUX;
  }

  /* On revient au dbut du fichier, aprs l'en-tte */
  
  fseek(fichier, lg_entete, SEEK_SET);
	
  /* On cr les graphes correspondants */

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_REPHC, QUESTION) )  /* On garde Re(phase atome central) */
  {
    char titre[200];

#ifdef _FRANCAIS_
    sprintf(titre, "D"e_AIGU"phase de l'atome central, %s\n", nom);  
    graphe[0] = GRF_creer(nbr_lignes, GR_EXAFSPHA, GR_NORMAL, titre,
			  "Nombre d'onde", "D"e_AIGU"phasage", UNIT_1_ANG, SS_UNITE);
#endif
#ifdef _ENGLISH_
    sprintf(titre, "Phase shift for central atom, %s\n", nom);  
    graphe[0] = GRF_creer(nbr_lignes, GR_EXAFSPHA, GR_NORMAL, titre,
			  "Wave number", "Phase shift", UNIT_1_ANG, SS_UNITE);
#endif
  }

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_AMP, QUESTION) )    /* On garde l'amplitude */
  {
    char titre[200];

#ifdef _FRANCAIS_
    sprintf(titre, "Amplitude, %s\n", nom);  
    graphe[1] = GRF_creer(nbr_lignes, GR_EXAFSAMP, GR_NORMAL, titre,
			  "Nombre d'onde", "D"e_AIGU"phasage", UNIT_1_ANG, SS_UNITE);
#endif
#ifdef _ENGLISH_
    sprintf(titre, "Amplitude, %s\n", nom);  
    graphe[1] = GRF_creer(nbr_lignes, GR_EXAFSAMP, GR_NORMAL, titre,
			  "Wave number", "Amplitude", UNIT_1_ANG, SS_UNITE);
#endif
  }

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_PHASE, QUESTION) ) /* On garde la phase */
  {
    char titre[200];

#ifdef _FRANCAIS_
    sprintf(titre, "D"e_AIGU"phase du r"e_AIGU"flecteur, %s\n", nom);  
    graphe[2] = GRF_creer(nbr_lignes, GR_EXAFSPHA, GR_NORMAL, titre,
			 "Nombre d'onde", "D"e_AIGU"phasage", UNIT_1_ANG, SS_UNITE);
#endif
#ifdef _ENGLISH_
    sprintf(titre, "Phase shift for scattering atom, %s\n", nom);  
    graphe[2] = GRF_creer(nbr_lignes, GR_EXAFSPHA, GR_NORMAL, titre,
			 "Wave number", "Phase shift", UNIT_1_ANG, SS_UNITE);
#endif
  }

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_REDFAC, QUESTION) ) /* On garde le facteur de rduction S0^2 */
  {
    char titre[200];

#ifdef _FRANCAIS_
    sprintf(titre, "Facteur de r"e_AIGU"duction, %s\n", nom);  
    graphe[3] = GRF_creer(nbr_lignes, GR_FAC_RED, GR_NORMAL, titre,
			  "Nombre d'onde", "S0"DEUX_EXPOSANT, UNIT_1_ANG, SS_UNITE);
#endif
#ifdef _ENGLISH_
    sprintf(titre, "Reduction factor, %s\n", nom);  
    graphe[3] = GRF_creer(nbr_lignes, GR_FAC_RED, GR_NORMAL, titre,
			  "Wave number", "S0"DEUX_EXPOSANT, UNIT_1_ANG, SS_UNITE);
#endif
  }

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_LIBRE, QUESTION) )  /* On garde le libre parcours moyen */
  {
    char titre[200];

#ifdef _FRANCAIS_
    sprintf(titre, "Libre parcours moyen, %s\n", nom);  
    graphe[4] = GRF_creer(nbr_lignes,  GR_LIBREP, GR_NORMAL, titre,
			  "Nombre d'onde", "Libre parcours", UNIT_1_ANG, UNIT_ANG);
#endif
#ifdef _ENGLISH_
    sprintf(titre, "Mean free path, %s\n", nom);  
    graphe[4] = GRF_creer(nbr_lignes, GR_LIBREP, GR_NORMAL, titre,
			  "Wave number", "Mean free path", UNIT_1_ANG, UNIT_ANG);
#endif
  }

  if ( OBJ_choisit(IMPORTE_FEFF, FEFF_RE_PH, QUESTION) )  /* On garde la partie relle de p */
  {
    char titre[200];

#ifdef _FRANCAIS_
    sprintf(titre, "Re(p), %s\n", nom);  
    graphe[5] = GRF_creer(nbr_lignes,  GR_AUTRETYPE, GR_NORMAL, titre,
			  "Nombre d'onde", "Re(p)", UNIT_1_ANG, UNIT_ARB);
#endif
#ifdef _ENGLISH_
    sprintf(titre, "Re(p), %s\n", nom);  
    graphe[5] = GRF_creer(nbr_lignes, GR_AUTRETYPE, GR_NORMAL, titre,
			  "Wave number", "Re(p)", UNIT_1_ANG, UNIT_ARB);
#endif
  }

  /* On lit maintenant le fichier */

  for (i = 0 ; i < nbr_lignes ; i++)
  {
    fscanf(fichier, "%f %f %f %f %f %f %f",
	   &k, &val_lue[0], &val_lue[1], &val_lue[2], &val_lue[3], &val_lue[4], &val_lue[5]);
    for (a = 0 ; a < 6 ; a++)
    {
      if ( graphe[a] ) /* Le graphe existe */
      {
	*(graphe[a]->x + i) = (double) k;
	*(graphe[a]->y + i) = (double) val_lue[a];
      }
    }
  }
	
  /* Recherche des bornes */
	
  for (a = 0 ; a < 6 ; a++) /* En X : les fichiers sont tris... */
  {
    if ( graphe[a] )
    {
      GRF_bornes(graphe[a], ORDONNE, DESORDONNE);
    }
  }

  /* On met  jour les titres et types de spectres */

  return VRAI; /* Fini, pas de problme */
}

/* ------------------ Import d'un fichier XMU.DAT de FEFF --------------------- */

boolean fef_xmu(char *nom)
{
  OBJECT *adresse = arbres[IMPORTE_FEFF].adresse;
  Graphe *(graphe[3]) = {NULL, NULL, NULL};
  char car[2];
  int i;
  double energie,e_rel,k,valeurs[3]; /* Valeurs lues dans le fichier */
  boolean fin = FAUX;
  unsigned int lg_entete = 0;
  unsigned long nombre_lignes = 0, index;

  fichier = fopen(nom, "r");
  if (fichier == NULL) return FAUX; /* Erreur  l'ouverture */
	
  car[0] = 0 ; car[1] = 0;
  while (fin == FAUX) /* FEFFnnnn.DAT : 10 lignes d'en-tte (?) */
  {
    car[1] = fgetc(fichier);
    if ( (car[0] == '@') && (car[1] == '#') ) /* Fin de l'en-tte */
    {
      fin = VRAI;
      while (fgetc(fichier) != 10) /* On va  la fin de la ligne */
	lg_entete++;
    }
    if (car[1] == 10) /* Ce stupide fgetc transforme les 13,10 en 10!!! */
      lg_entete++;

    lg_entete++;
    car[0] = car[1];
  }
	
  /* Dcompte du nombre de lignes de donnes */

  while	( feof(fichier) == FAUX ) /* Tant que l'on n'est pas  la fin */
  {
    if (fgetc(fichier) == 10)
      nombre_lignes++;
  }

  /* On revient au dbut du fichier, aprs l'en-tte */

  fseek(fichier, lg_entete, SEEK_SET);
  
  /* On cr les graphes dsirs */

  if ( OB_STATE(adresse, FEFF_ABSORBE) & SELECTED )
  {
    graphe[0] = creer_graphe(nombre_lignes, GR_MOINSMAT); /* Absorption totale */
    graphe[1] = creer_graphe(nombre_lignes, GR_MU0);      /* Mu0 soustrait (prseuil compris) */
  }
  if ( OB_STATE(adresse, FEFF_EXAFS) & SELECTED )        /* On veut le signal EXAFS */
    graphe[2] = creer_graphe(nombre_lignes, GR_EXAFS);

  /* On lit maintenant le fichier */

  for (index = 0 ; index < nombre_lignes ; index++)
  {
    fscanf(fichier,"%lf %lf %lf %lf %lf %lf",&energie,&e_rel,&k,&valeurs[0],&valeurs[1],&valeurs[2]);
    for (i = 0 ; i < 3 ; i++)
    {
      if (graphe[i]) /* Le graphe existe : on veut le charger */
      {
	if ( OB_STATE(adresse, FEFF_K) & SELECTED )
	  *(graphe[i]->x + index) = k;
	else if ( OB_STATE(adresse, FEFF_ENERGIE) & SELECTED )
	  *(graphe[i]->x + index) = energie;
	else
	  *(graphe[i]->x + index) = e_rel;
	
	*(graphe[i]->y + index) = valeurs[i];
      }
    }
  }
	
  /* On met  jour les titres et types de spectres */

  if ( graphe[0] ) /* On a gard tout le signal */
  {
    GRF_bornes(graphe[0], ORDONNE, DESORDONNE);
    
    sprintf(titre,"%s - signal calcul%c",nom,0); /* Premire colonne : signal total */
    strncpy(graphe[0]->nom, titre, LG_TITRE);

    if ( OB_STATE(adresse, FEFF_K) & SELECTED )
      sprintf(graphe[0]->axe_x, "Nombre d'onde (1/)%c", 0);
    else if ( OB_STATE(adresse, FEFF_ENERGIE) & SELECTED )
      sprintf(graphe[0]->axe_x, "nergie (eV)%c", 0);
    else
      sprintf(graphe[0]->axe_x, "E / seuil (eV)%c", 0);
  }

  if ( graphe[1] ) /* On a gard le mu0 calcul */
  {
    GRF_bornes(graphe[1], ORDONNE, DESORDONNE);
    
    sprintf(titre,"%s - %c0%c", nom, 224,0); /* Premire colonne : signal total */
    strncpy(graphe[1]->nom, titre, LG_TITRE);
    if ( OB_STATE(adresse, FEFF_K) & SELECTED )
      sprintf(graphe[1]->axe_x, "Nombre d'onde (1/)%c", 0);
    else if ( OB_STATE(adresse, FEFF_ENERGIE) & SELECTED )
      sprintf(graphe[1]->axe_x, "nergie (eV)%c", 0);
    else
      sprintf(graphe[1]->axe_x, "E / seuil (eV)%c", 0);
  }

  if ( graphe[2] ) /* On a gard l'EXAFS */
  {
    GRF_bornes(graphe[2], ORDONNE, DESORDONNE);

    sprintf(titre,"%s - signal EXAFS",nom); /* Premire colonne : signal total */
    strncpy(graphe[2]->nom,titre, LG_TITRE);
    if ( OB_STATE(adresse, FEFF_K) & SELECTED )
      sprintf(graphe[2]->axe_x, "Nombre d'onde (1/)%c", 0);
    else if ( OB_STATE(adresse, FEFF_ENERGIE) & SELECTED )
      sprintf(graphe[2]->axe_x, "nergie (eV)%c", 0);
    else
      sprintf(graphe[2]->axe_x, "E / seuil (eV)%c", 0);
  }

  return VRAI; /* Fini, pas de problme */
}

/* ~~~~~~~~ Fonctions "macros" ~~~~~~~~ */

/* -------- On passe l'en-tte FEFF : on lit jusqu' tomber sur @#
            qui marque la fin d'un en-tte dans un fichier FEFF normal */

static long passe_en_tete(FILE *fichier)
{
  int car[2];

  car[0] = 0; car[1] = 0;
  while ( ! ( (car[0] == '@') && (car[1] == '#') ) )
  {
    car[0] = car[1];
    car[1] = fgetc(fichier);

    if (car[1] == EOF)
    {
      printf("Erreur : fin de fichier atteinte, marque @# non trouve !\n");
      printf("tes-vous sr qu'il s'agisse d'un fichier FEFF ?\n");
      fclose(fichier);
      return FAUX;
    }
  }

  return ftell(fichier); /* On repre le dbut des donnes */
}
