/* Traitement de l'E.X.A.F.S. sur Atari
		   Tout ce qui concerne l'impression (GDOS, SpeedoGDOS ou NVDI obligatoire)
   (c) 1996 - Emmanuel CURIS

   25 mars 1998
     * Inclusion de "systeme.h"

   12 & 13, 16 janvier 1998
     * Corrig rotation des textes -> graphes superposs devraient avoir des lgendes correctes
     * Modifi mode d'appel de la fonction gnrale -> superposition passe par elle
     * Utilisation de Station_VDI
     * Impression de graphes superposs ok, mais dans le sens non tourn (-> mode paysage NVDI  utiliser)
     * Supprim appel aux objets de choix si axes imprims ou non, gradus ou non

	 22 novembre 1997
	   * Cration de cet historique
	   * Ajout inclusion de gem_aes.h et gem_vdi.h -> devrait compiler avec GCC
	   * Corrig 2 dtails donnant un avertissement sous GCC
*/

#define _IMPRIME_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"
#include "IMPRIME.H"

int imp_handle; /* Handle de la station VDI de l'imprimante */
int nbr_fontes; /* Nombre de fontes utilisables (GDOS)      */
int num_fonte;  /* Fonte utilise pour les textes           */
int imp_x,imp_y,imp_larg,imp_haut; /* Dimensions de la zone d'impression, hors textes */
int haut_txt; /* Hauteur des caractres du texte */

/* -------------- Impression des graphes (utilise GDOS) ------------------------------ *
 *   On a le choix du pilote, parmi 21  30
 ***************************************************************************************/

/* -------------- Paramtrages gnraux d'impression + routine gnrique --------------\
 |   Ne renvoie rien                                                                   |
 |                                                                                     |
 | type : VRAI = on superpose les graphes; FAUX = on trace seulement le graphe courant |
 \------------------------------------------------------------------------------------ */

void imprimer(int type)
{
	Station_VDI *IMP_station;
	int retour, i, pilote = 0;

	/* Si type = superposition, pas de marge */

	OBJ_active(PARAM_IMPRIME, IMP_MARGE_DROITE, !type);
	OBJ_active(PARAM_IMPRIME, IMP_MARGE_GAUCHE, !type);
	OBJ_active(PARAM_IMPRIME, IMP_MARGE_HAUTE,  !type);
	OBJ_active(PARAM_IMPRIME, IMP_MARGE_BASSE,  !type);

	/* On regarde quels sont les pilotes d'impression installs */

	for (i = 21 ; i < 31 ; i++)
	{
		int present;
		char nom[50];

		vqt_devinfo(_PRG_station->handle, i, &present, nom);
		if (present == DEV_INSTALLED)
		{
			if (pilote == 0) /* Pilote slectionn : premier pilote prsent */
			{
				pilote = i;
				OBJ_choisit(PARAM_IMPRIME, PILOTE_1 + i - 21, FORCE_OUI);
			}
			else
				OBJ_choisit(PARAM_IMPRIME, PILOTE_1 + i - 21, FORCE_NON);
				
			OBJ_active(PARAM_IMPRIME, PILOTE_1 + i - 21, FORCE_OUI);
			met_chaine(PARAM_IMPRIME, PILOTE_1 + i - 21, 12, nom, TXT_COPIER);
		}
		else
		{
			OBJ_active(PARAM_IMPRIME, PILOTE_1 + i - 21, FORCE_NON);
			met_chaine(PARAM_IMPRIME, PILOTE_1 + i - 21, 12, "Absent", TXT_COPIER);
		}
	}

	if (pilote == 0) /* Aucun pilote d'impression prsent */
	{
		form_alert(1,"[2][ Aucun pilote d'impression | n'est disponible ! ][ Annuler ]");
		return;
	}

	if ( ext_formdo(PARAM_IMPRIME) == IMP_ANNULE ) /* On annule l'impression */
		return;

	/* On cherche le pilote slectionn */
	for (i = PILOTE_1 ; i < PILOTE_10 ; i++)
	{
		if ( OBJ_choisit(PARAM_IMPRIME, i, QUESTION) == VRAI ) /* Choisi */
		{
			pilote = i - PILOTE_1 + 21;
			break;
		}
	}

	IMP_station = VDI_ouvre(pilote);
	imp_handle = IMP_station->handle;

	vst_height(IMP_station->handle, 50,&i,&i,&haut_txt,&i);
	vst_color (IMP_station->handle, NOIR);  /* Les textes sont en noir                      */

	imp_larg = (int) ( 0.85 * (double) IMP_station->larg ); /* 9/10 de la largeur de la feuille */
	imp_haut = (int) ( 0.80 * (double) IMP_station->haut ); /* 4/5  de la hauteur de la feuille */
	imp_x = (IMP_station->larg - imp_larg) / 2; /* Marges X : 1/2 - 1/2 */
	imp_y = (IMP_station->haut - imp_haut) / 4; /* Marges Y : 1/4 - 3/4 */

	if (type) /* On veut superposer les graphes */
	{
   	Cadre *rect;
  	float x_facteur, y_facteur; /* Facteurs d'chelle */

		x_facteur = ( (float) IMP_station->larg ) / ( (float) trace_fenetre->w_larg );
		y_facteur = ( (float) IMP_station->haut ) / ( (float) trace_fenetre->w_haut );

		for (rect = prem_rectangle ; rect ; rect = rect->apres)
		{
			rect->x *= x_facteur;    rect->y *= y_facteur;
			rect->larg *= x_facteur; rect->haut *= y_facteur;

			switch (rect->type)
			{
			case REC_GRAPHE: /* Contient des graphes - seul cas possible pour l'instant... */
				{
					Rect_graphe *a_trace = (Rect_graphe *) rect->pointeur;

          a_trace->x1 *= x_facteur; a_trace->y1 *= y_facteur;
          a_trace->x2 *= x_facteur; a_trace->y2 *= y_facteur;

          DES_Rgraphe(IMP_station, NULL, rect);

          a_trace->x1 /= x_facteur; a_trace->y1 /= y_facteur;
          a_trace->x2 /= x_facteur; a_trace->y2 /= y_facteur;
				}
				break;
			}

			rect->x    /= x_facteur; rect->y    /= y_facteur;
			rect->larg /= x_facteur; rect->haut /= y_facteur;
    }
	}
	else /* On veut imprimer le graphe courant */
	{
		vst_rotation(IMP_station->handle, 900); /* Les textes sont tourns de 90 : mode paysage */

		/* Cadre, graduation, axes */

		imp_cadre(graphe_courant->x_min,graphe_courant->x_max,graphe_courant->y_min,graphe_courant->y_max,
	  	        VRAI, VRAI);

		/* Trac du graphe */
		imp_graphe(graphe_courant,graphe_courant->x_min,graphe_courant->x_max,graphe_courant->y_min,graphe_courant->y_max);

		/* Impression du titre        */
		vst_alignment(IMP_station->handle, ALI_CENTER,ALI_BOTTOM,&retour,&retour);
		v_gtext(IMP_station->handle,imp_x-1,imp_y + imp_haut/2,graphe_courant->nom);

		/* Impression de la lgende de l'axe X */
		v_gtext(IMP_station->handle,imp_x + imp_larg + 2 * haut_txt , imp_y + imp_haut/2,graphe_courant->axe_x);
	
		/* Impression de la lgende de l'axe Y */
		vst_rotation(IMP_station->handle,1800); /* Texte tourn de 180 degrs */
		v_gtext(IMP_station->handle,imp_x + imp_larg/2, IMP_station->haut - 3*haut_txt, graphe_courant->axe_y);
	}

	v_updwk(IMP_station->handle); /* On envoie  l'imprimante */

	VDI_ferme(IMP_station);
}

/* -------------- Impression d'un graphe dans la zone indique ----------------------------- */

void imp_graphe(Graphe *graphe,double x_min, double x_max, double y_min, double y_max)
{
	register Complexe *y_tf;
	register double *zone_x,*zone_y;
	double k,x_echelle,y_echelle;
	int compteur = 0;
	unsigned long index;

	vsl_type  (imp_handle, graphe->style);
	vsl_color (imp_handle, graphe->couleur); /* Couleur du graphe */
	vsm_type  (imp_handle, PM_ASTERISQUE);
	vsm_height(imp_handle, 20);

	x_echelle = (double) (imp_larg) / (y_max - y_min);
	y_echelle = (double) (imp_haut) / (x_max - x_min);

	if (graphe->type == GR_TF)
	{
		/* Trac de la partie imaginaire */
		zone_x =              graphe->x;
		y_tf   = (Complexe *) graphe->y;
		for (index = 0 ; index < graphe->nombre_point ; index++)
		{
			points[compteur]   = imp_x + (y_max - (*y_tf).im) * x_echelle;
			points[compteur+1] = imp_y + (x_max - (*zone_x))  * y_echelle;
			compteur += 2;
			zone_x++;
			y_tf++;
			if (compteur >= 256) /* Tableau intin plein */
			{
				v_pline(imp_handle,128,points); /* On trace ces 128 points */
				points[0] = points[254]; points[1] = points[255];
				compteur = 2;
			}
		}
		v_pline(imp_handle,(int) (compteur+1)/2,points); /* On trace ce qui reste */
		
		/* Trac du module, couleur suivante */
		compteur = 0;
		vsl_color(imp_handle,graphe->couleur + 1);
		zone_x =              graphe->x;
		y_tf   = (Complexe *) graphe->y;
		for (index = 0 ; index < graphe->nombre_point ; index++)
		{
			k = sqrt( (*y_tf).re * (*y_tf).re + (*y_tf).im * (*y_tf).im );
			points[compteur]   = imp_x + (y_max - k) * x_echelle;
			points[compteur+1] = imp_y + (x_max - *zone_x) * y_echelle;
			compteur += 2;
			zone_x++;
			y_tf++;
			if (compteur >= 256) /* Tableau intin plein */
			{
				v_pline(imp_handle,128,points); /* On trace ces 128 points */
				points[0] = points[254]; points[1] = points[255];
				compteur = 2;
			}
		}
		v_pline(imp_handle,(int) (compteur+1)/2,points); /* On trace ce qui reste */
	}
	else
	{
		zone_x = graphe->x;
		zone_y = graphe->y;
		for (index = 0 ; index < graphe->nombre_point ; index++)
		{
			points[compteur]   = imp_x + (y_max - (*zone_y)) * x_echelle;
			points[compteur+1] = imp_y + (x_max - (*zone_x)) * y_echelle;
			compteur += 2;
			zone_x++;
			zone_y++;
			if (compteur >= 256) /* Tableau intin plein */
			{
				v_pline(imp_handle,128,points); /* On trace ces 128 points */
/*				v_pmarker(imp_handle,128,points); */
				points[0] = points[254]; points[1] = points[255];
				compteur = 2;
			}
		}
		v_pline(imp_handle,(int) (compteur+1)/2,points); /* On trace ce qui reste */
/*		v_pmarker(imp_handle,128,points); */
	}
}

/* ------------ Impression d'un cadre, avec axes et graduations ---------------------------- */

void imp_cadre(double x_min, double x_max, double y_min, double y_max, boolean axes, boolean gradue)
{
	double echelle,pas,x_0;
	int rien,degre;
	char chaine[100];

	vsl_type (imp_handle,1); /* Ligne continue                               */
	vsl_color(imp_handle,1); /* On trace le cadre et les graduations en noir */

	/* Trac du cadre */

	points[0] = imp_x;            points[1] = imp_y;
	points[2] = imp_x + imp_larg;	points[3] = imp_y;
	points[4] = points[2];        points[5] = imp_y + imp_haut;
	points[6] = imp_x;            points[7] = points[5];
	points[8] = imp_x;            points[9] = imp_y;
	v_pline(imp_handle,5,points);
	
	/* Trac des axes, s'il est demand */
	
	if (axes == VRAI)
	{
		if ( (x_min < 0) && (x_max > 0) ) /* Trac de l'axe des ordonnes */
		{
			points[0] = imp_x;            points[1] = imp_y + x_max * imp_haut / (x_max - x_min);
			points[2] = imp_x + imp_larg; points[3] = points[1];
			v_pline(imp_handle,2,points);
		}

		if ( (y_min < 0) && (y_max > 0) ) /* Trac de l'axe des abscisses */
		{
			points[0] = imp_x + y_max * imp_larg / (y_max - y_min); points[1] = imp_y;
			points[2] = points[0];                                  points[3] = imp_y + imp_haut;
			v_pline(imp_handle,2,points);
		}
	}
	
	/* Impression des bornes en X */
	vst_alignment(imp_handle,ALI_CENTER,ALI_TOP,&rien,&rien); /* Pour les X */
	sprintf(chaine,"%.2f%c",x_min,0);
	v_gtext(imp_handle,imp_x+imp_larg+haut_txt,imp_y+imp_haut,chaine);
	sprintf(chaine,"%.2f%c",x_max,0);
	v_gtext(imp_handle,imp_x+imp_larg+haut_txt,imp_y,chaine);
	if (gradue == VRAI) /* On veut en plus graduer */
	{
		degre = (int) ( log10(x_max-x_min) );
		pas = pow(10,degre) / 10; /* Graduations tous les 10e */
		x_0 = (double) ((int) (x_min / pas))*pas;
		echelle = ( (double) imp_haut ) / (x_max - x_min);
		rien = (int) (x_min / pas);
		while (x_0 < x_max)
		{
			if ( (x_0 > x_min) && (x_0 < x_max) ) /* On est dans l'intervalle */
			{
				points[0] = imp_x + imp_larg; points[1] = imp_y + echelle * (x_max - x_0);
				points[3] = points[1];
				if ((rien % 5) == 0)    /* Multiple de 5 */
				{
					if ((rien % 10) == 0) /* Multiple de 10 */
					{
						points[2] = imp_x + imp_larg - 30;
						sprintf(chaine,"%.2f%c",x_0,0);
						v_gtext(imp_handle,imp_x + imp_larg + haut_txt,points[1],chaine);
					}
					else
						points[2] = imp_x + imp_larg - 20;
				}
				else
					points[2] = points[0] - 10;
				v_pline(imp_handle,2,points);
			}
			x_0 += pas;
			rien++;
		}
	}
	
	vst_alignment(imp_handle,ALI_RIGHT,ALI_HALF,&rien,&rien); /* Pour les Y */
	sprintf(chaine,"%.2f %c",y_min,0);
	v_gtext(imp_handle,imp_x+imp_larg-5,imp_y+imp_haut+5,chaine);
	sprintf(chaine,"%.2f %c",y_max,0);
	v_gtext(imp_handle,imp_x,imp_y+imp_haut+5,chaine);
	if (gradue == VRAI)
	{
		degre = (int) ( log10(y_max-y_min) );
		pas = pow(10,degre) / 10; /* Graduations tous les 10e */
		x_0 = (double) ((int) (y_min / pas))*pas;
		echelle = ( (double) imp_larg ) / (y_max - y_min);
		rien = (int) (y_min / pas);
		while (x_0 < y_max)
		{
			if ( (x_0 > y_min) && (x_0 < y_max) ) /* On est dans l'intervalle */
			{
				points[0] = imp_x + echelle * (y_max - x_0); points[1] = imp_y + imp_haut;
				points[2] = points[0];
				if ((rien % 5) == 0)    /* Multiple de 5 */
				{
					if ((rien % 10) == 0) /* Multiple de 10 */
					{
						points[3] = points[1] - 30;
						sprintf(chaine,"%.2f%c",x_0,0);
						v_gtext(imp_handle,points[0],imp_y + imp_haut + 5,chaine);
					}
					else
						points[3] = points[1] - 20;
				}
				else
					points[3] = points[1] - 10;
				v_pline(imp_handle,2,points);
			}
			x_0 += pas;
			rien++;
		}
	}
}
