/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1998, Emmanuel CURIS - cration du fichier : 3 juin 1998

   Rsolution d'un systme par la mthode du pivot de Gauss
   (Merci  Bruno Blanchet pour le source en pascal)
   Suppose que le systme  une solution
   Utilis pour les moindres carrs avec les polynmes

*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <SYSTEME.H>

#include <ERREURS.H>
#include <GRAPHES.H>
#include <LISSAGE.H>

/* #define _DEBUG_GAUSS_ */

/* *********** Fonction  usage interne *************************** */

#ifdef _DEBUG_GAUSS_
static void aff_matrice  (int degre, double *matrice);
#endif

static void combine        (int degre, double *matrice, int ligne, int l_2, double coef);
static void echange_colonne(int degre, double *matrice, int c_1, int c_2);
static void echange_ligne  (int degre, double *matrice, int l_1, int l_2);

/* ----------- Rsolution effective --------------------------------\
 | Renvoie  0 si tout va bien                                       |
 |         <0 si erreur                                             |
 |                                                                  |
 | degre   : le degr du polynme que l'on cherche                  |
 | matrice : matrice des coefficients (elle fait degre + 2 colonnes |
 |                                               degre + 1 lignes)  |
 | coefs   : le vecteur solution = coeff. du polynme               |
 \----------------------------------------------------------------- */

int Gauss(int degre, double *matrice, double *coefs)
{
  int i, j, k, l_pivot, c_pivot; /* i : colonne, j : ligne */
  int *idx_sol;

  idx_sol = (int *) calloc(degre + 1, sizeof(int));
  if (idx_sol == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    return -MANQUE_MEMOIRE;
  }

  /* Au dbut : inconnue 1 = coef 1, ..., inconnue n = coef n */

  for (i = 0 ; i <= degre ; i++)
    idx_sol[i] = i;

#ifdef _DEBUG_GAUSS_
  aff_matrice(degre, matrice);
#endif

  for (k = degre ; k >= 0 ; k--) /* Nombre de passages pour rendre le systme diagonal */
  {
    /* Recherche du pivot dans toute la matrice k * k */
    double pivot = 0;

    for (i = 0 ; i <= k ; i++)
      for (j = 0 ; j <= k ; j++)
      {
	double val = matrice[ (degre + 2) * j + i ]; /* Valeur de l'lment de matrice */
	if ( fabs(val) > fabs(pivot) ) /* On cherche le plus grand pivot... */
	{
	  l_pivot = j; /* Ligne du pivot   */
	  c_pivot = i; /* Colonne du pivot */
	  pivot = val;
	}
      }

    if ( pivot == 0 )
    {
      LIS_ERREUR(NON_INVERSIBLE);
      free(idx_sol);
      return -NON_INVERSIBLE;
    }
#ifdef _DEBUG_GAUSS_
    else
      printf("Pivot choisi : %f\n", pivot);
#endif

    /* On s'arrange pour placer le pivot dans le coin infrieur droit de la matrice */

    if (l_pivot != k) /* Il faut changer les lignes */
    {
      echange_ligne(degre, matrice, k, l_pivot);
#ifdef _DEBUG_GAUSS_
      printf("Aprs change des lignes :\n");
      aff_matrice(degre, matrice);
#endif
    }

    if (c_pivot != k) /* Il faut changer les colonnes */
    {
      int tmp = idx_sol[k];
      idx_sol[k] = idx_sol[c_pivot];
      idx_sol[c_pivot] = tmp; /* On change aussi les inconnues */

      echange_colonne(degre, matrice, k, c_pivot);

#ifdef _DEBUG_GAUSS_
      printf("Aprs change des colonnes :\n");
      aff_matrice(degre, matrice);
#endif
    }

    /* On annule la colonne k en combinant les lignes grce au pivot */

    for (j = 0 ; j <= degre ; j++)
      if ( j != k )
      {
	combine(degre, matrice, j, k, -matrice[ (degre + 2) * j + k ] / pivot );
	matrice[ (degre + 2) * j + k ] = 0; /* On force pour viter des erreurs d'arrondi aprs */
      }

#ifdef _DEBUG_GAUSS_
      printf("Aprs combinaison linaire des lignes :\n");
      aff_matrice(degre, matrice);
#endif
  }

  /* Ici, on a une matrice triangulaire + la colonne des y... */

  for ( i = 0 ; i <= degre ; i++ )
  {
    double tmp = matrice[ (degre + 2) * i + i ]; /* lment sur la diagonale */
    if ( tmp == 0 )
    {
      LIS_ERREUR(NON_INVERSIBLE);
      free(idx_sol);
      return -NON_INVERSIBLE;
    }

    coefs[idx_sol[i]] = matrice[ (degre + 2) * i + (degre + 1) ] / tmp;
  }

  free(idx_sol);

#ifdef ancienne_methode

  for (i = degre ; i >= 0 ; i--)
  {
    pivot = 0;

    for (j = 0 ; j <= i ; j++)
    {
      double val = matrice[ (degre + 2) * j + i ]; /* Valeur de l'lment de matrice */
#ifdef _DEBUG_GAUSS_
      /*      printf("lment lu : %e\n", val); */
#endif
      if ( fabs(val) > fabs(pivot) ) /* On cherche le plus grand pivot... */
      {
	l_pivot = j; /* Ligne du pivot */
	pivot = val;
      }
    }
    if ( pivot == 0 )
    {
      LIS_ERREUR(NON_INVERSIBLE);
      return -NON_INVERSIBLE;
    }
#ifdef _DEBUG_GAUSS_
    else
      printf("Pivot choisi : %f\n", pivot);
#endif

    /* On change la dernire ligne et la ligne du pivot */

    if (l_pivot != i) /* Si ncessaire... */
    {
      echange_ligne(degre, matrice, i, l_pivot);
#ifdef _DEBUG_GAUSS_
      printf("Aprs change des lignes :\n");
      aff_matrice(degre, matrice);
#endif
    }

    /* On combine les lignes pour faire sauter le pivot... */

    for (j = 0 ; j <= degre ; j++)
      if ( j != i )
      {
#ifdef _DEBUG_GAUSS_
	/*	printf("Coefficient  utiliser : %e\n", matrice[ (degre + 2) * j + i ] ); */
#endif
	combine(degre, matrice, j, i, -matrice[ (degre + 2) * j + i ] / pivot );
      }

#ifdef _DEBUG_GAUSS_
      printf("Aprs combinaison linaire des lignes :\n");
      aff_matrice(degre, matrice);
#endif
  }

  /* Ici, on a une matrice triangulaire + la colonne des y... */

  for ( i = 0 ; i <= degre ; i++ )
  {
    double tmp = matrice[ (degre + 2) * i + i ]; /* lment sur la diagonale */
    if ( tmp == 0 )
    {
      LIS_ERREUR(NON_INVERSIBLE);
      return -NON_INVERSIBLE;
    }

    coefs[i] = matrice[ (degre + 2) * i + (degre + 1) ] / tmp;
#ifdef _DEBUG_GAUSS_
    /*    printf("Coefficient %d : %e (%f) \n", i, coefs[i], coefs[i]); */
#endif
  }

#endif /* ancienne_methode */

  return 0; /* Pas de problme */
}

#ifdef _DEBUG_GAUSS_

static void aff_matrice  (int degre, double *matrice)
{
  int i, j;

  for (i = 0 ; i <= degre ; i++)
  {
    printf("( ");
    for (j = 0 ; j <= degre + 1 ; j++)
      printf("%e ", matrice[i * (degre + 2) + j]);
    printf(" )\n");
  }
}

#endif


/* ------------ Combinaison linaire de deux lignes de la matrice -----\
 |  Ne renvoie rien                                                    |
 |                                                                     |
 | Remplace la ligne "ligne" par "ligne" + "l_2" * coef                |
 \-------------------------------------------------------------------- */

static void combine(int degre, double *matrice, int ligne, int l_2, double coef)
{
  int i;
  int deb_ligne, deb_l2;

  deb_ligne = (degre + 2) * ligne;
  deb_l2 = (degre + 2) * l_2;

#ifdef _DEBUG_GAUSS_
  printf("On ralise L%d = L%d + %e  L%d\n", ligne, ligne, coef, l_2);
#endif

  for (i = 0 ; i < degre + 2 ; i++)
  {
#ifdef _DEBUG_GAUSS_
    /*    printf("On ralise : %e + %e * %e =", matrice[deb_ligne + i],
	   coef, matrice[deb_l2 + i]); */
#endif
    matrice[deb_ligne + i] += coef * matrice[deb_l2 + i];
#ifdef _DEBUG_GAUSS_
    /*    printf(" %e\n", matrice[deb_ligne + i]); */
#endif
  }
}

/* ------------ change de deux lignes de la matrice -------- */

static void echange_ligne(int degre, double *matrice, int l_1, int l_2)
{
  int i;
  int deb_l1, deb_l2;

  deb_l1 = (degre + 2) * l_1;
  deb_l2 = (degre + 2) * l_2;

  for (i = 0 ; i < degre + 2 ; i++)
  {
    double tmp = matrice[ deb_l1 + i ];
    matrice[ deb_l1 + i ] = matrice[ deb_l2 + i ];
    matrice[ deb_l2 + i ] = tmp;
  }
}

/* ------------ change de deux colonnes de la matrice -------- */

static void echange_colonne(int degre, double *matrice, int c_1, int c_2)
{
  int i;

  for (i = 0 ; i <= degre ; i++)
  {
    double tmp = matrice[ (degre + 2) *i + c_1 ];
    matrice[ (degre + 2) *i + c_1 ] = matrice[ (degre + 2) *i + c_2 ];
    matrice[ (degre + 2) *i + c_2 ] = tmp;
  }
}
