/* LASE - Logiciel d'analyse des spectres EXAFS
   (c) 1998, Emmanuel CURIS - Cration de ce fichier: 23 juin 1998

   Module de lissage des graphes par diffrentes mthodes

   Lissage d'un graphe quelconque par des splines (partie interface utilisateur)
   Source original rcupr de ../XANES.c
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <SYSTEME.H>

#include <EXAFS.H>
#include <GRAPHES.H>
#include <ERREURS.H>
#include <LISSAGE.H>

/* ******** Ralisation effective du lissage ************************** */

/* -------- Lissage complet d'un spectre rel --------------------------\
 |  Renvoie le graphe liss (NULL si erreur)
 |
 | source    : le graphe  lisser [doit tre rel, tri, sans soublons] |
 | nv_points : le nombre de points dans le graphe liss                 |
 \--------------------------------------------------------------------- */

Graphe *lisse_reel(Graphe *source, unsigned long nv_points)
{
  Graphe *resultat;
  double *b, *c, *d; /* Coefficients de la spline */
  double pas;
  unsigned long index;
  register double *zone_x, *zone_y;

  b = (double *) malloc( sizeof(double) * source->nombre_point );
  c = (double *) malloc( sizeof(double) * source->nombre_point );
  d = (double *) malloc( sizeof(double) * source->nombre_point );
  if ( (d == NULL) || (c == NULL) || (b == NULL) )
  {
    ERREUR(MANQUE_MEMOIRE);
    free(d); free(c); free(b);
    return NULL;
  }
	
  /* Prparation des tableaux de coefficients */
  zone_x = source->x;
  zone_y = source->y;
  if ( zone_x[0] == zone_x[1] )
  {
    EXF_ERREUR(ERR_DOUBLONS);
    free(b); free(c); free(d);
    return NULL;
  }
  if (source->nombre_point == 2) /* Deux points : en fait, on fait tracer une droite... */
  { 
    b[0] = ( zone_y[1] - zone_y[0] ) / ( zone_x[1] - zone_x[0] );
    c[0] = 0;
    d[0] = 0;
    b[1] = b[0];
    c[0] = 0;
    d[0] = 0;
  }
  else
  {
    d[0] = zone_x[1] - zone_x[0];
    c[1] = ( zone_y[1] - zone_y[0] ) / d[0];
    for (index = 1 ; index < source->nombre_point - 1 ; index++)
    {
      d[index]     = zone_x[index + 1] - zone_x[index];
      if ( d[index] == 0 )
      {
	EXF_ERREUR(ERR_DOUBLONS);
	free(b); free(c); free(d);
	return NULL;
      }
      b[index]     = 2 * ( d[index - 1] + d[index] );
      c[index + 1] = ( zone_y[index + 1] - zone_y[index] ) / d[index];
      c[index]     = c[index+1] - c[index];
    }
    b[0] = -d[0];
    b[source->nombre_point - 1] = -d[source->nombre_point - 2];
		
    if (source->nombre_point > 3)
    {
      if ( (zone_x[3] == zone_x[1]) || (zone_x[2] == zone_x[0]) || (zone_x[3] == zone_x[0]) )
      {
	EXF_ERREUR(ERR_DOUBLONS);
	free(b); free(c); free(d);
	return NULL;
      }
      c[0]                         = c[2] / (zone_x[3] - zone_x[1])
	                           - c[1] / (zone_x[2] - zone_x[0]);
      c[0]                        *= d[0] * d[0] / (zone_x[3] - zone_x[0]);

      if ( (zone_x[source->nombre_point - 1] == zone_x[source->nombre_point - 3])
	|| (zone_x[source->nombre_point - 2] == zone_x[source->nombre_point - 4])
        || (zone_x[source->nombre_point - 1] == zone_x[source->nombre_point - 4]) )
      {
	EXF_ERREUR(ERR_DOUBLONS);
	free(b); free(c); free(d);
	return NULL;
      }
      c[source->nombre_point - 1]  = c[source->nombre_point - 2] / (zone_x[source->nombre_point - 1] - zone_x[source->nombre_point - 3])
	                           - c[source->nombre_point - 3] / (zone_x[source->nombre_point - 2] - zone_x[source->nombre_point - 4]);
      c[source->nombre_point - 1] *= - d[source->nombre_point - 2] * d[source->nombre_point - 2] / (zone_x[source->nombre_point - 1] - zone_x[source->nombre_point - 4]);
    }
    else
    {
      c[0] = 0;
      c[source->nombre_point - 1] = 0;
    }
		
    for (index = 1 ; index < source->nombre_point ; index ++)
    {
      double pas;

      if (b[index - 1] == 0)
      {
	EXF_ERREUR(DIV_ZERO);
	free(b); free(c); free(d);
	return NULL;
      }
      pas = d[index - 1] / b[index - 1];
      b[index] -= pas * d[index - 1];
      c[index] -= pas * c[index - 1];
    }
    if ( b[source->nombre_point - 1] == 0 )
    {
      EXF_ERREUR(DIV_ZERO);
      free(b); free(c); free(d);
      return NULL;
    }
    c[source->nombre_point - 1] /= b[source->nombre_point - 1];

    for (index = source->nombre_point - 2 ; index > 0 ; index--)
    {
      if (b[index] == 0)
      {
	EXF_ERREUR(DIV_ZERO);
	free(b); free(c); free(d);
	return NULL;
      }
      c[index] -= d[index] * c[index + 1];
      c[index] /= b[index];
    }
    c[0] -= d[0] * c[1];
    c[0] /= b[0];		

    if ( d[source->nombre_point - 2] == 0 )
    {
      EXF_ERREUR(DIV_ZERO);
      free(b); free(c); free(d);
      return NULL;
    }
    b[source->nombre_point - 1]  = (zone_y[source->nombre_point - 1] - zone_y[source->nombre_point - 2] ) / d[source->nombre_point - 2];
    b[source->nombre_point - 1] += d[source->nombre_point - 2] * (c[source->nombre_point - 2] + 2 * c[source->nombre_point - 1] );
		
    for (index = 0 ; index < source->nombre_point - 1 ; index++)
    {
      if (d[index] == 0)
      {
	EXF_ERREUR(DIV_ZERO);
	free(b); free(c); free(d);
	return NULL;
      }
      b[index]  = (zone_y[index + 1] - zone_y[index]) / d[index];
      b[index] -= d[index] * ( c[index + 1] + 2 * c[index] );
      d[index]  = ( c[index + 1] - c[index] ) / d[index];
      c[index] *= 3;
    }
    c[source->nombre_point - 1] *= 3;
    d[source->nombre_point - 1] = d[source->nombre_point - 2];
  }

  /* Pas de problme  la cration des tableaux -> on cr le graphe rsultat */

#ifdef _FRANCAIS_
  sprintf(titre, "%s (liss"e_AIGU")", source->nom);
#endif
#ifdef _ENGLISH_
  sprintf(titre, "%s (smoothed)", source->nom);
#endif
  resultat = GRF_creer(nv_points, source->type, GR_NORMAL, titre,
		       source->axe_x, source->axe_y, source->unit_x, source->unit_y);

  if (resultat == NULL)
  {
    free(b); free(c); free(d);
    return NULL;
  }

  /* Calcul du nouveau graphe */

  zone_x = resultat->x;
  zone_y = resultat->y;
  pas    = (source->x_max - source->x_min) / (resultat->nombre_point - 1);
	
  *zone_x = source->x_min; /* Ce point appartient forcment au graphe source et liss... */
  *zone_y = *(source->y); /* Graphe tri donc x_min = x[0] */

  /* On cherche le X le plus proche - rate si graphe mal tri... */

  for (index = 1 ; index < resultat->nombre_point - 1 ; index++)
  {
    unsigned long index2;
    zone_x++; zone_y++;
    *zone_x = *(zone_x - 1) + pas;

    /* On cherche le x prcdent dans le graphe source */
    index2 = 0;
    while ( *(source->x + index2) < *zone_x )
      index2++;

    if ( *(source->x + index2) == *zone_x)
      *zone_y = *(source->y + index2);
    else
    {
      double delta_x;
      index2--;
      delta_x = *zone_x - *(source->x + index2);
      *zone_y  = *(source->y + index2);
      *zone_y += delta_x * (b[index2] + delta_x * (c[index2] + delta_x * d[index2]));
    }
  }

  /* Le dernier point appartient forcment au graphe source */

  zone_x++; zone_y++;
  *(zone_x) = source->x_max;
  *zone_y = *(source->y + source->nombre_point - 1); /* Graphe tri -> x_max = dernier x */

  /* On met les bornes du graphe liss */

  resultat->x_min = source->x_min;
  resultat->x_max = source->x_max;
  GRF_bornes(resultat, NE_PAS_FAIRE, DESORDONNE);

  /* On libre les tableaux de coefficients */
  free(b); free(c); free(d);

  return resultat; /* On renvoie le graphe liss */
}

/* -------- Lissage complet d'un spectre rel --------------------------\
 |  Renvoie le graphe liss (NULL si erreur)
 |
 | source    : le graphe  lisser [doit tre rel, tri, sans soublons] |
 | nv_points : le nombre de points dans le graphe liss                 |
 \--------------------------------------------------------------------- */

Graphe *lisse_complexe(Graphe *source, unsigned long nv_points)
{
  Graphe *resultat;
  Complexe *b, *c, *d; /* Coefficients pour le lissage */
  char titre[300];
  unsigned long i, j = 0;
  register double *x_lisse, *x;
  register Complexe *y_lisse, *y;
  double pas;

  if ( CSPL_tt_prep(source, &b, &c, &d) )
  {
    printf("Erreur  la cration des tableaux\n");
    return NULL;
  }

  /* On cre le nouveau graphe */

#ifdef _FRANCAIS_
  sprintf(titre, "%s (liss"e_AIGU")", source->nom);
#endif
#ifdef _ENGLISH_
  sprintf(titre, "%s (smoothed)", source->nom);
#endif
  resultat = GRF_creer(nv_points, source->type, source->drapeaux.complexe, titre,
		       source->axe_x, source->axe_y, source->unit_x, source->unit_y);
  if (resultat == NULL)
  {
    free(b); free(c); free(d);
    return NULL;
  }

  /* On calcule le nouveau graphe - suppose que source est tri ! */

  x = source->x;
  y = (Complexe *) source->y;
  y_lisse = (Complexe *) resultat->y;

  /* Le premier point est identique... */

  resultat->x[0] = *x;
  *y_lisse = *y;

  x_lisse = resultat->x + 1;
  y_lisse++;

  pas = ( source->x_max - source->x_min ) / (nv_points - 1);

  for (i = 1 ; i < nv_points - 1 ; i++, x_lisse++, y_lisse++)
  {
    double x_apres, delta;

    *x_lisse = *(x_lisse - 1) + pas;

    if (j < source->nombre_point)
      x_apres = *(x + 1);
    else
      x_apres = *(x_lisse);

    while (*x_lisse > x_apres)
    {
      x++; j++; y++;
      if ( j >= source->nombre_point )
	x_apres = *x_lisse;
      else
	x_apres = *(x + 1);
    }

    delta = *x_lisse - *x;

    y_lisse->re  = delta * d[j].re;
    y_lisse->re += c[j].re;
    y_lisse->re *= delta;
    y_lisse->re += b[j].re;
    y_lisse->re *= delta;
    y_lisse->re += y->re;
    y_lisse->im  = delta * d[j].im;
    y_lisse->im += c[j].im;
    y_lisse->im *= delta;
    y_lisse->im += b[j].im;
    y_lisse->im *= delta;
    y_lisse->im += y->im;
  }

  /* Le dernier point est toujours conserv */

  *x_lisse = source->x_max;
  *y_lisse = *( ( (Complexe *) (source->y) ) + source->nombre_point );

  GRF_bornes(resultat, ORDONNE, DESORDONNE);

  free(b); free(c); free(d); /* Plus besoin d'eux */

  return resultat;
}

/* ******** Interface GEM pour le lissage ***************************** */

Graphe *lisse_graphe(Graphe *source)
{
  int retour;
  Graphe *lisse;           /* Graphe aprs lissage                  */
  unsigned long nbr_lisse; /* Nombre de points dans le graphe liss */

  if (source == NULL) return NULL; /* Rien  faire... */
  if (source->nombre_point == 1)
  {
    EXF_ERREUR(TROP_PEU_POINTS);
    return NULL;
  }
  if ( ( *(source->x) != source->x_min )
    || ( *(source->x + source->nombre_point - 1) != source->x_max ) )
  {
    EXF_ERREUR(ERR_NON_TRIE);
    return NULL;
  }

  met_ulvaleur(LISSAGE, LISS_EXNOMBRE, source->nombre_point);
  met_ulvaleur(LISSAGE, LISS_NOUVEAU , source->nombre_point);
  do
  {
    retour = ext_formdo(LISSAGE);
    if (retour == LISS_ANNULE) return NULL; /* Pas de graphe liss cr */

    if ( retour == LISS_PUISS_2 ) /* On prend la puissance de 2 la plus proche */
    {
      int puissance_2 = 0;
      nbr_lisse = prend_ulvaleur(LISSAGE, LISS_NOUVEAU);

      /* On cherche le premier bit non-nul */
      
      while ( ( nbr_lisse >> puissance_2 ) > 1)
	puissance_2++;

      if ( (1 << puissance_2) != nbr_lisse )
      {
	puissance_2++; /* On passe au nombre suprieur */
      }

      nbr_lisse = 1 << puissance_2;
      met_ulvaleur(LISSAGE, LISS_NOUVEAU, nbr_lisse);
    }
  }
  while ( retour != LISS_OK );

  nbr_lisse = prend_ulvaleur(LISSAGE, LISS_NOUVEAU); /* Nombre de points dans le graphe liss */
  if (nbr_lisse == 1)
  {
    EXF_ERREUR(ERR_UN_POINT);
    return NULL;
  }
  if (nbr_lisse <= source->nombre_point)
  {
    retour = form_alert(2,"[2][ Nombre de points infrieur | au nombre de points initial ! | On va donc perdre en prcision ][ Suite |Annuler]");
    if (retour == 2) return NULL; /* On annule */
  }

  if ( source->drapeaux.complexe == GR_NORMAL ) /* Graphe rel */
  {
    lisse = lisse_reel(source, nbr_lisse);
  }
  else /* Graphe complexe */
  {
    lisse = lisse_complexe(source, nbr_lisse);
  }

  if ( OBJ_choisit(LISSAGE, LISS_EFFACER, QUESTION) )
    GRF_efface(source, VRAI, FAUX); /* On veut effacer le graphe original - sauf si protg ! */

  return lisse; /* Le graphe liss */
}
