/* LASE - logiciel d'analyse des spectres E.X.A.F.S.
   (c) 1998, Emmanuel CURIS - Cration du fichier : 4 juin 1998

   Mthode des moindres carrs  brutale
    - pour un polynme
    - pour une fonction en A + B/x + C/x^2 + ...
   Dtermination des coefficients par rsolution du systme
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"
#include "ERREURS.H"
#include "LISSAGE.H"

/* ***** Fonctions internes de ce module ***************************************************** */

static int degre_1(double *somme_x, double *somme_xy, double *coefs);

/* ----- Moindres carrs directe avec polynmes complets --------------------------------------\
 |  Renvoie l'erreur si le polynme a pu tre calcul correctement, -1 sinon
 |
 | poly      : le tableau des coefficients du polynme en retour ([0] + [1]*x + [2]*x^2 + ...) |
 | degre     : le degr du polynme d'interpolation (ventuellement ngatif...)
 | graphe    : le graphe sur lequel on va travailler
 | nbr_zones : le nombre d'intervalles de points valables
 | zone      : tableau des bornes des intervalles (0 : min1, 1 : max1, 2 : min2, 3 : max2,...) |
 \-------------------------------------------------------------------------------------------- */

double moindre2(Graphe *graphe, int degre, char nbr_zones, double *zones, double *poly)
{
  double /* inter[11]    = {0,0,0,0,0,0,0,0,0,0,0}, */
          somme_x[22]  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
          somme_xy[11] = {0,0,0,0,0,0,0,0,0,0,0};
  double  /* delta, */ x,y,x_puissance /*,rien */ ;
  double  /* erreur = 0,*/ retour = 0; /* Erreur pour le polynme complet
				     et celui avec juste les extrmes */
  double  somme_y2 = 0;           /* Somme des y^2, pour le calcul d'erreur... */
  boolean convient = VRAI;
  int     i;
  unsigned long index; /* Compteur... */
  register double *zone_x,*zone_y;
  double norme; /* Pour mettre tous les X dans une chelle plus sympa... vite des erreurs ? */

  /* On initialise les coefficients  0 */
  for (i = 0 ; i <= abs(degre) ; poly[i++] = 0);

  /* On calcule le nombre de points et Sx -> pour les moyennes en normalisant */

  if (zones == NULL) /* On veut tous les points */
  {
    register double *x;
    unsigned long i;

    somme_x[0] = graphe->nombre_point;
    for (x = graphe->x, i = 0 ; i < graphe->nombre_point ; i++, x++)
      somme_x[1] += *x;
  }
  else
  {
    register double *x;
    unsigned long i;
    int i_zone;

    for (x = graphe->x, i = 0 ; i < graphe->nombre_point ; i++, x++)
    {
      boolean convient = FAUX;
      for (i_zone = 0 ; i_zone < 2*nbr_zones ; i_zone += 2)
      {
	if ( (*x >= zones[i_zone]) && (*x <= zones[i_zone + 1]) )
	  convient = VRAI; /* Le point appartient  un des intervalles proposs */
      }

      if ( convient == VRAI ) /* On doit prendre en compte ce point */
      {
	somme_x[0]++; /* Un point en plus */
	somme_x[1] += *x;
      }
    }
  }

  if (somme_x[0] == 0) /* Aucun point */
  {
    LIS_ERREUR(AUCUN_POINT);
    return -1;
  }

  if (somme_x[1] != 0)
  {
    norme = somme_x[0] / somme_x[1]; /* 1 / <x> */
    somme_x[1] = somme_x[0];
  }
  else
    norme = 1; /* Pas envie de rflchir pour l'instant ->  voir plus tard */

  /* On calcule les autres sommes ncessaires */

  zone_x = graphe->x;
  zone_y = graphe->y;
  for (index = 0 ; index < graphe->nombre_point ; index++)
  {
    x = *(zone_x); /* Valeur du point en cours */
		
    if (zones != NULL) /* Tout point ne convient pas */
    {
      convient = FAUX;
      for (i = 0 ; i < 2*nbr_zones ; i += 2)
      {
	if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	  convient = VRAI; /* Le point appartient  un des intervalles proposs */
      }
    }
    
    if ( convient == VRAI ) /* On est dans les bornes */
    {
      x *= norme;
      y = *(zone_y);
     
      somme_xy[0] += y;
      somme_xy[1] += x*y;
      somme_y2 += y*y;
      x_puissance = x * x;
      for (i = 2 ; i <= 2*abs(degre) ; i++)
      {
	somme_x[i]  += x_puissance;
	if (degre > 0)
	{
	  if ( i <= degre )
	    somme_xy[i] += x_puissance * y;
	}
	else
	{
	  if ( x_puissance == 0 )
	  {
	    EXF_ERREUR(DIV_ZERO);
	    return -1;
	  }
	  if ( i <= -degre )
	    somme_xy[i] += y / x_puissance;
	}
	x_puissance *= x;
      }
    }
    zone_x++; zone_y++;
  }

  switch (abs(degre))
  {
  case 0: /* Constante -> un seul calcul possible... */
    poly[0] = somme_xy[0] / somme_x[0];
    break;

  case 1: /* Un seul calcul possible */
    if ( degre_1(somme_x, somme_xy, poly) < 0 ) /* Erreur survenue */
      return -1;
    break;

#ifdef determinant
  case 2: /* Meilleure parabole, avec recherche de l'erreur minimale */
    inter[0] = somme_x[2] * somme_x[4] - somme_x[3] * somme_x[3];
		
    delta  =   somme_x[0] * inter[0];
    delta -=   somme_x[1] * somme_x[1] * somme_x[4] + somme_x[2] * somme_x[2] * somme_x[2];
    delta += 2*somme_x[1] * somme_x[2] * somme_x[3];
    if (delta == 0)
    {
      retour = -1;
    }
    else
    {
      poly[0]  = somme_xy[0] * inter[0];
      poly[0] -= somme_xy[1] * ( somme_x[1] * somme_x[4] - somme_x[3] * somme_x[2] );
      poly[0] += somme_xy[2] * ( somme_x[1] * somme_x[3] - somme_x[2] * somme_x[2] );
      poly[0] /= delta;
		
      poly[1]  = somme_x[0] * ( somme_x[4] * somme_xy[1] - somme_x[3] * somme_xy[2] );
      poly[1] -= somme_x[1] * ( somme_x[4] * somme_xy[0] - somme_x[2] * somme_xy[2] );
      poly[1] += somme_x[2] * ( somme_x[3] * somme_xy[0] - somme_x[2] * somme_xy[1] );
      poly[1] /= delta;
      poly[1] *= norme;
      
      poly[2]  = somme_x[0] * ( somme_x[2] * somme_xy[2] - somme_x[3] * somme_xy[1] );
      poly[2] -= somme_x[1] * ( somme_x[1] * somme_xy[2] - somme_x[3] * somme_xy[0] );
      poly[2] += somme_x[2] * ( somme_x[1] * somme_xy[1] - somme_x[2] * somme_xy[0] );
      poly[2] /= delta;
      poly[2] *= norme * norme;
      
      zone_x = graphe->x; zone_y = graphe->y;
      for (index = 0 ; index < graphe->nombre_point ; index++)
      {
	x = *(zone_x);
	if (zones != NULL) /* Tout point ne convient pas */
	{
	  convient = FAUX;
	  for (i = 0 ; i < 2*nbr_zones ; i+=2)
	  {
	    if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	      convient = VRAI; /* Le point appartient  un des intervalles proposs */
	  }
	}
	
	if ( convient == VRAI ) /* On est dans les bornes */
	{
	  y  = poly[0] + poly[1] * x + poly[2] * x * x;
	  y -= *zone_y;
	  retour += y * y;
	}
	zone_x++; zone_y++;		
      }
    }
    
    delta = somme_x[0] * somme_x[4] - somme_x[2] * somme_x[2];
    if (delta == 0)
    {
      erreur = -1;
    }
    else /* Calcul pour A + B * x^2 */
    {
      coefs[0]  = somme_xy[0] * somme_x[4] - somme_x[2] * somme_xy[1];
      coefs[0] /= delta;
      
      coefs[2] = somme_xy[2] * somme_x[0] - somme_x[2] * somme_xy[0];
      coefs[2] /= delta;
      coefs[2] *= norme * norme;
      
      zone_x = graphe->x; zone_y = graphe->y;
      for (index = 0 ; index < graphe->nombre_point ; index++)
      {
	x = *(zone_x);
	if (zones != NULL) /* Tout point ne convient pas */
	{
	  convient = FAUX;
	  for (i = 0 ; i < 2*nbr_zones ; i+=2)
	  {
	    if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	      convient = VRAI; /* Le point appartient  un des intervalles proposs */
	  }
	}

	if ( convient == VRAI ) /* On est dans les bornes */
	{
	  y  = coefs[0] + coefs[2] * x * x;
	  y -= *zone_y;
	  erreur += y * y;
	}
	zone_x++; zone_y++;		
      }
    }
    
    if ( (erreur == -1) && (retour == -1) )
    {
      EXF_ERREUR(DIV_ZERO);
      return -1;
    }
		
    if ( (erreur == -1) || (erreur > retour) )
    {
      printf("\n\nCoefs : %e - %e - %e\n", poly[0], poly[1], poly[2]);
      goto suite;
      return retour / somme_y2; /* La premire tait la bonne */
    }
    
    poly[0] = coefs[0];
    poly[1] = 0;
    poly[2] = coefs[2];
    return erreur / somme_y2;
		
  case 3: /* Cubique : A + B * X + C * X^2 + D * X^3 */
    inter[0] = somme_x[3] * somme_x[6] - somme_x[4] * somme_x[5];
    inter[1] = somme_x[4] * somme_x[6] - somme_x[5] * somme_x[5];
    
    /* Discriminant calcul par Maple V -> correct */
    
    delta  = somme_x[5] * somme_x[5] - somme_x[4] * somme_x[6];
    delta *= somme_x[1];
    
    rien  = somme_x[3] * somme_x[4] * somme_x[4];
    rien -= somme_x[4] * somme_x[2] * somme_x[5];
    rien += somme_x[3] * somme_x[2] * somme_x[6];
    rien -= somme_x[3] * somme_x[3] * somme_x[5];
    rien *= 2;

    delta += rien;
    delta *= somme_x[1];
    delta += somme_x[2] * somme_x[2] * somme_x[4] * somme_x[4];
    delta -= somme_x[2] * somme_x[3] * somme_x[3] * somme_x[4] * 3;
    delta += somme_x[3] * somme_x[3] * somme_x[3] * somme_x[3];
    delta -= somme_x[2] * somme_x[2] * somme_x[2] * somme_x[6];
    delta += somme_x[2] * somme_x[2] * somme_x[3] * somme_x[5] * 2;
    
    rien  = somme_x[2] * somme_x[4] - somme_x[3] * somme_x[3];
    rien *= somme_x[6];
    rien += somme_x[3] * somme_x[4] * somme_x[5] * 2;
    rien -= somme_x[4] * somme_x[4] * somme_x[4];
    rien -= somme_x[2] * somme_x[5] * somme_x[5];
    rien *= somme_x[0];

    delta += rien;
    
    if (delta == 0)
    {
      retour = -1;
    }
    else
    {
      poly[0]  = inter[1]    * ( somme_xy[0] * somme_x[2] - somme_xy[1] * somme_x[1] );
      poly[0] += inter[0]    * ( somme_xy[1] * somme_x[2] - somme_xy[0] * somme_x[3] );
      poly[0] +=               ( somme_x[3]  * somme_x[5] - somme_x[4]  * somme_x[4] )
	       * ( somme_xy[0] * somme_x[4] - somme_xy[1] * somme_x[3] - somme_x[1] * somme_xy[3] );
      poly[0] += somme_xy[2] * ( somme_x[1] * ( somme_x[3] * somme_x[6] - somme_x[5] * somme_x[4] )
     + somme_x[2] * ( somme_x[3]  * somme_x[5] + somme_x[4] * somme_x[4] - somme_x[2] * somme_x[6] )
     - somme_x[3] * somme_x[3] * somme_x[4] );
      poly[0] += somme_xy[3] * ( somme_x[2] * (somme_x[2] * somme_x[5] - somme_x[3] * somme_x[4]*2)
      + somme_x[3] * somme_x[3] * somme_x[3] );
      
      /* Coefficient B calcul par Maple V -> correct */

      poly[1]  = somme_x[1] * somme_x[5];
      poly[1] -= (somme_x[2] * somme_x[4] + somme_x[3] * somme_x[3]);
      poly[1] *= somme_x[5];
      poly[1] -= somme_x[1] * somme_x[4] * somme_x[6];
      poly[1] += somme_x[3] * somme_x[4] * somme_x[4];
      poly[1] += somme_x[3] * somme_x[2] * somme_x[6];
      poly[1] *= somme_xy[0];
      
      rien  = somme_x[0] * somme_x[6]  - somme_x[3] * somme_x[3];
      rien *= somme_x[4];
      rien -= somme_x[0] * somme_x[5] * somme_x[5];
      rien -= somme_x[2] * somme_x[2] * somme_x[6];
      rien += somme_x[3] * somme_x[2] * somme_x[5] * 2;
      poly[1] += rien * somme_xy[1];

      rien  = somme_x[4] * somme_x[5] - somme_x[3] * somme_x[6];
      rien *= somme_x[0];
      rien += somme_x[2] * somme_x[1] * somme_x[6];
      rien -= somme_x[2] * somme_x[4] * somme_x[3];
      rien -= somme_x[3] * somme_x[1] * somme_x[5];
      rien += somme_x[3] * somme_x[3] * somme_x[3];
      poly[1] += rien * somme_xy[2];

      rien  = somme_x[5] * somme_x[3] - somme_x[4] * somme_x[4];
      rien *= somme_x[0];
      rien -= somme_x[1] * somme_x[2] * somme_x[5];
      rien += somme_x[1] * somme_x[4] * somme_x[3];
      rien += somme_x[2] * somme_x[2] * somme_x[4];
      rien -= somme_x[2] * somme_x[3] * somme_x[3];
      poly[1] += rien * somme_xy[3];
      
      /* Coefficient C, calcul par Maple V -> correct */

      poly[2]  = -somme_x[3] * somme_x[4];
      poly[2] += somme_x[1] * somme_x[6] + somme_x[2] * somme_x[5];
      poly[2] *= somme_x[3];
      poly[2] -= somme_x[1] * somme_x[4] * somme_x[5];
      poly[2] += somme_x[2] * somme_x[4] * somme_x[4];
      poly[2] -= somme_x[2] * somme_x[2] * somme_x[6];
      poly[2] *= somme_xy[0];

      rien  = somme_x[3] * somme_x[3];
      rien -= somme_x[0] * somme_x[6];
      rien -= somme_x[2] * somme_x[4];
      rien -= somme_x[1] * somme_x[5];
      rien *= somme_x[3];
      rien += somme_x[0] * somme_x[4] * somme_x[5];
      rien += somme_x[2] * somme_x[1] * somme_x[6];
      poly[2] += somme_xy[1] * rien;

      rien  = somme_x[0] * somme_x[2] - somme_x[1] * somme_x[1];
      rien *= somme_x[6];
      rien -= somme_x[2] * somme_x[3] * somme_x[3];
      rien -= somme_x[0] * somme_x[4] * somme_x[4];
      rien += somme_x[1] * somme_x[4] * somme_x[3] * 2;
      poly[2] += somme_xy[2] * rien;

      rien  = somme_x[3] * somme_x[2];
      rien -= somme_x[0] * somme_x[5];
      rien -= somme_x[1] * somme_x[4];
      rien *= somme_x[2];
      rien += somme_x[0] * somme_x[3] * somme_x[4];
      rien += somme_x[1] * somme_x[1] * somme_x[5];
      rien -= somme_x[1] * somme_x[3] * somme_x[3];
      poly[2] += somme_xy[3] * rien;

      /* Coefficient D calcul avec Maple V -> correct */
      
      poly[3]  = somme_x[4] * somme_x[4] - somme_x[3] * somme_x[5];
      poly[3] *= somme_x[1];
      poly[3] += somme_x[2] * somme_x[2] * somme_x[5];

      rien   = somme_x[3] * somme_x[3];
      rien  -= somme_x[2] * somme_x[4] * 2;
      rien  *= somme_x[3];
      poly[3] += rien;
      poly[3] *= somme_xy[0];

      rien  = somme_x[0] * somme_x[3] - somme_x[2] * somme_x[1];
      rien *= somme_x[5];
      rien += somme_x[3] * somme_x[1] * somme_x[4];
      rien += somme_x[4] * somme_x[2] * somme_x[2];
      rien -= somme_x[3] * somme_x[3] * somme_x[2];
      rien -= somme_x[0] * somme_x[4] * somme_x[4];
      rien *= somme_xy[1];
      poly[3] += rien;

      rien  = somme_x[0] * somme_x[3] - somme_x[2] * somme_x[1];
      rien *= somme_x[4];
      rien -= somme_x[0] * somme_x[2] * somme_x[5];
      rien += somme_x[3] * somme_x[2] * somme_x[2];
      rien += somme_x[1] * somme_x[1] * somme_x[5];
      rien -= somme_x[1] * somme_x[3] * somme_x[3];
      rien *= somme_xy[2];
      poly[3] += rien;

      rien  = somme_x[2] * somme_x[4] - somme_x[3] * somme_x[3];
      rien *= somme_x[0];
      rien -= somme_x[2] * somme_x[2] * somme_x[2];
      rien -= somme_x[1] * somme_x[1] * somme_x[4];
      rien += somme_x[1] * somme_x[2] * somme_x[3] * 2;
      rien *= somme_xy[3];
      poly[3] += rien;

      poly[0] /= delta;
      poly[1] /= delta;
      poly[1] *= norme;
      poly[2] /= delta;
      poly[2] *= norme * norme;
      poly[3] /= delta;
      poly[3] *= norme * norme * norme;

      zone_x = graphe->x; zone_y = graphe->y;
      for (index = 0 ; index < graphe->nombre_point ; index++)
      {
	x = *(zone_x);
	if (zones != NULL) /* Tout point ne convient pas */
	{
	  convient = FAUX;
	  for (i = 0 ; i < 2*nbr_zones ; i+=2)
	  {
	    if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	      convient = VRAI; /* Le point appartient  un des intervalles proposs */
	  }
	}
	
	if ( convient == VRAI ) /* On est dans les bornes */
	{
	  y  = poly[0] + x * (poly[1] + x * (poly[2] + x * poly[3]));
	  y -= *zone_y;
	  retour += y * y;
	}
	zone_x++; zone_y++;		
      }
    }

    inter[0] = somme_x[2] * somme_x[6] - somme_x[4] * somme_x[4];
    inter[1] = somme_x[1] * somme_x[6] - somme_x[3] * somme_x[4];
    inter[2] = somme_x[1] * somme_x[4] - somme_x[3] * somme_x[2];

    delta  = somme_x[0] * inter[0];
    delta -= somme_x[1] * inter[1];
    delta += somme_x[3] * inter[2];
    if (delta == 0)
    {
      erreur = -1;
    }
    else
    {
      coefs[0]  = somme_xy[0] * inter[0];
      coefs[0] -= somme_xy[1] * inter[1];
      coefs[0] += somme_xy[3] * inter[2];
      coefs[0] /= delta;
		
      coefs[1]  = somme_x[0] * ( somme_x[6] * somme_xy[1] - somme_x[4] * somme_xy[3] );
      coefs[1] -= somme_xy[0] * inter[1];
      coefs[1] += somme_x[3] * ( somme_x[1] * somme_xy[3] - somme_x[3] * somme_xy[1] );
      coefs[1] /= delta;
      coefs[1] *= norme;

      /*	Coefficient de X^2 nul par hypothse */
      
      coefs[3]  = somme_x[0] * ( somme_x[2] * somme_xy[3] - somme_x[4] * somme_xy[1] );
      coefs[3] -= somme_x[1] * ( somme_x[1] * somme_xy[3] - somme_x[3] * somme_xy[1] );
      coefs[3] += somme_xy[0] * inter[2];
      coefs[3] /= delta;
      coefs[3] *= norme * norme * norme;
      
      zone_x = graphe->x; zone_y = graphe->y;
      for (index = 0 ; index < graphe->nombre_point ; index++)
      {
	x = *(zone_x);
	if (zones != NULL) /* Tout point ne convient pas */
	{
	  convient = FAUX;
	  for (i = 0 ; i < 2*nbr_zones ; i+=2)
	  {
	    if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	      convient = VRAI; /* Le point appartient  un des intervalles proposs */
	  }
	}

	if ( convient == VRAI ) /* On est dans les bornes */
	{
	  y  = coefs[0] + x * (coefs[1] + coefs[3] * x * x);
	  y -= *zone_y;
	  erreur += y * y;
	}
	zone_x++; zone_y++;		
      }
    }

    if ( (erreur != -1) && ( (erreur < retour) || (retour == -1) ) )
    {
      retour = erreur; /* La deuxime est meilleure -> on la prend */
      poly[0] = coefs[0];
      poly[1] = coefs[1];
      poly[2] = 0;
      poly[3] = coefs[3];
    }

    /* Calcul pour A + B*x^2 + C*x^3 */

    inter[0] = somme_x[4] * somme_x[6] - somme_x[5] * somme_x[5];
    delta  = somme_x[0] * inter[0];
    delta -= somme_x[2] * somme_x[2] * somme_x[6];
    delta += somme_x[2] * somme_x[3] * somme_x[5] * 2;
    delta -= somme_x[3] * somme_x[3] * somme_x[4];

    if (delta == 0)
    {
      erreur = -1;
    }
    else
    {
      coefs[0]  = somme_xy[0] * inter[0];
      rien  = somme_x[3] * somme_x[5] - somme_x[2] * somme_x[6];
      rien *= somme_xy[2];
      coefs[0] += rien;
      rien  = somme_x[2] * somme_x[5] - somme_x[3] * somme_x[4];
      rien *= somme_xy[3];
      coefs[0] += rien; /* Calcul de Maple V */
      coefs[0] /= delta;

      coefs[2]  = somme_x[3] * somme_x[5] - somme_x[2] * somme_x[6];
      coefs[2] *= somme_xy[0];
      rien  = somme_x[0] * somme_x[6] - somme_x[3] * somme_x[3];
      rien *= somme_xy[2];
      coefs[2] += rien;
      rien  = somme_x[2] * somme_x[3] - somme_x[0] * somme_x[5];
      rien *= somme_xy[3];
      coefs[2] += rien;
      coefs[2] /= delta;
      coefs[2] *= norme * norme;

      coefs[3]  = somme_x[2] * somme_x[5] - somme_x[3] * somme_x[4];
      coefs[3] *= somme_xy[0];
      rien  = somme_x[3] * somme_x[2] - somme_x[0] * somme_x[5];
      rien *= somme_xy[2];
      coefs[3] += rien;
      rien  = somme_x[0] * somme_x[4] - somme_x[2] * somme_x[2];
      rien *= somme_xy[3];
      coefs[3] += rien;
      coefs[3] /= delta;
      coefs[3] *= norme * norme * norme;
			
      erreur = 0;
      zone_x = graphe->x; zone_y = graphe->y;
      for (index = 0 ; index < graphe->nombre_point ; index++)
      {
	x = *(zone_x);
	if (zones != NULL) /* Tout point ne convient pas */
	{
	  convient = FAUX;
	  for (i = 0 ; i < 2*nbr_zones ; i+=2)
	  {
	    if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	      convient = VRAI; /* Le point appartient  un des intervalles proposs */
	  }
	}

	if ( convient == VRAI ) /* On est dans les bornes */
	{
	  y  = coefs[0] + x * x * (coefs[2] + coefs[3] * x);
	  y -= *zone_y;
	  erreur += y * y;
	}
	zone_x++; zone_y++;		
      }
    }

    /* Termin -> on renvoie la meilleure */

    if ( (erreur == -1) && (retour == -1) )
    {
      EXF_ERREUR(DIV_ZERO);
      return -1;
    }
		
    if (erreur < retour)
    {
      poly[0] = coefs[0];
      poly[1] = 0;
      poly[2] = coefs[2];
      poly[3] = coefs[3];
      retour = erreur;
    }
    return retour / somme_y2; /* Renvoi de l'erreur */

#endif
  default: /* Approximation par un polynme de degr suprieur : pivot de Gauss */
    {
      double *matrice;
      int i, j;
      int deg = abs(degre);

      /* Matrice du systme */
      matrice = (double *) malloc( sizeof(double) * (deg + 2) * (deg + 1) );
      if (matrice == NULL)
      {
	ERREUR(MANQUE_MEMOIRE);
	return -1;
      }

      /* On prpare la matrice  rsoudre */

      for (j = 0 ; j <= deg ; j++)
	for (i = 0 ; i <= deg ; i++)
	  matrice[ (deg + 2) * j + i ] = somme_x[i + j];

      for (j = 0 ; j <= deg ; j++)
	matrice[ (deg + 2) * j + (deg + 1) ] = somme_xy[j];

      /* Affichage de la matrice des sommes */

      /*      printf("\n\n Matrice carre associe au systme\n");

      for (i = 0 ; i <= degre ; i++)
      {
	printf("( ");
	for (j = 0 ; j <= degre ; j++)
	  printf("%e ", somme_x[ i + j ]);

	printf(")\n");
      }

      printf("\n"); */

      if ( Gauss(deg, matrice, poly) < 0 ) /* Rsolution par le pivot de Gauss */
      {
	free(matrice);
	return -1;
      }
      free(matrice);

      /* On vrifie que c'est bien solution */
      /*      for (i = 0 ; i <= degre ; i++)
      {
	double y_th = 0;

	for (j = 0 ; j <= degre ; j++)
	  y_th += somme_x[ i + j ] * poly[j];

	printf("Y attendu : %f - calcul : %f\n", somme_xy[i], y_th);
      } */
    }
  }

  /* Prise en compte du facteur de normalisation */

  retour = 1;
  for (i = 1 ; i <= degre ; i++)
  {
    retour *= norme;
    poly[i] *= retour;
  }

  /* Calcul de l'erreur */
  zone_x = graphe->x; zone_y = graphe->y;
  retour = 0;
  for (index = 0 ; index < graphe->nombre_point ; index++)
  {
    x = *(zone_x);
    if (zones != NULL) /* Tout point ne convient pas */
    {
      convient = FAUX;
      for (i = 0 ; i < 2*nbr_zones ; i+=2)
      {
	if ( (x >= zones[i]) && (x <= zones[i + 1]) )
	  convient = VRAI; /* Le point appartient  un des intervalles proposs */
      }
    }
    if ( convient == VRAI ) /* On est dans les bornes */
    {
      y = 0;
      for (i = degre ; i > 0 ; i--)
      {
	y += poly[i] * x;
      }
      y += poly[0] - *zone_y;
      retour += y * y;
    }
    zone_x++; zone_y++;		
  }

  return retour / somme_y2; /* Renvoi de l'erreur */  
}

/* ---- Meilleure droite par les moindres carrs simples --------------------------------\
 | Renvoie 0 si pas de problme, <0 en cas d'erreur                                      |
 |                                                                                       |
 | somme_x  : tableau des sommes ncessaires (somme_x[n] = Somme des x^n)                |
 | somme_xy : idem (somme_xy[n] = Somme des x^n * y)                                     |
 | poly     : coefficients du polynme trouv ( P(x) = somme des poly[i] * x^i )         |
 \-------------------------------------------------------------------------------------- */

static int degre_1(double *somme_x, double *somme_xy, double *poly)
{
  double delta; /* Discriminant du systme */

  delta = somme_x[0] * somme_x[2] - somme_x[1] * somme_x[1];
  if (delta == 0)
  {
    EXF_ERREUR(DIV_ZERO);
    return -DIV_ZERO;
  }
		
  poly[0]  = somme_xy[0] * somme_x[2] - somme_x[1] * somme_xy[1];
  poly[0] /= delta;
  poly[1]  = somme_xy[1] * somme_x[0] - somme_x[1] * somme_xy[0];
  poly[1] /= delta;

  return 0; /* Pas de problme */
}
