/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS
   Ce fichier : cration le 17 septembre 1998

   Module d'ajustement par un modle  n couches

   Paramtrage des paramtres spcifiques  chaque couche de l'ajustement
   Fonctions de gestion des couches, au passage
*/

#include <stdio.h>
#include <stdlib.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "couches.h"

/* extern Couche *premiere_couche; /* Dans variables.c */

/* ~~~~~~~~ Variables locales  ce fichier ~~~~~~~~ */

static Couche *couche_editee = NULL;
static int var_editee = COUCHE_N;  /* Nombre de voisins dit par dfaut */
static unsigned long nombre_couches = 0; /* Nombre de couches existantes */

static const Variable constante = {0, 0, 0, 0, VAR_CONSTANTE, {0}};

/* ~~~~~~~~ Fonctions locales  ce fichier ~~~~~~~~ */

static void enregistre_valeurs(Variable *var);
static void affiche_valeurs   (Variable *var);

static void enregistre_couche (Couche *couche);
static void affiche_couche    (Couche *couche);

void active_tout(boolean sens);

/* ~~~~~~~~ Ralisation pratique des diffrentes fonctions ~~~~~~~~ */

/* ~-~-~-~- Fonctions de cration/effacement des couches ~-~-~-~-~ */

/* -------- Cration d'une nouvelle couche, mise aprs la couche actuellement dite --------\
 |   Renvoie NULL en cas d'erreur, l'adresse de la nouvelle couche sinon                     |
 |                                                                                           |
 \------------------------------------------------------------------------------------------ */

Couche *creer_couche(void)
{
  Couche *couche;

  couche = (Couche *) malloc( sizeof(Couche) );
  if (couche == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("Module : N_couches/couches.c - Fonction : creer_couche\n");
    return NULL;
  }

  if (premiere_couche == NULL) /* C'est la premire couche que l'on cre */
  {
    premiere_couche = couche;
    couche->suivante = NULL; /* C'est donc aussi la dernire           */
    couche_editee = couche;  /* Et c'est aussi celle-ci que l'on dite */

    active_tout(VRAI);
  }
  else                         /* Il y en a -> on place aprs la couche dite */
  {
    couche->suivante = couche_editee->suivante;
    couche_editee->suivante = couche;
    couche_editee = couche; /* On ditera celle-ci aprs */
  }
  nombre_couches++; /* Une couche de plus */

  /* On fait l'init. de cette couche */

#ifdef _FRANCAIS_
  sprintf(couche->nom, "Sans nom (%ld)", nombre_couches);
#endif
#ifdef _ENGLISH_
  sprintf(couche->nom, "Noname (%ld)", nombre_couches);
#endif
#ifdef _DEUTSCH_
  sprintf(couche->nom, "Kein Name (%ld)", nombre_couches);
#endif

  couche->utilisee = VRAI; /* Par dfaut, sera prise en compte dans le modle */

  couche->N.valeur_initiale = 6; /* Nombre de voisins = dgnrescence */
  couche->N.min   = 5.5;
  couche->N.max   = 6.5;
  couche->N.delta = 0.001;
  couche->N.type  = VAR_LIBRE;
  couche->N.contenu.valeur = 6;

  couche->R.valeur_initiale = 1.8; /* Distance  l'absorbeur = longueur du chemin (en angstrom) */
  couche->R.min   = 1.5;
  couche->R.max   = 2.5;
  couche->R.delta = 0.0001;
  couche->R.type  = VAR_LIBRE;
  couche->R.contenu.valeur = 1.8;

  couche->Debye_Waller.valeur_initiale = 0.01; /* Facteur de Debye-Waller (en angstrom^2) */
  couche->Debye_Waller.min   = 0.001;
  couche->Debye_Waller.max   = 0.05;
  couche->Debye_Waller.delta = 0.00001;
  couche->Debye_Waller.type  = VAR_LIBRE;
  couche->Debye_Waller.contenu.valeur = 0.01;

  couche->Cumul_1 = constante; /* Premier cumulant   (dsordre) - fixe dans le cas gnral */
  couche->Cumul_2 = constante; /* Deuxime cumulant  (phases)   - fixe dans le cas gnral */
  couche->Cumul_3 = constante; /* Troisime cumulant (dsordre) - fixe dans le cas gnral */
  couche->Cumul_4 = constante; /* Quatrime cumulant (phases)   - fixe dans le cas gnral */
  couche->Delta_E = constante; /* Dcalage de E0, mme remarque que ci-dessous                   */
  couche->Gamma   = constante; /* Libre parcours moyen, cas 1 valeur par couche = non par dfaut */

  couche->signal    = NULL; /* Aucun signal associ */
  couche->amplitude = NULL;
  couche->phase     = NULL;

  couche->b_amp = NULL; /* Aucunes infos pour le lissage de l'amplitude */
  couche->c_amp = NULL;
  couche->d_amp = NULL;

  couche->b_pha = NULL; /* Aucunes infos pour le lissage de la phase    */
  couche->c_pha = NULL;
  couche->d_pha = NULL;

  /* On s'occupe de l'ascenseur */

  {
    Fenetre *fenetre = ARB_fenetre( arbres + PARAM_COUCHES );
    float taille_asc, pos_asc = 0;
    unsigned long num_couche;
    Couche *couche;

    taille_asc = 1000.0 / (float) nombre_couches; /* Il y a au moins 1 couche en fin de cration ! */
    if (nombre_couches == 1) /* Une seule couche */
      pos_asc = 0;
    else
    {
      unsigned long num_couche;
      Couche *couche = premiere_couche;

      for (num_couche = 0 ; couche != couche_editee ; couche = couche->suivante, num_couche++);
      pos_asc = 1000.0 * num_couche / ((float) (nombre_couches - 1));
    }

    FEN_ascenseur(fenetre, ASC_HORIZONTAL, (int) pos_asc, (int) taille_asc);
  }

  return couche;
}

/* -------- Effacer une couche ---------------------------\
 |  Ne renvoie rien                                       |
 |                                                        |
 | couche : la couche que l'on veut effacer               |
 | forcer : VRAI = on ne demande pas confirmation         |
 \------------------------------------------------------- */

void effacer_couche(Couche *couche, boolean forcer)
{
  if (couche == NULL)
    return;

  if (forcer == FAUX) /* On demande confirmation */
  {
    int retour = form_alert(2, "[2][---- Mod"e_GRAVE"le "a_GRAVE" n couches ----"
                                "| |   Voulez-vous r"e_AIGU"ellement "
                                  "| effacer cette couche ?][ Oui | Non ]");
    if (retour == 2) return; /* On annule l'effacement */
  }

  if (couche == premiere_couche)
    premiere_couche = couche->suivante;

  if (couche == couche_editee)
  {
  }

  free(couche->b_amp); free(couche->c_amp); free(couche->d_amp);
  free(couche->b_pha); free(couche->c_pha); free(couche->d_pha);

  free(couche);
  nombre_couches--;
  if (premiere_couche == NULL)
    active_tout(FAUX);
}

/* ~-~-~-~- Fonctions de gestion de l'interface ~-~-~-~-~ */

/* -------- Ascenseurs de la fentre = slection de la couche dite -------- */

void ASC_nc_couches(Fenetre *fenetre, int type, int ss_type, int position)
{
  float pos_asc;
  Couche *couche = couche_editee;

  if (fenetre    == NULL) return;  /* Ne devrait jamais arriver normalement... */
  if (nombre_couches < 2 ) return; /* Aucune ou une seule couche en mmoire : inutile de dfiler... */

  if (type == WM_ARROWED) /* On a utilis une des flches de l'ascenseur */
  {
    switch (ss_type) /* Action qui peut tre envisage */
    {
    case WA_LFPAGE: /* On descend de 10 couches */
      if (nombre_couches <= 10)
      {
	couche = premiere_couche;
      }
      else
      {
	unsigned long num_courante = 0;
	for (couche = premiere_couche ;
	     couche != couche_editee ;
	     couche = couche->suivante, num_courante++);

	if (num_courante > 10)
	{
	  unsigned long i;
	  for (i = 0, couche = premiere_couche ;
	       i < (num_courante - 10) ;
	       couche = couche->suivante, i++);
	}
	else
	{
	  couche = premiere_couche;
	}
      }
      break;

    case WA_RTPAGE: /* On monte de 10 couches */
      if (nombre_couches <= 10)
      {
	couche = premiere_couche;
	while (couche->suivante != NULL) /* On se place sur la dernire couche */
	  couche = couche->suivante;
      }
      else
      {
	unsigned int compteur = 0;

	for (couche = couche_editee ;
	     (compteur < 10) && (couche) ;
	     couche = couche->suivante, couche++);
      }
      break;

    case WA_LFLINE: /* On descend d'une couche */
      if (couche_editee != premiere_couche)
	for (couche = premiere_couche ; couche->suivante != couche_editee ; couche = couche->suivante);
      break;

    case WA_RTLINE: /* On monte d'une couche   */
      if (couche_editee->suivante) /* Ce n'est pas la dernire */
      {
	couche = couche_editee->suivante;
      }
      break;
    }
  }
  else if (type == WM_HSLID) /* Ascenseur horizontal dplac */
  {
    unsigned long num_couche, i;

    num_couche = position * (nombre_couches - 1) / 1000;
    for (i = 0, couche = premiere_couche ; i < num_couche ; i++, couche = couche->suivante);
  }

  enregistre_couche(couche_editee); /* On enregistre la couche actuelle */

  couche_editee = couche; /* Nouvelle couche */

  affiche_couche(couche_editee);    /* On affiche la nouvelle couche    */

  /* On s'occupe de l'ascenseur */

  if (nombre_couches == 1) /* Une seule couche */
    pos_asc = 0;
  else
  {
    unsigned long num_couche;
    Couche *couche = premiere_couche;

    for (num_couche = 0 ; couche != couche_editee ; couche = couche->suivante, num_couche++);
    pos_asc = 1000.0 * num_couche / ((float) (nombre_couches - 1));
  }

  FEN_ascenseur(fenetre, ASC_HORIZONTAL, (int) pos_asc, -1);
}

/* -------- Clics sur un objet = paramtrage de la couche active ------------ */

boolean GER_nc_couches(int objet, int x, int y)
{
  Couche *couche;

  /* ---- D'abord les objets gnriques ---- */

  if (objet == NV_COUCHE)        /* Cration d'une nouvelle couche */
  {
    couche = creer_couche();
    if (couche == NULL)
      return VRAI;
    
    affiche_couche(couche);
    return VRAI;
  }
  if (objet == EFFACER_COUCHE)   /* Effacement de la couche courante */
  {
    OBJ_choisit(PARAM_COUCHES, EFFACER_COUCHE, FORCE_OUI);
    effacer_couche(couche_editee, FAUX);
    if (couche_editee)   /* Il reste des couches dfinies */
      affiche_couche(couche_editee);
    OBJ_choisit(PARAM_COUCHES, EFFACER_COUCHE, FORCE_NON);
    return VRAI;
  }
  if (objet == TTEFFACER_COUCHE) /* Effacer le modle complet */
  {
    while (premiere_couche)
      effacer_couche(premiere_couche, VRAI);
    return VRAI;
  }
  if (objet == COUCHE_CREER)     /* Crer un modle  partir des chemins en mmoire */
  {
    return VRAI;
  }

  /* ---- Ici, on s'occupe donc des paramtres d'une couche -> que si une couche au moins existe ! ---- */

  if ( couche_editee == NULL )
    return VRAI;

  /* ---- On sauve les valeurs de la variable en cours d'dition ---- */

  switch (var_editee)
  {
  case COUCHE_N:
    enregistre_valeurs( &(couche_editee->N) );
    break;

  case COUCHE_DEBYEW:
    enregistre_valeurs( &(couche_editee->Debye_Waller) );
    break;

  case COUCHE_LONGUEUR:
    enregistre_valeurs( &(couche_editee->R) );
    break;

  case COUCHE_C1:
    enregistre_valeurs( &(couche_editee->Cumul_1) );
    break;
  case COUCHE_C2:
    enregistre_valeurs( &(couche_editee->Cumul_2) );
    break;
  case COUCHE_C3:
    enregistre_valeurs( &(couche_editee->Cumul_3) );
    break;
  case COUCHE_C4:
    enregistre_valeurs( &(couche_editee->Cumul_4) );
    break;

  case COUCHE_DELTAE:
    enregistre_valeurs( &(couche_editee->Delta_E) );
    break;

  case COUCHE_GAMMA:
    enregistre_valeurs( &(couche_editee->Gamma) );
    break;
  }

  if (objet == COUCHE_UTILISER)
  {
    OBJ_choisit(PARAM_COUCHES, objet, ECHANGE);
    return VRAI;
  }

  OBJ_choisit(PARAM_COUCHES, objet, FORCE_OUI);

  /* ---- On traite l'objet cliqu ---- */
  
  switch (objet)
  {
  case COUCHE_N:
    var_editee = COUCHE_N;
    affiche_valeurs( &(couche_editee->N) );
    return VRAI;

  case COUCHE_DEBYEW:
    var_editee = COUCHE_DEBYEW;
    affiche_valeurs( &(couche_editee->Debye_Waller) );
    return VRAI;

  case COUCHE_LONGUEUR:
    var_editee = COUCHE_LONGUEUR;
    affiche_valeurs( &(couche_editee->R) );
    return VRAI;

  case COUCHE_C1:
    var_editee = COUCHE_C1;
    affiche_valeurs( &(couche_editee->Cumul_1) );
    return VRAI;
  case COUCHE_C2:
    var_editee = COUCHE_C2;
    affiche_valeurs( &(couche_editee->Cumul_2) );
    return VRAI;
  case COUCHE_C3:
    var_editee = COUCHE_C3;
    affiche_valeurs( &(couche_editee->Cumul_3) );
    return VRAI;
  case COUCHE_C4:
    var_editee = COUCHE_C4;
    affiche_valeurs( &(couche_editee->Cumul_4) );
    return VRAI;

  case COUCHE_DELTAE:
    var_editee = COUCHE_DELTAE;
    affiche_valeurs( &(couche_editee->Delta_E) );
    return VRAI;

  case COUCHE_GAMMA:
    var_editee = COUCHE_GAMMA;
    affiche_valeurs( &(couche_editee->Gamma) );
    break;

  case COUCHE_CONSTANT:
    OBJ_active(PARAM_COUCHES, COUCHE_VAL_MIN, FORCE_NON);
    OBJ_active(PARAM_COUCHES, COUCHE_VAL_MAX, FORCE_NON);
    OBJ_active(PARAM_COUCHES, COUCHE_PAS    , FORCE_NON);
    OBJ_active(PARAM_COUCHES, COUCHE_VAL_RELAT, FORCE_NON);
    return VRAI;

  case COUCHE_VAR_REF:
    OBJ_active(PARAM_COUCHES, COUCHE_VAL_MIN, FORCE_OUI);
    OBJ_active(PARAM_COUCHES, COUCHE_VAL_MAX, FORCE_OUI);
    OBJ_active(PARAM_COUCHES, COUCHE_PAS    , FORCE_OUI);
    OBJ_active(PARAM_COUCHES, COUCHE_VAL_RELAT, FORCE_NON);
    return VRAI;

  case COUCHE_RELATION:
    OBJ_active(PARAM_COUCHES, COUCHE_VAL_MIN, FORCE_OUI);
    OBJ_active(PARAM_COUCHES, COUCHE_VAL_MAX, FORCE_OUI);
    OBJ_active(PARAM_COUCHES, COUCHE_PAS    , FORCE_OUI);
    OBJ_active(PARAM_COUCHES, COUCHE_VAL_RELAT, FORCE_OUI);
    return VRAI;

  case COUCHE_CHEMIN: /* Demander le chemin associ  cet ensemble de paramtres */
    break;

  case COUCHE_EXAFS:  /* Demander le signal EXAFS associ  cette couche */
    couche_editee->signal = GRF_choisir(GR_TT_EXAFS);
    if (couche_editee->signal)
      met_chaine(PARAM_COUCHES, COUCHE_EXAFS, 30, couche_editee->signal->nom, TXT_COPIER);
    else
    {
#ifdef _FRANCAIS_
      met_chaine(PARAM_COUCHES, COUCHE_EXAFS, 30, "Aucun signal EXAFS associ"e_AIGU, TXT_COPIER);
#endif
#ifdef _ENGLISH_
      met_chaine(PARAM_COUCHES, COUCHE_EXAFS, 30, "No EXAFS signal", TXT_COPIER);
#endif
#ifdef _DEUTSCH_
      met_chaine(PARAM_COUCHES, COUCHE_EXAFS, 30, "Kein EXAFS signal", TXT_COPIER);
#endif
    }
    return VRAI;

  case COUCHE_GRF_AMP:
    couche_editee->amplitude = GRF_choisir(GR_EXAFSAMP);
    if ( couche_editee->amplitude )
      met_chaine(PARAM_COUCHES, COUCHE_GRF_AMP, 30, couche_editee->amplitude->nom, TXT_COPIER);
    else
    {
#ifdef _FRANCAIS_
      met_chaine(PARAM_COUCHES, COUCHE_GRF_AMP, 30, "Aucune amplitude associ"e_AIGU"e", TXT_COPIER);
#endif
#ifdef _ENGLISH_
      met_chaine(PARAM_COUCHES, COUCHE_GRF_AMP, 30, "No amplitude associated", TXT_COPIER);
#endif
    }
    return VRAI;

  case COUCHE_GRF_PHASE:
    couche_editee->phase = GRF_choisir(GR_EXAFSPHA);
    if ( couche_editee->phase )
      met_chaine(PARAM_COUCHES, COUCHE_GRF_PHASE, 30, couche_editee->phase->nom, TXT_COPIER);
    else
    {
#ifdef _FRANCAIS_
      met_chaine(PARAM_COUCHES, COUCHE_GRF_PHASE, 30, "Aucun d"e_AIGU"phasage associ"e_AIGU, TXT_COPIER);
#endif
#ifdef _ENGLISH_
      met_chaine(PARAM_COUCHES, COUCHE_GRF_PHASE, 30, "No phase-shift associated", TXT_COPIER);
#endif
    }
    return VRAI;
  }

  return VRAI;
}

/* -------- Affichage des donnes associes  une couche --------------------------------- */

static void affiche_couche(Couche *couche)
{
  met_chaine(PARAM_COUCHES, COUCHE_NOM, 0, couche->nom, TXT_COPIER);

  switch (var_editee) /* Suivant la variable qui est dite, on affiche telle ou telle chose */
  {
  case COUCHE_N:
    affiche_valeurs( &(couche->N) );
    break;

  case COUCHE_LONGUEUR:
    affiche_valeurs( &(couche->R) );
    break;

  case COUCHE_DEBYEW:
    affiche_valeurs( &(couche->Debye_Waller) );
    break;

  case COUCHE_C1:
    affiche_valeurs( &(couche->Cumul_1) );
    break;
  case COUCHE_C2:
    affiche_valeurs( &(couche->Cumul_2) );
    break;
  case COUCHE_C3:
    affiche_valeurs( &(couche->Cumul_3) );
    break;
  case COUCHE_C4:
    affiche_valeurs( &(couche->Cumul_4) );
    break;

  case COUCHE_DELTAE:
    affiche_valeurs( &(couche->Delta_E) );
    break;

  case COUCHE_GAMMA:
    affiche_valeurs( &(couche->Gamma) );
    break;
  }

  if ( couche->signal )
    met_chaine(PARAM_COUCHES, COUCHE_EXAFS, 30, couche->signal->nom, TXT_COPIER);
  else
  {
#ifdef _FRANCAIS_
    met_chaine(PARAM_COUCHES, COUCHE_EXAFS, 30, "Aucun signal associ"e_AIGU, TXT_COPIER);
#endif
#ifdef _ENGLISH_
    met_chaine(PARAM_COUCHES, COUCHE_EXAFS, 30, "No signal associated", TXT_COPIER);
#endif
  }

  if ( couche->amplitude )
    met_chaine(PARAM_COUCHES, COUCHE_EXAFS, 30, couche->amplitude->nom, TXT_COPIER);
  else
  {
#ifdef _FRANCAIS_
    met_chaine(PARAM_COUCHES, COUCHE_GRF_AMP, 30, "Aucune amplitude associ"e_AIGU"e", TXT_COPIER);
#endif
#ifdef _ENGLISH_
    met_chaine(PARAM_COUCHES, COUCHE_GRF_AMP, 30, "No amplitude associated", TXT_COPIER);
#endif
  }

  if ( couche->phase )
    met_chaine(PARAM_COUCHES, COUCHE_GRF_PHASE, 30, couche->phase->nom, TXT_COPIER);
  else
  {
#ifdef _FRANCAIS_
    met_chaine(PARAM_COUCHES, COUCHE_GRF_PHASE, 30, "Aucun d"e_AIGU"phasage associ"e_AIGU, TXT_COPIER);
#endif
#ifdef _ENGLISH_
    met_chaine(PARAM_COUCHES, COUCHE_GRF_PHASE, 30, "No phase-shift associated", TXT_COPIER);
#endif
  }

  OBJ_choisit(PARAM_COUCHES, COUCHE_UTILISER, couche->utilisee);
}

/* -------- Mmoristation des donnes associes  une couche ------------------------------- */

static void enregistre_couche(Couche *couche)
{
  lit_chaine(PARAM_COUCHES, COUCHE_NOM, 99, couche->nom, TXT_COPIER);

  switch (var_editee) /* Suivant la variable qui est dite, on affiche telle ou telle chose */
  {
  case COUCHE_N:
    enregistre_valeurs( &(couche->N) );
    break;

  case COUCHE_LONGUEUR:
    enregistre_valeurs( &(couche->R) );
    break;

  case COUCHE_DEBYEW:
    enregistre_valeurs( &(couche->Debye_Waller) );
    break;

  case COUCHE_C1:
    enregistre_valeurs( &(couche->Cumul_1) );
    break;
  case COUCHE_C2:
    enregistre_valeurs( &(couche->Cumul_2) );
    break;
  case COUCHE_C3:
    enregistre_valeurs( &(couche->Cumul_3) );
    break;
  case COUCHE_C4:
    enregistre_valeurs( &(couche->Cumul_4) );
    break;

  case COUCHE_DELTAE:
    enregistre_valeurs( &(couche->Delta_E) );
    break;

  case COUCHE_GAMMA:
    enregistre_valeurs( &(couche->Gamma) );
    break;
  }

  couche->utilisee = OBJ_choisit(PARAM_COUCHES, COUCHE_UTILISER, QUESTION);

  /* Les infos de phase, amplitude, exafs... ont t modifies au vol donc... */
}

/* -------- Affichage des donnes associes  une variable ------------------------------- */

static void affiche_valeurs(Variable *var)
{
  met_fvaleur(PARAM_COUCHES, COUCHE_VAL_DEBUT, var->valeur_initiale);

  OBJ_active (PARAM_COUCHES, COUCHE_VAL_MIN  , var->type != VAR_CONSTANTE);
  OBJ_active (PARAM_COUCHES, COUCHE_VAL_MAX  , var->type != VAR_CONSTANTE);
  OBJ_active (PARAM_COUCHES, COUCHE_VAL_RELAT, var->type == VAR_RELATION );

  if ( var->type == VAR_CONSTANTE )
  {
    OBJ_choisit(PARAM_COUCHES, COUCHE_CONSTANT, FORCE_OUI);
  }
  else
  {
    met_fvaleur(PARAM_COUCHES, COUCHE_VAL_MIN, var->min);
    met_fvaleur(PARAM_COUCHES, COUCHE_VAL_MAX, var->max);
    met_fvaleur(PARAM_COUCHES, COUCHE_PAS, var->delta);

    if ( var->type == VAR_RELATION )
    {
      OBJ_choisit(PARAM_COUCHES, COUCHE_RELATION, FORCE_OUI);
      /* Rafficher la relation */
    }
    else
      OBJ_choisit(PARAM_COUCHES, COUCHE_VAR_REF, FORCE_OUI);
  }
}

/* -------- Mmorisation des donnes associes  une variable --------------------------- */

static void enregistre_valeurs(Variable *var)
{
  var->valeur_initiale = prend_fvaleur(PARAM_COUCHES, COUCHE_VAL_DEBUT);

  if ( OBJ_choisit(PARAM_COUCHES, COUCHE_CONSTANT, QUESTION) ) /* Variable constante */
  {
    var->type = VAR_CONSTANTE;
    return;
  }

  var->min = prend_fvaleur(PARAM_COUCHES, COUCHE_VAL_MIN);
  var->max = prend_fvaleur(PARAM_COUCHES, COUCHE_VAL_MAX);
  var->delta = prend_fvaleur(PARAM_COUCHES, COUCHE_PAS);

  if ( OBJ_choisit(PARAM_COUCHES, COUCHE_VAR_REF, QUESTION) ) /* Variable de type normal */
  {
    var->type = VAR_LIBRE;
    return;
  }

  var->type = VAR_RELATION; /* Variable de type relation */
  /* Mmoriser la relation */
}

/* -------- Active / inactive tous les boutons de l'arbre de paramtrage des couches ------\
 |   Ne renvoie rien                                                                       |
 |                                                                                         |
 | sens : VRAI = il faut activer (il existe des couches)                                   |
 |        FAUX = il faut dsactiver (il n'existe aucune couche)                            |
 \---------------------------------------------------------------------------------------- */

void active_tout(boolean sens)
{
  OBJ_active(PARAM_COUCHES, TTEFFACER_COUCHE, sens);
  OBJ_active(PARAM_COUCHES, EFFACER_COUCHE  , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_NOM      , sens);
  if (sens == FAUX) /* Aucune couche */
  {
#ifdef _FRANCAIS_
    met_chaine(PARAM_COUCHES, COUCHE_NOM, 0, "Aucune couche d"e_AIGU"finie pour l'instant", TXT_COPIER);
#endif
#ifdef _ENGLISH_
    met_chaine(PARAM_COUCHES, COUCHE_NOM, 0, "No couche defined", TXT_COPIER);
#endif
  }
  OBJ_active(PARAM_COUCHES, COUCHE_N, sens);
  OBJ_active(PARAM_COUCHES, COUCHE_DEBYEW, sens);
  OBJ_active(PARAM_COUCHES, COUCHE_C1, sens);
  OBJ_active(PARAM_COUCHES, COUCHE_C2, sens);
  OBJ_active(PARAM_COUCHES, COUCHE_C3, sens);
  OBJ_active(PARAM_COUCHES, COUCHE_C4, sens);
  OBJ_active(PARAM_COUCHES, COUCHE_LONGUEUR, sens);
  if (sens == VRAI) /* On cherche  activer */
  {
    OBJ_active(PARAM_COUCHES, COUCHE_DELTAE, OBJ_choisit(AJUSTE_GENERAL, DELTAE_GENERAL, QUESTION) == FAUX );
    OBJ_active(PARAM_COUCHES, COUCHE_GAMMA , OBJ_choisit(AJUSTE_GENERAL, GAMMA_GENERAL , QUESTION) == FAUX );
  }
  else
  {
    OBJ_active(PARAM_COUCHES, COUCHE_DELTAE, FORCE_NON);
    OBJ_active(PARAM_COUCHES, COUCHE_GAMMA , FORCE_NON);
  }
  OBJ_active(PARAM_COUCHES, COUCHE_VAL_DEBUT, sens);
  OBJ_active(PARAM_COUCHES, COUCHE_VAR_REF  , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_VAL_MIN  , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_VAL_MAX  , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_PAS      , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_CONSTANT , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_VAL_RELAT, sens);
  OBJ_active(PARAM_COUCHES, COUCHE_RELATION , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_CHEMIN   , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_EXAFS    , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_GRF_AMP  , sens);
  OBJ_active(PARAM_COUCHES, COUCHE_GRF_PHASE, sens);
}
