/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS
   Ce fichier : cration le 16 septembre 1998

   Module d'ajustement par un modle  n couches

   Dclarations pour l'ensemble de ce module
*/

#ifndef _N_COUCHES_

#define _N_COUCHES_ /* Protection contre l'inclusion multiple */

#include "c_formel.h" /* Dclaration de ce qu'est une variable */

typedef struct couche
{
  struct couche *suivante; /* Pointeur sur la couche suivante dans le modle */
  char nom[100];           /* Nom de la couche */

  boolean utilisee;        /* VRAI s'il faut la prendre en compte dans le modle */

  Variable N;              /* Nombre de voisins = dgnrescence    */
  Variable R;              /* Longueur du chemin                    */
  Variable Debye_Waller;   /* Facteur de Debye-Waller               */
  Variable Cumul_1;        /* Cumulants pour le dsordre            */
  Variable Cumul_3;
  Variable Cumul_2;        /* Cumulants pour la phase, si empirique */
  Variable Cumul_4;
  Variable Delta_E;        /* Delta_E, si un par couche             */
  Variable Gamma;          /* Libre parcours moyen, mme remarque   */

  /* Ajouter un pointeur vers un chemin exafs = structure  crer */

  Graphe *signal;          /* Signal EXAFS associ au chemin, s'il existe  */
  Graphe *amplitude;       /* Amplitude associe au chemin, si elle existe */
  Graphe *phase;           /* Dphasage associ au chemin, s'il existe     */

  double *b_amp, *c_amp, *d_amp; /* Coefficients de lissage pour l'amplitude */
  double *b_pha, *c_pha, *d_pha; /* Coefficients de lissage pour la phase    */
} Couche;

#define VAR_SO2          1 /* Type de la variable pour l'ajustement */
#define VAR_DELTAE_GENE  2
#define VAR_GAMMA_GENE   3

#define VAR_N           10
#define VAR_DEBYE       11
#define VAR_R           12
#define VAR_CUMUL_1     13
#define VAR_CUMUL_2     14
#define VAR_CUMUL_3     15
#define VAR_CUMUL_4     16
#define VAR_DELTAE      17
#define VAR_GAMMA       18

#define NOMBRE_ALGORITHMES 4

#define LEVENBERG  1 /* Algorithme de type Levenberg-Maquardt modifi (EDA) */
#define GRADIENT   2
#define SIMPLEXE   3
#define GR_ET_SIMP 4

/* ------ Routines de gestion des couches (couches.c) ------ */

Couche *creer_couche  (void);
void    effacer_couche(Couche *couche, boolean forcer);

/* ------ Routines de paramtrage de l'ajustement ---------- */

void    ASC_nc_couches(Fenetre *fenetre, int type, int ss_type, int position);
boolean GER_nc_couches(int objet, int x, int y);

#ifndef _VARIABLES_

extern Variable Delta_E; /* Dans le cas o il est commun  toutes les couches */
extern Variable Gamma; /* Mme remarque */
extern Variable S0_2; /* Attnuation due aux effets multi-lectroniques */

extern unsigned long type_algorithme; /* Type d'algorithme utilis - UL pour simplifier utilisation du dfileur */

extern Couche *premiere_couche;

extern boolean deltae_general;    /* FAUX si un delta_e par couche */
extern boolean gamma_general;     /* FAUX si un gamma par couche   */
extern int type_gamma;            /* Faon de calculer gamma       */
extern double val_eta;            /* Valeur de ta, si ncessaire  */

extern Graphe *phase_central;     /* Dphasage de l'atome central  */

extern double kmin_ajuste, kmax_ajuste; /* Bornes  considrer pour l'ajustement */
extern int ponder_ajuste;               /* Coefficient de pondration            */

#endif /* _VARIABLES_ */

/* ------ Routines de calcul effectif de l'ajustement ------ */

typedef struct
{
  Graphe *reference;    /* Spectre de rfrence                     */
  Graphe *ajuste;       /* Spectre ajust,  l'tape indique aprs */
  Graphe *residu;       /* Diffrence entre les deux                */
  double erreur;        /* Erreur entre les deux (type moindres carrs simples) */
  boolean en_cours;     /* VRAI si le calcul est en cours, FAUX s'il est fini   */
  unsigned int etape;   /* Itration en cours...                  */
  unsigned int nbr_var; /* Nombre de variables vraies             */
} Infos_ncouches;       /* Informations sur l'ajustement en cours */

void lancer_ncouches(Graphe *reference);

double calc_derivee       (double k, Couche *couche, int type, double val_khi);
double calc_erreur        (Infos_ncouches *infos);
int    calc_exafs_ncouches(Couche *prem_couche, Graphe *exafs);
double calc_exafs_1point  (double k, Couche *couche);
double calc_residu        (Infos_ncouches *infos);

int NC_levenberg(Infos_ncouches *infos); /* Algo. de Levenberg */

#endif /* _N_COUCHES_ */
