/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS
   Ce fichier : cration le 16 septembre 1998

   Module d'ajustement par un modle  n couches

   Paramtrage des paramtres gnraux de l'ajustement
*/

#include <stdio.h>
#include <stdlib.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "couches.h"

/* Dfinies dans variables.c */

/* extern Variable Delta_E; /* Dans le cas o il est commun  toutes les couches */
/* extern Variable Gamma;   /* Mme remarque */
/* extern Variable S0_2;

extern Couche *premiere_couche;

extern unsigned long type_algorithme; */

void active_tout(boolean sens); /* Dans couches.c */

/* ~~~~~~~~ Variables locales  ce module ~~~~~~~~ */

static const char *txt_algos[] =
{
  "Levenberg-Maquardt modifi"e_AIGU,
  "Gradient",
  "Simplexe",
  "Monte-Carlo"
};

static int var_editee = AJ_GENE_S02; /* Variable en cours d'dition */
static unsigned long prem_algo_affiche = 1;

/* ~~~~~~~~ Fonctions locales  ce module ~~~~~~~~ */

static boolean GER_nc_generaux(int objet, int x, int y);
static void enregistre_valeurs(Variable *var);
static void affiche_valeurs   (Variable *var);

/* ~~~~~~~~ Ralisation effective du module ~~~~~~ */

/* -------- Initialisation des diffrents arbres GEM concerns -------- */

void init_ncouches(void)
{
  /* Arbre des paramtres gnraux - init. : S0_2 propos, en variable, valeur par dfaut = 1 */

  arbres[AJUSTE_GENERAL].fonction.arbre = GER_nc_generaux;
  PDFL_textes(AJUSTE_GENERAL, GENE_PREM_ALGO, GENE_DER_ALGO,
	      GENE_ALGO_AVANT, GENE_ALGO_APRES, GENE_ALGO_FOND, GENE_ALGO_BOUTON,
	      NOMBRE_ALGORITHMES, (char **) txt_algos, NULL, 1, 30, 0);
  OBJ_choisit(AJUSTE_GENERAL, AJ_GENE_S02, FORCE_OUI);
  met_fvaleur(AJUSTE_GENERAL, GENE_VAL_DEBUT, S0_2.valeur_initiale);
  met_fvaleur(AJUSTE_GENERAL, GENE_MINIMUM, S0_2.min);
  met_fvaleur(AJUSTE_GENERAL, GENE_MAXIMUM, S0_2.max);
  met_fvaleur(AJUSTE_GENERAL, GENE_PAS, S0_2.delta);
  OBJ_choisit(AJUSTE_GENERAL, GENE_VARIABLE, FORCE_OUI);

  /* Arbre des paramtres de couche - aucune couche cre */

  arbres[PARAM_COUCHES].fonction.arbre = GER_nc_couches;
  OBJ_choisit(PARAM_COUCHES, COUCHE_N, FORCE_OUI);
  active_tout( premiere_couche != NULL );
}

/* -------- Gestion de l'arbre de paramtrage : paramtres gnraux --- */

static boolean GER_nc_generaux(int objet, int x, int y)
{
  if ( (objet >= GENE_PREM_ALGO) && (objet <= GENE_DER_ALGO) )
  {
    GDFL_textes(AJUSTE_GENERAL, GENE_PREM_ALGO, GENE_DER_ALGO,
		GENE_ALGO_AVANT, GENE_ALGO_APRES, GENE_ALGO_FOND, GENE_ALGO_BOUTON,
		NOMBRE_ALGORITHMES, (char **) txt_algos, NULL, &type_algorithme, 30,
		&prem_algo_affiche, objet, y);

    return VRAI;
  }

  if (var_editee == AJ_GENE_S02)
    enregistre_valeurs( &S0_2 );
  else if (var_editee == GENE_GAMMA)
    enregistre_valeurs( &Gamma );
  else if (var_editee == GENE_DELTAE)
    enregistre_valeurs( &Delta_E );

  switch (objet)
  {
  case AJ_GENE_S02: /* On demande l'dition des valeurs de S_0^2 */
    OBJ_choisit(AJUSTE_GENERAL, AJ_GENE_S02, FORCE_OUI);
    affiche_valeurs( &S0_2 );
    var_editee = AJ_GENE_S02;
    return VRAI;

  case GENE_GAMMA:  /* On demande l'dition des valeurs de Gamma, si global au modle */
    OBJ_choisit(AJUSTE_GENERAL, GENE_GAMMA, FORCE_OUI);
    affiche_valeurs( &Gamma );
    var_editee = GENE_GAMMA;
    return VRAI;
    
  case GENE_DELTAE:  /* On demande l'dition des valeurs de Delta_E, si global au modle */
    OBJ_choisit(AJUSTE_GENERAL, GENE_DELTAE, FORCE_OUI);
    affiche_valeurs( &Delta_E );
    var_editee = GENE_DELTAE;
    return VRAI;

  case GENE_CONSTANT: /* Variable de type constant */
    OBJ_choisit(AJUSTE_GENERAL, GENE_CONSTANT, FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_MINIMUM , FORCE_NON);
    OBJ_active (AJUSTE_GENERAL, GENE_MAXIMUM , FORCE_NON);
    OBJ_active (AJUSTE_GENERAL, GENE_PAS, FORCE_NON);
    return VRAI;

  case GENE_VARIABLE: /* Variable de type simple */
    OBJ_choisit(AJUSTE_GENERAL, GENE_VARIABLE, FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_MINIMUM , FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_MAXIMUM , FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_PAS, FORCE_OUI);
    return VRAI;

  case DELTAE_GENERAL: /* On utilise une seule valeur de Delta_E pour toutes les couches */
    OBJ_choisit(AJUSTE_GENERAL, DELTAE_GENERAL, FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_DELTAE   , FORCE_OUI); /* Donc le paramtrage a un sens */
    OBJ_active (PARAM_COUCHES , COUCHE_DELTAE , FORCE_NON); /* Mais pas pour le couche par couche */
    return VRAI;

  case DELTAE_COUCHE: /* On utilise une valeur de Delta_E par couche */
    OBJ_choisit(AJUSTE_GENERAL, DELTAE_COUCHE, FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_DELTAE  , FORCE_NON); /* Donc le paramtrage n'a pas de sens */
    if (var_editee == GENE_DELTAE) /* On passe  une autre variable */
    {
      enregistre_valeurs( &Delta_E );
      var_editee = AJ_GENE_S02;
      affiche_valeurs( &S0_2 );
      OBJ_choisit(AJUSTE_GENERAL, AJ_GENE_S02, FORCE_OUI);
    }  
    OBJ_active (PARAM_COUCHES , COUCHE_DELTAE, FORCE_OUI); /* Mais il en a un pour le couche par couche */
    return VRAI;

  case GAMMA_GENERAL: /* On utilise une seule valeur de Gamma pour toutes les couches */
    OBJ_choisit(AJUSTE_GENERAL, GAMMA_GENERAL, FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_GAMMA   , FORCE_OUI); /* Donc le paramtrage a un sens */
    OBJ_active (PARAM_COUCHES , COUCHE_GAMMA , FORCE_NON); /* Mais pas pour le couche par couche */
    break;

  case GAMMA_COUCHE:  /* On utilise une valeur de Gamma par couche */
    OBJ_choisit(AJUSTE_GENERAL, GAMMA_COUCHE , FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_GAMMA   , FORCE_NON); /* Donc le paramtrage n'a pas de sens */
    if (var_editee == GENE_GAMMA) /* On passe  une autre variable */
    {
      enregistre_valeurs( &Gamma );
      var_editee = AJ_GENE_S02;
      affiche_valeurs( &S0_2 );
      OBJ_choisit(AJUSTE_GENERAL, AJ_GENE_S02, FORCE_OUI);
    }  
    OBJ_active (PARAM_COUCHES , COUCHE_GAMMA, FORCE_OUI); /* Mais il en a un pour le couche par couche */
    break;

  case GENE_UTIL_SIGMA: /* Faut-il utiliser les barres d'erreur ? */
    OBJ_choisit(AJUSTE_GENERAL, GENE_UTIL_SIGMA, ECHANGE);
    return VRAI;

  case GENE_ALGO_AVANT:
  case GENE_ALGO_FOND:
  case GENE_ALGO_BOUTON:
  case GENE_ALGO_APRES:
    return VRAI; /* Voir quand il y aura plus d'algorithmes... */

  case FIN_VAR_FAIBLE:
  case FIN_N_TOURS:
  case FIN_ERREUR:
    OBJ_choisit(AJUSTE_GENERAL, objet, ECHANGE);
    break;

  case GENE_LPM_INCLUS: /* Le libre parcours moyen est inclus dans les amplitudes */
    OBJ_choisit(AJUSTE_GENERAL, GENE_LPM_INCLUS, FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_VAL_ETA, FORCE_NON);
    OBJ_active (PARAM_COUCHES , COUCHE_GAMMA, FORCE_NON); /* Donc le paramtrage n'a pas de sens */
    OBJ_active (AJUSTE_GENERAL, GENE_GAMMA  , FORCE_NON);
    if (var_editee == GENE_GAMMA) /* On passe  une autre variable */
    {
      enregistre_valeurs( &Gamma );
      var_editee = AJ_GENE_S02;
      affiche_valeurs( &S0_2 );
      OBJ_choisit(AJUSTE_GENERAL, AJ_GENE_S02, FORCE_OUI);
    }
    OBJ_active(AJUSTE_GENERAL, GAMMA_GENERAL, FORCE_NON);
    OBJ_active(AJUSTE_GENERAL, GAMMA_COUCHE , FORCE_NON);
    type_gamma = objet;
    break;

  case GENE_LPM_GAMMA:  /* Le libre parcours moyen est calcul thoriquement */
  case GENE_LPM_G_ETA:
    OBJ_choisit(AJUSTE_GENERAL, objet, FORCE_OUI);
    OBJ_active (AJUSTE_GENERAL, GENE_VAL_ETA, objet == GENE_LPM_G_ETA); /* Activer seulement si G et eta utiliss */
    OBJ_active(AJUSTE_GENERAL, GAMMA_GENERAL, FORCE_OUI);
    OBJ_active(AJUSTE_GENERAL, GAMMA_COUCHE , FORCE_OUI);
    if ( OBJ_choisit(AJUSTE_GENERAL, GAMMA_GENERAL, QUESTION) ) /* Si un gamma par couche */
      OBJ_active (AJUSTE_GENERAL, GENE_GAMMA  , FORCE_OUI); /* Donc le paramtrage n'a pas de sens */
    else
      OBJ_active (PARAM_COUCHES , COUCHE_GAMMA, FORCE_OUI); /* Donc le paramtrage n'a pas de sens */
    type_gamma = objet;
    break;

  case NCGEN_PH_CENTRAL: /* Choix du dphasage de l'atome central */
    phase_central = GRF_choisir(GR_EXAFSPHA);
    if (phase_central)
      met_chaine(AJUSTE_GENERAL, NCGEN_PH_CENTRAL, 30, phase_central->nom, TXT_COPIER);
    else
    {
#ifdef _FRANCAIS_
      met_chaine(AJUSTE_GENERAL, NCGEN_PH_CENTRAL, 30,
		 "Inclus dans les phases des couches", TXT_COPIER);
#endif
#ifdef _ENGLISH_
      met_chaine(AJUSTE_GENERAL, NCGEN_PH_CENTRAL, 30,
		 "Included in layers phases", TXT_COPIER);
#endif
    }
    break;
  }

  return VRAI;
}

/* -------- Mmorisation des valeurs associes  une variable --------- */

static void enregistre_valeurs(Variable *variable)
{
  if ( OBJ_choisit(AJUSTE_GENERAL, GENE_CONSTANT, QUESTION) ) /* La variable est dclare constante */
    variable->type = VAR_CONSTANTE;
  else if ( OBJ_choisit(AJUSTE_GENERAL, GENE_VARIABLE, QUESTION) )
    variable->type = VAR_LIBRE;
  else /* Pas d'autre solution pour le moment */
    variable->type = VAR_RELATION;

  variable->valeur_initiale = prend_fvaleur(AJUSTE_GENERAL, GENE_VAL_DEBUT);
  if (variable->type == VAR_LIBRE) /* On prend aussi les bornes et le pas */
  {
    variable->min = prend_fvaleur(AJUSTE_GENERAL, GENE_MINIMUM);
    variable->max = prend_fvaleur(AJUSTE_GENERAL, GENE_MAXIMUM);
    variable->delta = prend_fvaleur(AJUSTE_GENERAL, GENE_PAS);
  }
}

/* -------- Affichage des valeurs associes  une variable --------- */

static void affiche_valeurs(Variable *variable)
{

  met_fvaleur(AJUSTE_GENERAL, GENE_VAL_DEBUT, variable->valeur_initiale);

  if (variable->type == VAR_CONSTANTE)
  {
    GER_nc_generaux(GENE_CONSTANT, 0, 0);
  }
  else
  {
    GER_nc_generaux(GENE_VARIABLE, 0, 0);

    met_fvaleur(AJUSTE_GENERAL, GENE_MINIMUM, variable->min);
    met_fvaleur(AJUSTE_GENERAL, GENE_MAXIMUM, variable->max);
    met_fvaleur(AJUSTE_GENERAL, GENE_PAS, variable->delta);
  }
}
