/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS
   Ce fichier : cration le 17 septembre 1998

   Module d'ajustement par un modle  n couches

   Lancement de l'ajustement
*/

#include <stdio.h>
#include <stdlib.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"
#include "ERREURS.H"

#include "couches.h"
#include "LISSAGE.H"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static void    RDS_ncouches(Fenetre *fenetre, int x, int y, int larg, int haut);
static boolean MSG_ncouches(Fenetre *);

static void init_exafs(Infos_ncouches *infos, Graphe *reference); /* Si le spectre  ajuster est un EXAFS */
static void init_tf   (Infos_ncouches *infos, Graphe *reference); /* Si le spectre  ajuster est une TF   */

static void termine_ncouches(Infos_ncouches *infos);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Ouverture de la fentre de lancement de l'ajustement, avec affichage des rsultats ------\
 |   Ne renvoie rien                                                                                 |
 |                                                                                                   |
 | reference : le spectre EXAFS que l'on veut ajuster par le thorique                               |
 \-------------------------------------------------------------------------------------------------- */

void lancer_ncouches(Graphe *reference)
{
  Fenetre *fenetre;
  Infos_ncouches *infos;
  Couche *couche;
  unsigned int nbr_couches = 0; /* Nombre de couches utilises */

  if (reference == NULL) /* Pas de rfrence */
    return;

  if (premiere_couche == NULL)
  {
    printf("ERREUR, ajustement, modle  n couches : aucune couche dfinie\n");
    return;
  }

  kmin_ajuste = prend_fvaleur(AJUSTE_GENERAL, NCGEN_KMIN);
  kmax_ajuste = prend_fvaleur(AJUSTE_GENERAL, NCGEN_KMAX);
  if ( kmax_ajuste < kmin_ajuste )
  {
    double k = kmax_ajuste; kmax_ajuste = kmin_ajuste ; kmin_ajuste = k;
  }
  if ( kmin_ajuste == kmax_ajuste )
  {
    printf("Ajustement, modle  n couches : intervalle d'ajustement nul !!\n");
    return;
  }
  if ( (kmax_ajuste < reference->x_min) || (kmin_ajuste > reference->x_max) )
  {
    printf("Ajustement, modle  n couches : zone d'ajustement hors du graphe !\n");
    return;
  }
  if ( kmax_ajuste > reference->x_max ) /* On dborde de la rfrence : correction automatique */
    kmax_ajuste = reference->x_max;
  if ( kmin_ajuste < reference->x_min ) /* Mme remarque */
    kmin_ajuste = reference->x_min;

  if ( (reference->type != GR_TF) && (reference->drapeaux.exafs == FAUX) )
  {
    printf("Ajustement, modle  n couches : ajustement impossible sur ce type de graphes !\n");
    return;
  }

  ponder_ajuste = prend_valeur(AJUSTE_GENERAL, GENE_PONDERE);

  fenetre = creer_mixte_fenetre(NAME|MOVER|CLOSER|SIZER|FULLER|INFO, RESULT_N_COUCHES, GAUCHE,
				RDS_ncouches,
				( boolean (*)(Fenetre *, unsigned int, int) ) NULL,
				( boolean (*)(Fenetre *, int, int, int, int, int) ) NULL,
				MSG_ncouches,
				( boolean (*)(Fenetre *, int, int) ) NULL,
				( boolean (*)(Fenetre *) ) NULL );
  if (fenetre == NULL) /* Problme  la cration de la fentre */
    return;

#ifdef _FRANCAIS_
  FEN_titre(fenetre, "Ajustement en cours - Mod"e_GRAVE"le "a_GRAVE" n couches");
  FEN_infos(fenetre, "Noir : spectre exp"e_AIGU"rimental - Rouge : spectre ajust"e_AIGU" - Bleu : diff"e_AIGU"rence");
#endif
#ifdef _ENGLISH_
  FEN_titre(fenetre, "Fitting - n layers model");
  FEN_infos(fenetre, "Black : experimental spectrum - Red : fitted spectra - Blue : difference");
#endif

  if ( fenetre->pointeur != NULL ) /* Il y a dj des infos */
  {
    infos = (Infos_ncouches *) (fenetre->pointeur);
    if (infos->en_cours == VRAI)
    {
      printf("Ajustement en cours - Impossible d'en lancer un autre dans la version actuelle\n");
      return;
    }

    infos->reference->drapeaux.protege = FAUX;
    infos->ajuste->drapeaux.protege = FAUX;
    infos->residu->drapeaux.protege = FAUX;
  }
  else
  {
    infos = (Infos_ncouches *) malloc( sizeof(Infos_ncouches) );
    if (infos == NULL)
    {
      ferme_fenetre(fenetre); /* Avant message pour viter redessin intempestif */
      ERREUR(MANQUE_MEMOIRE);
      return;
    }
  }

  reference->drapeaux.protege = VRAI; /* Utilis dans un calcul */
  infos->reference = reference;       /* Spectre de rfrence   */
  fenetre->pointeur = (void *) infos;
  infos->nbr_var = 0;

  deltae_general = OBJ_choisit(AJUSTE_GENERAL, DELTAE_GENERAL, QUESTION);              /* VRAI si DE gnral */
  gamma_general  =   OBJ_choisit(AJUSTE_GENERAL, GAMMA_GENERAL, QUESTION)
                && ( OBJ_choisit(AJUSTE_GENERAL, GENE_LPM_INCLUS, QUESTION) == FAUX ); /* VRAI si G gnral non inclus */

  /* On affiche les valeurs de dmarrage */

  /* 1) valeurs gnrales */

  met_fvaleur( RESULT_N_COUCHES, RNC_VAL_S02    , VAR_prend_valeur( &S0_2    ) );
  OBJ_active ( RESULT_N_COUCHES, RNC_DELTAE_GENE, deltae_general);
  met_fvaleur( RESULT_N_COUCHES, RNC_DELTAE_GENE, VAR_prend_valeur( &Delta_E ) );
  OBJ_active ( RESULT_N_COUCHES, RNC_GAMMA_GENE, gamma_general);
  met_fvaleur( RESULT_N_COUCHES, RNC_GAMMA_GENE , VAR_prend_valeur( &Gamma   ) );

  /* 2) valeurs pour les couches : on commence par la premire couche */

  OBJ_active(RESULT_N_COUCHES, RNC_COUCHE_AVANT, FORCE_NON); /* Aucune couche avant */

  couche = premiere_couche;
  while ( couche->utilisee == FAUX ) /* On se place sur la premire couche utilise */
  {
    couche = couche->suivante;
    if (couche == NULL)
    {
      printf("ERREUR, ajustement, modle  n couches : aucune couche choisie - FIT termin !\n");
      infos->en_cours == FAUX;
      OBJ_active(RESULT_N_COUCHES, RNC_ARRETER, FORCE_NON); /* Inutile... */
      OBJ_active(RESULT_N_COUCHES, RNC_COUCHE_APRES, FORCE_NON); /* Aucune couche aprs non plus */
      termine_ncouches(infos);
      return;
    }
  }

  met_chaine ( RESULT_N_COUCHES, RNC_NOM_COUCHE, 23, couche->nom, TXT_COPIER);
  met_fvaleur( RESULT_N_COUCHES, RNC_VAL_N       , VAR_prend_valeur( &(couche->N) ) );
  met_fvaleur( RESULT_N_COUCHES, RNC_DEBYE_WALLER, VAR_prend_valeur( &(couche->Debye_Waller) ) );
  met_fvaleur( RESULT_N_COUCHES, RNC_VAL_R       , VAR_prend_valeur( &(couche->R) ) );
  met_fvaleur( RESULT_N_COUCHES, RNC_DELTAE_COUCH, VAR_prend_valeur( &(couche->Delta_E) ) );
  met_fvaleur( RESULT_N_COUCHES, RNC_GAMMA_COUCHE, VAR_prend_valeur( &(couche->Gamma) ) );
  met_fvaleur( RESULT_N_COUCHES, RNC_CUMUL1      , VAR_prend_valeur( &(couche->Cumul_1) ) );
  met_fvaleur( RESULT_N_COUCHES, RNC_CUMUL_3     , VAR_prend_valeur( &(couche->Cumul_3) ) );
  met_fvaleur( RESULT_N_COUCHES, RNC_CUMUL2      , VAR_prend_valeur( &(couche->Cumul_2) ) );
  met_fvaleur( RESULT_N_COUCHES, RNC_CUMUL4      , VAR_prend_valeur( &(couche->Cumul_4) ) );

  /* On calcule les coefficients des splines pour l'amplitude et le dphasage, pour les couches utilises */

  for ( ; couche ; couche = couche->suivante )
  {
    if ( couche->utilisee ) /* La couche est utilise */
    {
      if (couche->amplitude)
      {
	if ( SPL_tt_prep(couche->amplitude, &(couche->b_amp), &(couche->c_amp), &(couche->d_amp)) < 0)
	  couche->utilisee = FAUX; /* Impossible de l'utiliser car erreur en crant les coefficients des splines */
	else
	{
	  if (couche->phase)
	  {
	    if ( SPL_tt_prep(couche->phase, &(couche->b_pha), &(couche->c_pha), &(couche->d_pha)) < 0)
	    {
	      couche->utilisee = FAUX; /* Impossible de l'utiliser car erreur en crant les coefficients des splines */
	      free(couche->b_amp); couche->b_amp = NULL;
	      free(couche->c_amp); couche->c_amp = NULL;
	      free(couche->d_amp); couche->d_amp = NULL;
	    }
	  }
	  else
	  {
	    printf("ERREUR, ajustement, n couches : pas de phase pour la couche %s\n", couche->nom);
	    couche->utilisee = FAUX;
	    free(couche->b_amp); couche->b_amp = NULL;
	    free(couche->c_amp); couche->c_amp = NULL;
	    free(couche->d_amp); couche->d_amp = NULL;
	  }
	}
      }
      else
      {
	printf("ERREUR, ajustement, n couches : pas d'amplitude pour la couche %s\n", couche->nom);
	couche->utilisee = FAUX;
      }
    }

    if ( couche->utilisee ) /* Elle est vraiment utilise -> on regarde les variables  utiliser */
    {
      nbr_couches++;

      if ( couche->N.type            == VAR_LIBRE ) (infos->nbr_var)++;
      if ( couche->R.type            == VAR_LIBRE ) (infos->nbr_var)++;
      if ( couche->Debye_Waller.type == VAR_LIBRE ) (infos->nbr_var)++;
      if ( couche->Cumul_1.type      == VAR_LIBRE ) (infos->nbr_var)++;
      if ( couche->Cumul_2.type      == VAR_LIBRE ) (infos->nbr_var)++;
      if ( couche->Cumul_3.type      == VAR_LIBRE ) (infos->nbr_var)++;
      if ( couche->Cumul_4.type      == VAR_LIBRE ) (infos->nbr_var)++;

      if ( (deltae_general == FAUX) && (couche->Delta_E.type == VAR_LIBRE) ) (infos->nbr_var)++;
      if ( (gamma_general  == FAUX) && (type_gamma != GENE_LPM_INCLUS)
	  && (couche->Gamma.type == VAR_LIBRE) ) (infos->nbr_var)++;
    }
  }

  if (nbr_couches == 0) /* Aucune couche utilisable */
  {
    termine_ncouches(infos);
    return;
  }
  met_valeur(RESULT_N_COUCHES, RNC_NCOUCHES, nbr_couches); /* Nombre de couches dans le modle */

  if (S0_2.type == VAR_LIBRE ) (infos->nbr_var)++;
  if ( (deltae_general == VRAI) && (Delta_E.type == VAR_LIBRE) ) (infos->nbr_var)++;
  if ( (gamma_general  == VRAI) && (Gamma.type   == VAR_LIBRE) ) (infos->nbr_var)++;

  if ( infos->nbr_var == 0 )
  {
    printf("Ajustement  n couches : aucune variable dfinie !\n");
    termine_ncouches(infos);
    return;
  }
  met_valeur(RESULT_N_COUCHES, RNC_NBR_PARAMS, infos->nbr_var);

  /* On rserve les spectres ajusts et d'erreur */

  if ( reference->drapeaux.exafs == VRAI )
    init_exafs(infos, reference);
  else
    init_tf(infos, reference);
}

/* -------- Initialisation des spectres, cas d'un ajustement sur un spectre EXAFS directement -------- */

static void init_exafs(Infos_ncouches *infos, Graphe *reference)
{
  unsigned long i, nbr_pts = 0;
  register double *x, *x_res;
  char titre[100];

  for ( i = 0 , x = reference->x ; i < reference->nombre_point ; i++, x++)
    if ( (*x >= kmin_ajuste) && (*x <= kmax_ajuste) )
      nbr_pts++;

#ifdef _FRANCAIS_
  sprintf(titre, "Mod"e_GRAVE"le th"e_AIGU"orique de %s", reference->nom);
  infos->ajuste = GRF_creer(nbr_pts, GR_AJUSTE, reference->drapeaux.complexe, titre,
			    "Nombre d'onde", KHI"(k)", UNIT_1_ANG, UNIT_NORM);
#endif
#ifdef _ENGLISH_
  sprintf(titre, "Theoretical model of %s", reference->nom);
  infos->ajuste = GRF_creer(nbr_pts, GR_AJUSTE, reference->drapeaux.complexe, titre,
			    "Wave number", KHI"(k)", UNIT_1_ANG, UNIT_NORM);
#endif
  if (infos->ajuste == NULL)
  {
    termine_ncouches(infos);
    return;
  }
  infos->ajuste->drapeaux.protege = VRAI;

#ifdef _FRANCAIS_
  sprintf(titre, E_AIGU"cart entre %s et le mod"e_GRAVE"le th"e_AIGU"orique", reference->nom);
  infos->residu = GRF_creer(nbr_pts, GR_ECART, reference->drapeaux.complexe, titre,
			    "Nombre d'onde", "Diff"e_AIGU"rence", UNIT_1_ANG, UNIT_NORM);
#endif
#ifdef _ENGLISH_
  sprintf(titre, "Difference between %s and theoretical model", reference->nom);
  infos->residu = GRF_creer(nbr_pts, GR_ECART, reference->drapeaux.complexe, titre,
			    "Wave number", "Difference", UNIT_1_ANG, UNIT_NORM);
#endif
  OBJ_active (RESULT_N_COUCHES, RNC_RESIDU, (infos->residu != NULL) );
  OBJ_choisit(RESULT_N_COUCHES, RNC_RESIDU, (infos->residu != NULL) );

  /* On initialise les X de ce spectre */

  for ( i = 0 , x = reference->x , x_res = (infos->ajuste->x) ;
       i < reference->nombre_point ; i++, x++)
    if ( (*x >= kmin_ajuste) && (*x <= kmax_ajuste) )
    {
      *x_res = *x;
      x_res++;
    }

  if ( infos->residu != NULL )
  {
    memcpy(infos->residu->x, infos->ajuste->x, sizeof(double) * (size_t) (infos->ajuste->nombre_point) );
    infos->residu->drapeaux.protege = VRAI;
  }

  GRF_bornes(infos->ajuste, ORDONNE, ORDONNE); /* Forcment, puisque 0 en Y de toute faon */
  GRF_bornes(infos->residu, ORDONNE, NE_PAS_FAIRE);

  calc_exafs_ncouches(premiere_couche, infos->ajuste); /* On calcule le spectre EXAFS  ajuster */

  infos->erreur = calc_residu(infos); /* Calcul du rsidu et de l'erreur qui va avec */

  /* On demande un redessin avant le dbut de la minimisation */

  FEN_redessin( ARB_fenetre( arbres + RESULT_N_COUCHES ) ); 

  attend_evenement(MU_TIMER|MU_KEYBD, /* vnements attendus */
		   0, 0, 0, 0, 0,     /* rectangle 2 : aucun */
		   0,              /* Compteur  1, sinon plantage sur TOS 1.0 */
		   &_rien_, &_rien_, &_rien_, &_rien_, &_rien_, &_rien_);
  
  printf("Appuyer sur une touche\n");
  getchar();

  /* On lance le calcul effectif */

  switch (type_algorithme)
  {
  case LEVENBERG: /* Algorithme de type EDA */
    NC_levenberg(infos);
    break;
  }
}

/* -------- Initialisation des spectres, cas d'un ajustement sur une TF au passage -------- */

static void init_tf(Infos_ncouches *infos, Graphe *reference)
{
  printf("En cours de ralisation\n");
}

/* ------ Quand on a fini un ajustement, on met tout comme il faut ------ */

static void termine_ncouches(Infos_ncouches *infos)
{
  Couche *couche;

  /* On libre les coefficients de lissage des phases et amplitudes */

  for (couche = premiere_couche ; couche ; couche = couche->suivante)
    if (couche->utilisee)
    {
      free(couche->b_amp); couche->b_amp = NULL;
      free(couche->c_amp); couche->c_amp = NULL;
      free(couche->d_amp); couche->d_amp = NULL;
      free(couche->b_pha); couche->b_pha = NULL;
      free(couche->c_pha); couche->c_pha = NULL;
      free(couche->d_pha); couche->d_pha = NULL;
    }

  /* L'ajustement est termin */

  infos->en_cours = FAUX;
}

/* ~~~~~~~~ Fonctions d'interfaage pur ~~~~~~~~ */

/* -------- Redessin de la fentre -------- */

static void RDS_ncouches(Fenetre *fenetre, int x, int y, int larg, int haut)
{
  DES_graphe param_dessin;
  Infos_ncouches *infos;
  int larg_trace, rien, x_trace;
  double x_echelle, y_echelle;
  Graphe *graphe;

  if (fenetre == NULL) return;
  infos = (Infos_ncouches *) (fenetre->pointeur); /* Adresse des infos */

  OBJ_taille  (RESULT_N_COUCHES, ROOT, -1, -1, &x_trace, &rien);
  x_trace++;
  larg_trace = fenetre->w_larg - x_trace;

  /* Le fond en blanc */
  vswr_mode(_PRG_station->handle, MD_REMPLACE);
  vsf_color(_PRG_station->handle, BLANC); /* couleur de remplissage: blanc */

  points[0] = x;
  points[1] = y;
  points[2] = x + larg - 1;
  points[3] = y + haut - 1;
  v_bar(_PRG_station->handle, points);

  graphe = infos->reference;

  x_echelle = ( (double) larg_trace )      / (graphe->x_max - graphe->x_min);
  y_echelle = ( (double) fenetre->w_haut ) / (graphe->y_max - graphe->y_min);

  /* On trace le graphe de rfrence, en ligne noire continue */

  /* On trace la courbe exprimentale - en noir */

  param_dessin.graphe  = graphe;
  param_dessin.couleur = NOIR;
  param_dessin.style   = 1;
  TRC_graphe(_PRG_station, &param_dessin, x_trace, fenetre->w_y,
	     graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);

  /* On trace le spectre calcul - en rouge */

  if (infos->ajuste) /* Le graphe d'ajustement existe */
  {
    param_dessin.graphe  = infos->ajuste;
    param_dessin.couleur = ROUGE;
    TRC_graphe(_PRG_station, &param_dessin, x_trace, fenetre->w_y,
	       graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);
  }

  /* On trace le spectre diffrence - en bleu */

  if ( OBJ_choisit(RESULT_N_COUCHES, RNC_RESIDU, QUESTION) == VRAI )
  {
    param_dessin.graphe  = infos->residu;
    param_dessin.couleur = BLEU;
    TRC_graphe(_PRG_station, &param_dessin, x_trace, fenetre->w_y,
	       graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);
  }
}

/* -------- Messages pour la fentre ------ */

static boolean MSG_ncouches(Fenetre *fenetre)
{
  Infos_ncouches *infos;

  if (fenetre == NULL)
    return;

  infos = (Infos_ncouches *) (fenetre->pointeur);
  if (infos == NULL)
    return;

  if ( buffer[0] == WM_CLOSED ) /* Fermeture de la fentre */
  {
  }

  return VRAI;
}


