/* LASE - Logiciel d'Analyse des Spectres EXAFS
   (c) 1996-1998, Emmanuel CURIS
   Ce fichier : cration le 28 septembre 1998

   Module d'ajustement par un modle  n couches

   Ajustement par la mthode de Levenberg - Maquardt modifie
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"
#include "LISSAGE.H"
/* #include "SCIENCE.H" */

#include "couches.h"

/* ~~~~~~~~ Variables locales ~~~~~~~~ */

typedef struct
{
  Variable *var;
  double ex_valeur; /* Valeur avant le tour d'ajustement, mmorise si algo. diverge */
  double increment;
  double num, den ; /* Numrateur et dnominateur           */
  int type;         /* Le type de variable dont il s'agit   */
  Couche *couche;   /* La couche  laquelle elle appartient */
} LM_variable;

static LM_variable *variables = NULL; /* Table des variables rellement utilises */

/* ~~~~~~~~ Fonctions locales ~~~~~~~~ */

static int init_levenberg   (Infos_ncouches *infos);
static int termine_levenberg(Infos_ncouches *infos);

static int calc_increments  (Infos_ncouches *infos);

/* ~~~~~~~~ Ralisation pratique ~~~~~ */

/* -------- Ralisation effective de la minimisation ----------\
 |   Renvoie 0 si tout va bien, <0 si sortie "anormale"        |
 |                                                             |
 | Suppose que _toutes_ les variables soient de type VAR_LIBRE |
 \------------------------------------------------------------ */

int NC_levenberg(Infos_ncouches *infos)
{
  LM_variable *var;
  unsigned int max_iter, i;

  if ( OBJ_choisit(AJUSTE_GENERAL, FIN_N_TOURS, QUESTION) ) /* On impose un nombre maximal de tours */
  {
    max_iter = prend_valeur(AJUSTE_GENERAL, FIN_MAX_TOURS);
  }
  else
    max_iter = 1 << ( 8 * sizeof(int) - 1); /* Nombre de tours quasi-infini... surtout sous Unix ! */

  if ( init_levenberg(infos) )
    return -1; /* Erreur survenue */

  while (infos->etape < max_iter) /* On doit continuer les itrations */
  {
    double pas, pas_max = 1;
    double nv_erreur;

    calc_increments(infos); /* Calcul des incrments pour chacune des variables */

    /* On en dduit le pas pour chacune des variables */
    do
    {
      for (i = 0, var = variables ; i < infos->nbr_var ; i++, var++)
      {
	double nv_valeur;
	pas = pas_max;
	var->ex_valeur = var->var->contenu.valeur;
	nv_valeur = var->ex_valeur + pas * var->increment;

	while ( (nv_valeur < var->var->min) || (nv_valeur > var->var->max) )
	{
	  pas /= 2;
	  nv_valeur = var->ex_valeur + pas * var->increment;
	}

	var->var->contenu.valeur = nv_valeur;
      }

      calc_exafs_ncouches(premiere_couche, infos->ajuste); /* Nouveau spectre EXAFS associ  ce rsultat */
      nv_erreur = calc_erreur(infos);

      FEN_redessin( ARB_fenetre( arbres + RESULT_N_COUCHES ) ); /* On demande le redessin de cette fentre */
      /* met_svaleur(RESULT_N_COUCHES, RNC_ERREUR, nv_erreur);*/

#if _DEVELOPPE_
      printf("Itration %d : erreur de dpart %g, erreur trouve %g\n", infos->etape, infos->erreur, nv_erreur);
#endif

      attend_evenement(MU_TIMER|MU_KEYBD, /* vnements attendus */
		       0, 0, 0, 0, 0,     /* rectangle 2 : aucun */
		       0,              /* Compteur  1, sinon plantage sur TOS 1.0 */
		       &_rien_, &_rien_, &_rien_, &_rien_, &_rien_, &_rien_);

      printf("Appuyer sur une touche\n");
      getchar();

      /* mettre une attente d'vnement */

      pas_max /= 2;
    } while ( (pas_max > 2e-5) && (nv_erreur > infos->erreur) );

    if ( nv_erreur > infos->erreur )
    {
#if _DEVELOPPE_
      printf("Itration %d : impossible de diminuer l'erreur - ajustement arrt\n", infos->etape);
#endif

      /* On remet les anciennes valeurs, qui donnaient un meilleur rsultat */

      for (i = 0, var = variables ; i < infos->nbr_var ; i++, var++)
	var->var->contenu.valeur = var->ex_valeur;

      /* On recalcule le spectre ajust */
      calc_exafs_ncouches(premiere_couche, infos->ajuste); /* Nouveau spectre EXAFS associ  ce rsultat */

      /* On termine l'ajustement */
      termine_levenberg(infos);
      return -10; /* Impossible de minimiser */
    }

    /* Si on est l, c'est que l'on a russi  minimiser l'erreur (joie !) */
    /* Le spectre ajust est dj calcul, PAS le rsidu, les valeurs ne sont pas mises  jour */

    /* Ajouter les vrifications de sortie ventuelles */

    /* On calcule ce qu'il faut, on raffiche tout */

    infos->erreur = calc_residu(infos); /* Calcul du rsidu et de la nouvelle erreur */
    FEN_redessin( ARB_fenetre( &(arbres[RESULT_N_COUCHES]) ) ); /* On redessine tout */

    (infos->etape)++; /* On passe  l'itration suivante */
    met_valeur(RESULT_N_COUCHES, RNC_ITERATION, (infos->etape));

      /* mettre une attente d'vnement */

  } /* Et c'est reparti pour un tour... */

  termine_levenberg(infos);
}

/* -------- Calcul des incrments selon l'algorithme de L & M modifi --------\
 |   Renvoie 0 si tout s'est bien pass, <0 sinon                             |
 |                                                                            |
 | incrment I = 0.5 * ( dS/dvariable ) / Somme_sur_k ( dmodle/dvariable )  |
 |                                                                            |
 |               Somme_sur_k [ residu(k) * (dmodle / dvariable)(k) ]         |
 |             = ----------------------------------------------------         |
 |                  Somme_sur_k [ ( (dmodle / dvariable)(k) ) ]             |
 |                                                                            |
 | Suppose implicitement que var->x_max > var->x_min et                       |
 |                       que toutes les varibles soient  faire varier        |
 \--------------------------------------------------------------------------- */

static int calc_increments(Infos_ncouches *infos)
{
  register LM_variable *var;
  unsigned int num_var; /* Index de la variable utilise */
  unsigned long i; /* Index sur les k */
  register double *x, *y_residu, *y_modele;
  double khi_ajuste = 0;

  for (num_var = 0, var = variables ; num_var < infos->nbr_var ; num_var++, var++)
  {
    var->den = 0;
    var->num = 0;
  }

  /* Ici, boucle sur les points. On utilise une boucle sur les variables  l'intrieur
                                 pour profiter des calculs de khi_ajuste(k) communs 
				 plusieurs variables, le cas chant.                   */

  for (i = 0, x = infos->ajuste->x, y_modele = infos->ajuste->y, y_residu = infos->residu->y ;
       i < infos->ajuste->nombre_point ;
       i++, x++, y_modele++, y_residu++)
  {
    Couche *couche = NULL;
    double khi_calc = 0;

    for (num_var = 0, var = variables ; num_var < infos->nbr_var ; num_var++, var++)
    {
      double derivee;

      if ( (couche != var->couche) ) /* On change de couche : il faut recalculer le signal EXAFS en ce point */
      {
	couche = var->couche;
	khi_calc = calc_exafs_1point(*x, couche);
      }

      if (couche)
	derivee = calc_derivee(*x, couche, var->type, khi_calc);
      else
	derivee = calc_derivee(*x, NULL, var->type, *y_modele);

      var->num += derivee * *y_residu;
      var->den += derivee * derivee;
    }
  }

  for (num_var = 0, var = variables ; num_var < infos->nbr_var ; num_var++, var++)
  {
    var->increment = var->num / var->den;
  }

  return 0; /* Tout s'est bien pass */
}

/* -------- Initialisation de la minimisation -------- */

static int init_levenberg(Infos_ncouches *infos)
{
  Couche *couche;
  LM_variable *var;

  variables = (LM_variable *) calloc(infos->nbr_var, sizeof(LM_variable) );
  if (variables == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
#ifdef _DEVELOPPE_
    printf("Dans init_levenberg, levenberg.c, rservation de variables\n");
#endif
    return -1;
  }

  /* On initialise les variables */

  var = variables;

  /* 1) Variables globales */

  if ( S0_2.type == VAR_LIBRE )
  {
    var->var    = &S0_2;
    var->type   = VAR_SO2;
    var->couche = NULL;
    var++;
  }

  if ( (deltae_general == VRAI) && (Delta_E.type == VAR_LIBRE) )
  {
    var->var  = &Delta_E;
    var->type = VAR_DELTAE_GENE;
    var->couche = NULL;
    var++;
  }

  if ( (gamma_general == VRAI) && (type_gamma != GENE_LPM_INCLUS) && (Gamma.type == VAR_LIBRE) )
  {
    var->var  = &Gamma;
    var->type = VAR_GAMMA_GENE;
    var->couche = NULL;
    var++;
  }

  /* 2) Variables des couches */

  for (couche = premiere_couche ; couche ; couche = couche->suivante)
  {
    if ( couche->utilisee )
    {
      if ( couche->N.type == VAR_LIBRE )
      {
	var->var  = &(couche->N);
	var->type = VAR_N;
	var->couche = couche;
	var++;
      }
      if ( couche->R.type == VAR_LIBRE )
      {
	var->var  = &(couche->R);
	var->type = VAR_R;
	var->couche = couche;
	var++;
      }
      if ( couche->Debye_Waller.type == VAR_LIBRE )
      {
	var->var  = &(couche->Debye_Waller);
	var->type = VAR_DEBYE;
	var->couche = couche;
	var++;
      }
      if ( couche->Cumul_1.type == VAR_LIBRE )
      {
	var->var  = &(couche->Cumul_1);
	var->type = VAR_CUMUL_1;
	var->couche = couche;
	var++;
      }
      if ( couche->Cumul_2.type == VAR_LIBRE )
      {
	var->var  = &(couche->Cumul_2);
	var->type = VAR_CUMUL_2;
	var->couche = couche;
	var++;
      }
      if ( couche->Cumul_3.type == VAR_LIBRE )
      {
	var->var  = &(couche->Cumul_3);
	var->type = VAR_CUMUL_3;
	var->couche = couche;
	var++;
      }
      if ( couche->Cumul_4.type == VAR_LIBRE )
      {
	var->var  = &(couche->Cumul_4);
	var->type = VAR_CUMUL_4;
	var->couche = couche;
	var++;
      }
      if ( (deltae_general == FAUX) && (couche->Delta_E.type == VAR_LIBRE) )
      {
	var->var  = &(couche->Delta_E);
	var->type = VAR_DELTAE;
	var->couche = couche;
	var++;
      }
      if ( (gamma_general == FAUX) && (type_gamma != GENE_LPM_INCLUS) && (couche->Gamma.type == VAR_LIBRE ) )
      {
	var->var  = &(couche->Gamma);
	var->type = VAR_GAMMA;
	var->couche = couche;
	var++;
      }
    }
  }

  infos->en_cours = VRAI;

  return 0; /* Tout s'est bien pass */
}

/* -------- Fin de la minimalisation ----------------- */

static int termine_levenberg(Infos_ncouches *infos)
{
  calc_residu(infos); /* Calcul du rsidu, pour savoir ce qu'il reste */

  free(variables);
  infos->en_cours = FAUX;

  /* Mettre un message d'alerte : "FIT termin" */

  return 0;
}
