/* Traitement de l'E.X.A.F.S. sous TOS, MagiC, ...
				   Calculs thoriques de divers graphes utiles
   (c) Emmanuel CURIS 1996 - 1998

 25 mars 1998
   * Utilisation de "SYSTEME.H"

 16 mars 1998
   * Inclus messages d'erreurs EXAFS

  5 dcembre 1997
   * Cration de cet historique
   * Corrig erreur dans calcul des phases thoriques : choix thorie/semi-exp pour rflecteur marchait pas
*/

#define _THEORIE_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "SYSTEME.H"

#include "GRAPHES.H"

#include "ERREURS.H"
#include "EXAFS.H"
#include "SCIENCE.H"

#include <LISSAGE.H>

/* ~~~~~~~~~~~~~~~~ Diverses choses pour l'extraction ~~~~~~~~~~~~~~~~~ */

/* ---------------- Absorption atomique (mu_0) ------------------------ */

Graphe *CTH_mu0(void)
{
  int reponse = 0;
  double coef_a, coef_b, e_0; /* Paramtres de calcul */
  double facteur;
  double x_min, x_max, pas, x;   /* Bornes de calcul et tout et tout */
  unsigned long i, nbr_pts;
  Graphe *mu_0;
  register double *zone_x, *zone_y;

  do
  {
    reponse = ext_formdo(TH_CALC_MU0);   /* On appelle l'arbre  */
    if (reponse == THMU0_ANNULE) return NULL; /* On annule le calcul */

    if (reponse == THMU0_ADAPTE) /* Suivant l'lment considr, met les A, B et E0 idaux */
    {
      /* Pas encore fait... */
    }
  } while (reponse != THMU0_OK);

  x_min = prend_fvaleur(TH_CALC_MU0, THMU0_KMIN); /* Bornes de calcul */
  x_max = prend_fvaleur(TH_CALC_MU0, THMU0_KMAX);
  pas   = prend_fvaleur(TH_CALC_MU0, THMU0_PAS ); /* Pas  utiliser   */

  /* On vrifie la validit de ces bornes */

  if (x_min == x_max)
  {
    EXF_ERREUR(BORNES_FAUSSES);
    return NULL;
  }
  if (x_min > x_max) /* On change les deux */
  {
    double x = x_min; x_min = x_max; x_max = x;
  }
  if ( (pas <= 0) || (pas >= x_max - x_min) )
  {
    EXF_ERREUR(PAS_FAUX);
    return NULL;
  }

  nbr_pts = 1 + (int) ( (x_max - x_min) / pas );
  mu_0 = creer_graphe(nbr_pts, GR_MU0);
  if (mu_0 == NULL)
    return NULL;

  /* Mthode de Victoreen ---> calcul en k(x) = 12,39852067 / (e0 + x^2 * 3.81)
     On pose A = C * k(0)^3 - D * k(0)^4
     et absorption en (C * k(x)^3 - D * k(x)^4) / A                       */

  coef_a = prend_fvaleur(TH_CALC_MU0, THMU0_COEF_C1); /* On lit les coefficients de la Victorine */
  coef_b = prend_fvaleur(TH_CALC_MU0, THMU0_COEF_D1);
  e_0    = prend_fvaleur(TH_CALC_MU0, THMU0_SEUIL ); /* nergie de seuil */

  {
    double k, k_3;
    k = 12.39852067 / e_0;
    k_3 = k * k * k;
    facteur =  coef_a * k_3 - coef_b * k * k_3; /* A * k^3 - D * k^4 */

    if (facteur == 0)
    {
      GRF_efface(mu_0, VRAI, VRAI);
      EXF_ERREUR(DIV_ZERO);
      return NULL;
    }
  }

  x = x_min;
  zone_x = mu_0->x; zone_y = mu_0->y;

  for (i = 0 ; i < nbr_pts ; i++)
  {
    double k, k_3, y;

    k = 12.39852067	/ (e_0 + x * x * 3.81); /* On calcule k */
    k_3 = k * k * k;
    y  = coef_a * k_3 - coef_b * k * k_3;

    *zone_x = x;
    *zone_y = y / facteur;
    
    x += pas; zone_x++; zone_y++;
  }

  GRF_bornes(mu_0, ORDONNE, DESORDONNE);

  return mu_0;
}

/* ~~~~~~~~~~~~~~~~ Composantes du signal d'E.X.A.F.S. ~~~~~~~~~~~~~~~~ */

/* ---------------- Dphasage, pour un atome central et un rflecteur - */

Graphe *CTH_phase(void)
{
	Graphe *ph_centre, *ph_reflecteur, *phase;
	PHA_infos inf_phases;
	OBJECT *adresse = arbres[CALC_PHASE].adresse;
	int reponse;
	double k_min, k_max, pas, y;
	unsigned long nbr_pts, index;
	boolean seuil_k;
	double *b,*c,*d;
	register double *x_phase, *y_phase;

	arbres[CALC_PHASE].pointeur = &inf_phases;

	inf_phases.centre = GRF_par_type(GR_EXAFSPHA, FAUX);
	inf_phases.reflecteur = GRF_par_type(GR_EXAFSPHA, FAUX);

	/* On rgle en fonction de ce que l'on vient de recevoir */

	if (inf_phases.centre == NULL) /* Pas de liste de fichiers de phase -> calcul seulement thorique... */
	{
		OB_STATE(adresse,PHA_CTR_EXP) |= DISABLED;     /* Calcul thorique seul possible */
		OB_STATE(adresse,PHA_CTR_EXP) &= ~SELECTED;
		OB_STATE(adresse,PHA_CTR_THEORIE) |= SELECTED;
	}
	if ( OB_STATE(adresse,PHA_CTR_THEORIE) & SELECTED ) /* Calcul thorique */
	{
		OB_FLAGS(adresse,PHA_CTR_1 - 1) |= HIDETREE; /* Slecteur invisible */
		OB_FLAGS(adresse,PHA_CTR_AVANT) |= HIDETREE;
		OB_FLAGS(adresse,PHA_CTR_FOND)  |= HIDETREE;
		OB_FLAGS(adresse,PHA_CTR_APRES) |= HIDETREE;

		OB_FLAGS(adresse,PHA_CTR_NATURE) &= ~HIDETREE; /* Choix de l'lment possible */
		arbres[CALC_PHASE].premier_edit = PHA_CTR_NATURE;
	}
	else
	{
		OB_FLAGS(adresse,PHA_CTR_1 - 1) &= ~HIDETREE; /* Slecteur visible */
		OB_FLAGS(adresse,PHA_CTR_AVANT) &= ~HIDETREE;
		OB_FLAGS(adresse,PHA_CTR_FOND)  &= ~HIDETREE;
		OB_FLAGS(adresse,PHA_CTR_APRES) &= ~HIDETREE;

		OB_FLAGS(adresse,PHA_CTR_NATURE) |= HIDETREE; /* Choix de l'lment invisible */
		arbres[CALC_PHASE].premier_edit = CALC_PHA_DEBUT;
	}

	if (inf_phases.reflecteur == NULL) /* Pas de liste de fichiers de phase -> calcul seulement thorique... */
	{
		OB_STATE(adresse,PHA_EXP) |= DISABLED;     /* Calcul thorique seul possible */
		OB_STATE(adresse,PHA_EXP) &= ~SELECTED;
		OB_STATE(adresse,PHA_THEORIE) |= SELECTED;
	}
	if ( OB_STATE(adresse,PHA_THEORIE) & SELECTED ) /* Calcul thorique pour le rflecteur */
	{
		OB_FLAGS(adresse,PHA_EXP_1 - 1) |= HIDETREE; /* Slecteur invisible */
		OB_FLAGS(adresse,PHA_EXP_AVANT) |= HIDETREE;
		OB_FLAGS(adresse,PHA_EXP_FOND)  |= HIDETREE;
		OB_FLAGS(adresse,PHA_EXP_APRES) |= HIDETREE;

		OB_FLAGS(adresse,PHA_EXP_NATURE) &= ~HIDETREE; /* Choix de l'lment possible */
		if ( ( OB_STATE(adresse, PHA_CTR_THEORIE) & SELECTED ) == FAUX )
			arbres[CALC_PHASE].premier_edit = PHA_EXP_NATURE;
	}
	else
	{
		OB_FLAGS(adresse,PHA_EXP_1 - 1) &= ~HIDETREE; /* Slecteur visible */
		OB_FLAGS(adresse,PHA_EXP_AVANT) &= ~HIDETREE;
		OB_FLAGS(adresse,PHA_EXP_FOND)  &= ~HIDETREE;
		OB_FLAGS(adresse,PHA_EXP_APRES) &= ~HIDETREE;

		OB_FLAGS(adresse,PHA_EXP_NATURE) |= HIDETREE; /* Choix de l'lment invisible */
		if ( ( OB_STATE(adresse, PHA_CTR_THEORIE) & SELECTED ) == FAUX )
			arbres[CALC_PHASE].premier_edit = CALC_PHA_DEBUT;
	}

	/* On affiche les listes ventuelles de graphes convenant */
	GRF_affiche(inf_phases.centre,     CALC_PHASE, PHA_CTR_1, PHA_CTR_4, PHA_CTR_AVANT, PHA_CTR_APRES, PHA_CTR_FOND, PHA_CTR_BOUTON);
	GRF_affiche(inf_phases.reflecteur, CALC_PHASE, PHA_EXP_1, PHA_EXP_4, PHA_EXP_AVANT, PHA_EXP_APRES, PHA_EXP_FOND, PHA_EXP_BOUTON);

	reponse = ext_formdo(CALC_PHASE);
	if (reponse == PHA_ANNULER)
	{
		free(inf_phases.centre);
		free(inf_phases.reflecteur);
		return NULL;
	}

	k_min = prend_fvaleur(CALC_PHASE, CALC_PHA_DEBUT);
	k_max = prend_fvaleur(CALC_PHASE, CALC_PHA_FIN);
	pas   = prend_fvaleur(CALC_PHASE, CALC_PHA_PAS);

	if ( (k_min == k_max) || (pas <= 0) )
	{
		form_alert(1,"[2][ Ces paramtres (bornes ou pas) | conduisent  ce que rien ne | soit calcul ][ Abandon ]");
		free(inf_phases.centre);
		free(inf_phases.reflecteur);
		return NULL;		
	}

	if (k_min > k_max) /* On inverse les deux... */
	{
		double k = k_max; k_max = k_min; k_min = k;
	}

	seuil_k  =  OB_STATE(adresse, PHA_K ) & SELECTED;
	seuil_k |= (OB_STATE(adresse, PHA_L1) & SELECTED); /* donc VRAI -> seuil K ou L1 -> - M_PI */

	nbr_pts = 1 + (unsigned long) ( (k_max - k_min) / pas ); /* Nombre de points dans le graphe */
	phase = GRF_creer(nbr_pts, GR_EXAFSPHA, GR_NORMAL, "Phase thorique", "Nombre d'onde k", "Dphasage", UNIT_1_ANG, UNIT_ARB);
	if (phase == NULL) /* Plus rien  faire */
	{
		free(inf_phases.centre);
		free(inf_phases.reflecteur);
		return NULL;				
	}

	x_phase = phase->x; y_phase = phase->y;
	for (index = 0 ; index < nbr_pts ; index++, x_phase++, y_phase++)
	{
		*x_phase = k_min; k_min += pas; /* On passe au point suivant */
		if (seuil_k)
			*y_phase = -M_PI;
		else
			*y_phase = 0;
	}

	/* On calcule la phase de l'atome central */

	if ( OB_STATE(adresse, PHA_CTR_THEORIE) & SELECTED ) /* On veut un calcul thorique, atome central */
	{
	}
	else
	{
		ph_centre = (inf_phases.centre)->graphes[(inf_phases.centre)->num_select];

		/* On calcule la SPLINE pour lisser le spectre exprimental de rfrence */
		y = SPL_tt_prep(ph_centre, &b, &c, &d);
		if (y == -1) /* Erreur en calculant la spline */
		{
			free(inf_phases.centre);
			free(inf_phases.reflecteur);
			free(b); free(c); free(d);
			return NULL;
		}

		for (index = 0 ; index < nbr_pts ; index++)
		{
			y = SPL_tt_calc(ph_centre, (phase->x)[index], b, c, d);
			(phase->y)[index] += y;
		}

		free(b); free(c); free(d); /* Plus besoin de a */
	}

	/* On calcule la phase de l'atome rflecteur */

	if ( OB_STATE(adresse, PHA_THEORIE) & SELECTED ) /* On veut un calcul thorique pour le rflecteur */
	{
	}
	else
	{
	  ph_reflecteur = (inf_phases.reflecteur)->graphes[(inf_phases.reflecteur)->num_select];

	  /* On calcule la SPLINE pour lisser le spectre exprimental de rfrence */
	  y = SPL_tt_prep(ph_reflecteur, &b, &c, &d);
	  if (y == -1) /* Erreur en calculant la spline */
	  {
	    free(inf_phases.centre);
	    free(inf_phases.reflecteur);
	    free(b); free(c); free(d);
	    return NULL;
	  }

	  for (index = 0 ; index < nbr_pts ; index++)
	  {
	    y = SPL_tt_calc(ph_reflecteur, (phase->x)[index], b, c, d);
	    (phase->y)[index] += y;
	  }

	  free(b); free(c); free(d); /* Plus besoin de a */
	}

	/* Calcul des bornes */

	GRF_bornes(phase, ORDONNE, DESORDONNE); /* Calcul en X inutile */

	/* On a fini */

	free(inf_phases.centre);
	free(inf_phases.reflecteur);

	return phase;
}
