/* Traitement de l'E.X.A.F.S. sur Atari
				Tout ce qui concerne le X.A.N.E.S.
   (c) 1996, 1997 - Emmanuel CURIS             

  2 avril 1998
    * Remplac tous les OB_STATE par OBJ_...

  1er avril 1998
    * Normalisation : normalise sur la valeur du polynme -> vraiment reproductible...
                      sauvegarde l'historique
    * Remplac tous les vdi_handle par _PRG_station

  16 & 17, 19, 25 & 26 mars 1998
    * Utilisation de _PRG_station plutt que vdi_handle
    * Remplac effacer_graphe par GRF_efface()
  	* Utilisation de "systeme.h"
    * Historique sauv quand on soustrait la contribution de la matrice
	  * Paramtres de soustraction de mu_matrice mis en place dans l'arbre gnral
	  * Inclus "extrait.h" : extraction du signal
    * Inclus les messages d'erreur exafs "erreurs.h"

   1 & 2 dcembre 1997
    * appel  DER_seconde(), quand on veut tout faire en XANES, fonctionne
    * lissage par spline corrig (mme erreur que dans lissage.c - merci Bruno)
    * Supprim derive() et der_seconde(), calcul de la drive report dans grf_math.c
	  * Quelques remplacements de effacer_graphe() par GRF_effacer()

	30 novembre 1997
	  * Quelques remplacements de effacer_graphe() par GRF_effacer()
	  * Cration de cet historique
	  * Ajout inclusion de science.h
*/

#define _XANES_ 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SYSTEME.H>

#include <EXAFS.H>
#include <GRAPHES.H>

#include <ERREURS.H>
#include <EXTRAIT.H>
#include <SCIENCE.H>

/* ************ Variables spcifiques de ce module *************************************** */

/* --------------- Traitement du XANES : procdures spcifiques -------------------------- *
 *******************************************************************************************/

/* --------------- Intgration du prseuil ----------------------------------------------- */

double INTG_min, INTG_max; /* Bornes d'intgration */

/* ~~~~~~~~~~~~~~~ Calcul proprement dit ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ *
 *   Suppose que le graphe est ordonn                                                     *
 *   Utilise la mthode des trapzes                                                       */

double XAN_intg_calcule(Graphe *graphe)
{
	double vx_x,vx_y;
	double somme = 0;
	register double *zone_x,*zone_y;

	if (INTG_min == INTG_max) /* Pas d'intgration  faire... */
		return 0;

	if (INTG_min > INTG_max) /* On inverse les bornes */
	{
		vx_x = INTG_max;
		INTG_max = INTG_min;
		INTG_min = vx_x;
	}

	/* On se place au dbut de l'intgration */
	zone_x = graphe->x;
	zone_y = graphe->y;
	while (*zone_x < INTG_min)
	{
		 zone_x++;
		 zone_y++;
	}

	/* On commence l'intgration */
	vx_x = *zone_x;
	vx_y = *zone_y;
	while (*zone_x < INTG_max)
	{
		zone_x++; zone_y++;
		somme += (*zone_x - vx_x) * (*zone_y + vx_y);
		vx_x = *zone_x;
		vx_y = *zone_y;
	}

	return somme/2.0;
}

/* ~~~~~~~~~~~~~~~ Appel de l'arbre ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

boolean XAN_integre(Graphe *xanes)
{
	Fenetre *fen_integre;
	double resultat;

	if (xanes == NULL) return FAUX; /* Calcul n'a pas de raison d'tre */

	fen_integre = creer_mixte_fenetre(NAME|MOVER|CLOSER|FULLER|SIZER|INFO,INTG_XANES,HAUT,
	                                  RDS_intg, (boolean (*)()) NULL, CLC_intg,
	                                  (boolean (*)()) NULL, RCT_pointeur, (boolean (*)()) NULL );
	if (fen_integre == NULL) return FAUX; /* Mme chose */

	fen_integre->drapeaux.redessin = VRAI; /* Redessin si changement de taille */
	fen_integre->adresse =  xanes;  /* Adresse du graphe associ */
	fen_integre->pointeur = xanes;
	INTG_min = xanes->x_min;
	INTG_max = xanes->x_max;
	met_fvaleur(INTG_XANES, INTG_XMIN, xanes->x_min);
	met_fvaleur(INTG_XANES, INTG_XMAX, xanes->x_max);

	resultat = XAN_intg_calcule(xanes); /* On calcule le rsultat de l'intgration */
	met_fvaleur(INTG_XANES, INTG_RESULTAT, resultat);

	return VRAI;
}

/* ~~~~~~~~~~~~~~~ Redessin de cette fentre ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

void RDS_intg(Fenetre *fenetre, int x, int y, int larg, int haut)
{
	OBJECT *adresse = arbres[INTG_XANES].adresse;
	Graphe *graphe;
	double x_echelle,y_echelle;
	int x_trace,y_trace,haut_trace;
	double *zone_x,*zone_y;
	double x_depart,x_fin;
	unsigned long index, compteur = 0;

	x_trace = fenetre->w_x;
	y_trace = fenetre->w_y + OB_HEIGHT(adresse,ROOT) + 1;
	haut_trace = fenetre->w_haut - OB_HEIGHT(adresse,ROOT) - 2;

	graphe = (Graphe *) fenetre->adresse; /* Adresse du graphe  redessiner */

	vswr_mode(_PRG_station->handle, MD_REMPLACE);    /* Mode recouvrir                 */

	vsf_color(_PRG_station->handle, BLANC);
	points[0] = x;
	points[1] = y;
	points[2] = x + larg - 1;
	points[3] = y + haut - 1;
	v_bar(_PRG_station->handle,points);     /* Le fond blanc */

	x_echelle = (double) (fenetre->w_larg) / (graphe->x_max - graphe->x_min);
	y_echelle = (double) (haut_trace) / (graphe->y_max - graphe->y_min);
	x_depart = graphe->x_min + (x - x_trace) / x_echelle;
	x_fin = graphe->x_min + (x + larg - x_trace) / x_echelle;

	/* On trace la courbe exprimentale - en noir */
	vsl_color(_PRG_station->handle, NOIR);

	zone_x = graphe->x; zone_y = graphe->y;

	for (index = 0 ; index < graphe->nombre_point ; index ++)
	{
		if ( ( (*zone_x) <= x_fin) && ( (*zone_x) >= x_depart ) )
		{
			points[compteur]   = x_trace + x_echelle * ((*zone_x) - graphe->x_min);
			points[compteur+1] = y_trace + y_echelle * (graphe->y_max - (*zone_y));
			compteur += 2;
		}
		zone_x++;
		zone_y++;

		if (compteur >= 256) /* Tableau de points complet */
		{
			v_pline(_PRG_station->handle, 128,points);
			points[0] = points[254];
			points[1] = points[255];
			compteur = 2;
		}
	}
	v_pline(_PRG_station->handle, (int) (compteur+1)/2,points);

	/* On place les deux curseurs indiquant les limites */

	if ( ( INTG_min >= x_depart) && (INTG_min <= x_fin) )
	{
		vsl_color(_PRG_station->handle, ROUGE); /* Rouge pour la position minimale */
		points[0] = x_trace + x_echelle * ( INTG_min - graphe->x_min );
		points[1] = y_trace;
		points[2] = points[0];
		points[3] = y_trace + haut_trace;
		v_pline(_PRG_station->handle, 2, points);
	}

	if ( ( INTG_max >= x_depart) && (INTG_max <= x_fin) )
	{
		vsl_color(_PRG_station->handle, VERT); /* Vert pour la position minimale */
		points[0] = x_trace + x_echelle * ( INTG_max - graphe->x_min );
		points[1] = y_trace;
		points[2] = points[0];
		points[3] = y_trace + haut_trace;
		v_pline(_PRG_station->handle,2,points);
	}
}

/* --------------- Gestion du clic souris dans cette fentre ----------------------------- *
 * Si clic : c'est qu'elle est en premier plan => on peut dessiner tranquille              */

boolean CLC_intg(Fenetre *fenetre, int bouton, int n_clic, int etat, int x_souris, int y_souris)
{
	Graphe *graphe;
	double x_clic, x_echelle;
	double resultat;
	int x;

	graphe = (Graphe *) (fenetre->adresse);

	x_echelle = (double) (fenetre->w_larg) / (graphe->x_max - graphe->x_min);
	x_clic = graphe->x_min + (x_souris - fenetre->w_x) / x_echelle;

	graf_mouse(M_OFF,NULL);
	vswr_mode(_PRG_station->handle, 3); /* Mode inverser */
	points[0] = fenetre->w_x;                   points[1] = fenetre->w_y;
	points[2] = fenetre->w_x + fenetre->w_larg; points[3] = fenetre->w_y + fenetre->w_haut;
	vs_clip(_PRG_station->handle, 1,points);

	points[1] = fenetre->w_y + OB_HEIGHT(fenetre->arbre->adresse,ROOT);
	points[3] = fenetre->w_y + fenetre->w_haut;
	if (bouton == 1) /* Clic du bouton gauche */
	{
		vsl_color(_PRG_station->handle, ROUGE);
		x = fenetre->w_x + x_echelle * (INTG_min - graphe->x_min);
		points[0] = x; points[2] = x;
		v_pline(_PRG_station->handle,2,points); /* On efface l'ancienne ligne */

		INTG_min = x_clic;
		met_fvaleur(INTG_XANES,INTG_XMIN,x_clic);

		x = fenetre->w_x + x_echelle * (INTG_min - graphe->x_min);
		points[0] = x; points[2] = x;
		v_pline(_PRG_station->handle, 2, points); /* On met la nouvelle */

		resultat = XAN_intg_calcule(graphe);
		met_fvaleur(INTG_XANES, INTG_RESULTAT, resultat);
	}
	else
	{
		vsl_color(_PRG_station->handle, VERT);
		x = (INTG_max - graphe->x_min) * x_echelle + fenetre->w_x;
		points[0] = x; points[2] = x;
		v_pline(_PRG_station->handle,2,points); /* On efface l'ancienne ligne */

		INTG_max = x_clic;
		met_fvaleur(INTG_XANES,INTG_XMAX,x_clic);

		x = (INTG_max - graphe->x_min) * x_echelle + fenetre->w_x;
		points[0] = x; points[2] = x;
		v_pline(_PRG_station->handle,2,points); /* On met la nouvelle */

		resultat = XAN_intg_calcule(graphe);
		met_fvaleur(INTG_XANES, INTG_RESULTAT, resultat);
	}

	vswr_mode(_PRG_station->handle, 1); /* Mode recouvrir */
	graf_mouse(M_ON,NULL);

	evnt_button(1,3,0,&x,&y_souris,&etat,&n_clic); /* On attend que le bouton soit relch */

	return VRAI; /* vnement trait */
}

/* --------------- Traitement du XANES : procdures plus gnrales ----------------------- *
 *******************************************************************************************/

/* ------------ Recherche de E0 : maximum de la drive premire, en attendant mieux... -- */

double cherche_e0(Graphe *graphe, boolean affiche)
{
	register double *zone_x,*zone_y;
	double delta_x,derivee,derivee_max=0,*pos_x = NULL;
	unsigned long index;
	
	if (graphe->type == GR_DERIVEE) /* Le graphe est la drive => rapide */
	{
		zone_y = graphe->y;
		while ( *(zone_y) < graphe->y_max ) /* On n'est pas au maximum */
			zone_y++;
		
		pos_x = graphe->x + (zone_y - graphe->y); /* X correspondant  ce Y */
	}
	else
	{
		zone_x = graphe->x;
		zone_y = graphe->y;
		for (index = 0 ; index < graphe->nombre_point-1 ; index++)
		{
			delta_x = *(zone_x + 1) - *(zone_x);
			if (delta_x == 0) /* Drive infinie */
				return *(zone_x);
		
			derivee  = *(zone_y + 1) - *(zone_y);
			derivee /= delta_x;
			if (derivee > derivee_max)
			{
				derivee_max = derivee;
				pos_x = zone_x; /* Point juste avant la drive maximale... */
			}
			zone_x++;
			zone_y++;
		}
	}

	if (pos_x == NULL)
	{
		form_alert(1,"[2][ La d"e_AIGU"termination automatique de E0 | n'a pas pu "e_CIRC"tre effectu"e_AIGU"e ][ Tant pis ]");
		return 0;
	}
	
	if (affiche == VRAI) /* On affiche le rsultat */
	{
		met_fvaleur(AFFICHE_VALEUR, AFF_VALEUR, *pos_x);
		ARB_message(AFFICHE_VALEUR, 1); /* Message "permanent" */
	}

	return *(pos_x);
}
