/* Traitement de l'E.X.A.F.S. sur Atari
   Tout ce qui concerne le X.A.N.E.S.
   (c) 1996-1998, Emmanuel CURIS - Cration : 22 mai 1998

   Soustraction de la contribution de l'chantillon (mu_matrice)
   Rcupr de ../XANES.C et de ../EXF_GEM.C ( GER_matrice() )
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SYSTEME.H"

#include "EXAFS.H"
#include "GRAPHES.H"

#include "ERREURS.H"
#include "EXTRAIT.H" /* Pour MU1_ouvre() */
#include "LISSAGE.H" /* Pour lisse_graphe et les splines */
#include "SCIENCE.H"

#include "XANES.H"

/* ************ Variables spcifiques de ce module *************************************** */

static int num_methode = MUM_VICTORINE_1; /* Mthode par dfaut */

#ifdef _FRANCAIS_

static const char *Meth_mu_matrice[] = 
{
  "Droite",
  "Victorine (1)",
  "Victorine (2)",
  "Polyn"o_CIRC"me",
  "Th"e_AIGU"orique"
};

#endif

#ifdef _ENGLISH_

static const char *Meth_mu_matrice[] = 
{
  "Linear",
  "Victorine (1)",
  "Victorine (2)",
  "Polynomial",
  "Theoretic"
};

#endif

/* ************ Fonctions spcifiques  ce module **************************************** */

static boolean CLC_matrice(Fenetre *fenetre, int bouton, int n_clic, int x, int y, int etat);
static boolean MSG_matrice(Fenetre *fenetre);
static void    RDS_matrice(Fenetre *fenetre, int x, int y, int larg, int haut);

/* ************ Dfinitions effectives des oprations  effectuer ************************ */

/* ~~~~~~~~~~~~ Partie mathmatique : on effectue les calculs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ------------- Calcul effectif de la fonction  soustraire ------------------------------\
 |   Suppose que le graphe est tri selon les X croissants                                 |
 \---------------------------------------------------------------------------------------- */

void MAT_calc(Graphe *graphe, MAT_infos *infos)
{
  register double *zone_x, *zone_y;
  double somme_xy[5] = {0, 0, 0, 0, 0},
  somme_x[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
  double delta;

  zone_x = graphe->x; zone_y = graphe->y;

  if ( infos->mode == DEUX_POINTS ) /* On vrifie que les deux points sont senss */
  {
    if (infos->e_1p > infos->e_1) /* Par convention, graphe->x_min < infos->e_1p < infos->e_1 < graphe->x_max */
    {
      double x = infos->e_1 ; infos->e_1 = infos->e_1p ; infos->e_1p = x;
    }

    if ( infos->e_1p < graphe->x_min )
      infos->e_1p = graphe->x_min;
    
    if ( infos->e_1p == infos->e_1 )
    {
      XAN_ERREUR(XAN_E1_EGAL_E1P);
      return;
    }
  }

  if ( infos->e_1 <= graphe->x_min ) /* Ca n'a pas de sens : aucune approximation possible... */
  {
    XAN_ERREUR(XAN_E1_TROPPETIT);
    return;
  }

  if (infos->type == MUM_DROITE) /* Approximation par une droite */
  {
    infos->coefs[2] = 0; infos->coefs[3] = 0; infos->coefs[4] = 0;

    if (infos->mode == DEUX_POINTS)
    {
      for (zone_x = graphe->x ; *zone_x < infos->e_1p ; zone_x++, zone_y++);
      /* On passe les premiers points... */
    }
    else
      zone_x = graphe->x;

    for ( ; *(zone_x) <= infos->e_1 ; zone_x++, zone_y++)
    {
      double x = *(zone_x);

      somme_x[0]++;                 /* Un point de plus */
      somme_x[1]  += x;             /* Somme des x   */
      somme_x[2]  += x * x;         /* Somme des x^2 */
      somme_xy[0] += (*zone_y);     /* Somme des y   */
      somme_xy[1] += (*zone_y) * x; /* Somme des x*y */
    }
			
    delta = somme_x[0] * somme_x[2] - somme_x[1] * somme_x[1];
    if (delta == 0)
    {
      EXF_ERREUR(DIV_ZERO);
      infos->coefs[0] = 0; infos->coefs[1] = 0;
    }
    else
    {
      infos->coefs[0] = (somme_x[2] * somme_xy[0] - somme_x[1] * somme_xy[1]) / delta;
      infos->coefs[1] = (somme_x[0] * somme_xy[1] - somme_x[1] * somme_xy[0]) / delta;
    }

    return;
  }
	
  if (infos->type == MUM_VICTORINE_1)
  {
    double x_cube;

    infos->coefs[2] = 0; infos->coefs[3] = 0; infos->coefs[4] = 0;

    if (infos->mode == DEUX_POINTS)
    {
      for (zone_x = graphe->x ; *zone_x < infos->e_1p ; zone_x++, zone_y++);
      /* On passe les premiers points... suppose le graphe tri */
    }
    else
      zone_x = graphe->x;

    for ( ; *(zone_x) <= infos->e_1 ; zone_x++, zone_y++)
    {
      double x = *(zone_x);

      if (x == 0)
      {
	EXF_ERREUR(DIV_ZERO);
	infos->coefs[0] = 0; infos->coefs[1] = 0;
	return;
      }
      x_cube = 1 / (x * x * x);
      somme_x[0]++; /* Un point en plus */
      somme_x[3]  += x_cube;
      somme_x[6]  += x_cube * x_cube;
      somme_xy[0] += *(zone_y);
      somme_xy[3] += *(zone_y) * x_cube;
    }
			
    delta = somme_x[0] * somme_x[6] - somme_x[3] * somme_x[3];
    if (delta == 0)
    {
      EXF_ERREUR(DIV_ZERO);
      infos->coefs[0] = 0; infos->coefs[1] = 0;
    }
    else
    {
      infos->coefs[0] = (somme_x[6] * somme_xy[0] - somme_x[3] * somme_xy[3]) / delta;
      infos->coefs[1] = (somme_x[0] * somme_xy[3] - somme_x[3] * somme_xy[0]) / delta;
    }

    return;
  }

  if ( infos->type == MUM_VICTORINE_2 )
  {
    double x_cube, x_quatre;
    double inter[4];

    infos->coefs[3] = 0; infos->coefs[4] = 0;

    if (infos->mode == DEUX_POINTS)
    {
      for (zone_x = graphe->x ; *zone_x < infos->e_1p ; zone_x++, zone_y++);
      /* On passe les premiers points... suppose le graphe tri */
    }
    else
      zone_x = graphe->x;

    for ( ; *(zone_x) <= infos->e_1 ; zone_x++, zone_y++)
    {
      double x = *(zone_x);

      if (x == 0)
      {
	EXF_ERREUR(DIV_ZERO);
	infos->coefs[0] = 0; infos->coefs[1] = 0;
	return;
      }
      x_cube = 1/ (x * x * x); /* x^-3 */
      x_quatre = x_cube / x;   /* x^-4 */
      somme_x[0]++;
      somme_x[3] += x_cube;
      somme_x[4] += x_quatre;
      somme_x[6] += x_cube * x_cube;
      somme_x[7] += x_cube * x_quatre;
      somme_x[8] += x_quatre * x_quatre;
      somme_xy[0] += *(zone_y);
      somme_xy[3] += *(zone_y) * x_cube;
      somme_xy[4] += *(zone_y) * x_quatre;
    }
    
    inter[0] = somme_x[8] * somme_x[6] - somme_x[7] * somme_x[7];
    inter[1] = somme_x[4] * somme_x[6] - somme_x[3] * somme_x[7];
    inter[2] = somme_x[4] * somme_x[7] - somme_x[3] * somme_x[8];
    inter[3] = somme_x[3] * somme_x[4] - somme_x[0] * somme_x[7];
    
    delta  = somme_x[0] * inter[0];
    delta -= somme_x[4] * inter[1];
    delta += somme_x[3] * inter[2];

    if (delta == 0)
    {
      EXF_ERREUR(DIV_ZERO);
      infos->coefs[0] = 0; infos->coefs[1] = 0; infos->coefs[2] = 0;
    }
    else
    {
      infos->coefs[0]  = somme_xy[0] * inter[0];
      infos->coefs[0] += somme_xy[3] * inter[2];
      infos->coefs[0] -= somme_xy[4] * inter[1];
      infos->coefs[0] /= delta;

      infos->coefs[2]  = - somme_xy[0] * inter[1];
      infos->coefs[2] += somme_xy[3] * inter[3];
      infos->coefs[2] += somme_xy[4] * ( somme_x[0] * somme_x[6] - somme_x[3] * somme_x[3] );
      infos->coefs[2] /= delta;
			
      infos->coefs[1]  = somme_xy[0] * inter[2];
      infos->coefs[1] += somme_xy[3] * ( somme_x[0] * somme_x[8] - somme_x[4] * somme_x[4] );
      infos->coefs[1] += somme_xy[4] * inter[3];
      infos->coefs[1] /= delta;
    }
    
    return;	
  }

  if ( infos->type == MUM_POLYNOME ) /* On soustrait un polynme */
  {
    double intervalle[2];

    if (infos->mode == DEUX_POINTS)
      intervalle[0] = infos->e_1p;
    else
      intervalle[0] = graphe->x_min;
    intervalle[1] = infos->e_1;

    moindre2( graphe, infos->degre, 1, intervalle, infos->coefs );

    return;
  }
}

/* ------------- Soustraction effective de la contribution de la matrice ------------------\
 |   Ne renvoie rien
 |   Suppose que les coefficients de la fonction ont dj t dfinis                      |
 |           que les X sont dj installs dans traite et sont classs par ordre croissant |
 |
 | brut : le spectre brut
 | traite : le spectre contenant (brut - mu de la matrice)
 | infos  : les paramtres de calcul de la chose
 \---------------------------------------------------------------------------------------- */

void MAT_soustrait(Graphe *brut, Graphe *traite, MAT_infos *infos)
{
  unsigned long index;
  double y;
  register double *zone_x, *zone_y;

  if (brut   == NULL) return;
  if (traite == NULL) return;
  if (infos  == NULL) return;

  zone_x = traite->x;
  zone_y = traite->y;

  if ( (infos->type == MUM_DROITE) /* On utilise une droite */
    || ( (infos->type == MUM_POLYNOME) && (infos->degre == 1) ) )
  {
    y = infos->coefs[0] + infos->coefs[1] * *zone_x;
    *zone_y = *(brut->y) - y;

    traite->y_min = *zone_y; traite->y_max = *zone_y;
    zone_x++; zone_y++;

    for (index = 1 ; index < traite->nombre_point ; index++)
    {
      y = infos->coefs[0] + infos->coefs[1] * *zone_x;
      *zone_y = (brut->y)[index] - y;
      
      if (*zone_y < traite->y_min) traite->y_min = *zone_y; /* Les bornes */
      if (*zone_y > traite->y_max) traite->y_max = *zone_y;

      zone_x++; zone_y++; /* On passe au point suivant */
    }

    return;
  }

  if (infos->type == MUM_VICTORINE_1) /* Victorine de type A + B * X^3 */
  {
    double x_cube;

    x_cube = *zone_x * *zone_x * *zone_x;
    if (x_cube == 0)
    {
      EXF_ERREUR(DIV_ZERO);
      return;
    }

    y = infos->coefs[0] + infos->coefs[1] / x_cube;
    *zone_y = *(brut->y) - y;
    traite->y_min = *zone_y; traite->y_max = *zone_y;
    zone_x++; zone_y++;

    for (index = 1 ; index < traite->nombre_point ; index++)
    {
      x_cube = *zone_x * *zone_x * *zone_x;
      if (x_cube == 0)
      {
	EXF_ERREUR(DIV_ZERO);
	return;
      }

      y = infos->coefs[0] + infos->coefs[1] / x_cube;
      *zone_y = (brut->y)[index] - y;

      if (*zone_y < traite->y_min) traite->y_min = *zone_y; /* Les bornes */
      if (*zone_y > traite->y_max) traite->y_max = *zone_y;

      zone_x++; zone_y++; /* On passe au point suivant */
    }

    return;
  }

  if (infos->type == MUM_VICTORINE_2) /* Victorine de type A + B * X^3 */
  {
    double x_cube;

    x_cube = *zone_x * *zone_x * *zone_x;
    if (x_cube == 0)
    {
      EXF_ERREUR(DIV_ZERO);
      return;
    }

    y = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * *zone_x);
    *zone_y = *(brut->y) - y;
    traite->y_min = *zone_y; traite->y_max = *zone_y;
    zone_x++; zone_y++;

    for (index = 1 ; index < traite->nombre_point ; index++)
    {
      x_cube = *zone_x * *zone_x * *zone_x;
      if (x_cube == 0)
      {
	EXF_ERREUR(DIV_ZERO);
	return;
      }

      y = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * *zone_x);
      *zone_y = (brut->y)[index] - y;

      if (*zone_y < traite->y_min) traite->y_min = *zone_y; /* Les bornes */
      if (*zone_y > traite->y_max) traite->y_max = *zone_y;

      zone_x++; zone_y++; /* On passe au point suivant */
    }

    return;
  }

  if (infos->type == MUM_POLYNOME) /* On est sr que le degr est suprieur  1... */
  {
    register double *x = brut->x, *y = brut->y, *y_ote = traite->y;
    unsigned long i;

    for (i = 0 ; i < brut->nombre_point ; i++, x++, y++, y_ote++)
    {
      double y_poly = 0;
      int j;

      for (j = infos->degre ; j > 0 ; j--)
      {
	y_poly += infos->coefs[j];
	y_poly *= *x;
      }
      y_poly += infos->coefs[0];

      *y_ote = *y - y_poly;
    }

    GRF_bornes(traite, ORDONNE, DESORDONNE);
    return;
  }
}


/* ~~~~~~~~~~~~ Partie utilisateur  : on gre l'interface ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* ------------- Ouverture de la fentre d'limination de la matrice --------------------- */

void MAT_ouvre(Graphe *brut)
{
  Fenetre *fenetre;
  MAT_infos *infos = NULL;
  double mu;
  register double *zone_x, *zone_y;
  char titre[300];

  if (brut == NULL) return; /* Vraiment rien  faire */
  
  if ( arbres[ELIM_MUMATRICE].ouvert == VRAI ) /* Arbre dj ouvert */
  {
    Graphe *vieux; /* Ancien spectre travaill dans ce module */
    int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

    fenetre = ARB_fenetre( &(arbres[ELIM_MUMATRICE]) );
    vieux = (Graphe *) fenetre->pointeur;
    if ( (reponse == 1) || (vieux == brut) ) /* On active seulement */
    {
      FEN_premierplan(fenetre);
      return; /* Rien d'autre  faire */
    }
    else if (reponse == 2) /* On rappelle, pour le nouveau graphe */
    {
      if ( brut->drapeaux.protege )
      {
	EXF_ERREUR(GRF_PROTEGE);
	return;
      }

      infos = (MAT_infos *) fenetre->adresse;
      vieux->drapeaux.protege = FAUX; /* On a fini de travailler avec */
      GRF_efface(infos->extrait, VRAI, VRAI); /* On efface le graphe associ */

      FEN_premierplan(fenetre);
    }
    else /* On annule l'appel */
      return;
  }
  else /* Arbre non ouvert : on l'ouvre */
  {
    if ( brut->drapeaux.protege )
    {
      EXF_ERREUR(GRF_PROTEGE);
      return;
    }

    fenetre = creer_mixte_fenetre(NAME|CLOSER|FULLER|INFO|SIZER|MOVER, ELIM_MUMATRICE, HAUT,
				  RDS_matrice, ( boolean (*)(Fenetre *, unsigned int, int) ) NULL, CLC_matrice, 
				  MSG_matrice, RCT_pointeur, ( boolean (*)(Fenetre *) ) NULL);
    if (fenetre == NULL) return; /* Rien  faire... */

#ifdef _FRANCAIS_
    FEN_titre(fenetre, E_AIGU"limination de l'absorption des atomes autres que l'absorbeur");
#endif
#ifdef _ENGLISH_
    FEN_titre(fenetre, "Removing background asborption (from atoms other than absorber)");
#endif

    PDFL_textes(ELIM_MUMATRICE, ELIM_MUM_PREMIER, ELIM_MUM_DERNIER,
		ELIM_MUM_AVANT, ELIM_MUM_APRES, ELIM_MUM_FOND, ELIM_MUM_FOND + 1,
		NBR_MUM_METHODES, (char **) Meth_mu_matrice, NULL, num_methode, 19, MUM_DROITE);
  }

  if (infos == NULL) /* Aucune info n'existe dj */
  {
    infos = (MAT_infos *) malloc( (size_t) sizeof(MAT_infos) );
    if (infos == NULL)
    {
      ferme_fenetre(fenetre);
      ERREUR(MANQUE_MEMOIRE); /* Sinon, risque de plantage sous X  cause des redessins... */
      return;
    }
    infos->type = num_methode;
    infos->type_depart = MUM_DROITE; /* On affiche  partir de cette mthode */
  }

  fenetre->pointeur = brut;  /* Graphe trait                    */
  fenetre->adresse  = infos; /* Infos d'extraction de mu_matrice */

#ifdef _FRANCAIS_
  sprintf(titre, "%s - "MU"(matrice)", brut->nom);
  infos->extrait = GRF_creer(brut->nombre_point, GR_MOINSMAT, GR_NORMAL, titre,
			     E_AIGU"nergie", "Absorbance", UNIT_EV, UNIT_ARB);
#endif
#ifdef _ENGLISH_
  sprintf(titre, "%s - "MU"(matrix)", brut->nom);
  infos->extrait = GRF_creer(brut->nombre_point, GR_MOINSMAT, GR_NORMAL, titre,
			     "Energy", "Absorption", UNIT_EV, UNIT_ARB);
#endif

  if (infos->extrait == NULL)
  {
    free(infos);
    ferme_fenetre(fenetre);
    return;
  }

  brut->drapeaux.protege = VRAI; /* Calcul en cours sur ce spectre -> rien de faisable */

  /* On cherche la valeur de E1 plausible */
  zone_x = brut->x;
  zone_y = brut->y;

  mu = *(zone_y+1);
  while (mu <= *zone_y) /* On descend */
  {
    zone_y++; zone_x++;
    mu =*(zone_y + 1);
  }
  if (zone_x == brut->x) /* Dtermination de E1 a chou */
    infos->e_1 = *(zone_x + 1);
  else
    infos->e_1 = *zone_x;
  met_fvaleur(ELIM_MUMATRICE, VALEUR_E1, infos->e_1);
  infos->e_1p = brut->x_min;
  met_fvaleur(ELIM_MUMATRICE, MUM_E1_PRIME, infos->e_1);

  OBJ_active(ELIM_MUMATRICE, MUM_DEGRE_AVANT, infos->type == MUM_POLYNOME);
  OBJ_active(ELIM_MUMATRICE, MUM_DEGRE_FOND , infos->type == MUM_POLYNOME);
  OBJ_active(ELIM_MUMATRICE, MUM_DEGRE      , infos->type == MUM_POLYNOME);
  OBJ_active(ELIM_MUMATRICE, MUM_DEG_APRES  , infos->type == MUM_POLYNOME);
  OBJ_active(ELIM_MUMATRICE, MUM_MU_0, infos->type == MUM_LANSBERGER);

  if ( OBJ_choisit(ELIM_MUMATRICE, MUM_POINT_2, QUESTION) )
    infos->mode = DEUX_POINTS;
  else
    infos->mode = UN_POINT;

  infos->degre = prend_valeur(ELIM_MUMATRICE, MUM_DEGRE);

  /* On calcule la fonction  soustraire */

  MAT_calc(brut, infos);

  /* On prpare le graphe contenant l'extraction */
  infos->extrait->drapeaux.protege = VRAI; /* On le protge de toute btise */
  infos->extrait->x_min = brut->x_min;
  infos->extrait->x_max = brut->x_max;

  memcpy(infos->extrait->x, brut->x, (size_t) sizeof(double) * brut->nombre_point);

  /* On fait la soustraction */
  MAT_soustrait(brut, infos->extrait, infos);

  fenetre->drapeaux.redessin    = VRAI;
  fenetre->drapeaux.pointeur    = VRAI;
  fenetre->msg_masque.fermeture = VRAI; /* Messages traits par MSG_matrice */
}

/* ------------- Redessin de la fentre d'limination de mu matrice -----------------------\
 |   Le clipping est dj install correctement lors de l'appel                            |
 |
 | fenetre    : structure associe  la fentre considre
 | x, y       : coordonnes de la zone de redessin
 | larg, haut : dimensions de la zone de redessin
 \---------------------------------------------------------------------------------------- */

static void RDS_matrice(Fenetre *fenetre, int x, int y, int larg, int haut)
{
  Graphe *graphe;
  MAT_infos *infos;
  DES_graphe param_dessin;
  double y_min, y_max;
  double x_echelle, y_echelle;
  int y_trace, haut_trace;

  graf_mouse(M_OFF, NULL); /* On coupe la souris */

  vsl_type (_PRG_station->handle, 1);
  vswr_mode(_PRG_station->handle, MD_REMPLACE);
  vsf_color(_PRG_station->handle, BLANC); /* couleur de remplissage: blanc */

  points[0] = x;            points[1] = y;
  points[2] = x + larg - 1; points[3] = y + haut + 1;
  v_bar(_PRG_station->handle, points); /* On met le fond au blanc */

  graphe = (Graphe *)    fenetre->pointeur;
  infos  = (MAT_infos *) fenetre->adresse;
  x_echelle =  ( (double) fenetre->w_larg ) / (graphe->x_max - graphe->x_min);

  /* Recherche des bornes de dessin */

  y_min = graphe->y_min;
  y_max = graphe->y_max;

  if (   (infos->type == MUM_DROITE)
    || ( (infos->type == MUM_POLYNOME) && (infos->degre == 1) ) )
  {
    if (infos->coefs[1] > 0) /* Fonction croissante */
    {
      y_min = graphe->x_min * infos->coefs[1] + infos->coefs[0];
      y_max = graphe->x_max * infos->coefs[1] + infos->coefs[0];
    }
    else
    {
      y_min = graphe->x_max * infos->coefs[1] + infos->coefs[0];
      y_max = graphe->x_min * infos->coefs[1] + infos->coefs[0];
    }
  }

  if ( infos->type == MUM_VICTORINE_1)
  {
    if (infos->coefs[1] > 0) /* Fonction dcroissante */
    {
      y_min = infos->coefs[0] + infos->coefs[1]/( (graphe->x_max) * (graphe->x_max) * (graphe->x_max) );	
      y_max = infos->coefs[0] + infos->coefs[1]/( (graphe->x_min) * (graphe->x_min) * (graphe->x_min) );
    }
    else
    {
      y_min = infos->coefs[0] + infos->coefs[1]/(graphe->x_min * graphe->x_min * graphe->x_min);
      y_max = infos->coefs[0] + infos->coefs[1]/(graphe->x_max * graphe->x_max * graphe->x_max);
    }
  }

  if ( infos->type == MUM_VICTORINE_2 )
  {
    double x_cube; /* X du minimum */

    if ( infos->coefs[1] > 0 ) /* Croissante puis dcroissante */
    {
      double x_max = - 1.33333333333333333 * infos->coefs[2] / infos->coefs[1];

      if (x_max < graphe->x_min) /* Maximum avant le graphe -> dcroissante */
      {
	x_cube = graphe->x_min * graphe->x_min * graphe->x_min;
	y_max = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_min);
	x_cube = graphe->x_max * graphe->x_max * graphe->x_max;
	y_min = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_max);
      }
      else if (x_max > graphe->x_max) /* Minimum aprs le graphe -> croissante */
      {
	x_cube = graphe->x_min * graphe->x_min * graphe->x_min;
	y_min = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_min);
	x_cube = graphe->x_max * graphe->x_max * graphe->x_max;
	y_max = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_max);
      }
      else
      {
	double inter_y;

	x_cube = x_max * x_max * x_max;
	y_max = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * x_max);

	x_cube = graphe->x_min * graphe->x_min * graphe->x_min;
	y_min = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_min);

	x_cube = graphe->x_max * graphe->x_max * graphe->x_max;
	inter_y = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_max);
	if (inter_y < y_min) y_min = inter_y;
      }
    }
    else if ( infos->coefs[1] < 0) /* dcroissante puis croissante */
    {
      double x_min = - 1.33333333333333333 * infos->coefs[2] / infos->coefs[1];

      if (x_min < graphe->x_min) /* Maximum avant le graphe -> croissante */
      {
	x_cube = graphe->x_min * graphe->x_min * graphe->x_min;
	y_min = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_min);
	x_cube = graphe->x_max * graphe->x_max * graphe->x_max;
	y_max = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_max);
      }
      else if (x_min > graphe->x_max) /* Minimum aprs le graphe -> dcroissante */
      {
	x_cube = graphe->x_min * graphe->x_min * graphe->x_min;
	y_max = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_min);
	x_cube = graphe->x_max * graphe->x_max * graphe->x_max;
	y_min = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_max);
      }
      else /* Minimum dans le graphe -> dcrot puis crot */
      {
	double inter_y;

	x_cube = x_min * x_min * x_min;
	y_min = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * x_min);

	x_cube = graphe->x_min * graphe->x_min * graphe->x_min;
	y_max = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_min);

	x_cube = graphe->x_max * graphe->x_max * graphe->x_max;
	inter_y = infos->coefs[0] + infos->coefs[1] / x_cube + infos->coefs[2] / (x_cube * graphe->x_max);
	if (inter_y > y_max) y_max = inter_y;
      }
    }
    else /* De la forme A + C * x^-4 */
    {
      if ( infos->coefs[2] > 0 ) /* Dcroissante */
      {
	x_cube = graphe->x_min * graphe->x_min * graphe->x_min * graphe->x_min;
	y_max  = infos->coefs[0] + infos->coefs[2] / x_cube;
	x_cube = graphe->x_max * graphe->x_max * graphe->x_max * graphe->x_max;
	y_min  = infos->coefs[0] + infos->coefs[2] / x_cube;
      }
      else
      {
	x_cube = graphe->x_min * graphe->x_min * graphe->x_min * graphe->x_min;
	y_min  = infos->coefs[0] + infos->coefs[2] / x_cube;
	x_cube = graphe->x_max * graphe->x_max * graphe->x_max * graphe->x_max;
	y_max  = infos->coefs[0] + infos->coefs[2] / x_cube;
      }
    }
  }
  if ( (infos->type == MUM_POLYNOME) && (infos->degre > 1) )
  {
    y_min = graphe->y_min; /* Voir  raffiner plus tard... */
    y_max = graphe->y_max;
  }

  if (y_min > graphe->y_min) y_min = graphe->y_min;
  if (y_max < graphe->y_max) y_max = graphe->y_max;

  /* On calcule les facteurs de conversion */

  y_trace    = fenetre->w_y + OB_HEIGHT(arbres[ELIM_MUMATRICE].adresse, ROOT);
  haut_trace = fenetre->w_haut - OB_HEIGHT(arbres[ELIM_MUMATRICE].adresse, ROOT);
  y_echelle  = ( (double) haut_trace ) / (y_max - y_min);

  /* On trace la courbe exprimentale - en noir */

  param_dessin.graphe  = graphe;
  param_dessin.couleur = NOIR;
  param_dessin.style   = 1;
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	     graphe->x_min, graphe->x_max, y_min, y_max, x_echelle, y_echelle);

  /* On trace la courbe  soustraire - en bleu */	
  vsl_color(_PRG_station->handle, BLEU);
  vsl_type (_PRG_station->handle, 1);

  if ( (infos->type == MUM_DROITE) /* C'est une droite : pas dur... */
    || ( (infos->type == MUM_POLYNOME) && (infos->degre == 1) ) )
  {
    points[0] = fenetre->w_x;
    points[1] = y_trace + y_echelle * (y_max - (infos->coefs[0] + infos->coefs[1]*graphe->x_min));
    points[2] = fenetre->w_x + fenetre->w_larg - 1;
    points[3] = y_trace + y_echelle * (y_max - (infos->coefs[0] + infos->coefs[1]*graphe->x_max));
    v_pline(_PRG_station->handle, 2, points);
  }

  if ( (infos->type == MUM_VICTORINE_1)
    || (infos->type == MUM_VICTORINE_2) )/* C'est une victorine */
  {
    double x_cube;
    register double *zone_x, *zone_y;
    int compteur;
    unsigned long index;

    compteur = 0;
    zone_x = graphe->x; zone_y = graphe->y;
    for (index = 0 ; index < graphe->nombre_point ; index ++)
    {
      double y;

      x_cube = 1/(*zone_x * *zone_x * *zone_x);
      y = infos->coefs[0] + infos->coefs[1] * x_cube + infos->coefs[2] * x_cube / (*zone_x);
      points[compteur]   = fenetre->w_x + x_echelle * ((*zone_x) - graphe->x_min);
      points[compteur+1] = y_trace + y_echelle * ( y_max - y );
      compteur += 2;
      zone_x++;	zone_y++;
      if (compteur >= 256) /* Tableau de points complet */
      {
	v_pline(_PRG_station->handle, 128, points);
	points[0] = points[254];
	points[1] = points[255];
	compteur = 2;
      }
    }
    v_pline(_PRG_station->handle, compteur >> 1, points);
  }

  if ( (infos->type == MUM_POLYNOME) && (infos->degre > 1) )
  {
    TRC_poly(graphe->nombre_point, infos->degre, infos->coefs, graphe->x,
	     x_echelle, y_echelle, fenetre->w_x, y_trace,
	     graphe->x_min, graphe->x_max, y_min, y_max);
  }

  /* On trace le rsultat obtenu - en gris fonc */

  y_echelle = ( (double) haut_trace) / (infos->extrait->y_max - infos->extrait->y_min);

  param_dessin.graphe  = infos->extrait;
  param_dessin.couleur = GRIS_FONCE;
  TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace,
	     graphe->x_min, graphe->x_max, infos->extrait->y_min, infos->extrait->y_max,
	     x_echelle, y_echelle);

  /* On met la barre rouge associe  E1 */

  vsl_color(_PRG_station->handle, ROUGE); /* On trace la barre */
  points[0] = fenetre->w_x + x_echelle * (infos->e_1 - graphe->x_min);
  points[1] = y_trace;
  points[2] = points[0];
  points[3] = y_trace + haut_trace;
  v_pline(_PRG_station->handle,2,points);
  
  /* Si ncessaire, on met la barre verte associe  E1' */

  if (infos->mode == DEUX_POINTS)
  {
    vsl_color(_PRG_station->handle, VERT);
    points[0] = fenetre->w_x + x_echelle * (infos->e_1p - graphe->x_min);
    points[2] = points[0];
    v_pline(_PRG_station->handle,2,points);		
  }

  graf_mouse(M_ON, NULL);
}

/* -------------- Messages reus par la fentre d'limination de mu_matrice -------------- */

static boolean MSG_matrice(Fenetre *fenetre)
{
  int retour;
  Graphe *brut, *traite;
  MAT_infos *infos;

  brut = (Graphe *) fenetre->pointeur;
  infos = (MAT_infos *) fenetre->adresse;

  switch ( buffer[0] ) /* Nature du message */
  {
  case WM_CLOSED: /* On veut fermer la fentre */
    retour = EXF_ERREUR(FIN_MODULE);
    if (retour == 3) /* On annule la fermeture */
    {
      return VRAI;
    }

    /* On ferme la fentre de toute faon */

    ferme_fenetre(fenetre);
    traite = infos->extrait;
    traite->drapeaux.protege = FAUX; /* Graphe n'a plus a tre protg */
    brut->drapeaux.protege = FAUX;

    if (retour == 2) /* On ferme sans soustraire */
    {
      GRF_efface(traite, VRAI, VRAI);
      free(infos);
      return VRAI;
    }

    /* On veut soustraire la contribution de la matrice */

    if ( Histoire ) /* On sauve les informations d'extraction */
    {
      fprintf(Histoire, "\n--- Soustraction de la contribution de la matrice ---\n");
      fprintf(Histoire, "Spectre initial  : %s\n", brut->nom);
      fprintf(Histoire, "Spectre obtenu   : %s\n", traite->nom);
      fprintf(Histoire, "M"e_AIGU"thode utilis"e_AIGU" : ");
      if (infos->type == MUM_DROITE)
      {
	fprintf(Histoire, " soustraction d'une droite, A * X + B\n");
	fprintf(Histoire, "                   A = %f\n", infos->coefs[1]);
	fprintf(Histoire, "                   B = %f\n", infos->coefs[0]);
      }
      if (infos->type == MUM_VICTORINE_1)
      {
	fprintf(Histoire, " soustraction d'une victorine, A + B / X^3\n");
	fprintf(Histoire, "                   A = %f\n", infos->coefs[0]);
	fprintf(Histoire, "                   B = %f\n", infos->coefs[1]);
      }
      if (infos->type == MUM_VICTORINE_2)
      {
	fprintf(Histoire, " soustraction d'une victorine, A + B / X^3 + C / X^4\n");
	fprintf(Histoire, "                   A = %f\n", infos->coefs[0]);
	fprintf(Histoire, "                   B = %f\n", infos->coefs[1]);
	fprintf(Histoire, "                   C = %f\n", infos->coefs[2]);
      }
      if ( infos->type != MUM_LANSBERGER ) /* Dfinition de E1 a un sens */
      {
	fprintf(Histoire, "  Premier point de contr"o_CIRC"le, E1 : %f\n", infos->e_1);
	if (infos->mode == DEUX_POINTS)
	  fprintf(Histoire, "  Second point de contr"o_CIRC"le, E1' : %f\n", infos->e_1p);
      }
    }

    num_methode = infos->type; /* On mmorise pour l'appel suivant */
    met_fvaleur(EXAFS_GENERAL, EXF_VALEUR_E1, infos->e_1);
    OBJ_choisit(EXAFS_GENERAL, EXF_MUM_DROITE, infos->type == MUM_DROITE);
    OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT1 , infos->type == MUM_VICTORINE_1);
    OBJ_choisit(EXAFS_GENERAL, EXF_MUM_VICT2 , infos->type == MUM_VICTORINE_2);
    OBJ_choisit(EXAFS_GENERAL, EXF_MUM_LANS  , infos->type == MUM_LANSBERGER);

    free(infos); /* On peut maintenant librer */

    sprintf(_intermediaire, "%s - "MU"(matrice)", brut->nom);
    GRF_titre(traite, _intermediaire); /* On met le titre */
    traite->drapeaux.sauve = FAUX;   /* Graphe non sauv            */
    GRF_aller(traite); /* On va  ce graphe */

    if (TT_xanes) /* On veut faire le traitement du XANES dans son ensemble */
    {
      Graphe *retour; /* Graphe renvoy */
      double *b, *c, *d, erreur;

      FEN_infos(principale, "Calcul du spectre liss"e_AIGU" en cours");
      retour = lisse_graphe(traite);        /* Lissage du spectre obtenu     */
      if (retour == NULL) /* Lissage n'a pas pu tre fait */
	retour = traite;

#ifdef _FRANCAIS_
      FEN_infos(principale, "Calcul des d"e_AIGU"riv"e_AIGU"es en cours");
#endif
#ifdef _ENGLISH_
      FEN_infos(principale, "Computing the first and second derivative");
#endif

      erreur = SPL_tt_prep(retour, &b, &c, &d);
      if (erreur < 0) /* Mthode prcise ne fonctionne pas -> mthode rapide */
      {
	Graphe *derive = DER_simple(retour); /* Drive premire */
	DER_simple(derive); /* Drive seconde  */
      }
      else
      {
	DER_lisse  (retour, retour->nombre_point, b, c, d);
	DER_seconde(retour, retour->nombre_point, b, c, d);
	free(b); free(c); free(d);
      }
      GRF_aller(retour); /* On va  ce graphe */
      
      TT_xanes = FAUX; /* On a fini */
    }

    if (TT_exafs) /* On veut faire le traitement de tout l'EXAFS d'un coup */
    {
      Graphe *retour;

#ifdef _FRANCAIS_
      FEN_infos(principale, "Calcul du spectre liss"e_AIGU" en cours");
#endif
#ifdef _ENGLISH_
      FEN_infos(principale, "Computing the smoothed spectrum");
#endif
      retour = lisse_graphe(traite);        /* Lissage du spectre obtenu  */
      if (retour == NULL)
	retour = traite;
      
      MU1_ouvre(retour); /* On passe  la premire tape d'extraction */
    }
    return VRAI;
  }

  return VRAI;
}

/* -------------- Clic sur la fentre d'limination de mu_matrice ------------------------ */

static boolean CLC_matrice(Fenetre *fenetre, int bouton, int n_clic, int etat, int x_clic, int y_clic)
{
  MAT_infos *infos;
  Graphe *graphe;
  double x_echelle;

#if GEM
  int handle;

  wind_get(0, WF_TOP, &handle, &n_clic, &etat, &bouton);
  if (handle != fenetre->handle)
  {
    wind_set(fenetre->handle, WF_TOP);
    return VRAI;
  }
#endif

  if ( (x_clic < fenetre->w_x) || (x_clic > fenetre->w_x + fenetre->w_larg)
    || (y_clic < fenetre->w_y) || (y_clic > fenetre->w_y + fenetre->w_haut) )
  {
    return VRAI; /* Clic en-dehors de la zone de travail */
  }

  infos  = (MAT_infos *) fenetre->adresse;
  graphe = (Graphe *)    fenetre->pointeur;

  x_echelle = (graphe->x_max - graphe->x_min) / ( (double) (fenetre->w_larg) );
  if (x_echelle)
  {
    if ( (infos->mode == UN_POINT) || (bouton == 1) ) /* Clic gauche, 2 points ou quelconque, 1 point */
    {
      infos->e_1 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
      met_fvaleur(ELIM_MUMATRICE, VALEUR_E1, infos->e_1);
    }
    if ( (bouton == 2) && (infos->mode == DEUX_POINTS) )
    {
      infos->e_1p = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
      met_fvaleur(ELIM_MUMATRICE, MUM_E1_PRIME, infos->e_1p);
    }

    graf_mouse(S_ABEILLE, NULL);
    MAT_calc     (graphe, infos); /* On recalcule la fonction  soustraire */
    MAT_soustrait(graphe, infos->extrait, infos);    /* Et on soustrait    */
    graf_mouse(S_FLECHE, NULL);
    FEN_redessin (fenetre);       /* On redessine le graphe...             */
  }

  return VRAI;
}

/* -------------- Gestion de l'arbre GEM associ  la soustraction de la matrice --------- */

boolean GER_matrice(int num_objet, int x_souris, int y_souris)
{
  MAT_infos *infos;
  Graphe *brut;
  Fenetre *fenetre;
  boolean etat;

  fenetre = ARB_fenetre(arbres + ELIM_MUMATRICE); /* Fentre actuelle */
  infos = (MAT_infos *) fenetre->adresse;
  brut  = (Graphe *)    fenetre->pointeur;

  if ( (num_objet >= ELIM_MUM_PREMIER) && (num_objet <= ELIM_MUM_DERNIER) )
  {
    unsigned long nv_methode = infos->type; /* Nouvelle mthode choisie */
    unsigned long prem_visible = infos->type_depart;
    
    GDFL_textes(ELIM_MUMATRICE, ELIM_MUM_PREMIER, ELIM_MUM_DERNIER,
		ELIM_MUM_AVANT, ELIM_MUM_APRES, ELIM_MUM_FOND, ELIM_MUM_FOND + 1,
		NBR_MUM_METHODES, (char **) Meth_mu_matrice, NULL, &nv_methode, 19,
		&prem_visible, num_objet, y_souris);

    if ( nv_methode != infos->type ) /* On change de mthode */
    {
      infos->type = (int) nv_methode;

      OBJ_active(ELIM_MUMATRICE, MUM_DEGRE_AVANT, nv_methode == MUM_POLYNOME);
      OBJ_active(ELIM_MUMATRICE, MUM_DEGRE_FOND , nv_methode == MUM_POLYNOME);
      OBJ_active(ELIM_MUMATRICE, MUM_DEGRE      , nv_methode == MUM_POLYNOME);
      OBJ_active(ELIM_MUMATRICE, MUM_DEG_APRES  , nv_methode == MUM_POLYNOME);

      MAT_calc     (brut, infos);                 /* On recalcule    */
      MAT_soustrait(brut, infos->extrait, infos); /* Et on soustrait */
      FEN_redessin (fenetre);
    }
  }

  switch (num_objet)
  {
  case ELIM_MUM_AVANT:
  case ELIM_MUM_FOND:
  case ELIM_MUM_FOND + 1: /* Le poussoir */
  case ELIM_MUM_APRES:
    {
      unsigned long nv_methode   = infos->type,
      prem_visible = infos->type_depart;

      GDFL_textes(ELIM_MUMATRICE, ELIM_MUM_PREMIER, ELIM_MUM_DERNIER,
		  ELIM_MUM_AVANT, ELIM_MUM_APRES, ELIM_MUM_FOND, ELIM_MUM_FOND + 1,
		  NBR_MUM_METHODES, (char **) Meth_mu_matrice, NULL, &nv_methode, 19,
		  &prem_visible, num_objet, y_souris);
      infos->type_depart = (int) prem_visible;
    }
    break;

  case VALEUR_E1:
    met_chaine(CHOIX_VALEUR, CHXVAL_UNITE, 0, "eV", TXT_REMPLACE);
#ifdef _FRANCAIS_
    met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "Valeur de E1", TXT_REMPLACE);
#endif
#ifdef _ENGLISH_
    met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "E1 value", TXT_REMPLACE);
#endif
    met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->e_1);

    ext_formdo(CHOIX_VALEUR);

    infos->e_1 = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
    met_fvaleur(ELIM_MUMATRICE, VALEUR_E1, infos->e_1);

    MAT_calc     (brut, infos);                 /* On recalcule    */
    MAT_soustrait(brut, infos->extrait, infos); /* Et on soustrait */
    FEN_redessin(fenetre);
    break;

  case MUM_E1_PRIME: /* Valeur du deuxime point, si on travaille avec 2 points */
    met_chaine(CHOIX_VALEUR, CHXVAL_UNITE, 0, "eV", TXT_REMPLACE);
#ifdef _FRANCAIS_
    met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "Second point de contr"o_CIRC"le", TXT_REMPLACE);
#endif
#ifdef _ENGLISH_
    met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, "Second control point", TXT_REMPLACE);
#endif
    met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->e_1p);

    ext_formdo(CHOIX_VALEUR);

    infos->e_1p = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
    met_fvaleur(ELIM_MUMATRICE, MUM_E1_PRIME, infos->e_1p);

    MAT_calc     (brut, infos);                 /* On recalcule    */
    MAT_soustrait(brut, infos->extrait, infos); /* Et on soustrait */
    FEN_redessin(fenetre);
    break;

  case MUM_POINT_2: /* Travail avec 1 ou 2 points de contrle ? */
    etat = OBJ_choisit(ELIM_MUMATRICE, MUM_POINT_2, ECHANGE);
    OBJ_active(ELIM_MUMATRICE, MUM_E1_PRIME, etat);
    if (etat) /* On travaille avec deux points */
      infos->mode = DEUX_POINTS;
    else
      infos->mode = UN_POINT;

    MAT_calc     (brut, infos);                 /* On recalcule    */
    MAT_soustrait(brut, infos->extrait, infos); /* Et on soustrait */
    FEN_redessin(fenetre);
    break;		

  case MUM_DEGRE_AVANT: /* Choix du degr du polynme */
  case MUM_DEGRE_FOND:
  case MUM_DEGRE:
  case MUM_DEG_APRES:
    infos->degre = gere_ascenseur(ELIM_MUMATRICE, MUM_DEGRE_AVANT,
		                              1, MUM_DEGRE_MAXIMAL, 1, FAUX, num_objet, x_souris, y_souris);
    MAT_calc     (brut, infos);                 /* On recalcule    */
    MAT_soustrait(brut, infos->extrait, infos); /* Et on soustrait */
    FEN_redessin(fenetre);
    break;
  }

  return VRAI;
}
