/* Traitement de l'E.X.A.F.S. sur Atari
   Tout ce qui concerne le X.A.N.E.S.
   (c) 1996-1998, Emmanuel CURIS - Cration : 22 mai 1998

   Normalisation du XANES
   Rcupr de ../XANES.C et de ../EXF_GEM.C ( GER_xnorm() )
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SYSTEME.H>

#include <EXAFS.H>
#include <GRAPHES.H>

#include <ERREURS.H>
#include <EXTRAIT.H> /* Pour lisse_graphe() et MU1_ouvre() */
#include <SCIENCE.H>

#include <XANES.H>

/* ************ Fonctions locales, qui n'apparaissent qu'ici ******************************* */

static boolean CLC_xnorm(Fenetre *fenetre, int bouton, int n_clic, int etat, int x_clic, int y_clic);
static boolean MSG_xnorm(Fenetre *fenetre);
static void    RDS_xnorm(Fenetre *fenetre, int x, int y, int larg, int haut);

/* ~~~~~~~~~~~~ Normalisation en cherchant le point de fin de seuil par modlisation ~~~~~~~ */

/* ------------ Ouverture de la fentre ---------------------------------------------------- */

void norm_xanes(Graphe *signal)
{
	Fenetre *fenetre;
	MU1_infos *infos = NULL;

	if (signal == NULL) return; /* Calcul n'a pas de raison d'tre */

	if ( arbres[NORM_XANES].ouvert == VRAI ) /* Arbre dj ouvert */
	{
		Graphe *vieux; /* Ancien spectre travaill dans ce module */
		int reponse = EXF_ERREUR(DEJA_ACTIF); /* Module dj ouvert */

		fenetre = ARB_fenetre( &(arbres[NORM_XANES]) );
		vieux = (Graphe *) fenetre->pointeur;
		if ( (reponse == 1) || (vieux == signal) ) /* On active seulement */
		{
			FEN_premierplan(fenetre); /* Fentre au premier plan */
			return; /* Rien d'autre  faire */
		}
		else if (reponse == 2) /* On rappelle, pour le nouveau graphe */
		{
			if ( signal->drapeaux.protege )
			{
				EXF_ERREUR(GRF_PROTEGE);
				return;
			}

			infos = (MU1_infos *) fenetre->adresse;
			vieux->drapeaux.protege = FAUX; /* On a fini de travailler avec */

			FEN_premierplan(fenetre); /* Fentre au premier plan */
		}
		else /* On annule l'appel */
			return;
	}
	else /* Arbre non ouvert : on l'ouvre */
	{
		if ( signal->drapeaux.protege )
		{
			EXF_ERREUR(GRF_PROTEGE);
			return;
		}

		fenetre = creer_mixte_fenetre(NAME|MOVER|CLOSER|FULLER|SIZER|INFO, NORM_XANES, HAUT,
	                         RDS_xnorm, (boolean (*)()) NULL, CLC_xnorm,
	                         MSG_xnorm, RCT_pointeur, (boolean (*)()) NULL );
		if (fenetre == NULL) return; /* Rien  faire... */
	}

	if (infos == NULL) /* Aucune info n'existe dj */
	{
		infos = (MU1_infos *) malloc( (size_t) sizeof(MU1_infos) );
		if (infos == NULL)
		{
			ferme_fenetre(fenetre);
			ERREUR(MANQUE_MEMOIRE); /* Sinon dangereux sous X... */
			return;
		}
	}
	fenetre->adresse  = infos;  /* Informations de normalisation    */
	fenetre->pointeur = signal; /* Adresse du graphe associ        */

	fenetre->drapeaux.redessin = VRAI; /* Redessin si changement de taille */
	fenetre->drapeaux.pointeur = VRAI;
	fenetre->msg_masque.fermeture = VRAI; /* Messages traits par MSG_mu01 */

	/* On prpare les calculs... */

	infos->e_2 = cherche_e0 (signal, FAUX) + 100;    /* E2 100 eV aprs le seuil */	
	met_fvaleur(NORM_XANES, NXAN_E2, infos->e_2);
	infos->e_0 = infos->e_2;
	met_fvaleur(NORM_XANES, NXAN_ENORM, infos->e_0);
	infos->degre = prend_valeur(NORM_XANES, NXAN_DEGRE);
	if ( OBJ_choisit(NORM_XANES, NXAN_RAPIDE, QUESTION) )
		infos->methode = ELIM_MU0_RAPIDE;
	else if ( OBJ_choisit(NORM_XANES, NXAN_LENTE, QUESTION) )
		infos->methode = ELIM_MU0_LENT;

	/* On calcule le polynome de dpart */

	MU1_calc(signal, infos);
}

/* -------------- Gestion du clic souris --------------------------------------------------- */

static boolean CLC_xnorm(Fenetre *fenetre, int bouton, int n_clic, int etat, int x_clic, int y_clic)
{
	MU1_infos *infos;
	Graphe *graphe;
	double x_echelle;

#if GEM
	int handle;

	wind_get(0, WF_TOP, &handle);
	if (handle != fenetre->handle)
	{
		wind_set(fenetre->handle, WF_TOP);
		return VRAI;
	}
#endif

	if ( (x_clic < fenetre->w_x) || (x_clic > fenetre->w_x + fenetre->w_larg)
	  || (y_clic < fenetre->w_y) || (y_clic > fenetre->w_y + fenetre->w_haut) )
	{
		return VRAI; /* Clic en-dehors de la zone de travail */
	}

	infos  = (MU1_infos *) fenetre->adresse;
	graphe = (Graphe *)    fenetre->pointeur;

	x_echelle = (graphe->x_max - graphe->x_min) / ( (double) (fenetre->w_larg) );
	if (x_echelle)
	{
		if (bouton == 1) /* Clic du bouton gauche -> E de normalisation */
		{
			infos->e_0 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
			met_fvaleur(NORM_XANES, NXAN_ENORM, infos->e_0);
		}
		else /* Bouton droit -> E2 */
		{
			infos->e_2 = graphe->x_min + (x_clic - fenetre->w_x) * x_echelle;
			met_fvaleur(NORM_XANES, NXAN_E2, infos->e_2);

			MU1_calc(graphe, infos); /* On recalcule le polynome */
		}
		FEN_redessin(fenetre);     /* On redessine             */
	}

	return VRAI;
}

/* -------------- Messages reus par la fentre de normalisation XANES --------------------- */

static boolean MSG_xnorm(Fenetre *fenetre)
{
	int i;
	Graphe *signal, *norm;
	MU1_infos *infos;
	register double *zone_y;
	double facteur;
	unsigned long index;

	signal = (Graphe *) fenetre->pointeur;
	infos  = (MU1_infos *) fenetre->adresse;

	if ( buffer[0] == WM_CLOSED ) /* On veut fermer la fentre */
	{
		char titre[300]; /* Titre du graphe normalis */
		int retour;

		if ( ( buffer[7] != 0 ) && ( buffer[1] == application ) ) /* Demande de fermeture interne */
			retour = 2; /* On ferme sans rien demander et sans rien faire */
		else
		  retour = EXF_ERREUR(FIN_MODULE); /* On demande s'il faut fermer */

		if (retour == 3) /* On annule la fermeture */
			return VRAI;

		if (retour == 2) /* On ferme sans normaliser */
		{
			free(infos);
			ferme_fenetre(fenetre);
			signal->drapeaux.protege = FAUX;
			return VRAI;
		}

		/* On veut normaliser le spectre */

		ferme_fenetre(fenetre);

#ifdef _FRANCAIS_
			sprintf(titre, "%s normalis"e_AIGU, signal->nom);
#endif
#ifdef _ENGLISH_
			sprintf(titre, "Normalized %s", signal->nom);
#endif

		if ( OBJ_choisit(NORM_XANES, NORM_NOUVEAU, QUESTION) ) /* On veut un nouveau graphe */
		{
			norm = GRF_creer(signal->nombre_point, signal->type, GR_NORMAL, titre, signal->axe_x, signal->axe_y, signal->unit_x, UNIT_NORM);
			if (norm == NULL)
				return VRAI; /* Problme... */
			memcpy(norm->x, signal->x, sizeof(double) * signal->nombre_point); /* On met les X */
			norm->x_min = signal->x_min;
			norm->x_max = signal->x_max;
		}
		else
		{
			norm = signal; /* On normalise sur le mme */
			GRF_titre(norm, titre);
		}

		/* On calcule la valeur du polynme en e_norm : vaut 1 par hypothse */

		facteur = 0;
		for (i = infos->degre ; i > 0 ; i--)
		{
			facteur += infos->coefs[i];
			facteur *= infos->e_0;
		}
		facteur += infos->coefs[0];
		facteur -= signal->y_min;
		if (facteur == 0)
		{
			EXF_ERREUR(DIV_ZERO);
			if (norm != signal)
				GRF_efface(norm, VRAI, VRAI);
			return VRAI;
		}

		/* On commence la normalisation */

		zone_y = signal->y;
		for (index = 0, zone_y = signal->y ; index < signal->nombre_point ; index++, zone_y++)
		{
			(norm->y)[index] = (*zone_y - signal->y_min) / facteur;
		}

		norm->y_max = (signal->y_max - signal->y_min) / facteur;
		norm->y_min = 0;

		if (Histoire)
		{
			fprintf(Histoire, "\n--- Normalisation d'un spectre d'absorption X\n");
			if (norm != signal)
				fprintf(Histoire, "Spectre original : %s\n", signal->nom);
			fprintf(Histoire, "Spectre normalis"e_AIGU" : %s\n", norm->nom);
			fprintf(Histoire, E_AIGU"nergie de normalisation : %f\n", infos->e_0);
			fprintf(Histoire, "Polyn"o_CIRC"me de normalisation : calcul"e_AIGU" de E2 = %f  Emax = %f\n", infos->e_2, signal->x_max);
			fprintf(Histoire, "                         extrapol"e_AIGU" jusqu'"a_GRAVE" Enorm\n");
			fprintf(Histoire, "    M"e_AIGU" de calcul : ");
			if (infos->methode == ELIM_MU0_RAPIDE)
				fprintf(Histoire, "moindres carr"e_AIGU"s simples\n");
			else
				fprintf(Histoire, "algorithme EDA\n");
			fprintf(Histoire, "    Degr"e_AIGU"           : %d\n", infos->degre);
			fprintf(Histoire, "    Polyn"o_CIRC"me trouv"e_AIGU" : A + B * X + ...\n");
			for (i = 0 ; i <= infos->degre ; i++)
			{
				fprintf(Histoire, "        %c = ", 'A' + i);
				if ( fabs(infos->coefs[i]) < 0.01 )
					fprintf(Histoire, "%e\n", infos->coefs[i]);
				else
					fprintf(Histoire, "%f\n", infos->coefs[i]);
			}
		}

		free(infos);
		GRF_aller(norm); /* Pour tre sr du redessin */
		return VRAI;
	}

	return VRAI;
}

/* ------------- Redessin de la fentre de normalisation X.A.N.E.S. -----------------------\
 |   Le clipping est dj install directement lors de l'appel                             |
 |
 | fenetre    : structure associe  la fentre considre 
 | x, y       : coordonnes de la zone de redessin
 | larg, haut : dimensions de la zone de redessin
 \------------------------------------------------------------------------------------- */

static void RDS_xnorm(Fenetre *fenetre, int x, int y, int larg, int haut)
{
	Graphe *graphe;
	MU1_infos *infos;
	DES_graphe param_dessin;
	double x_echelle, y_echelle;
	int y_trace, haut_trace;

	vsl_type (_PRG_station->handle, 1);
	vswr_mode(_PRG_station->handle, MD_REMPLACE);
	vsf_color(_PRG_station->handle, BLANC); /* Couleur de remplissage : blanc */

	points[0] = x;            points[2] = y;
	points[2] = x + larg - 1; points[3] = y + haut + 1;
	v_bar(_PRG_station->handle, points); /* On met le fond en blanc */

	y_trace    = fenetre->w_y    + OB_HEIGHT(arbres[NORM_XANES].adresse, ROOT);
	haut_trace = fenetre->w_haut - OB_HEIGHT(arbres[NORM_XANES].adresse, ROOT);

	graphe = (Graphe *)    fenetre->pointeur;
	infos  = (MU1_infos *) fenetre->adresse;
	x_echelle = ( (double) fenetre->w_larg ) / (graphe->x_max - graphe->x_min);
	y_echelle = ( (double) haut_trace )      / (graphe->y_max - graphe->y_min);

	/* On trace la courbe exprimentale - en noir */

	param_dessin.graphe  = graphe;
	param_dessin.couleur = NOIR;
	param_dessin.style   = 1;
	TRC_graphe(_PRG_station, &param_dessin, fenetre->w_x, y_trace, 
	           graphe->x_min, graphe->x_max, graphe->y_min, graphe->y_max, x_echelle, y_echelle);

	/* On trace la courbe  soustraire - en bleu */	
	vsl_color(_PRG_station->handle, BLEU);
	vsl_type (_PRG_station->handle, 1);

	if (infos->degre == 1) /* C'est une droite : pas dur... */
	{
		points[0] = fenetre->w_x;
		points[1] = y_trace + y_echelle * (graphe->y_max - (infos->coefs[0] + infos->coefs[1]*graphe->x_min));
		points[2] = fenetre->w_x + fenetre->w_larg - 1;
		points[3] = y_trace + y_echelle * (graphe->y_max - (infos->coefs[0] + infos->coefs[1]*graphe->x_max));
		v_pline(_PRG_station->handle, 2, points);
	}
	else
	{
		double x, y;
		unsigned long compteur = 0, index;
		register double *zone_x = graphe->x;
		int i;

		for (index = 0 ; index < graphe->nombre_point ; index++)
		{
			y = infos->coefs[0]; /* La constante */
			x = 1;
			for (i = 1 ; i <= infos->degre ; i++)
			{
				x *= *(zone_x);
				y += infos->coefs[i] * x;
			}
			points[compteur]   = fenetre->w_x + x_echelle * ((*zone_x) - graphe->x_min);
			points[compteur+1] = y_trace + y_echelle * ( graphe->y_max - y );
			compteur += 2;
			zone_x++;
			if (compteur >= 256) /* Tableau de points complet */
			{
				v_pline(_PRG_station->handle,128,points);
				points[0] = points[254];
				points[1] = points[255];
				compteur = 2;
			}
		}
		v_pline(_PRG_station->handle,(int) (compteur-1)/2,points);
	}

	/* On met les barres */

	vsl_color(_PRG_station->handle, ROUGE); /* Barre rouge : E de normalisation */
	points[0] = fenetre->w_x + x_echelle * (infos->e_0 - graphe->x_min);
	points[1] = y_trace;
	points[2] = points[0];
	points[3] = y_trace + haut_trace;
	v_pline(_PRG_station->handle,2,points);

	vsl_color(_PRG_station->handle, VERT); /* Barre verte : E2 */
	points[0] = fenetre->w_x + x_echelle * (infos->e_2 - graphe->x_min);
	points[1] = y_trace;
	points[2] = points[0];
	points[3] = y_trace + haut_trace;
	v_pline(_PRG_station->handle,2,points);
}

boolean GER_xnorm(int num_objet, int x, int y)
{
	MU1_infos *infos;
	Graphe *brut;
	Fenetre *fenetre;

	fenetre = ARB_fenetre(arbres + NORM_XANES); /* Fentre actuelle */
	infos = (MU1_infos *) fenetre->adresse;
	brut  = (Graphe *)    fenetre->pointeur;

	switch (num_objet)
	{
	case NXAN_RAPIDE: /* Mthode utilise pour approcher l'absorption atomique */
		if ( infos->methode != ELIM_MU0_RAPIDE )
		{
			OBJ_choisit(NORM_XANES, NXAN_RAPIDE, FORCE_OUI);
			infos->methode = ELIM_MU0_RAPIDE;
			MU1_calc   (brut, infos); /* On recalcule  */
			FEN_redessin(fenetre);
		}
		return VRAI;

	case NXAN_LENTE:
		if ( infos->methode != ELIM_MU0_LENT) /* On change de mthode */
		{
			OBJ_choisit(NORM_XANES, NXAN_LENTE, FORCE_OUI);
			infos->methode = ELIM_MU0_LENT;
			MU1_calc   (brut, infos); /* On recalcule  */
			FEN_redessin(fenetre);
		}
		break;

	case NXAN_ENORM_TXT: /* nergie de normalisation */
	case NXAN_ENORM:
		met_chaine(CHOIX_VALEUR, CHXVAL_UNITE, 6, "eV", TXT_COPIER);
		met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, 
		           E_AIGU"nergie de normalisation", TXT_REMPLACE);
		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->e_0);

		ext_formdo(CHOIX_VALEUR);

		infos->e_0 = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		met_fvaleur(NORM_XANES, NXAN_ENORM, infos->e_0);
		FEN_redessin(fenetre);
		return VRAI;

	case NXAN_E2_TXT:
	case NXAN_E2: /* Valeur  utiliser pour E2 */
		met_chaine(CHOIX_VALEUR, CHXVAL_UNITE, 6, "eV", TXT_COPIER);
		met_chaine(CHOIX_VALEUR, CHXVAL_TITRE, 0, 
		           "Valeur de E2", TXT_REMPLACE);
		met_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR, infos->e_2);

		ext_formdo(CHOIX_VALEUR);

		infos->e_2 = prend_fvaleur(CHOIX_VALEUR, CHXVAL_VALEUR);
		met_fvaleur(NORM_XANES, NXAN_E2, infos->e_2);
		MU1_calc   (brut, infos); /* On recalcule le polynome */
		FEN_redessin(fenetre);
		break;

	case NXAN_D_MOINS:
	case NXAN_D_FOND:
	case NXAN_DEGRE:
	case NXAN_D_PLUS:
		infos->degre = gere_ascenseur(NORM_XANES, NXAN_D_MOINS, 1, 9, 1, FAUX, num_objet, x, y);
		MU1_calc   (brut, infos); /* On recalcule    */
		FEN_redessin(fenetre);
		break;

	case NORM_NOUVEAU:
		OBJ_choisit(NORM_XANES, NORM_NOUVEAU, ECHANGE);
	}
	return VRAI;
}
