/* Bibliothque X-Window pour les interfaces
            Routines gnrales
	    Compatible avec gem_gene.c (version GEM : rfrence)
   (C) 1998, Emmanuel CURIS - Cration le 19 mars 1998
*/

#define _X_GENERAL_ 
#define _LIBRAIRIE_X_ 

#include <malloc.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <signal.h>   /* Pour le timer */
#include <sys/time.h> /* Pour le timer */

#include "Bib_X.h"

#include "X_motifs.h"

extern Pixmap PIX_inactif; /* Dans X_objets.c */

/* ****** Variables globales limites  ce module ************** */

static XEvent vx_evenement; /* Ancien vnement souris - static global pour le signal */
static int    _n_clic;      /* Version interne pour viter des problmes btes */

/**************** Fonctions internes  ce module *****************/

int CLAV_arbre(Arbre *arbre, XKeyEvent *clavier);          /* Gestion du clavier pour un arbre */
int CLIC_arbre(Arbre *arbre, XButtonEvent *souris, int *bouton); /* Clic sur un arbre d'objets */

int CLAV_menu(XKeyEvent *clavier);

static boolean XAllouerCouleur(const char *nom, XColor *couleur);

static boolean attend_ss_timer(int masque, int rect, int x_rc, int y_rc, int larg_rc, int haut_rc);
static boolean Traite_evenement(Fenetre *fenetre, XEvent evenement, XEvent *vx_evenement);

static void Traite_signal(void);

/**************** Fonctions proprement dites ******************* */

/* ~~~~~~~~~~ Lancement, fin du programme ~~~~~~~~~~~ */

/* ---------- Initialisation du programme --------------\
 |   Renvoie VRAI si l'initialisation a russi          |
 |           FAUX sinon                                 |
 |                                                      |
 | nom_ressource : le fichier *.RSC  charger           |
 | nbr_arbres    : le nombre d'arbres dans le ressource |
 | total_arbres  : le nombre total d'arbres             |
 \----------------------------------------------------- */

boolean initialise(char *nom_ressource, int total_arbres, int nbr_arbres)
{
  XGCValues GC_valeurs;
  unsigned long GC_masque;
  extern OBJECT *adresse_selecteur; /* Dans X_fichiers.c */

  _Display = XOpenDisplay(NULL); /* Ouverture du display */
  if (_Display == NULL)
  {

#ifdef _FRANCAIS_

    printf("Ouverture de l'affichage (Display) impossible\n");
    printf("V"e_AIGU"rifiez que vous utilisez bien X-Window\n");
    printf("V"e_AIGU"rifiez que la variable DISPLAY est correctement configur"e_AIGU"e\n");
    printf(" -------- Fin du programme --------\n");

#endif

#ifdef _ENGLISH_
    printf("Can't open display. Program will end now\n");
#endif

    return FAUX;
  }

  /*  XSynchronize(_Display, VRAI); *//* On synchronise les vnements X et le programme : dbogage
				   Ralentit normment... */

  /* Lecture des paramtres essentiels  l'utilisation de X */

  _Ecran        = DefaultScreenOfDisplay(_Display);
  _Num_ecran    = DefaultScreen(_Display);               /* cran utilis pour l'affichage */
  _Visual       = DefaultVisual(_Display, _Num_ecran);   /* ????                           */
  _Fen_racine   = RootWindow(_Display, _Num_ecran);      /* Fentre racine X-Window        */
  _GC_programme = DefaultGC(_Display, _Num_ecran);       /* GC du programme                */
  _Palette      = DefaultColormap(_Display, _Num_ecran); /* Palette de couleurs par dfaut */

  XAllouerCouleur("black"           , &_Couleurs[NOIR]        );
  XAllouerCouleur("white"           , &_Couleurs[BLANC]       );
  XAllouerCouleur("red"             , &_Couleurs[ROUGE]       );
  XAllouerCouleur("green"           , &_Couleurs[VERT]        );
  XAllouerCouleur("blue"            , &_Couleurs[BLEU]        );
  XAllouerCouleur("cyan"            , &_Couleurs[CYAN]        );
  XAllouerCouleur("yellow"          , &_Couleurs[JAUNE]       );
  XAllouerCouleur("magenta"         , &_Couleurs[VIOLET]      );
  XAllouerCouleur("light grey"      , &_Couleurs[GRIS]        );
  XAllouerCouleur("dim grey"        , &_Couleurs[GRIS_FONCE]  );
  XAllouerCouleur("indian red"      , &_Couleurs[ROUGE_FONCE] );
  XAllouerCouleur("dark green"      , &_Couleurs[VERT_FONCE]  );
  XAllouerCouleur("midnight blue"   , &_Couleurs[BLEU_FONCE]  );
  XAllouerCouleur("light steel blue", &_Couleurs[CYAN_FONCE]  );
  XAllouerCouleur("orange"          , &_Couleurs[JAUNE_FONCE] );
  XAllouerCouleur("violet red"      , &_Couleurs[VIOLET_FONCE]);

  /* chargement d'une fonte X... */

  _Fonte = XLoadQueryFont(_Display, _Nom_fonte);
  if (_Fonte == NULL)
  {
#ifdef _FRANCAIS_
    printf("Chargement de la fonte impossible...\n Chargement du programme annul"e_AIGU"\n");
    printf("Essayez de modifier le nom (_Nom_fonte) dans X_var.c\n");
    printf("   (Fonte actuelle : %s)\n", _Nom_fonte);
#endif
#ifdef _ENGLISH_
    printf("Font loading failed. Programm aborted\n");
#endif
    return FAUX;
  }
  XSetFont(_Display, _GC_programme, _Fonte->fid);

  GC_valeurs.function   = 0x3; /* Mode: remplacement */
  GC_valeurs.background = _Couleurs[GRIS].pixel;
  GC_valeurs.foreground = _Couleurs[NOIR].pixel;
  GC_masque = GCForeground | GCBackground | GCFunction;

  _GC_objets = XCreateGC(_Display, _Fen_racine, GC_masque, &GC_valeurs);
  XSetFont(_Display, _GC_objets, _Fonte->fid);

  /* On s'occupe de la VDI */

  _PRG_station = VDI_ouvre(1); /* On ouvre une station sur l'cran */

  gdos = -2; /* Pas de GDOS sous cette version... On verra plus tard */

  /* On s'occupe des arbres GEM */

  _nbr_arbres = total_arbres; /* Nombre total d'arbres utilis */

  arbres = (Arbre *) calloc( _nbr_arbres + 2, sizeof(Arbre) );
  if (arbres == NULL)
  {
    ERREUR(MANQUE_MEMOIRE);
    printf("dans X_gene.c\n");
    return FAUX;
  }

  _ARB_alertes = arbres + _nbr_arbres;
  _ARB_alertes->adresse = Boite_d_alerte;
  ARB_init(_nbr_arbres, VRAI);

  /* Prparation du slecteur de fichiers */

  _ARB_fselect = arbres + _nbr_arbres + 1;
  _ARB_fselect->adresse = adresse_selecteur;
  ARB_init(_nbr_arbres + 1, VRAI);

  if (_nbr_arbres) /* Il y a plus d'un arbre */
  {
    if ( (nom_ressource) && (nbr_arbres > 0) ) /* On veut charger un ressource */
    {
      int i;
      Ressource *ressource;

      ressource = RSC_creer(); /* On cre le ressource  charger */
      if (ressource == NULL)
	return FAUX;

      if ( RSC_charge(nom_ressource, ressource) == FAUX )
      {
	printf("Erreur  la lecture du ressource.\n");
	printf("Chargement du programme annul\n");
	return FAUX;
      }

      if ( ressource->nombre_arbres != nbr_arbres )
      {
	printf("ATTENTION : il y a %ld arbres dans le ressource, pas %d!\n", 
	       ressource->nombre_arbres, nbr_arbres);
	if ( ressource->nombre_arbres < nbr_arbres )
	  nbr_arbres = ressource->nombre_arbres;
      }

      for (i = 0 ; i < nbr_arbres ; i++)
      {
	if ( RSC_lit_adresse(ressource, R_TREE, i, &(arbres[i].adresse) ) )
	{
	  printf("Problme lors du chargement du ressource\n");
	  return FAUX;
	}
	ARB_init(i, FAUX);
      }
    }
  }

  /* Init. des diffrents motifs de remplissage */

  PIX_inactif = XCreateBitmapFromData(_Display, _Fen_racine, RMP_inactif, 32, 32);

  return VRAI; /* Tout s'est bien pass, on peut continuer */
}

static boolean XAllouerCouleur(const char *nom, XColor *couleur)
{
  XColor reelle;
  Status allouee; /* (0 = non, reste = oui) */

  if ( (couleur == NULL) || (nom == NULL) )
    return FAUX;

  allouee = XAllocNamedColor(_Display, _Palette, nom, couleur, &reelle);
  if (allouee == FAUX)
  {
#ifdef _FRANCAIS_
    printf("Impossible de d"e_AIGU"clarer la couleur %s\n", nom);
#endif
#ifdef _ENGLISH_
    printf("Can't allocate color %s\n", nom);
#endif
    return FAUX;
  }

  return VRAI;
}

/* ------------ Fin du programme : on ferme tout proprement et tout et tout ----------- */

void fin_programme(int num_menu)
{
  if (_Menu) /* Il y a un menu d'install */
    supprime_menu();

  while (_premiere_fenetre)
    ferme_fenetre(_premiere_fenetre); /* On ferme toutes les fentres */

  RSC_tt_effacer(); /* On efface toutes les ressources */

  free(arbres);
  XFreeFont(_Display, _Fonte);
  XCloseDisplay(_Display); /* On termine X */
}

/* ~~~~~~~~~~~~ Fonctions gnrales de traitement ~~~~~~~~~~~ */

/* ------------ Nouvelle mouture de evnt_multi ---------------\
 |    Renvoie les vnements existant, comme evnt_multi       |
 |            _FIN_PRG si il faut quitter le programme        |
 |                                                            |
 | masque : vnements que veut l'utilisateur (evnt_multi)    |
 | rect   : vnement Rectangle (comme evnt_multi)            |
 | x_rc,y_rc,larg_rc,haut_rc : coordonnes de ce rectangle    |
 | compteur : temps d'attente demand                         |
 | x,y    : coordonnes de la souris                          |
 | bouton : bouton qui a t cliqu                           |
 | etat   : tat des touches spciales du clavier             |
 | touche : touche appuye (octet 1 : ASCII, 2 : SCAN)        |
 | n_clic : nombre de clics souris                            |
 \----------------------------------------------------------- */

int attend_evenement(int masque,
                     int rect, int x_rc, int y_rc, int larg_rc, int haut_rc,
                     unsigned long compteur,
                     int *x, int *y, int *bouton, int *etat, unsigned int *touche, int *n_clic)
{
  int final = VRAI;
  unsigned int _touche;
  unsigned long temps;
  struct itimerval timer, ancien_timer;
  Window vx_fenetre = 0;
  Fenetre *fenetre = NULL;
  XEvent evenement /*, vx_evenement */;

  vx_evenement.type = 0;         /* On commence sans ancien vnement         */

  timer.it_interval.tv_sec = 0; /* Quoi qu'il arrive, le timer ne se remet pas tout seul */
  timer.it_interval.tv_usec = 0;

  if ( (masque & MU_TIMER) == FAUX ) /* Pas de timer, donc beaucoup plus simple */
  {
    final = attend_ss_timer(masque, rect, x_rc, y_rc, larg_rc, haut_rc);
  }
  else if (compteur == 0) /* TIMER mis  zro -> on traite tous les vnements X et on ressort */
  {
    while ( XPending(_Display) && (final == VRAI) ) /* Tant qu'il y a des vnements X dans la queue */
    {
      XNextEvent(_Display, &evenement); /* On rcupre l'venement */

      timer.it_value.tv_sec = 0;
      timer.it_value.tv_usec = 0;
      setitimer(ITIMER_REAL, &timer, &ancien_timer); /* On met le Timer  zro : pas de double clic */

      _Fen_redessin = evenement.xany.window; /* Fentre concerne -> pour les redessin, ok... */
      if (_Fen_redessin != vx_fenetre)
      {
	vx_fenetre = _Fen_redessin;
	fenetre = quelle_fenetre(_Fen_redessin);
	/*      printf("Fentre concerne : %p\n", fenetre); */
      }

      final = Traite_evenement(fenetre, evenement, &vx_evenement);
    }

    /* Sortie parce que plus d'vnement -> on retourne avec le Timer */
    if ( final == VRAI )
      final = MU_TIMER;
  }
  else
  {
    temps = clock(); /* On initialise */

    while (XPending(_Display) == 0) /* Il n'y a pas d'autre vnement */
    {
      if ( (clock() - temps) > compteur )
	return MU_TIMER; /* vnement timer */
    }

    _n_clic = 0; /* Un clic normalement */

    while (final == VRAI) /* On traite en interne, donc rien  renvoyer... */
    {
      XNextEvent(_Display, &evenement); /* On rcupre l'venement */

      timer.it_value.tv_sec = 0;
      timer.it_value.tv_usec = 0;
      setitimer(ITIMER_REAL, &timer, &ancien_timer); /* On met le Timer  zro : pas de double clic */

      _Fen_redessin = evenement.xany.window; /* Fentre concerne -> pour les redessin, ok... */
      if (_Fen_redessin != vx_fenetre)
      {
	vx_fenetre = _Fen_redessin;
	fenetre = quelle_fenetre(_Fen_redessin);
	/*      printf("Fentre concerne : %p\n", fenetre); */
      }

      final = Traite_evenement(fenetre, evenement, &vx_evenement);
    }

    if (final == _FIN_PRG) /* On demande de quitter : important... */
      return _FIN_PRG;

    if ( (masque & MU_TIMER) && ((clock() - temps) > compteur) )
      final = MU_TIMER;

    /*    if (final != VRAI)
      printf("Valeur de final : %d\n", final); */
  }

  *x = _x_souris ; *y = _y_souris ; *bouton = _bouton; *n_clic = _n_clic; /* Souris  */
  *touche = _touche; *etat = _etat;                                       /* Clavier */

  return final;
}

/* ---------------- Attente d'un vnement X, version sans TIMER ------\
 | Paramtres exactement comme ci-dessus                               |
 \-------------------------------------------------------------------- */

static boolean attend_ss_timer(int masque,
			       int rect, int x_rc, int y_rc, int larg_rc, int haut_rc)
{
  int final = VRAI;
  struct itimerval timer, ancien_timer;
  Window vx_fenetre = 0;
  Fenetre *fenetre = NULL;
  XEvent evenement /*, vx_evenement */;

  vx_evenement.type = 0;         /* On commence sans ancien vnement         */

  timer.it_interval.tv_sec = 0; /* Quoi qu'il arrive, le timer ne se remet pas tout seul */
  timer.it_interval.tv_usec = 0;

  _n_clic = 0; /* Un clic normalement */

  while (final == VRAI) /* On traite en interne, donc rien  renvoyer... */
  {
    XNextEvent(_Display, &evenement); /* On rcupre l'venement */

    /*    printf("On annule le timer, il s'est produit quelque chose\n"); */

    timer.it_value.tv_sec = 0;
    timer.it_value.tv_usec = 0;
    setitimer(ITIMER_REAL, &timer, &ancien_timer);

    _Fen_redessin = evenement.xany.window; /* Fentre concerne -> pour les redessin, ok... */
    if (_Fen_redessin != vx_fenetre)
    {
      vx_fenetre = _Fen_redessin;
      fenetre = quelle_fenetre(_Fen_redessin);
      /*      printf("Fentre concerne : %p\n", fenetre); */
    }

    final = Traite_evenement(fenetre, evenement, &vx_evenement);

    if (final == _FIN_PRG) /* On demande de quitter : important... */
      return _FIN_PRG;

    /*    if (final != VRAI)
      printf("Valeur de final : %d\n", final); */
  }

  return final;
}

/* ~~~~~~~~~~~~~~~~ Traitement des vnements X en fonction du type ~~~~~~~~~~~~ *
 |  Renvoie l'tat de sortie correspondant au traitement de l'vnement          |
 |                                                                               |
 | fenetre      = la fentre pour laquelle l'vnement X a eu lieu               |
 | evenement    = l'vnement X ayant eu lieu                                    |
 | vx_evenement = le prcdent vnement X ayant eu lieu (pour double clic)      | 
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

static boolean Traite_evenement(Fenetre *fenetre, XEvent evenement, XEvent *vx_evenement)
{
  boolean final = VRAI;
  struct itimerval timer, ancien_timer;

  timer.it_interval.tv_sec = 0;  /* Quoi qu'il arrive, le timer ne se remet pas tout seul         */
  timer.it_interval.tv_usec = 0; /* (Si pas  zro, timer priodique -> fonction jamais appele ! */

  timer.it_value.tv_sec = 0;
  timer.it_value.tv_usec = 100000; /* Timer de 100 ms : temps maximal entre 2 clics */

  if ( evenement.type == ButtonRelease )      /* On relche le bouton */
  {
/*    printf("Bouton relch : on enregistre le clic\n"); */
    if ( EVNT_relache_bouton(fenetre, &(evenement.xbutton)) == VRAI )
    {
      vx_evenement->type = 0;
    }
    else if (vx_evenement->type == ButtonPress) /* Le clic a bien t reu... */
    {
      vx_evenement->xbutton = evenement.xbutton;

      /* On met en place un timer pour tre sr de traiter ce clic aprs quelques ms */

/*      printf("On installe le Timer\n"); */

      signal(SIGALRM, Traite_signal);

      setitimer(ITIMER_REAL, &timer, &ancien_timer);
    }
    else
    {
      vx_evenement->type = 0;
    }

    return VRAI; /* vnement trait */
  }

  if ( evenement.type == ButtonPress ) /* On enfonce le bouton */
  {
/*    printf("Clic du bouton souris\n"); */
    return EVNT_clic(fenetre, &(evenement.xbutton), vx_evenement, &_n_clic);
  }

  /* Si on est l, c'est que l'vnement n'est pas un clic sur un bouton de la souris */

  if ( (vx_evenement->type == ButtonRelease)
    || (vx_evenement->type == ButtonPress) ) /* Il y avait un clic enregistr */
  {
/*    printf("Autre vnement : on traite le clic (%d clics)\n", _n_clic); */
    if (fenetre)
      final = EVNT_souris(fenetre, &(vx_evenement->xbutton), _n_clic ); /* Clic */
    else
      final = VRAI;
    _n_clic = 0; /* Un seul clic */
    _Fen_redessin = evenement.xany.window; /* Car peut changer au cours du traitement... */
    vx_evenement->type = 0; /* Aucun vnement */
  }

  switch (evenement.type)
  {
  case KeyPress: /* On a appuy sur une touche */
    if ( fenetre == NULL ) /* Raccourci pour le menu, a priori */
      final = CLAV_menu( &(evenement.xkey) ); /* Raccourci pour le menu ? */
    else
      final = EVNT_clavier(fenetre, &(evenement.xkey) ); /* VRAI si a t trait */
    break;
      
  case KeyRelease: /* On ignore - sauf pour les touches d'tat ? */
    EVNT_relache( &(evenement.xkey) );
    break;

  case ConfigureNotify: /* Changement de taille de la fentre - XConfigureEvent */
    if (fenetre) /* Il y a une fentre classique associe */
    {
      int delta_x, delta_y;

      delta_x = evenement.xconfigure.width - fenetre->larg;
      delta_y = evenement.xconfigure.height - fenetre->haut;
      if ( (delta_x == 0) && (delta_y == 0) )
	break;

      /* Si fentre demande un message spcial en cas de changement de taille, on le fait */
	  
      if ( fenetre->msg_masque.taille == VRAI )
      {
	buffer[0] = WM_SIZED;
	buffer[1] = 0;
	buffer[2] = 0;
	/*	  buffer[3] = fenetre->handle; - mettre la bonne fentre si possible... */
	buffer[4] = fenetre->x;
	buffer[5] = fenetre->y;
	buffer[6] = evenement.xconfigure.width;
	buffer[7] = evenement.xconfigure.height;
	(fenetre->message)(fenetre);
      }

      /* Si fentre demande un message de redessin en cas de changement de taille, on le fait */
	  
      if ( fenetre->drapeaux.redessin == VRAI )
	FEN_redessin(fenetre);

      /* On effectue le changement de taille */
	  
      change_taille_fenetre(fenetre, evenement.xconfigure.width, evenement.xconfigure.height);
	  
      if ( fenetre->type == MIXTE_FENETRE ) /* On s'occupe de l'arbre */
      {
	Arbre *arbre = fenetre->arbre;

	if ( fenetre->drapeau == HAUT ) /* Arbre en haut de la fentre */
	{
	  OB_WIDTH(arbre->adresse, ROOT) += delta_x;
	  XResizeWindow(_Display, (arbre->objets)[ROOT],
			OB_WIDTH(arbre->adresse, ROOT), OB_HEIGHT(arbre->adresse, ROOT) );
	}

	if ( fenetre->drapeau == GAUCHE ) /* Arbre  gauche de la fentre */
	{
	  OB_HEIGHT(arbre->adresse, ROOT) += delta_y;
	  XResizeWindow(_Display, (arbre->objets)[ROOT],
			OB_WIDTH(arbre->adresse, ROOT), OB_HEIGHT(arbre->adresse, ROOT) );
	}
      }

      /* Si fentre demande un message de redessin en cas de changement de taille, on le fait */
	  
      if ( fenetre->drapeaux.redessin == VRAI )
	FEN_redessin(fenetre);
    }
    break;

  case Expose: /* On affiche la fentre = redessin  - XExposeEvent */
    final = EVNT_redessin(fenetre, (XExposeEvent *) &evenement );
    break;

  case MotionNotify: /* Dplacement du curseur */
    EVNT_pointeur(fenetre, &(evenement.xmotion) );
    break;

  case ResizeRequest: /* Changement de taille */
    break;

  case EnterNotify:  /* Entre dans une fentre */
    _x_pointeur = -1;
    _y_pointeur = -1;
    _x_souris = evenement.xcrossing.x;
    _y_souris = evenement.xcrossing.y;
    if ( fenetre )
    {
      XRectangle rectangle;
      rectangle.x = fenetre->w_x; rectangle.y = fenetre->w_y;
      rectangle.width = (unsigned short) fenetre->w_larg;
      rectangle.height = (unsigned short) fenetre->w_haut;
      XSetClipRectangles(_Display, _PRG_station->gr_contexte, fenetre->x, fenetre->y,
			 &rectangle, 1, Unsorted);
      if (fenetre->rect)
	(fenetre->rect)(fenetre, _x_souris, _y_souris);
    }
    break;

  case LeaveNotify:  /* ? */
    _x_pointeur = _x_souris;
    _y_pointeur = _y_souris;
    _x_souris = -1;
    _y_souris = -1;
    if ( (fenetre) && (fenetre->rect) )
      (fenetre->rect)(fenetre, _x_souris, _y_souris);
    break;
    
  case DestroyNotify:  /* Fentre dtruite      */
  case UnmapNotify:    /* Fentre plus affiche */
  case ReparentNotify: /* ? */
  case MapNotify:      /* Affichage effectif    */
    break;
	
  default:
    printf("Message inconnu : type %d\n", evenement.xany.type);
  }

  return final; /* Comment a pu tre trait l'vnement */
}

/* ~~~~~~~~~~~~~~~~ Gestion des dplacements du pointeur souris ~~~~~~~ */

int EVNT_pointeur(Fenetre *fenetre, XMotionEvent *pointeur)
{
  if (fenetre == NULL) return FAUX;

  _x_pointeur = _x_souris; _y_pointeur = _y_souris;
  _x_souris = pointeur->x; _y_souris = pointeur->y;

  if ( fenetre->drapeaux.pointeur == VRAI ) /* Fentre a un pointeur */
  {
    XRectangle rectangle;

    rectangle.x = (short) (fenetre->w_x);
    rectangle.y = (short) (fenetre->w_y);
    rectangle.width = (unsigned short) (fenetre->w_larg);
    rectangle.height = (unsigned short) (fenetre->w_haut);

    XSetClipRectangles(_Display, _PRG_station->gr_contexte, 0, 0, &rectangle, 1, Unsorted);

    if (fenetre->rect) /* Fonction spciale associe */
    {
      return (fenetre->rect)(fenetre, pointeur->x, pointeur->y);
    }
    else
      return FAUX; /* On a demand de s'en occup mais pas de fonction */
  }

  if (fenetre->type == GEM_FENETRE)
    return VRAI; /* On nglige sauf si fonction spciale */

  return FAUX; /* Autres cas */
}

/* ~~~~~~~~~~~~~~~~ Gestion du clavier ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

/* Relchement d'une touche -> touches d'tat pour control et autres... */

int EVNT_relache(XKeyEvent *clavier)
{
  KeySym touche;
  char tampon[128];

  /* On rcupre le code de la touche utilise */

  XLookupString(clavier, tampon, 128, &touche, (XComposeStatus *) NULL);

  if ( (touche == XK_Control_L) || (touche == XK_Control_R) )
  {
    _etat &= ~K_CTRL; /* On vient de relcher Control */
    return VRAI; /* Trait */
  }

  if ( touche == XK_Shift_L )
  {
    _etat &= ~K_LSHIFT;
    return VRAI;
  }

  if ( touche == XK_Shift_R )
  {
    _etat &= ~K_RSHIFT;
    return VRAI;
  }

  return FAUX;
}

/* -------------- Rception et traitement primaire de l'vnement ----- */

/* ~~~~~~~~~~~~~~~~ Gestion des redessins ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

int EVNT_redessin(Fenetre *fenetre, XExposeEvent *redessin)
{
  XRectangle rectangle;

  rectangle.x = (short) (redessin->x);
  rectangle.y = (short) (redessin->y);
  rectangle.width = (unsigned short) (redessin->width);
  rectangle.height = (unsigned short) (redessin->height);

  if ( (fenetre == NULL) && (_Menu) ) /* Il y a un menu avec des titres */
  {
    int i;

    if (redessin->window == _Menu->racine) /* Fentre racine : rien  faire */
      return VRAI;

    for (i = 0 ; i < _Menu->nbr_titres ; i++)
    {
      if ( redessin->window == (_Menu->titres)[i] )
      {
	if (redessin->count == 0)
	  OBJ_dessine(redessin->window, _Menu->adresse + _Menu->prem_titre + i);
	return VRAI;
      }
    }

    return FAUX;
  }

  /* On s'occupe des lments internes de la fentre */

  if ( redessin->window == fenetre->closer ) /* Bouton de fermeture */
  {
    if (redessin->count == 0)
      OBJ_dessine(fenetre->closer, &FOBJ_closer);
    return VRAI;
  }

  if ( redessin->window == fenetre->barre_d_infos ) /* On affiche juste le texte */
  {
    if (redessin->count == 0)
    {
      FOBJ_infos.ob_spec.texte = fenetre->infos;
      OBJ_dessine(fenetre->barre_d_infos, &FOBJ_infos);
    }
    return VRAI;
  }

  if ( redessin->window == fenetre->fleche_haute)
  {
    OBJ_dessine(fenetre->fleche_haute, &FOBJ_fl_haut);
    return VRAI;
  }

  if ( redessin->window == fenetre->bouton_asc_vertical )
  {
    if (redessin->count == 0)
      OBJ_dessine(fenetre->bouton_asc_vertical, &FOBJ_asc_ver);
    return VRAI;
  }

  if ( redessin->window == fenetre->fond_asc_vertical )
    return VRAI;

  if ( redessin->window == fenetre->fleche_basse )
  {
    OBJ_dessine(fenetre->fleche_basse, &FOBJ_fl_bas);
    return VRAI;
  }

  if ( redessin->window == fenetre->fleche_gauche)
  {
    OBJ_dessine(fenetre->fleche_gauche, &FOBJ_fl_gauche);
    return VRAI;
  }

  if ( redessin->window == fenetre->bouton_asc_horizontal )
  {
    if (redessin->count == 0)
      OBJ_dessine(fenetre->bouton_asc_horizontal, &FOBJ_asc_hor);
    return VRAI;
  }

  if ( redessin->window == fenetre->fleche_droite )
  {
    OBJ_dessine(fenetre->fleche_droite, &FOBJ_fl_droite);
    return VRAI;
  }

  /* Ici : fentre proprement dite */

  if ( (fenetre->type == GEM_FENETRE) || (fenetre->type == MIXTE_FENETRE) )
  {
    Arbre *arbre = arbres + fenetre->numero_arbre; /* Arbre contenu dans la fentre */
    int i;

    for (i = 0 ; i < arbre->nbr_objets ; i++)
    {
      if (redessin->window == (arbre->objets)[i]) /* Redessiner l'objet i */
      {
	OBJ_dessine(redessin->window, arbre->adresse + i);
	if ( i == arbre->edit_courant ) /* On affiche le curseur texte */
        {
	  objc_edit(arbre->adresse, arbre->edit_courant, 0,
		    arbre->pos_curseur, ED_END + 1, &(arbre->pos_curseur) );
	}
	return VRAI;
      }
    }
  }

  /* Si on est l : il s'agit de la fentre utilisateur */

  if ( fenetre->redessin ) /* Il y a une fonction de redessin */
  {
    if ( (fenetre->rect) && (fenetre->drapeaux.pointeur == VRAI) )
    {
      _x_pointeur = _x_souris;
      _y_pointeur = _y_souris; /* Pour viter des problmes ? */
      (fenetre->rect)(fenetre, -1, -1);
      _x_souris = -1;
      _y_souris = -1;
    }

    XSetClipRectangles(_Display, _PRG_station->gr_contexte, 0, 0,
		       &rectangle, 1, Unsorted);

    (fenetre->redessin)(fenetre, rectangle.x, rectangle.y, rectangle.width, rectangle.height);

    return VRAI;
  }

  return FAUX; /* Redessin non trait */
}

int EVNT_relache_bouton(Fenetre *fenetre, XButtonEvent *souris)
{
  if (fenetre == NULL) /* Pas la peine d'enregistrer */
    return VRAI;

  /* Tous les gadgets de fentre : inutile d'enregistrer */

  if (souris->window == fenetre->closer)
    return VRAI;
  if (souris->window == fenetre->barre_d_infos )
    return VRAI;
  if (souris->window == fenetre->fleche_haute )
    return VRAI;
  if (souris->window == fenetre->fond_asc_vertical)
    return VRAI;
  if (souris->window == fenetre->bouton_asc_vertical)
    return VRAI;
  if (souris->window == fenetre->fleche_basse )
    return VRAI;
  if (souris->window == fenetre->fleche_gauche )
    return VRAI;
  if (souris->window == fenetre->fond_asc_horizontal)
    return VRAI;
  if (souris->window == fenetre->bouton_asc_horizontal)
    return VRAI;
  if (souris->window == fenetre->fleche_droite )
    return VRAI;

  /* Fentre utilisateur */

  return FAUX;
}

int EVNT_clic(Fenetre *fenetre, XButtonEvent *souris, XEvent *ancien, int *n_clics)
{
  if (fenetre == NULL) /* Fentre  usage interne -> pas de double-clic pour l'instant */
  {
    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    if (_Menu) /* Il y a bien un menu */
    {
      int i, retour;

      if (souris->window == _Menu->racine)
	return VRAI; /* Rien  faire si clic sur la racine */

      for (i = 0 ; i < _Menu->nbr_titres ; i++)
      {
	if ( _Menu->titres[i] == souris->window ) /* Clic sur le titre i */
	{
	  if ( OB_STATE(_Menu->adresse, _Menu->prem_titre + i) & DISABLED ) /* Inactif -> non */
	    return VRAI;

	  retour = CLIC_menu(i);

	  while (retour < -99) /* Clic sur un autre titre */
	    retour = CLIC_menu( -retour - 100 );

	  return retour;
	}
      }
    }

    return VRAI; /* Probablement une fentre ferme entre-temps */
  }

  if (souris->window == fenetre->closer) /* Bouton de fermeture */
  {
    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    if (fenetre->msg_masque.fermeture) /* On veut grer les messages de fermeture */
    {
      int handle = quel_handle(fenetre);
      buffer[0] = WM_CLOSED;
      buffer[1] = 0;
      buffer[2] = 0;
      buffer[3] = handle;
      buffer[7] = 0; /* Dans le cas o != 0 <=> fermeture force... */

      if (fenetre->message)
      {
	(fenetre->message)(fenetre);
      }
      else
      {
      }
    }
    else
      ferme_fenetre(fenetre);

    return VRAI;
  }

  if (souris->window == fenetre->barre_d_infos )
  {
    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    return VRAI;
  }

  if (souris->window == fenetre->fleche_haute )
  {
    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    if (fenetre->ascenseur)
      (fenetre->ascenseur)(fenetre, WM_ARROWED, WA_UPLINE, 0);
    else
    {
      int handle = quel_handle(fenetre);
      buffer[0] = WM_ARROWED;
      buffer[1] = 0;
      buffer[2] = 0;
      buffer[3] = handle;
      buffer[4] = WA_UPLINE; /* On prpare un message */
    }

    return VRAI;
  }

  if (souris->window == fenetre->fond_asc_vertical)
  {
    Window w;
    int x, y;
    unsigned int larg, haut, i;

    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    XGetGeometry(_Display, fenetre->bouton_asc_vertical, &w, &x, &y, &larg, &haut, &i, &i);

    if (fenetre->ascenseur)
    {
      if (souris->y < y) /* Clic avant le poussoir */
	(fenetre->ascenseur)(fenetre, WM_ARROWED, WA_UPPAGE, 0);
      else
	(fenetre->ascenseur)(fenetre, WM_ARROWED, WA_DNPAGE, 0);
    }
    else
    {
      int handle = quel_handle(fenetre);
      buffer[0] = WM_ARROWED;
      buffer[1] = 0;
      buffer[2] = 0;
      buffer[3] = handle;
      if (souris-> y < y)
	buffer[4] = WA_UPPAGE; /* On prpare un message */
      else
	buffer[4] = WA_DNPAGE; /* On prpare un message */
    }
    return VRAI;
  }

  if (souris->window == fenetre->fleche_basse )
  {
    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    if (fenetre->ascenseur)
      (fenetre->ascenseur)(fenetre, WM_ARROWED, WA_DNLINE, 0);
    else
    {
      int handle = quel_handle(fenetre);
      buffer[0] = WM_ARROWED;
      buffer[1] = 0;
      buffer[2] = 0;
      buffer[3] = handle;
      buffer[4] = WA_DNLINE; /* On prpare un message */
    }

    return VRAI;
  }

  if (souris->window == fenetre->fleche_gauche )
  {
    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    if (fenetre->ascenseur)
      (fenetre->ascenseur)(fenetre, WM_ARROWED, WA_LFLINE, 0);
    else
    {
      int handle = quel_handle(fenetre);
      buffer[0] = WM_ARROWED;
      buffer[1] = 0;
      buffer[2] = 0;
      buffer[3] = handle;
      buffer[4] = WA_LFLINE; /* On prpare un message */
    }

    return VRAI;
  }

  if (souris->window == fenetre->bouton_asc_vertical)
  {
    int nv_pos;

    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    nv_pos = X_ascenseur(fenetre->fond_asc_vertical, fenetre->bouton_asc_vertical, VERTICAL);
    if (fenetre->ascenseur)
      (fenetre->ascenseur)(fenetre, WM_VSLID, 0, nv_pos);
    else
    {
      int handle = quel_handle(fenetre);
      buffer[0] = WM_VSLID;
      buffer[1] = 0;
      buffer[2] = 0;
      buffer[3] = handle;
      buffer[4] = nv_pos; /* On prpare un message */
    }

    return VRAI;
  }

  if (souris->window == fenetre->fond_asc_horizontal)
  {
    Window w;
    int x, y;
    unsigned int larg, haut, i;

    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    XGetGeometry(_Display, fenetre->bouton_asc_horizontal, &w, &x, &y, &larg, &haut, &i, &i);

    if (fenetre->ascenseur)
    {
      if (souris->x > x) /* Clic avant le poussoir */
	(fenetre->ascenseur)(fenetre, WM_ARROWED, WA_RTPAGE, 0);
      else
	(fenetre->ascenseur)(fenetre, WM_ARROWED, WA_LFPAGE, 0);
    }
    else
    {
      int handle = quel_handle(fenetre);
      buffer[0] = WM_ARROWED;
      buffer[1] = 0;
      buffer[2] = 0;
      buffer[3] = handle;
      if (souris->x < x)
	buffer[4] = WA_LFPAGE; /* On prpare un message */
      else
	buffer[4] = WA_RTPAGE; /* On prpare un message */
    }

    return VRAI;
  }

  if (souris->window == fenetre->bouton_asc_horizontal)
  {
    int nv_pos;

    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    nv_pos = X_ascenseur(fenetre->fond_asc_horizontal, fenetre->bouton_asc_horizontal, HORIZONTAL);
    if (fenetre->ascenseur)
      (fenetre->ascenseur)(fenetre, WM_HSLID, 0, nv_pos);
    else
    {
      int handle = quel_handle(fenetre);
      buffer[0] = WM_HSLID;
      buffer[1] = 0;
      buffer[2] = 0;
      buffer[3] = handle;
      buffer[4] = nv_pos; /* On prpare un message */
    }

    return VRAI;
  }

  if (souris->window == fenetre->fleche_droite )
  {
    ancien->type = 0; /* De toute faon */
    *n_clics = 0;

    if (fenetre->ascenseur)
      (fenetre->ascenseur)(fenetre, WM_ARROWED, WA_RTLINE, 0);
    else
    {
      int handle = quel_handle(fenetre);
      buffer[0] = WM_ARROWED;
      buffer[1] = 0;
      buffer[2] = 0;
      buffer[3] = handle;
      buffer[4] = WA_RTLINE; /* On prpare un message */
    }

    return VRAI;
  }

  if ( ancien->type == ButtonRelease ) /* Il y avait un vnement enregistr */
  {
    if ( (souris->time - ancien->xbutton.time) > 100 ) /* clics distincts */
    {
      int final;
/*      printf("Clics distincts : on traite l'ancien - %d clics\n", *n_clics); */
      final = EVNT_souris(fenetre, &(ancien->xbutton), *n_clics);
      ancien->xbutton = *souris; /* On enregistre le nouvel vnement souris */
      *n_clics = 0;

      return final;
    }

    ancien->xbutton = *souris; /* On enregistre le nouvel vnement souris */
    (*n_clics)++; /* Un clic en plus */

    return VRAI;
  }

  /* Aucun vnement enregistr -> on se contente de conter 1 clic */
  ancien->xbutton = *souris; /* On enregistre le nouvel vnement souris */
  *n_clics = 1;

  return VRAI;
}

/* ------ Signal SIGALRM : pas d'vnement X depuis le dernier clic + 100 ms -> grer le clic
    Normalement, ne peut tre reu que si vx_evenement contient un vnement ButtonPress
    Normalement aussi, dsactiv aussitt qu'il est appel (?)                           ----- */

static void Traite_signal(void)
{
  Fenetre *fenetre;
  XEvent evenement;

/*  printf("On a reu le signal : il faut grer le clic seul\n"); */

  fenetre = quelle_fenetre(vx_evenement.xbutton.window);
  if (fenetre == NULL)
  {
    return;
  }

  _Serveur_X_bloque = VRAI;
  EVNT_souris(fenetre, &(vx_evenement.xbutton), _n_clic );
  _Serveur_X_bloque = FAUX;

  vx_evenement.type = 0;  /* Il n'y a plus d'vnement enregistr */

  /* Raliser toutes les requtes X demandes - sans perdre les vnements */

  XSync(_Display, False);

  /* Traiter les messages de redessin */

  while ( XCheckTypedEvent(_Display, Expose, &evenement) )
  {
    fenetre = quelle_fenetre(evenement.xexpose.window);
    _Fen_redessin = evenement.xexpose.window;
    EVNT_redessin(fenetre, &(evenement.xexpose) );
  }

  /* On peut retourner dans la procdure d'attente... */
}

int EVNT_souris(Fenetre *fenetre, XButtonEvent *souris, int n_clics)
{
  _bouton = 0; /* Aucun bouton sinon */
  if (souris->button == Button1)
    _bouton = 1;
  else if (souris->button == Button2)
    _bouton = 2;
  else
    return VRAI; /* Aucun bouton avec quivalent GEM -> on annule en fait */

  _x_souris = souris->x;
  _y_souris = souris->y;

  /*  partir de l: on est rellement dans la partie utile de la fentre */

  if (fenetre->type == GEM_FENETRE)
  {
    int i;
    return CLIC_arbre(fenetre->arbre, souris, &i);
  }

  if (fenetre->type == MIXTE_FENETRE)
  {
    int i;

    if (souris->window != fenetre->handle) /* Clic sur une autre fentre que la racine */
      return CLIC_arbre(fenetre->arbre, souris, &i); /* Clic sur l'arbre donc */
  }

  if (fenetre->clic) /* Il y a une fonction spciale pour les clics souris */
  {
    return (fenetre->clic)(fenetre, _bouton, n_clics, _etat, souris->x, souris->y);
  }

  printf("vnement souris. Fentre : %ld ; bouton : %d\n", souris->window, _bouton);

  return VRAI;
}

/* ----- Clic sur un arbre d'objets GEM --------\
 | Renvoie VRAI     si trait                   |
 |        _QUITTER  si clic sur bouton EXIT     |
 |                                              |
 | arbre  : pointeur sur l'arbre trait         |
 | souris : l'vnement souris qui a eu lieu    |
 | bouton : contient le bouton qui a t cliqu |
 \--------------------------------------------- */

int CLIC_arbre(Arbre *arbre, XButtonEvent *souris, int *bouton)
{
  int num_objet = 0, retour = VRAI;

  for (num_objet = ROOT ; arbre->objets[num_objet] != souris->window ; num_objet++)
  {
    if ( num_objet >= arbre->nbr_objets )
    {
      printf("Clic sur un objet trop loin dans l'arborescence\n");
      return FAUX;
    }
  }

  *bouton = num_objet;

  if ( OB_STATE(arbre->adresse, num_objet) & DISABLED )
    return VRAI;

  if ( (OB_FLAGS(arbre->adresse, num_objet) & EXIT)
       || (OB_FLAGS(arbre->adresse, num_objet) & TOUCHEXIT) )
    retour = _QUITTER;

  if ( OB_FLAGS(arbre->adresse, num_objet) & EDITABLE )
  {
    int pos_curseur = arbre->pos_curseur;
    int objet_edit  = arbre->edit_courant;

    objc_edit(arbre->adresse, objet_edit, 0, pos_curseur, ED_END, &pos_curseur);
    /* En attendant pos. prcise */
    objc_edit(arbre->adresse, num_objet, 0, pos_curseur, ED_INIT, &pos_curseur);

    arbre->edit_courant = num_objet;
    arbre->pos_curseur  = pos_curseur;
    return VRAI;
  }

  if (arbre->fonction.menu) /* Une fonction associe  l'arbre */
  {
    (arbre->fonction.arbre)(num_objet, souris->x, souris->y);
  }
  else /* Fonctionnement par dfaut */
  {
    int num_arbre = arbre - arbres; /* Numro de l'arbre */

    OBJ_choisit(num_arbre, num_objet, ECHANGE);
  }

  return retour;
}

/* ~~~~~~~~~~~~~~~~ Gestion des menus ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */


/* ~~~~~~~~~~ Gestion d'un menu : clavier ~~~~~~~~~~~~~~~~ */

int CLAV_menu(XKeyEvent *clavier)
{
  Arbre *menu;
  KeySym touche;
  int touche_gem;

  if (_Menu == NULL) return FAUX; /* Pas de menu ! */

  /* On rcupre le code de la touche utilise */

  touche_gem = XToucheVersGEM(clavier, TOUCHE_PRESSEE, &touche);

  menu = quel_arbre(_Menu->adresse); /* Arbre associ au menu */
  if ( (menu) && (menu->clavier) )   /* Si fonction de clavier associe au menu */
  {
    boolean retour;
    int renvoi; /* Bouton utilis pour sortir */

    retour = (menu->clavier)(touche_gem, _etat, &renvoi);
    if (retour != FAUX) /* A t trait */
      return retour;
  }

  if ( (touche >= XK_F1) && (touche <= XK_F35) ) /* Touches de fonction */
  {
    int num_titre;

    num_titre = (int) (touche - XK_F1); /* Numro du menu  droul */
    if (num_titre < _Menu->nbr_titres) /* Ce menu existe bien */
    {
      int option;

      option = CLIC_menu(num_titre);
      while (option < -99) /* Clic sur un autre titre */
	option = CLIC_menu( -option - 100 );
    }
  }

  return VRAI; /* Trait comme un raccourci gnral */
}


/* --------- Rectangle intersection de 2 rectangles - renvoie FAUX s'il est vide -------------- */

boolean Rc_intersect(int x_1, int y_1, int larg_1, int haut_1, int *x, int *y, int *larg, int *haut)
{
  int x_2,y_2,larg_2,haut_2;

  if ( (x_1 > *x)
       || ( (x_1 == *x) && (y_1 < *y) ) )  /* On change les deux rectangles ==> 1 tjrs x minimal */
  {
    x_2 = x_1;
    y_2 = y_1;
    larg_2 = larg_1;
    haut_2 = haut_1;
    x_1 = *x;
    y_1 = *y;
    larg_1 = *larg;
    haut_1 = *haut;
  }
  else
  {
    x_2 = *x;
    y_2 = *y;
    larg_2 = *larg;
    haut_2 = *haut;
  }
  
  if ( (x_1 + larg_1) <= x_2 )  /* Le rectangle 1 finit avant que 2 commence, en x */
    return FAUX;
  
  *x=x_2;
  if ( (x_1 + larg_1) > (x_2 + larg_2) )
    *larg = larg_2;
  else
    *larg = (x_1 + larg_1) - x_2;
  
  if (y_1 >= y_2)
  {
    if ( (y_2 + haut_2) <= y_1)
      return FAUX;
    
    *y = y_1;
    if ( (y_2 + haut_2) > (y_1 + haut_1) )
      *haut = haut_1;
    else
      *haut = haut_2 - (y_1 - y_2);
    
    return VRAI;   /* Les rectangles se coupent */
  }
  else  /* y_1 < y_2 */
  {
    if ( (y_1 + haut_1) <= y_2 )
      return FAUX;

    *y = y_2;
    
    if ( (y_1 + haut_1) > (y_2 + haut_2) )
      *haut = haut_2;
    else
      *haut = haut_1 - (y_2 - y_1);
    
    return VRAI;
  }
}
